import java.util.Calendar;

import l2r.Config;
import l2r.extensions.scripts.Functions;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.instancemanager.TownManager;
import l2r.gameserver.instancemanager.ZoneManager;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Zone;
import l2r.gameserver.model.L2Zone.ZoneType;
import l2r.gameserver.model.entity.Castle;
import l2r.gameserver.model.entity.SevenSigns;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.util.Location;

public class Util extends Functions implements ScriptFile
{
	public static L2Object self;
	public static L2Object npc;

	public void onLoad()
	{
		System.out.println("Utilites Loaded");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	/**
	 * Перемещает за плату в аденах
	 *
	 * @param x
	 * @param y
	 * @param z
	 * @param price
	 */
	public void Gatekeeper(String[] param)
	{
		if(param.length < 4)
			throw new IllegalArgumentException();

		L2Player player = (L2Player) self;

		int price = Integer.parseInt(param[3]);

		int day = Calendar.getInstance().get(Calendar.DAY_OF_WEEK);
		int hour = Calendar.getInstance().get(Calendar.HOUR_OF_DAY);
		if(day != 1 && day != 7 && (hour <= 12 || hour >= 22))
			price /= 2;

		if(player.isActionsDisabled() || player.isSitting() || player.getLastNpc().getDistance(player) > 300)
			return;

		if(player.getAdena() < price)
		{
			player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			return;
		}

		int x = Integer.parseInt(param[0]);
		int y = Integer.parseInt(param[1]);
		int z = Integer.parseInt(param[2]);

		// Нельзя телепортироваться в города, где идет осада
		// Узнаем, идет ли осада в ближайшем замке к точке телепортации
		Castle castle = TownManager.getInstance().getClosestTown(x, y).getCastle();
		if(castle != null && castle.getSiege().isInProgress())
		{
			// Определяем, в город ли телепортируется чар
			boolean teleToTown = false;
			int townId = 0;
			for(L2Zone town : ZoneManager.getInstance().getZoneByType(ZoneType.Town))
				if(town.checkIfInZone(x, y))
				{
					teleToTown = true;
					townId = town.getIndex();
					break;
				}

			if(teleToTown && townId == castle.getTown())
			{
				player.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_TELEPORT_TO_A_VILLAGE_THAT_IS_IN_A_SIEGE));
				return;
			}
		}

		Location pos = GeoClient.getInstance().findPointToStay(x, y, z, 50, 100);

		if(price > 0)
			player.reduceAdena(price, false);
		player.teleToLocation(pos);
	}

	public void SSGatekeeper(String[] param)
	{
		if(param.length < 4)
			throw new IllegalArgumentException();

		L2Player player = (L2Player) self;
		int type = Integer.parseInt(param[3]);

		if(player.isActionsDisabled() || player.isSitting() || player.getLastNpc().getDistance(player) > 300)
			return;

		if(type > 0)
		{
			if(SevenSigns.getInstance().getCurrentPeriod() == SevenSigns.PERIOD_COMPETITION && SevenSigns.getInstance().getPlayerCabal(player) == SevenSigns.CABAL_NULL)
				return;

			if(SevenSigns.getInstance().getCurrentPeriod() == SevenSigns.PERIOD_SEAL_VALIDATION)
			{
				if(type == 1 && SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_AVARICE) != SevenSigns.getInstance().getPlayerCabal(player))
					return;
				if(type == 2 && SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_GNOSIS) != SevenSigns.getInstance().getPlayerCabal(player))
					return;
			}
		}

		player.teleToLocation(Integer.parseInt(param[0]), Integer.parseInt(param[1]), Integer.parseInt(param[2]));
	}

	/**
	 * Перемещает за определенный предмет
	 *
	 * @param x
	 * @param y
	 * @param z
	 * @param count
	 * @param item
	 */
	public void QuestGatekeeper(String[] param)
	{
		if(param.length < 5)
			throw new IllegalArgumentException();

		L2Player player = (L2Player) self;

		int count = Integer.parseInt(param[3]);
		int item = Integer.parseInt(param[4]);
		L2ItemInstance ii = player.getInventory().getItemByItemId(item);

		if(player.isActionsDisabled() || player.isSitting() || player.getLastNpc().getDistance(player) > 300)
			return;

		if(ii == null || ii.getIntegerLimitedCount() < count)
		{
			player.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_REQUIRED_ITEMS));
			return;
		}

		int x = Integer.parseInt(param[0]);
		int y = Integer.parseInt(param[1]);
		int z = Integer.parseInt(param[2]);

		Location pos = GeoClient.getInstance().findPointToStay(x, y, z, 20, 70);

		player.getInventory().destroyItem(ii, count, true);

		if(count > 1)
			player.sendPacket(new SystemMessage(SystemMessage.S2_S1_HAS_DISAPPEARED).addItemName(item).addNumber(count));
		else
			player.sendPacket(new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(item));

		player.teleToLocation(pos);
	}

	public void ReflectionGatekeeper(String[] param)
	{
		if(param.length < 5)
			throw new IllegalArgumentException();

		L2Player player = (L2Player) self;
		player.setReflection(Integer.parseInt(param[4]));

		Gatekeeper(param);
	}

	/**
	 * Используется для телепортации за Newbie Token, проверяет уровень и передает
	 * параметры в QuestGatekeeper
	 */
	public void TokenJump(String[] param)
	{
		L2Player player = (L2Player) self;
		if(player.getLevel() <= 19)
			QuestGatekeeper(param);
		else
			show("Only for newbies", player);
	}

	public void NoblessTeleport()
	{
		L2Player player = (L2Player) self;
		if(player.isNoble() || Config.ALLOW_NOBLE_TP_TO_ALL)
			show("data/scripts/noble.htm", player);
		else
			show("data/scripts/nobleteleporter-no.htm", player);
	}

	public void PayPage(String[] param)
	{
		if(param.length < 2)
			throw new IllegalArgumentException();

		L2Player player = (L2Player) self;
		String page = param[0];
		int item = Integer.parseInt(param[1]);
		int price = Integer.parseInt(param[2]);

		if(getItemCount(player, item) < price)
		{
			if(item == 57)
				player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			else
				player.sendPacket(Msg.INCORRECT_ITEM_COUNT);
			return;
		}

		removeItem(player, item, price);
		show(page, player);
	}
}