package ai;

import java.util.concurrent.Future;

import npc.model.AntharasInstance;
import l2r.gameserver.ai.Fighter;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Spawn;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.tables.NpcTable;
import l2r.util.Location;

/**
 * AI боса Aнтараса.<br> - Спавниться разной силы в зависимости от количесва людей прошедших через кубик. - проверяет наличие людей в регионе каждые 10 минут -
 * При смерти спаунит обратный портал, который удаляется через 15 минут - если переходит в состояние бездействия,то возвращаеться на спавн - через 10 минут
 * начинает снова проверять людей - если его атакуют проверяет сколько людей на территории если надо переспавниваеться
 *
 * @author FlareDrakon
 */
public class Antharas extends Fighter
{
	private long _wait_timeout = 0;
	private boolean _isMovingBack = false;
	private static final int TELEPORTATION_CUBIC_ID = 31859;
	private static final Location TELEPORT_CUBIC_LOCATOIN = new Location(177615, 114941, -7709, 0);
	private static int MINUTE = 60000;
	private static final int ANTHARAS_WAIT_FOR_PLAYERS_TIMEOUT = MINUTE * 10; // 10 минут
	private static final int CUBIC_DESPAWN_TIME = MINUTE * 15; // 15 минут
	AntharasInstance Boss = (AntharasInstance) _thisActor;
	public boolean _firstTimeAttacked = true;

	@SuppressWarnings("unchecked")
	public Future _UseKamicadze;

	public Antharas(L2Character actor)
	{
		super(actor);
	}

	@Override
	public boolean isGlobalAI()
	{
		return true;
	}

	@Override
	protected boolean thinkActive()
	{
		if(_thisActor.isDead())
			return true;

		if(_def_think)
		{
			doTask();
			return true;
		}

		if(!Boss.getWait())
		{
			Boss.setWait(true);
			_wait_timeout = System.currentTimeMillis() + ANTHARAS_WAIT_FOR_PLAYERS_TIMEOUT;
		}

		if(_wait_timeout != 0 && Boss.getWait() && _wait_timeout < System.currentTimeMillis())
			if(!_isMovingBack) // Двигаемся на базу
			{
				_isMovingBack = true;
				// Добавить новое задание
				Task task = new Task();
				task.type = TaskType.MOVE;
				task.loc = _thisActor.getSpawnedLoc();
				_task_list.add(task);
				_def_think = true;
				return true;
			}
			else if(_isMovingBack && !_thisActor.isMoving) // Пришли на базу
			{
				_isMovingBack = false;
				onSpawn();
				_wait_timeout = 0;
			}

		return super.thinkActive();
	}

	@Override
	protected void onEvtAttacked(L2Character attacker, int damage)
	{
		if(_firstTimeAttacked)
		{
			_firstTimeAttacked = false;
			int RealId = Boss.checkSpawn();
			Boss.setparam(true, false, RealId);
			Boss.stopTask(Boss.chekPlayerTask);
			Boss.stopTask(Boss.spawnMinionTask);
			Boss.stopTask(Boss.spawnMinion2Task);
		}

		super.onEvtAttacked(attacker, damage);
	}

	@Override
	protected void onEvtDead()
	{
		Boss._playersEnter = 0;
		Boss.setWait(false);
		_wait_timeout = 0;
		_isMovingBack = false;
		Boss.setDespawned(true);
		Boss.stopTask(Boss.spawnMinionTask);
		Boss.stopTask(Boss.spawnMinion2Task);
		Boss.stopTask(Boss.chekPlayerTask);
		try
		{
			L2Spawn spawn = new L2Spawn(NpcTable.getTemplate(TELEPORTATION_CUBIC_ID));
			spawn.setLoc(TELEPORT_CUBIC_LOCATOIN);
			spawn.doSpawn(true);
			spawn.stopRespawn();
			ThreadPoolManager.getInstance().scheduleGeneral(new DeSpawnScheduleTimerTask(spawn), CUBIC_DESPAWN_TIME);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		_firstTimeAttacked = true;

		super.onEvtDead();
	}

	public void onSpawn()
	{
		int RealId = Boss.checkSpawn();
		Boss.stopTask(Boss.spawnMinionTask);
		Boss.stopTask(Boss.spawnMinion2Task);
		_firstTimeAttacked = true;
		Boss.setparam(false, true, RealId);// despawn //wait
	}

	private class UseSkill implements Runnable
	{
		L2NpcInstance _minion;

		public UseSkill(L2NpcInstance minion)
		{
			_minion = minion;
		}

		public void run()
		{
			for(L2Skill skill : _minion.getAllSkills())
				if(skill.getId() == 5097 || skill.getId() == 5094)
				{
					_minion.doCast(skill, null, false);
					Boss.stopTask(_UseKamicadze);
				}
		}
	}

	public class DeSpawnScheduleTimerTask implements Runnable
	{
		L2Spawn _spawn = null;

		public DeSpawnScheduleTimerTask(L2Spawn spawn1)
		{
			_spawn = spawn1;
		}

		public void run()
		{
			try
			{
				_spawn.getLastSpawn().deleteMe();
			}
			catch(Throwable t)
			{}
		}
	}
}