package ai;

import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_ACTIVE;

import java.util.ArrayList;

import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.ai.DefaultAI;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Spawn;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.tables.NpcTable;
import l2r.util.Location;
import l2r.util.Rnd;

/**
 * AI боса Байума.<br>
 * - Мгновенно убивает первого ударившего<br>
 * - Если его не трогали 30 минут, то засыпает обратно в каменную статую<br>
 * - При смерти спаунит обратный портал, который удаляется через 15 минут<br>
 * - Для атаки использует только скилы по следующей схеме:
 * <li>Стандартный набор: 80% - 4127, 10% - 4128, 10% - 4129
 * <li>если хп < 50%: 70% - 4127, 10% - 4128, 10% - 4129, 10% - 4131
 * <li>если хп < 25%: 60% - 4127, 10% - 4128, 10% - 4129, 10% - 4131, 10% - 4130
 *
 * @author SYS
 */
public class Baium extends DefaultAI
{
	private long _wait_timeout = 0;
	private boolean _wait = false;
	private boolean _isMovingBack = false;
	private boolean _firstTimeAttacked = true;
	private static final Location BAIUM_NPC_POSITION = new Location(116127, 17368, 10107, 35431);
	private static final Location BAIUM_BACK_POSITION = new Location(115986, 17227, 10051, 0);
	private static final int BAIUM_NPC_ID = 29025;
	private static final int BAIUM_SLEEP_TIMEOUT = 1000 * 60 * 30; // 30 минут
	private static final int TELEPORTATION_CUBIC_ID = 31759;
	private static final int CUBIC_DESPAWN_TIME = 15 * 60 * 1000; // 15 min

	// Боевые скилы байума
	private static final int s_thunderbolt_ID = 4130;
	private static final int s_group_hold_ID = 4131;
	private static final int s_energy_wave_ID = 4128;
	private static final int s_earth_quake_ID = 4129;
	private static final int s_baium_normal_attack_ID = 4127;

	public Baium(L2Character actor)
	{
		super(actor);
	}

	@Override
	public boolean isGlobalAI()
	{
		return true;
	}

	@Override
	protected void onEvtAttacked(L2Character attacker, int damage)
	{
		if(_firstTimeAttacked)
		{
			_firstTimeAttacked = false;
			if(attacker.isSummon || attacker.isPet)
				attacker = attacker.getOwner();
			attacker.doDie(_thisActor);
		}

		super.onEvtAttacked(attacker, damage);
	}

	@Override
	protected boolean thinkActive()
	{
		if(_thisActor.isDead())
			return true;

		if(_def_think)
		{
			doTask();
			_wait = false;
			return true;
		}

		if(!_wait)
		{
			_wait = true;
			_wait_timeout = System.currentTimeMillis() + BAIUM_SLEEP_TIMEOUT;
		}

		// Пора засыпать
		if(_wait_timeout != 0 && _wait && _wait_timeout < System.currentTimeMillis())
			if(!_isMovingBack) // Двигаемся на базу
			{
				_isMovingBack = true;
				// Добавить новое задание
				Task task = new Task();
				task.type = TaskType.MOVE;
				task.loc = BAIUM_BACK_POSITION;
				_task_list.add(task);
				_def_think = true;
				return true;
			}
			else if(_isMovingBack && !_thisActor.isMoving) // Пришли на базу
			{
				// Спауним каменную статую
				L2NpcInstance baiumNpc = null;
				try
				{
					L2Spawn spawn = new L2Spawn(NpcTable.getTemplate(BAIUM_NPC_ID));
					spawn.setLoc(BAIUM_NPC_POSITION);
					spawn.setRespawnTime(0);
					spawn.storeRespawnTimeInDb();
					baiumNpc = spawn.doSpawn(true);
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}

				// Удаляем боса, если успешно заспаунилась каменная статуя
				if(baiumNpc != null)
				{
					_firstTimeAttacked = true;
					_thisActor.decayMe(); // Делаем его сперва невидимым
					_thisActor.doDie(null); // Убиваем, иначе останутся миньоны
					_thisActor.deleteMe(); // Удаляем из мира
				}
			}

		return super.thinkActive();
	}

	@Override
	protected boolean createNewTask()
	{
		// Удаляем все задания
		clearTasks();

		L2Character _temp_attack_target = _attack_target;

		// Новая цель исходя из агрессивности
		L2Character hated = _thisActor.isConfused() ? _temp_attack_target : _thisActor.getMostHated();

		if(hated != null && hated != _thisActor)
		{
			_attack_target = hated;
			_temp_attack_target = hated;
		}
		else
		{
			_thisActor.setAttackTimeout(Integer.MAX_VALUE);
			_attack_target = null;
			_temp_attack_target = null;
			clientStopMoving();
			setIntention(AI_INTENTION_ACTIVE);
			return false;
		}

		L2Skill r_skill = null;
		ArrayList<L2Skill> d_skill = new ArrayList<L2Skill>();
		ArrayList<L2Skill> t_skill = new ArrayList<L2Skill>();

		// Шансы использования скилов
		int s_thunderbolt = 0; // >50% хп, s_thunderbolt не используем
		int s_group_hold = 0; // >50% хп, s_group_hold не используем
		int s_energy_wave = 10;
		int s_earth_quake = 10;

		// <50% хп, добавляем s_group_hold
		if(_thisActor.getCurrentHp() < _thisActor.getMaxHp() * 0.5)
			s_group_hold = 10;

		// <25% хп, добавляем s_thunderbolt
		if(_thisActor.getCurrentHp() < _thisActor.getMaxHp() * 0.25)
			s_thunderbolt = 10;

		// Вычислим шанс обычной атаки
		int s_baium_normal_attack = 100 - s_thunderbolt - s_group_hold - s_energy_wave - s_earth_quake;

		// Выбираем скилл атаки
		if(!Rnd.chance(s_baium_normal_attack))
		{
			t_skill.clear();
			d_skill.clear();

			L2Skill sk = _thisActor.getTemplate().getSkills().get(s_energy_wave_ID);
			if(sk != null)
				t_skill.add(sk);

			sk = _thisActor.getTemplate().getSkills().get(s_earth_quake_ID);
			if(sk != null)
				t_skill.add(sk);

			if(s_thunderbolt > 0)
			{
				sk = _thisActor.getTemplate().getSkills().get(s_thunderbolt_ID);
				if(sk != null)
					t_skill.add(sk);
			}

			if(s_group_hold > 0)
			{
				sk = _thisActor.getTemplate().getSkills().get(s_group_hold_ID);
				if(sk != null && _temp_attack_target.getEffectsBySkill(sk) == null)
					t_skill.add(sk);
			}

			// Фильтруем неподходящие по дальности скилы
			double distance = _thisActor.getDistance(_temp_attack_target);
			for(L2Skill skill : t_skill)
			{
				if(skill == null || skill.getCastRange() < distance && skill.getTargetType() != L2Skill.SkillTargetType.TARGET_AURA)
					continue;
				d_skill.add(skill);
			}

			// Выбрать 1 скилл из полученных
			if(d_skill.size() > 0)
				r_skill = d_skill.get(Rnd.get(d_skill.size()));
		}

		// Если в руте, то использовать массовый скилл дальнего боя
		if(_thisActor.isRooted())
			r_skill = _thisActor.getTemplate().getSkills().get(s_thunderbolt_ID);

		// Использовать скилл если можно, иначе атаковать скилом s_baium_normal_attack
		if(r_skill != null)
		{
			// Проверка таргета
			if(r_skill.getTargetType() == L2Skill.SkillTargetType.TARGET_SELF)
				_temp_attack_target = _thisActor;
			if(r_skill.getTargetType() == L2Skill.SkillTargetType.TARGET_AURA)
				_temp_attack_target = _thisActor;
		}
		else
			r_skill = _thisActor.getTemplate().getSkills().get(s_baium_normal_attack_ID);

		// Добавить новое задание
		Task task = new Task();
		task.type = TaskType.CAST;
		task.target = _temp_attack_target;
		task.skill = r_skill;
		_task_list.add(task);
		_def_think = true;
		r_skill = null;
		return true;
	}

	@Override
	protected boolean maybeMoveToHome()
	{
		return false;
	}

	@Override
	protected void onEvtDead()
	{
		_wait = false;
		_wait_timeout = 0;
		_isMovingBack = false;
		_firstTimeAttacked = true;

		try
		{
			L2Spawn spawn = new L2Spawn(NpcTable.getTemplate(TELEPORTATION_CUBIC_ID));
			spawn.setLoc(_thisActor.getLoc());
			spawn.doSpawn(true);
			spawn.stopRespawn();

			ThreadPoolManager.getInstance().scheduleGeneral(new DeSpawnScheduleTimerTask(spawn), CUBIC_DESPAWN_TIME);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		super.onEvtDead();
	}

	public class DeSpawnScheduleTimerTask implements Runnable
	{
		L2Spawn _spawn = null;

		public DeSpawnScheduleTimerTask(L2Spawn spawn)
		{
			_spawn = spawn;
		}

		public void run()
		{
			try
			{
				_spawn.getLastSpawn().decayMe();
				_spawn.getLastSpawn().deleteMe();
			}
			catch(Throwable t)
			{}
		}
	}
}