package ai;

import l2r.extensions.scripts.Functions;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.ai.Fighter;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Spawn;
import l2r.gameserver.serverpackets.PlaySound;
import l2r.gameserver.tables.NpcTable;
import l2r.util.Location;
import l2r.util.Rnd;

/**
 * AI боса Core:<br> - Бубнит при атаке и смерти.<br> - При смерти играет музыку.<br> - При смерти спаунит обратные порталы, которые удаляются через 15 минут
 *
 * @author SYS
 */
public class Core extends Fighter
{
	private boolean _firstTimeAttacked = true;
	private static final int TELEPORTATION_CUBIC_ID = 31842;
	private static final Location CUBIC_1_POSITION = new Location(16502, 110165, -6394, 0);
	private static final Location CUBIC_2_POSITION = new Location(18948, 110165, -6394, 0);
	private static final int CUBIC_DESPAWN_TIME = 15 * 60 * 1000; // 15 min

	public Core(L2Character actor)
	{
		super(actor);
	}

	@Override
	protected void onEvtAttacked(L2Character attacker, int damage)
	{
		if(_firstTimeAttacked)
		{
			_firstTimeAttacked = false;
			Functions.npcShout(_thisActor, "A non-permitted target has been discovered.");
			Functions.npcShout(_thisActor, "Starting intruder removal system.");
		}
		else if(Rnd.chance(1))
			Functions.npcShout(_thisActor, "Removing intruders.");

		super.onEvtAttacked(attacker, damage);
	}

	@Override
	protected void onEvtDead()
	{
		_thisActor.broadcastPacket(new PlaySound(1, "BS02_D", 1, 0, _thisActor.getLoc()));
		Functions.npcShout(_thisActor, "A fatal error has occurred");
		Functions.npcShout(_thisActor, "System is being shut down...");
		Functions.npcShout(_thisActor, "......");

		try
		{
			L2Spawn spawn1 = new L2Spawn(NpcTable.getTemplate(TELEPORTATION_CUBIC_ID));
			spawn1.setLoc(CUBIC_1_POSITION);
			spawn1.doSpawn(true);
			spawn1.stopRespawn();

			L2Spawn spawn2 = new L2Spawn(NpcTable.getTemplate(TELEPORTATION_CUBIC_ID));
			spawn2.setLoc(CUBIC_2_POSITION);
			spawn2.doSpawn(true);
			spawn2.stopRespawn();

			ThreadPoolManager.getInstance().scheduleGeneral(new DeSpawnScheduleTimerTask(spawn1, spawn2), CUBIC_DESPAWN_TIME);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		_firstTimeAttacked = true;
		super.onEvtDead();
	}

	public class DeSpawnScheduleTimerTask implements Runnable
	{
		L2Spawn _spawn1 = null;
		L2Spawn _spawn2 = null;

		public DeSpawnScheduleTimerTask(L2Spawn spawn1, L2Spawn spawn2)
		{
			_spawn1 = spawn1;
			_spawn2 = spawn2;
		}

		public void run()
		{
			try
			{
				_spawn1.getLastSpawn().decayMe();
				_spawn2.getLastSpawn().decayMe();
				_spawn1.getLastSpawn().deleteMe();
				_spawn2.getLastSpawn().deleteMe();
			}
			catch(Throwable t)
			{}
		}
	}
}