package ai;

import java.util.concurrent.ScheduledFuture;

import l2r.extensions.scripts.Functions;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.ai.CtrlEvent;
import l2r.gameserver.ai.Fighter;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.model.L2Character;
import l2r.util.Location;
import l2r.util.Rnd;

/**
 * AI для Ol Mahum Guard ID: 20058
 *
 * @author Diamond
 */
public class OlMahumGuard extends Fighter
{
	private L2Character _attacker;
	
	public OlMahumGuard(L2Character actor)
	{
		super(actor);
	}

	@Override
	protected void onEvtAttacked(L2Character attacker, int damage)
	{
		if(attacker != null && !_thisActor.isAfraid() && _thisActor.getCurrentHp() < _thisActor.getMaxHp() / 2)
		{
			_attacker = attacker;
			
			if(Rnd.chance(50))
				Functions.npcShout(_thisActor, "I'll be back");
			else
				Functions.npcShout(_thisActor, "You are stronger than expected");
			
			// Удаляем все задания
			clearTasks();
			
			_thisActor.breakAttack();
			_thisActor.breakCast(true);
			_thisActor.stopMove();
			_thisActor.startFear();

			int posX = _thisActor.getX();
			int posY = _thisActor.getY();
			int posZ = _thisActor.getZ();

			int signx = -1;
			int signy = -1;

			if(posX > attacker.getX())
				signx = 1;
			if(posX > attacker.getY())
				signy = 1;

			int range = 1000;

			posX += Math.round(signx * range);
			posY += Math.round(signy * range);
			posZ = GeoClient.getInstance().getHeight(posX, posY, posZ);

			_thisActor.setRunning();
			_thisActor.moveToLocation(new Location(posX, posY, posZ), 0, true);
			
			startEndFearTask();
		}
		else
			super.onEvtAttacked(attacker, damage);
	}
	
	@Override
	protected void onEvtDead()
	{
		_attacker = null;
		_thisActor.stopFear();
		if(_endFearTask != null)
			_endFearTask.cancel(true);
		_endFearTask = null;
		super.onEvtDead();
	}
	
	private ScheduledFuture<EndFearTask> _endFearTask;
	
	public void startEndFearTask()
	{
		if(_endFearTask != null)
			_endFearTask.cancel(true);
		_endFearTask = ThreadPoolManager.getInstance().scheduleAi(new EndFearTask(), 10000, false);
	}
	
	public class EndFearTask implements Runnable
	{
		public void run()
		{
			_thisActor.stopFear();
			_endFearTask = null;
			if(_attacker != null)
				notifyEvent(CtrlEvent.EVT_AGGRESSION, _attacker, 100);
		}
	}

	@Override
	public void checkAggression(L2Character target)
	{}

	@Override
	protected void onEvtAggression(L2Character target, int aggro)
	{}
}