package ai;

import l2r.extensions.listeners.DayNightChangeListener;
import l2r.extensions.listeners.L2ZoneEnterLeaveListener;
import l2r.extensions.listeners.PropertyCollection;
import l2r.gameserver.GameTimeController;
import l2r.gameserver.ai.Fighter;
import l2r.gameserver.instancemanager.ZoneManager;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Zone;
import l2r.gameserver.model.L2Zone.ZoneType;

/**
 * Индивидуальное АИ эпик боса Zaken.<BR> - неуязвим ночью<BR> - получает 25% пенальти на регены солнечной комнате - зона zaken_sunlight_room, id: 1335
 *
 * @Author: SYS
 */
public class Zaken extends Fighter implements PropertyCollection
{
	private static L2Zone _zone = ZoneManager.getInstance().getZoneById(ZoneType.no_restart, 1335, true);
	private ZoneListener _zoneListener = new ZoneListener();
	private final float _baseHpReg;
	private final float _baseMpReg;

	public Zaken(L2Character actor)
	{
		super(actor);
		GameTimeController.getInstance().getListenerEngine().addPropertyChangeListener(GameTimeControllerDayNightChange, new NightInvulDayNightListener());
		_zone.getListenerEngine().addMethodInvokedListener(_zoneListener);
		_baseHpReg = _thisActor.getTemplate().baseHpReg;
		_baseMpReg = _thisActor.getTemplate().baseMpReg;
	}

	private class NightInvulDayNightListener extends DayNightChangeListener
	{
		private NightInvulDayNightListener()
		{
			if(GameTimeController.getInstance().isNowNight())
				switchToNight();
			else
				switchToDay();
		}

		/**
		 * Вызывается, когда на сервере наступает ночь
		 */
		@Override
		public void switchToNight()
		{
			_thisActor.setIsInvul(true);
		}

		/**
		 * Вызывается, когда на сервере наступает день
		 */
		@Override
		public void switchToDay()
		{
			_thisActor.setIsInvul(false);
		}
	}

	private class ZoneListener extends L2ZoneEnterLeaveListener
	{
		@Override
		public void objectEntered(L2Zone zone, L2Object object)
		{
			// Регены снижаем в любое время суток, все равно закен ночью бессмертен
			// что бы лишний раз не дергать GameTimeController
			_thisActor.getTemplate().baseHpReg = (float) (_baseHpReg * 0.75);
			_thisActor.getTemplate().baseMpReg = (float) (_baseMpReg * 0.75);
		}

		@Override
		public void objectLeaved(L2Zone zone, L2Object object)
		{
			_thisActor.getTemplate().baseHpReg = _baseHpReg;
			_thisActor.getTemplate().baseMpReg = _baseMpReg;
		}
	}

	@Override
	protected void onEvtDead()
	{
		_zone.getListenerEngine().removeMethodInvokedListener(_zoneListener);
		super.onEvtDead();
	}
}