package commands.admin;

import java.lang.reflect.Field;
import java.util.StringTokenizer;

import l2r.Config;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.serverpackets.PlaySound;
import l2r.gameserver.serverpackets.SystemMessage;

/**
 * This class handles following admin commands: - admin = shows menu
 */
public class AdminAdmin implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_admin",
			"admin_play_sounds",
			"admin_play_sound",
			"admin_silence",
			"admin_atmosphere",
			"admin_diet",
			"admin_tradeoff",
			"admin_config",
			"admin_show_html" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().Menu)
			return false;
		if(command.equalsIgnoreCase("admin_admin"))
			AdminHelpPage.showHelpPage(activeChar, "admin.htm");

		if(command.equalsIgnoreCase("admin_play_sounds"))
			AdminHelpPage.showHelpPage(activeChar, "songs/songs.htm");
		else if(command.startsWith("admin_play_sounds"))
			try
			{
				AdminHelpPage.showHelpPage(activeChar, "songs/songs" + command.substring(17) + ".htm");
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.startsWith("admin_play_sound"))
			try
			{
				playAdminSound(activeChar, command.substring(17));
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.startsWith("admin_silence"))
			if(activeChar.getMessageRefusal()) // already in message refusal
			// mode
			{
				activeChar.unsetVar("gm_silence");
				activeChar.setMessageRefusal(false);
				activeChar.sendPacket(new SystemMessage(SystemMessage.MESSAGE_ACCEPTANCE_MODE));
			}
			else
			{
				if(Config.SAVE_GM_EFFECTS)
					activeChar.setVar("gm_silence", "true");
				activeChar.setMessageRefusal(true);
				activeChar.sendPacket(new SystemMessage(SystemMessage.MESSAGE_REFUSAL_MODE));
			}
		else if(command.startsWith("admin_tradeoff"))
			try
			{
				StringTokenizer st = new StringTokenizer(command);
				st.nextToken();
				if(st.nextToken().equalsIgnoreCase("on"))
				{
					activeChar.setTradeRefusal(true);
					activeChar.sendMessage("tradeoff enabled");
				}
				else if(st.nextToken().equalsIgnoreCase("off"))
				{
					activeChar.setTradeRefusal(false);
					activeChar.sendMessage("tradeoff disabled");
				}
			}
			catch(Exception ex)
			{
				if(activeChar.getTradeRefusal())
					activeChar.sendMessage("tradeoff currently enabled");
				else
					activeChar.sendMessage("tradeoff currently disabled");
			}
		else if(command.startsWith("admin_config"))
		{
			StringTokenizer st = new StringTokenizer(command);
			st.nextToken();
			String pName = "";
			try
			{
				String[] parameter = st.nextToken().split("=");
				pName = parameter[0].trim();
				String pValue = parameter[1].trim();
				Field field = Config.class.getField(pName);
				if(setField(activeChar, field, pValue))
					activeChar.sendMessage("Config set succesfully");
			}
			catch(NoSuchFieldException e)
			{
				activeChar.sendMessage("Parameter " + pName + " not found");
			}
			catch(Exception e)
			{
				activeChar.sendMessage("Usage:  //config parameter=value");
			}
		}
		else if(command.startsWith("admin_show_html"))
		{
			String html = command.substring(16);
			try
			{
				if(html != null)
					AdminHelpPage.showHelpPage(activeChar, html);
				else
					activeChar.sendMessage("Html page not found");
			}
			catch(Exception npe)
			{
				activeChar.sendMessage("Html page not found");
			}
		}

		return true;
	}

	private boolean setField(L2Player activeChar, Field field, String param)
	{
		try
		{
			field.setBoolean(null, Boolean.parseBoolean(param));
		}
		catch(Exception e)
		{
			try
			{
				field.setInt(null, Integer.parseInt(param));
			}
			catch(Exception e1)
			{
				try
				{
					field.setLong(null, Long.parseLong(param));
				}
				catch(Exception e2)
				{
					try
					{
						field.set(null, param);
					}
					catch(Exception e3)
					{
						activeChar.sendMessage("Error while set field: " + param + " " + e.getMessage());
						return false;
					}
				}
			}
		}
		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void playAdminSound(L2Player activeChar, String sound)
	{
		activeChar.broadcastPacket(new PlaySound(sound));
		AdminHelpPage.showHelpPage(activeChar, "admin.htm");
		activeChar.sendMessage("Playing " + sound + ".");
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"file://com_admin.htm",
				"Описание команды play_sounds",
				"Описание команды play_sound",
				"Описание команды silence",
				"Описание команды atmosphere",
				"Описание команды diet",
				"Описание команды tradeoff",
				"Описание команды config",
				"Описание команды show_html" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER };
	}
}