package commands.admin;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.Announcements;
import l2r.gameserver.clientpackets.Say2C;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.serverpackets.ExShowScreenMessage;
import l2r.gameserver.serverpackets.ExShowScreenMessage.ScreenMessageAlign;

/**
 * This class handles following admin commands: - announce text = announces text
 * to all players - list_announcements = show menu - reload_announcements =
 * reloads announcements from txt file - announce_announcements = announce all
 * stored announcements to all players - add_announcement text = adds text to
 * startup announcements - del_announcement id = deletes announcement with
 * respective id
 */
public class AdminAnnouncements implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_list_announcements",
			"admin_announce_announcements",
			"admin_add_announcement",
			"admin_del_announcement",
			"admin_announce",
			"admin_a",
			"admin_announce_menu",
			"admin_crit_announce",
			"admin_toscreen" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanAnnounce)
			return false;
		String[] args = command.split(" ");

		if(args[0].equals("admin_list_announcements"))
			Announcements.getInstance().listAnnouncements(activeChar);
		else if(args[0].equals("admin_announce_menu"))
		{
			Announcements sys = new Announcements();
			sys.handleAnnounce(command, 20);
			Announcements.getInstance().listAnnouncements(activeChar);
		}
		else if(args[0].equals("admin_announce_announcements"))
		{
			for(L2Player player : L2World.getAllPlayers())
				Announcements.getInstance().showAnnouncements(player);
			Announcements.getInstance().listAnnouncements(activeChar);
		}
		else if(args[0].equals("admin_add_announcement"))
		{
			// FIXME the player can send only 16 chars (if you try to send more it
			// sends null), remove this function or not?
			if(args.length < 2)
				return false;
			try
			{
				String val = command.substring(23);
				Announcements.getInstance().addAnnouncement(val);
				Announcements.getInstance().listAnnouncements(activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		}
		else if(args[0].equals("admin_del_announcement"))
		{
			if(args.length < 2)
				return false;
			try
			{
				int val = new Integer(command.substring(23));
				Announcements.getInstance().delAnnouncement(val);
				Announcements.getInstance().listAnnouncements(activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		}
		// Command is admin announce
		else if(args[0].equals("admin_announce"))
		{
			// Call method from another class
			Announcements sys = new Announcements();
			sys.handleAnnounce(command, 15);
		}
		else if(args[0].equals("admin_a"))
		{
			// Call method from another class
			Announcements sys = new Announcements();
			sys.handleAnnounce(command, 8);
		}
		else if(args[0].equals("admin_crit_announce") || args[0].equals("admin_c"))
		{
			// Call method from another class
			Announcements sys = new Announcements();
			sys.handleAnnounce(command, 20, Say2C.CRITICAL_ANNOUNCEMENT);
		}
		else if(args[0].equals("admin_toscreen") || args[0].equals("admin_s"))
		{
			if(args.length < 2)
				return false;
			String _text = command.replaceFirst("admin_toscreen ", "");
			int _time = 3000 + _text.length() * 100; // 3 секунды + 100мс на символ
			boolean _font_big = _text.length() < 64;

			ExShowScreenMessage sm = new ExShowScreenMessage(_text, _time, ScreenMessageAlign.TOP_CENTER, _font_big);
			for(L2Player player : L2World.getAllPlayers())
				player.sendPacket(sm);
		}

		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"file://com_list_announcements.htm",
				"file://com_announce_announcements.htm",
				"file://com_add_announcement.htm",
				"file://com_del_announcement.htm",
				"file://com_announce.htm",
				"file://com_a.htm",
				"file://com_announce_menu.htm",
				"Описание для crit_announce",
				"Аннонс прямо на экран, а не в чат" };

	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER };
	}
}