package commands.admin;

import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.instancemanager.CursedWeaponsManager;
import l2r.gameserver.model.CursedWeapon;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.tables.ItemTable;

public class AdminCursedWeapons implements IAdminCommandHandler, ScriptFile
{
	private static final String[] ADMIN_COMMANDS = { "admin_cw_info", "admin_cw_remove", "admin_cw_goto", "admin_cw_reload", "admin_cw_add" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().Menu)
			return false;

		CursedWeaponsManager cwm = CursedWeaponsManager.getInstance();
		int id = 0;

		StringTokenizer st = new StringTokenizer(command);
		st.nextToken();

		if(command.startsWith("admin_cw_info"))
		{
			activeChar.sendMessage("======= Cursed Weapons: =======");
			for(CursedWeapon cw : cwm.getCursedWeapons())
			{
				activeChar.sendMessage("> " + cw.getName() + " (" + cw.getItemId() + ")");
				if(cw.isActivated())
				{
					L2Player pl = cw.getPlayer();
					activeChar.sendMessage("  Player holding: " + pl.getName());
					activeChar.sendMessage("  Player karma: " + cw.getPlayerKarma());
					activeChar.sendMessage("  Time Remaining: " + cw.getTimeLeft() / 60000 + " min.");
					activeChar.sendMessage("  Kills : " + cw.getNbKills());
				}
				else if(cw.isDropped())
				{
					activeChar.sendMessage("  Lying on the ground.");
					activeChar.sendMessage("  Time Remaining: " + cw.getTimeLeft() / 60000 + " min.");
					activeChar.sendMessage("  Kills : " + cw.getNbKills());
				}
				else
					activeChar.sendMessage("  Don't exist in the world.");
			}
		}
		else if(command.startsWith("admin_cw_reload"))
		{
			cwm.reload();
			activeChar.sendMessage("Cursed weapons reloaded.");
		}
		else
		{
			CursedWeapon cw = null;
			try
			{
				String parameter = st.nextToken();
				Pattern pattern = Pattern.compile("[0-9]*");
				Matcher regexp = pattern.matcher(parameter);
				if(regexp.matches())
					id = Integer.parseInt(parameter);
				else
				{
					parameter = parameter.replace('_', ' ');
					for(CursedWeapon cwp : cwm.getCursedWeapons())
						if(cwp.getName().toLowerCase().contains(parameter.toLowerCase()))
						{
							id = cwp.getItemId();
							break;
						}
				}
				cw = cwm.getCursedWeapon(id);
				if(cw == null)
				{
					activeChar.sendMessage("Unknown cursed weapon ID.");
					return false;
				}
			}
			catch(Exception e)
			{
				activeChar.sendMessage("Usage: //cw_remove|//cw_goto|//cw_add <itemid|name>");
			}

			if(cw == null)
				return false;

			if(command.startsWith("admin_cw_remove "))
				CursedWeaponsManager.getInstance().endOfLife(cw);
			else if(command.startsWith("admin_cw_goto "))
				activeChar.teleToLocation(cw.getLoc());
			else if(command.startsWith("admin_cw_add"))
			{
				if(cw.isActive())
					activeChar.sendMessage("This cursed weapon is already active.");
				else
				{
					L2Object target = activeChar.getTarget();
					if(target != null && target.isPlayer)
					{
						L2Player player = (L2Player) target;

						L2ItemInstance item = ItemTable.getInstance().createItem(id, activeChar.getObjectId(), 0, "AdminCursedWeapons");
						cwm.activate(player, player.getInventory().addItem(item));
						cwm.showUsageTime(player, cw);
					}
				}
			}
			else
				activeChar.sendMessage("Unknown command.");
		}
		return true;
	}

	public String[] getAdminCommandList()
	{
		return ADMIN_COMMANDS;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"file://com_cw_info.htm",
				"file://com_cw_remove.htm",
				"file://com_cw_goto.htm",
				"file://com_cw_reload.htm",
				"file://com_cw_add.htm" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_CW,
				AdminCommandHandler.GROUP_CW,
				AdminCommandHandler.GROUP_CW,
				AdminCommandHandler.GROUP_CW,
				AdminCommandHandler.GROUP_CW };
	}
}