package commands.admin;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2DoorInstance;
import l2r.gameserver.tables.DoorTable;

/**
 * This class handles following admin commands: - open1 = open coloseum door
 * 24190001 - open2 = open coloseum door 24190002 - open3 = open coloseum door
 * 24190003 - open4 = open coloseum door 24190004 - openall = open all coloseum
 * door - close1 = close coloseum door 24190001 - close2 = close coloseum door
 * 24190002 - close3 = close coloseum door 24190003 - close4 = close coloseum
 * door 24190004 - closeall = close all coloseum door
 *  - open = open selected door - close = close selected door
 */
public class AdminDoorControl implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = { "admin_open", "admin_close", "admin_openall", "admin_closeall" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().Door)
			return false;

		try
		{
			if(command.startsWith("admin_open "))
			{
				int doorId = Integer.parseInt(command.split(" ")[1]);
				DoorTable.getInstance().getDoor(doorId).openMe();
			}
			else if(command.startsWith("admin_close "))
			{
				int doorId = Integer.parseInt(command.split(" ")[1]);
				DoorTable.getInstance().getDoor(doorId).closeMe();
			}
			if(command.equals("admin_closeall"))
				for(L2DoorInstance door : DoorTable.getInstance().getDoors())
					door.closeMe();
			if(command.equals("admin_openall"))
				for(L2DoorInstance door : DoorTable.getInstance().getDoors())
					door.openMe();
			if(command.equals("admin_open"))
			{
				L2Object target = activeChar.getTarget();
				if(target instanceof L2DoorInstance)
					((L2DoorInstance) target).openMe();
				else
					activeChar.sendPacket(Msg.INCORRECT_TARGET);
			}

			if(command.equals("admin_close"))
			{
				L2Object target = activeChar.getTarget();
				if(target instanceof L2DoorInstance)
					((L2DoorInstance) target).closeMe();
				else
					activeChar.sendPacket(Msg.INCORRECT_TARGET);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] { "file://com_open.htm", "file://com_close.htm", "file://com_openall.htm", "file://com_closeall.htm" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1 };
	}
}