package commands.admin;

import java.text.NumberFormat;
import java.util.Collection;
import java.util.Locale;
import java.util.StringTokenizer;

import l2r.database.mysql;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.Announcements;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.serverpackets.NpcHtmlMessage;
import l2r.gameserver.serverpackets.SkillList;
import l2r.gameserver.serverpackets.SocialAction;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.SkillTable;
import l2r.util.Log;

public class AdminEditChar implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_edit_character",
			"admin_character_actions",
			"admin_current_player",
			"admin_nokarma",
			"admin_setkarma",
			"admin_character_list",
			"admin_show_characters",
			"admin_find_character",
			"admin_save_modifications",
			"admin_rec",
			"admin_settitle",
			"admin_setname",
			"admin_setsex",
			"admin_setcolor",
			"admin_add_exp_sp_to_character",
			"admin_add_exp_sp",
			"admin_sethero",
			"admin_setnoble",
			"admin_trans" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(activeChar.getPlayerAccess().CanRename)
			if(command.startsWith("admin_settitle"))
				try
				{
					String val = command.substring(15);
					L2Object target = activeChar.getTarget();
					L2Player player = null;
					if(target != null && target.isPlayer)
						player = (L2Player) target;
					else
						return false;
					player.setTitle(val);
					player.sendMessage("Your title has been changed by a GM");
					player.sendChanges();

					Log.add("change title for player " + player.getName() + " to " + val, "gm_ext_actions", activeChar);
				}
				catch(StringIndexOutOfBoundsException e)
				{ // Case of empty character title
					activeChar.sendMessage("You need to specify the new title.");
				}
			else if(command.startsWith("admin_setname"))
				try
				{
					String val = command.substring(14);
					L2Object target = activeChar.getTarget();
					L2Player player;
					if(target != null && target.isPlayer)
						player = (L2Player) target;
					else
						return false;
					if(mysql.simple_get_int("count(*)", "characters", "`char_name` like '" + val + "'") > 0)
					{
						activeChar.sendMessage("Name already exist.");
						return false;
					}
					Log.add("Character " + player.getName() + " renamed to " + val + " by GM " + activeChar.getName(), "renames");
					Log.add("set name for player " + player.getName() + " to " + val, "gm_ext_actions", activeChar);
					player.setName(val);
					player.sendMessage("Your name has been changed by a GM");
					player.broadcastUserInfo();
				}
				catch(StringIndexOutOfBoundsException e)
				{ // Case of empty character name
					activeChar.sendMessage("You need to specify the new name.");
				}

		if(!activeChar.getPlayerAccess().CanEditChar && !activeChar.getPlayerAccess().CanViewChar)
			return false;
		else if(command.equals("admin_current_player"))
			showCharacterList(activeChar, null);
		else if(command.startsWith("admin_character_list"))
			try
			{
				String val = command.substring(21);
				L2Player target = L2World.getPlayer(val);
				showCharacterList(activeChar, target);
			}
			catch(StringIndexOutOfBoundsException e)
			{
				// Case of empty character name
			}
		else if(command.startsWith("admin_show_characters"))
			try
			{
				String val = command.substring(22);
				int page = Integer.parseInt(val);
				listCharacters(activeChar, page);
			}
			catch(StringIndexOutOfBoundsException e)
			{
				// Case of empty page
			}
		else if(command.startsWith("admin_find_character"))
			try
			{
				String val = command.substring(21);
				findCharacter(activeChar, val);
			}
			catch(StringIndexOutOfBoundsException e)
			{ // Case of empty character name
				activeChar.sendMessage("You didnt enter a character name to find.");

				listCharacters(activeChar, 0);
			}
		else if(!activeChar.getPlayerAccess().CanEditChar)
			return false;
		else if(command.equals("admin_edit_character"))
			editCharacter(activeChar);
		else if(command.equals("admin_character_actions"))
			showCharacterActions(activeChar);
		else if(command.equals("admin_nokarma"))
			setTargetKarma(activeChar, 0);
		else if(command.startsWith("admin_setkarma"))
			try
			{
				String val = command.substring(15);
				int karma = Integer.parseInt(val);
				setTargetKarma(activeChar, karma);
			}
			catch(StringIndexOutOfBoundsException e)
			{
				activeChar.sendMessage("Please specify new karma value.");
			}
		else if(command.startsWith("admin_save_modifications"))
			try
			{
				String val = command.substring(24);
				adminModifyCharacter(activeChar, val);

				Log.add("save modifications for player " + val, "gm_ext_actions", activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{ // Case of empty character name
				activeChar.sendMessage("Error while modifying character.");
				listCharacters(activeChar, 0);
			}
		else if(command.equals("admin_rec"))
		{
			L2Object target = activeChar.getTarget();
			L2Player player = null;
			if(target != null && target.isPlayer)
				player = (L2Player) target;
			else
				return false;
			player.setRecomHave(player.getRecomHave() + 1);
			player.sendMessage("You have been recommended by a GM");
			player.broadcastUserInfo();

			Log.add("recommend player " + player.getName() + " + 1", "gm_ext_actions", activeChar);
		}
		else if(command.startsWith("admin_rec"))
			try
			{
				String val = command.substring(10);
				int recVal = Integer.parseInt(val);
				L2Object target = activeChar.getTarget();
				L2Player player = null;
				if(target != null && target.isPlayer)
					player = (L2Player) target;
				else
					return false;
				player.setRecomHave(player.getRecomHave() + recVal);
				player.sendMessage("You have been recommended by a GM");
				player.broadcastUserInfo();

				Log.add("recommend player " + player.getName() + " + " + recVal, "gm_ext_actions", activeChar);
			}
			catch(NumberFormatException e)
			{
				activeChar.sendMessage("Command format is //rec <number>");
			}
		else if(command.startsWith("admin_sethero"))
		{
			// Статус меняется только на текущую логон сессию
			String[] wordList = command.split(" ");
			L2Object target = activeChar.getTarget();
			L2Player player;
			if(wordList.length > 1 && wordList[1] != null)
			{
				player = L2World.getPlayer(wordList[1]);
				if(player == null)
				{
					activeChar.sendMessage("Character " + wordList[1] + " not found in game.");
					return false;
				}
			}
			else if(target != null && target.isPlayer)
				player = (L2Player) target;
			else
			{
				activeChar.sendMessage("You must specify the name or target character.");
				return false;
			}

			if(player.isHero())
			{
				player.setHero(false);
				player.removeSkill(SkillTable.getInstance().getInfo(395, 1));
				player.removeSkill(SkillTable.getInstance().getInfo(396, 1));
				player.removeSkill(SkillTable.getInstance().getInfo(1374, 1));
				player.removeSkill(SkillTable.getInstance().getInfo(1375, 1));
				player.removeSkill(SkillTable.getInstance().getInfo(1376, 1));
			}
			else
			{
				player.setHero(true);
				player.addSkill(SkillTable.getInstance().getInfo(395, 1));
				player.addSkill(SkillTable.getInstance().getInfo(396, 1));
				player.addSkill(SkillTable.getInstance().getInfo(1374, 1));
				player.addSkill(SkillTable.getInstance().getInfo(1375, 1));
				player.addSkill(SkillTable.getInstance().getInfo(1376, 1));
			}

			player.sendPacket(new SkillList(player));

			if(player.isHero())
			{
				player.broadcastPacket(new SocialAction(player.getObjectId(), 16));
				Announcements.getInstance().announceToAll(player.getName() + " has become a hero.");
			}
			player.sendMessage("Admin changed your hero status.");
			player.broadcastUserInfo();

			Log.add("add hero status to player " + player.getName(), "gm_ext_actions", activeChar);
		}
		else if(command.startsWith("admin_setnoble"))
		{
			// Статус сохраняется в базе
			String[] wordList = command.split(" ");
			L2Object target = activeChar.getTarget();
			L2Player player;
			if(wordList.length > 1 && wordList[1] != null)
			{
				player = L2World.getPlayer(wordList[1]);
				if(player == null)
				{
					activeChar.sendMessage("Character " + wordList[1] + " not found in game.");
					return false;
				}
			}
			else if(target != null && target.isPlayer)
				player = (L2Player) target;
			else
			{
				activeChar.sendMessage("You must specify the name or target character.");
				return false;
			}

			if(player.isNoble())
			{
				player.setNoble(false);
				player.removeSkill(SkillTable.getInstance().getInfo(1323, 1));
				player.removeSkill(SkillTable.getInstance().getInfo(325, 1));
				player.removeSkill(SkillTable.getInstance().getInfo(326, 1));
				player.removeSkill(SkillTable.getInstance().getInfo(327, 1));
				player.removeSkill(SkillTable.getInstance().getInfo(1324, 1));
				player.removeSkill(SkillTable.getInstance().getInfo(1325, 1));
				player.removeSkill(SkillTable.getInstance().getInfo(1326, 1));
				player.removeSkill(SkillTable.getInstance().getInfo(1327, 1));
			}
			else
			{
				player.setNoble(true);
				player.addSkill(SkillTable.getInstance().getInfo(1323, 1));
				player.addSkill(SkillTable.getInstance().getInfo(325, 1));
				player.addSkill(SkillTable.getInstance().getInfo(326, 1));
				player.addSkill(SkillTable.getInstance().getInfo(327, 1));
				player.addSkill(SkillTable.getInstance().getInfo(1324, 1));
				player.addSkill(SkillTable.getInstance().getInfo(1325, 1));
				player.addSkill(SkillTable.getInstance().getInfo(1326, 1));
				player.addSkill(SkillTable.getInstance().getInfo(1327, 1));
			}

			player.sendPacket(new SkillList(player));

			if(player.isNoble())
				player.broadcastPacket(new SocialAction(player.getObjectId(), 16));

			player.sendMessage("Admin changed your noble status.");
			player.broadcastUserInfo();

			Log.add("add noble status to player " + player.getName(), "gm_ext_actions", activeChar);
		}
		else if(command.startsWith("admin_setsex"))
		{
			L2Object target = activeChar.getTarget();
			L2Player player = null;
			if(target != null && target.isPlayer)
				player = (L2Player) target;
			else
				return false;
			player.changeSex();
			player.sendMessage("Your gender has been changed by a GM");
			player.broadcastUserInfo();

			Log.add("change gender to player " + player.getName(), "gm_ext_actions", activeChar);
		}
		else if(command.startsWith("admin_setcolor"))
			try
			{
				String val = command.substring(15);
				L2Object target = activeChar.getTarget();
				L2Player player = null;
				if(target != null && target.isPlayer)
					player = (L2Player) target;
				else
					return false;
				player.setNameColor(Integer.decode("0x" + val));
				player.sendMessage("Your name color has been changed by a GM");
				player.broadcastUserInfo();

				Log.add("change name color for player " + player.getName(), "gm_ext_actions", activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{ // Case of empty color
				activeChar.sendMessage("You need to specify the new color.");
			}
		else if(command.startsWith("admin_add_exp_sp_to_character"))
			addExpSp(activeChar);
		else if(command.startsWith("admin_add_exp_sp"))
			try
			{
				final String val = command.substring(16);
				adminAddExpSp(activeChar, val);
			}
			catch(final StringIndexOutOfBoundsException e)
			{ // Case of empty character name
				activeChar.sendMessage("Error while adding Exp-Sp.");
			}
		else if(command.startsWith("admin_trans"))
		{
			StringTokenizer st = new StringTokenizer(command);
			if(st.countTokens() > 1)
			{
				st.nextToken();
				int transformId = 0;
				try
				{
					transformId = Integer.parseInt(st.nextToken());
				}
				catch(Exception e)
				{
					activeChar.sendMessage("Specify a valid integer value.");
					return false;
				}
			}
			else
				activeChar.sendMessage("Usage: //trans <ID>");
		}
		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	private void listCharacters(L2Player activeChar, int page)
	{
		Collection<L2Player> allPlayers = L2World.getAllPlayers();
		L2Player[] players = allPlayers.toArray(new L2Player[allPlayers.size()]);

		int MaxCharactersPerPage = 20;
		int MaxPages = players.length / MaxCharactersPerPage;

		if(players.length > MaxCharactersPerPage * MaxPages)
			MaxPages++;

		// Check if number of users changed
		if(page > MaxPages)
			page = MaxPages;

		int CharactersStart = MaxCharactersPerPage * page;
		int CharactersEnd = players.length;
		if(CharactersEnd - CharactersStart > MaxCharactersPerPage)
			CharactersEnd = CharactersStart + MaxCharactersPerPage;

		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);

		StringBuffer replyMSG = new StringBuffer("<html><body>");
		replyMSG.append("<table width=260><tr>");
		replyMSG.append("<td width=40><button value=\"Main\" action=\"bypass -h admin_admin\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("<td width=180><center>Character Selection Menu</center></td>");
		replyMSG.append("<td width=40><button value=\"Back\" action=\"bypass -h admin_admin\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("</tr></table>");
		replyMSG.append("<br><br>");
		replyMSG.append("<table width=270>");
		replyMSG.append("<tr><td width=270>You can find a character by writing his name and</td></tr>");
		replyMSG.append("<tr><td width=270>clicking Find bellow.<br></td></tr>");
		replyMSG.append("<tr><td width=270>Note: Names should be written case sensitive.</td></tr>");
		replyMSG.append("</table><br>");
		replyMSG.append("<center><table><tr><td>");
		replyMSG.append("<edit var=\"character_name\" width=80></td><td><button value=\"Find\" action=\"bypass -h admin_find_character $character_name\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\">");
		replyMSG.append("</td></tr></table></center><br><br>");

		for(int x = 0; x < MaxPages; x++)
		{
			int pagenr = x + 1;
			replyMSG.append("<center><a action=\"bypass -h admin_show_characters " + x + "\">Page " + pagenr + "</a></center>");
		}
		replyMSG.append("<br>");

		// List Players in a Table
		replyMSG.append("<table width=270>");
		replyMSG.append("<tr><td width=80>Name:</td><td width=110>Class:</td><td width=40>Level:</td></tr>");
		for(int i = CharactersStart; i < CharactersEnd; i++)
			replyMSG.append("<tr><td width=80>" + "<a action=\"bypass -h admin_character_list " + players[i].getName() + "\">" + players[i].getName() + "</a></td><td width=110>" + players[i].getTemplate().className + "</td><td width=40>" + players[i].getLevel() + "</td></tr>");
		replyMSG.append("</table>");
		replyMSG.append("</body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	public static void showCharacterList(L2Player activeChar, L2Player player)
	{
		if(player == null)
		{
			L2Object target = activeChar.getTarget();
			if(target.isPlayer)
				player = (L2Player) target;
			else
				return;
		}
		else
			activeChar.setTarget(player);

		String clanName = "No Clan";
		if(player.getClan() != null)
			clanName = player.getClan().getName();

		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);

		StringBuffer replyMSG = new StringBuffer("<html><body>");
		replyMSG.append("<table width=260><tr>");
		replyMSG.append("<td width=40><button value=\"Main\" action=\"bypass -h admin_admin\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("<td width=180><center>Character Selection Menu</center></td>");
		replyMSG.append("<td width=40><button value=\"Back\" action=\"bypass -h admin_show_characters 0\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("</tr></table>");
		replyMSG.append("<br><br>");
		replyMSG.append("<center>Character Information:</center>");
		replyMSG.append("<br1>");

		// Character Player Info
		replyMSG.append("<table width=270>");
		replyMSG.append("<tr><td width=135>IP: " + player.getNetConnection().getIpAddr() + "</td><td width=135>Acc: " + player.getNetConnection().getLoginName() + "</td></tr>");
		replyMSG.append("<tr><td width=135>Name: " + player.getName() + "</td><td width=135>Lv: " + player.getLevel() + " " + player.getTemplate().className + "</td></tr>");
		replyMSG.append("<tr><td width=135>Clan: " + clanName + "</td><td width=135>Exp: " + player.getExp() + "</td></tr>");
		replyMSG.append("</table>");

		// Character ClassID & Coordinates
		replyMSG.append("<table width=270>");
		replyMSG.append("<tr><td width=270>Class Template Id: " + player.getClassId() + "/" + player.getClassId().getId() + "</td></tr>");
		replyMSG.append("<tr><td width=270>Character Coordinates: " + player.getX() + " " + player.getY() + " " + player.getZ() + "</td></tr>");
		replyMSG.append("</table>");
		replyMSG.append("<br>");

		NumberFormat df = NumberFormat.getNumberInstance(Locale.ENGLISH);
		df.setMaximumFractionDigits(4);
		df.setMinimumFractionDigits(1);

		// Character Stats
		replyMSG.append("<table width=250>");
		replyMSG.append("<tr><td width=40></td><td width=70>Curent:</td><td width=70>Max:</td><td width=70></td></tr>");
		replyMSG.append("<tr><td width=40>HP:</td><td width=70>" + (int) player.getCurrentHp() + "</td><td width=70>" + player.getMaxHp() + "</td><td width=70>Karma: " + player.getKarma() + "</td></tr>");
		replyMSG.append("<tr><td width=40>MP:</td><td width=70>" + (int) player.getCurrentMp() + "</td><td width=70>" + player.getMaxMp() + "</td><td width=70>Pvp: " + player.getPvpKills() + "</td></tr>");
		replyMSG.append("<tr><td width=40>Load:</td><td width=70>" + player.getCurrentLoad() + "</td><td width=70>" + player.getMaxLoad() + "</td><td width=70>Pvp Flag: " + player.getPvpFlag() + "</td></tr>");
		replyMSG.append("<tr><td width=40>SP:</td><td width=70>" + player.getSp() + "</td><td width=70></td><td width=70></td></tr>");
		replyMSG.append("</table>");
		replyMSG.append("<table width=270>");
		replyMSG.append("<tr><td width=60>P.ATK: " + player.getPAtk(null) + "</td><td width=100>M.ATK: " + player.getMAtk(null, null) + "</td><td width=40>Manage:</td></tr>");
		replyMSG.append("<tr><td width=60>P.DEF: " + player.getPDef(null) + "</td><td width=100>M.DEF: " + player.getMDef(null, null) + "</td><td width=40></td></tr>");
		replyMSG.append("<tr><td width=90>Accuracy: " + player.getAccuracy() + "</td><td width=70>Evasion: " + player.getEvasionRate(null) + "</td><td width=40><button value=\"Skills\" action=\"bypass -h admin_show_skills\" width=60 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td></tr>");
		replyMSG.append("<tr><td width=90>Critical: " + player.getCriticalHit(null, null) + "</td><td width=70>Speed: " + player.getWalkSpeed() + "/" + player.getRunSpeed() + "</td><td width=40><button value=\"Stats\" action=\"bypass -h admin_edit_character\" width=60 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td></tr>");
		replyMSG.append("<tr><td width=90>ATK Spd: " + player.getPAtkSpd() + "</td><td width=70>Casting Spd: " + player.getMAtkSpd() + "</td><td width=40><button value=\"Exp & Sp\" action=\"bypass -h admin_add_exp_sp_to_character\" width=60 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td></tr>");
		replyMSG.append("<tr><td width=90>MCrit: " + df.format(player.getCriticalMagic(null, null) / 10) + "%</td><td width=70> </td><td width=40><button value=\"Actions\" action=\"bypass -h admin_character_actions\" width=60 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td></tr>");
		replyMSG.append("</table></body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	private void setTargetKarma(L2Player activeChar, int newKarma)
	{
		L2Object target = activeChar.getTarget();
		if(target == null)
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		L2Player player;
		if(target.isPlayer)
			player = (L2Player) target;
		else
			return;

		if(newKarma >= 0)
		{
			int oldKarma = player.getKarma();
			player.setKarma(newKarma);

			player.sendMessage("Admin has changed your karma from " + oldKarma + " to " + newKarma + ".");
			activeChar.sendMessage("Successfully Changed karma for " + player.getName() + " from (" + oldKarma + ") to (" + newKarma + ").");

			Log.add("Changed karma for " + player.getName() + " from (" + oldKarma + ") to (" + newKarma + ")", "gm_ext_actions", activeChar);
		}
		else
			activeChar.sendMessage("You must enter a value for karma greater than or equal to 0.");
	}

	private void adminModifyCharacter(L2Player activeChar, String modifications)
	{
		L2Object target = activeChar.getTarget();
		L2Player player;
		if(target.isPlayer)
			player = (L2Player) target;
		else
			return;

		StringTokenizer st = new StringTokenizer(modifications);
		if(st.countTokens() != 6)
			editCharacter(player);
		else
		{
			String hp = st.nextToken();
			String mp = st.nextToken();
			String karma = st.nextToken();
			String pvpflag = st.nextToken();
			String pvpkills = st.nextToken();
			String classid = st.nextToken();
			int hpval = Integer.parseInt(hp);
			int mpval = Integer.parseInt(mp);
			// int loadval = Integer.parseInt(load);
			int karmaval = Integer.parseInt(karma);
			int pvpflagval = Integer.parseInt(pvpflag);
			int pvpkillsval = Integer.parseInt(pvpkills);
			short classidval = Short.parseShort(classid);

			// Common character information
			player.sendMessage("Admin has changed your stats. Hp: " + hpval + " Mp: " + mpval + " Karma: " + karmaval + " Pvp: " + pvpflagval + " / " + pvpkillsval + " ClassId: " + classidval);

			player.setCurrentHp(hpval);
			player.setCurrentMp(mpval);
			player.setKarma(karmaval);
			player.setPvpFlag(pvpflagval);
			player.setPvpKills(pvpkillsval);
			player.setClassId(classidval, true);

			player.sendChanges();

			// Admin information
			activeChar.sendMessage("Changed stats of " + player.getName() + ".  Hp: " + hpval + " Mp: " + mpval + " Karma: " + karmaval + " Pvp: " + pvpflagval + " / " + pvpkillsval + " ClassId: " + classidval);

			showCharacterList(activeChar, null); // Back to start
			player.broadcastUserInfo();
			player.decayMe();
			player.spawnMe(activeChar.getLoc());
		}
	}

	private void editCharacter(L2Player activeChar)
	{
		L2Object target = activeChar.getTarget();
		L2Player player;
		if(target != null && target.isPlayer)
			player = (L2Player) target;
		else
			return;

		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);

		StringBuffer replyMSG = new StringBuffer("<html><body>");
		replyMSG.append("<table width=260><tr>");
		replyMSG.append("<td width=40><button value=\"Main\" action=\"bypass -h admin_admin\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("<td width=180><center>Character Selection Menu</center></td>");
		replyMSG.append("<td width=40><button value=\"Back\" action=\"bypass -h admin_current_player\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("</tr></table>");
		replyMSG.append("<br><br>");
		replyMSG.append("<center>Editing character: " + player.getName() + "</center><br>");
		replyMSG.append("<table width=250>");
		replyMSG.append("<tr><td width=40></td><td width=70>Curent:</td><td width=70>Max:</td><td width=70></td></tr>");
		replyMSG.append("<tr><td width=40>HP:</td><td width=70>" + player.getCurrentHp() + "</td><td width=70>" + player.getMaxHp() + "</td><td width=70>Karma: " + player.getKarma() + "</td></tr>");
		replyMSG.append("<tr><td width=40>MP:</td><td width=70>" + player.getCurrentMp() + "</td><td width=70>" + player.getMaxMp() + "</td><td width=70>Pvp Kills: " + player.getPvpKills() + "</td></tr>");
		replyMSG.append("<tr><td width=40>Load:</td><td width=70>" + player.getCurrentLoad() + "</td><td width=70>" + player.getMaxLoad() + "</td><td width=70>Pvp Flag: " + player.getPvpFlag() + "</td></tr>");
		replyMSG.append("</table>");
		replyMSG.append("<table width=270><tr><td>Class Template Id: " + player.getClassId() + "/" + player.getClassId().getId() + "</td></tr></table><br>");
		replyMSG.append("<table width=270>");
		replyMSG.append("<tr><td>Note: Fill all values before saving the modifications.</td></tr>");
		replyMSG.append("</table><br>");
		replyMSG.append("<table width=270>");
		replyMSG.append("<tr><td width=50>Hp:</td><td><edit var=\"hp\" width=50></td><td width=50>Mp:</td><td><edit var=\"mp\" width=50></td></tr>");
		replyMSG.append("<tr><td width=50>Pvp Flag:</td><td><edit var=\"pvpflag\" width=50></td><td width=50>Karma:</td><td><edit var=\"karma\" width=50></td></tr>");
		replyMSG.append("<tr><td width=50>Class Id:</td><td><edit var=\"classid\" width=50></td><td width=50>Pvp Kills:</td><td><edit var=\"pvpkills\" width=50></td></tr>");
		replyMSG.append("</table><br>");
		replyMSG.append("<center><button value=\"Save Changes\" action=\"bypass -h admin_save_modifications $hp $mp $karma $pvpflag $pvpkills $classid\" width=80 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></center><br>");
		replyMSG.append("</body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	private void showCharacterActions(L2Player activeChar)
	{
		L2Object target = activeChar.getTarget();
		L2Player player;
		if(target != null && target.isPlayer)
			player = (L2Player) target;
		else
			return;

		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);

		StringBuffer replyMSG = new StringBuffer("<html><body>");
		replyMSG.append("<table width=260><tr>");
		replyMSG.append("<td width=40><button value=\"Main\" action=\"bypass -h admin_admin\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("<td width=180><center>Character Selection Menu</center></td>");
		replyMSG.append("<td width=40><button value=\"Back\" action=\"bypass -h admin_current_player\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("</tr></table><br><br>");
		replyMSG.append("<center>Admin Actions for: " + player.getName() + "</center><br>");
		replyMSG.append("<center><table width=200><tr>");
		replyMSG.append("<td width=100>Argument(*):</td><td width=100><edit var=\"arg\" width=100></td>");
		replyMSG.append("</tr></table><br></center>");
		replyMSG.append("<table width=270>");

		replyMSG.append("<tr><td width=90><button value=\"Teleport\" action=\"bypass -h admin_teleportto " + player.getName() + "\" width=85 height=20 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("<td width=90><button value=\"Recall\" action=\"bypass -h admin_recall " + player.getName() + "\" width=85 height=20 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("<td width=90></td></tr>");

		replyMSG.append("</body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	// FIXME: needs removal, whole thing needs to use getTarget()
	private void findCharacter(L2Player activeChar, String CharacterToFind)
	{
		Collection<L2Player> allPlayers = L2World.getAllPlayers();
		L2Player[] players = allPlayers.toArray(new L2Player[allPlayers.size()]);
		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);
		int CharactersFound = 0;

		StringBuffer replyMSG = new StringBuffer("<html><body>");
		replyMSG.append("<table width=260><tr>");
		replyMSG.append("<td width=40><button value=\"Main\" action=\"bypass -h admin_admin\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("<td width=180><center>Character Selection Menu</center></td>");
		replyMSG.append("<td width=40><button value=\"Back\" action=\"bypass -h admin_show_characters 0\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("</tr></table>");
		replyMSG.append("<br><br>");

		for(L2Player element : players)
			if(element.getName().startsWith(CharacterToFind))
			{
				CharactersFound = CharactersFound + 1;
				replyMSG.append("<table width=270>");
				replyMSG.append("<tr><td width=80>Name</td><td width=110>Class</td><td width=40>Level</td></tr>");
				replyMSG.append("<tr><td width=80><a action=\"bypass -h admin_character_list " + element.getName() + "\">" + element.getName() + "</a></td><td width=110>" + element.getTemplate().className + "</td><td width=40>" + element.getLevel() + "</td></tr>");
				replyMSG.append("</table>");
			}

		if(CharactersFound == 0)
		{
			replyMSG.append("<table width=270>");
			replyMSG.append("<tr><td width=270>Your search did not find any characters.</td></tr>");
			replyMSG.append("<tr><td width=270>Please try again.<br></td></tr>");
			replyMSG.append("</table><br>");
			replyMSG.append("<center><table><tr><td>");
			replyMSG.append("<edit var=\"character_name\" width=80></td><td><button value=\"Find\" action=\"bypass -h admin_find_character $character_name\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\">");
			replyMSG.append("</td></tr></table></center>");
		}
		else
		{
			replyMSG.append("<center><br>Found " + CharactersFound + " character");

			if(CharactersFound == 1)
				replyMSG.append(".");
			else if(CharactersFound > 1)
				replyMSG.append("s.");
		}

		replyMSG.append("</center></body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	private void addExpSp(final L2Player activeChar)
	{
		final L2Object target = activeChar.getTarget();
		L2Player player;
		if(target != null && target.isPlayer && (activeChar == target || activeChar.getPlayerAccess().CanEditCharAll))
			player = (L2Player) target;
		else
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		final NpcHtmlMessage adminReply = new NpcHtmlMessage(5);

		final StringBuffer replyMSG = new StringBuffer("<html><body>");
		replyMSG.append("<table width=260><tr>");
		replyMSG.append("<td width=40><button value=\"Main\" action=\"bypass -h admin_admin\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("<td width=180><center>Character Selection Menu</center></td>");
		replyMSG.append("<td width=40><button value=\"Back\" action=\"bypass -h admin_current_player\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("</tr></table>");
		replyMSG.append("<br><br>");
		replyMSG.append("<table width=270><tr><td>Name: " + player.getName() + "</td></tr>");
		replyMSG.append("<tr><td>Lv: " + player.getLevel() + " " + player.getTemplate().className + "</td></tr>");
		replyMSG.append("<tr><td>Exp: " + player.getExp() + "</td></tr>");
		replyMSG.append("<tr><td>Sp: " + player.getSp() + "</td></tr></table>");
		replyMSG.append("<br><table width=270><tr><td>Note: Dont forget that modifying players skills can</td></tr>");
		replyMSG.append("<tr><td>ruin the game...</td></tr></table><br>");
		replyMSG.append("<table width=270><tr><td>Note: Fill all values before saving the modifications.,</td></tr>");
		replyMSG.append("<tr><td>Note: Use 0 if no changes are needed.</td></tr></table><br>");
		replyMSG.append("<center><table><tr>");
		replyMSG.append("<td>Exp: <edit var=\"exp_to_add\" width=50></td>");
		replyMSG.append("<td>Sp:  <edit var=\"sp_to_add\" width=50></td>");
		replyMSG.append("<td>&nbsp;<button value=\"Save Changes\" action=\"bypass -h admin_add_exp_sp $exp_to_add $sp_to_add\" width=80 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("</tr></table></center>");
		replyMSG.append("</body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	private void adminAddExpSp(final L2Player activeChar, final String ExpSp)
	{
		if(!activeChar.getPlayerAccess().CanEditCharAll)
		{
			activeChar.sendMessage("You have not enough privileges, for use this function.");
			return;
		}

		final L2Object target = activeChar.getTarget();
		if(target == null || !target.isPlayer)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.SELECT_TARGET));
			return;
		}

		L2Player player = (L2Player) target;
		final StringTokenizer st = new StringTokenizer(ExpSp);
		if(st.countTokens() != 2)
			addExpSp(activeChar);
		else
		{
			final String exp = st.nextToken();
			final String sp = st.nextToken();
			long expval = 0;
			long spval = 0;
			try
			{
				expval = Long.parseLong(exp);
				spval = Long.parseLong(sp);
			}
			catch(final NumberFormatException e)
			{
				// Wrong number (maybe it's too big?)
				activeChar.sendMessage("Wrong number format.");
			}
			if(expval != 0 || spval != 0)
			{
				// Common character information
				player.sendMessage("Admin is adding you " + expval + " exp and " + spval + " SP.");
				player.addExpAndSp(expval, spval, false, false);

				// Admin information
				activeChar.sendMessage("Added " + expval + " exp and " + spval + " SP to " + player.getName() + ".");

				Log.add("Added " + expval + " exp and " + spval + " SP to " + player.getName(), "gm_ext_actions", activeChar);
			}
		}
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"file://com_edit_character.htm",
				"file://com_character_actions.htm",
				"file://com_current_player.htm",
				"file://com_nokarma.htm",
				"file://com_setkarma.htm",
				"file://com_character_list.htm",
				"file://com_show_characters.htm",
				"file://com_find_character.htm",
				"file://com_save_modifications.htm",
				"file://com_rec.htm",
				"file://com_settitle.htm",
				"file://com_setname.htm",
				"file://com_setsex.htm",
				"file://com_setcolor.htm",
				"file://com_add_exp_sp_to_character.htm",
				"file://com_add_exp_sp.htm",
				"file://com_sethero.htm",
				"Описание команды //setnoble",
				"Описание для //trans" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR };
	}
}