package commands.admin;

import javolution.util.FastList;
import l2r.Config;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Effect;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.L2WorldRegion;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.serverpackets.Earthquake;
import l2r.gameserver.serverpackets.NpcInfo;
import l2r.gameserver.serverpackets.SocialAction;
import l2r.gameserver.serverpackets.StopMove;
import l2r.gameserver.serverpackets.UserInfo;
import l2r.gameserver.tables.SkillTable;
import l2r.util.Rnd;

import java.util.StringTokenizer;

@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public class AdminEffects implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_invis",
			"admin_vis",
			"admin_earthquake",
			"admin_bighead",
			"admin_shrinkhead",
			"admin_unpara_all",
			"admin_para_all",
			"admin_unpara",
			"admin_para",
			"admin_polyself",
			"admin_unpolyself",
			"admin_changename",
			"admin_gmspeed",
			"admin_invul",
			"admin_setinvul",
			"admin_social",
			"admin_abnormal" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().GodMode)
			return false;

		if(command.startsWith("admin_social"))
		{
			String[] wordList = command.split(" ");
			int val;
			if(wordList.length == 1)
				val = Rnd.get(1, 7);
			else
				try
				{
					val = Integer.parseInt(wordList[1]);
				}
				catch(NumberFormatException nfe)
				{
					activeChar.sendMessage("Specify a valid social action number.");
					return false;
				}
			L2Character target = (L2Character) activeChar.getTarget();
			if(target == null || target == activeChar)
				activeChar.broadcastPacket(new SocialAction(activeChar.getObjectId(), val));
			else
				target.broadcastPacket(new SocialAction(target.getObjectId(), val));
		}
		else if(command.startsWith("admin_gmspeed"))
			try
			{
				int val = Integer.parseInt(command.substring(14));
				FastList<L2Effect> superhaste = activeChar.getEffectsBySkillId(7029);
				int sh_level = superhaste == null ? 0 : superhaste.size() == 0 ? 0 : superhaste.get(0).getSkill().getLevel();

				if(val == 0)
				{
					if(sh_level != 0)
						activeChar.doCast(SkillTable.getInstance().getInfo(7029, sh_level), activeChar, true); //снимаем еффект
					activeChar.unsetVar("gm_gmspeed");
				}
				else if(val >= 1 && val <= 4)
				{
					if(Config.SAVE_GM_EFFECTS)
						activeChar.setVar("gm_gmspeed", String.valueOf(val));
					if(val != sh_level)
					{
						if(sh_level != 0)
							activeChar.doCast(SkillTable.getInstance().getInfo(7029, sh_level), activeChar, true); //снимаем еффект
						activeChar.doCast(SkillTable.getInstance().getInfo(7029, val), activeChar, true);
					}
				}
				else
					activeChar.sendMessage("Use //gmspeed value = [0...4].");
			}
			catch(Exception e)
			{
				activeChar.sendMessage("Use //gmspeed value = [0...4].");
			}
			finally
			{
				activeChar.updateEffectIcons();
			}
		else if(command.equalsIgnoreCase("admin_invis") || command.equalsIgnoreCase("admin_vis"))
		{
			if(activeChar.isInvisible())
			{
				activeChar.setInvisible(false);
				activeChar.broadcastUserInfo();
				if(activeChar.getPet() != null)
					activeChar.getPet().broadcastPetInfo();
			}
			else
			{
				activeChar.setInvisible(true);
				activeChar.sendPacket(new UserInfo(activeChar));

				if(activeChar.getCurrentRegion() != null)
					for(L2WorldRegion neighbor : activeChar.getCurrentRegion().getNeighbors())
						if(neighbor != null)
							neighbor.removePlayerFromOtherPlayers(activeChar);
			}
		}
		else if(command.equalsIgnoreCase("admin_earthquake"))
			try
			{
				String val = command.substring(17);
				StringTokenizer st = new StringTokenizer(val);
				String val1 = st.nextToken();
				int intensity = Integer.parseInt(val1);
				String val2 = st.nextToken();
				int duration = Integer.parseInt(val2);
				activeChar.broadcastPacket(new Earthquake(activeChar.getLoc(), intensity, duration));
			}
			catch(Exception e)
			{}
		else if(command.equalsIgnoreCase("admin_para"))
		{
			String type = "1";
			StringTokenizer st = new StringTokenizer(command);
			try
			{
				st.nextToken();
				type = st.nextToken();
			}
			catch(Exception e)
			{}
			try
			{
				L2Object target = activeChar.getTarget();
				L2Character player;
				if(target.isCharacter)
				{
					player = (L2Character) target;
					if(type.equals("1"))
						player.startAbnormalEffect((short) L2Character.ABNORMAL_EFFECT_HOLD_1);
					else
						player.startAbnormalEffect((short) L2Character.ABNORMAL_EFFECT_HOLD_2);
					player.setParalyzed(true);

					player.broadcastPacket(new StopMove(player));
				}
			}
			catch(Exception e)
			{}
		}
		else if(command.equalsIgnoreCase("admin_unpara"))
			try
			{
				L2Object target = activeChar.getTarget();
				L2Character player = null;
				if(target.isCharacter)
				{
					player = (L2Character) target;
					player.stopAbnormalEffect((short) L2Character.ABNORMAL_EFFECT_HOLD_1);
					player.setParalyzed(false);
				}
			}
			catch(Exception e)
			{}
		else if(command.equalsIgnoreCase("admin_para_all"))
			try
			{
				FastList<L2Player> players = activeChar.getAroundPlayers(1250);
				for(FastList.Node<L2Player> n = players.head(), end = players.tail(); (n = n.getNext()) != end;)
				{
					L2Player player = n.getValue();
					if(!player.isGM())
					{
						player.startAbnormalEffect((short) L2Character.ABNORMAL_EFFECT_HOLD_1);
						player.setParalyzed(true);

						player.broadcastPacket(new StopMove(player));
					}
				}
			}
			catch(Exception e)
			{}
		else if(command.equalsIgnoreCase("admin_unpara_all"))
			try
			{
				for(L2Player player : activeChar.getAroundPlayers(2500))
				{
					player.stopAbnormalEffect((short) L2Character.ABNORMAL_EFFECT_HOLD_1);
					player.setParalyzed(false);
				}
			}
			catch(Exception e)
			{}
		else if(command.equalsIgnoreCase("admin_bighead") || command.equalsIgnoreCase("admin_shrinkhead"))
			try
			{
				L2Character target = (L2Character) activeChar.getTarget();
				if(target == null)
					activeChar.sendPacket(Msg.INCORRECT_TARGET);
				else if(target.isCharacter)
				{
					if(target.getAbnormalEffect() == L2Character.ABNORMAL_EFFECT_BIG_HEAD)
						target.setBigHead(false);
					else
						target.setBigHead(true);
					if(target.isPlayer)
						target.sendMessage("Admin changed your head type.");
				}
			}
			catch(Exception e)
			{}
		else if(command.equalsIgnoreCase("admin_polyself"))
		{
			StringTokenizer st = new StringTokenizer(command);
			try
			{
				st.nextToken();
				String id = st.nextToken();
				activeChar.setPolyInfo("npc", id);
				activeChar.teleToLocation(activeChar.getLoc());
				activeChar.broadcastUserInfo();
			}
			catch(Exception e)
			{}
		}
		else if(command.equalsIgnoreCase("admin_unpolyself"))
		{
			activeChar.setPolyInfo(null, "1");
			activeChar.decayMe();
			activeChar.spawnMe(activeChar.getLoc());
			activeChar.broadcastUserInfo();
		}
		else if(command.equalsIgnoreCase("admin_changename"))
			try
			{
				String name = command.substring(17);
				String oldName = "null";
				try
				{
					L2Object target = activeChar.getTarget();
					L2Character player = null;
					if(target == null)
					{
						player = activeChar;
						oldName = activeChar.getName();
					}
					else if(target.isCharacter)
					{
						player = (L2Character) target;
						oldName = player.getName();
					}
					else
						return false;
					if(player.isPlayer)
					{
						L2World.removeObject(player);
						player.decayMe();
					}
					player.setName(name);
					if(player.isPlayer)
						player.spawnMe();

					if(player.isPlayer)
						player.broadcastUserInfo();
					else if(player instanceof L2NpcInstance)
						player.broadcastPacket(new NpcInfo((L2NpcInstance) player, null));

					activeChar.sendMessage("Changed name from " + oldName + " to " + name + ".");
				}
				catch(Exception e)
				{}
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.equalsIgnoreCase("admin_invul"))
		{
			handleInvul(activeChar);
			if(activeChar.isInvul())
			{
				if(Config.SAVE_GM_EFFECTS)
					activeChar.setVar("gm_invul", "true");
			}
			else
				activeChar.unsetVar("gm_invul");

		}
		else if(command.equalsIgnoreCase("admin_setinvul"))
		{
			L2Object target = activeChar.getTarget();
			if(target.isPlayer)
				handleInvul((L2Player) target);
		}
		else if(command.startsWith("admin_abnormal"))
		{
			String[] wordList = command.split(" ");
			int val;
			try
			{
				val = Integer.parseInt(wordList[1]);
			}
			catch(Exception e)
			{
				activeChar.sendMessage("Specify a valid abnormal effect.");
				return false;
			}

			switch(val)
			{
				case 1:
					activeChar.setCustomEffect(0x01000000);
					break;
				case 2:
					activeChar.setCustomEffect(0x02000000);
					break;
				case 3:
					activeChar.setCustomEffect(0x04000000);
					break;
				case 4:
					activeChar.setCustomEffect(0x08000000);
					break;
				case 5:
					activeChar.setCustomEffect(0x10000000);
					break;
				case 6:
					activeChar.setCustomEffect(0x20000000);
					break;
				case 7:
					activeChar.setCustomEffect(0x40000000);
					break;
				case 8:
					activeChar.setCustomEffect(0x80000000);
					break;
			}

			activeChar.updateAbnormalEffect();
		}

		return true;
	}

	private void handleInvul(L2Player activeChar)
	{
		if(activeChar.isInvul())
		{
			activeChar.setIsInvul(false);
			if(activeChar.getPet() != null)
				activeChar.getPet().setIsInvul(false);
			activeChar.sendMessage(activeChar.getName() + " is now mortal.");
		}
		else
		{
			activeChar.setIsInvul(true);
			if(activeChar.getPet() != null)
				activeChar.getPet().setIsInvul(true);
			activeChar.sendMessage(activeChar.getName() + " is now immortal.");
		}
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"Описание команды invis",
				"Описание команды vis",
				"Описание команды earthquake",
				"Описание команды bighead",
				"Описание команды shrinkhead",
				"Описание команды unpara_all",
				"Описание команды para_all",
				"Описание команды unpara",
				"Описание команды para",
				"Описание команды polyself",
				"Описание команды unpolyself",
				"Описание команды changename",
				"Описание команды gmspeed",
				"Описание команды invul",
				"Описание команды setinvul",
				"Описание команды social",
				"Описание команды abnormal" };

	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2 };
	}
}
