package commands.admin;

import l2r.extensions.multilang.CustomMessage;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.geodata.geoeditorcon.GeoEditorConnector;
import l2r.gameserver.geodata.loader.GeoLoader;
import l2r.gameserver.geodata.loader.GeoLoaderFactory;
import l2r.gameserver.geodata.loader.GeoFileInfo;
import l2r.gameserver.geodata.network.GeoConfig;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.geodata.GeoEngine;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.model.L2Player;

import java.io.File;

public class AdminGeodata implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_geo_z",
			"admin_geo_type",
			"admin_geo_nswe",
			"admin_geo_los",
			"admin_geo_bug",
			"admin_geo_load",
			"admin_geo_unload",
			"admin_geo_enabled",
			"admin_geo_disabled",
			"admin_geo_info",
			"admin_geoeditor_connect",
			"admin_geoeditor_join",
			"admin_geoeditor_leave" };

	private enum CommandEnum
	{
		admin_geo_z,
		admin_geo_type,
		admin_geo_nswe,
		admin_geo_los,
		admin_geo_load,
		admin_geo_unload,
		admin_geo_bug,
		admin_geo_enabled,
		admin_geo_disabled,
		admin_geo_info,
		admin_geoeditor_connect,
		admin_geoeditor_join,
		admin_geoeditor_leave
	}

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanReload)
			return false;

		String[] wordList = command.split(" ");
		CommandEnum comm;

		try
		{
			comm = CommandEnum.valueOf(wordList[0]);
		}
		catch(Exception e)
		{
			return false;
		}

		CommandEnum commandEnum = comm;
		switch(commandEnum)
		{
			case admin_geo_z:
			case admin_geo_type:
			case admin_geo_nswe:
			case admin_geo_los:
			case admin_geo_load:
			case admin_geo_unload:
			case admin_geo_bug:
			case admin_geo_info:
				if(GeoConfig.GEODATA_MODE == 0)
				{
					activeChar.sendMessage("Геодата неактивна.");
					return true;
				}
				break;
		}

		switch(commandEnum)
		{
			case admin_geo_z:
				activeChar.sendMessage("GeoEngine: Geo_Z = " + GeoClient.getInstance().getHeight(activeChar.getLoc()) + " Loc_Z = " + activeChar.getZ());
				break;
			case admin_geo_type:
				int type = GeoClient.getInstance().getType(activeChar.getX(), activeChar.getY());
				activeChar.sendMessage("GeoEngine: Geo_Type = " + type);
				break;
			case admin_geo_nswe:
				String result = "";
				int nswe = GeoClient.getInstance().getNSWE(activeChar.getX(), activeChar.getY(), activeChar.getZ());
				if((nswe & 8) == 0)
					result += " N";
				if((nswe & 4) == 0)
					result += " S";
				if((nswe & 2) == 0)
					result += " W";
				if((nswe & 1) == 0)
					result += " E";
				activeChar.sendMessage("GeoEngine: Geo_NSWE -> " + nswe + "->" + result);
				break;
			case admin_geo_los:
				if(activeChar.getTarget() != null)
					if(GeoClient.getInstance().canSeeTarget(activeChar, activeChar.getTarget()))
						activeChar.sendMessage("GeoEngine: Can See Target");
					else
						activeChar.sendMessage("GeoEngine: Can't See Target");
				else
					activeChar.sendMessage("None Target!");
				break;
			case admin_geo_load:
				if(wordList.length != 2)
					activeChar.sendMessage("Usage: //admin_geo_load <filename>");
				else
					try
					{
						File file = new File("data/geodata/" + wordList[1]);
						if(file.exists() && !file.isDirectory()){
							activeChar.sendMessage("GeoEngine: File found, making attempt to load geodata.");

							GeoLoader geoLoader =  GeoLoaderFactory.getInstance().getGeoLoader(file);

							if(geoLoader != null){
								activeChar.sendMessage("GeoEngine: Using " + geoLoader.getClass().getSimpleName());

								GeoFileInfo geoFileInfo = geoLoader.readFile(file);

								if(geoFileInfo != null && geoFileInfo.getData() != null){

									int x = geoFileInfo.getX() - GeoEngine.GEODATA_ARRAY_OFFSET_X;
									int y = geoFileInfo.getY() - GeoEngine.GEODATA_ARRAY_OFFSET_Y;

									GeoEngine.addToGeoArray(x, y, geoFileInfo.getData());

									activeChar.sendMessage("GeoEngine: File " + file.getName() + " was successfully loaded!");
								} else {
									activeChar.sendMessage("GeoEngine: Loading aborted, file is corrupted?");
								}

							} else {
								activeChar.sendMessage("GeoEngine: file "+ file.getName() + " is not supported.");
							}

						} else {
							activeChar.sendMessage("GeoEngine: file " + file.getName() + " missing or not readable");
						}


//						if(GeoEngine.LoadGeodataFile(rx, ry))
//							activeChar.sendMessage("GeoEngine: Регион [" + rx + "," + ry + "] успешно загружен");
//						else
//							activeChar.sendMessage("GeoEngine: Регион [" + rx + "," + ry + "] не загрузился");
					}
					catch(Exception e)
					{
						activeChar.sendMessage(new CustomMessage("common.Error", activeChar));
					}
				break;
			case admin_geo_unload:
				/*
				if(wordList.length != 3)
					activeChar.sendMessage("Формат: //admin_geo_unload <regionX> <regionY>");
				else
					try
					{
						byte rx = Byte.parseByte(wordList[1]);
						byte ry = Byte.parseByte(wordList[2]);
						GeoData.unloadGeodataFile(rx, ry);
						activeChar.sendMessage("GeoEngine: Регион [" + rx + "," + ry + "] выгружен.");
					}
					catch(Exception e)
					{
						activeChar.sendMessage(new CustomMessage("common.Error", activeChar));
					}
				*/
				activeChar.sendMessage("Временно не доступно.");
				break;
			case admin_geo_bug:
				/*
				if(wordList.length != 3)
					activeChar.sendMessage("Формат: //admin_geo_bug комментарии");
				else
					try
					{
						GeoData.getInstance().addGeoDataBug(activeChar, wordList[1]);
					}
					catch(StringIndexOutOfBoundsException e)
					{
						activeChar.sendMessage(new CustomMessage("common.Error", activeChar));
					}
				*/
				activeChar.sendMessage("Временно не доступно.");
				break;
			case admin_geo_enabled:
				//Config.GEODATA = true;
				//activeChar.sendMessage("Геодата включена.");
				activeChar.sendMessage("Временно не доступно.");
				break;
			case admin_geo_disabled:
				//Config.GEODATA = false;
				//activeChar.sendMessage("Геодата выключена.");
				activeChar.sendMessage("Временно не доступно.");
				break;
			case admin_geo_info:
				/**
				GeoClient.getInstance().getInfo(activeChar, activeChar.getX(), activeChar.getY(), activeChar.getZ());
				 */
				activeChar.sendMessage("Временно не доступно.");
				break;
			case admin_geoeditor_connect:
				try
				{
					int ticks = Integer.parseInt(command.substring(24));
					GeoEditorConnector.getInstance().connect(activeChar, ticks);
				}
				catch(Exception e)
				{
					activeChar.sendMessage("Usage: //geoeditor_connect <number>");
				}
				break;
			case admin_geoeditor_join:
				GeoEditorConnector.getInstance().join(activeChar);
				break;
			case admin_geoeditor_leave:
				GeoEditorConnector.getInstance().leave(activeChar);
				break;
		}

		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"Описание команды geo_z",
				"Описание команды geo_type",
				"Описание команды geo_nswe",
				"Описание команды geo_los",
				"Описание команды geo_bug",
				"Описание команды geo_load",
				"Описание команды geo_unload",
				"Описание команды geo_enabled",
				"Описание команды geo_disabled",
				"Описание команды geo_info",
				"Описание команды geoeditor_connect",
				"Описание команды geoeditor_join",
				"Описание команды geoeditor_leave" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3 };
	}
}