package commands.admin;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.serverpackets.Say2;
import l2r.gameserver.tables.GmListTable;

/**
 * This class handles following admin commands: - gmchat text = sends text to
 * all online GM's
 */
public class AdminGmChat implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = { "admin_gmchat", "admin_snoop" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanAnnounce)
			return false;
		if(command.startsWith("admin_gmchat"))
			handleGmChat(command, activeChar);
		else if(command.startsWith("admin_snoop"))
			snoop(command, activeChar);
		return true;
	}

	private void snoop(String command, L2Player activeChar)
	{
		L2Object target = activeChar.getTarget();
		if(target == null)
		{
			activeChar.sendMessage("You must select a target.");
			return;
		}
		if(!target.isPlayer)
		{
			activeChar.sendMessage("Target must be a player.");
			return;
		}
		L2Player player = (L2Player) target;
		player.addSnooper(activeChar);
		activeChar.addSnooped(player);
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	private void handleGmChat(String command, L2Player activeChar)
	{
		try
		{
			String text = command.substring(13);
			Say2 cs = new Say2(0, 9, activeChar.getName(), text);
			GmListTable.broadcastToGMs(cs);
		}
		catch(StringIndexOutOfBoundsException e)
		{
			// empty message.. ignore
		}
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] { "admin_gmchat", "admin_snoop" };
	}

	public String[] getCommandGroup()
	{
		return new String[] { AdminCommandHandler.GROUP_MANAGESERVER3, AdminCommandHandler.GROUP_EDITCHAR };
	}
}