package commands.admin;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.util.Log;

public class AdminHeal implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = { "admin_heal" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().Heal)
			return false;
		if(command.equals("admin_heal"))
			handleRes(activeChar);
		else if(command.startsWith("admin_heal"))
			try
			{
				String healTarget = command.substring(11);
				handleRes(activeChar, healTarget);
			}
			catch(StringIndexOutOfBoundsException e)
			{
				activeChar.sendMessage("Incorrect target/radius specified.");
			}
		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	private void handleRes(L2Player activeChar)
	{
		handleRes(activeChar, null);
	}

	private void handleRes(L2Player activeChar, String player)
	{

		L2Object obj = activeChar.getTarget();
		if(player != null)
		{
			L2Player plyr = L2World.getPlayer(player);

			if(plyr != null)
				obj = plyr;
			else
			{
				int radius = Math.max(Integer.parseInt(player), 100);
				for(L2Character character : activeChar.getKnownCharacters(radius))
				{
					character.setCurrentHpMp(character.getMaxHp(), character.getMaxMp());
					if(character.isPlayer)
						character.setCurrentCp(character.getMaxCp());
				}
				activeChar.sendMessage("Healed within " + radius + " unit radius.");
				return;
			}
		}

		if(obj == null)
			obj = activeChar;

		if(obj.isCharacter)
		{
			L2Character target = (L2Character) obj;
			target.setCurrentHpMp(target.getMaxHp(), target.getMaxMp());
			if(target.isPlayer)
				target.setCurrentCp(target.getMaxCp());
			Log.add("heal character " + target.getName(), "gm_ext_actions", activeChar);
		}
		else
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] { "Описание команды heal" };
	}

	public String[] getCommandGroup()
	{
		return new String[] { AdminCommandHandler.GROUP_EDITCHAR };
	}
}