package commands.admin;

import java.util.StringTokenizer;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.util.Location;

/**
 * This class handles following admin commands: - handles ever admin menu
 * command
 */
public class AdminMenu implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_char_manage",
			"admin_teleport_character_to_menu",
			"admin_recall_char_menu",
			"admin_goto_char_menu",
			"admin_kick_menu",
			"admin_kill_menu",
			"admin_ban_menu",
			"admin_unban_menu" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().Menu)
			return false;

		if(command.equals("admin_char_manage"))
			AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
		else if(command.startsWith("admin_teleport_character_to_menu"))
		{
			String[] data = command.split(" ");
			if(data.length == 5)
			{
				String playerName = data[1];
				L2Player player = L2World.getPlayer(playerName);
				if(player != null)
					teleportCharacter(player, new Location(Integer.parseInt(data[2]), Integer.parseInt(data[3]), Integer.parseInt(data[4])), activeChar);
			}
			AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
		}
		else if(command.startsWith("admin_recall_char_menu"))
			try
			{
				String targetName = command.substring(23);
				L2Player player = L2World.getPlayer(targetName);
				teleportCharacter(player, activeChar.getLoc(), activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.startsWith("admin_goto_char_menu"))
			try
			{
				String targetName = command.substring(21);
				L2Player player = L2World.getPlayer(targetName);
				teleportToCharacter(activeChar, player);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.equals("admin_kill_menu"))
		{
			L2Object obj = activeChar.getTarget();
			StringTokenizer st = new StringTokenizer(command);
			if(st.countTokens() > 1)
			{
				st.nextToken();
				String player = st.nextToken();
				L2Player plyr = L2World.getPlayer(player);
				if(plyr != null)
					activeChar.sendMessage("You kicked " + plyr.getName() + " from the game.");
				else
					activeChar.sendMessage("Player " + player + " not found in game.");
				obj = plyr;
			}
			if(obj != null && obj.isCharacter)
			{
				L2Character target = (L2Character) obj;
				target.reduceCurrentHp(target.getMaxHp() + 1, activeChar, true, true, true);
			}
			else
				activeChar.sendPacket(Msg.INCORRECT_TARGET);
			AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
		}
		else if(command.startsWith("admin_kick_menu"))
		{
			StringTokenizer st = new StringTokenizer(command);
			if(st.countTokens() > 1)
			{
				st.nextToken();
				String player = st.nextToken();
				L2Player plyr = L2World.getPlayer(player);
				if(plyr != null)
					plyr.logout(false, false, true);
				if(plyr != null)
					activeChar.sendMessage("You kicked " + plyr.getName() + " from the game.");
				else
					activeChar.sendMessage("Player " + player + " not found in game.");
			}
			AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
		}
		else if(command.startsWith("admin_ban_menu"))
		{
			StringTokenizer st = new StringTokenizer(command);
			if(st.countTokens() > 1)
			{
				st.nextToken();
				String player = st.nextToken();
				L2Player plyr = L2World.getPlayer(player);
				if(plyr != null)
				{
					plyr.setAccountAccesslevel(-100, "admin_ban_menu", -1);
					plyr.logout(false, false, true);
				}
			}
			AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
		}
		else if(command.startsWith("admin_unban_menu"))
		{
			StringTokenizer st = new StringTokenizer(command);
			if(st.countTokens() > 1)
			{
				st.nextToken();
				String player = st.nextToken();
				L2Player plyr = L2World.getPlayer(player);
				if(plyr != null)
					plyr.setAccountAccesslevel(0, "admin_unban_menu", 0);
			}
			AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
		}
		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	private void teleportCharacter(L2Player player, Location loc, L2Player activeChar)
	{
		if(player != null)
		{
			player.sendMessage("Admin is teleporting you.");
			player.teleToLocation(loc);
		}
		AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
	}

	private void teleportToCharacter(L2Player activeChar, L2Object target)
	{
		L2Player player;
		if(target != null && target.isPlayer)
			player = (L2Player) target;
		else
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		if(player.getObjectId() == activeChar.getObjectId())
			activeChar.sendMessage("You cannot self teleport.");
		else
		{
			activeChar.teleToLocation(player.getLoc());
			activeChar.sendMessage("You have teleported to character " + player.getName() + ".");
		}
		AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"Описание команды char_manage",
				"Описание команды teleport_character_to_menu",
				"Описание команды recall_char_menu",
				"Описание команды goto_char_menu",
				"Описание команды kick_menu",
				"Описание команды kill_menu",
				"Описание команды ban_menu",
				"Описание команды unban_menu" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_NONE,
				AdminCommandHandler.GROUP_NONE,
				AdminCommandHandler.GROUP_NONE,
				AdminCommandHandler.GROUP_NONE,
				AdminCommandHandler.GROUP_NONE,
				AdminCommandHandler.GROUP_NONE,
				AdminCommandHandler.GROUP_NONE,
				AdminCommandHandler.GROUP_NONE };
	}
}