package commands.admin;

import l2r.Config;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.entity.Olympiad;
import l2r.util.Log;

public class AdminOlympiad implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = { "admin_oly_start", "admin_oly_save", "admin_manualhero", "admin_add_oly_points" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(command.startsWith("admin_oly_start"))
		{
			if(!Olympiad.getInstance().startOlympiad())
				activeChar.sendMessage("Olympiad already started.");
		}
		else if(command.startsWith("admin_oly_save"))
		{
			if(!Config.ENABLE_OLYMPIAD)
				return false;

			try
			{
				Olympiad.getInstance().save();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
			activeChar.sendMessage("olympaid data saved.");
		}
		else if(command.startsWith("admin_manualhero"))
		{
			if(!Config.ENABLE_OLYMPIAD)
				return false;

			try
			{
				Olympiad.getInstance().manualSelectHeroes();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
			activeChar.sendMessage("Heroes formed.");
		}
		else if(command.startsWith("admin_add_oly_points"))
		{
			String[] wordList = command.split(" ");
			if(wordList.length < 3)
			{
				activeChar.sendMessage("Command syntax: //add_oly_points <char_name> <point_to_add>");
				activeChar.sendMessage("This command can be applied only for online players.");
				return false;
			}

			L2Player player = L2World.getPlayer(wordList[1]);
			if(player == null)
			{
				activeChar.sendMessage("Character " + wordList[1] + " not found in game.");
				return false;
			}

			int pointToAdd;

			try
			{
				pointToAdd = Integer.parseInt(wordList[2]);
			}
			catch(NumberFormatException e)
			{
				activeChar.sendMessage("Please specify integer value for olympiad points.");
				return false;
			}

			int curPoints = Olympiad.getInstance().getNoblePoints(player.getObjectId());
			Olympiad.getInstance().setNoblePoints(player.getObjectId(), curPoints + pointToAdd);
			int newPoints = Olympiad.getInstance().getNoblePoints(player.getObjectId());

			activeChar.sendMessage("Added " + pointToAdd + " points to character " + player.getName());
			activeChar.sendMessage("Old points: " + curPoints + ", new points: " + newPoints);

			Log.add("add olympiad points to player " + player.getName(), "gm_ext_actions", activeChar);
		}

		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public String[] getDescription()
	{
		return new String[] { "Описание команды oly_start", "Описание команды oly_save", "Описание команды manualhero", "Описание команды add_oly_points" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4 };
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}