package commands.admin;

import java.util.StringTokenizer;

import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.instancemanager.SiegeManager;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.serverpackets.PledgeShowInfoUpdate;
import l2r.gameserver.serverpackets.PledgeStatusChanged;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.serverpackets.UserInfo;
import l2r.gameserver.tables.ClanTable;

/**
 * Pledge Manipulation //pledge <create|dismiss|setlevel|resetcreate|resetwait|addrep>
 */
public class AdminPledge implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = { "admin_pledge" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(activeChar.getPlayerAccess() == null || !activeChar.getPlayerAccess().CanEditCharAll || activeChar.getTarget() == null || !activeChar.getTarget().isPlayer)
			return false;

		L2Player target = (L2Player) activeChar.getTarget();

		if(command.startsWith("admin_pledge"))
		{
			StringTokenizer st = new StringTokenizer(command);
			st.nextToken();

			String action = st.nextToken(); // create|dismiss|setlevel|resetcreate|resetwait|addrep

			if(action.equals("create"))
				try
				{
					String pledgeName = st.nextToken();
					L2Clan clan = ClanTable.getInstance().createClan(target, pledgeName);
					if(clan != null)
					{
						target.sendPacket(new PledgeShowInfoUpdate(clan));
						target.sendPacket(new UserInfo(target));
						target.sendPacket(new SystemMessage(SystemMessage.CLAN_HAS_BEEN_CREATED));
						return true;
					}
				}
				catch(Exception e)
				{}
			else if(action.equals("dismiss"))
			{
				if(target.getClan() == null || target.getObjectId() != target.getClan().getLeaderId())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.ONLY_THE_CLAN_LEADER_IS_ENABLED));
					return false;
				}

				SiegeManager.removeSiegeSkills(target);
				SystemMessage sm = new SystemMessage(SystemMessage.CLAN_HAS_DISPERSED);
				for(L2Player clanMember : target.getClan().getOnlineMembers(null))
				{
					clanMember.setClan(null);
					clanMember.setTitle(null);
					clanMember.sendPacket(sm);
					clanMember.broadcastUserInfo();
				}

				ThreadConnection con = null;
				FiltredPreparedStatement statement = null;
				try
				{
					con = L2DatabaseFactory.getInstance().getConnection();
					statement = con.prepareStatement("UPDATE characters SET clanid = 0 WHERE clanid=?");
					statement.setInt(1, target.getClanId());
					statement.execute();
					DatabaseUtils.closeStatement(statement);

					statement = con.prepareStatement("DELETE FROM clan_data WHERE clan_id=?");
					statement.setInt(1, target.getClanId());
					statement.execute();
					DatabaseUtils.closeStatement(statement);
					statement = null;
					target.sendPacket(sm);
					target.broadcastUserInfo();
				}
				catch(Exception e)
				{}
				finally
				{
					DatabaseUtils.closeDatabaseCS(con, statement);
				}
				return true;
			}
			else if(action.equals("setlevel"))
			{
				if(target.getClan() == null || target.getObjectId() != target.getClan().getLeaderId())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.ONLY_THE_CLAN_LEADER_IS_ENABLED));
					return false;
				}

				try
				{
					byte level = Byte.parseByte(st.nextToken());
					L2Clan clan = target.getClan();

					activeChar.sendMessage("You set level " + level + " for clan " + clan.getName());
					clan.setLevel(level);
					clan.updateClanInDB();

					if(level < 4)
						SiegeManager.removeSiegeSkills(target);
					else if(level > 3)
						SiegeManager.addSiegeSkills(target);

					if(level == 5)
						target.sendPacket(new SystemMessage(SystemMessage.NOW_THAT_YOUR_CLAN_LEVEL_IS_ABOVE_LEVEL_5_IT_CAN_ACCUMULATE_CLAN_REPUTATION_POINTS));

					SystemMessage sm = new SystemMessage(SystemMessage.CLANS_SKILL_LEVEL_HAS_INCREASED);
					PledgeShowInfoUpdate pu = new PledgeShowInfoUpdate(clan);

					for(L2Player member : clan.getOnlineMembers(null))
					{
						member.updatePledgeClass();
						member.sendPacket(sm);
						member.sendPacket(pu);
						member.broadcastUserInfo();
					}

					clan.broadcastToOnlineMembers(new PledgeStatusChanged(clan));
					clan.broadcastClanStatus();

					return true;
				}
				catch(Exception e)
				{}
			}
			else if(action.equals("resetcreate"))
			{
				if(target.getClan() == null)
				{
					activeChar.sendPacket(Msg.INCORRECT_TARGET);
					return false;
				}
				target.getClan().setExpelledMemberTime(0);
				activeChar.sendMessage("The penalty for creating a clan has been lifted for" + target.getName());
			}
			else if(action.equals("resetwait"))
			{
				target.setLeaveClanTime(0);
				activeChar.sendMessage("The penalty for leaving a clan has been lifted for " + target.getName());
			}
			else if(action.equals("addrep"))
				try
				{
					int rep = Integer.parseInt(st.nextToken());

					if(target.getClan() == null || target.getClan().getLevel() < 5)
					{
						activeChar.sendPacket(Msg.INCORRECT_TARGET);
						return false;
					}
					target.getClan().incReputation(rep, false, "admin_manual");
					activeChar.sendMessage("Added " + rep + " clan points to clan " + target.getClan().getName() + ".");
				}
				catch(NumberFormatException nfe)
				{
					activeChar.sendMessage("Please specify a number of clan points to add.");
				}
		}

		return false;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] { "file://com_pledge.htm" };
	}

	public String[] getCommandGroup()
	{
		return new String[] { AdminCommandHandler.GROUP_MANAGESERVER4 };
	}
}