package commands.admin;

import l2r.Config;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.Announcements;
import l2r.gameserver.TradeController;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.instancemanager.ZoneManager;
import l2r.gameserver.model.L2Multisell;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.gameserver.tables.FishTable;
import l2r.gameserver.tables.GmListTable;
import l2r.gameserver.tables.NpcTable;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.tables.SpawnTable;
import l2r.gameserver.tables.TerritoryTable;
import l2r.util.Files;
import l2r.util.Strings;

public class AdminReload implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_reload_multisell",
			"admin_reload_announcements",
			"admin_reload_gmaccess",
			"admin_reload_htm",
			"admin_reload_qs",
			"admin_reload_qs_help",
			"admin_reload_loc",
			"admin_reload_skills",
			"admin_reload_npc",
			"admin_reload_spawn",
			"admin_reload_fish",
			"admin_reload_abuse",
			"admin_reload_translit",
			"admin_reload_shops" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanReload)
			return false;
		if(command.startsWith("admin_reload_multisell"))
		{
			try
			{
				L2Multisell.getInstance().reload();
			}
			catch(Exception e)
			{
				return false;
			}
			activeChar.sendMessage("Multisell list reloaded!");
		}
		else if(command.startsWith("admin_reload_gmaccess"))
		{
			try
			{
				Config.loadGMAccess();
				for(L2Player player : L2World.getAllPlayers())
					if(!Config.EVERYBODY_HAS_ADMIN_RIGHTS)
						player.setPlayerAccess(Config.gmlist.get(player.getObjectId()));
					else
						player.setPlayerAccess(Config.gmlist.get(new Integer(0)));
			}
			catch(Exception e)
			{
				return false;
			}
			activeChar.sendMessage("GMAccess reloaded!");
		}
		else if(command.startsWith("admin_reload_htm"))
		{
			Files.cacheClean();
			activeChar.sendMessage("HTML cache clearned.");
		}
		else if(command.equals("admin_reload_announcements"))
		{
			Announcements.getInstance().loadAnnouncements();
			Announcements.getInstance().listAnnouncements(activeChar);
		}
		if(command.startsWith("admin_reload_qs"))
		{
			if(command.endsWith("all"))
				for(L2Player p : L2World.getAllPlayers())
					reloadQuestStates(p);
			else
			{
				L2Object t = activeChar.getTarget();

				if(t != null && t.isPlayer)
				{
					L2Player p = (L2Player) t;
					reloadQuestStates(p);
				}
				else
					reloadQuestStates(activeChar);
			}
			return true;
		}
		else if(command.startsWith("admin_reload_qs_help"))
		{
			activeChar.sendMessage("");
			activeChar.sendMessage("Quest Help:");
			activeChar.sendMessage("reload_qs_help - This Message.");
			activeChar.sendMessage("reload_qs <selected target> - reload all quest states for target.");
			activeChar.sendMessage("reload_qs <no target or target is not player> - reload quests for self.");
			activeChar.sendMessage("reload_qs all - reload quests for all players in world.");
			activeChar.sendMessage("");

			return true;
		}
		else if(command.startsWith("admin_reload_loc"))
		{
			TerritoryTable.getInstance().reloadData();
			ZoneManager.getInstance().reload();
			GmListTable.broadcastMessageToGMs("Locations and zones reloaded.");
		}
		else if(command.startsWith("admin_reload_skills"))
		{
			SkillTable.getInstance().reload();
			GmListTable.broadcastMessageToGMs("Skill table reloaded by " + activeChar.getName() + ".");
			System.out.println("Skill table reloaded by " + activeChar.getName() + ".");
		}
		else if(command.startsWith("admin_reload_npc"))
		{
			NpcTable.getInstance().reloadAllNpc();
			GmListTable.broadcastMessageToGMs("Npc table reloaded.");
		}
		else if(command.startsWith("admin_reload_spawn"))
		{
			SpawnTable.getInstance().reloadAll();
			GmListTable.broadcastMessageToGMs("All npc respawned.");
		}
		else if(command.startsWith("admin_reload_fish"))
		{
			FishTable.getInstance().reload();
			GmListTable.broadcastMessageToGMs("Fish table reloaded.");
		}
		else if(command.startsWith("admin_reload_abuse"))
		{
			Config.abuseLoad();
			GmListTable.broadcastMessageToGMs("Abuse reloaded.");
		}
		else if(command.startsWith("admin_reload_translit"))
		{
			Strings.reload();
			GmListTable.broadcastMessageToGMs("Translit reloaded.");
		}
		else if(command.startsWith("admin_reload_shops"))
		{
			TradeController.reload();
			GmListTable.broadcastMessageToGMs("Shops reloaded.");
		}
		return true;
	}

	private void reloadQuestStates(L2Player p)
	{
		for(QuestState qs : p.getAllQuestsStates())
			p.delQuestState(qs.getQuest().getName());
		Quest.playerEnter(p);
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"Описание команды reload_multisell",
				"Описание команды reload_announcements",
				"Описание команды reload_gmaccess",
				"Описание команды reload_htm",
				"Описание команды reload_qs",
				"Описание команды reload_qs_help",
				"Описание команды reload_loc",
				"Описание команды reload_skills",
				"Описание команды reload_npc",
				"Описание команды reload_spawn",
				"Описание команды reload_fish",
				"Описание команды reload_abuse",
				"Описание команды reload_translit",
				"Описание команды reload_shops" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER4,
				AdminCommandHandler.GROUP_MANAGESERVER5,
				AdminCommandHandler.GROUP_MANAGESERVER5 };
	}
}