package commands.admin;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.serverpackets.Revive;
import l2r.gameserver.serverpackets.SocialAction;
import l2r.gameserver.taskmanager.DecayTaskManager;
import l2r.util.Log;

public class AdminRes implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = { "admin_res" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().Res)
			return false;
		if(command.startsWith("admin_res "))
			handleRes(activeChar, command.split(" ")[1]);
		if(command.equals("admin_res"))
			handleRes(activeChar);
		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	private void handleRes(L2Player activeChar)
	{
		handleRes(activeChar, null);
	}

	private void handleRes(L2Player activeChar, String player)
	{
		L2Object obj = activeChar.getTarget();
		if(player != null)
		{
			L2Player plyr = L2World.getPlayer(player);
			if(plyr != null)
				obj = plyr;
			else
				try
				{
					int radius = Math.max(Integer.parseInt(player), 100);
					for(L2Character character : activeChar.getKnownCharacters(radius))
					{
						character.setCurrentHpMp(character.getMaxHp(), character.getMaxMp());
						character.setCurrentCp(character.getMaxCp());
						if(character.isPlayer)
							((L2Player) character).restoreExp();
						// If the target is an NPC, then abort it's auto decay and respawn.
						else
							DecayTaskManager.getInstance().cancelDecayTask(character);

						character.broadcastPacket(new SocialAction(character.getObjectId(), 15));
						character.broadcastPacket(new Revive(character));
						character.doRevive();
					}
					activeChar.sendMessage("Resurrected within " + radius + " unit radius.");
					return;
				}
				catch(NumberFormatException e)
				{
					activeChar.sendMessage("Enter valid player name or radius");
					return;
				}
		}

		if(obj == null)
			obj = activeChar;
		if(obj.isCharacter)
		{
			L2Character target = (L2Character) obj;
			if(!target.isDead())
				return;

			target.setCurrentHpMp(target.getMaxHp(), target.getMaxMp());
			target.setCurrentCp(target.getMaxCp());
			// GM Resurrection will restore any lost exp
			if(target.isPlayer)
			{
				L2Player deadplayer = (L2Player) target;
				deadplayer.restoreExp();
			}
			target.broadcastPacket(new SocialAction(target.getObjectId(), 15));
			target.broadcastPacket(new Revive(target));
			target.doRevive();
			Log.add("resurrected character " + target.getObjectId() + " " + target.getName(), "gm_ext_actions", activeChar);
		}
		else
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] { "admin_res" };
	}

	public String[] getCommandGroup()
	{
		return new String[] { AdminCommandHandler.GROUP_EDITCHAR };
	}
}