package commands.admin;

import java.util.ArrayList;
import java.util.Collection;
import java.util.StringTokenizer;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2SkillLearn;
import l2r.gameserver.serverpackets.MagicSkillUse;
import l2r.gameserver.serverpackets.NpcHtmlMessage;
import l2r.gameserver.serverpackets.SkillList;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.tables.SkillTreeTable;
import l2r.util.Log;

/**
 * This class handles following admin commands: - show_skills - remove_skills -
 * skill_list - skill_index - add_skill - remove_skill - get_skills -
 * reset_skills - give_all_skills
 */
public class AdminSkill implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_show_skills",
			"admin_remove_skills",
			"admin_skill_list",
			"admin_skill_index",
			"admin_add_skill",
			"admin_remove_skill",
			"admin_get_skills",
			"admin_reset_skills",
			"admin_give_all_skills" };

	private static L2Skill[] adminSkills;

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanEditChar)
			return false;

		if(command.equals("admin_show_skills"))
			showSkillsPage(activeChar);
		else if(command.equals("admin_remove_skills"))
			removeSkillsPage(activeChar);
		else if(command.startsWith("admin_skill_list"))
			AdminHelpPage.showHelpPage(activeChar, "skills.htm");
		else if(command.startsWith("admin_skill_index"))
			try
			{
				String val = command.substring(18);
				AdminHelpPage.showHelpPage(activeChar, "skills/" + val + ".htm");
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.startsWith("admin_add_skill"))
			try
			{
				String val = command.substring(15);
				adminAddSkill(activeChar, val);
			}
			catch(StringIndexOutOfBoundsException e)
			{ // Case of empty character name
				activeChar.sendMessage("Error while adding skill.");
			}
		else if(command.startsWith("admin_remove_skill"))
			try
			{
				String id = command.substring(19);
				Integer idval = Integer.parseInt(id);
				adminRemoveSkill(activeChar, idval);
				activeChar.sendPacket(new SkillList(activeChar));
			}
			catch(StringIndexOutOfBoundsException e)
			{ // Case of empty character name
				activeChar.sendMessage("Error while removing skill.");
			}
		else if(command.equals("admin_get_skills"))
			adminGetSkills(activeChar);
		else if(command.equals("admin_reset_skills"))
			adminResetSkills(activeChar);
		else if(command.equals("admin_give_all_skills"))
			adminGiveAllSkills(activeChar);

		return true;
	}

	/**
	 * This function will give all the skills that the gm target can have at its
	 * level to the traget
	 *
	 * @param activeChar: the gm char
	 */
	private void adminGiveAllSkills(L2Player activeChar)
	{
		L2Object target = activeChar.getTarget();
		L2Player player = null;
		if(target != null && target.isPlayer && (activeChar == target || activeChar.getPlayerAccess().CanEditCharAll))
			player = (L2Player) target;
		else
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}
		int unLearnable = 0;
		int skillCounter = 0;
		ArrayList<L2SkillLearn> skills = SkillTreeTable.getInstance().getAvailableSkills(player, player.getClassId());
		while(skills.size() > unLearnable)
		{
			unLearnable = 0;
			for(L2SkillLearn s : skills)
			{
				L2Skill sk = SkillTable.getInstance().getInfo(s.id, s.skillLevel);
				if(sk == null || !sk.getCanLearn(player.getClassId()))
				{
					unLearnable++;
					continue;
				}
				if(player.getSkillLevel(sk.getId()) == -1)
					skillCounter++;
				player.addSkill(sk, true);
			}
			skills = SkillTreeTable.getInstance().getAvailableSkills(player, player.getClassId());
		}

		player.sendMessage("Admin gave you " + skillCounter + " skills.");
		player.sendPacket(new SkillList(player));
		activeChar.sendMessage("You gave " + skillCounter + " skills to " + player.getName());

		Log.add("gave " + skillCounter + " skills to " + player.getName(), "gm_ext_actions", activeChar);
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	// ok
	private void removeSkillsPage(L2Player activeChar)
	{
		L2Object target = activeChar.getTarget();
		L2Player player;
		if(target.isPlayer && (activeChar == target || activeChar.getPlayerAccess().CanEditCharAll))
			player = (L2Player) target;
		else
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		Collection<L2Skill> skills = player.getAllSkills();

		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);
		StringBuffer replyMSG = new StringBuffer("<html><body>");
		replyMSG.append("<table width=260><tr>");
		replyMSG.append("<td width=40><button value=\"Main\" action=\"bypass -h admin_admin\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("<td width=180><center>Character Selection Menu</center></td>");
		replyMSG.append("<td width=40><button value=\"Back\" action=\"bypass -h admin_show_skills\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("</tr></table>");
		replyMSG.append("<br><br>");
		replyMSG.append("<center>Editing character: " + player.getName() + "</center>");
		replyMSG.append("<br><table width=270><tr><td>Lv: " + player.getLevel() + " " + player.getTemplate().className + "</td></tr></table>");
		replyMSG.append("<br><table width=270><tr><td>Note: Dont forget that modifying players skills can</td></tr>");
		replyMSG.append("<tr><td>ruin the game...</td></tr></table>");
		replyMSG.append("<br><center>Click on the skill you wish to remove:</center>");
		replyMSG.append("<br><table width=270>");
		replyMSG.append("<tr><td width=80>Name:</td><td width=60>Level:</td><td width=40>Id:</td></tr>");
		for(L2Skill element : skills)
			replyMSG.append("<tr><td width=80><a action=\"bypass -h admin_remove_skill " + element.getId() + "\">" + element.getName() + "</a></td><td width=60>" + element.getLevel() + "</td><td width=40>" + element.getId() + "</td></tr>");
		replyMSG.append("</table>");
		replyMSG.append("<br><center><table>");
		replyMSG.append("Remove custom skill:");
		replyMSG.append("<tr><td>Id: </td>");
		replyMSG.append("<td><edit var=\"id_to_remove\" width=110></td></tr>");
		replyMSG.append("</table></center>");
		replyMSG.append("<center><button value=\"Remove skill\" action=\"bypass -h admin_remove_skill $id_to_remove\" width=110 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></center>");
		replyMSG.append("<br><center><button value=\"Back\" action=\"bypass -h admin_current_player\" width=40 height=15></center>");
		replyMSG.append("</body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	// ok
	private void showSkillsPage(L2Player activeChar)
	{
		L2Object target = activeChar.getTarget();
		L2Player player;
		if(target != null && target.isPlayer && (activeChar == target || activeChar.getPlayerAccess().CanEditCharAll))
			player = (L2Player) target;
		else
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);

		StringBuffer replyMSG = new StringBuffer("<html><body>");
		replyMSG.append("<table width=260><tr>");
		replyMSG.append("<td width=40><button value=\"Main\" action=\"bypass -h admin_admin\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("<td width=180><center>Character Selection Menu</center></td>");
		replyMSG.append("<td width=40><button value=\"Back\" action=\"bypass -h admin_current_player\" width=40 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("</tr></table>");
		replyMSG.append("<br><br>");
		replyMSG.append("<center>Editing character: " + player.getName() + "</center>");
		replyMSG.append("<br><table width=270><tr><td>Lv: " + player.getLevel() + " " + player.getTemplate().className + "</td></tr></table>");
		replyMSG.append("<br><table width=270><tr><td>Note: Dont forget that modifying players skills can</td></tr>");
		replyMSG.append("<tr><td>ruin the game...</td></tr></table>");
		replyMSG.append("<br><center><table>");
		replyMSG.append("<tr><td><button value=\"Add skills\" action=\"bypass -h admin_skill_list\" width=70 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("<td><button value=\"Get skills\" action=\"bypass -h admin_get_skills\" width=70 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td></tr>");
		replyMSG.append("<tr><td><button value=\"Delete skills\" action=\"bypass -h admin_remove_skills\" width=70 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td>");
		replyMSG.append("<td><button value=\"Reset skills\" action=\"bypass -h admin_reset_skills\" width=70 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td></tr>");
		replyMSG.append("<tr><td><button value=\"Give All Skills\" action=\"bypass -h admin_give_all_skills\" width=70 height=15 back=\"L2UI_CT1.Button_DF\" fore=\"L2UI_CT1.Button_DF\"></td></tr>");
		replyMSG.append("</table></center>");
		replyMSG.append("</body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	private void adminGetSkills(L2Player activeChar)
	{
		L2Object target = activeChar.getTarget();
		L2Player player;
		if(target.isPlayer && (activeChar == target || activeChar.getPlayerAccess().CanEditCharAll))
			player = (L2Player) target;
		else
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		if(player.getName().equals(activeChar.getName()))
			player.sendMessage("There is no point in doing it on your character.");
		else
		{
			Collection<L2Skill> skills = player.getAllSkills();
			adminSkills = activeChar.getAllSkillsArray();
			for(L2Skill element : adminSkills)
				activeChar.removeSkill(element, true);
			for(L2Skill element : skills)
				activeChar.addSkill(element, true);

			activeChar.sendMessage("You now have all the skills of  " + player.getName() + ".");
		}
		showSkillsPage(activeChar);
	}

	private void adminResetSkills(L2Player activeChar)
	{
		L2Object target = activeChar.getTarget();
		L2Player player = null;
		if(target.isPlayer && (activeChar == target || activeChar.getPlayerAccess().CanEditCharAll))
			player = (L2Player) target;
		else
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		if(adminSkills == null)
			activeChar.sendMessage("You must first get the skills of someone to do this.");
		else
		{
			L2Skill[] skills = player.getAllSkillsArray();
			for(L2Skill element : skills)
				player.removeSkill(element, true);
			for(L2Skill s : activeChar.getAllSkills())
				player.addSkill(s, true);
			for(L2Skill element : skills)
				activeChar.removeSkill(element, true);
			for(L2Skill element : adminSkills)
				activeChar.addSkill(element);
			player.sendMessage("[GM]" + activeChar.getName() + " has updated your skills.");
			activeChar.sendMessage("You now have all your skills back.");
			adminSkills = null;
		}
		showSkillsPage(activeChar);
	}

	private void adminAddSkill(L2Player activeChar, String val)
	{
		L2Object target = activeChar.getTarget();
		L2Player player;
		if(target != null && target.isPlayer && (activeChar == target || activeChar.getPlayerAccess().CanEditCharAll))
			player = (L2Player) target;
		else
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		StringTokenizer st = new StringTokenizer(val);
		if(st.countTokens() != 2)
			showSkillsPage(activeChar);
		else
		{
			String id = st.nextToken();
			String level = st.nextToken();
			int idval = Integer.parseInt(id);
			int levelval = Integer.parseInt(level);

			L2Skill skill = SkillTable.getInstance().getInfo(idval, levelval);

			if(skill != null)
			{
				player.sendMessage("Admin gave you the skill " + skill.getName() + ".");
				player.addSkill(skill, true);
				player.sendPacket(new SkillList(player));

				// Admin information
				activeChar.sendMessage("You gave the skill " + skill.getName() + " to " + player.getName() + ".");

				Log.add("gave the skill " + skill.getName() + " to " + player.getName(), "gm_ext_actions", activeChar);
			}
			else
				activeChar.sendMessage("Error: there is no such skill.");

			showSkillsPage(activeChar); // Back to start
		}
	}

	private void adminRemoveSkill(L2Player activeChar, Integer idval)
	{
		L2Object target = activeChar.getTarget();
		L2Player player = null;
		if(target.isPlayer && (activeChar == target || activeChar.getPlayerAccess().CanEditCharAll))
			player = (L2Player) target;
		else
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		L2Skill skill = SkillTable.getInstance().getInfo(idval, player.getSkillLevel(idval));

		if(skill != null)
		{
			player.sendMessage("Admin removed the skill " + skill.getName() + ".");
			player.removeSkill(skill, true);

			// Admin information
			activeChar.sendMessage("You removed the skill " + skill.getName() + " from " + player.getName() + ".");

			Log.add("removed the skill " + skill.getName() + " from " + player.getName(), "gm_ext_actions", activeChar);
		}
		else
			activeChar.sendMessage("Error: there is no such skill.");

		removeSkillsPage(activeChar); // Back to start
	}

	public void showSkill(L2Player activeChar, String val)
	{
		int skillid = Integer.parseInt(val);
		L2Skill skill = SkillTable.getInstance().getInfo(skillid, 1);

		if(skill != null)
		{
			if(skill.getTargetType() == L2Skill.SkillTargetType.TARGET_SELF)
				activeChar.broadcastPacket(new MagicSkillUse(activeChar, activeChar, skillid, 1, skill.getHitTime(), skill.getReuseDelay()));
		}
		else
			activeChar.broadcastPacket(Msg.ActionFail);
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"file://com_show_skills",
				"file://com_remove_skills",
				"file://com_skill_list",
				"file://com_skill_index",
				"file://com_add_skill",
				"file://com_remove_skill",
				"file://com_get_skills",
				"file://com_reset_skills",
				"file://com_give_all_skills" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR };
	}
}
