package commands.admin;

import java.lang.reflect.Constructor;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import l2r.database.mysql;
import l2r.extensions.scripts.ScriptFile;
import l2r.extensions.scripts.Scripts;
import l2r.gameserver.ai.L2CharacterAI;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.idfactory.IdFactory;
import l2r.gameserver.instancemanager.BoatManager;
import l2r.gameserver.instancemanager.RaidBossSpawnManager;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Spawn;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.GmListTable;
import l2r.gameserver.tables.NpcTable;
import l2r.gameserver.tables.SpawnTable;
import l2r.gameserver.templates.L2CharTemplate;
import l2r.gameserver.templates.L2NpcTemplate;
import l2r.gameserver.templates.StatsSet;
import l2r.util.Log;

/**
 * This class handles following admin commands: - show_spawns = shows menu -
 * spawn_index lvl = shows menu for monsters with respective level -
 * spawn_monster id = spawns monster id on target spawn1 id count - заспавнить
 * count мобов только 1 раз
 */
public class AdminSpawn implements IAdminCommandHandler, ScriptFile
{

	private static String[] _adminCommands = {
			"admin_show_spawns",
			"admin_spawn",
			"admin_spawn_monster",
			"admin_spawn_index",
			"admin_unspawnall",
			"admin_spawn1",
			"admin_setheading",
			"admin_airship",
			"admin_setai" };

	@SuppressWarnings("unchecked")
	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanEditNPC)
			return false;

		if(command.equals("admin_show_spawns"))
			AdminHelpPage.showHelpPage(activeChar, "spawns.htm");
		else if(command.startsWith("admin_spawn_index"))
			try
			{
				String val = command.substring(18);
				AdminHelpPage.showHelpPage(activeChar, "spawns/" + val + ".htm");
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.startsWith("admin_spawn1"))
		{
			StringTokenizer st = new StringTokenizer(command, " ");
			try
			{
				st.nextToken();
				String id = st.nextToken();
				int mobCount = 1;
				if(st.hasMoreTokens())
					mobCount = Integer.parseInt(st.nextToken());
				spawnMonster(activeChar, id, 0, mobCount);
			}
			catch(Exception e)
			{
				// Case of wrong monster data
			}
		}
		else if(command.startsWith("admin_spawn") || command.startsWith("admin_spawn_monster"))
		{
			StringTokenizer st = new StringTokenizer(command, " ");
			try
			{
				st.nextToken();
				String id = st.nextToken();
				int respawnTime = 30;
				int mobCount = 1;
				if(st.hasMoreTokens())
					mobCount = Integer.parseInt(st.nextToken());
				if(st.hasMoreTokens())
					respawnTime = Integer.parseInt(st.nextToken());
				spawnMonster(activeChar, id, respawnTime, mobCount);
			}
			catch(Exception e)
			{
				// Case of wrong monster data
			}
		}
		else if(command.startsWith("admin_unspawnall"))
		{
			for(L2Player player : L2World.getAllPlayers())
				player.sendPacket(new SystemMessage(SystemMessage.THE_NPC_SERVER_IS_NOT_OPERATING));
			L2World.deleteVisibleNpcSpawns();
			GmListTable.broadcastMessageToGMs("NPC Unspawn completed!");
		}
		else if(command.startsWith("admin_setai"))
		{
			if(activeChar.getTarget() == null || !activeChar.getTarget().isNpc)
			{
				activeChar.sendMessage("Please select target NPC or mob.");
				return false;
			}

			StringTokenizer st = new StringTokenizer(command, " ");
			st.nextToken();
			if(!st.hasMoreTokens())
			{
				activeChar.sendMessage("Please specify AI name.");
				return false;
			}
			String aiName = st.nextToken();
			L2NpcInstance target = (L2NpcInstance) activeChar.getTarget();

			Constructor aiConstructor = null;
			try
			{
				if(!aiName.equalsIgnoreCase("npc"))
					aiConstructor = Class.forName("l2r.gameserver.ai." + aiName).getConstructors()[0];
			}
			catch(Exception e)
			{
				try
				{
					aiConstructor = Scripts.getInstance().getClasses().get("ai." + aiName).getRawClass().getConstructors()[0];
				}
				catch(Exception e1)
				{
					activeChar.sendMessage("This type AI not found.");
					return false;
				}
			}

			target.detachAI();

			if(aiConstructor != null)
			{
				try
				{
					target.setAI((L2CharacterAI) aiConstructor.newInstance(new Object[] { target }));
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
				target.getAI().startAITask();
			}
		}
		else if(command.startsWith("admin_setheading"))
		{
			L2Object obj = activeChar.getTarget();
			if(!(obj instanceof L2NpcInstance))
			{
				activeChar.sendMessage("Target is incorrect!");
				return false;
			}

			L2NpcInstance npc = (L2NpcInstance) obj;

			L2Spawn spawn = npc.getSpawn();
			if(spawn == null)
			{
				activeChar.sendMessage("Spawn for this npc == null!");
				return false;
			}

			if(!mysql.set("update spawnlist set heading = " + activeChar.getHeading() //
					+ " where npc_templateid = " + npc.getNpcId() //
					+ " and locx = " + spawn.getLocx() //
					+ " and locy = " + spawn.getLocy() //
					+ " and locz = " + spawn.getLocz() //
					+ " and loc_id = " + spawn.getLocation()))
			{
				activeChar.sendMessage("Error in mysql query!");
				return false;
			}

			npc.setHeading(activeChar.getHeading());
			npc.decayMe();
			npc.spawnMe();
			activeChar.sendMessage("New heading : " + activeChar.getHeading());
		}
		else if(command.startsWith("admin_airship"))
		{
			StatsSet npcDat = new StatsSet();
			npcDat.set("npcId", 1);
			npcDat.set("level", 0);
			npcDat.set("jClass", "airship");

			npcDat.set("baseSTR", 0);
			npcDat.set("baseCON", 0);
			npcDat.set("baseDEX", 0);
			npcDat.set("baseINT", 0);
			npcDat.set("baseWIT", 0);
			npcDat.set("baseMEN", 0);

			npcDat.set("baseShldDef", 0);
			npcDat.set("baseShldRate", 0);
			npcDat.set("baseAccCombat", 38);
			npcDat.set("baseEvasRate", 38);
			npcDat.set("baseCritRate", 38);

			npcDat.set("collision_radius", 0);
			npcDat.set("collision_height", 0);
			npcDat.set("sex", "male");
			npcDat.set("type", "");
			npcDat.set("baseAtkRange", 0);
			npcDat.set("baseMpMax", 0);
			npcDat.set("baseCpMax", 0);
			npcDat.set("revardExp", 0);
			npcDat.set("revardSp", 0);
			npcDat.set("basePAtk", 0);
			npcDat.set("baseMAtk", 0);
			npcDat.set("basePAtkSpd", 0);
			npcDat.set("aggroRange", 0);
			npcDat.set("baseMAtkSpd", 0);
			npcDat.set("rhand", 0);
			npcDat.set("lhand", 0);
			npcDat.set("armor", 0);
			npcDat.set("baseWalkSpd", 0);
			npcDat.set("baseRunSpd", 0);
			npcDat.set("name", "airship");
			npcDat.set("baseHpMax", 50000);
			npcDat.set("baseHpReg", 3.e-3f);
			npcDat.set("baseCpReg", 0);
			npcDat.set("baseMpReg", 3.e-3f);
			npcDat.set("basePDef", 100);
			npcDat.set("baseMDef", 100);
			L2CharTemplate template = new L2CharTemplate(npcDat);
		}

		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	private void spawnMonster(L2Player activeChar, String monsterId, int respawnTime, int mobCount)
	{
		L2Object target = activeChar.getTarget();
		if(target == null)
			target = activeChar;

		Pattern pattern = Pattern.compile("[0-9]*");
		Matcher regexp = pattern.matcher(monsterId);
		L2NpcTemplate template;
		if(regexp.matches())
		{
			// First parameter was an ID number
			int monsterTemplate = Integer.parseInt(monsterId);
			template = NpcTable.getTemplate(monsterTemplate);
		}
		else
		{
			// First parameter wasn't just numbers so go by name not ID
			monsterId = monsterId.replace('_', ' ');
			template = NpcTable.getTemplateByName(monsterId);
		}

		if(template == null)
		{
			activeChar.sendMessage("Incorrect monster template.");
			return;
		}

		try
		{
			L2Spawn spawn = new L2Spawn(template);
			spawn.setLoc(target.getLoc());
			spawn.setLocation(0);
			spawn.setAmount(mobCount);
			spawn.setHeading(activeChar.getHeading());
			spawn.setRespawnDelay(respawnTime);

			if(RaidBossSpawnManager.getInstance().isDefined(spawn.getNpcId()))
				activeChar.sendMessage("Raid Boss " + template.name + " already spawned.");
			else
			{
				if(RaidBossSpawnManager.getInstance().getValidTemplate(spawn.getNpcId()) != null)
				{
					if(respawnTime != 0)
						RaidBossSpawnManager.getInstance().addNewSpawn(spawn, true);
				}
				else
					SpawnTable.getInstance().addNewSpawn(spawn, respawnTime != 0, activeChar);

				spawn.init();
				if(respawnTime == 0)
					spawn.stopRespawn();

				activeChar.sendMessage("Created " + template.name + " on " + target.getObjectId() + ".");

				Log.add("Created " + template.name + " on " + target.getObjectId(), "gm_ext_actions", activeChar);
			}
		}
		catch(Exception e)
		{
			activeChar.sendMessage("Target is not ingame.");
		}
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"admin_show_spawns",
				"admin_spawn",
				"admin_spawn_monster",
				"admin_spawn_index",
				"admin_unspawnall",
				"admin_spawn1",
				"admin_setheading",
				"admin_airship",
				"Установка указанного АИ выделенному мобу/NPC" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6 };
	}
}