package commands.voiced;

import static l2r.gameserver.model.L2Zone.ZoneType.OlympiadStadia;
import static l2r.gameserver.model.L2Zone.ZoneType.Siege;
import static l2r.gameserver.model.L2Zone.ZoneType.no_restart;
import l2r.extensions.multilang.CustomMessage;
import l2r.extensions.scripts.Functions;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.handler.IVoicedCommandHandler;
import l2r.gameserver.handler.VoicedCommandHandler;
import l2r.gameserver.instancemanager.DimensionalRiftManager;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.util.Location;

public class Relocate extends Functions implements IVoicedCommandHandler, ScriptFile
{
	public static L2Object self;
	public static L2NpcInstance npc;
	public static int SUMMON_PRICE = 200000;

	private final String[] _commandList = new String[] { "km-all-to-me" };

	public String[] getVoicedCommandList()
	{
		return _commandList;
	}

	public boolean useVoicedCommand(String command, L2Player activeChar, String target)
	{
		if(command.equalsIgnoreCase("km-all-to-me"))
		{
			if(!activeChar.isClanLeader())
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.ONLY_THE_CLAN_LEADER_IS_ENABLED));
				return false;
			}

			if(activeChar.isAlikeDead())
			{
				activeChar.sendMessage(new CustomMessage("scripts.commands.voiced.Relocate.Dead", activeChar));
				return false;
			}

			if(activeChar.isInCombat())
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_SUMMON_DURING_COMBAT));
				return false;
			}

			if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE || activeChar.isTransactionInProgress())
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_SUMMON_DURING_A_TRADE_OR_WHILE_USING_THE_PRIVATE_SHOPS));
				return false;
			}

			// "Нельзя вызывать персонажей в/из зоны свободного PvP"
			// "в зоны осад"
			// "на Олимпийский стадион"
			// "в зоны определенных рейд-боссов и эпик-боссов"
			if(activeChar.isInZoneBattle() || activeChar.isInZone(Siege) || activeChar.isInZoneIncludeZ(no_restart))
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_MAY_NOT_SUMMON_FROM_YOUR_CURRENT_LOCATION));
				return false;
			}

			// Нельзя саммонить в зону рифта
			if(DimensionalRiftManager.getInstance().checkIfInRiftZone(activeChar.getLoc(), false))
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_MAY_NOT_SUMMON_FROM_YOUR_CURRENT_LOCATION));
				return false;
			}

			L2Player[] clan = activeChar.getClan().getOnlineMembers(activeChar.getName());
			int price = 0;

			if(activeChar.getAdena() < SUMMON_PRICE * clan.length)
			{
				activeChar.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
				return false;
			}

			for(L2Player pl : clan)
			{
				if(pl == null)
					continue;

				if(pl.isInZoneBattle() || pl.isInZone(Siege) || pl.isInZone(OlympiadStadia) || pl.isInZoneIncludeZ(no_restart) || pl.isFlying())
					continue;

				// Нельзя призывать торгующих, мертвых персонажей
				if(pl.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE || pl.isTransactionInProgress() || pl.isAlikeDead())
					continue;

				// Нельзя призывать персонажей, которые находятся в режиме PvP
				if(pl.getPvpFlag() != 0)
					continue;

				// Снимаем адену за каждого призываемого члена клана
				price += SUMMON_PRICE;

				Location pos = GeoClient.getInstance().findPointToStay(activeChar.getX(), activeChar.getY(), activeChar.getZ(), 100, 150);

				// Спрашиваем, согласие на призыв
				pl.summonCharacterRequest(activeChar.getName(), pos);
			}

			if(price != 0)
			{
				activeChar.reduceAdena(price, true);
				activeChar.sendPacket(new SystemMessage(SystemMessage.S1_ADENA_DISAPPEARED).addNumber(price));
			}

			return true;
		}
		return false;
	}

	public void onLoad()
	{
		VoicedCommandHandler.getInstance().registerVoicedCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}