package events.Christmas;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.handler.IItemHandler;
import l2r.gameserver.handler.ItemHandler;
import l2r.gameserver.idfactory.IdFactory;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Spawn;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.NpcTable;
import l2r.gameserver.templates.L2NpcTemplate;

public class Seed implements IItemHandler, ScriptFile
{
	public class DeSpawnScheduleTimerTask implements Runnable
	{
		L2Spawn spawnedTree = null;

		public DeSpawnScheduleTimerTask(L2Spawn spawn)
		{
			spawnedTree = spawn;
		}

		public void run()
		{
			try
			{
				spawnedTree.getLastSpawn().decayMe();
				spawnedTree.getLastSpawn().deleteMe();
			}
			catch(Throwable t)
			{}
		}
	}

	private static short[] _itemIds = { 5560, // Christmas Tree
			5561 // Special Christmas Tree
	};

	private static int[] _npcIds = { 13006, // Christmas Tree
			13007 // Special Christmas Tree
	};

	private static final int DESPAWN_TIME = 600000; //10 min

	public void useItem(L2Playable playable, L2ItemInstance item)
	{
		L2Player activeChar = (L2Player) playable;
		L2NpcTemplate template = null;

		int itemId = item.getItemId();
		for(int i = 0; i < _itemIds.length; i++)
			if(_itemIds[i] == itemId)
			{
				template = NpcTable.getTemplate(_npcIds[i]);
				break;
			}

		if(template == null)
			return;

		try
		{
			L2Spawn spawn = new L2Spawn(template);
			spawn.setId(IdFactory.getInstance().getNextId());
			spawn.setLoc(activeChar.getLoc());
			L2NpcInstance npc = spawn.doSpawn(false);
			npc.setTitle(activeChar.getName()); //FIXME Почему-то не устанавливается
			spawn.respawnNpc(npc);

			// АИ вещающее бафф регена устанавливается только для большой елки
			if(itemId == 5561)
			{
				npc.setAI(new ctreeAI(npc));
				npc.getAI().startAITask();
			}

			ThreadPoolManager.getInstance().scheduleGeneral(new DeSpawnScheduleTimerTask(spawn), DESPAWN_TIME);
			activeChar.getInventory().destroyItem(item.getObjectId(), 1, false);
		}
		catch(Exception e)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.TARGET_CAN_NOT_BE_FOUND));
		}
	}

	public short[] getItemIds()
	{
		return _itemIds;
	}

	public void onLoad()
	{
		ItemHandler.getInstance().registerItemHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}