package events.arena;

import java.util.ArrayList;

import l2r.Config;
import l2r.extensions.listeners.L2ZoneEnterLeaveListener;
import l2r.extensions.scripts.Functions;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Zone;
import l2r.gameserver.model.L2Zone.ZoneType;
import l2r.gameserver.model.base.Experience;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.tables.SkillTable;
import l2r.util.Location;
import l2r.util.Util;

public abstract class ArenaTemplate extends Functions implements ScriptFile
{
	// Эти переменные выставляются автоматически при вызове скрипта
	public static L2Object self; // Тот, от чьего имени выполняется код
	public static L2NpcInstance npc; // npc, с которым мы разговариваем

	protected int _managerId;
	protected String _className;

	protected L2Player _creator;
	protected L2NpcInstance _manager;
	protected int _status = 0;
	protected int _battleType = 1;
	protected int _team1exp = 0;
	protected int _team2exp = 0;
	protected int _price = 10000;
	protected int _team1count = 1;
	protected int _team2count = 1;
	protected int _team1min = 1;
	protected int _team1max = 80;
	protected int _team2min = 1;
	protected int _team2max = 80;
	protected int _timeToStart = 10;
	protected boolean _timeOutTask;

	protected ArrayList<Location> _team1points;
	protected ArrayList<Location> _team2points;

	protected ArrayList<L2Player> _team1list = new ArrayList<L2Player>();
	protected ArrayList<L2Player> _team2list = new ArrayList<L2Player>();
	protected ArrayList<L2Player> _team1live = new ArrayList<L2Player>();
	protected ArrayList<L2Player> _team2live = new ArrayList<L2Player>();

	protected L2Zone _zone;
	protected ZoneListener _zoneListener;

	public void template_stop()
	{
		sayInRange(_manager, "Бой прерван по техническим причинам, деньги возвращены", 4000);
		if(_battleType == 1)
			returnAdenaToTeams();
		else if(_battleType == 2)
			returnExpToTeams();
		unParalyzeTeams();
		clearTeams();
		_status = 0;
		_timeOutTask = false;
	}

	public void template_create1()
	{
		if(_status > 0)
			show("Дождитесь окончания боя", (L2Player) self);
		else
			show("data/scripts/events/arena/" + _managerId + "-1.html", (L2Player) self);
	}

	public void template_create2()
	{
		if(_status > 0)
			show("Дождитесь окончания боя", (L2Player) self);
		else
			show("data/scripts/events/arena/" + _managerId + "-2.html", (L2Player) self);
	}

	public void template_register()
	{
		if(_status > 1)
			show("Дождитесь окончания боя", (L2Player) self);
		else
			show("data/scripts/events/arena/" + _managerId + "-3.html", (L2Player) self);
	}

	public void template_check1(String[] var)
	{
		L2Player player = (L2Player) self;

		if(var.length != 8)
		{
			show("Некорректные данные", player);
			return;
		}

		if(_status > 0)
		{
			show("Дождитесь окончания боя", player);
			return;
		}
		if(npc == null || !npc.isNpc)
		{
			show("Hacker? :) " + npc, player);
			return;
		}
		_manager = npc;
		try
		{
			_price = Integer.valueOf(var[0]);
			_team1count = Integer.valueOf(var[1]);
			_team2count = Integer.valueOf(var[2]);
			_team1min = Integer.valueOf(var[3]);
			_team1max = Integer.valueOf(var[4]);
			_team2min = Integer.valueOf(var[5]);
			_team2max = Integer.valueOf(var[6]);
			_timeToStart = Integer.valueOf(var[7]);
		}
		catch(Exception e)
		{
			show("Некорректные данные", player);
			return;
		}
		if(_price < 10000 || _price > 100000000)
		{
			show("Неправильная ставка", player);
			return;
		}
		if(_team1count < 1 || _team1count > 5 || _team2count < 1 || _team2count > 5)
		{
			show("Неправильный размер команды", player);
			return;
		}
		if(_team1min < 1 || _team1min > 80 || _team2min < 1 || _team2min > 80 || _team1max < 1 || _team1max > 80 || _team2max < 1 || _team2max > 80 || _team1min > _team1max || _team2min > _team2max)
		{
			show("Неправильный уровень", player);
			return;
		}
		if(player.getLevel() < _team1min || player.getLevel() > _team1max)
		{
			show("Неправильный уровень", player);
			return;
		}
		if(_timeToStart < 1 || _timeToStart > 10)
		{
			show("Неправильное время", player);
			return;
		}
		if(player.getAdena() < _price)
		{
			player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			return;
		}
		_battleType = 1;
		_creator = (L2Player) self;
		_creator.reduceAdena(_price, true);
		_creator.setTeam(1);
		_status = 1;
		_team1list.clear();
		_team2list.clear();
		_team1live.clear();
		_team2live.clear();
		_team1list.add(_creator);
		sayInRange(_manager, _creator.getName() + " создал бой " + _team1count + "х" + _team2count + ", " + _team1min + "-" + _team1max + "lv vs " + _team2min + "-" + _team2max + "lv, ставка " + _price + "а, начало через " + _timeToStart + " мин", 4000);
		executeTask("events.arena." + _className, "announce", new Object[0], 60000);
	}

	public void template_check2(String[] var)
	{
		L2Player player = (L2Player) self;

		if(var.length != 7)
		{
			show("Некорректные данные", player);
			return;
		}

		if(_status > 0)
		{
			show("Дождитесь окончания боя", player);
			return;
		}
		if(npc == null || !npc.isNpc)
		{
			show("Hacker? :) " + npc, player);
			return;
		}
		_manager = npc;
		try
		{
			_team1count = Integer.valueOf(var[0]);
			_team2count = Integer.valueOf(var[1]);
			_team1min = Integer.valueOf(var[2]);
			_team1max = Integer.valueOf(var[3]);
			_team2min = Integer.valueOf(var[4]);
			_team2max = Integer.valueOf(var[5]);
			_timeToStart = Integer.valueOf(var[6]);
		}
		catch(Exception e)
		{
			show("Некорректные данные", player);
			return;
		}
		if(_team1count < 1 || _team1count > 5 || _team2count < 1 || _team2count > 5)
		{
			show("Неправильный размер команды", player);
			return;
		}
		if(_team1min < 1 || _team1min > 80 || _team2min < 1 || _team2min > 80 || _team1max < 1 || _team1max > 80 || _team2max < 1 || _team2max > 80 || _team1min > _team1max || _team2min > _team2max)
		{
			show("Неправильный уровень", player);
			return;
		}
		if(player.getLevel() - _team1min > 10 || _team1max - player.getLevel() > 10 || player.getLevel() - _team2min > 10 || _team2max - player.getLevel() > 10)
		{
			show("Разница в уровнях не может быть более 10", player);
			return;
		}
		if(player.getLevel() < _team1min || player.getLevel() > _team1max)
		{
			show("Неправильный уровень", player);
			return;
		}
		if(_timeToStart < 1 || _timeToStart > 10)
		{
			show("Неправильное время", player);
			return;
		}

		_battleType = 2;
		_creator = (L2Player) self;
		_team1exp = 0;
		_team2exp = 0;

		removeExp(_creator, 1);

		_creator.setTeam(1);
		_status = 1;
		_team1list.clear();
		_team2list.clear();
		_team1live.clear();
		_team2live.clear();
		_team1list.add((L2Player) self);
		sayInRange(_manager, _creator.getName() + " создал бой " + _team1count + "х" + _team2count + ", " + _team1min + "-" + _team1max + "lv vs " + _team2min + "-" + _team2max + "lv, ставка " + "опыт, начало через " + _timeToStart + " мин", 4000);
		executeTask("events.arena." + _className, "announce", new Object[0], 60000);
	}

	public void template_register_check(String[] var)
	{
		L2Player player = (L2Player) self;

		if(_status > 1)
		{
			show("Дождитесь окончания боя", player);
			return;
		}

		if(var.length != 1)
		{
			show("Некорректные данные", player);
			return;
		}

		int _regTeam;
		try
		{
			_regTeam = Integer.valueOf(var[0]);
		}
		catch(Exception e)
		{
			show("Некорректные данные", player);
			return;
		}

		if(_regTeam != 1 && _regTeam != 2)
		{
			show("Неправильный номер команды, введите 1 или 2", player);
			return;
		}
		if(_team1list.contains(player) || _team2list.contains(player))
		{
			show("Вы уже зарегистрированы", player);
			return;
		}
		if(_battleType == 1 && player.getAdena() < _price)
		{
			player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			return;
		}
		if(_regTeam == 1)
		{
			if(player.getLevel() < _team1min || player.getLevel() > _team1max)
			{
				show("Вы не подходите по уровню", player);
				return;
			}
			if(_team1list.size() >= _team1count)
			{
				show("Команда 1 переполнена", player);
				return;
			}

			if(_battleType == 1)
				player.reduceAdena(_price, true);
			else if(_battleType == 2)
				removeExp(player, 1);

			_team1list.add(player);
			player.setTeam(1);
			sayInRange(_manager, player.getName() + " зарегистрировался за 1 команду", 4000);
			if(_team1list.size() >= _team1count && _team2list.size() >= _team2count)
			{
				sayInRange(_manager, "Команды готовы, старт через 1 минуту.", 4000);
				_timeToStart = 1;
			}
		}
		else
		{
			if(player.getLevel() < _team2min || player.getLevel() > _team2max)
			{
				show("Вы не подходите по уровню", player);
				return;
			}
			if(_team2list.size() >= _team2count)
			{
				show("Команда 2 переполнена", player);
				return;
			}

			if(_battleType == 1)
				player.reduceAdena(_price, true);
			else if(_battleType == 2)
				removeExp(player, 2);

			_team2list.add(player);
			player.setTeam(2);
			sayInRange(_manager, player.getName() + " зарегистрировался за 2 команду", 4000);
			if(_team1list.size() >= _team1count && _team2list.size() >= _team2count)
			{
				sayInRange(_manager, "Команды готовы, старт через 1 минуту.", 4000);
				_timeToStart = 1;
			}
		}
	}

	public void template_announce()
	{
		if(_status != 1)
			return;

		if(_timeToStart > 1)
		{
			_timeToStart--;
			sayInRange(_manager, _creator.getName() + " создал бой " + _team1count + "х" + _team2count + ", " + _team1min + "-" + _team1max + "lv vs " + _team2min + "-" + _team2max + "lv, ставка " + (_battleType == 1 ? _price + "а" : "опыт") + ", начало через " + _timeToStart + " мин", 4000);
			executeTask("events.arena." + _className, "announce", new Object[0], 60000);
		}
		else if(_team2list.size() > 0)
		{
			sayInRange(_manager, "Подготовка к бою", 4000);
			executeTask("events.arena." + _className, "prepare", new Object[0], 5000);
		}
		else
		{
			sayInRange(_manager, "Бой не состоялся, нет противников", 4000);
			_status = 0;
			if(_battleType == 1)
				returnAdenaToTeams();
			else if(_battleType == 2)
				returnExpToTeams();
			clearTeams();
		}
	}

	public void template_prepare()
	{
		if(_status != 1)
			return;

		_status = 2;
		for(L2Player player : _team1list)
			if(player != null && !player.isDead())
				_team1live.add(player);
		for(L2Player player : _team2list)
			if(player != null && !player.isDead())
				_team2live.add(player);
		if(!checkTeams())
			return;
		clearArena();
		paralyzeTeams();
		teleportTeamsToArena();
		sayInRange(_manager, "Бой начнется через 15 секунд", 4000);
		executeTask("events.arena." + _className, "start", new Object[0], 15000);
	}

	public void template_start()
	{
		if(_status != 2)
			return;

		if(!checkTeams())
			return;
		sayInRange(_manager, "Go!!!", 4000);
		unParalyzeTeams();
		_status = 3;
		executeTask("events.arena." + _className, "timeOut", new Object[0], 180000);
		_timeOutTask = true;
	}

	public void clearArena()
	{
		for(L2Player player : _manager.getAroundPlayers(2000))
			if(player.isInZone(ZoneType.battle_zone))
				teleportToArenaSpawn(player);
	}

	public boolean checkTeams()
	{
		if(_team1live.size() == 0)
		{
			teamHasLost(1);
			return false;
		}
		else if(_team2live.size() == 0)
		{
			teamHasLost(2);
			return false;
		}
		return true;
	}

	public void paralyzeTeams()
	{
		L2Skill revengeSkill = SkillTable.getInstance().getInfo(4515, 1);
		for(L2Player player : _team1live)
			if(player != null)
			{
				revengeSkill.getEffects(player, player, false);
				if(player.getPet() != null)
					revengeSkill.getEffects(player, player.getPet(), false);
			}
		for(L2Player player : _team2live)
			if(player != null)
			{
				revengeSkill.getEffects(player, player, false);
				if(player.getPet() != null)
					revengeSkill.getEffects(player, player.getPet(), false);
			}
	}

	public void unParalyzeTeams()
	{
		for(L2Player player : _team1list)
			if(player != null)
			{
				player.stopEffect(4515);
				if(player.getPet() != null)
					player.getPet().stopEffect(4515);
			}
		for(L2Player player : _team2list)
			if(player != null)
			{
				player.stopEffect(4515);
				if(player.getPet() != null)
					player.getPet().stopEffect(4515);
			}
	}

	public void teleportTeamsToArena()
	{
		Integer n = 0;
		for(L2Player player : _team1live)
		{
			if(player != null)
			{
				player.teleToLocation(_team1points.get(n));
				if(player.getPet() != null)
					player.getPet().teleToLocation(_team1points.get(n));
			}
			n++;
		}
		n = 0;
		for(L2Player player : _team2live)
		{
			if(player != null)
			{
				player.teleToLocation(_team2points.get(n));
				if(player.getPet() != null)
					player.getPet().teleToLocation(_team1points.get(n));
			}
			n++;
		}
	}

	public void teleportToArenaSpawn(L2Character cha)
	{
		cha.teleToLocation(_zone.getSpawn());
	}

	public boolean playerHasLost(L2Player player)
	{
		if(player.getTeam() == 1)
			_team1live.remove(player);
		else
			_team2live.remove(player);
		L2Skill revengeSkill = SkillTable.getInstance().getInfo(4515, 1);
		revengeSkill.getEffects(player, player, false);
		return !checkTeams();
	}

	public void teamHasLost(Integer team_id)
	{
		if(team_id == 1)
		{
			sayInRange(_manager, "Команда 2 победила", 4000);
			if(_battleType == 1)
				payAdenaToTeam(2);
			else if(_battleType == 2)
				payExpToTeam(2);
		}
		else
		{
			sayInRange(_manager, "Команда 1 победила", 4000);
			if(_battleType == 1)
				payAdenaToTeam(1);
			else if(_battleType == 2)
				payExpToTeam(1);
		}
		unParalyzeTeams();
		clearTeams();
		_status = 0;
		_timeOutTask = false;
	}

	public void template_timeOut()
	{
		if(_timeOutTask && _status == 3)
		{
			sayInRange(_manager, "Время истекло, ничья!", 4000);
			if(_battleType == 1)
				returnAdenaToTeams();
			else if(_battleType == 2)
				returnExpToTeams();
			unParalyzeTeams();
			clearTeams();
			_status = 0;
			_timeOutTask = false;
		}
	}

	public void payAdenaToTeam(Integer team_id)
	{
		if(team_id == 1)
			for(L2Player player : _team1list)
			{
				if(player != null)
					player.addAdena(_price + _team2list.size() * _price / _team1list.size());
			}
		else
			for(L2Player player : _team2list)
				if(player != null)
					player.addAdena(_price + _team1list.size() * _price / _team2list.size());
	}

	@SuppressWarnings("cast")
	public void payExpToTeam(Integer team_id)
	{
		if(team_id == 1)
		{
			for(L2Player player : _team1list)
				if(player != null)
				{
					returnExp(player);
					addExp(player, _team2exp / _team1list.size() / 2);
					if(Config.ALT_HONOR_SYSTEM)
						player.getInventory().addItem(4674, 3, 0, _className);
				}
			if(Config.ALT_HONOR_SYSTEM)
				for(L2Player player : _team2list)
					if(player != null)
						player.getInventory().addItem(4674, 1, 0, _className);
		}
		else
		{
			for(L2Player player : _team2list)
				if(player != null)
				{
					returnExp(player);
					addExp(player, _team1exp / _team2list.size() / 2);
					if(Config.ALT_HONOR_SYSTEM)
						player.getInventory().addItem(4674, 3, 0, _className);
				}
			if(Config.ALT_HONOR_SYSTEM)
				for(L2Player player : _team1list)
					if(player != null)
						player.getInventory().addItem(4674, 1, 0, _className);
		}
	}

	public void returnAdenaToTeams()
	{
		for(L2Player player : _team1list)
			if(player != null)
				player.addAdena(_price);
		for(L2Player player : _team2list)
			if(player != null)
				player.addAdena(_price);
	}

	public void returnExpToTeams()
	{
		for(L2Player player : _team1list)
			if(player != null)
				returnExp(player);
		for(L2Player player : _team2list)
			if(player != null)
				returnExp(player);
	}

	public void clearTeams()
	{
		for(L2Player player : _team1list)
			player.setTeam(0);
		for(L2Player player : _team2list)
			player.setTeam(0);
		_team1list.clear();
		_team2list.clear();
		_team1live.clear();
		_team2live.clear();
	}

	public void removeExp(L2Player player, int team)
	{
		int _lostExp = Math.round((Experience.LEVEL[player.getLevel() + 1] - Experience.LEVEL[player.getLevel()]) * 4 / 100);
		player.addExpAndSp(-1 * _lostExp, 0, false, false);

		if(team == 1)
			_team1exp += _lostExp;
		else if(team == 2)
			_team2exp += _lostExp;
	}

	public void returnExp(L2Player player)
	{
		double percentAdd = 4.0;
		int lvl = player.getLevel();
		int _addExp = (int) Math.round((Experience.LEVEL[lvl + 1] - Experience.LEVEL[lvl]) * percentAdd / 100);
		player.addExpAndSp(_addExp, 0, false, false);
	}

	public void addExp(L2Player player, int exp)
	{
		player.addExpAndSp(exp, 0, false, false);
	}

	public void onDie(L2Object self, L2Character killer)
	{
		if(_status >= 2 && self != null && self.isPlayer && (_team1list.contains(self) || _team2list.contains(self)))
		{
			L2Player player = self.getPlayer();
			L2Player kplayer = killer.getPlayer();
			// TODO реализовать тюрьму
			if(kplayer != null)
			{
				sayInRange(_manager, kplayer.getName() + " убил " + player.getName(), 4000);
				if(player.getTeam() == kplayer.getTeam() || !_team1list.contains(kplayer) && !_team2list.contains(kplayer))
				{
					sayInRange(_manager, "Нарушение правил, игрок " + kplayer.getName() + " оштрафован на " + _price, 4000);
					kplayer.reduceAdena(_price, true);
				}
				playerHasLost(player);
				if(Config.ALT_HONOR_SYSTEM)
					kplayer.getInventory().addItem(4674, 1, self.getObjectId(), _className);
			}
			else
			{
				sayInRange(_manager, player.getName() + " убит", 4000);
				playerHasLost(player);
			}
		}
	}

	public void OnPlayerExit(L2Player player)
	{
		if(player != null && _status > 0 && (_team1list.contains(player) || _team2list.contains(player)))
			switch(_status)
			{
				case 1:
					player.setTeam(0);
					_team1list.remove(player);
					_team2list.remove(player);
					sayInRange(_manager, player.getName() + " дисквалифицирован", 4000);
					if(player.equals(_creator))
					{
						sayInRange(_manager, "Бой прерван, деньги возвращены", 4000);
						if(_battleType == 1)
							returnAdenaToTeams();
						else if(_battleType == 2)
							returnExpToTeams();
						unParalyzeTeams();
						clearTeams();
						_status = 0;
						_timeOutTask = false;
					}
					break;
				case 2:
					player.setTeam(0);
					_team1list.remove(player);
					_team2list.remove(player);
					_team1live.remove(player);
					_team2live.remove(player);
					sayInRange(_manager, player.getName() + " дисквалифицирован", 4000);
					checkTeams();
					break;
				case 3:
					player.setTeam(0);
					_team1list.remove(player);
					_team2list.remove(player);
					_team1live.remove(player);
					_team2live.remove(player);
					sayInRange(_manager, player.getName() + " дисквалифицирован", 4000);
					checkTeams();
					break;
			}
	}

	public Location onEscape(L2Player player)
	{
		if(player != null && _status > 1 && player.isInZone(_zone))
			OnPlayerExit(player);
		return null;
	}

	public class ZoneListener extends L2ZoneEnterLeaveListener
	{
		@Override
		public void objectEntered(L2Zone zone, L2Object object)
		{
			L2Player player = object.getPlayer();
			if(_status >= 2 && player != null && !(_team1list.contains(player) || _team2list.contains(player)))
				teleportToArenaSpawn(player);
		}

		@Override
		public void objectLeaved(L2Zone zone, L2Object object)
		{
			L2Player player = object.getPlayer();
			if(_status >= 2 && player != null && (_team1list.contains(player) || _team2list.contains(player)))
			{
				L2Playable playable = (L2Playable) object;
				double angle = Util.convertHeadingToDegree(playable.getHeading()); // угол в градусах
				double radian = Math.toRadians(angle - 90); // угол в радианах
				playable.teleToLocation((int) (playable.getX() + 50 * Math.sin(radian)), (int) (playable.getY() - 50 * Math.cos(radian)), playable.getZ());
			}
		}
	}
}