package events.transport;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.LineNumberReader;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Stack;
import java.util.StringTokenizer;
import java.util.concurrent.ConcurrentHashMap;

import l2r.Config;
import l2r.extensions.listeners.AbstractAINotifyEventListener;
import l2r.extensions.scripts.Functions;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.Announcements;
import l2r.gameserver.ai.AbstractAI;
import l2r.gameserver.ai.CtrlEvent;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.instancemanager.ServerVariables;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.serverpackets.ValidateLocation;
import l2r.util.Files;
import l2r.util.Location;

public class Transport extends Functions implements ScriptFile
{
	public static L2Object self;
	public static L2Object npc;

	private static HashMap<String, Wyvern> wyverns;
	private static ConcurrentHashMap<Integer, Rider> _riders = new ConcurrentHashMap<Integer, Rider>();

	private static NotifyEventListener _notifyEventListener = new NotifyEventListener();

	private static boolean _active = false;

	/**
	 * Читает статус эвента из базы.
	 * @return
	 */
	private static boolean isActive()
	{
		return ServerVariables.getString("transport", "off").equalsIgnoreCase("on");
	}

	public void onLoad()
	{
		if(isActive())
		{
			_active = true;
			loadWyvernPath();
			System.out.println("Loaded Event: Transport [state: activated]");
		}
		else
		{
			wyverns = null;
			System.out.println("Loaded Event: Transport [state: deactivated]");
		}
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	/**
	* Запускает эвент
	*/
	public void startEvent()
	{
		L2Player player = (L2Player) self;
		if(!player.getPlayerAccess().IsEventGm)
			return;

		if(!isActive())
		{
			ServerVariables.set("transport", "on");
			loadWyvernPath();
			System.out.println("Event 'Transport' started.");
			Announcements.getInstance().announceByCustomMessage("scripts.events.transport.AnnounceEventStarted", null);
		}
		else
			player.sendMessage("Event 'Transport' already started.");

		_active = true;

		show(Files.read("data/html/admin/events.htm", player), player);
	}

	/**
	* Останавливает эвент
	*/
	public void stopEvent()
	{
		L2Player player = (L2Player) self;
		if(!player.getPlayerAccess().IsEventGm)
			return;
		if(isActive())
		{
			ServerVariables.unset("transport");
			wyverns = null;
			System.out.println("Event 'Transport' stopped.");
			Announcements.getInstance().announceByCustomMessage("scripts.events.transport.AnnounceEventStoped", null);
		}
		else
			player.sendMessage("Event 'Transport' not started.");

		_active = false;

		show(Files.read("data/html/admin/events.htm", player), player);
	}

	private void loadWyvernPath()
	{
		LineNumberReader lnr = null;
		wyverns = new HashMap<String, Wyvern>();
		try
		{
			File wyvernData = new File(Config.DATAPACK_ROOT, "data/wyvernpath.csv");
			lnr = new LineNumberReader(new BufferedReader(new FileReader(wyvernData)));
			String line = null;
			while((line = lnr.readLine()) != null)
			{
				if(line.trim().length() == 0 || line.startsWith("#"))
					continue;
				Wyvern W = new Wyvern();
				W.parseLine(line);
				wyverns.put(W.name, W);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			try
			{
				if(lnr != null)
					lnr.close();
			}
			catch(Exception e1)
			{ /* ignore problems */}
		}
	}

	public class Wyvern
	{
		public ArrayList<Integer[]> path;
		public String name;

		public void parseLine(String line)
		{
			path = new ArrayList<Integer[]>();
			StringTokenizer st = new StringTokenizer(line, " ");
			name = st.nextToken();
			while(st.hasMoreTokens())
			{
				Integer[] point = null;
				String token = st.nextToken();
				if(token.startsWith("t"))
				{
					StringTokenizer points = new StringTokenizer(token, ";");
					points.nextToken();
					point = new Integer[] {
							Integer.parseInt(points.nextToken()),
							Integer.parseInt(points.nextToken()),
							Integer.parseInt(points.nextToken()),
							-1 };
				}
				else
				{
					StringTokenizer points = new StringTokenizer(token, ";");
					point = new Integer[] { Integer.parseInt(points.nextToken()), Integer.parseInt(points.nextToken()), Integer.parseInt(points.nextToken()) };
				}
				path.add(point);
			}
		}
	}

	public class Rider
	{
		public Wyvern W;
		public L2Player P;
		public Stack<Integer[]> way;
	}

	public void HireWyvern(String[] param)
	{
		if(param.length < 2)
			throw new IllegalArgumentException();

		if(!_active)
			return;

		if(wyverns == null)
			return;

		L2Player player = (L2Player) self;

		int price = Integer.parseInt(param[1]);

		int day = Calendar.getInstance().get(Calendar.DAY_OF_WEEK);
		int hour = Calendar.getInstance().get(Calendar.HOUR_OF_DAY);
		if(day != 1 && day != 7 && (hour <= 12 || hour >= 22))
			price /= 2;

		if(player.isActionsDisabled() || player.isSitting() || player.isMounted() || player.getLastNpc().getDistance(player) > 300)
			return;

		if(player.getPet() != null)
		{
			player.sendMessage("You can't ride wyvern while controlling pet or summon.");
			return;
		}

		if(player.getAdena() < price)
		{
			player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			return;
		}

		if(price > 0)
			player.reduceAdena(price, true);

		player._stablePoint = new Location(player.getX(), player.getY(), player.getZ(), price);
		player.setVar("wyvern_moneyback", String.valueOf(price));

		Wyvern W = wyverns.get(param[0]);

		player.setMount(12621, 0);
		player.block();
		player.setIsInvul(true);

		Rider r = new Rider();
		r.P = player;
		r.W = W;
		r.way = new Stack<Integer[]>();
		r.way.addAll(W.path);
		_riders.put(player.getObjectId(), r);

		player.getAI().getListenerEngine().addMethodInvokedListener(_notifyEventListener);
		flyNext(r);
		player.broadcastUserInfo();
	}

	public static class NotifyEventListener extends AbstractAINotifyEventListener
	{
		@Override
		public void NotifyEvent(AbstractAI ai, CtrlEvent evt, Object arg0, Object arg1)
		{
			if(evt == CtrlEvent.EVT_ARRIVED)
			{
				if(ai == null || ai.getActor() == null)
					return;
				Rider r = _riders.get(ai.getActor().getObjectId());
				if(r == null)
					return;
				flyNext(r);
			}
		}
	}

	private static void flyNext(Rider r)
	{
		if(!r.way.empty())
		{
			// летим в следующую точку
			Integer[] next = r.way.firstElement();
			r.way.remove(0);
			if(next.length == 3)
				r.P.moveToLocation(next[0], next[1], next[2], 0, false);
			else
			{
				r.P.setXYZ(next[0], next[1], next[2]);
				r.P.sendPacket(new ValidateLocation(r.P));
				r.P.moveToLocation(next[0], next[1], next[2], 0, false);
			}
		}
		else
		{
			// прилетели
			r.P.setMount(0, 0);
			r.P._stablePoint = null;
			r.P.unsetVar("wyvern_moneyback");
			r.P.setIsInvul(false);
			r.P.unblock();
			r.P.getAI().getListenerEngine().removeMethodInvokedListener(_notifyEventListener);
			_riders.remove(r);
			r.P.broadcastUserInfo();
		}
	}

	public static String DialogAppend_31212(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31212.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31213(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31213.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31214(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31214.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31215(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31215.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31216(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31216.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31217(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31217.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31218(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31218.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31219(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31219.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31220(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31220.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31221(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31221.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31222(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31222.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31223(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31223.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31224(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31224.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31767(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31767.htm", player);
		}
		return "";
	}

	public static String DialogAppend_31768(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/31768.htm", player);
		}
		return "";
	}

	public static String DialogAppend_32048(Integer val)
	{
		if(_active && val == 0)
		{
			L2Player player = (L2Player) self;
			return Files.read("data/scripts/events/transport/32048.htm", player);
		}
		return "";
	}
}
