package items;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.handler.IItemHandler;
import l2r.gameserver.handler.ItemHandler;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.ExAutoSoulShot;
import l2r.gameserver.serverpackets.MagicSkillUse;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.templates.L2Item;
import l2r.gameserver.templates.L2Weapon;

public class BlessedSpiritShot implements IItemHandler, ScriptFile
{
	// all the items ids that this handler knowns
	private static final short[] _itemIds = { 3947, 3948, 3949, 3950, 3951, 3952 };
	private static final short[] _skillIds = { 2061, 2160, 2161, 2162, 2163, 2164, 2164, 2164 };

	static final SystemMessage POWER_OF_MANA_ENABLED = new SystemMessage(SystemMessage.POWER_OF_MANA_ENABLED);
	static final SystemMessage NOT_ENOUGH_SPIRITSHOTS = new SystemMessage(SystemMessage.NOT_ENOUGH_SPIRITSHOTS);
	static final SystemMessage SPIRITSHOT_DOES_NOT_MATCH_WEAPON_GRADE = new SystemMessage(SystemMessage.SPIRITSHOT_DOES_NOT_MATCH_WEAPON_GRADE);
	static final SystemMessage CANNOT_USE_SPIRITSHOTS = new SystemMessage(SystemMessage.CANNOT_USE_SPIRITSHOTS);

	public void useItem(L2Playable playable, L2ItemInstance item)
	{
		if(playable == null || !playable.isPlayer)
			return;
		L2Player player = (L2Player) playable;

		if(player.isInOlympiadMode())
		{
			player.sendPacket(Msg.THIS_ITEM_IS_NOT_AVAILABLE_FOR_THE_OLYMPIAD_EVENT);
			return;
		}

		L2ItemInstance weaponInst = player.getActiveWeaponInstance();
		L2Weapon weaponItem = player.getActiveWeaponItem();
		int SoulshotId = item.getItemId();
		boolean isAutoSoulShot = false;
		L2Item itemTemplate = ItemTable.getInstance().getTemplate(item.getItemId());

		if(player.getAutoSoulShot().contains(SoulshotId))
			isAutoSoulShot = true;

		if(weaponInst == null)
		{
			if(isAutoSoulShot)
			{
				player.removeAutoSoulShot(SoulshotId);
				player.sendPacket(new ExAutoSoulShot(SoulshotId, false));
				player.sendPacket(new SystemMessage(SystemMessage.THE_AUTOMATIC_USE_OF_S1_WILL_NOW_BE_CANCELLED).addString(itemTemplate.getName()));
				return;
			}
			player.sendPacket(CANNOT_USE_SPIRITSHOTS);
			return;
		}

		if(weaponInst.getChargedSpiritshot() == L2ItemInstance.CHARGED_BLESSED_SPIRITSHOT)
			// already charged by blessed spirit shot
			// btw we cant charge only when bsps is charged
			return;

		int spiritshotId = item.getItemId();
		int grade = weaponItem.getCrystalType();
		int blessedsoulSpiritConsumption = weaponItem.getSpiritShotCount();
		int count = item.getIntegerLimitedCount();

		if(blessedsoulSpiritConsumption == 0)
		{
			// Can't use Spiritshots
			if(isAutoSoulShot)
			{
				player.removeAutoSoulShot(SoulshotId);
				player.sendPacket(new ExAutoSoulShot(SoulshotId, false));
				player.sendPacket(new SystemMessage(SystemMessage.THE_AUTOMATIC_USE_OF_S1_WILL_NOW_BE_CANCELLED).addString(itemTemplate.getName()));
				return;
			}
			player.sendPacket(CANNOT_USE_SPIRITSHOTS);
			return;
		}

		if(grade == L2Item.CRYSTAL_NONE && spiritshotId != 3947 || grade == L2Item.CRYSTAL_D && spiritshotId != 3948 || grade == L2Item.CRYSTAL_C && spiritshotId != 3949 || grade == L2Item.CRYSTAL_B && spiritshotId != 3950 || grade == L2Item.CRYSTAL_A && spiritshotId != 3951 || grade >= L2Item.CRYSTAL_S && spiritshotId != 3952)
		{
			if(isAutoSoulShot)
				return;
			player.sendPacket(SPIRITSHOT_DOES_NOT_MATCH_WEAPON_GRADE);
			return;
		}

		if(count < blessedsoulSpiritConsumption)
		{
			if(isAutoSoulShot)
			{
				player.removeAutoSoulShot(SoulshotId);
				player.sendPacket(new ExAutoSoulShot(SoulshotId, false));
				player.sendPacket(new SystemMessage(SystemMessage.THE_AUTOMATIC_USE_OF_S1_WILL_NOW_BE_CANCELLED).addString(itemTemplate.getName()));
				return;
			}
			player.sendPacket(NOT_ENOUGH_SPIRITSHOTS);
			return;
		}

		weaponInst.setChargedSpiritshot(L2ItemInstance.CHARGED_BLESSED_SPIRITSHOT);
		player.getInventory().destroyItem(item, blessedsoulSpiritConsumption, false);
		player.sendPacket(POWER_OF_MANA_ENABLED);
		player.broadcastPacket(new MagicSkillUse(player, player, _skillIds[grade], 1, 0, 0));
	}

	public final short[] getItemIds()
	{
		return _itemIds;
	}

	public void onLoad()
	{
		ItemHandler.getInstance().registerItemHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}