package items;

import l2r.Config;
import l2r.extensions.multilang.CustomMessage;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.handler.IItemHandler;
import l2r.gameserver.handler.ItemHandler;
import l2r.util.Rnd;
import l2r.gameserver.model.FishDropData;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.FishTable;
import l2r.gameserver.tables.ItemTable;

import java.util.List;

public class FishItem implements IItemHandler, ScriptFile
{
	private static short[] _itemIds = null;

	public FishItem()
	{
		FishTable ft = FishTable.getInstance();
		_itemIds = new short[ft.GetFishItemCount()];
		for(int i = 0; i < ft.GetFishItemCount(); i++)
			_itemIds[i] = ft.getFishIdfromList(i);
	}

	public synchronized void useItem(L2Playable playable, L2ItemInstance item)
	{
		if(playable == null || !playable.isPlayer)
			return;
		L2Player player = (L2Player) playable;

		List<FishDropData> rewards = FishTable.getInstance().getFishReward(item.getItemId());
		int count = 0;
		player.getInventory().destroyItem(item, 1, true);
		for(FishDropData d : rewards)
			if(Rnd.chance(d.getChance()))
			{
				giveItems(player, d.getRewardItemId(), Rnd.get((int) (d.getMinCount() * Config.RATE_FISH_DROP_COUNT), (int) (d.getMaxCount() * Config.RATE_FISH_DROP_COUNT)));
				count++;
				continue;
			}
		if(count == 0)
			player.sendMessage(new CustomMessage("scripts.items.FishItem.Nothing", player));
	}

	public void giveItems(L2Player activeChar, short itemId, long count)
	{
		L2ItemInstance item = ItemTable.getInstance().createItem(itemId, activeChar.getObjectId(), 0, "FishItem_add");
		item.setCount(count);
		activeChar.getInventory().addItem(item);
		if(count > 1)
			activeChar.sendPacket(new SystemMessage(SystemMessage.EARNED_S2_S1_s).addItemName(itemId).addNumber((int) count));
		else
			activeChar.sendPacket(new SystemMessage(SystemMessage.EARNED_S1).addItemName(itemId));
	}

	public short[] getItemIds()
	{
		return _itemIds;
	}

	public void onLoad()
	{
		ItemHandler.getInstance().registerItemHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}