package items;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.handler.IItemHandler;
import l2r.gameserver.handler.ItemHandler;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.SystemMessage;

public class Scrolls implements IItemHandler, ScriptFile
{
	private static final short[] _itemIds = { 8515, 8516, 8517, 8518, 8519, 8520 // charms of courage
	};

	static final SystemMessage INCOMPATIBLE_ITEM_GRADE_THIS_ITEM_CANNOT_BE_USED = new SystemMessage(SystemMessage.INCOMPATIBLE_ITEM_GRADE_THIS_ITEM_CANNOT_BE_USED);

	public void useItem(L2Playable playable, L2ItemInstance item)
	{
		if(playable == null || !playable.isPlayer)
			return;
		L2Player player = (L2Player) playable;

		if(player.isActionsDisabled() || player.isSitting())
		{
			player.sendActionFailed();
			return;
		}

		int itemId = item.getItemId();

		if(!validateScrollGrade(player.getLevel(), itemId))
		{
			player.sendPacket(INCOMPATIBLE_ITEM_GRADE_THIS_ITEM_CANNOT_BE_USED);
			return;
		}
		player.getInventory().destroyItem(item, 1, true);
		player.sendPacket(new SystemMessage(SystemMessage.YOU_CAN_FEEL_S1S_EFFECT).addItemName(itemId));
		player.setCharmOfCourage(true);
	}

	public boolean validateScrollGrade(byte playerLvl, int itemId)
	{
		switch(itemId)
		{
			case 8515:
				return playerLvl <= 19;
			case 8516:
				return playerLvl >= 20 && playerLvl <= 39;
			case 8517:
				return playerLvl >= 40 && playerLvl <= 51;
			case 8518:
				return playerLvl >= 52 && playerLvl <= 60;
			case 8519:
				return playerLvl >= 61 && playerLvl <= 75;
			case 8520:
				return playerLvl >= 76;
		}
		return false;
	}

	public final short[] getItemIds()
	{
		return _itemIds;
	}

	public void onLoad()
	{
		ItemHandler.getInstance().registerItemHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}