package npc.model;

import java.util.concurrent.Future;
import java.util.logging.Logger;

import javolution.util.FastList;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.ai.CtrlEvent;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Spawn;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.instances.L2BossInstance;
import l2r.gameserver.tables.NpcTable;
import l2r.gameserver.tables.SpawnTable;
import l2r.gameserver.templates.L2NpcTemplate;

/**
 * This class for Antharas
 *
 * @Autor FlareDrakon
 */
public final class AntharasInstance extends L2BossInstance
{
	protected static Logger _log = Logger.getLogger(AntharasInstance.class.getName());
	private static final int ANTHARAS_WAIT_FOR_PLAYERS_TIMEOUT = 60000 * 10; // 10 минут
	private static final int[] antharasIds = { 29066, 29067, 29068 };
	public int _playersEnter = 0;
	private boolean _wait = false;
	private boolean _despawned = false;

	@SuppressWarnings("unchecked")
	public Future chekPlayerTask;

	@SuppressWarnings("unchecked")
	public Future spawnMinionTask;

	@SuppressWarnings("unchecked")
	public Future spawnMinion2Task;

	public AntharasInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public void onSpawn()
	{
		Howplayers();
		setparam(false, false, checkSpawn());
	}

	private final void Howplayers()
	{
		FastList<L2Player> players = L2World.getAroundPlayers(this);
		for(L2Player pl : players)
			if(pl.isDead())
				players.remove(pl);
		_playersEnter = players.size();
	}

	public int checkSpawn()
	{
		if(_playersEnter >= 400)
			return antharasIds[2];
		else if(_playersEnter >= 300)
			return antharasIds[1];
		else
			return antharasIds[0];
	}

	public void setparam(boolean despawned, boolean wait, int RealId)
	{
		setDespawned(despawned);
		setWait(wait);

		if(RealId != getNpcId())
		{
			try
			{
				L2Spawn spawn = new L2Spawn(NpcTable.getTemplate(RealId));
				spawn.setAmount(1);
				spawn.setLoc(getLoc());
				spawn.setRespawnDelay(getSpawn().getRespawnDelay());
				spawn.setRespawnTime(getSpawn().getRespawnTime());

				//стапим все таски
				stopTask(chekPlayerTask);
				stopTask(spawnMinionTask);
				stopTask(spawnMinion2Task);

				decayMe(); // инвиз
				doDie(this); // убиваем
				deleteMe(); // убираем

				// спавним нового
				SpawnTable.getInstance().addNewSpawn(spawn, false, null);
				spawn.doSpawn(true);
			}
			catch(Exception e)
			{
				_log.warning(e.getMessage());
			}

			return;
		}
		if(_wait)
		{
			//дождались=)
			Howplayers();
			if(_playersEnter > 0)
				setWait(false);

			if(chekPlayerTask == null)
				chekPlayerTask = ThreadPoolManager.getInstance().scheduleGeneral(new AntharasSpawnScheduleTimerTask(), ANTHARAS_WAIT_FOR_PLAYERS_TIMEOUT);
			return;
		}
		if(!_despawned)
			getAI().notifyEvent(CtrlEvent.EVT_THINK);
	}

	/*	public void onEnterPlayer()
		{
			Howplayers();
			if(!_despawned && chekPlayerTask == null)
				chekPlayerTask = ThreadPoolManager.getInstance().scheduleGeneral(new AntharasSpawnScheduleTimerTask(), ANTHARAS_WAIT_FOR_PLAYERS_TIMEOUT);
		}
	*/

	public boolean getDespawned()
	{
		return _despawned;
	}

	public void setDespawned(boolean despawned)
	{
		_despawned = despawned;
	}

	public boolean getWait()
	{
		return _wait;
	}

	public void setWait(boolean wait)
	{
		_wait = wait;
	}

	public class AntharasSpawnScheduleTimerTask implements Runnable
	{
		public AntharasSpawnScheduleTimerTask()
		{}

		public void run()
		{
			try
			{
				setparam(_despawned, _wait, checkSpawn());
			}
			catch(Throwable t)
			{}
		}
	}

	public void stopTask(Future Task)
	{
		if(Task != null)
		{
			Task.cancel(true);
			Task = null;
		}
	}
}