package npc.model;

import l2r.gameserver.instancemanager.CHSiegeManager;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_ACTIVE;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.entity.CHSiege;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.templates.L2NpcTemplate;
import l2r.util.Location;

public class L2SiegeMonsterInstance extends L2NpcInstance
{
	private Location _homeLoc;

	protected final class RewardInfo
	{
		protected L2Character attacker;
		protected int dmg = 0;

		public RewardInfo(final L2Character attacker, final int dmg)
		{
			this.attacker = attacker;
			this.dmg = dmg;
		}

		public void addDamage(int dmg)
		{
			if(dmg < 0)
				dmg = 0;
			this.dmg += dmg;
		}

		@Override
		public int hashCode()
		{
			return attacker.getObjectId();
		}
	}

	/**
	 * Constructor of L2MonsterInstance (use L2Character and L2NpcInstance constructor).<BR><BR>
	 * <p/>
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Call the L2Character constructor to set the _template of the L2MonsterInstance (copy skills from template to object and link _calculators to NPC_STD_CALCULATOR) </li>
	 * <li>Set the name of the L2MonsterInstance</li>
	 * <li>Create a RandomAnimation Task that will be launched after the calculated delay if the server allow it </li><BR><BR>
	 *
	 * @param objectId	  Identifier of the object to initialized
	 * @param L2NpcTemplate Template to apply to the NPC
	 */
	public L2SiegeMonsterInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public boolean isAutoAttackable(L2Character attacker)
	{
		CHSiege siege = CHSiegeManager.getInstance().getSiege(this);
		if(!siege.isInProgress())
			return false;

		if(attacker.isPlayer)
		{
			if(!siege.getAttackers().contains(((L2Player) attacker).getClan()))
				return false;
		}
		else if(attacker.isPet || attacker.isSummon)
			if(!siege.getAttackers().contains(attacker.getOwner().getClan()))
				return false;

		return true;
	}

	public Location getHomeLoc()
	{
		return _homeLoc;
	}

	@Override
	public void onSpawn()
	{
		_homeLoc = getLoc();
		super.onSpawn();
	}

	public void returnHome()
	{
		if(!isInRangeSq(_homeLoc, 40000))
		{
			clearAggroList();
			moveToLocation(_homeLoc, 0, false);
		}
	}

	public void teleportHome()
	{
		clearAggroList();
		setAttackTimeout(Integer.MAX_VALUE);
		if(hasAI())
		{
			getAI().setAttackTarget(null);
			getAI().clientStopMoving();
			getAI().setIntention(AI_INTENTION_ACTIVE);
		}
		teleToLocation(_homeLoc);
	}

	@Override
	public boolean hasRandomAnimation()
	{
		return false;
	}

	@Override
	public boolean isInvul()
	{
		return false;
	}

	@Override
	public boolean isFearImmune()
	{
		return true;
	}
}