package quests._12535_WakeBaium;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.model.L2Spawn;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.gameserver.serverpackets.Earthquake;
import l2r.gameserver.serverpackets.PlaySound;
import l2r.gameserver.serverpackets.SocialAction;
import l2r.gameserver.tables.NpcTable;
import l2r.util.Location;

public class _12535_WakeBaium extends Quest implements ScriptFile
{
	private static final int Baium = 29020;
	private static final int BaiumNpc = 29025;
	private static final int Angel = 29021;
	private static final int AngelicVortex = 31862;
	private static final int BloodedFabric = 4295;
	private static final Location BAIUM_SPAWN_POSITION = new Location(116127, 17368, 10107, 35431);
	private static final Location TELEPORT_POSITION = new Location(113100, 14500, 10077);

	public void onLoad()
	{
		System.out.println("Loaded Quest: 12535: Wake Up Baium");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public _12535_WakeBaium()
	{
		super(12535, "_12535_WakeBaium", "Wake Up Baium", true);

		addStartNpc(BaiumNpc);
		addStartNpc(AngelicVortex);
		addTalkId(BaiumNpc);
		addKillId(Baium);
		addKillId(Angel);
		addTalkId(AngelicVortex);
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		if(npcId == BaiumNpc)
		{
			if(st.getInt("ok") == 1)
			{
				if(!npc.isBusy())
				{
					npc.setBusy(true);
					npc.setBusyMessage("Attending another player's request");
					npc.broadcastPacket(new Earthquake(npc.getLoc(), 40, 10));
					npc.broadcastPacket(new PlaySound(1, "BS02_A", 1, 0, npc.getLoc()));
					// Через 10 секунд из статуи вылезет Ктулху
					st.startQuestTimer("baiumup_timer", 10000);
					return "<html><head><body>Baium:<br>You call my name! Now you gonna die!</body></html>";
				}
				return "<html><head><body>Baium is busy!</body></html>";
			}
			st.exitCurrentQuest(true);
			return "<html><head><body>Conditions are not right to wake up Baium!</body></html>";
		}
		else if(npcId == AngelicVortex)
		{
			if(st.getQuestItemsCount(BloodedFabric) > 0)
			{
				L2NpcInstance baiumBoss = L2World.findNpcByNpcId(Baium);
				if(baiumBoss != null)
					return "<html><head><body>Angelic Vortex:<br>Baium is already woken up! You can't enter!</body></html>";
				L2NpcInstance isbaiumNpc = L2World.findNpcByNpcId(BaiumNpc);
				if(isbaiumNpc == null)
					return "<html><head><body>Angelic Vortex:<br>Baium now here is not present!</body></html>";
				st.takeItems(BloodedFabric, 1);
				st.getPlayer().teleToLocation(TELEPORT_POSITION);
				st.set("ok", "1");
				return "";
			}
			return "<html><head><body>Angelic Vortex:<br>You do not have enough items!</body></html>";
		}
		return null;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		if(npcId == Angel)
		{
			// TODO Если убивают 1 ангела, то количество миньонов у Байума возврастает на 1
			/*
			L2NpcInstance baiumBoss = L2World.findNpcByNpcId(Baium);
			if(baiumBoss != null)
			{
				L2NpcTemplate templateAngel = NpcTable.getTemplate(Angel);
				if(templateAngel == null)
					return null;

				L2NpcInstance newAngel = null;
				try
				{
					L2Spawn spawn = new L2Spawn(templateAngel);
					spawn.setLocx(baiumBoss.getX());
					spawn.setLocy(baiumBoss.getY());
					spawn.setLocz(baiumBoss.getZ());
					spawn.setAmount(1);
					spawn.setHeading(0);
					spawn.setRespawnDelay(0);
					spawn.init();
					spawn.stopRespawn(); // stop respawn
					newAngel = spawn.doSpawn(true);
				}
				catch(SecurityException e)
				{
						e.printStackTrace();
				}
				catch(ClassNotFoundException e)
				{
					e.printStackTrace();
				}

				if(newAngel != null)
				{
					((L2MinionInstance) newAngel).setLeader((L2MonsterInstance) baiumBoss);
					((L2MonsterInstance) baiumBoss).notifyMinionSpawned((L2MinionInstance) newAngel);
				}
			}
			*/
		}
		else if(npcId == Baium)
		{
			npc.broadcastPacket(new PlaySound(1, "BS02_D", 1, 0, npc.getLoc()));
			st.exitCurrentQuest(true);
		}
		return null;
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		if(event.equalsIgnoreCase("baiumup_timer"))
		{
			// Спаун Байума стандартными средствами иначе при релогине будильщика Байум исчезнет

			L2NpcInstance baiumBoss = null;

			try
			{
				L2Spawn spawn = new L2Spawn(NpcTable.getTemplate(Baium));
				spawn.setLoc(BAIUM_SPAWN_POSITION);
				spawn.setAmount(1);
				spawn.setRespawnDelay(0);
				spawn.stopRespawn(); // stop respawn
				baiumBoss = spawn.doSpawn(true);
			}
			catch(ClassNotFoundException e)
			{
				e.printStackTrace();
			}

			// Парализуем байума на 10 секунд пока он встает, чтоб никуда не побежал
			if(baiumBoss != null)
			{
				// Wake up Neo
				baiumBoss.setImobilised(true);
				baiumBoss.broadcastPacket(new SocialAction(baiumBoss.getObjectId(), 2));
				st.startQuestTimer("baiumup_unpara_timer", 10000);
			}

			// Удалим статую через doDie, иначе не начнется респ
			L2NpcInstance baiumNpc = L2World.findNpcByNpcId(BaiumNpc);
			if(baiumNpc != null)
			{
				baiumNpc.decayMe(); // Сперва делаем его невидимым
				baiumNpc.doDie(null);
				baiumNpc.deleteMe(); // Удаляем из мира
			}
		}
		else if(event.equalsIgnoreCase("baiumup_unpara_timer"))
		{
			// Снимаем паралич с Байума
			L2NpcInstance baiumBoss = L2World.findNpcByNpcId(Baium);
			if(baiumBoss != null)
				baiumBoss.setImobilised(false);
		}
		return null;
	}
}