package quests._136_MoreThanMeetsTheEye;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.util.Rnd;

/**
 * Квест More Than Meets The Eye
 * @author Sergey Ibryaev aka Artful
 */

public class _136_MoreThanMeetsTheEye extends Quest implements ScriptFile
{
	//NPC
	private static final int HARDIN = 30832;
	private static final int ERRICKIN = 30701;
	private static final int CLAYTON = 30464;
	//Item
	private static final int TransformSealbook = 9648;
	//Quest Item
	private static final int Ectoplasm = 9787;
	private static final int StabilizedEctoplasm = 9786;
	private static final int HardinsInstructions = 9788;
	private static final int MordeoCrystal = 9789;
	private static final int BlankSealbook = 9790;

	//Drop Cond
	//# [COND, NEWCOND, ID, REQUIRED, ITEM, NEED_COUNT, CHANCE, DROP]	
	private static final int[][] DROPLIST_COND = {
			{ 3, 4, 20636, 0, Ectoplasm, 35, 100, 1 },
			{ 3, 4, 20637, 0, Ectoplasm, 35, 100, 1 },
			{ 3, 4, 20638, 0, Ectoplasm, 35, 100, 1 },
			{ 3, 4, 20639, 0, Ectoplasm, 35, 100, 2 },
			{ 7, 8, 20215, 0, MordeoCrystal, 5, 100, 1 } };

	public void onLoad()
	{
		System.out.println("Loaded Quest: 136: More Than Meets The Eye");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public _136_MoreThanMeetsTheEye()
	{
		super(136, "_136_MoreThanMeetsTheEye", "More Than Meets The Eye", false);

		addStartNpc(HARDIN);
		addTalkId(HARDIN);
		addTalkId(ERRICKIN);
		addTalkId(CLAYTON);

		addQuestItem(new int[] { StabilizedEctoplasm, HardinsInstructions, BlankSealbook, Ectoplasm, MordeoCrystal });

		for(int i = 0; i < DROPLIST_COND.length; i++)
			addKillId(DROPLIST_COND[i][2]);
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;
		if(event.equalsIgnoreCase("30832-06.htm"))
		{
			st.set("cond", "2");
			st.set("id", "0");
			st.setState(STARTED);
			st.playSound(SOUND_ACCEPT);
		}
		else if(event.equalsIgnoreCase("30701-02.htm"))
		{
			st.set("cond", "3");
			st.setState(STARTED);
		}
		else if(event.equalsIgnoreCase("30832-12.htm"))
		{
			st.giveItems(HardinsInstructions, 1);
			st.set("cond", "6");
			st.setState(STARTED);
		}
		else if(event.equalsIgnoreCase("30464-02.htm"))
		{
			st.set("cond", "7");
			st.setState(STARTED);
		}
		else if(event.equalsIgnoreCase("30832-17.htm"))
		{
			st.playSound(SOUND_FINISH);
			st.giveItems(TransformSealbook, 1);
			st.giveItems(57, 67550, true);
			st.unset("id");
			st.unset("cond");
			st.exitCurrentQuest(false);
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		String htmltext = "noquest";
		int cond = st.getInt("cond");
		if(npcId == HARDIN)
		{
			if(cond == 0)
			{
				if(st.getPlayer().getLevel() >= 50)
				{
					st.set("cond", "1");
					htmltext = "30832-02.htm";
				}
				else
				{
					htmltext = "30832-01.htm";
					st.exitCurrentQuest(true);
				}
			}
			else if(cond == 2 || cond == 3 || cond == 4)
				htmltext = "30832-07.htm";
			else if(cond == 5)
			{
				st.takeItems(StabilizedEctoplasm, -1);
				htmltext = "30832-08.htm";
			}
			else if(cond == 6)
				htmltext = "30832-13.htm";
			else if(cond == 9)
			{
				st.takeItems(BlankSealbook, -1);
				htmltext = "30832-14.htm";
			}
		}
		else if(npcId == ERRICKIN)
		{
			if(cond == 2)
				htmltext = "30701-01.htm";
			else if(cond == 3)
				htmltext = "30701-02.htm";
			else if(cond == 4 && st.getQuestItemsCount(Ectoplasm) < 35 && st.getInt("id") == 0)
			{
				st.set("cond", "3");
				htmltext = "30701-02.htm";
			}
			else if(cond == 4 && st.getInt("id") == 0)
			{
				st.takeItems(Ectoplasm, -1);
				htmltext = "30701-03.htm";
				st.set("id", "1");
			}
			else if(cond == 4 && st.getInt("id") == 1)
			{
				htmltext = "30701-04.htm";
				st.giveItems(StabilizedEctoplasm, 1);
				st.set("id", "0");
				st.set("cond", "5");
				st.setState(STARTED);
			}
			else if(cond == 5)
				htmltext = "30701-05.htm";
		}
		else if(npcId == CLAYTON)
			if(cond == 6)
			{
				st.takeItems(HardinsInstructions, -1);
				htmltext = "30464-01.htm";
			}
			else if(cond == 7)
				htmltext = "30464-03.htm";
			else if(cond == 8 && st.getQuestItemsCount(MordeoCrystal) < 5)
			{
				htmltext = "30464-03.htm";
				st.set("cond", "7");
			}
			else if(cond == 8)
			{
				htmltext = "30464-04.htm";
				st.takeItems(MordeoCrystal, -1);
				st.giveItems(BlankSealbook, 1);
				st.set("cond", "9");
				st.setState(STARTED);
			}
			else if(cond == 9)
				htmltext = "30464-05.htm";
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		int cond = st.getInt("cond");
		for(int i = 0; i < DROPLIST_COND.length; i++)
			if(cond == DROPLIST_COND[i][0] && npcId == DROPLIST_COND[i][2])
				if(DROPLIST_COND[i][3] == 0 || st.getQuestItemsCount(DROPLIST_COND[i][3]) > 0)
				{
					long count = st.getQuestItemsCount(DROPLIST_COND[i][4]);
					if(DROPLIST_COND[i][5] > count && Rnd.chance(DROPLIST_COND[i][6]))
					{
						long random = 0;
						if(DROPLIST_COND[i][7] > 1)
						{
							random = Rnd.get(DROPLIST_COND[i][7]) + 1;
							if(count + random > DROPLIST_COND[i][5])
								random = DROPLIST_COND[i][5] - count;
						}
						else
							random = 1;
						//Аддон
						if(cond == 3)
						{
							if(random == 1)
							{
								if(Rnd.chance(15))
									random = 2;
							}
							else if(Rnd.chance(15))
								random = 3;
							if(count + random > DROPLIST_COND[i][5])
								random = DROPLIST_COND[i][5] - count;
						}
						//Конец Аддона
						st.giveItems(DROPLIST_COND[i][4], random);
						if(count + random == DROPLIST_COND[i][5])
						{
							st.playSound(SOUND_MIDDLE);
							if(DROPLIST_COND[i][1] != 0)
							{
								st.set("cond", String.valueOf(DROPLIST_COND[i][1]));
								st.setState(STARTED);
							}
						}
						else
							st.playSound(SOUND_ITEMGET);
					}
				}
		return null;
	}
}