package quests._235_MimirsElixir;

import l2r.Config;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.gameserver.model.quest.State;
import l2r.util.Rnd;

public class _235_MimirsElixir extends Quest implements ScriptFile
{
	public void onLoad()
	{
		System.out.println("Loaded Quest: 235: Mimirs Elixir");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	private final static int chance = 45;

	//prerequisites:
	private final static int STAR_OF_DESTINY = 5011;
	private final static int MINLEVEL = 75;

	//Quest items
	private final static int PURE_SILVER = 6320;
	private final static int TRUE_GOLD = 6321;
	private final static int SAGES_STONE = 6322;
	private final static int BLOOD_FIRE = 6318;
	private final static int MIMIRS_ELIXIR = 6319;

	private final static int SCROLL_ENCHANT_WEAPON_A = 729;

	//Messages
	private final static String _default = "noquest";

	//NPCs
	private final static int LADD = 30721;
	private final static int JOAN = 30718;
	private final static int Chimera_Piece = 20965;
	private final static int Bloody_Guardian = 21090;

	public _235_MimirsElixir()
	{
		super(235, "_235_MimirsElixir", "Mimirs Elixir", false);

		addStartNpc(LADD);

		addTalkId(JOAN);
		addTalkId(LADD);

		addKillId(Chimera_Piece);
		addKillId(Bloody_Guardian);
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;
		if(event.equalsIgnoreCase("1"))
		{
			st.takeItems(STAR_OF_DESTINY, -1);
			st.setState(STARTED);
			st.set("cond", "1");
			htmltext = "30721-02a.htm";
		}
		else if(event.equalsIgnoreCase("30718_1"))
		{
			st.set("cond", "3");
			htmltext = "30718-01a.htm";
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		State id = st.getState();
		if(id == COMPLETED)
			return "completed";
		String htmltext = _default;
		int npcId = npc.getNpcId();
		int cond = st.getInt("cond");
		if(npcId == LADD)
		{
			if(id.equals(CREATED))
			{
				st.set("cond", "0");
				if(st.getPlayer().getLevel() < MINLEVEL)
				{
					st.exitCurrentQuest(true);
					htmltext = "30721-01.htm"; //not qualified
				}
				else if(st.getPlayer().getQuestState("_234_FatesWhisper") != null && st.getPlayer().getQuestState("_234_FatesWhisper").isCompleted() || st.getQuestItemsCount(STAR_OF_DESTINY) > 0)
				{
					if(st.getQuestItemsCount(STAR_OF_DESTINY) > 0)
					{
						st.takeItems(STAR_OF_DESTINY, 1);
						htmltext = "30721-02.htm"; // Successful start: Bring me Pure silver from Reagents quest
					}
					else
						htmltext = "30721-02.htm";
				}
				else
				{
					st.exitCurrentQuest(true);
					htmltext = "30721-01a.htm"; //not qualified
				}
			}
			// was asked to get pure silver but has not done so yet.  Repeat: get pure silver
			else if(cond == 1 && st.getQuestItemsCount(PURE_SILVER) < 1)
				htmltext = "30721-03.htm"; // Bring me Pure silver from Reagents quest
			// got the pure silver and came back.  Ask for TrueGold.
			else if(cond == 1 && st.getQuestItemsCount(PURE_SILVER) > 0)
			{
				st.set("cond", "2");
				htmltext = "30721-04.htm"; // Bring me True Gold from Joan
			}
			else if(1 < cond && cond < 5) // интересное условие...
				htmltext = "30721-05.htm"; // Where is my GOLD?!  Bring to me first.
			// got the true gold...look for Blood fire
			else if(cond == 5)
			{
				st.set("cond", "6");
				htmltext = "30721-06.htm"; // find Blood Fire from "bloody guardians"
			}
			// still looking for blood fire?
			else if(cond == 6)
				htmltext = "30721-07.htm"; // find Blood Fire from "bloody guardians"
			// Ah, you got the blood fire!  Time to mix them up!
			else if(cond == 7 && st.getQuestItemsCount(PURE_SILVER) > 0 && st.getQuestItemsCount(TRUE_GOLD) > 0)
				htmltext = "30721-08.htm"; // what are you standing there for?  Go to the cauldron and mix them...
			// you idiot, how did you lose your quest items?
			else if(cond == 7)
			{
				htmltext = "30721-09.htm"; // Well...you already know what to do...go get the 3 items...
				st.set("cond", "3"); // start over...yay...
			}
			// cond for this quest is set to 8 from Supplier or Reagents, when you create Mimir's Elixir.
			// Finally, all is done...time to learn how to use the Elixir...
			else if(cond == 8)
			{
				htmltext = "30721-10.htm"; // here's what you do...
				st.takeItems(MIMIRS_ELIXIR,-1);
				st.giveItems(SCROLL_ENCHANT_WEAPON_A, 1, Config.RATE_QUESTS_OCCUPATION_CHANGE);
				st.unset("cond");
				st.exitCurrentQuest(false);
			}

		}
		else if(npcId == JOAN)
			// first time talking to Joan: You ask for True Gold, she sends you for Sage's stone
			if(cond == 2)
				htmltext = "30718-01.htm"; // You want True Gold?  Please get the sage's stone.  Kill Chimera!
			// Why are you back alraedy?  You don't have the stone.
			else if(cond == 3)
				htmltext = "30718-02.htm"; // you haven't gotten the sage's stone yet?
			// aha!  Here is the sage's stone!  Cool, now we can make true gold
			else if(cond == 4)
			{
				st.takeItems(SAGES_STONE, -1);
				st.giveItems(TRUE_GOLD, 1);
				st.set("cond", "5");
				htmltext = "30718-03.htm"; // here you go...take the gold.  Now go back to ladd.
			}
			else if(cond >= 5)
				htmltext = "30718-04.htm"; // Go back to ladd already!
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		int cond = st.getInt("cond");
		if(npcId == Chimera_Piece && cond == 3 && st.getQuestItemsCount(SAGES_STONE) == 0 && Rnd.chance(chance))
		{
			st.giveItems(SAGES_STONE, 1);
			st.playSound(SOUND_ITEMGET);
			st.set("cond", str(cond + 1));
		}
		if(npcId == Bloody_Guardian && cond == 6 && st.getQuestItemsCount(BLOOD_FIRE) == 0 && Rnd.chance(chance))
		{
			st.giveItems(BLOOD_FIRE, 1);
			st.playSound(SOUND_ITEMGET);
			st.set("cond", str(cond + 1));
		}
		return null;
	}
}