package quests._355_FamilyHonor;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.util.Rnd;

public class _355_FamilyHonor extends Quest implements ScriptFile
{
	//NPC
	private static final int GALIBREDO = 30181;
	private static final int PATRIN = 30929;

	//CHANCES
	private static final int CHANCE_FOR_GALFREDOS_BUST = 80;
	private static final int CHANCE_FOR_GODDESS_BUST = 30;

	//ITEMS
	private static final int GALFREDOS_BUST = 4252;
	private static final int BUST_OF_ANCIENT_GODDESS = 4349;
	private static final int WORK_OF_BERONA = 4350;
	private static final int STATUE_PROTOTYPE = 4351;
	private static final int STATUE_ORIGINAL = 4352;
	private static final int STATUE_REPLICA = 4353;
	private static final int STATUE_FORGERY = 4354;

	public void onLoad()
	{
		System.out.println("Loaded Quest: 355: Family Honor");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public _355_FamilyHonor()
	{
		super(355, "_355_FamilyHonor", "Family Honor", true);

		addStartNpc(GALIBREDO);
		addTalkId(GALIBREDO);
		addTalkId(PATRIN);

		// TIMAK ORC TROOPS
		addKillId(20767);
		addKillId(20768);
		addKillId(20769);
		addKillId(20770);
		addKillId(20771);
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;
		if(event.equals("30181-1.htm"))
			return htmltext;
		else if(event.equals("30181-2.htm"))
		{
			st.set("cond", "1");
			st.setState(STARTED);
			st.playSound(SOUND_ACCEPT);
		}
		else if(event.equals("30181-4.htm"))
		{
			long count = st.getQuestItemsCount(BUST_OF_ANCIENT_GODDESS);
			st.takeItems(BUST_OF_ANCIENT_GODDESS, count);
			st.giveItems(WORK_OF_BERONA, count);
		}
		else if(event.equals("30929-0.htm"))
			return htmltext;
		else if(event.equals("30929-1.htm"))
			return htmltext;
		else if(event.equals("appraise"))
		{
			int appraising = Rnd.get(100);
			if(appraising < 20)
			{
				htmltext = "30929-2.htm";
				st.takeItems(WORK_OF_BERONA, 1);
			}
			else if(appraising < 40)
			{
				htmltext = "30929-3.htm";
				st.takeItems(WORK_OF_BERONA, 1);
				st.giveItems(STATUE_REPLICA, 1);
			}
			else if(appraising < 60)
			{
				htmltext = "30929-4.htm";
				st.takeItems(WORK_OF_BERONA, 1);
				st.giveItems(STATUE_ORIGINAL, 1);
			}
			else if(appraising < 80)
			{
				htmltext = "30929-5.htm";
				st.takeItems(WORK_OF_BERONA, 1);
				st.giveItems(STATUE_FORGERY, 1);
			}
			else if(appraising < 100)
			{
				htmltext = "30929-6.htm";
				st.takeItems(WORK_OF_BERONA, 1);
				st.giveItems(STATUE_PROTOTYPE, 1);
			}
		}
		else if(event.equals("30181-5.htm"))
		{
			st.playSound(SOUND_FINISH);
			st.exitCurrentQuest(true);
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = "noquest";
		int npcId = npc.getNpcId();
		int cond = st.getInt("cond");
		long count = st.getQuestItemsCount(GALFREDOS_BUST);
		if(npcId == GALIBREDO)
		{
			if(cond == 0)
				if(st.getPlayer().getLevel() >= 36)
					htmltext = "30181-0.htm";
				else
				{
					htmltext = "30181-0a.htm";
					st.exitCurrentQuest(true);
				}
			else if(cond == 1)
				if(count > 0)
				{
					long reward = count * 232;
					if(count >= 100)
						reward = reward + 5000; // custom - need more info
					st.takeItems(GALFREDOS_BUST, count);
					st.giveItems(57, reward);
					htmltext = "30181-3.htm";
				}
				else
					htmltext = "30181-2a.htm";
		}
		else if(npcId == PATRIN)
			if(st.getQuestItemsCount(WORK_OF_BERONA) > 0)
				htmltext = "30929-0.htm";
			else
				htmltext = "<html><head><body>You have nothing to appraise.</body></html>";
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		if(st.getInt("cond") == 1)
		{
			if(Rnd.chance(CHANCE_FOR_GALFREDOS_BUST))
			{
				st.giveItems(GALFREDOS_BUST, 1);
				st.playSound(SOUND_ITEMGET);
			}
			if(Rnd.chance(CHANCE_FOR_GODDESS_BUST))
				st.giveItems(BUST_OF_ANCIENT_GODDESS, 1);
		}
		return null;
	}
}