package quests._374_WhisperOfDreams1;

import l2r.Config;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.gameserver.model.quest.State;
import l2r.util.Rnd;

public class _374_WhisperOfDreams1 extends Quest implements ScriptFile
{
	private int DROP_MAX_2 = 250; //default: ~ 1/250
	//This setting alters the original price for fabrics.
	//By default it gathers server's adena drop multiplier
	private int ADENA_X = (int) Config.RATE_DROP_ADENA;
	//Rewards
	private static final String[][] SHOP_LIST = {
			{ "etc_leather_yellow_i00", "Sealed Tallum Tunic Textures" },
			{ "etc_leather_gray_i00", "Sealed Dark Crystal Robe Fabrics" },
			{ "etc_leather_gray_i00", "Sealed Nightmare Robe Fabric" },
			{ "etc_leather_gray_i00", "Sealed Majestic Robe Frabrics" },
			{ "etc_leather_gray_i00", "Sealed Tallum Stockings Fabrics" } };

	private static final int[][] SHOP_PRICES = {
	//id,kolvo,price
			{ 5485, 4, 10450 },
			{ 5486, 3, 2950 },
			{ 5487, 2, 18050 },
			{ 5488, 2, 18050 },
			{ 5489, 6, 15550 } };

	//Quest items
	private int CB_TOOTH = 5884; //Cave Beast Tooth
	private int DW_LIGHT = 5885; //Death Wave Light
	private int SEALD_MSTONE = 5886; //Sealed Mysterious Stone
	private int MSTONE = 5887; //Mysterious Stone

	private String _default = "noquest";

	//NPCs
	private int MANAKIA = 30515;
	private int TORAI = 30557;

	//Mobs & Drop
	private int[][] DROPLIST = { { 20620, CB_TOOTH }, { 20621, DW_LIGHT } };

	private String render_shop()
	{
		String html = "<html><head><body><font color=\"LEVEL\">Robe Armor Fabrics:</font><table border=0 width=300>";
		for(int i = 0; i < SHOP_LIST.length; i++)
		{
			html += "<tr><td width=35 height=45><img src=icon." + SHOP_LIST[i][0] + " width=32 height=32 align=left></td><td width=365 valign=top><table border=0 width=100%>";
			html += "<tr><td><a action=\"bypass -h Quest _374_WhisperOfDreams1 " + SHOP_PRICES[i][0] + "\"><font color=\"FFFFFF\">" + SHOP_LIST[i][1] + " x" + SHOP_PRICES[i][1] + "</font></a></td></tr>";
			html += "<tr><td><a action=\"bypass -h Quest _374_WhisperOfDreams1 " + SHOP_PRICES[i][0] + "\"><font color=\"B09878\">" + SHOP_PRICES[i][2] * ADENA_X + " adena</font></a></td></tr></table></td></tr>";
		}
		html += "</table></body></html>";
		return html;
	}

	public _374_WhisperOfDreams1()
	{
		super(374, "_374_WhisperOfDreams1", "Whisper Of Dreams 1", true);

		//Quest NPC starter initialization
		addStartNpc(MANAKIA);
		//Quest initialization
		addTalkId(MANAKIA);
		addTalkId(TORAI);

		addKillId(20620);
		addKillId(20621);
		addQuestItem(new int[] { CB_TOOTH, DW_LIGHT });
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		int DROP_RATE = 20 * (int) st.getRateQuestsReward();
		if(DROP_RATE > 100)
			DROP_RATE = 100;

		String htmltext = event;
		if(event.equalsIgnoreCase("30515-4.htm"))
		{
			st.setState(STARTED);
			st.set("cond", "1");
			st.playSound(SOUND_ACCEPT);
		}
		else if(event.equalsIgnoreCase("30515-5.htm"))
		{
			st.takeItems(CB_TOOTH, -1);
			st.takeItems(DW_LIGHT, -1);
			st.takeItems(SEALD_MSTONE, -1);
			st.exitCurrentQuest(true);
		}
		else if(event.equalsIgnoreCase("30515-6.htm"))
		{
			long q1 = st.getQuestItemsCount(CB_TOOTH);
			long q2 = st.getQuestItemsCount(DW_LIGHT);
			if(q1 == q2 && q1 == 65)
			{
				st.set("allow", "1");
				st.takeItems(CB_TOOTH, -1);
				st.takeItems(DW_LIGHT, -1);
				htmltext = "30515-7.htm";
			}
		}
		else if(event.equalsIgnoreCase("30515-8.htm"))
		{
			if(st.getQuestItemsCount(SEALD_MSTONE) > 0 && st.getInt("cond") < 2)
			{
				st.set("cond", "2");
				htmltext = "30515-9.htm";
			}
			else if(st.getInt("cond") == 2)
				htmltext = "30515-10.htm";
		}
		else if(event.equalsIgnoreCase("buy"))
			htmltext = render_shop();
		else if(isdigit(event))
		{
			int evt = Integer.parseInt(event);
			for(int[] element : SHOP_PRICES)
				if(evt == element[0])
				{
					int price = element[2] * ADENA_X;
					if(st.getQuestItemsCount(57) >= price)
					{
						st.takeItems(57, price);
						st.giveItems(element[0], element[1]);
						st.playSound(SOUND_FINISH);
						st.set("allow", "0");
						st.exitCurrentQuest(true);
						htmltext = "30515-11.htm";
					}
					else
						st.getPlayer().sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
					break;
				}
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = _default;
		State id = st.getState();
		int npcid = npc.getNpcId();
		if(npcid == MANAKIA)
		{
			if(id.equals(CREATED))
			{
				st.set("cond", "0");
				st.set("allow", "0");
				htmltext = "30515-1.htm";
				if(st.getPlayer().getLevel() < 56)
				{
					st.exitCurrentQuest(true);
					htmltext = "30515-2.htm";
				}
			}
			else if(st.getInt("allow") == 1)
				htmltext = "30515-3.htm";
			else
				htmltext = "30515-3a.htm";
		}
		else if(npcid == TORAI)
			if(st.getQuestItemsCount(SEALD_MSTONE) > 0)
			{
				htmltext = "30557-1.htm";
				st.takeItems(SEALD_MSTONE, 1);
				st.giveItems(MSTONE, 1);
				st.set("cond", "3");
				st.playSound(SOUND_MIDDLE);
			}
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int DROP_RATE = 20 * (int) st.getRateQuestsReward();
		if(DROP_RATE > 100)
			DROP_RATE = 100;

		// Mysterious Stone drop rate
		int DROP_RATE_2 = (int) st.getRateQuestsReward();
		if(DROP_RATE_2 > 250)
			DROP_RATE_2 = 250;

		int npcid = npc.getNpcId();
		int item = 0;
		for(int[] element : DROPLIST)
			if(npcid == element[0])
				item = element[1];
		long count = st.getQuestItemsCount(item);
		if(count < 65 && Rnd.chance(DROP_RATE))
		{
			st.giveItems(item, 1);
			if(count + 1 >= 65)
				st.playSound(SOUND_MIDDLE);
			else
				st.playSound(SOUND_ITEMGET);
		}
		if(st.getState().equals(STARTED))
			if(Rnd.get(DROP_MAX_2) < DROP_RATE_2 && st.getQuestItemsCount(SEALD_MSTONE) < 1)
			{
				st.giveItems(SEALD_MSTONE, 1);
				st.playSound(SOUND_MIDDLE);
			}
		return null;
	}

	public void onLoad()
	{
		System.out.println("Loaded Quest: 374: Whisper Of Dreams 1");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}