package quests._377_GiantsExploration2;

import l2r.Config;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.gameserver.model.quest.State;
import l2r.util.Rnd;

public class _377_GiantsExploration2 extends Quest implements ScriptFile
{
	public void onLoad()
	{
		System.out.println("Loaded Quest: 377: Giants Cave Exploration part 2");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	// Titan Ancient Books drop rate in %
	private static final int DROP_RATE = 20;

	// Quest items
	private static final int ANC_BOOK = 5955;
	private static final int DICT2 = 5892;

	// Quest collections
	private static final int[][] EXCHANGE = { { 5945, 5946, 5947, 5948, 5949 }, //science basis
			{ 5950, 5951, 5952, 5953, 5954 } //culture
	};

	// Messages
	private static final String error_1 = "<html><head><body>Head Researcher Sobling:<br><br>I think it is too early for you to help me. Come back after you have gained some more experience. <br><font color=\"LEVEL\">(Quest for characters level 57 and above.)</font></body></html>";
	private static final String start = "<html><head><body>Head Researcher Sobling:<br><br>So Cliff sent us this dictionary, i can see clearly now. It's very impressive... There are more relics for we to find out and maybe you will help us as a future member of our excavation team. We should look for <font color=\"LEVEL\">The book of the Titan's science, and the Book of the Titan's Culture.</font><br><br>Our payment for such a discovery cannot be rejected so easily, <font color=\"LEVEL\">A grade recipes</font> used in the manufacture of top level armors... Of course i won't give you anything just for fragments, you will have to gather every piece of a given book.<br><br><a action=\"bypass -h Quest _377_GiantsExploration2 yes\">I will search for ancient books</a><br><a action=\"bypass -h Quest _377_GiantsExploration2 0\">I won't help you this time</a><br></body></html>";
	private static final String starting = "Starting.htm";
	private static final String checkout = "<html><head><body>Head Researcher Sobling:<br><br>Excellent! You came back! Was it difficult to collect ancient books?<br><br>Let me see what you've found thus far...<br><br><a action=\"bypass -h Quest _377_GiantsExploration2 show\">Show him the books you collected</a></body></html>";
	private static final String checkout2 = "<html><head><body>Head Researcher Sobling:<br><br>Excellent! You came back! Was it difficult to collect ancient books?<br><br>Hum... what is it? You have some untranslated book fragments here, but those are of no use to me until you translate its contents, i have no time to do it on my own and that's why i gave you the dictionary Cliff sent to me. Anyway i can check any other translated fragments you may have...<br><br><a action=\"bypass -h Quest _377_GiantsExploration2 show\">Show him the fragments</a></body></html>";
	private static final String no_items = "<html><head><body>Head Researcher Sobling:<br><br>Hum... I don't see any valuable or complete book here, you should continue your research. I'm pretty sure you can do it better if you put more effort... what you think?<br><br><a action=\"bypass -h Quest _377_GiantsExploration2 Starting.htm\">I will continue</a><br><a action=\"bypass -h Quest _377_GiantsExploration2 0\">I will quit</a><br></body></html>";
	private static final String tnx4items = "<html><head><body>Head Researcher Sobling:<br><br>Amazing! These are the sort of items i was looking for... Take this rare recipes as a proof of my gratitude. Anyhow, I'm sure there are more ancient relics guarded by those monsters, would you like to search some more?<br><br><a action=\"bypass -h Quest _377_GiantsExploration2 Starting.htm\">I will continue</a><br><a action=\"bypass -h Quest _377_GiantsExploration2 0\">I will quit</a><br></body></html>";
	private static final String ext_msg = "Quest aborted";

	// NPCs
	private static final int HR_SOBLING = 31147;

	// Mobs
	private static final int[] MOBS = { 20654, 20656, 20657, 20658 };

	public _377_GiantsExploration2()
	{
		super(377, "_377_GiantsExploration2", "Giants Cave Exploration part 2", true);
		addStartNpc(HR_SOBLING);

		addKillId(MOBS);
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;
		if(event.equalsIgnoreCase("yes"))
		{
			htmltext = starting;
			st.setState(STARTED);
			st.set("cond", "1");
			st.playSound(SOUND_ACCEPT);
		}
		else if(event.equalsIgnoreCase("0"))
		{
			htmltext = ext_msg;
			st.playSound(SOUND_FINISH);
			st.exitCurrentQuest(true);
		}
		else if(event.equalsIgnoreCase("show"))
		{
			htmltext = no_items;
			for(int[] i : EXCHANGE)
			{
				long count = Long.MAX_VALUE;
				for(int j : i)
					count = Math.min(count, st.getQuestItemsCount(j));
				if(count > 0)
				{
					htmltext = tnx4items;
					for(int j : i)
						st.takeItems(j, count);
					for(int n = 0; n < count; n++)
					{
						int luck = Rnd.get(100);
						int item = 0;
						if(luck > 75)
							item = 5420; // nightmare leather 60%
						else if(luck > 50)
							item = 5422; // majestic plate 60%
						else if(luck > 25)
							item = 5336; // nightmare armor 60%
						else
							item = 5338; // majestic leather 60%
						if(Config.ALT_100_RECIPES)
							item += 1;
						st.giveItems(item, 1);
					}
				}
			}
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = "noquest";
		State id = st.getState();
		if(st.getQuestItemsCount(DICT2) == 0)
			st.exitCurrentQuest(true);
		else if(id == CREATED)
		{
			htmltext = start;
			if(st.getPlayer().getLevel() < 57)
			{
				st.exitCurrentQuest(true);
				htmltext = error_1;
			}
		}
		else if(id == STARTED)
			if(st.getQuestItemsCount(ANC_BOOK) == 0)
				htmltext = checkout;
			else
				htmltext = checkout2;
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		st.rollAndGive(ANC_BOOK, 1, DROP_RATE);
		return null;
	}
}