package quests._416_PathToOrcShaman;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.gameserver.model.quest.QuestTimer;
import l2r.util.Rnd;

/**
 * Квест Path To Orc Shaman
 * @author Sergey Ibryaev aka Artful
 */

public class _416_PathToOrcShaman extends Quest implements ScriptFile
{
	//NPC
	private static final int Hestui = 30585;
	private static final int HestuiTotemSpirit = 30592;
	private static final int SeerUmos = 30502;
	private static final int DudaMaraTotemSpirit = 30593;
	//Quest Items
	private static final int FireCharm = 1616;
	private static final int KashaBearPelt = 1617;
	private static final int KashaBladeSpiderHusk = 1618;
	private static final int FieryEgg1st = 1619;
	private static final int HestuiMask = 1620;
	private static final int FieryEgg2nd = 1621;
	private static final int TotemSpiritClaw = 1622;
	private static final int TatarusLetterOfRecommendation = 1623;
	private static final int FlameCharm = 1624;
	private static final int GrizzlyBlood = 1625;
	private static final int BloodCauldron = 1626;
	private static final int SpiritNet = 1627;
	private static final int BoundDurkaSpirit = 1628;
	private static final int DurkaParasite = 1629;
	private static final int TotemSpiritBlood = 1630;
	//Items
	private static final int MaskOfMedium = 1631;
	//MOB
	private static final int KashaBear = 20479;
	private static final int KashaBladeSpider = 20478;
	private static final int ScarletSalamander = 20415;
	private static final int GrizzlyBear = 20335;
	private static final int VenomousSpider = 20038;
	private static final int ArachnidTracker = 20043;
	private static final int QuestMonsterDurkaSpirit = 27056;

	//Drop Cond
	//# [COND, NEWCOND, ID, REQUIRED, ITEM, NEED_COUNT, CHANCE, DROP]	
	private static final int[][] DROPLIST_COND = {
			{ 1, 0, KashaBear, FireCharm, KashaBearPelt, 1, 70, 1 },
			{ 1, 0, KashaBladeSpider, FireCharm, KashaBladeSpiderHusk, 1, 70, 1 },
			{ 1, 0, ScarletSalamander, FireCharm, FieryEgg1st, 1, 70, 1 },
			{ 6, 7, GrizzlyBear, FlameCharm, GrizzlyBlood, 3, 70, 1 } };

	private static boolean QuestProf = true;

	public void onLoad()
	{
		System.out.println("Loaded Quest: 416 Path To Orc Shaman");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public _416_PathToOrcShaman()
	{
		super(416, "_416_PathToOrcShaman", "Path To Orc Shaman", false);
		addStartNpc(Hestui);
		addTalkId(Hestui);
		addTalkId(HestuiTotemSpirit);
		addTalkId(SeerUmos);
		addTalkId(DudaMaraTotemSpirit);
		//Mob Drop
		for(int i = 0; i < DROPLIST_COND.length; i++)
		{
			addKillId(DROPLIST_COND[i][2]);
			addQuestItem(DROPLIST_COND[i][4]);
		}
		addKillId(VenomousSpider);
		addKillId(ArachnidTracker);
		addKillId(QuestMonsterDurkaSpirit);
		addQuestItem(new int[] {
				FireCharm,
				HestuiMask,
				FieryEgg2nd,
				TotemSpiritClaw,
				TatarusLetterOfRecommendation,
				FlameCharm,
				BloodCauldron,
				SpiritNet,
				BoundDurkaSpirit,
				DurkaParasite,
				TotemSpiritBlood });
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;
		if(event.equalsIgnoreCase("30585-06.htm"))
		{
			st.giveItems(FireCharm, 1);
			st.set("cond", "1");
			st.setState(STARTED);
			st.playSound(SOUND_ACCEPT);
		}
		else if(event.equalsIgnoreCase("30592-03.htm"))
		{
			st.takeItems(HestuiMask, -1);
			st.takeItems(FieryEgg2nd, -1);
			st.giveItems(TotemSpiritClaw, 1);
			st.set("cond", "4");
			st.setState(STARTED);
		}
		else if(event.equalsIgnoreCase("30585-11.htm"))
		{
			st.takeItems(TotemSpiritClaw, -1);
			st.giveItems(TatarusLetterOfRecommendation, 1);
			st.set("cond", "5");
			st.setState(STARTED);
		}
		else if(event.equalsIgnoreCase("30593-03.htm"))
		{
			st.takeItems(BloodCauldron, -1);
			st.giveItems(SpiritNet, 1);
			st.set("cond", "9");
			st.setState(STARTED);
		}
		else if(event.equalsIgnoreCase("30502-07.htm"))
		{
			st.takeItems(TotemSpiritBlood, -1);
			st.giveItems(MaskOfMedium, 1);
			st.playSound(SOUND_FINISH);
			st.exitCurrentQuest(true);
		}
		if(event.equalsIgnoreCase("QuestMonsterDurkaSpirit_Fail"))
		{
			L2NpcInstance isQuest = L2World.findNpcByNpcId(QuestMonsterDurkaSpirit);
			if(isQuest != null)
				isQuest.deleteMe();
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		String htmltext = "noquest";
		int cond = st.getInt("cond");
		if(npcId == Hestui)
		{
			if(st.getQuestItemsCount(MaskOfMedium) != 0)
			{
				htmltext = "30502-04.htm";
				st.exitCurrentQuest(true);
			}
			else if(cond == 0)
			{
				if(st.getPlayer().getClassId().getId() != 0x31)
				{
					if(st.getPlayer().getClassId().getId() == 0x32)
						htmltext = "30585-02a.htm";
					else
						htmltext = "30585-02.htm";
					st.exitCurrentQuest(true);
				}
				else if(st.getPlayer().getLevel() < 19)
				{
					htmltext = "30585-03.htm";
					st.exitCurrentQuest(true);
				}
				else
					htmltext = "30585-01.htm";
			}
			else if(cond == 1)
				htmltext = "30585-07.htm";
			else if(cond == 2)
			{
				htmltext = "30585-08.htm";
				st.takeItems(KashaBearPelt, -1);
				st.takeItems(KashaBladeSpiderHusk, -1);
				st.takeItems(FieryEgg1st, -1);
				st.takeItems(FireCharm, -1);
				st.giveItems(HestuiMask, 1);
				st.giveItems(FieryEgg2nd, 1);
				st.set("cond", "3");
				st.setState(STARTED);
			}
			else if(cond == 3)
				htmltext = "30585-09.htm";
			else if(cond == 4)
				htmltext = "30585-10.htm";
			else if(cond == 5)
				htmltext = "30585-12.htm";
			else if(cond > 5)
				htmltext = "30585-13.htm";
		}
		else if(npcId == HestuiTotemSpirit)
		{
			if(cond == 3)
				htmltext = "30592-01.htm";
			else if(cond == 4)
				htmltext = "30592-04.htm";
		}
		else if(npcId == HestuiTotemSpirit && st.getInt("cond") > 0 && (st.getQuestItemsCount(GrizzlyBlood) > 0 || st.getQuestItemsCount(FlameCharm) > 0 || st.getQuestItemsCount(BloodCauldron) > 0 || st.getQuestItemsCount(SpiritNet) > 0 || st.getQuestItemsCount(BoundDurkaSpirit) > 0 || st.getQuestItemsCount(TotemSpiritBlood) > 0 || st.getQuestItemsCount(TatarusLetterOfRecommendation) > 0))
			htmltext = "30592-05.htm";
		else if(npcId == SeerUmos)
		{
			if(cond == 5)
			{
				st.takeItems(TatarusLetterOfRecommendation, -1);
				st.giveItems(FlameCharm, 1);
				htmltext = "30502-01.htm";
				st.set("cond", "6");
				st.setState(STARTED);
			}
			else if(cond == 6)
				htmltext = "30502-02.htm";
			else if(cond == 7)
			{
				st.takeItems(GrizzlyBlood, -1);
				st.takeItems(FlameCharm, -1);
				st.giveItems(BloodCauldron, 1);
				htmltext = "30502-03.htm";
				st.set("cond", "8");
				st.setState(STARTED);
			}
			else if(cond == 8)
				htmltext = "30502-04.htm";
			else if(cond == 9 || cond == 10)
				htmltext = "30502-05.htm";
			else if(cond == 11)
				htmltext = "30502-06.htm";
		}
		else if(npcId == DudaMaraTotemSpirit)
			if(cond == 8)
				htmltext = "30593-01.htm";
			else if(cond == 9)
				htmltext = "30593-04.htm";
			else if(cond == 10)
			{
				st.takeItems(BoundDurkaSpirit, -1);
				st.giveItems(TotemSpiritBlood, 1);
				htmltext = "30593-05.htm";
				st.set("cond", "11");
				st.setState(STARTED);
			}
			else if(cond == 11)
				htmltext = "30593-06.htm";
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		int cond = st.getInt("cond");
		for(int i = 0; i < DROPLIST_COND.length; i++)
			if(cond == DROPLIST_COND[i][0] && npcId == DROPLIST_COND[i][2])
				if(DROPLIST_COND[i][3] == 0 || st.getQuestItemsCount(DROPLIST_COND[i][3]) > 0)
				{
					if(DROPLIST_COND[i][5] == 0)
						st.rollAndGive(DROPLIST_COND[i][4], DROPLIST_COND[i][7], DROPLIST_COND[i][6], QuestProf);
					else if(st.rollAndGive(DROPLIST_COND[i][4], DROPLIST_COND[i][7], DROPLIST_COND[i][7], DROPLIST_COND[i][5], DROPLIST_COND[i][6], QuestProf))
					{
						if(DROPLIST_COND[i][1] != cond && DROPLIST_COND[i][1] != 0)
						{
							st.set("cond", String.valueOf(DROPLIST_COND[i][1]));
							st.setState(STARTED);
						}
					}
				}
		if(st.getQuestItemsCount(KashaBearPelt) != 0 && st.getQuestItemsCount(KashaBladeSpiderHusk) != 0 && st.getQuestItemsCount(FieryEgg1st) != 0)
		{
			st.set("cond", "2");
			st.setState(STARTED);
		}
		else if(cond == 9 && (npcId == VenomousSpider || npcId == ArachnidTracker))
		{
			if(st.getQuestItemsCount(DurkaParasite) < 8)
			{
				st.giveItems(DurkaParasite, 1);
				st.playSound(SOUND_ITEMGET);
			}
			if(st.getQuestItemsCount(DurkaParasite) == 8 || st.getQuestItemsCount(DurkaParasite) >= 5 && Rnd.chance(st.getQuestItemsCount(DurkaParasite) * 10))
			{
				L2NpcInstance isQuest = L2World.findNpcByNpcId(QuestMonsterDurkaSpirit);
				if(isQuest == null)
				{
					st.takeItems(DurkaParasite, -1);
					st.getPcSpawn().addSpawn(QuestMonsterDurkaSpirit);
					st.startQuestTimer("QuestMonsterDurkaSpirit_Fail", 300000);
				}
			}
		}
		else if(npcId == QuestMonsterDurkaSpirit)
		{
			QuestTimer timer = st.getQuestTimer("QuestMonsterDurkaSpirit_Fail");
			if(timer != null)
				timer.cancel();
			L2NpcInstance isQuest = L2World.findNpcByNpcId(QuestMonsterDurkaSpirit);
			if(isQuest != null)
				isQuest.deleteMe();
			if(cond == 9)
			{
				st.takeItems(SpiritNet, -1);
				st.takeItems(DurkaParasite, -1);
				st.giveItems(BoundDurkaSpirit, 1);
				st.playSound(SOUND_MIDDLE);
				st.set("cond", "10");
				st.setState(STARTED);
			}
		}
		return null;
	}
}