package quests._503_PursuitClanAmbition;

import java.sql.ResultSet;

import javolution.util.FastList;
import javolution.util.FastMap;
import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.gameserver.model.quest.QuestStateManager;
import l2r.gameserver.model.quest.State;
import l2r.util.Rnd;

public class _503_PursuitClanAmbition extends Quest implements ScriptFile
{
	// Items

	// first part
	private final short G_Let_Martien = 3866;
	private final short Th_Wyrm_Eggs = 3842;
	private final short Drake_Eggs = 3841;
	private final short Bl_Wyrm_Eggs = 3840;
	private final short Mi_Drake_Eggs = 3839;
	private final short Brooch = 3843;
	private final short Bl_Anvil_Coin = 3871;

	// second Part
	private final short G_Let_Balthazar = 3867;
	private final short Recipe_Power_Stone = 3838;
	private final short Power_Stones = 3846;
	private final short Nebulite_Crystals = 3844;
	private final short Broke_Power_Stone = 3845;

	// third part
	private final short G_Let_Rodemai = 3868;
	private final short Imp_Keys = 3847;
	private final short Scepter_Judgement = 3869;

	// the final item
	private final short Proof_Aspiration = 3870;

	private final int[] EggList = new int[4];
	private final int[] NPC = new int[12];
	private final String[] STATS = new String[5];

	private final FastMap<Integer, Quest.Drop> DROPLIST = new FastMap<Integer, Quest.Drop>();

	public void onLoad()
	{
		System.out.println("Loaded Quest: 503: Pursuit Clan Ambition");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public _503_PursuitClanAmbition()
	{
		super(503, "_503_PursuitClanAmbition", "Pursuit Clan Ambition", true);

		EggList[0] = Mi_Drake_Eggs;
		EggList[1] = Bl_Wyrm_Eggs;
		EggList[2] = Drake_Eggs;
		EggList[3] = Th_Wyrm_Eggs;

		NPC[0] = 30645; // Martien
		NPC[1] = 30758; // Athrea
		NPC[2] = 30759; // Kalis
		NPC[3] = 30760; // Gustaf
		NPC[4] = 30761; // Fritz
		NPC[5] = 30762; // Lutz
		NPC[6] = 30763; // Kurtz
		NPC[7] = 30512; // Kusto
		NPC[8] = 30764; // Balthazar
		NPC[9] = 30868; // Rodemai
		NPC[10] = 30765; // Coffer
		NPC[11] = 30766; // Cleo

		STATS[0] = "cond";
		STATS[1] = "Fritz";
		STATS[2] = "Lutz";
		STATS[3] = "Kurtz";
		STATS[4] = "ImpGraveKeeper";

		// Thunder Wyrm 1
		Quest.Drop drop = new Quest.Drop(2, 10, 20);
		drop.addItem(Th_Wyrm_Eggs);
		DROPLIST.put(20282, drop);

		// Thunder Wyrm 2
		drop = new Quest.Drop(2, 10, 15);
		drop.addItem(Th_Wyrm_Eggs);
		DROPLIST.put(20243, drop);

		// Drake 1
		drop = new Quest.Drop(2, 10, 20);
		drop.addItem(Drake_Eggs);
		DROPLIST.put(20137, drop);

		// Drake 2
		drop = new Quest.Drop(2, 10, 25);
		drop.addItem(Drake_Eggs);
		DROPLIST.put(20285, drop);

		// Blitz Wyrm
		drop = new Quest.Drop(2, 10, 100);
		drop.addItem(Bl_Wyrm_Eggs);
		DROPLIST.put(27178, drop);

		// Giant Soldier
		drop = new Quest.Drop(5, 10, 25);
		drop.addItem(Broke_Power_Stone);
		drop.addItem(Power_Stones);
		drop.addItem(Nebulite_Crystals);
		DROPLIST.put(20654, drop);

		// Giant Scouts
		drop = new Quest.Drop(5, 10, 35);
		drop.addItem(Broke_Power_Stone);
		drop.addItem(Power_Stones);
		drop.addItem(Nebulite_Crystals);
		DROPLIST.put(20656, drop);

		// Grave Guard
		drop = new Quest.Drop(10, 0, 15);
		DROPLIST.put(20668, drop);

		// GraveKeyKeeper
		drop = new Quest.Drop(10, 6, 80);
		drop.addItem(Imp_Keys);
		DROPLIST.put(27179, drop);

		// Imperial Gravekeeper
		drop = new Quest.Drop(10, 0, 100);
		DROPLIST.put(27181, drop);

		addStartNpc(NPC[3]);

		addTalkId(NPC[3]);

		for(int npcId : NPC)
			addTalkId(npcId);

		for(int mobId : DROPLIST.keySet())
			addKillId(mobId);

		addAttackId(27181);

		for(int i = 3839; i <= 3848; i++)
			addQuestItem(i);

		for(int i = 3866; i <= 3869; i++)
			addQuestItem(i);
	}

	public void suscribe_members(QuestState st)
	{
		int clan = st.getPlayer().getClan().getClanId();
		ThreadConnection con = null;
		FiltredPreparedStatement offline = null, insertion = null;
		ResultSet rs = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			offline = con.prepareStatement("SELECT obj_Id FROM characters WHERE clanid=? AND online=0");
			insertion = con.prepareStatement("REPLACE INTO character_quests (char_id,name,var,value) VALUES (?,?,?,?)");
			offline.setInt(1, clan);
			rs = offline.executeQuery();
			while(rs.next())
			{
				int char_id = rs.getInt("obj_Id");
				try
				{
					insertion.setInt(1, char_id);
					insertion.setString(2, getName());
					insertion.setString(3, "<state>");
					insertion.setString(4, "Progress");
					insertion.executeUpdate();
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeStatement(insertion);
			DatabaseUtils.closeDatabaseCSR(con, offline, rs);
		}
	}

	public void offlineMemberExit(QuestState st)
	{
		int clan = st.getPlayer().getClan().getClanId();
		ThreadConnection con = null;
		FiltredPreparedStatement offline = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			offline = con.prepareStatement("DELETE FROM character_quests WHERE name=? AND char_id IN (SELECT obj_id FROM characters WHERE clanId=? AND online=0)");
			offline.setString(1, getName());
			offline.setInt(2, clan);
			offline.executeUpdate();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, offline);
		}
	}

	// returns leaders quest cond, if he is offline will read out of database :)
	public int getLeaderVar(QuestState st, String var)
	{
		try
		{
			L2Clan clan = st.getPlayer().getClan();
			if(clan == null)
				return -1;
			L2Player leader = clan.getLeader().getPlayer();
			if(leader != null)
				return leader.getQuestState(getName()).getInt(var);
		}
		catch(Exception e)
		{
			return -1;
		}
		int leaderId = st.getPlayer().getClan().getLeaderId();
		ThreadConnection con = null;
		FiltredPreparedStatement offline = null;
		ResultSet rs = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			offline = con.prepareStatement("SELECT value FROM character_quests WHERE char_id=? AND var=? AND name=?");
			offline.setInt(1, leaderId);
			offline.setString(2, var);
			offline.setString(3, getName());
			int val = -1;
			rs = offline.executeQuery();
			if(rs.next())
				val = rs.getInt("value");
			return val;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return -1;
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, offline, rs);
		}
	}

	// set's leaders quest cond, if he is offline will read out of database :)
	// for now, if the leader is not logged in, this assumes that the variable
	// has already been inserted once (initialized, in some sense).
	public void setLeaderVar(QuestState st, String var, String value)
	{
		L2Clan clan = st.getPlayer().getClan();
		if(clan == null)
			return;
		L2Player leader = clan.getLeader().getPlayer();
		if(leader != null)
			leader.getQuestState(getName()).set(var, value);
		else
		{
			int leaderId = st.getPlayer().getClan().getLeaderId();
			ThreadConnection con = null;
			FiltredPreparedStatement offline = null;
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				offline = con.prepareStatement("UPDATE character_quests SET value=? WHERE char_id=? AND var=? AND name=?");
				offline.setString(1, value);
				offline.setInt(2, leaderId);
				offline.setString(3, var);
				offline.setString(4, getName());
				offline.executeUpdate();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
			finally
			{
				DatabaseUtils.closeDatabaseCS(con, offline);
			}
		}
	}

	public boolean checkEggs(QuestState st)
	{
		int count = 0;
		for(int item : EggList)
			if(st.getQuestItemsCount(item) > 9)
				count += 1;
		return count > 3;
	}

	public void giveItem(short item, long maxcount, QuestState st)
	{
		long count = st.getQuestItemsCount(item);
		if(count < maxcount)
		{
			st.giveItems(item, 1);
			if(count == maxcount - 1)
				st.playSound(SOUND_MIDDLE);
			else
				st.playSound(SOUND_ITEMGET);
		}
	}

	public String exit503(boolean completed, QuestState st)
	{
		if(completed)
		{
			st.giveItems(Proof_Aspiration, 1);
			st.addExpAndSp(0, 250000);
			for(String var : STATS)
				st.unset(var);
			st.exitCurrentQuest(false);
		}
		else
			st.exitCurrentQuest(true);
		st.takeItems(Scepter_Judgement, -1);
		st.getPcSpawn().removeAllSpawn();
		try
		{
			L2Player[] members = st.getPlayer().getClan().getOnlineMembers("");
			for(L2Player player : members)
			{
				if(player == null)
					continue;
				QuestState qs = player.getQuestState(getName());
				if(qs != null)
					qs.exitCurrentQuest(true);
			}
			offlineMemberExit(st);
		}
		catch(Exception e)
		{
			return "You dont have any members in your Clan, so you can't finish the Pursuit of Aspiration";
		}
		return "Congratulations, you have finished the Pursuit of Clan Ambition";
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;
		// Events Gustaf
		if(event.equalsIgnoreCase("30760-08.htm"))
		{
			st.giveItems(G_Let_Martien, 1);
			for(String var : STATS)
				st.set(var, "1");
			st.setState(STARTED);
		}
		else if(event.equalsIgnoreCase("30760-12.htm"))
		{
			st.giveItems(G_Let_Balthazar, 1);
			st.set("cond", "4");
		}
		else if(event.equalsIgnoreCase("30760-16.htm"))
		{
			st.giveItems(G_Let_Rodemai, 1);
			st.set("cond", "7");
		}
		else if(event.equalsIgnoreCase("30760-20.htm"))
			exit503(true, st);
		else if(event.equalsIgnoreCase("30760-22.htm"))
			st.set("cond", "13");
		else if(event.equalsIgnoreCase("30760-23.htm"))
			exit503(true, st);
		// Events Martien
		else if(event.equalsIgnoreCase("30645-03.htm"))
		{
			st.takeItems(G_Let_Martien, -1);
			st.set("cond", "2");
			suscribe_members(st);
			L2Player[] members = st.getPlayer().getClan().getOnlineMembers(st.getPlayer().getName());
			for(L2Player player : members)
				QuestStateManager.getInstance().addQuestState(this, player, STARTED);
		}
		// Events Kurtz
		else if(event.equalsIgnoreCase("30763-03.htm"))
		{
			if(st.getInt("Kurtz") == 1)
			{
				htmltext = "30763-02.htm";
				st.giveItems(Mi_Drake_Eggs, 6);
				st.giveItems(Brooch, 1);
				st.set("Kurtz", "2");
			}
		}
		// Events Lutz
		else if(event.equalsIgnoreCase("30762-03.htm"))
		{
			int lutz = st.getInt("Lutz");
			if(lutz == 1)
			{
				htmltext = "30762-02.htm";
				st.giveItems(Mi_Drake_Eggs, 4);
				st.giveItems(Bl_Wyrm_Eggs, 3);
				st.set("Lutz", "2");
			}
			st.getPcSpawn().addSpawn(27178, 112268, 112761, -2770, 120000);
			st.getPcSpawn().addSpawn(27178, 112234, 112705, -2770, 120000);
		}
		// Events Fritz
		else if(event.equalsIgnoreCase("30761-03.htm"))
		{
			int fritz = st.getInt("Fritz");
			if(fritz == 1)
			{
				htmltext = "30761-02.htm";
				st.giveItems(Bl_Wyrm_Eggs, 3);
				st.set("Fritz", "2");
			}
			st.getPcSpawn().addSpawn(27178, 103841, 116809, -3025, 120000);
			st.getPcSpawn().addSpawn(27178, 103848, 116910, -3020, 120000);
		}
		// Events Kusto
		else if(event.equalsIgnoreCase("30512-03.htm"))
		{
			st.takeItems(Brooch, -1);
			st.giveItems(Bl_Anvil_Coin, 1);
			st.set("Kurtz", "3");
		}
		// Events Balthazar
		else if(event.equalsIgnoreCase("30764-03.htm"))
		{
			st.takeItems(G_Let_Balthazar, -1);
			st.set("cond", "5");
			st.set("Kurtz", "3");
		}
		else if(event.equalsIgnoreCase("30764-05.htm"))
		{
			st.takeItems(G_Let_Balthazar, -1);
			st.set("cond", "5");
		}
		else if(event.equalsIgnoreCase("30764-06.htm"))
		{
			st.takeItems(Bl_Anvil_Coin, -1);
			st.set("Kurtz", "4");
			st.giveItems(Recipe_Power_Stone, 1);
		}
		// Events Rodemai
		else if(event.equalsIgnoreCase("30868-04.htm"))
		{
			st.takeItems(G_Let_Rodemai, -1);
			st.set("cond", "8");
		}
		else if(event.equalsIgnoreCase("30868-06a.htm"))
			st.set("cond", "10");
		else if(event.equalsIgnoreCase("30868-10.htm"))
			st.set("cond", "12");
		// Events Cleo
		else if(event.equalsIgnoreCase("30766-04.htm"))
		{
			st.set("cond", "9");
			String[] msg = new String[1];
			msg[0] = "Blood and Honour";
			st.getPcSpawn().addSpawn(30766, 160622, 21230, -3710, 100, msg, 0);
			msg[0] = "Ambition and Power";
			st.getPcSpawn().addSpawn(30759, 160665, 21209, -3710, 6000, msg, 0);
			msg[0] = "War and Death";
			st.getPcSpawn().addSpawn(30758, 160665, 21291, -3710, 6000, msg, 0);
		}
		else if(event.equalsIgnoreCase("30766-08.htm"))
		{
			st.takeItems(Scepter_Judgement, -1);
			exit503(false, st);
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		State id = st.getState();
		int Martien = 30645;
		int Athrea = 30758;
		int Kalis = 30759;
		int Gustaf = 30760;
		int Fritz = 30761;
		int Lutz = 30762;
		int Kurtz = 30763;
		int Kusto = 30512;
		int Balthazar = 30764;
		int Rodemai = 30868;
		int Coffer = 30765;
		int Cleo = 30766;

		String htmltext = "noquest";
		boolean isLeader = st.getPlayer().isClanLeader();
		if(id.equals(CREATED) && npcId == Gustaf)
		{
			for(String var : STATS)
				// adds all the vars for initialisation
				st.set(var, "0");
			if(st.getPlayer().getClan() != null) // has Clan
			{
				if(isLeader) // check if player is clan leader
				{
					int clanLevel = st.getPlayer().getClan().getLevel();
					if(st.getQuestItemsCount(Proof_Aspiration) > 0) // if he has the proof
					// already, tell him
					// what to do now
					{
						htmltext = "30760-03.htm";
						st.exitCurrentQuest(true);
					}
					else if(clanLevel > 3) // if clanLevel > 3 you can take this quest,
						// because repeatable
						htmltext = "30760-04.htm";
					else
					// if clanLevel < 4 you cant take it
					{
						htmltext = "30760-02.htm";
						st.exitCurrentQuest(true);
					}
				}
				else
				// player isnt a leader
				{
					htmltext = "30760-04t.htm";
					st.exitCurrentQuest(true);
				}
			}
			else
			// no Clan
			{
				htmltext = "30760-01.htm";
				st.exitCurrentQuest(true);
			}
			return htmltext;
		}
		else if(st.getPlayer().getClan() != null && st.getPlayer().getClan().getLevel() == 5) // player has level 5 clan already
			return "completed";
		else
		// ######## Leader Area ######
		if(isLeader)
		{
			if(st.get("cond") == null)
				st.set("cond", "1");
			if(st.get("Kurtz") == null)
				st.set("Kurtz", "1");
			if(st.get("Lutz") == null)
				st.set("Lutz", "1");
			if(st.get("Fritz") == null)
				st.set("Fritz", "1");
			int cond = st.getInt("cond");
			int kurtz = st.getInt("Kurtz");
			int lutz = st.getInt("Lutz");
			int fritz = st.getInt("Fritz");

			if(npcId == Gustaf)
			{
				if(cond == 1)
					htmltext = "30760-09.htm";
				else if(cond == 2)
					htmltext = "30760-10.htm";
				else if(cond == 3)
					htmltext = "30760-11.htm";
				else if(cond == 4)
					htmltext = "30760-13.htm";
				else if(cond == 5)
					htmltext = "30760-14.htm";
				else if(cond == 6)
					htmltext = "30760-15.htm";
				else if(cond == 7)
					htmltext = "30760-17.htm";
				else if(cond == 12)
					htmltext = "30760-19.htm";
				else if(cond == 13)
					htmltext = "30760-24.htm";
				else
					htmltext = "30760-18.htm";
			}
			else if(npcId == Martien)
			{
				if(cond == 1)
					htmltext = "30645-02.htm";
				else if(cond == 2)
					if(checkEggs(st) && kurtz > 1 && lutz > 1 && fritz > 1)
					{
						htmltext = "30645-05.htm";
						st.set("cond", "3");
						for(int item : EggList)
							st.takeItems(item, -1);
					}
					else
						htmltext = "30645-04.htm";
				else if(cond == 3)
					htmltext = "30645-07.htm";
				else
					htmltext = "30645-08.htm";
			}
			else if(npcId == Lutz && cond == 2)
				htmltext = "30762-01.htm";
			else if(npcId == Kurtz && cond == 2)
				htmltext = "30763-01.htm";
			else if(npcId == Fritz && cond == 2)
				htmltext = "30761-01.htm";
			else if(npcId == Kusto)
			{
				if(kurtz == 1)
					htmltext = "30512-01.htm";
				else if(kurtz == 2)
					htmltext = "30512-02.htm";
				else
					htmltext = "30512-04.htm";
			}
			else if(npcId == Balthazar)
			{
				if(cond == 4)
					if(kurtz > 2)
						htmltext = "30764-04.htm";
					else
						htmltext = "30764-02.htm";
				else if(cond == 5)
					if(st.getQuestItemsCount(Power_Stones) > 9 && st.getQuestItemsCount(Nebulite_Crystals) > 9)
					{
						htmltext = "30764-08.htm";
						st.takeItems(Power_Stones, -1);
						st.takeItems(Nebulite_Crystals, -1);
						st.takeItems(Brooch, -1);
						st.set("cond", "6");
					}
					else
						htmltext = "30764-07.htm";
				else if(cond == 6)
					htmltext = "30764-09.htm";
			}
			else if(npcId == Rodemai)
			{
				if(cond == 7)
					htmltext = "30868-02.htm";
				else if(cond == 8)
					htmltext = "30868-05.htm";
				else if(cond == 9)
					htmltext = "30868-06.htm";
				else if(cond == 10)
					htmltext = "30868-08.htm";
				else if(cond == 11)
					htmltext = "30868-09.htm";
				else if(cond == 12)
					htmltext = "30868-11.htm";
			}
			else if(npcId == Cleo)
			{
				if(cond == 8)
					htmltext = "30766-02.htm";
				else if(cond == 9)
					htmltext = "30766-05.htm";
				else if(cond == 10)
					htmltext = "30766-06.htm";
				else if(cond == 11 || cond == 12 || cond == 13)
					htmltext = "30766-07.htm";
			}
			else if(npcId == Coffer)
			{
				if(st.getInt("cond") == 10)
					if(st.getQuestItemsCount(Imp_Keys) < 6)
						htmltext = "30765-03a.htm";
					else if(st.getInt("ImpGraveKeeper") == 3)
					{
						htmltext = "30765-02.htm";
						st.set("cond", "11");
						st.takeItems(Imp_Keys, 6);
						st.giveItems(Scepter_Judgement, 1);
						st.getPcSpawn().removeAllSpawn();
					}
					else
						htmltext = "<html><head><body>(You and your Clan didn't kill the Imperial Gravekeeper by your own, do it try again.)</body></html>";
				else
					htmltext = "<html><head><body>(You already have the Scepter of Judgement.)</body></html>";
			}
			else if(npcId == Kalis)
				htmltext = "30759-01.htm";
			else if(npcId == Athrea)
				htmltext = "30758-01.htm";
			return htmltext;
		}
		// ######## Member Area ######
		else
		{
			int cond = getLeaderVar(st, "cond");
			if(npcId == Martien && (cond == 1 || cond == 2 || cond == 3))
				htmltext = "30645-01.htm";
			else if(npcId == Rodemai)
			{
				if(cond == 9 || cond == 10)
					htmltext = "30868-07.htm";
				else if(cond == 7)
					htmltext = "30868-01.htm";
			}
			else if(npcId == Balthazar && cond == 4)
				htmltext = "30764-01.htm";
			else if(npcId == Cleo && cond == 8)
				htmltext = "30766-01.htm";
			else if(npcId == Kusto && cond > 2 && cond < 6)
				htmltext = "30512-01a.htm";
			else if(npcId == Coffer && cond == 10)
				htmltext = "30765-01.htm";
			else if(npcId == Gustaf)
				if(cond == 3)
					htmltext = "30760-11t.htm";
				else if(cond == 4)
					htmltext = "30760-15t.htm";
				else if(cond == 12)
					htmltext = "30760-19t.htm";
				else if(cond == 13)
					htmltext = "30766-24t.htm";
			return htmltext;
		}
	}

	@Override
	public String onAttack(L2NpcInstance npc, QuestState st)
	{
		if(npc.getMaxHp() / 2 > npc.getCurrentHp())
			if(Rnd.chance(4))
			{
				int ImpGraveKepperStat = getLeaderVar(st, "ImpGraveKeeper");
				if(ImpGraveKepperStat == 1)
				{
					for(int i = 1; i <= 4; i++)
						st.getPcSpawn().addSpawn(27180);
					setLeaderVar(st, "ImpGraveKeeper", "2");
				}
				else
				{
					FastList<L2Player> players = npc.getAroundPlayers(900);
					if(players.size() > 0)
					{
						L2Player player = players.get(Rnd.get(players.size()));
						if(player != null)
							player.teleToLocation(185462, 20342, -3250);
					}
				}
			}
		return null;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		Quest.Drop drop = DROPLIST.get(npcId);

		int condition = drop.condition;
		int maxcount = drop.maxcount;
		int chance = drop.chance;
		FastList<Short> itemList = drop.itemList;

		int random = Rnd.get(100);
		int cond = getLeaderVar(st, "cond");

		if(cond == condition && random < chance)
			if(itemList.size() > 1)
			{
				int stoneRandom = Rnd.get(3);
				if(stoneRandom == 0)
				{
					if(getLeaderVar(st, "Kurtz") < 4)
						return null;
					maxcount *= 4;
				}
				giveItem(itemList.get(stoneRandom), maxcount, st);
			}
			else if(itemList.size() > 0)
				giveItem(itemList.get(0), maxcount, st);
			else if(npcId == 27181) // Imperial Gravekeeper
			{
				String[] msg = new String[1];
				msg[0] = "Curse of the gods on the one that defiles the property of the empire!";
				st.getPcSpawn().addSpawn(30765, 6000000, msg, 0);
				setLeaderVar(st, "ImpGraveKeeper", "3");
			}
			else
				st.getPcSpawn().addSpawn(27179);
		return null;
	}
}