package quests._509_TheClansPrestige;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.gameserver.model.quest.State;
import l2r.gameserver.serverpackets.SystemMessage;

public class _509_TheClansPrestige extends Quest implements ScriptFile
{
	// Quest NPC
	private static final int GRAND_MAGISTER_VALDIS = 31331;

	// Quest Items
	private static final int DAIMONS_EYES = 8489; // Daimon's Eyes : Eyes obtained by defeating Daimon the White-Eyed.
	private static final int HESTIAS_FAIRY_STONE = 8490; // Hestia's Fairy Stone : Obtain this Stone by defeating Hestia, Guardian Deity of the Hot Springs.
	private static final int NUCLEUS_OF_LESSER_GOLEM = 8491; // Nucleus of Lesser Golem : Nucleus obtained by defeating Plague Golem.
	private static final int FALSTONS_FANG = 8492; // Falston's Fang : Fang obtained by defeating Demon's Agent Falston.
	private static final int SHAIDS_TALON = 8493; // Shaid's Talon : Talon obtained by defeating Queen Shyeed.

	// Quest Raid Bosses
	private static final int DAIMON_THE_WHITE_EYED = 25290;
	private static final int HESTIA_GUARDIAN_DEITY = 25293;
	private static final int PLAGUE_GOLEM = 25523;
	private static final int DEMONS_AGENT_FALSTON = 25322;
	private static final int QUEEN_SHYEED = 25514;

	// Reward 100-300 points per boss (off-like)
	private static final int CLAN_POINTS_REWARD = 150; // middle

	// id:[RaidBossNpcId,questItemId]
	private static final int[][] REWARDS_LIST = {
			{ 0, 0 },
			{ DAIMON_THE_WHITE_EYED, DAIMONS_EYES },
			{ HESTIA_GUARDIAN_DEITY, HESTIAS_FAIRY_STONE },
			{ PLAGUE_GOLEM, NUCLEUS_OF_LESSER_GOLEM },
			{ DEMONS_AGENT_FALSTON, FALSTONS_FANG },
			{ QUEEN_SHYEED, SHAIDS_TALON } };

	private static final int[][] RADAR = {
			{ 0, 0, 0 },
			{ 186304, -43744, -3193 },
			{ 134672, -115600, -1216 },
			{ 168641, -60417, -3888 },
			{ 93296, -75104, -1824 },
			{ 79635, -55434, -6135 }, };

	public void onLoad()
	{
		System.out.println("Loaded Quest: 509: The Clans Prestige");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public _509_TheClansPrestige()
	{
		super(509, "_509_TheClansPrestige", "The Clans Prestige", true);

		addStartNpc(GRAND_MAGISTER_VALDIS);
		addTalkId(GRAND_MAGISTER_VALDIS);

		for(int[] i : REWARDS_LIST)
		{
			addKillId(i[0]);
			addQuestItem(i[1]);
		}
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		int cond = st.getInt("cond");
		String htmltext = event;
		if(event.equalsIgnoreCase("31331-0.htm") && cond == 0)
		{
			st.set("cond", "1");
			st.setState(STARTED);
		}
		else if(isdigit(event))
		{
			int evt = Integer.parseInt(event);
			st.set("raid", event);
			htmltext = "31331-" + event + ".htm";
			int x = RADAR[evt][0];
			int y = RADAR[evt][1];
			int z = RADAR[evt][2];
			if(x + y + z > 0)
				st.addRadar(x, y, z);
			st.playSound(SOUND_ACCEPT);
		}
		else if(event.equalsIgnoreCase("31331-6.htm"))
		{
			st.playSound(SOUND_FINISH);
			st.exitCurrentQuest(true);
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = "noquest";
		L2Clan clan = st.getPlayer().getClan();

		if(clan == null)
		{
			st.exitCurrentQuest(true);
			htmltext = "31331-0a.htm";
		}
		else if(clan.getLeader().getPlayer() != st.getPlayer())
		{
			st.exitCurrentQuest(true);
			htmltext = "31331-0a.htm";
		}
		else if(clan.getLevel() < 6)
		{
			st.exitCurrentQuest(true);
			htmltext = "31331-0b.htm";
		}
		else
		{
			int cond = st.getInt("cond");
			int raid = st.getInt("raid");
			State id = st.getState();
			if(id.equals(CREATED) && cond == 0)
				htmltext = "31331-0c.htm";
			else if(id.equals(STARTED) && cond == 1)
			{
				int item = REWARDS_LIST[raid][1];
				long count = st.getQuestItemsCount(item);
				if(count == 0)
					htmltext = "31331-" + raid + "a.htm";
				else if(count == 1)
				{
					htmltext = "31331-" + raid + "b.htm";
					int increasedPoints = clan.incReputation(CLAN_POINTS_REWARD, true, "CastleSiege");
					st.getPlayer().sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_SUCCESSFULLY_COMPLETED_A_CLAN_QUEST_S1_POINTS_HAVE_BEEN_ADDED_TO_YOUR_CLAN_REPUTATION_SCORE).addNumber(increasedPoints));
					st.takeItems(item, 1);
				}
			}
		}
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		QuestState id = null;
		if(st.getPlayer().getClan().getLeader().getPlayer() == st.getPlayer())
			id = st.getPlayer().getQuestState("_509_TheClansPrestige");
		else if(st.getPlayer().getClan().getLeader().getPlayer().getDistance(npc) <= 1600)
			id = st.getPlayer().getClan().getLeader().getPlayer().getQuestState("_509_TheClansPrestige");
		if(id == null)
			return null;
		if(st.getInt("cond") == 1 && st.getState() == STARTED)
		{
			int raid = REWARDS_LIST[st.getInt("raid")][0];
			int item = REWARDS_LIST[st.getInt("raid")][1];
			int npcId = npc.getNpcId();
			if(npcId == raid && st.getQuestItemsCount(item) == 0)
			{
				st.giveItems(item, 1);
				st.playSound(SOUND_MIDDLE);
			}
		}
		return null;
	}
}