package quests._633_InTheForgottenVillage;

import java.util.HashMap;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.gameserver.model.quest.State;
import l2r.util.Rnd;

public class _633_InTheForgottenVillage extends Quest implements ScriptFile
{
	// NPC
	private static int MINA = 31388;
	// ITEMS
	private static int RIB_BONE = 7544;
	private static int Z_LIVER = 7545;

	// Mobid : DROP CHANCES
	private static HashMap<Integer, Integer> DAMOBS = new HashMap<Integer, Integer>();
	private static HashMap<Integer, Integer> UNDEADS = new HashMap<Integer, Integer>();

	public void onLoad()
	{
		System.out.println("Loaded Quest: 633: In The Forgotten Village");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public _633_InTheForgottenVillage()
	{
		super(633, "_633_InTheForgottenVillage", "In The Forgotten Village", true);

		DAMOBS.put(21557, 328); // Bone Snatcher
		DAMOBS.put(21558, 328); // Bone Snatcher
		DAMOBS.put(21559, 337); // Bone Maker
		DAMOBS.put(21560, 337); // Bone Shaper
		DAMOBS.put(21563, 342); // Bone Collector
		DAMOBS.put(21564, 348); // Skull Collector
		DAMOBS.put(21565, 351); // Bone Animator
		DAMOBS.put(21566, 359); // Skull Animator
		DAMOBS.put(21567, 359); // Bone Slayer
		DAMOBS.put(21572, 365); // Bone Sweeper
		DAMOBS.put(21574, 383); // Bone Grinder
		DAMOBS.put(21575, 383); // Bone Grinder
		DAMOBS.put(21580, 385); // Bone Caster
		DAMOBS.put(21581, 395); // Bone Puppeteer
		DAMOBS.put(21583, 397); // Bone Scavenger
		DAMOBS.put(21584, 401); // Bone Scavenger

		UNDEADS.put(21553, 347); // Trampled Man
		UNDEADS.put(21554, 347); // Trampled Man
		UNDEADS.put(21561, 450); // Sacrificed Man
		UNDEADS.put(21578, 501); // Behemoth Zombie
		UNDEADS.put(21596, 359); // Requiem Lord
		UNDEADS.put(21597, 370); // Requiem Behemoth
		UNDEADS.put(21598, 441); // Requiem Behemoth
		UNDEADS.put(21599, 395); // Requiem Priest
		UNDEADS.put(21600, 408); // Requiem Behemoth
		UNDEADS.put(21601, 411); // Requiem Behemoth

		addStartNpc(MINA);
		addQuestItem(RIB_BONE);

		for(int i : UNDEADS.keySet())
			addKillId(i);

		for(int i : DAMOBS.keySet())
			addKillId(i);
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;
		if(event.equalsIgnoreCase("accept"))
		{
			st.set("cond", "1");
			st.setState(STARTED);
			st.playSound(SOUND_ACCEPT);
			htmltext = "31388-04.htm";
		}
		if(event.equalsIgnoreCase("quit"))
		{
			st.takeItems(RIB_BONE, -1);
			st.playSound(SOUND_FINISH);
			htmltext = "31388-10.htm";
			st.exitCurrentQuest(true);
		}
		else if(event.equalsIgnoreCase("stay"))
			htmltext = "31388-07.htm";
		else if(event.equalsIgnoreCase("reward"))
			if(st.getInt("cond") == 2)
				if(st.getQuestItemsCount(RIB_BONE) >= 200)
				{
					st.takeItems(RIB_BONE, -1);
					st.giveItems(57, 25000);
					st.addExpAndSp(305235, 0);
					st.playSound(SOUND_FINISH);
					st.set("cond", "1");
					htmltext = "31388-08.htm";
				}
				else
					htmltext = "31388-09.htm";
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = "noquest";
		int npcId = npc.getNpcId();
		int cond = st.getInt("cond");
		State id = st.getState();
		if(npcId == MINA)
			if(id == CREATED)
			{
				if(st.getPlayer().getLevel() > 64)
					htmltext = "31388-01.htm";
				else
				{
					htmltext = "31388-03.htm";
					st.exitCurrentQuest(true);
				}
			}
			else if(cond == 1)
				htmltext = "31388-06.htm";
			else if(cond == 2)
				htmltext = "31388-05.htm";
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		if(UNDEADS.get(npcId) != null)
		{
			if(Rnd.chance(UNDEADS.get(npcId)))
			{
				st.giveItems(Z_LIVER, (int) st.getRateQuestsReward());
				st.playSound(SOUND_ITEMGET);
			}
		}
		else if(DAMOBS.get(npcId) != null)
		{
			long count = st.getQuestItemsCount(RIB_BONE);
			if(count < 200 && Rnd.chance(DAMOBS.get(npcId)))
			{
				long numItems = (int) st.getRateQuestsReward();
				if(count + numItems > 200)
					numItems = 200 - count;
				st.giveItems(RIB_BONE, 1);
				if(count + numItems >= 200)
				{
					st.set("cond", "2");
					st.playSound(SOUND_MIDDLE);
				}
				else
					st.playSound(SOUND_ITEMGET);
			}
		}
		return null;
	}
}