package quests._643_RiseAndFallOfTheElrokiTribe;

import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.util.Rnd;

public class _643_RiseAndFallOfTheElrokiTribe extends Quest implements ScriptFile
{
	private static int DROP_CHANCE = 75;
	private static int BONES_OF_A_PLAINS_DINOSAUR = 8776;

	private static int[] PLAIN_DINOSAURS = new int[] { 22208, 22209, 22210, 22211, 22212, 22213, 22221, 22222, 22226, 22227 };
	private static int[] REWARDS = new int[] { 8712, 8713, 8714, 8715, 8716, 8717, 8718, 8719, 8720, 8721, 8722 };

	public void onLoad()
	{
		System.out.println("Loaded Quest: 643: Rise And Fall Of The Elroki Tribe");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public _643_RiseAndFallOfTheElrokiTribe()
	{
		super(643, "_643_RiseAndFallOfTheElrokiTribe", "Rise And Fall Of The Elroki Tribe", true);

		addStartNpc(32106);
		addTalkId(32117);

		for(int npc : PLAIN_DINOSAURS)
			addKillId(npc);

		addQuestItem(BONES_OF_A_PLAINS_DINOSAUR);
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;
		long count = st.getQuestItemsCount(BONES_OF_A_PLAINS_DINOSAUR);
		if(event.equalsIgnoreCase("32106-03.htm"))
		{
			st.set("cond", "1");
			st.setState(STARTED);
			st.playSound(SOUND_ACCEPT);
		}
		else if(event.equalsIgnoreCase("32117-03.htm"))
		{
			if(count >= 300)
			{
				st.takeItems(BONES_OF_A_PLAINS_DINOSAUR, -1);
				st.giveItems(REWARDS[Rnd.get(REWARDS.length)], 5);
			}
			else
				htmltext = "32117-04.htm";
		}
		else if(event.equalsIgnoreCase("None"))
			htmltext = null;
		else if(event.equalsIgnoreCase("Quit"))
		{
			htmltext = null;
			st.playSound(SOUND_FINISH);
			st.exitCurrentQuest(true);
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = "noquest";
		int npcId = npc.getNpcId();
		int cond = st.getInt("cond");
		long count = st.getQuestItemsCount(BONES_OF_A_PLAINS_DINOSAUR);
		if(cond == 0)
		{
			if(st.getPlayer().getLevel() >= 75)
				htmltext = "32106-01.htm";
			else
			{
				htmltext = "32106-00.htm";
				st.exitCurrentQuest(true);
			}
		}
		else if(st.getState() == STARTED)
			if(npcId == 32106)
			{
				if(count == 0)
					htmltext = "32106-05.htm";
				else
				{
					htmltext = "32106-06.htm";
					st.takeItems(BONES_OF_A_PLAINS_DINOSAUR, -1);
					st.giveItems(57, count * 1374);
				}
			}
			else if(npcId == 32117)
				htmltext = "32117-01.htm";
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int cond = st.getInt("cond");
		long count = st.getQuestItemsCount(BONES_OF_A_PLAINS_DINOSAUR);
		if(cond == 1 && count < 300 && Rnd.chance(DROP_CHANCE))
		{
			long numItems = (int) st.getRateQuestsReward();
			if(count + numItems > 300)
				numItems = 300 - count;
			if(count + numItems >= 300)
				st.playSound(SOUND_MIDDLE);
			else
				st.playSound(SOUND_ITEMGET);
			st.giveItems(BONES_OF_A_PLAINS_DINOSAUR, numItems);
		}
		return null;
	}
}