package quests._648_AnIceMerchantsDream;

import javolution.util.FastList;
import l2r.Config;
import l2r.extensions.scripts.ScriptFile;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.gameserver.model.quest.State;
import l2r.util.Rnd;

public class _648_AnIceMerchantsDream extends Quest implements ScriptFile
{
	// NPCs
	private static int Rafforty = 32020;
	private static int Ice_Shelf = 32023;
	// Items
	private static int ADENA = 57;
	private static int Silver_Hemocyte = 8057;
	private static int Silver_Ice_Crystal = 8077;
	private static int Black_Ice_Crystal = 8078;
	// Chances
	private static int Silver_Hemocyte_Chance = 10;
	private static int Silver2Black_Chance = 30;

	private static FastList<Integer> silver2black = new FastList<Integer>();

	public _648_AnIceMerchantsDream()
	{
		super(648, "_648_AnIceMerchantsDream", "An Ice Merchant's Dream", false);
		addStartNpc(Rafforty);
		addStartNpc(Ice_Shelf);
		for(int i = 22080; i <= 22098; i++)
			if(i != 22095)
				addKillId(i);
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		State _state = st.getState();
		if(event.equalsIgnoreCase("32020-02.htm") && _state == CREATED)
		{
			st.setState(STARTED);
			st.set("cond", "1");
			st.playSound(SOUND_ACCEPT);
		}
		else if(event.equalsIgnoreCase("32020-09.htm") && _state == STARTED)
		{
			st.playSound(SOUND_FINISH);
			st.exitCurrentQuest(true);
		}
		if(_state != STARTED)
			return event;

		long Silver_Ice_Crystal_Count = st.getQuestItemsCount(Silver_Ice_Crystal);
		long Black_Ice_Crystal_Count = st.getQuestItemsCount(Black_Ice_Crystal);

		if(event.equalsIgnoreCase("32020-07.htm"))
		{
			long reward = Silver_Ice_Crystal_Count * 300 + Black_Ice_Crystal_Count * 1200;
			if(reward > 0)
			{
				st.takeItems(Silver_Ice_Crystal, -1);
				st.takeItems(Black_Ice_Crystal, -1);
				st.giveItems(ADENA, reward);
			}
			else
				return "32020-07a.htm";
		}
		else if(event.equalsIgnoreCase("32023-04.htm"))
		{
			int char_obj_id = st.getPlayer().getObjectId();
			synchronized (silver2black)
			{
				if(silver2black.contains(char_obj_id))
					return event;
				else if(Silver_Ice_Crystal_Count > 0)
					silver2black.add(char_obj_id);
				else
					return "cheat.htm";
			}

			st.takeItems(Silver_Ice_Crystal, 1);
			st.playSound(SOUND_BROKEN_KEY);
		}
		else if(event.equalsIgnoreCase("32023-05.htm"))
		{
			Integer char_obj_id = st.getPlayer().getObjectId();
			synchronized (silver2black)
			{
				if(silver2black.contains(char_obj_id))
					while(silver2black.contains(char_obj_id))
						silver2black.remove(char_obj_id);
				else
					return "cheat.htm";
			}

			if(Rnd.chance(Silver2Black_Chance))
			{
				st.giveItems(Black_Ice_Crystal, 1);
				st.playSound(SOUND_ENCHANT_SUCESS);
			}
			else
			{
				st.playSound(SOUND_ENCHANT_FAILED);
				return "32023-06.htm";
			}
		}

		return event;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		State _state = st.getState();
		int npcId = npc.getNpcId();
		int cond = st.getInt("cond");

		if(_state == CREATED)
		{
			if(npcId == Rafforty)
			{
				if(st.getPlayer().getLevel() >= 53)
				{
					st.set("cond", "0");
					return "32020-01.htm";
				}
				st.exitCurrentQuest(true);
				return "32020-00.htm";
			}
			if(npcId == Ice_Shelf)
				return "32023-00.htm";
		}

		if(_state != STARTED)
			return "noquest";

		long Silver_Ice_Crystal_Count = st.getQuestItemsCount(Silver_Ice_Crystal);
		if(npcId == Ice_Shelf)
			return Silver_Ice_Crystal_Count > 0 ? "32023-02.htm" : "32023-01.htm";

		long Black_Ice_Crystal_Count = st.getQuestItemsCount(Black_Ice_Crystal);
		if(npcId == Rafforty)
		{
			if(cond == 1)
			{
				if(Silver_Ice_Crystal_Count > 0 || Black_Ice_Crystal_Count > 0)
				{
					QuestState st_115 = st.getPlayer().getQuestState("_115_TheOtherSideOfTruth");
					if(st_115 != null && st_115.getState() == COMPLETED)
					{
						st.set("cond", "2");
						st.playSound(SOUND_MIDDLE);
						return "32020-10.htm";
					}
					return "32020-05.htm";
				}
				return "32020-04.htm";
			}
			if(cond == 2)
				return Silver_Ice_Crystal_Count > 0 || Black_Ice_Crystal_Count > 0 ? "32020-10.htm" : "32020-04a.htm";
		}

		return "noquest";
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState qs)
	{
		L2Player player = qs.getRandomPartyMember(STARTED, Config.ALT_PARTY_DISTRIBUTION_RANGE);
		if(player != null)
		{
			QuestState st = player.getQuestState(qs.getQuest().getName());
			if(st != null && Rnd.chance(npc.getNpcId() - 22050))
			{
				st.giveItems(Silver_Ice_Crystal, 1);
				st.playSound(SOUND_ITEMGET);
			}
		}

		if(qs.getState() == STARTED && qs.getInt("cond") == 2 && Rnd.chance(Silver_Hemocyte_Chance))
		{
			qs.giveItems(Silver_Hemocyte, 1);
			qs.playSound(SOUND_ITEMGET);
		}

		return null;
	}

	public void onLoad()
	{
		System.out.println("Loaded Quest: 648: An Ice Merchant's Dream");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}