package l2r;

import static l2r.gameserver.model.L2Zone.ZoneType.peace_zone;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilderFactory;

import javolution.util.FastList;
import l2r.gameserver.loginservercon.AdvIP;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.base.Experience;
import l2r.gameserver.model.base.PlayerAccess;
import l2r.util.NetList;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * This class contain global server configuration.
 * It has static final fields initialized from configuration files.
 * It's initialized at the very begin of startup, and later JIT will optimize
 * away debug/unused code.
 *
 * @author mkizub
 */
public class Config
{
	protected static Logger _log = Logger.getLogger(Config.class.getName());

	/** Debug/release mode */
	public static boolean DEBUG;
	public static boolean LOGIN_DEBUG;

	public static boolean COMBO_MODE;

	/** GS Packets Logger */
	public static boolean LOG_CLIENT_PACKETS;
	public static boolean LOG_SERVER_PACKETS;
	public static int PACKETLOGGER_FLUSH_SIZE;
	public static NetList PACKETLOGGER_IPS;
	public static FastList<String> PACKETLOGGER_ACCOUNTS;
	public static FastList<String> PACKETLOGGER_CHARACTERS;

	/** Game/Login Server ports */
	public static int PORT_GAME;
	public static int PORT_LOGIN;
	public static int LOGIN_TRY_BEFORE_BAN;
	public static String GAMESERVER_HOSTNAME;
	public static boolean ADVIPSYSTEM;
	public static ArrayList<AdvIP> GAMEIPS = new ArrayList<AdvIP>();
	public static int IP_UPDATE_TIME;
	public static boolean LOGIN_PING;
	public static int LOGIN_PING_TIME;

	/** AntiFlood for Game/Login */
	public static boolean ANTIFLOOD_ENABLE;
	public static int MAX_UNHANDLED_SOCKETS_PER_IP;
	public static int UNHANDLED_SOCKET_MIN_TTL;

	public static int WebServerPort;
	public static String WebServerRoot;

	public static String DATABASE_DRIVER;
	public static String DATABASE_URL;
	public static String DATABASE_LOGIN;
	public static String DATABASE_PASSWORD;
	public static int DATABASE_MAX_CONNECTIONS;

	// Database additional options
	/** lazy item update */
	public static boolean LAZY_ITEM_UPDATE;
	/** lazy item update time */
	public static int LAZY_ITEM_UPDATE_TIME;

	public static int MAXIMUM_ONLINE_USERS;
	public static boolean AUTO_CREATE_ACCOUNTS;

	public static boolean SERVER_LIST_BRACKET;
	public static boolean SERVER_LIST_CLOCK;
	public static boolean SERVER_LIST_TESTSERVER;
	public static boolean SERVER_GMONLY;

	public static boolean CHECK_LANG_FILES_MODIFY;
	public static boolean USE_FILE_CACHE;

	public static int LINEAR_TERRITORY_CELL_SIZE;
	public static boolean DONTLOADSPAWN;
	public static boolean DONTLOADQUEST;

	/** ChatBan */
	public static boolean MAT_BANCHAT;
	public static String BAN_CHANNEL;
	public static int[] BAN_CHANNEL_LIST = new int[18];
	public static int MAT_BAN_COUNT_CHANNELS;
	public static boolean MAT_REPLACE;
	public static String MAT_REPLACE_STRING;
	public static int UNCHATBANTIME;
	public static ArrayList<String> MAT_LIST = new ArrayList<String>();
	public static boolean MAT_ANNOUNCE;
	public static boolean MAT_ANNOUNCE_NICK;

	public static boolean SAVING_SPS;
	public static boolean MANAHEAL_SPS_BONUS;

	public static int ALT_ADD_RECIPES;
	public static boolean ALT_100_RECIPES;

	public static int ALT_MAX_ALLY_SIZE;

	public static int ALT_PARTY_DISTRIBUTION_RANGE;

	public static int DaysBeforeCreateAClan;
	public static int HoursBeforeJoinAClan;
	public static int AltClanMembersForWar;
	public static int AltMinClanLvlForWar;
	public static int AltClanWarMax;
	public static int DaysBeforeCreateNewAllyWhenDissolved;
	public static int MinLevelToCreatePledge;
	public static int MemberForLevel6;
	public static int MemberForLevel7;
	public static int MemberForLevel8;
	public static int MemberForLevel9;
	public static int MemberForLevel10;

	/** Count kills? */
	public static boolean KILL_COUNTER;
	public static boolean KILL_COUNTER_PRELOAD;

	/** Count dropped? */
	public static boolean DROP_COUNTER;

	/** Count crafted? */
	public static boolean CRAFT_COUNTER;

	/** Thread pools size */
	public static int THREAD_P_EFFECTS;
	public static int THREAD_P_GENERAL;
	public static int GENERAL_PACKET_THREAD_CORE_SIZE;
	public static int URGENT_PACKET_THREAD_CORE_SIZE;
	public static int AI_MAX_THREAD;
    public static boolean INTEREST_ALT;
    public static int INTEREST_MAX_THREAD;

	/** Auto-loot? */
	public static boolean AUTO_LOOT;

	/** Auto-loot for/from players with karma also? */
	public static boolean AUTO_LOOT_PK;

	/** Auto-loot herbs? */
	public static boolean AUTO_LOOT_HERBS;

	/** Разделение эффектов хербов с петом или саммоном пополам */
	public static boolean HERBS_DIVIDE;

	/** Account name template */
	public static String ANAME_TEMPLATE;

	/** Account password template */
	public static String APASSWD_TEMPLATE;

	/** Character name template */
	public static String CNAME_TEMPLATE;

	/** Clan name template */
	public static String CLAN_NAME_TEMPLATE;

	/** Clan title template */
	public static String CLAN_TITLE_TEMPLATE;

	/** Ally name template */
	public static String ALLY_NAME_TEMPLATE;

	/** Global chat state */
	public static int GLOBAL_CHAT;
	public static int GLOBAL_TRADE_CHAT;
	public static int SHOUT_CHAT_MODE;
	public static int TRADE_CHAT_MODE;

	/** For test servers - evrybody has admin rights */
	public static boolean EVERYBODY_HAS_ADMIN_RIGHTS;
	public static boolean ALLOW_SPECIAL_COMMANDS;

	/** For small servers */
	public static boolean ALT_GAME_MATHERIALSDROP;

	/** Все мобы не являющиеся рейдами спавнятся в двойном количестве */
	public static boolean ALT_DOUBLE_SPAWN;

	public static boolean ALT_ALLOW_AUGMENT_ALL;
	public static boolean ALT_ALLOW_DROP_AUGMENTED;

	/** Give exp and sp for craft */
	public static boolean ALT_GAME_EXP_FOR_CRAFT;

	public static boolean ALT_GAME_UNREGISTER_RECIPE;

	/** Delay for announce SS period (in minutes) */
	public static int SS_ANNOUNCE_PERIOD;

	/** Show mob stats/droplist to players? */
	public static boolean ALT_GAME_SHOW_DROPLIST;
	public static boolean ALT_GAME_GEN_DROPLIST_ON_DEMAND;
	public static boolean ALT_FULL_NPC_STATS_PAGE;
	public static boolean ALLOW_NPC_SHIFTCLICK;

	/* Show html window at login */
	public static boolean SHOW_HTML_WELCOME;

	public static boolean ALT_SKILLS_CHANCES;
	public static double SKILLS_CHANCE_CAP;
	public static boolean SKILLS_SHOW_CHANCE;
	public static boolean ALT_CUBICS_SAVE;
	public static boolean ALT_SHOW_REUSE_MSG;

	/** Система наподобии той, что в WoW */
	public static boolean ALT_HONOR_SYSTEM;

	/** Таймаут на использование social action */
	public static boolean ALT_SOCIAL_ACTION_REUSE;

	/** Отключение книг для изучения скилов */
	public static boolean ALT_DISABLE_SPELLBOOKS;

	/** Отключение яйца для изучения клан скилов */
	public static boolean ALT_DISABLE_EGGS;

	/** Alternative gameing - loss of XP on death */
	public static boolean ALT_GAME_DELEVEL;

	public static boolean ALT_GAME_SUBCLASS_WITHOUT_QUESTS;
	public static int ALT_GAME_LEVEL_TO_GET_SUBCLASS;
	public static int ALT_MAX_LEVEL;
	public static int ALT_MAX_SUB_LEVEL;
	public static int ALT_GAME_SUB_ADD;

	public static boolean ALT_SIMPLE_SIGNS;
	public static int ALT_MAMMOTH_EXCHANGE;
	public static int ALT_MAMMOTH_UPGRADE;
	public static boolean ALT_ALLOW_TATTOO;

	public static int ALT_BUFF_LIMIT;

	public static int MULTISELL_SIZE;

	public static boolean SERVICES_CHANGE_NICK_ENABLED;
	public static int SERVICES_CHANGE_NICK_PRICE;
	public static int SERVICES_CHANGE_NICK_ITEM;

	public static boolean SERVICES_CHANGE_NICK_COLOR_ENABLED;
	public static int SERVICES_CHANGE_NICK_COLOR_PRICE;
	public static int SERVICES_CHANGE_NICK_COLOR_ITEM;
	public static String[] SERVICES_CHANGE_NICK_COLOR_LIST;

	public static boolean SERVICES_BASH_ENABLED;
	public static boolean SERVICES_BASH_SKIP_DOWNLOAD;
	public static int SERVICES_BASH_RELOAD_TIME;

	public static boolean SERVICES_RATE_BONUS_ENABLED;
	public static int SERVICES_RATE_BONUS_PRICE;
	public static int SERVICES_RATE_BONUS_ITEM;
	public static float SERVICES_RATE_BONUS_VALUE;
	public static int SERVICES_RATE_BONUS_DAYS;
	public static float SERVICES_RATE_BONUS_LUCK_EFFECT;

	public static boolean SERVICES_NOBLESS_SELL_ENABLED;
	public static int SERVICES_NOBLESS_SELL_PRICE;
	public static int SERVICES_NOBLESS_SELL_ITEM;

	public static boolean SERVICES_HOW_TO_GET_COL;

	public static boolean SERVICES_OFFLINE_TRADE_ALLOW;
	public static int SERVICES_OFFLINE_TRADE_NAME_COLOR;
	public static boolean SERVICES_OFFLINE_TRADE_KICK_NOT_TRADING;
	public static int SERVICES_OFFLINE_TRADE_PRICE;
	public static int SERVICES_OFFLINE_TRADE_PRICE_ITEM;
	public static int SERVICES_OFFLINE_TRADE_DAYS_TO_KICK;
	public static boolean SERVICES_OFFLINE_TRADE_RESTORE_AFTER_RESTART;

	public static boolean SERVICES_GIRAN_HARBOR_ENABLED;

	public static boolean SERVICES_CHANGE_PASSWORD;
	public static boolean SERVICES_LOCK_ACCOUNT_IP;

	public static boolean SERVICES_DONATE_SHOP_ENABLED;

	public static boolean ALT_ALLOW_OTHERS_WITHDRAW_FROM_CLAN_WAREHOUSE;
	public static boolean ALT_GAME_REQUIRE_CLAN_CASTLE;
	public static boolean ALT_GAME_REQUIRE_CASTLE_DAWN;

	/** Olympiad Compitition Starting time */
	public static int ALT_OLY_START_TIME;
	/** Olympiad Compition Min */
	public static int ALT_OLY_MIN;
	/** Olympaid Comptetition Period */
	public static long ALT_OLY_CPERIOD;
	/** Olympiad Battle Period */
	public static long ALT_OLY_BATTLE;
	/** Olympiad Battle Wait */
	public static long ALT_OLY_BWAIT;
	/** Olympiad Inital Wait */
	public static long ALT_OLY_IWAIT;
	/** Olympaid Weekly Period */
	public static long ALT_OLY_WPERIOD;
	/** Olympaid Validation Period */
	public static long ALT_OLY_VPERIOD;

	public static boolean ENABLE_OLYMPIAD;

	public static int ALT_TRUE_CHESTS;

	public static int NONOWNER_ITEM_PICKUP_DELAY;

	/** Logging Chat Window */
	public static boolean LOG_CHAT;
	public static boolean LOG_KILLS;
	public static boolean LOG_TELNET;
	public static boolean SQL_LOG;

	public static HashMap<Integer, PlayerAccess> gmlist = new HashMap<Integer, PlayerAccess>();

	/** Rate control */
	public static float RATE_XP;
	public static float RATE_SP;
	public static float RATE_QUESTS_REWARD;
	public static float RATE_QUESTS_DROP;
	public static float RATE_QUESTS_DROP_PROF;
	public static boolean RATE_QUESTS_OCCUPATION_CHANGE;
	public static float RATE_CLAN_REP_SCORE;
	public static int RATE_CLAN_REP_SCORE_MAX_AFFECTED;
	public static float RATE_DROP_ADENA;
	public static float RATE_DROP_ADENA_PARTY;
	public static float RATE_DROP_ITEMS_PARTY;
	public static float RATE_XP_PARTY;
	public static float RATE_SP_PARTY;
	public static float RATE_DROP_ITEMS;
	public static float RATE_DROP_RAIDBOSS;
	public static float RATE_DROP_SPOIL;
	public static int RATE_MANOR;
	public static float RATE_FISH_DROP_COUNT;
	public static float RATE_SIEGE_GUARDS_PRICE;

	/** Player Drop Rate control */
	public static boolean KARMA_DROP_GM;
	public static boolean KARMA_NEEDED_TO_DROP;

	public static int KARMA_DROP_ITEM_LIMIT;

	public static int KARMA_RANDOM_DROP_LOCATION_LIMIT;

	public static int KARMA_DROPCHANCE_MINIMUM;
	public static int KARMA_DROPCHANCE_MULTIPLIER;
	public static int KARMA_DROPCHANCE_EQUIPMENT;
	public static int KARMA_DROPCHANCE_EQUIPPED_WEAPON;

	public static int AUTODESTROY_ITEM_AFTER;

	public static int DELETE_DAYS;

	public static int PURGE_BYPASS_TASK_FREQUENCY;

	/** Datapack root directory */
	public static File DATAPACK_ROOT;

	public static String AttributeBonusFile;

	public static boolean WEAR_TEST_ENABLED;
	public static boolean USE_DATABASE_LAYER;

	public static float BUFFTIME_MODIFIER;
	public static float SONGDANCETIME_MODIFIER;
	public static float MAXLOAD_MODIFIER;

	/** Allow L2Walker client */
	public static enum L2WalkerAllowed
	{
		True,
		False,
		Peace
	}

	public static L2WalkerAllowed ALLOW_L2WALKER_CLIENT;
	public static int MAX_L2WALKER_LEVEL;
	public static boolean ALLOW_CRAFT_BOT;
	public static boolean ALLOW_BOT_CAST;
	public static boolean KILL_BOTS_WITH_SKILLCOOLTIME;
	public static int L2WALKER_PUNISHMENT;
	public static int BUGUSER_PUNISH;

	public static boolean ALLOW_DISCARDITEM;
	public static boolean ALLOW_FREIGHT;
	public static boolean ALLOW_WAREHOUSE;
	public static boolean ALLOW_WATER;
	public static boolean ALLOW_BOAT;
	public static boolean ALLOW_CURSED_WEAPONS;
	public static int CURSED_WEAPONS_MIN_PLAYERS_DROP;
	public static boolean DROP_CURSED_WEAPONS_ON_KICK;
	public static boolean ALLOW_NOBLE_TP_TO_ALL;

	/** Pets */
	public static int WYVERN_SPEED;
	public static int STRIDER_SPEED;
	public static int SWIMING_SPEED;

	/** protocol revision */
	public static int MIN_PROTOCOL_REVISION;
	public static int MAX_PROTOCOL_REVISION;

	/** random animation interval */
	public static int MIN_NPC_ANIMATION;
	public static int MAX_NPC_ANIMATION;

	public static String DEFAULT_LANG;

	/** Время регулярного рестарта (через определенное время) */
	public static int RESTART_TIME;
	/** Время запланированного на определенное время суток рестарта */
	public static int RESTART_AT_TIME;
	public static int LRESTART_TIME;

	/** Configuration files */
	public static final String OTHER_CONFIG_FILE = "./config/other.properties";
	public static final String CLANHALL_CONFIG_FILE = "./config/clanhall.properties";
	public static final String SPOIL_CONFIG_FILE = "./config/spoil.properties";
	public static final String ALT_SETTINGS_FILE = "./config/altsettings.properties";
	public static final String PVP_CONFIG_FILE = "./config/pvp.properties";
	public static final String GM_PERSONAL_ACCESS_FILE = "./config/GMAccess.xml";
	public static final String TELNET_FILE = "./config/telnet.properties";
	public static final String LOGIN_TELNET_FILE = "./config/login_telnet.properties";
	public static final String CONFIGURATION_FILE = "./config/server.properties";
	public static final String LOGIN_CONFIGURATION_FILE = "./config/loginserver.properties";
	public static final String VERSION_FILE = "l2r-version.properties";
	public static final String SIEGE_CASTLE_CONFIGURATION_FILE = "./config/siege_castle.properties";
	public static final String SIEGE_FORTRESS_CONFIGURATION_FILE = "./config/siege_fortress.properties";
	public static final String BANNED_IP_XML = "./config/banned.xml";
	public static final String HEXID_FILE = "./config/hexid.txt";
	public static final String MAT_CONFIG_FILE = "./config/mats.cfg";
	public static final String ADV_IP_FILE = "./config/advipsystem.properties";
	public static final String AI_CONFIG_FILE = "./config/ai.properties";
	public static final String EVENTS = "./config/events.properties";
	public static final String FAKE_PLAYERS_LIST = "./config/fake_players.list";
	public static final String SERVICES_FILE = "./config/services.properties";
	public static final String PROTECT_FILE = "./config/protection.properties";
    public static final String PVPCB_FILE = "./config/pvpcommunityboard.properties";

	/**
     * Settings for PVP CommunityBoard
     */
    public static boolean ALLOW_PVPCB_ABNORMAL;
    public static boolean ALLOW_PVPCB_BUFFER;
    public static boolean ALLOW_PVPCB_SHOP;
    public static boolean ALLOW_PVPCB_ECHANT;
    public static boolean ALLOW_PVPCB_CLASSMASTER;
    public static boolean ALLOW_PVPCB_TELEPORT;
    public static long PVPCB_BUFFER_ALT_TIME;
    public static int PVPCB_BUFFER_PRICE_ITEM;
    public static int PVPCB_BUFFER_PRICE_ONE;
    public static int PVPCB_BUFFER_PRICE_MOD_GRP;
    public static int PVPCB_BUFFER_MIN_LVL;
    public static int PVPCB_BUFFER_MAX_LVL;
    public static int[] ALLOWED_BUFFS;
    public static boolean PVPCB_BUFFER_ALLOW_SIEGE;
    public static boolean PVPCB_BUFFER_ALLOW_PK;
    public static boolean PVPCB_BUFFER_ALLOW_SAVE_RESTOR;
    public static int ALT_CB_ENCH_ITEM;
    public static int PVP_BBS_TELEPORT_PRICE;
	public static boolean PVP_BBS_TELEPORT_ADDITIONAL_RULES;
    
    /** DRiN's Protection config */
	public static boolean PROTECT_ENABLE;
	public static NetList PROTECT_UNPROTECTED_IPS;
	public static boolean PROTECT_GS_STORE_HWID;
	public static boolean PROTECT_GS_LOG_HWID;
	public static boolean PROTECT_ENABLE_HWID_BANS;
    public static int PROTECT_COMPRESSION_WRITEDELAY;

	public static String LOGIN_HOST;
	public static int GAME_SERVER_LOGIN_PORT;
	public static String GAME_SERVER_LOGIN_HOST;
	public static String INTERNAL_HOSTNAME;
	public static String EXTERNAL_HOSTNAME;

	public static boolean SERVER_SIDE_NPC_NAME;
	public static boolean SERVER_SIDE_NPC_TITLE;
	public static boolean SERVER_SIDE_NPC_TITLE_WITH_LVL;
	public static String ALLOW_CLASS_MASTERS;
	public static String CLASS_MASTERS_PRICE;
	public static int CLASS_MASTERS_PRICE_ITEM;
	public static ArrayList<Integer> ALLOW_CLASS_MASTERS_LIST = new ArrayList<Integer>();
	public static int[] CLASS_MASTERS_PRICE_LIST = new int[4];

	public static String SERVER_VERSION;
	public static String SERVER_BUILD_DATE;
	public final static String SERVER_VERSION_UNSUPPORTED = "Unknown Version";

	/** Inventory slots limits */
	public static int INVENTORY_MAXIMUM_NO_DWARF;
	public static int INVENTORY_MAXIMUM_DWARF;
	public static int INVENTORY_MAXIMUM_GM;

	/** Warehouse slots limits */
	public static int WAREHOUSE_SLOTS_NO_DWARF;
	public static int WAREHOUSE_SLOTS_DWARF;
	public static int WAREHOUSE_SLOTS_CLAN;

	/** Spoil Rates */
	public static float BASE_SPOIL_RATE;
	public static float MINIMUM_SPOIL_RATE;
	public static boolean ALT_SPOIL_FORMULA;

	/** Manor Config */
	public static double MANOR_SOWING_BASIC_SUCCESS;
	public static double MANOR_SOWING_ALT_BASIC_SUCCESS;
	public static double MANOR_HARVESTING_BASIC_SUCCESS;
	public static double MANOR_DIFF_PLAYER_TARGET;
	public static double MANOR_DIFF_PLAYER_TARGET_PENALTY;
	public static double MANOR_DIFF_SEED_TARGET;
	public static double MANOR_DIFF_SEED_TARGET_PENALTY;

	/** Karma System Variables */
	public static int KARMA_MIN_KARMA;
	public static int KARMA_SP_DIVIDER;
	public static int KARMA_LOST_BASE;

	public static int MIN_PK_TO_ITEMS_DROP;

	public static String KARMA_NONDROPPABLE_ITEMS;
	public static ArrayList<Integer> KARMA_LIST_NONDROPPABLE_ITEMS = new ArrayList<Integer>();

	public static int PVP_TIME;

	/** Karma Punishment */
	public static boolean ALT_GAME_KARMAPLAYERCANBEKILLEDINPEACEZONE;
	public static boolean ALT_GAME_KARMA_PLAYER_CAN_SHOP;

	/** Day/Night Status **/
	public static boolean DAY_STATUS_FORCE_CLIENT_UPDATE;
	public static int DAY_STATUS_SUN_RISE_AT;
	public static int DAY_STATUS_SUN_SET_AT;

	public static boolean AUTODELETE_INVALID_QUEST_DATA;
	public static boolean HARD_DB_CLEANUP_ON_START;

	/** Chance that an item will succesfully be enchanted */
	public static int ENCHANT_CHANCE_WEAPON;
	public static int ENCHANT_CHANCE_ARMOR;
	public static int ENCHANT_CHANCE_CRYSTAL_WEAPON;
	public static int ENCHANT_CHANCE_CRYSTAL_ARMOR;
	public static byte ENCHANT_MAX_WEAPON;
	public static byte ENCHANT_MAX_ACCESSORY;
	public static byte ENCHANT_MAX_ARMOR;
	public static double ENCHANT_MODIFER;
	public static boolean LUCK_BONUS;

	/** Config Alt Enchant Formula */
	public static boolean ALT_ENCHANT_FORMULA;
	public static int ALT_ENCHANT_CHANCE_W;
	public static int ALT_ENCHANT_CHANCE_MAGE_W;
	public static int ALT_ENCHANT_CHANCE_ARMOR;
	public static int PENALTY_TO_TWOHANDED_BLUNTS;

	public static boolean REGEN_SIT_WAIT;

	public static float RATE_RAID_REGEN;
	public static int RAID_MAX_LEVEL_DIFF;

	public static float ALT_PK_DEATH_RATE;
	public static int STARTING_ADENA;

	/** Deep Blue Mobs' Drop Rules Enabled */
	public static boolean DEEPBLUE_DROP_RULES;
	public static boolean UNSTUCK_SKILL;

	/** telnet enabled */
	public static boolean IS_TELNET_ENABLED;
	public static boolean IS_LOGIN_TELNET_ENABLED;

	/** Percent CP is restore on respawn */
	public static double RESPAWN_RESTORE_CP;
	/** Percent HP is restore on respawn */
	public static double RESPAWN_RESTORE_HP;
	/** Percent MP is restore on respawn */
	public static double RESPAWN_RESTORE_MP;

	/** Maximum number of available slots for pvt stores (sell/buy) - Dwarves */
	public static int MAX_PVTSTORE_SLOTS_DWARF;
	/** Maximum number of available slots for pvt stores (sell/buy) - Others */
	public static int MAX_PVTSTORE_SLOTS_OTHER;
	public static int MAX_PVTCRAFT_SLOTS;

	public static boolean ALLOW_CH_DOOR_OPEN_ON_CLICK;

	/** Clan Hall function related configs*/
	public static long CH_TELE_FEE_RATIO;
	public static int CH_TELE1_FEE;
	public static int CH_TELE2_FEE;
	public static int CH_TELE3_FEE;
	public static long CH_ITEM_FEE_RATIO;
	public static int CH_ITEM1_FEE;
	public static int CH_ITEM2_FEE;
	public static int CH_ITEM3_FEE;
	public static long CH_MPREG_FEE_RATIO;
	public static int CH_MPREG1_FEE;
	public static int CH_MPREG2_FEE;
	public static int CH_MPREG3_FEE;
	public static int CH_MPREG4_FEE;
	public static int CH_MPREG5_FEE;
	public static int CH_MPREG6_FEE;
	public static long CH_HPREG_FEE_RATIO;
	public static int CH_HPREG1_FEE;
	public static int CH_HPREG2_FEE;
	public static int CH_HPREG3_FEE;
	public static int CH_HPREG4_FEE;
	public static int CH_HPREG5_FEE;
	public static int CH_HPREG6_FEE;
	public static int CH_HPREG7_FEE;
	public static int CH_HPREG8_FEE;
	public static int CH_HPREG9_FEE;
	public static int CH_HPREG10_FEE;
	public static int CH_HPREG11_FEE;
	public static int CH_HPREG12_FEE;
	public static long CH_EXPREG_FEE_RATIO;
	public static int CH_EXPREG1_FEE;
	public static int CH_EXPREG2_FEE;
	public static int CH_EXPREG3_FEE;
	public static int CH_EXPREG4_FEE;
	public static int CH_EXPREG5_FEE;
	public static int CH_EXPREG6_FEE;
	public static int CH_EXPREG7_FEE;
	public static int CH_EXPREG8_FEE;
	public static long CH_SUPPORT_FEE_RATIO;
	public static int CH_SUPPORT1_FEE;
	public static int CH_SUPPORT2_FEE;
	public static int CH_SUPPORT3_FEE;
	public static int CH_SUPPORT4_FEE;
	public static int CH_SUPPORT5_FEE;
	public static int CH_SUPPORT7_FEE;
	public static int CH_SUPPORT8_FEE;
	public static long CH_CURTAIN_FEE_RATIO;
	public static int CH_CURTAIN1_FEE;
	public static int CH_CURTAIN2_FEE;
	public static long CH_PLATFORM_FEE_RATIO;
	public static int CH_PLATFORM1_FEE;
	public static int CH_PLATFORM2_FEE;

	/** Show licence or not just after login (if false, will directly go to the Server List */
	public static boolean SHOW_LICENCE;

	/** Deafult punishment for illegal actions */
	public static int DEFAULT_PUNISH;

	public static boolean ACCEPT_NEW_GAMESERVER;
	public static byte[] HEX_ID;
	public static boolean ACCEPT_ALTERNATE_ID;
	public static int REQUEST_ID;

	public static boolean ANNOUNCE_MAMMON_SPAWN;

	public static int GM_NAME_COLOUR;
	public static boolean GM_HERO_AURA;
	public static int NORMAL_NAME_COLOUR;
	public static int CLANLEADER_NAME_COLOUR;
	public static int BOT_NAME_COLOUR;
	public static String BOT_NAME_HEX_COLOUR;

	/** AI */
	public static boolean RND_WALK;
	public static int RND_WALK_RATE;
	public static int RND_ANIMATION_RATE;

	/** Maximum range mobs can randomly go from spawn point */
	public static int MAX_DRIFT_RANGE;

	/** Maximum range mobs can pursue agressor from spawn point */
	public static int MAX_PURSUE_RANGE;

	public static boolean ALT_AI_KELTIRS;

	/** It's EVIL :) */
	public static boolean MOBSLOOTERS;

	public static boolean RRD_ENABLED;
	public static boolean RRD_EXTENDED;
	public static boolean RRD_GRAPH_AA;
	public static String RRD_PATH;
	public static String RRD_EXT_PATH;
	public static String RRD_GRAPH_PATH;
	public static String RRD_GRAPH_FORMAT;
	public static long RRD_REDRAW_TIME;
	public static long RRD_UPDATE_TIME;
	public static int RRD_GRAPH_HEIGHT;
	public static int RRD_GRAPH_WIDTH;

	public static boolean ALT_DEATH_PENALTY;
	public static boolean ALLOW_DEATH_PENALTY_C5;
	public static int ALT_DEATH_PENALTY_C5_CHANCE;
	public static boolean ALT_DEATH_PENALTY_C5_CHAOTIC_RECOVERY;
	public static int ALT_DEATH_PENALTY_C5_EXPERIENCE_PENALTY;
	public static int ALT_DEATH_PENALTY_C5_KARMA_PENALTY;

	public static boolean HIDE_GM_STATUS;
	public static boolean SHOW_GM_LOGIN;
	public static boolean SAVE_GM_EFFECTS; //Silence, gmspeed, etc...

	/** security options */
	public static String DISABLE_CREATION_IDs;
	public static ArrayList<Integer> DISABLE_CREATION_ID_LIST = new ArrayList<Integer>();
	public static String LOG_MULTISELL_IDs;
	public static ArrayList<Integer> LOG_MULTISELL_ID_LIST = new ArrayList<Integer>();

	public static boolean ENABLE_FISHING_CHECK_WATER;

	public static boolean AUTO_LEARN_SKILLS;

	/** Debug, only for GM */
	public static boolean GEODATA_DEBUG;

	public static int MOVE_DELAY;

	public static boolean DAMAGE_FROM_FALLING;

	public static String GEOEDITOR_HOST;

	/** Community Board */
	public static boolean ALLOW_COMMUNITYBOARD;
	public static boolean COMMUNITYBOARDLOGGIN;
    public static String COMMUNITYBOARD_HTML_ROOT;
	public static String BBS_DEFAULT;
	public static int NAME_PAGE_SIZE_COMMUNITYBOARD;
	public static int NAME_PER_ROW_COMMUNITYBOARD;

	/** Wedding Options */
	public static boolean ALLOW_WEDDING;
	public static int WEDDING_GIVE_ITEM;
	public static int WEDDING_GIVE_ITEM_COUNT;
	public static int WEDDING_PRICE;
	public static boolean WEDDING_PUNISH_INFIDELITY;
	public static boolean WEDDING_TELEPORT;
	public static int WEDDING_TELEPORT_PRICE;
	public static int WEDDING_TELEPORT_INTERVAL;
	public static boolean WEDDING_SAMESEX;
	public static boolean WEDDING_FORMALWEAR;
	public static int WEDDING_DIVORCE_COSTS;

	public static boolean FORCE_STATUSUPDATE;

	/** Castle siege options **/
	public static boolean SIEGE_OPERATE_DOORS;
	public static boolean SIEGE_OPERATE_DOORS_LORD_ONLY;

	/**limits of stats **/
	public static int LIM_MATK_SPD;
	public static int LIM_PATK_SPD;
	public static int LIM_CRIT;
	public static int LIM_MCRIT;

	/** Enchant Config **/
	public static int SAFE_ENCHANT_COMMON;
	public static int SAFE_ENCHANT_FULL_BODY;

	public static int FESTIVAL_MIN_PARTY_SIZE;

	/** Dimensional Rift Config **/
	public static int RIFT_MIN_PARTY_SIZE;
	public static int RIFT_MAX_JUMPS;
	public static int RIFT_AUTO_JUMPS_TIME;
	public static int RIFT_AUTO_JUMPS_TIME_RAND;
	public static int RIFT_ENTER_COST_RECRUIT;
	public static int RIFT_ENTER_COST_SOLDIER;
	public static int RIFT_ENTER_COST_OFFICER;
	public static int RIFT_ENTER_COST_CAPTAIN;
	public static int RIFT_ENTER_COST_COMMANDER;
	public static int RIFT_ENTER_COST_HERO;
	public static float RIFT_BOSS_ROOM_TIME_MUTIPLY;

	/** Some more LS Settings */
	public static FastList<String> INTERNAL_IP = null;
	/** Продвинутый список локальных сетей / ip-адресов */
	public static NetList INTERNAL_NETLIST = null;

	public static boolean ALLOW_TALK_WHILE_SITTING;

	public static boolean ALLOW_FAKE_PLAYERS;
	public static int FAKE_PLAYERS_PERCENT;

	public static boolean PARTY_LEADER_ONLY_CAN_INVITE;

	/** Разрешены ли клановые скилы? **/
	public static boolean ALLOW_CLANSKILLS;

	/** Разрешено ли изучение скилов трансформации без наличия выполненного квеста */
	public static boolean ALLOW_LEARN_TRANS_SKILLS_WO_QUEST;

	/** Player visibility **/
	public static int PLAYER_VISIBILITY;
	public static int PLAYER_VISIBILITY_Z;

	public static boolean LOGIN_GG_CHECK;
	public static boolean GG_CHECK;

	/** Allow Manor system */
	public static boolean ALLOW_MANOR;

	/** Manor Refresh Starting time */
	public static int MANOR_REFRESH_TIME;

	/** Manor Refresh Min */
	public static int MANOR_REFRESH_MIN;

	/** Manor Next Period Approve Starting time */
	public static int MANOR_APPROVE_TIME;

	/** Manor Next Period Approve Min */
	public static int MANOR_APPROVE_MIN;

	/** Manor Maintenance Time */
	public static int MANOR_MAINTENANCE_PERIOD;

	/** Manor Save All Actions */
	public static boolean MANOR_SAVE_ALL_ACTIONS;

	/** Manor Save Period Rate */
	public static int MANOR_SAVE_PERIOD_RATE;

	public static int SHIFT_BY;
	public static int SHIFT_BY_FOR_Z;

	/** NPC Buffer config */
	public static int EVENT_Min_lvl;
	public static int EVENT_Max_lvl;
	public static int EVENT_price;
	public static boolean EVENT_Buffer_Siege;

	public static int EVENT_CofferOfShadowsPriceRate;
	public static float EVENT_CofferOfShadowsRewardRate;

	/** Bonus event **/
	public static int EVENT_BonusItemID;
	public static int EVENT_BonusPrice;
	public static int EVENT_BonusRate;
	public static int EVENT_BonusDays;

	public static boolean EVENT_ClassmastersSellsSS;
	public static boolean EVENT_ClassmastersCoLShop;

	public static int EVENT_LastHeroItemID;
	public static int EVENT_LastHeroItemCOUNT;
	public static int EVENT_LastHeroTime;
	public static boolean EVENT_LastHeroRate;

	public static int TFH_POLLEN_CHANCE;

	public static int GLIT_MEDAL_CHANCE;
	public static int GLIT_GLITTMEDAL_CHANCE;
	public static boolean GLIT_EnableRate;

	public static boolean SERVICES_NO_TRADE_ONLY_OFFLINE;
	public static float SERVICES_TRADE_TAX;
	public static float SERVICES_OFFSHORE_TRADE_TAX;
	public static boolean SERVICES_OFFSHORE_NO_CASTLE_TAX;
	public static boolean SERVICES_TRADE_TAX_ONLY_OFFLINE;
	public static boolean SERVICES_TRADE_ONLY_FAR;
	public static int SERVICES_TRADE_RADIUS;

	public static String VipSkillsCfg;
	public static int[] VipSkillsList = new int[15];

	/** Shutdown settings **/
	public static boolean OLD_SHUTDOWN_MSG;

	private static void initProtection(boolean gs)
	{
		File fp = new File(PROTECT_FILE);
		PROTECT_ENABLE = fp.exists();
		if(PROTECT_ENABLE)
			try
			{
				Properties protectSettings = new Properties();
				InputStream is = new FileInputStream(fp);
				protectSettings.load(is);
				is.close();

				PROTECT_UNPROTECTED_IPS = new NetList();

				String _ips = protectSettings.getProperty("UpProtectedIPs", "");
				if(_ips != "")
					PROTECT_UNPROTECTED_IPS.LoadFromString(_ips, ",");

				if(gs)
				{
					PROTECT_GS_STORE_HWID = Boolean.parseBoolean(protectSettings.getProperty("StoreHWID", "false"));
					PROTECT_GS_LOG_HWID = Boolean.parseBoolean(protectSettings.getProperty("LogHWIDs", "false"));
					PROTECT_ENABLE_HWID_BANS = Boolean.parseBoolean(protectSettings.getProperty("EnableHWIDBans", "false"));
                    PROTECT_COMPRESSION_WRITEDELAY = Integer.parseInt(protectSettings.getProperty("CompressionWriteDelay", "2"));
				}

			}
			catch(Exception e)
			{}
	}

	private static void loadAntiFlood(Properties _settings)
	{
		try
		{
			ANTIFLOOD_ENABLE = Boolean.parseBoolean(_settings.getProperty("AntiFloodEnable", "false"));
			if(!ANTIFLOOD_ENABLE)
				return;
			MAX_UNHANDLED_SOCKETS_PER_IP = Integer.parseInt(_settings.getProperty("MaxUnhandledSocketsPerIP", "5"));
			UNHANDLED_SOCKET_MIN_TTL = Integer.parseInt(_settings.getProperty("UnhandledSocketsMinTTL", "5000"));
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new Error("Failed to Load AntiFlood Properties.");
		}
	}

	public static void reloadPacketLoggerConfig()
	{
		try
		{
			Properties serverSettings = new Properties();
			InputStream is = new FileInputStream(new File(CONFIGURATION_FILE));
			serverSettings.load(is);
			is.close();

			LOG_CLIENT_PACKETS = Boolean.parseBoolean(serverSettings.getProperty("LogClientPackets", "False"));
			LOG_SERVER_PACKETS = Boolean.parseBoolean(serverSettings.getProperty("LogServerPackets", "False"));
			PACKETLOGGER_FLUSH_SIZE = Integer.parseInt(serverSettings.getProperty("LogPacketsFlushSize", "8192"));

			String temp = serverSettings.getProperty("LogPacketsFromIPs", "").trim();
			if(temp.isEmpty())
				PACKETLOGGER_IPS = null;
			else
			{
				PACKETLOGGER_IPS = new NetList();
				PACKETLOGGER_IPS.LoadFromString(temp, ",");
			}

			temp = serverSettings.getProperty("LogPacketsFromAccounts", "").trim();
			if(temp.isEmpty())
				PACKETLOGGER_ACCOUNTS = null;
			else
			{
				PACKETLOGGER_ACCOUNTS = new FastList<String>();
				for(String s : temp.split(","))
					PACKETLOGGER_ACCOUNTS.add(s);
			}

			temp = serverSettings.getProperty("LogPacketsFromChars", "").trim();
			if(temp.isEmpty())
				PACKETLOGGER_CHARACTERS = null;
			else
			{
				PACKETLOGGER_CHARACTERS = new FastList<String>();
				for(String s : temp.split(","))
					PACKETLOGGER_CHARACTERS.add(s);
			}

			temp = null;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new Error("Failed to Load PacketLogger Config.");
		}
	}

	public static void load()
	{
		//protection
		initProtection(Server.SERVER_MODE == Server.MODE_GAMESERVER || Server.SERVER_MODE == Server.MODE_COMBOSERVER);

		if(Server.SERVER_MODE == Server.MODE_GAMESERVER || Server.SERVER_MODE == Server.MODE_COMBOSERVER)
		{
			_log.info("Loading gameserver config.");
			try
			{
				Properties serverSettings = new Properties();
				InputStream is = new FileInputStream(new File(CONFIGURATION_FILE));
				serverSettings.load(is);
				is.close();

				loadAntiFlood(serverSettings);
				reloadPacketLoggerConfig();
				GAME_SERVER_LOGIN_HOST = serverSettings.getProperty("LoginHost", "127.0.0.1");
				GAME_SERVER_LOGIN_PORT = Integer.parseInt(serverSettings.getProperty("LoginPort", "9013"));
				DEBUG = Boolean.parseBoolean(serverSettings.getProperty("Debug", "false"));
				SERVER_LIST_TESTSERVER = Boolean.parseBoolean(serverSettings.getProperty("TestServer", "false"));
				ADVIPSYSTEM = Boolean.parseBoolean(serverSettings.getProperty("AdvIPSystem", "false"));
				WebServerPort = Integer.parseInt(serverSettings.getProperty("WebServerPort", "7778"));
				WebServerRoot = serverSettings.getProperty("WebServerRoot", "./data/webserver");
				HIDE_GM_STATUS = Boolean.parseBoolean(serverSettings.getProperty("HideGMStatus", "false"));
				SHOW_GM_LOGIN = Boolean.parseBoolean(serverSettings.getProperty("ShowGMLogin", "true"));
				SAVE_GM_EFFECTS = Boolean.parseBoolean(serverSettings.getProperty("SaveGMEffects", "false"));

				REQUEST_ID = Integer.parseInt(serverSettings.getProperty("RequestServerID", "0"));
				ACCEPT_ALTERNATE_ID = Boolean.parseBoolean(serverSettings.getProperty("AcceptAlternateID", "True"));

				PORT_GAME = Integer.parseInt(serverSettings.getProperty("GameserverPort", "7777"));
				PORT_LOGIN = Integer.parseInt(serverSettings.getProperty("LoginserverPort", "2106"));
				CNAME_TEMPLATE = serverSettings.getProperty("CnameTemplate", "[A-Za-z0-9\u0410-\u042f\u0430-\u044f]{2,16}");
				CLAN_NAME_TEMPLATE = serverSettings.getProperty("ClanNameTemplate", "[A-Za-z0-9\u0410-\u042f\u0430-\u044f]{3,16}");
				CLAN_TITLE_TEMPLATE = serverSettings.getProperty("ClanTitleTemplate", "[A-Za-z0-9\u0410-\u042f\u0430-\u044f \\p{Punct}]{1,16}");
				ALLY_NAME_TEMPLATE = serverSettings.getProperty("AllyNameTemplate", "[A-Za-z0-9\u0410-\u042f\u0430-\u044f]{3,16}");
				ANAME_TEMPLATE = serverSettings.getProperty("AnameTemplate", "[A-Za-z0-9]{3,14}");
				APASSWD_TEMPLATE = serverSettings.getProperty("ApasswdTemplate", "[A-Za-z0-9]{5,16}");
				LOGIN_TRY_BEFORE_BAN = Integer.parseInt(serverSettings.getProperty("LoginTryBeforeBan", "10"));
				GAMESERVER_HOSTNAME = serverSettings.getProperty("GameserverHostname");

				GLOBAL_CHAT = Integer.parseInt(serverSettings.getProperty("GlobalChat", "0"));
				GLOBAL_TRADE_CHAT = Integer.parseInt(serverSettings.getProperty("GlobalTradeChat", "0"));
				SHOUT_CHAT_MODE = Integer.parseInt(serverSettings.getProperty("ShoutChatMode", "1"));
				TRADE_CHAT_MODE = Integer.parseInt(serverSettings.getProperty("TradeChatMode", "1"));
				EVERYBODY_HAS_ADMIN_RIGHTS = Boolean.parseBoolean(serverSettings.getProperty("EverybodyHasAdminRights", "false"));
				ALLOW_SPECIAL_COMMANDS = Boolean.parseBoolean(serverSettings.getProperty("AllowSpecialCommands", "false"));
				LOG_CHAT = Boolean.valueOf(serverSettings.getProperty("LogChat", "false"));
				LOG_KILLS = Boolean.valueOf(serverSettings.getProperty("LogKills", "false"));
				LOG_TELNET = Boolean.valueOf(serverSettings.getProperty("LogTelnet", "false"));
				SQL_LOG = Boolean.valueOf(serverSettings.getProperty("SqlLog", "false"));

				RATE_XP = Float.parseFloat(serverSettings.getProperty("RateXp", "1."));
				RATE_SP = Float.parseFloat(serverSettings.getProperty("RateSp", "1."));
				RATE_QUESTS_REWARD = Float.parseFloat(serverSettings.getProperty("RateQuestsReward", "1."));
				RATE_QUESTS_DROP = Float.parseFloat(serverSettings.getProperty("RateQuestsDrop", "1."));
				RATE_QUESTS_DROP_PROF = Float.parseFloat(serverSettings.getProperty("RateQuestsDropProf", "1."));
				RATE_QUESTS_OCCUPATION_CHANGE = Boolean.parseBoolean(serverSettings.getProperty("RateQuestsRewardOccupationChange", "true"));
				RATE_CLAN_REP_SCORE = Float.parseFloat(serverSettings.getProperty("RateClanRepScore", "1."));
				RATE_CLAN_REP_SCORE_MAX_AFFECTED = Integer.parseInt(serverSettings.getProperty("RateClanRepScoreMaxAffected", "2"));
				RATE_DROP_ADENA = Float.parseFloat(serverSettings.getProperty("RateDropAdena", "1."));
				RATE_DROP_ADENA_PARTY = Float.parseFloat(serverSettings.getProperty("RateDropAdenaParty", "1."));
				RATE_DROP_ITEMS_PARTY = Float.parseFloat(serverSettings.getProperty("RateDropItemsParty", "1."));
				RATE_XP_PARTY = Float.parseFloat(serverSettings.getProperty("RateXpParty", "1."));
				RATE_SP_PARTY = Float.parseFloat(serverSettings.getProperty("RateSpParty", "1."));
				RATE_DROP_ITEMS = Float.parseFloat(serverSettings.getProperty("RateDropItems", "1."));
				RATE_DROP_RAIDBOSS = Float.parseFloat(serverSettings.getProperty("RateRaidBoss", "1."));
				RATE_DROP_SPOIL = Float.parseFloat(serverSettings.getProperty("RateDropSpoil", "1."));
				RATE_MANOR = Integer.parseInt(serverSettings.getProperty("RateManor", "1"));
				RATE_FISH_DROP_COUNT = Float.parseFloat(serverSettings.getProperty("RateFishDropCount", "1."));
				RATE_SIEGE_GUARDS_PRICE = Float.parseFloat(serverSettings.getProperty("RateSiegeGuardsPrice", "1."));
				RATE_RAID_REGEN = Float.parseFloat(serverSettings.getProperty("RateRaidRegen", "1."));
				RAID_MAX_LEVEL_DIFF = Integer.parseInt(serverSettings.getProperty("RaidMaxLevelDiff", "8"));

				AUTODESTROY_ITEM_AFTER = Integer.parseInt(serverSettings.getProperty("AutoDestroyDroppedItemAfter", "0"));
				DELETE_DAYS = Integer.parseInt(serverSettings.getProperty("DeleteCharAfterDays", "7"));
				PURGE_BYPASS_TASK_FREQUENCY = Integer.parseInt(serverSettings.getProperty("PurgeTaskFrequency", "60"));

				DATAPACK_ROOT = new File(serverSettings.getProperty("DatapackRoot", ".")).getCanonicalFile();
				AttributeBonusFile = serverSettings.getProperty("AttributeBonusFile", "data/attribute_bonus.xml");

				ALLOW_L2WALKER_CLIENT = L2WalkerAllowed.valueOf(serverSettings.getProperty("AllowL2Walker", "False"));
				MAX_L2WALKER_LEVEL = Integer.parseInt(serverSettings.getProperty("MaxL2WalkerLevel", "5"));

				ALLOW_CRAFT_BOT = Boolean.parseBoolean(serverSettings.getProperty("AllowCraftBot", "True"));
				ALLOW_BOT_CAST = Boolean.parseBoolean(serverSettings.getProperty("AllowBotCast", "False"));
				KILL_BOTS_WITH_SKILLCOOLTIME = Boolean.parseBoolean(serverSettings.getProperty("KillBotsWithSkillCoolTime", "True"));
				L2WALKER_PUNISHMENT = Integer.parseInt(serverSettings.getProperty("L2WalkerPunishment", "1"));
				BUGUSER_PUNISH = Integer.parseInt(serverSettings.getProperty("BugUserPunishment", "2"));
				DEFAULT_PUNISH = Integer.parseInt(serverSettings.getProperty("IllegalActionPunishment", "1"));

				ALLOW_DISCARDITEM = Boolean.valueOf(serverSettings.getProperty("AllowDiscardItem", "True"));
				ALLOW_FREIGHT = Boolean.valueOf(serverSettings.getProperty("AllowFreight", "True"));
				ALLOW_WAREHOUSE = Boolean.valueOf(serverSettings.getProperty("AllowWarehouse", "True"));
				ALLOW_WATER = Boolean.valueOf(serverSettings.getProperty("AllowWater", "True"));
				ALLOW_BOAT = Boolean.valueOf(serverSettings.getProperty("AllowBoat", "False"));
				ALLOW_CURSED_WEAPONS = Boolean.valueOf(serverSettings.getProperty("AllowCursedWeapons", "False"));
				CURSED_WEAPONS_MIN_PLAYERS_DROP = Integer.parseInt(serverSettings.getProperty("CursedWeaponsMinPlayersDrop", "0")); 
				DROP_CURSED_WEAPONS_ON_KICK = Boolean.valueOf(serverSettings.getProperty("DropCursedWeaponsOnKick", "False"));

				MIN_PROTOCOL_REVISION = Integer.parseInt(serverSettings.getProperty("MinProtocolRevision", "709"));
				MAX_PROTOCOL_REVISION = Integer.parseInt(serverSettings.getProperty("MaxProtocolRevision", "746"));

				if(MIN_PROTOCOL_REVISION > MAX_PROTOCOL_REVISION)
					throw new Error("MinProtocolRevision is bigger than MaxProtocolRevision in server configuration file.");

				MIN_NPC_ANIMATION = Integer.parseInt(serverSettings.getProperty("MinNPCAnimation", "5"));
				MAX_NPC_ANIMATION = Integer.parseInt(serverSettings.getProperty("MaxNPCAnimation", "90"));

				ALLOW_COMMUNITYBOARD = Boolean.valueOf(serverSettings.getProperty("AllowCommunityBoard", "True"));
				COMMUNITYBOARDLOGGIN = Boolean.valueOf(serverSettings.getProperty("CommunityBoardLoggin", "False")); 
				BBS_DEFAULT = serverSettings.getProperty("BBSDefault", "_bbshome");
				NAME_PAGE_SIZE_COMMUNITYBOARD = Integer.parseInt(serverSettings.getProperty("NamePageSizeOnCommunityBoard", "50"));
				NAME_PER_ROW_COMMUNITYBOARD = Integer.parseInt(serverSettings.getProperty("NamePerRowOnCommunityBoard", "5"));
                COMMUNITYBOARD_HTML_ROOT = serverSettings.getProperty("CommunityBoardHtmlRoot", "data/html/CommunityBoard/");

				INTERNAL_HOSTNAME = serverSettings.getProperty("InternalHostname", "*");
				EXTERNAL_HOSTNAME = serverSettings.getProperty("ExternalHostname", "*");

				SERVER_SIDE_NPC_NAME = Boolean.valueOf(serverSettings.getProperty("ServerSideNpcName", "False"));
				SERVER_SIDE_NPC_TITLE_WITH_LVL = Boolean.valueOf(serverSettings.getProperty("ServerSideNpcTitleWithLvl", "False"));
				SERVER_SIDE_NPC_TITLE = Boolean.valueOf(serverSettings.getProperty("ServerSideNpcTitle", "False"));
				AUTODELETE_INVALID_QUEST_DATA = Boolean.valueOf(serverSettings.getProperty("AutoDeleteInvalidQuestData", "False"));

				HARD_DB_CLEANUP_ON_START = Boolean.valueOf(serverSettings.getProperty("HardDbCleanUpOnStart", "False"));

				MAXIMUM_ONLINE_USERS = Integer.parseInt(serverSettings.getProperty("MaximumOnlineUsers", "100"));

				DAY_STATUS_FORCE_CLIENT_UPDATE = Boolean.valueOf(serverSettings.getProperty("DayStatusForceClientUpdate", "True"));
				DAY_STATUS_SUN_RISE_AT = Integer.parseInt(serverSettings.getProperty("DayStatusSunRiseAt", "6"));
				DAY_STATUS_SUN_SET_AT = Integer.parseInt(serverSettings.getProperty("DayStatusSunSetAt", "18"));

				DATABASE_DRIVER = serverSettings.getProperty("Driver", "com.mysql.jdbc.Driver");
				DATABASE_URL = serverSettings.getProperty("URL", "jdbc:mysql://localhost/l2rdb");
				DATABASE_LOGIN = serverSettings.getProperty("Login", "root");
				DATABASE_PASSWORD = serverSettings.getProperty("Password", "");
				DATABASE_MAX_CONNECTIONS = Integer.parseInt(serverSettings.getProperty("MaximumDbConnections", "10"));
				USE_DATABASE_LAYER = Boolean.parseBoolean(serverSettings.getProperty("UseDatabaseLayer", "true"));

				LAZY_ITEM_UPDATE = Boolean.parseBoolean(serverSettings.getProperty("LazyItemUpdate", "False"));
				LAZY_ITEM_UPDATE_TIME = Integer.parseInt(serverSettings.getProperty("LazyItemUpdateTime", "60000"));

				SERVER_LIST_BRACKET = Boolean.valueOf(serverSettings.getProperty("ServerListBrackets", "false"));
				SERVER_LIST_CLOCK = Boolean.valueOf(serverSettings.getProperty("ServerListClock", "false"));
				SERVER_GMONLY = Boolean.valueOf(serverSettings.getProperty("ServerGMOnly", "false"));

				THREAD_P_EFFECTS = Integer.parseInt(serverSettings.getProperty("ThreadPoolSizeEffects", "10"));
				THREAD_P_GENERAL = Integer.parseInt(serverSettings.getProperty("ThreadPoolSizeGeneral", "15"));
				GENERAL_PACKET_THREAD_CORE_SIZE = Integer.parseInt(serverSettings.getProperty("GeneralPacketThreadCoreSize", "4"));
				URGENT_PACKET_THREAD_CORE_SIZE = Integer.parseInt(serverSettings.getProperty("UrgentPacketThreadCoreSize", "2"));
				AI_MAX_THREAD = Integer.parseInt(serverSettings.getProperty("AiMaxThread", "10"));
                INTEREST_ALT = Boolean.parseBoolean(serverSettings.getProperty("InterestAlt", "true"));
                INTEREST_MAX_THREAD = Integer.parseInt(serverSettings.getProperty("InterestMaxThread", "10"));

				MAT_BANCHAT = Boolean.valueOf(serverSettings.getProperty("MAT_BANCHAT", "False"));
				BAN_CHANNEL = String.valueOf(serverSettings.getProperty("MAT_BAN_CHANNEL", "0"));
				MAT_BAN_COUNT_CHANNELS = 1;
				for(String id : BAN_CHANNEL.split(","))
				{
					BAN_CHANNEL_LIST[MAT_BAN_COUNT_CHANNELS] = Integer.parseInt(id);
					MAT_BAN_COUNT_CHANNELS++;
				}
				MAT_REPLACE = Boolean.valueOf(serverSettings.getProperty("MAT_REPLACE", "False"));
				MAT_REPLACE_STRING = serverSettings.getProperty("MAT_REPLACE_STRING", "[censored]");
				MAT_ANNOUNCE = Boolean.valueOf(serverSettings.getProperty("MAT_ANNOUNCE", "True"));
				MAT_ANNOUNCE_NICK = Boolean.valueOf(serverSettings.getProperty("MAT_ANNOUNCE_NICK", "True"));
				UNCHATBANTIME = Integer.parseInt(serverSettings.getProperty("Timer_to_UnBan", "30"));
				DEFAULT_LANG = serverSettings.getProperty("DefaultLang", "ru");
				RESTART_TIME = Integer.parseInt(serverSettings.getProperty("AutoRestart", "0"));
				RESTART_AT_TIME = Integer.parseInt(serverSettings.getProperty("AutoRestartAt", "5"));
				if(RESTART_AT_TIME > 24)
					RESTART_AT_TIME = 24;

				CHECK_LANG_FILES_MODIFY = Boolean.valueOf(serverSettings.getProperty("checkLangFilesModify", "false"));

				USE_FILE_CACHE = Boolean.valueOf(serverSettings.getProperty("useFileCache", "true"));

				LINEAR_TERRITORY_CELL_SIZE = Integer.parseInt(serverSettings.getProperty("LinearTerritoryCellSize", "32"));

				DISABLE_CREATION_IDs = serverSettings.getProperty("DisableCreateItems", "");
				if(DISABLE_CREATION_IDs.length() != 0)
					for(String id : DISABLE_CREATION_IDs.split(","))
						DISABLE_CREATION_ID_LIST.add(Integer.parseInt(id));

				LOG_MULTISELL_IDs = serverSettings.getProperty("LogMultisellId", "");
				if(LOG_MULTISELL_IDs.length() != 0)
					for(String id : LOG_MULTISELL_IDs.split(","))
						LOG_MULTISELL_ID_LIST.add(Integer.parseInt(id));

				GEODATA_DEBUG = Boolean.parseBoolean(serverSettings.getProperty("GeodataDebug", "false"));

				MOVE_DELAY = Integer.parseInt(serverSettings.getProperty("MoveDelay", "500"));

				DAMAGE_FROM_FALLING = Boolean.parseBoolean(serverSettings.getProperty("DamageFromFalling", "True"));

				GEOEDITOR_HOST = serverSettings.getProperty("GeoEditorHost", "127.0.0.1");

				ALLOW_WEDDING = Boolean.valueOf(serverSettings.getProperty("AllowWedding", "False"));
				WEDDING_GIVE_ITEM = Integer.valueOf(serverSettings.getProperty("WeddingGiveItem", "0"));
				WEDDING_GIVE_ITEM_COUNT = Integer.valueOf(serverSettings.getProperty("WeddingGiveCount", "1"));
				WEDDING_PRICE = Integer.parseInt(serverSettings.getProperty("WeddingPrice", "500000"));
				WEDDING_PUNISH_INFIDELITY = Boolean.parseBoolean(serverSettings.getProperty("WeddingPunishInfidelity", "true"));
				WEDDING_TELEPORT = Boolean.parseBoolean(serverSettings.getProperty("WeddingTeleport", "true"));
				WEDDING_TELEPORT_PRICE = Integer.parseInt(serverSettings.getProperty("WeddingTeleportPrice", "500000"));
				WEDDING_TELEPORT_INTERVAL = Integer.parseInt(serverSettings.getProperty("WeddingTeleportInterval", "120"));
				WEDDING_SAMESEX = Boolean.parseBoolean(serverSettings.getProperty("WeddingAllowSameSex", "true"));
				WEDDING_FORMALWEAR = Boolean.parseBoolean(serverSettings.getProperty("WeddingFormalWear", "true"));
				WEDDING_DIVORCE_COSTS = Integer.parseInt(serverSettings.getProperty("WeddingDivorceCosts", "20"));

				FORCE_STATUSUPDATE = Boolean.parseBoolean(serverSettings.getProperty("ForceStatusUpdate", "false"));
				DONTLOADSPAWN = Boolean.parseBoolean(serverSettings.getProperty("StartWhisoutSpawn", "False"));
				DONTLOADQUEST = Boolean.parseBoolean(serverSettings.getProperty("StartWhisoutQuest", "False"));

				PLAYER_VISIBILITY = Integer.parseInt(serverSettings.getProperty("PlayerVisibilityHorizontal", "4000"));
				PLAYER_VISIBILITY_Z = Integer.parseInt(serverSettings.getProperty("PlayerVisibilityVertical", "1000"));
				if(PLAYER_VISIBILITY > 7000 || PLAYER_VISIBILITY < 1500)
					PLAYER_VISIBILITY = 4500;
				if(PLAYER_VISIBILITY_Z > 3000 || PLAYER_VISIBILITY_Z < 500)
					PLAYER_VISIBILITY_Z = 2000;
				GG_CHECK = Boolean.parseBoolean(serverSettings.getProperty("GGCheck", "True"));

				SHIFT_BY = Integer.parseInt(serverSettings.getProperty("ShiftBy", "12"));
				SHIFT_BY_FOR_Z = Integer.parseInt(serverSettings.getProperty("ShiftByForZ", "10"));

				VipSkillsCfg = serverSettings.getProperty("VipSkills", "");
				if(VipSkillsCfg.length() >= 2)
				{
					int num = 1;
					for(String id : VipSkillsCfg.split(","))
					{
						VipSkillsList[num] = Integer.parseInt(id);
						num++;
					}
				}

			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + CONFIGURATION_FILE + " File.");
			}

			/*
			 * Load L2R Version Properties file (if exists)
			 */
			try
			{
				Properties serverVersion = new Properties();
				InputStream is = Config.class.getResourceAsStream(Config.VERSION_FILE);
				serverVersion.load(is);
				is.close();

				SERVER_VERSION = serverVersion.getProperty("version", SERVER_VERSION_UNSUPPORTED);
				SERVER_BUILD_DATE = serverVersion.getProperty("builddate", "Undefined Date.");
				_log.info("Gameserver Version: " + SERVER_VERSION + ", build date: " + SERVER_BUILD_DATE);
			}
			catch(Exception e)
			{
				//Ignore Properties file if it doesn't exist
				SERVER_VERSION = SERVER_VERSION_UNSUPPORTED;
				SERVER_BUILD_DATE = "Undefined Date.";
			}

			// telnet
			try
			{
				Properties telnetSettings = new Properties();
				InputStream is = new FileInputStream(new File(TELNET_FILE));
				telnetSettings.load(is);
				is.close();

				IS_TELNET_ENABLED = Boolean.valueOf(telnetSettings.getProperty("EnableTelnet", "false"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + TELNET_FILE + " File.");
			}

			// clanhall settings
			try
			{
				Properties clanhallSettings = new Properties();
				InputStream is = new FileInputStream(new File(CLANHALL_CONFIG_FILE));
				clanhallSettings.load(is);
				is.close();

				ALLOW_CH_DOOR_OPEN_ON_CLICK = Boolean.valueOf(clanhallSettings.getProperty("AllowChDoorOpenOnClick", "true"));

				CH_TELE_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallTeleportFunctionFeeRation", "86400000"));
				CH_TELE1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallTeleportFunctionFeeLvl1", "86400000"));
				CH_TELE2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallTeleportFunctionFeeLvl2", "86400000"));
				CH_TELE3_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallTeleportFunctionFeeLvl3", "86400000"));
				CH_SUPPORT_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallSupportFunctionFeeRation", "86400000"));
				CH_SUPPORT1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl1", "86400000"));
				CH_SUPPORT2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl2", "86400000"));
				CH_SUPPORT3_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl3", "86400000"));
				CH_SUPPORT4_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl4", "86400000"));
				CH_SUPPORT5_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl5", "86400000"));
				CH_SUPPORT7_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl7", "86400000"));
				CH_SUPPORT8_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl8", "86400000"));
				CH_MPREG_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFunctionFeeRation", "86400000"));
				CH_MPREG1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFeeLvl1", "86400000"));
				CH_MPREG2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFeeLvl2", "86400000"));
				CH_MPREG3_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFeeLvl3", "86400000"));
				CH_MPREG4_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFeeLvl4", "86400000"));
				CH_MPREG5_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFeeLvl5", "86400000"));
				CH_MPREG6_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFeeLvl6", "86400000"));
				CH_HPREG_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFunctionFeeRation", "86400000"));
				CH_HPREG1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl1", "86400000"));
				CH_HPREG2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl2", "86400000"));
				CH_HPREG3_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl3", "86400000"));
				CH_HPREG4_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl4", "86400000"));
				CH_HPREG5_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl5", "86400000"));
				CH_HPREG6_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl6", "86400000"));
				CH_HPREG7_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl7", "86400000"));
				CH_HPREG8_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl8", "86400000"));
				CH_HPREG9_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl9", "86400000"));
				CH_HPREG10_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl10", "86400000"));
				CH_HPREG11_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl11", "86400000"));
				CH_HPREG12_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl12", "86400000"));
				CH_EXPREG_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFunctionFeeRation", "86400000"));
				CH_EXPREG1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl1", "86400000"));
				CH_EXPREG2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl2", "86400000"));
				CH_EXPREG3_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl3", "86400000"));
				CH_EXPREG4_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl4", "86400000"));
				CH_EXPREG5_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl5", "86400000"));
				CH_EXPREG6_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl6", "86400000"));
				CH_EXPREG7_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl7", "86400000"));
				CH_EXPREG8_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl8", "86400000"));
				CH_ITEM_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallItemCreationFunctionFeeRation", "86400000"));
				CH_ITEM1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallItemCreationFunctionFeeLvl1", "86400000"));
				CH_ITEM2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallItemCreationFunctionFeeLvl2", "86400000"));
				CH_ITEM3_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallItemCreationFunctionFeeLvl3", "86400000"));
				CH_CURTAIN_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallCurtainFunctionFeeRation", "86400000"));
				CH_CURTAIN1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallCurtainFunctionFeeLvl1", "86400000"));
				CH_CURTAIN2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallCurtainFunctionFeeLvl2", "86400000"));
				CH_PLATFORM_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallFrontPlatformFunctionFeeRation", "86400000"));
				CH_PLATFORM1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallFrontPlatformFunctionFeeLvl1", "86400000"));
				CH_PLATFORM2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallFrontPlatformFunctionFeeLvl2", "86400000"));

			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + CLANHALL_CONFIG_FILE + " File.");
			}

			// other
			try
			{
				Properties otherSettings = new Properties();
				InputStream is = new FileInputStream(new File(OTHER_CONFIG_FILE));
				otherSettings.load(is);
				is.close();

				DEEPBLUE_DROP_RULES = Boolean.parseBoolean(otherSettings.getProperty("UseDeepBlueDropRules", "True"));
				WYVERN_SPEED = Integer.parseInt(otherSettings.getProperty("WyvernSpeed", "100"));
				STRIDER_SPEED = Integer.parseInt(otherSettings.getProperty("StriderSpeed", "80"));
				SWIMING_SPEED = Integer.parseInt(otherSettings.getProperty("SwimingSpeedTemplate", "50"));

				/* Inventory slots limits */
				INVENTORY_MAXIMUM_NO_DWARF = Integer.parseInt(otherSettings.getProperty("MaximumSlotsForNoDwarf", "80"));
				INVENTORY_MAXIMUM_DWARF = Integer.parseInt(otherSettings.getProperty("MaximumSlotsForDwarf", "100"));
				INVENTORY_MAXIMUM_GM = Integer.parseInt(otherSettings.getProperty("MaximumSlotsForGMPlayer", "250"));

				MULTISELL_SIZE = Integer.parseInt(otherSettings.getProperty("MultisellPageSize", "10"));

				/* Warehouse slots limits */
				WAREHOUSE_SLOTS_NO_DWARF = Integer.parseInt(otherSettings.getProperty("BaseWarehouseSlotsForNoDwarf", "100"));
				WAREHOUSE_SLOTS_DWARF = Integer.parseInt(otherSettings.getProperty("BaseWarehouseSlotsForDwarf", "120"));
				WAREHOUSE_SLOTS_CLAN = Integer.parseInt(otherSettings.getProperty("MaximumWarehouseSlotsForClan", "200"));

				/* Alt Enchant Formula */
				ALT_ENCHANT_FORMULA = Boolean.valueOf(otherSettings.getProperty("AltEnchantFormulaForPvPServers", "False"));
				ALT_ENCHANT_CHANCE_W = Integer.parseInt(otherSettings.getProperty("EChanceWeapon", "80"));
				ALT_ENCHANT_CHANCE_MAGE_W = Integer.parseInt(otherSettings.getProperty("EChanceMageWeapon", "80"));
				ALT_ENCHANT_CHANCE_ARMOR = Integer.parseInt(otherSettings.getProperty("EChanceArmor", "80"));
				PENALTY_TO_TWOHANDED_BLUNTS = Integer.parseInt(otherSettings.getProperty("PenaltyforEChanceToHandBlunt", "18"));

				/* chance to enchant an item over safe level */
				ENCHANT_CHANCE_WEAPON = Integer.parseInt(otherSettings.getProperty("EnchantChance", "66"));
				ENCHANT_CHANCE_ARMOR = Integer.parseInt(otherSettings.getProperty("EnchantChanceArmor", String.valueOf(ENCHANT_CHANCE_WEAPON)));
				ENCHANT_CHANCE_CRYSTAL_WEAPON = Integer.parseInt(otherSettings.getProperty("EnchantChanceCrystal", "66"));
				ENCHANT_CHANCE_CRYSTAL_ARMOR = Integer.parseInt(otherSettings.getProperty("EnchantChanceCrystalArmor", String.valueOf(ENCHANT_CHANCE_CRYSTAL_WEAPON)));
				SAFE_ENCHANT_COMMON = Integer.parseInt(otherSettings.getProperty("SafeEnchantCommon", "3"));
				SAFE_ENCHANT_FULL_BODY = Integer.parseInt(otherSettings.getProperty("SafeEnchantFullBody", "4"));
				ENCHANT_MAX_WEAPON = Byte.parseByte(otherSettings.getProperty("EnchantMaxWeapon", "20"));
				ENCHANT_MAX_ACCESSORY = Byte.parseByte(otherSettings.getProperty("EnchantMaxAccessory", "20"));
				ENCHANT_MAX_ARMOR = Byte.parseByte(otherSettings.getProperty("EnchantMaxArmor", "20"));
				ENCHANT_MODIFER = Double.parseDouble(otherSettings.getProperty("EnchantModifer", "0"));
				LUCK_BONUS = Boolean.valueOf(otherSettings.getProperty("EnableBonusForEnchant", "False"));

				REGEN_SIT_WAIT = Boolean.valueOf(otherSettings.getProperty("RegenSitWait", "False"));

				STARTING_ADENA = Integer.parseInt(otherSettings.getProperty("StartingAdena", "0"));
				UNSTUCK_SKILL = Boolean.parseBoolean(otherSettings.getProperty("UnstuckSkill", "True"));

				/* Amount of HP, MP, and CP is restored */
				RESPAWN_RESTORE_CP = Double.parseDouble(otherSettings.getProperty("RespawnRestoreCP", "-1")) / 100;
				RESPAWN_RESTORE_HP = Double.parseDouble(otherSettings.getProperty("RespawnRestoreHP", "65")) / 100;
				RESPAWN_RESTORE_MP = Double.parseDouble(otherSettings.getProperty("RespawnRestoreMP", "-1")) / 100;

				/* Maximum number of available slots for pvt stores */
				MAX_PVTSTORE_SLOTS_DWARF = Integer.parseInt(otherSettings.getProperty("MaxPvtStoreSlotsDwarf", "5"));
				MAX_PVTSTORE_SLOTS_OTHER = Integer.parseInt(otherSettings.getProperty("MaxPvtStoreSlotsOther", "4"));
				MAX_PVTCRAFT_SLOTS = Integer.parseInt(otherSettings.getProperty("MaxPvtManufactureSlots", "20"));

				ANNOUNCE_MAMMON_SPAWN = Boolean.parseBoolean(otherSettings.getProperty("AnnounceMammonSpawn", "True"));

				GM_NAME_COLOUR = Integer.decode("0x" + otherSettings.getProperty("GMNameColour", "FFFFFF"));
				GM_HERO_AURA = Boolean.parseBoolean(otherSettings.getProperty("GMHeroAura", "True"));
				NORMAL_NAME_COLOUR = Integer.decode("0x" + otherSettings.getProperty("NormalNameColour", "FFFFFF"));
				CLANLEADER_NAME_COLOUR = Integer.decode("0x" + otherSettings.getProperty("ClanleaderNameColour", "FFFFFF"));
				BOT_NAME_COLOUR = Integer.decode("0x" + otherSettings.getProperty("BotNameColour", "FFFFFF"));
				BOT_NAME_HEX_COLOUR = otherSettings.getProperty("BotNameHexColour", "FFFFFF");

				RRD_ENABLED = Boolean.valueOf(otherSettings.getProperty("UseRRD", "True"));
				RRD_EXTENDED = Boolean.valueOf(otherSettings.getProperty("UseExtendedRRD", "False"));
				RRD_GRAPH_AA = Boolean.valueOf(otherSettings.getProperty("GraphAntiAliasing", "False"));
				RRD_PATH = otherSettings.getProperty("RRDPath", "./config/");
				RRD_GRAPH_PATH = otherSettings.getProperty("GraphPath", "./data/webserver/");
				RRD_REDRAW_TIME = Long.parseLong(otherSettings.getProperty("GraphRedrawDelay", "300"));
				RRD_UPDATE_TIME = Long.parseLong(otherSettings.getProperty("UpdateDelay", "30"));
				RRD_GRAPH_FORMAT = otherSettings.getProperty("GraphFormat", "png");
				RRD_GRAPH_HEIGHT = Integer.parseInt(otherSettings.getProperty("GraphHeight", "378"));
				RRD_GRAPH_WIDTH = Integer.parseInt(otherSettings.getProperty("GraphWidth", "580"));
				SHOW_HTML_WELCOME = Boolean.parseBoolean(otherSettings.getProperty("ShowHTMLWelcome", "false"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + OTHER_CONFIG_FILE + " File.");
			}

			// spoil & manor
			try
			{
				Properties spoilSettings = new Properties();
				InputStream is = new FileInputStream(new File(SPOIL_CONFIG_FILE));
				spoilSettings.load(is);
				is.close();

				BASE_SPOIL_RATE = Float.parseFloat(spoilSettings.getProperty("BasePercentChanceOfSpoilSuccess", "78."));
				MINIMUM_SPOIL_RATE = Float.parseFloat(spoilSettings.getProperty("MinimumPercentChanceOfSpoilSuccess", "1."));
				ALT_SPOIL_FORMULA = Boolean.parseBoolean(spoilSettings.getProperty("AltFormula", "False"));
				MANOR_SOWING_BASIC_SUCCESS = Integer.parseInt(spoilSettings.getProperty("BasePercentChanceOfSowingSuccess", "100"));
				MANOR_SOWING_ALT_BASIC_SUCCESS = Integer.parseInt(spoilSettings.getProperty("BasePercentChanceOfSowingAltSuccess", "10"));
				MANOR_HARVESTING_BASIC_SUCCESS = Integer.parseInt(spoilSettings.getProperty("BasePercentChanceOfHarvestingSuccess", "90"));
				MANOR_DIFF_PLAYER_TARGET = Integer.parseInt(spoilSettings.getProperty("MinDiffPlayerMob", "5"));
				MANOR_DIFF_PLAYER_TARGET_PENALTY = Integer.parseInt(spoilSettings.getProperty("DiffPlayerMobPenalty", "5"));
				MANOR_DIFF_SEED_TARGET = Integer.parseInt(spoilSettings.getProperty("MinDiffSeedMob", "5"));
				MANOR_DIFF_SEED_TARGET_PENALTY = Integer.parseInt(spoilSettings.getProperty("DiffSeedMobPenalty", "5"));
				ALLOW_MANOR = Boolean.parseBoolean(spoilSettings.getProperty("AllowManor", "true"));
				MANOR_REFRESH_TIME = Integer.parseInt(spoilSettings.getProperty("AltManorRefreshTime", "20"));
				MANOR_REFRESH_MIN = Integer.parseInt(spoilSettings.getProperty("AltManorRefreshMin", "00"));
				MANOR_APPROVE_TIME = Integer.parseInt(spoilSettings.getProperty("AltManorApproveTime", "6"));
				MANOR_APPROVE_MIN = Integer.parseInt(spoilSettings.getProperty("AltManorApproveMin", "00"));
				MANOR_MAINTENANCE_PERIOD = Integer.parseInt(spoilSettings.getProperty("AltManorMaintenancePeriod", "360000"));
				MANOR_SAVE_ALL_ACTIONS = Boolean.parseBoolean(spoilSettings.getProperty("AltManorSaveAllActions", "false"));
				MANOR_SAVE_PERIOD_RATE = Integer.parseInt(spoilSettings.getProperty("AltManorSavePeriodRate", "2"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + SPOIL_CONFIG_FILE + " File.");
			}

			// alternative settings
			try
			{
				Properties altSettings = new Properties();
				InputStream is = new FileInputStream(new File(ALT_SETTINGS_FILE));
				altSettings.load(is);
				is.close();

				ALT_GAME_DELEVEL = Boolean.parseBoolean(altSettings.getProperty("Delevel", "true"));

				ALT_SKILLS_CHANCES = Boolean.parseBoolean(altSettings.getProperty("AltSkillChances", "False"));
				SKILLS_CHANCE_CAP = Double.parseDouble(altSettings.getProperty("SkillsChanceCap", "95"));
				SKILLS_SHOW_CHANCE = Boolean.parseBoolean(altSettings.getProperty("SkillsShowChance", "True"));
				ALT_CUBICS_SAVE = Boolean.parseBoolean(altSettings.getProperty("AltCubicsSave", "False"));
				ALT_SHOW_REUSE_MSG = Boolean.parseBoolean(altSettings.getProperty("AltShowSkillReuseMessage", "True"));
				WEAR_TEST_ENABLED = Boolean.valueOf(altSettings.getProperty("WearTestEnabled", "False"));
				AUTO_LOOT = Boolean.parseBoolean(altSettings.getProperty("AutoLoot", "True"));
				AUTO_LOOT_PK = Boolean.parseBoolean(altSettings.getProperty("AutoLootPK", "True"));
				AUTO_LOOT_HERBS = Boolean.parseBoolean(altSettings.getProperty("AutoLootHerbs", "True"));
				HERBS_DIVIDE = Boolean.parseBoolean(altSettings.getProperty("HerbsDivide", "False"));
				ALT_GAME_KARMAPLAYERCANBEKILLEDINPEACEZONE = Boolean.valueOf(altSettings.getProperty("AltKarmaPlayerCanBeKilledInPeaceZone", "false"));
				ALT_GAME_KARMA_PLAYER_CAN_SHOP = Boolean.valueOf(altSettings.getProperty("AltKarmaPlayerCanShop", "false"));
				SAVING_SPS = Boolean.parseBoolean(altSettings.getProperty("SavingSpS", "false"));
				MANAHEAL_SPS_BONUS = Boolean.parseBoolean(altSettings.getProperty("ManahealSpSBonus", "false"));
				KILL_COUNTER = Boolean.parseBoolean(altSettings.getProperty("KillCounter", "true"));
				KILL_COUNTER_PRELOAD = Boolean.parseBoolean(altSettings.getProperty("KillCounterPreload", "true"));
				DROP_COUNTER = Boolean.parseBoolean(altSettings.getProperty("DropCounter", "true"));
				CRAFT_COUNTER = Boolean.parseBoolean(altSettings.getProperty("CraftCounter", "true"));
				ALT_TRUE_CHESTS = Integer.parseInt(altSettings.getProperty("TrueChests", "33"));
				ALT_GAME_MATHERIALSDROP = Boolean.parseBoolean(altSettings.getProperty("AltMatherialsDrop", "false"));
				ALT_DOUBLE_SPAWN = Boolean.parseBoolean(altSettings.getProperty("DoubleSpawn", "false"));
				ALT_ALLOW_AUGMENT_ALL = Boolean.parseBoolean(altSettings.getProperty("AugmentAll", "false"));
				ALT_ALLOW_DROP_AUGMENTED = Boolean.parseBoolean(altSettings.getProperty("AlowDropAugmented", "false"));
				ALT_GAME_EXP_FOR_CRAFT = Boolean.parseBoolean(altSettings.getProperty("AltExpForCraft", "true"));
				ALT_GAME_UNREGISTER_RECIPE = Boolean.parseBoolean(altSettings.getProperty("AltUnregisterRecipe", "true"));
				ALT_GAME_SHOW_DROPLIST = Boolean.parseBoolean(altSettings.getProperty("AltShowDroplist", "true"));
				ALT_GAME_GEN_DROPLIST_ON_DEMAND = Boolean.parseBoolean(altSettings.getProperty("AltGenerateDroplistOnDemand", "false"));
				ALLOW_NPC_SHIFTCLICK = Boolean.parseBoolean(altSettings.getProperty("AllowShiftClick", "true"));
				ALT_FULL_NPC_STATS_PAGE = Boolean.parseBoolean(altSettings.getProperty("AltFullStatsPage", "False"));
				ALT_GAME_SUBCLASS_WITHOUT_QUESTS = Boolean.parseBoolean(altSettings.getProperty("AltAllowSubClassWithoutQuest", "false"));
				ALT_GAME_LEVEL_TO_GET_SUBCLASS = Integer.parseInt(altSettings.getProperty("AltLevelToGetSubclass", "75"));
				ALT_GAME_SUB_ADD = Integer.parseInt(altSettings.getProperty("AltSubAdd", "0"));
				ALT_MAX_LEVEL = Math.min(Integer.parseInt(altSettings.getProperty("AltMaxLevel", "85")), Experience.LEVEL.length - 1);
				ALT_MAX_SUB_LEVEL = Math.min(Integer.parseInt(altSettings.getProperty("AltMaxSubLevel", "80")), Experience.LEVEL.length - 1);
				ALT_ALLOW_OTHERS_WITHDRAW_FROM_CLAN_WAREHOUSE = Boolean.parseBoolean(altSettings.getProperty("AltAllowOthersWithdrawFromClanWarehouse", "true"));
				ALT_GAME_REQUIRE_CLAN_CASTLE = Boolean.parseBoolean(altSettings.getProperty("AltRequireClanCastle", "false"));
				ALT_GAME_REQUIRE_CASTLE_DAWN = Boolean.parseBoolean(altSettings.getProperty("AltRequireCastleDawn", "true"));
				ALT_ADD_RECIPES = Integer.parseInt(altSettings.getProperty("AltAddRecipes", "0"));
				ALT_100_RECIPES = Boolean.parseBoolean(altSettings.getProperty("Alt100PercentRecipes", "false"));
				SS_ANNOUNCE_PERIOD = Integer.parseInt(altSettings.getProperty("SSAnnouncePeriod", "0"));
				ENABLE_FISHING_CHECK_WATER = Boolean.parseBoolean(altSettings.getProperty("EnableFishingWaterCheck", "true"));
				AUTO_LEARN_SKILLS = Boolean.parseBoolean(altSettings.getProperty("AutoLearnSkills", "false"));
				ENABLE_OLYMPIAD = Boolean.parseBoolean(altSettings.getProperty("EnableOlympiad", "false"));
				ALT_OLY_START_TIME = Integer.parseInt(altSettings.getProperty("AltOlyStartTime", "18"));
				ALT_OLY_MIN = Integer.parseInt(altSettings.getProperty("AltOlyMin", "00"));
				ALT_OLY_CPERIOD = Long.parseLong(altSettings.getProperty("AltOlyCPeriod", "21600000"));
				ALT_OLY_BATTLE = Long.parseLong(altSettings.getProperty("AltOlyBattle", "360000"));
				ALT_OLY_BWAIT = Long.parseLong(altSettings.getProperty("AltOlyBWait", "600000"));
				ALT_OLY_IWAIT = Long.parseLong(altSettings.getProperty("AltOlyIWait", "300000"));
				ALT_OLY_WPERIOD = Long.parseLong(altSettings.getProperty("AltOlyWPeriod", "604800000"));
				ALT_OLY_VPERIOD = Long.parseLong(altSettings.getProperty("AltOlyVPeriod", "43200000"));
				ALT_HONOR_SYSTEM = Boolean.parseBoolean(altSettings.getProperty("HonorSystem", "False"));
				ALT_SOCIAL_ACTION_REUSE = Boolean.parseBoolean(altSettings.getProperty("AltSocialActionReuse", "False"));
				ALT_DISABLE_SPELLBOOKS = Boolean.parseBoolean(altSettings.getProperty("AltDisableSpellbooks", "False"));
				ALT_DISABLE_EGGS = Boolean.parseBoolean(altSettings.getProperty("AltDisableEggs", "False"));
				ALT_SIMPLE_SIGNS = Boolean.parseBoolean(altSettings.getProperty("PushkinSignsOptions", "false"));
				ALT_MAMMOTH_UPGRADE = Integer.parseInt(altSettings.getProperty("MammonUpgrade", "2"));
				ALT_MAMMOTH_EXCHANGE = Integer.parseInt(altSettings.getProperty("MammonExchange", "2"));
				ALT_ALLOW_TATTOO = Boolean.parseBoolean(altSettings.getProperty("AllowTattoo", "false"));
				ALT_BUFF_LIMIT = Integer.parseInt(altSettings.getProperty("BuffLimit", "20"));
				ALT_DEATH_PENALTY = Boolean.parseBoolean(altSettings.getProperty("EnableAltDeathPenalty", "false"));
				ALLOW_DEATH_PENALTY_C5 = Boolean.parseBoolean(altSettings.getProperty("EnableDeathPenaltyC5", "true"));
				ALT_DEATH_PENALTY_C5_CHANCE = Integer.parseInt(altSettings.getProperty("DeathPenaltyC5Chance", "10"));
				ALT_DEATH_PENALTY_C5_CHAOTIC_RECOVERY = Boolean.parseBoolean(altSettings.getProperty("ChaoticCanUseScrollOfRecovery", "false"));
				ALT_DEATH_PENALTY_C5_EXPERIENCE_PENALTY = Integer.parseInt(altSettings.getProperty("DeathPenaltyC5RateExpPenalty", "1"));
				ALT_DEATH_PENALTY_C5_KARMA_PENALTY = Integer.parseInt(altSettings.getProperty("DeathPenaltyC5RateKarma", "1"));
				ALT_PK_DEATH_RATE = Float.parseFloat(altSettings.getProperty("AltPKDeathRate", "0."));
				NONOWNER_ITEM_PICKUP_DELAY = Integer.parseInt(altSettings.getProperty("NonOwnerItemPickupDelay", "15"));
				LIM_PATK_SPD = Integer.parseInt(altSettings.getProperty("LimitPatkSpd", "0"));
				LIM_MATK_SPD = Integer.parseInt(altSettings.getProperty("LimitMAtkSpd", "0"));
				LIM_CRIT = Integer.parseInt(altSettings.getProperty("LimitCritical", "0"));
				LIM_MCRIT = Integer.parseInt(altSettings.getProperty("LimitMCritical", "20")) * 10;

				FESTIVAL_MIN_PARTY_SIZE = Integer.parseInt(altSettings.getProperty("FestivalMinPartySize", "5"));

				RIFT_MIN_PARTY_SIZE = Integer.parseInt(altSettings.getProperty("RiftMinPartySize", "5"));
				RIFT_MAX_JUMPS = Integer.parseInt(altSettings.getProperty("MaxRiftJumps", "4"));
				RIFT_AUTO_JUMPS_TIME = Integer.parseInt(altSettings.getProperty("AutoJumpsDelay", "8"));
				RIFT_AUTO_JUMPS_TIME_RAND = Integer.parseInt(altSettings.getProperty("AutoJumpsDelayRandom", "120000"));

				RIFT_ENTER_COST_RECRUIT = Integer.parseInt(altSettings.getProperty("RecruitFC", "18"));
				RIFT_ENTER_COST_SOLDIER = Integer.parseInt(altSettings.getProperty("SoldierFC", "21"));
				RIFT_ENTER_COST_OFFICER = Integer.parseInt(altSettings.getProperty("OfficerFC", "24"));
				RIFT_ENTER_COST_CAPTAIN = Integer.parseInt(altSettings.getProperty("CaptainFC", "27"));
				RIFT_ENTER_COST_COMMANDER = Integer.parseInt(altSettings.getProperty("CommanderFC", "30"));
				RIFT_ENTER_COST_HERO = Integer.parseInt(altSettings.getProperty("HeroFC", "33"));
				RIFT_BOSS_ROOM_TIME_MUTIPLY = Float.parseFloat(altSettings.getProperty("BossRoomTimeMultiply", "1.5"));
				ALLOW_CLANSKILLS = Boolean.parseBoolean(altSettings.getProperty("AllowClanSkills", "true"));
				ALLOW_LEARN_TRANS_SKILLS_WO_QUEST = Boolean.parseBoolean(altSettings.getProperty("AllowLearnTransSkillsWOQuest", "false"));
				PARTY_LEADER_ONLY_CAN_INVITE = Boolean.parseBoolean(altSettings.getProperty("PartyLeaderOnlyCanInvite", "True"));
				ALLOW_TALK_WHILE_SITTING = Boolean.parseBoolean(altSettings.getProperty("AllowTalkWhileSitting", "True"));
				ALLOW_NOBLE_TP_TO_ALL = Boolean.parseBoolean(altSettings.getProperty("AllowNobleTPToAll", "False"));

				ALLOW_FAKE_PLAYERS = Boolean.parseBoolean(altSettings.getProperty("AllowFakePlayers", "False"));
				FAKE_PLAYERS_PERCENT = Integer.parseInt(altSettings.getProperty("FakePlayersPercent", "100"));
				BUFFTIME_MODIFIER = Float.parseFloat(altSettings.getProperty("BuffTimeModifier", "1.0"));
				SONGDANCETIME_MODIFIER = Float.parseFloat(altSettings.getProperty("SongDanceTimeModifier", "1.0"));
				MAXLOAD_MODIFIER = Float.parseFloat(altSettings.getProperty("MaxLoadModifier", "1.0"));

				OLD_SHUTDOWN_MSG = Boolean.parseBoolean(altSettings.getProperty("AltOldShutdownMsg", "True"));
				ALT_MAX_ALLY_SIZE = Integer.parseInt(altSettings.getProperty("AltMaxAllySize", "3"));
				ALT_PARTY_DISTRIBUTION_RANGE = Integer.parseInt(altSettings.getProperty("AltPartyDistributionRange", "1500"));

				DaysBeforeCreateAClan = Integer.parseInt(altSettings.getProperty("DaysBeforeCreateAClan", "1"));
				HoursBeforeJoinAClan = Integer.parseInt(altSettings.getProperty("HoursBeforeJoinAClan", "24"));
				AltClanMembersForWar = Integer.parseInt(altSettings.getProperty("AltClanMembersForWar", "15"));
				AltMinClanLvlForWar = Integer.parseInt(altSettings.getProperty("AltMinClanLvlForWar", "3"));
				AltClanWarMax = Integer.parseInt(altSettings.getProperty("AltClanWarMax", "30"));
				DaysBeforeCreateNewAllyWhenDissolved = Integer.parseInt(altSettings.getProperty("DaysBeforeCreateNewAllyWhenDissolved", "1"));
				MinLevelToCreatePledge = Integer.parseInt(altSettings.getProperty("MinLevelToCreatePledge", "10"));
				MemberForLevel6 = Integer.parseInt(altSettings.getProperty("MemberForLevel6", "30"));
				MemberForLevel7 = Integer.parseInt(altSettings.getProperty("MemberForLevel7", "80"));
				MemberForLevel8 = Integer.parseInt(altSettings.getProperty("MemberForLevel8", "120"));
				MemberForLevel9 = Integer.parseInt(altSettings.getProperty("MemberForLevel9", "120"));
				MemberForLevel10 = Integer.parseInt(altSettings.getProperty("MemberForLevel10", "140"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + ALT_SETTINGS_FILE + " File.");
			}

			// services settings
			try
			{
				Properties servicesSettings = new Properties();
				InputStream is = new FileInputStream(new File(SERVICES_FILE));
				servicesSettings.load(is);
				is.close();

				ALLOW_CLASS_MASTERS = servicesSettings.getProperty("AllowClassMasters", "0");
				if(ALLOW_CLASS_MASTERS.length() != 0 && !ALLOW_CLASS_MASTERS.equals("0"))
					for(String id : ALLOW_CLASS_MASTERS.split(","))
						ALLOW_CLASS_MASTERS_LIST.add(Integer.parseInt(id));
				CLASS_MASTERS_PRICE = servicesSettings.getProperty("ClassMastersPrice", "0,0,0");
				if(CLASS_MASTERS_PRICE.length() >= 5)
				{
					int level = 1;
					for(String id : CLASS_MASTERS_PRICE.split(","))
					{
						CLASS_MASTERS_PRICE_LIST[level] = Integer.parseInt(id);
						level++;
					}
				}
				CLASS_MASTERS_PRICE_ITEM = Integer.parseInt(servicesSettings.getProperty("ClassMastersPriceItem", "57"));

				SERVICES_CHANGE_NICK_ENABLED = Boolean.parseBoolean(servicesSettings.getProperty("NickChangeEnabled", "False"));
				SERVICES_CHANGE_NICK_PRICE = Integer.parseInt(servicesSettings.getProperty("NickChangePrice", "100"));
				SERVICES_CHANGE_NICK_ITEM = Integer.parseInt(servicesSettings.getProperty("NickChangeItem", "4037"));

				SERVICES_CHANGE_NICK_COLOR_ENABLED = Boolean.parseBoolean(servicesSettings.getProperty("NickColorChangeEnabled", "False"));
				SERVICES_CHANGE_NICK_COLOR_PRICE = Integer.parseInt(servicesSettings.getProperty("NickColorChangePrice", "100"));
				SERVICES_CHANGE_NICK_COLOR_ITEM = Integer.parseInt(servicesSettings.getProperty("NickColorChangeItem", "4037"));
				SERVICES_CHANGE_NICK_COLOR_LIST = servicesSettings.getProperty("NickColorChangeList", "00FF00").split(";");

				SERVICES_BASH_ENABLED = Boolean.parseBoolean(servicesSettings.getProperty("BashEnabled", "False"));
				SERVICES_BASH_SKIP_DOWNLOAD = Boolean.parseBoolean(servicesSettings.getProperty("BashSkipDownload", "False"));
				SERVICES_BASH_RELOAD_TIME = Integer.parseInt(servicesSettings.getProperty("BashReloadTime", "24"));

				SERVICES_RATE_BONUS_ENABLED = Boolean.parseBoolean(servicesSettings.getProperty("RateBonusEnabled", "False"));
				SERVICES_RATE_BONUS_PRICE = Integer.parseInt(servicesSettings.getProperty("RateBonusPrice", "1500"));
				SERVICES_RATE_BONUS_ITEM = Integer.parseInt(servicesSettings.getProperty("RateBonusItem", "4037"));
				SERVICES_RATE_BONUS_VALUE = Float.parseFloat(servicesSettings.getProperty("RateBonusValue", "1.25"));
				SERVICES_RATE_BONUS_DAYS = Integer.parseInt(servicesSettings.getProperty("RateBonusTime", "30"));
				SERVICES_RATE_BONUS_LUCK_EFFECT = Float.parseFloat(servicesSettings.getProperty("RateBonusLuckEffect", "1.0"));

				SERVICES_NOBLESS_SELL_ENABLED = Boolean.parseBoolean(servicesSettings.getProperty("NoblessSellEnabled", "False"));
				SERVICES_NOBLESS_SELL_PRICE = Integer.parseInt(servicesSettings.getProperty("NoblessSellPrice", "1000"));
				SERVICES_NOBLESS_SELL_ITEM = Integer.parseInt(servicesSettings.getProperty("NoblessSellItem", "4037"));

				SERVICES_HOW_TO_GET_COL = Boolean.parseBoolean(servicesSettings.getProperty("HowToGetCoL", "False"));

				SERVICES_OFFLINE_TRADE_ALLOW = Boolean.parseBoolean(servicesSettings.getProperty("AllowOfflineTrade", "False"));
				SERVICES_OFFLINE_TRADE_NAME_COLOR = Integer.decode("0x" + servicesSettings.getProperty("OfflineTradeNameColor", "B0FFFF"));
				SERVICES_OFFLINE_TRADE_KICK_NOT_TRADING = Boolean.parseBoolean(servicesSettings.getProperty("KickOfflineNotTrading", "True"));
				SERVICES_OFFLINE_TRADE_PRICE_ITEM = Integer.parseInt(servicesSettings.getProperty("OfflineTradePriceItem", "0"));
				SERVICES_OFFLINE_TRADE_PRICE = Integer.parseInt(servicesSettings.getProperty("OfflineTradePrice", "0"));
				SERVICES_OFFLINE_TRADE_DAYS_TO_KICK = Integer.parseInt(servicesSettings.getProperty("OfflineTradeDaysToKick", "14"));
				SERVICES_OFFLINE_TRADE_RESTORE_AFTER_RESTART = Boolean.parseBoolean(servicesSettings.getProperty("OfflineRestoreAfterRestart", "True"));

				SERVICES_NO_TRADE_ONLY_OFFLINE = Boolean.parseBoolean(servicesSettings.getProperty("NoTradeOnlyOffline", "False"));
				SERVICES_TRADE_TAX = Float.parseFloat(servicesSettings.getProperty("TradeTax", "0.0"));
				SERVICES_OFFSHORE_TRADE_TAX = Float.parseFloat(servicesSettings.getProperty("OffshoreTradeTax", "0.0"));
				SERVICES_TRADE_TAX_ONLY_OFFLINE = Boolean.parseBoolean(servicesSettings.getProperty("TradeTaxOnlyOffline", "False"));
				SERVICES_OFFSHORE_NO_CASTLE_TAX = Boolean.parseBoolean(servicesSettings.getProperty("NoCastleTaxInOffshore", "False"));
				SERVICES_TRADE_RADIUS = Integer.parseInt(servicesSettings.getProperty("TradeRadius", "30"));
				SERVICES_TRADE_ONLY_FAR = Boolean.parseBoolean(servicesSettings.getProperty("TradeOnlyFar", "False"));

				SERVICES_GIRAN_HARBOR_ENABLED = Boolean.parseBoolean(servicesSettings.getProperty("GiranHarborZone", "False"));

				SERVICES_LOCK_ACCOUNT_IP = Boolean.parseBoolean(servicesSettings.getProperty("LockAccountIP", "False"));

				SERVICES_CHANGE_PASSWORD = Boolean.parseBoolean(servicesSettings.getProperty("ChangePassword", "True"));

				SERVICES_DONATE_SHOP_ENABLED = Boolean.parseBoolean(servicesSettings.getProperty("DonateShop", "False"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + SERVICES_FILE + " File.");
			}

			//PVPCOMMUNITBOARD
           try
			{
                Properties communityboardpvpSettings = new Properties();
				InputStream is = new FileInputStream(new File(PVPCB_FILE));
				communityboardpvpSettings.load(is);
				is.close();

            ALLOW_PVPCB_ABNORMAL = Boolean.parseBoolean(communityboardpvpSettings.getProperty( "AllowBBSAbnormal", "false"));
            ALLOW_PVPCB_BUFFER = Boolean.parseBoolean(communityboardpvpSettings.getProperty( "AllowBBSBuffer", "true"));
            ALLOW_PVPCB_SHOP = Boolean.parseBoolean(communityboardpvpSettings.getProperty( "AllowBBSShop", "true"));
            ALLOW_PVPCB_ECHANT = Boolean.parseBoolean(communityboardpvpSettings.getProperty( "AllowBBSEnchant", "true"));
            ALLOW_PVPCB_CLASSMASTER = Boolean.parseBoolean(communityboardpvpSettings.getProperty( "AllowBBSClassMaster", "true"));
            ALLOW_PVPCB_TELEPORT = Boolean.parseBoolean(communityboardpvpSettings.getProperty( "AllowBBSTeleport", "true"));
            PVPCB_BUFFER_ALT_TIME = Long.parseLong(communityboardpvpSettings.getProperty( "AltBBSBufferTime", "14400000"));
            PVPCB_BUFFER_PRICE_ITEM = Integer.parseInt(communityboardpvpSettings.getProperty( "AltBBSBufferPriceItem", "57"));
            PVPCB_BUFFER_PRICE_ONE = Integer.parseInt(communityboardpvpSettings.getProperty( "AltBBSBufferPriceOne", "20000"));
            PVPCB_BUFFER_PRICE_MOD_GRP = Integer.parseInt(communityboardpvpSettings.getProperty( "AltBBSBufferPriceModGrp"," 10"));
            PVPCB_BUFFER_MIN_LVL = Integer.parseInt(communityboardpvpSettings.getProperty( "AltBBSBufferMinLvl", "1"));
            PVPCB_BUFFER_MAX_LVL =Integer.parseInt(communityboardpvpSettings.getProperty( "AltBBSBufferMaxLvl", "99"));
            PVPCB_BUFFER_ALLOW_SIEGE = Boolean.parseBoolean(communityboardpvpSettings.getProperty( "AltBBSBufferAllowOnSiege", "true"));
            PVPCB_BUFFER_ALLOW_SAVE_RESTOR = Boolean.parseBoolean(communityboardpvpSettings.getProperty( "AltBBSBufferAllowSaveRestor", "true"));
            ALT_CB_ENCH_ITEM = Integer.parseInt(communityboardpvpSettings.getProperty( "AltBBSEnchantPriceItem", "4356"));
            }
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + PVP_CONFIG_FILE + " File.");
			}
            
            // pvp config
			try
			{
				Properties pvpSettings = new Properties();
				InputStream is = new FileInputStream(new File(PVP_CONFIG_FILE));
				pvpSettings.load(is);
				is.close();

				/* KARMA SYSTEM */
				KARMA_MIN_KARMA = Integer.parseInt(pvpSettings.getProperty("MinKarma", "240"));
				KARMA_SP_DIVIDER = Integer.parseInt(pvpSettings.getProperty("SPDivider", "7"));
				KARMA_LOST_BASE = Integer.parseInt(pvpSettings.getProperty("BaseKarmaLost", "0"));

				KARMA_DROP_GM = Boolean.parseBoolean(pvpSettings.getProperty("CanGMDropEquipment", "false"));
				KARMA_NEEDED_TO_DROP = Boolean.parseBoolean(pvpSettings.getProperty("KarmaNeededToDrop", "True"));

				KARMA_NONDROPPABLE_ITEMS = pvpSettings.getProperty("ListOfNonDroppableItems", "57,1147,425,1146,461,10,2368,7,6,2370,2369,3500,3501,3502,4422,4423,4424,2375,6648,6649,6650,6842,6834,6835,6836,6837,6838,6839,6840,5575,7694,6841,8181");

				KARMA_DROP_ITEM_LIMIT = Integer.parseInt(pvpSettings.getProperty("MaxItemsDroppable", "10"));
				MIN_PK_TO_ITEMS_DROP = Integer.parseInt(pvpSettings.getProperty("MinPKToDropItems", "5"));

				KARMA_RANDOM_DROP_LOCATION_LIMIT = Integer.parseInt(pvpSettings.getProperty("MaxDropThrowDistance", "70"));

				KARMA_DROPCHANCE_MINIMUM = Integer.parseInt(pvpSettings.getProperty("ChanceOfDropMinimum", "1"));
				KARMA_DROPCHANCE_MULTIPLIER = Integer.parseInt(pvpSettings.getProperty("ChanceOfDropMultiplier", "1"));
				KARMA_DROPCHANCE_EQUIPMENT = Integer.parseInt(pvpSettings.getProperty("ChanceOfDropEquipped", "20"));
				KARMA_DROPCHANCE_EQUIPPED_WEAPON = Integer.parseInt(pvpSettings.getProperty("ChanceOfDropEquippedWeapon", "25"));

				KARMA_LIST_NONDROPPABLE_ITEMS = new ArrayList<Integer>();
				for(String id : KARMA_NONDROPPABLE_ITEMS.split(","))
					KARMA_LIST_NONDROPPABLE_ITEMS.add(Integer.parseInt(id));

				PVP_TIME = Integer.parseInt(pvpSettings.getProperty("PvPTime", "40000"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + PVP_CONFIG_FILE + " File.");
			}

			// AI
			try
			{
				Properties aiSettings = new Properties();
				InputStream is = new FileInputStream(new File(AI_CONFIG_FILE));
				aiSettings.load(is);
				is.close();

				MOBSLOOTERS = Boolean.parseBoolean(aiSettings.getProperty("MonstersLooters", "true"));

				RND_WALK = Boolean.valueOf(aiSettings.getProperty("RndWalk", "False"));
				RND_WALK_RATE = Integer.parseInt(aiSettings.getProperty("RndWalkRate", "2"));
				RND_ANIMATION_RATE = Integer.parseInt(aiSettings.getProperty("RndAnimationRate", "3"));

				MAX_DRIFT_RANGE = Integer.parseInt(aiSettings.getProperty("MaxDriftRange", "300"));
				MAX_PURSUE_RANGE = Integer.parseInt(aiSettings.getProperty("MaxPursueRange", "10000"));

				ALT_AI_KELTIRS = Boolean.parseBoolean(aiSettings.getProperty("AltAiKeltirs", "False"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + AI_CONFIG_FILE + " File.");
			}

			// siege options
			try
			{
				Properties siegeSettings = new Properties();
				InputStream is = new FileInputStream(new File(SIEGE_CASTLE_CONFIGURATION_FILE));
				siegeSettings.load(is);
				is.close();

				SIEGE_OPERATE_DOORS = Boolean.parseBoolean(siegeSettings.getProperty("OperateDoors", "True"));
				SIEGE_OPERATE_DOORS_LORD_ONLY = Boolean.parseBoolean(siegeSettings.getProperty("OperateDoorsLordOnly", "True"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + SIEGE_CASTLE_CONFIGURATION_FILE + " File.");
			}

			// hexid
			try
			{
				Properties Settings = new Properties();
				InputStream is = new FileInputStream(HEXID_FILE);
				Settings.load(is);
				is.close();
				HEX_ID = new BigInteger(Settings.getProperty("HexID"), 16).toByteArray();
			}
			catch(Exception e)
			{}

			// Events config
			try
			{
				Properties EventSettings = new Properties();
				InputStream is = new FileInputStream(new File(EVENTS));
				EventSettings.load(is);
				is.close();

				EVENT_Min_lvl = Integer.parseInt(EventSettings.getProperty("Min_lvl", "1"));
				EVENT_Max_lvl = Integer.parseInt(EventSettings.getProperty("Max_lvl", "85"));
				EVENT_price = Integer.parseInt(EventSettings.getProperty("price", "5000"));
				EVENT_Buffer_Siege = Boolean.parseBoolean(EventSettings.getProperty("Buffer_Siege", "False"));

				EVENT_CofferOfShadowsPriceRate = Integer.parseInt(EventSettings.getProperty("CofferOfShadowsPriceRate", "1"));
				EVENT_CofferOfShadowsRewardRate = Float.parseFloat(EventSettings.getProperty("CofferOfShadowsRewardRate", "1."));
				EVENT_BonusItemID = Integer.parseInt(EventSettings.getProperty("BONUS_ITEM", "4037"));
				EVENT_BonusPrice = Integer.parseInt(EventSettings.getProperty("BONUS_PRICE", "50"));
				EVENT_BonusRate = Integer.parseInt(EventSettings.getProperty("BONUS_RATE", "50"));
				EVENT_BonusDays = Integer.parseInt(EventSettings.getProperty("BONUS_DAYS", "7"));

				EVENT_ClassmastersSellsSS = Boolean.parseBoolean(EventSettings.getProperty("CM_SellSS", "False"));
				EVENT_ClassmastersCoLShop = Boolean.parseBoolean(EventSettings.getProperty("CM_CoLShop", "False"));

				EVENT_LastHeroItemID = Integer.parseInt(EventSettings.getProperty("LastHero_bonus_id", "57"));
				EVENT_LastHeroItemCOUNT = Integer.parseInt(EventSettings.getProperty("LastHero_bonus_count", "5000"));
				EVENT_LastHeroTime = Integer.parseInt(EventSettings.getProperty("LastHero_time", "5"));
				EVENT_LastHeroRate = Boolean.parseBoolean(EventSettings.getProperty("LastHero_rate", "True"));

				TFH_POLLEN_CHANCE = Integer.parseInt(EventSettings.getProperty("TFH_POLLEN_CHANCE", "5"));

				GLIT_MEDAL_CHANCE = Integer.parseInt(EventSettings.getProperty("MEDAL_CHANCE", "10"));
				GLIT_GLITTMEDAL_CHANCE = Integer.parseInt(EventSettings.getProperty("GLITTMEDAL_CHANCE", "5"));
				GLIT_EnableRate = Boolean.parseBoolean(EventSettings.getProperty("GLIT_EnableRate", "True"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + EVENTS + " File.");
			}

			abuseLoad();
			loadGMAccess();
			_log.info("loading xml GMAccess");
			if(ADVIPSYSTEM)
				ipsLoad();
		}
		else if(Server.SERVER_MODE == Server.MODE_LOGINSERVER)
		{
			_log.info("loading login config");
			try
			{
				Properties serverSettings = new Properties();
				InputStream is = new FileInputStream(new File(LOGIN_CONFIGURATION_FILE));
				serverSettings.load(is);
				is.close();

				loadAntiFlood(serverSettings);
				LOGIN_HOST = serverSettings.getProperty("LoginserverHostname", "127.0.0.1");
				GAME_SERVER_LOGIN_PORT = Integer.parseInt(serverSettings.getProperty("LoginPort", "9013"));
				GAME_SERVER_LOGIN_HOST = serverSettings.getProperty("LoginHost", "127.0.0.1");
				PORT_LOGIN = Integer.parseInt(serverSettings.getProperty("LoginserverPort", "2106"));

				COMBO_MODE = Boolean.parseBoolean(serverSettings.getProperty("ComboMode", "false"));
				LOGIN_DEBUG = Boolean.parseBoolean(serverSettings.getProperty("Debug", "false"));

				ACCEPT_NEW_GAMESERVER = Boolean.parseBoolean(serverSettings.getProperty("AcceptNewGameServer", "True"));

				LOGIN_TRY_BEFORE_BAN = Integer.parseInt(serverSettings.getProperty("LoginTryBeforeBan", "10"));
				LOGIN_PING = Boolean.parseBoolean(serverSettings.getProperty("PingServer", "True"));
				LOGIN_PING_TIME = Integer.parseInt(serverSettings.getProperty("WaitPingTime", "5"));

				DATABASE_DRIVER = serverSettings.getProperty("Driver", "com.mysql.jdbc.Driver");
				DATABASE_URL = serverSettings.getProperty("URL", "jdbc:mysql://localhost/l2rdb");
				DATABASE_LOGIN = serverSettings.getProperty("Login", "root");
				DATABASE_PASSWORD = serverSettings.getProperty("Password", "");
				DATABASE_MAX_CONNECTIONS = Integer.parseInt(serverSettings.getProperty("MaximumDbConnections", "10"));

				SHOW_LICENCE = Boolean.parseBoolean(serverSettings.getProperty("ShowLicence", "true"));
				SQL_LOG = Boolean.parseBoolean(serverSettings.getProperty("SqlLog", "false"));
				AUTO_CREATE_ACCOUNTS = Boolean.parseBoolean(serverSettings.getProperty("AutoCreateAccounts", "true"));
				IP_UPDATE_TIME = Integer.parseInt(serverSettings.getProperty("IpUpdateTime", "15"));
				ANAME_TEMPLATE = serverSettings.getProperty("AnameTemplate", "[A-Za-z0-9]{3,14}");
				APASSWD_TEMPLATE = serverSettings.getProperty("ApasswdTemplate", "[A-Za-z0-9]{5,16}");
				LOGIN_GG_CHECK = Boolean.parseBoolean(serverSettings.getProperty("GGCheck", "True"));
				LRESTART_TIME = Integer.parseInt(serverSettings.getProperty("AutoRestart", "-1"));

				String internalIpList = serverSettings.getProperty("InternalIpList", "127.0.0.1,192.168.0.0-192.168.255.255,10.0.0.0-10.255.255.255,172.16.0.0-172.16.31.255");
				if(internalIpList.startsWith("NetList@"))
				{
					INTERNAL_NETLIST = new NetList();
					INTERNAL_NETLIST.LoadFromFile(internalIpList.replaceFirst("NetList@", ""));
					_log.info("Loaded " + INTERNAL_NETLIST.NetsCount() + " Internal Nets");
				}
				else
				{
					INTERNAL_IP = new FastList<String>();
					INTERNAL_IP.addAll(Arrays.asList(internalIpList.split(",")));
				}

			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + LOGIN_CONFIGURATION_FILE + " File.");
			}

			//	      telnet
			try
			{
				Properties telnetSettings = new Properties();
				InputStream is = new FileInputStream(new File(LOGIN_TELNET_FILE));
				telnetSettings.load(is);
				is.close();

				IS_LOGIN_TELNET_ENABLED = Boolean.valueOf(telnetSettings.getProperty("EnableTelnet", "false"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + TELNET_FILE + " File.");
			}
		}
		else
			_log.severe("Could not Load Config: server mode was not set");

	}

	public static boolean allowL2Walker(L2Player player)
	{
		return ALLOW_L2WALKER_CLIENT == L2WalkerAllowed.True || player.getPlayerAccess().AllowWalker || ALLOW_L2WALKER_CLIENT == L2WalkerAllowed.Peace && player.isInZone(peace_zone) && player.getLevel() <= MAX_L2WALKER_LEVEL;
	}

	// it has no instancies
	private Config()
	{}

	public static void saveHexid(String string)
	{
		saveHexid(string, HEXID_FILE);
	}

	public static void saveHexid(String string, String fileName)
	{
		try
		{
			Properties hexSetting = new Properties();
			File file = new File(fileName);
			file.createNewFile();
			OutputStream out = new FileOutputStream(file);
			hexSetting.setProperty("HexID", string);
			hexSetting.store(out, "the hexID to auth into login");
			out.close();
		}
		catch(Exception e)
		{
			System.out.println("Failed to save hex id to " + fileName + " File.");
			e.printStackTrace();
		}
	}

	public static void abuseLoad()
	{
		MAT_LIST = new ArrayList<String>();

		LineNumberReader lnr = null;
		try
		{
			int i = 0;
			String line;

			lnr = new LineNumberReader(new InputStreamReader(new FileInputStream(Server.RebellionHomeDir + MAT_CONFIG_FILE), "UTF-8"));

			while((line = lnr.readLine()) != null)
			{
				StringTokenizer st = new StringTokenizer(line, "\n\r");
				if(st.hasMoreTokens())
				{
					String Mat = st.nextToken();
					MAT_LIST.add(Mat);

					i++;
				}
			}

			_log.info("Abuse: Loaded " + i + " abuse words.");

		}
		catch(IOException e1)
		{
			_log.warning("Error reading abuse: " + e1);
		}
		finally
		{
			try
			{
				if(lnr != null)
					lnr.close();
			}
			catch(Exception e2)
			{
				// nothing
			}
		}
	}

	private static void ipsLoad()
	{
		try
		{
			Properties ipsSettings = new Properties();
			InputStream is = new FileInputStream(new File(ADV_IP_FILE));
			ipsSettings.load(is);
			is.close();

			String NetMask;
			String ip;
			for(int i = 0; i < ipsSettings.size() / 2; i++)
			{
				NetMask = ipsSettings.getProperty("NetMask" + (i + 1));
				ip = ipsSettings.getProperty("IPAdress" + (i + 1));
				for(String mask : NetMask.split(","))
				{
					AdvIP advip = new AdvIP();
					advip.ipadress = ip;
					advip.ipmask = mask.split("/")[0];
					advip.bitmask = mask.split("/")[1];
					GAMEIPS.add(advip);
				}
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new Error("Failed to Load " + ADV_IP_FILE + " File.");
		}
	}

	public static void loadGMAccess()
	{
		gmlist.clear();
		loadGMAccess(new File(GM_PERSONAL_ACCESS_FILE));
		File dir = new File("./config/GMAccess.d/");
		if(!dir.exists())
		{
			_log.config("Dir " + dir.getAbsolutePath() + " not exists");
			return;
		}
		File[] files = dir.listFiles();
		for(File f : files)
			if(f.getName().endsWith(".xml"))
				loadGMAccess(f);
	}

	@SuppressWarnings("unchecked")
	public static void loadGMAccess(File file)
	{
		try
		{
			Field fld;
			//File file = new File(filename);
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setValidating(false);
			factory.setIgnoringComments(true);
			Document doc = factory.newDocumentBuilder().parse(file);

			for(Node z = doc.getFirstChild(); z != null; z = z.getNextSibling())
				for(Node n = z.getFirstChild(); n != null; n = n.getNextSibling())
				{
					if(!n.getNodeName().equalsIgnoreCase("char"))
						continue;

					PlayerAccess pa = new PlayerAccess();
					for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
					{
						Class cls = pa.getClass();
						String node = d.getNodeName();

						if(node.equalsIgnoreCase("#text"))
							continue;
						try
						{
							fld = cls.getField(node);
						}
						catch(NoSuchFieldException e)
						{
							_log.info("Not found desclarate ACCESS name: " + node + " in XML Player access Object");
							continue;
						}

						if(fld.getType().getName().equalsIgnoreCase("boolean"))
							fld.setBoolean(pa, Boolean.parseBoolean(d.getAttributes().getNamedItem("set").getNodeValue()));
						else if(fld.getType().getName().equalsIgnoreCase("int"))
							fld.setInt(pa, Integer.valueOf(d.getAttributes().getNamedItem("set").getNodeValue()));
					}
					gmlist.put(pa.PlayerID, pa);
				}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new Error("Failed to Load " + GM_PERSONAL_ACCESS_FILE + " File.");
		}
	}
}
