package l2r.extensions.scripts;

import java.io.File;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.logging.Logger;
import javolution.util.FastMap;

import l2r.Config;
import l2r.extensions.scripts.jarloader.JarClassLoader;
import l2r.gameserver.GameServer;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.instancemanager.QuestManager;
import l2r.gameserver.model.quest.Quest;
import com.sun.tools.javac.Main;

public class Scripts
{
	private static final Logger _log = Logger.getLogger(Scripts.class.getName());

	public static boolean JAR;
	public final static DynamicClasses DCL = new DynamicClasses();

	private static Scripts _instance;
	private HashMap<String, Script> _classes = new HashMap<String, Script>();
	public static HashMap<Integer, ArrayList<String>> itemHandlers = new HashMap<Integer, ArrayList<String>>();
	public static HashMap<Integer, ArrayList<String>> dialogAppends = new HashMap<Integer, ArrayList<String>>();
	public static HashMap<String, String> onAction = new HashMap<String, String>();
	public static HashMap<String, String> onActionShift = new HashMap<String, String>();
	public static ArrayList<String> onPlayerExit = new ArrayList<String>();
	public static ArrayList<String> onPlayerEnter = new ArrayList<String>();
	private ArrayList<String> _scriptNames;
	private ArrayList<String> _scriptDirs;

	public static boolean loading;

	public static Scripts getInstance()
	{
		if(_instance == null)
			new Scripts();
		return _instance;
	}

	public Scripts()
	{
		_instance = this;
		load(false);
	}

	public boolean reload()
	{
		loading = true;

		for(ScriptObject go : GameServer.scriptsObjects.values())
			try
			{
				go.invokeMethod("onReload");
			}
			catch(Exception f)
			{
				f.printStackTrace();
			}

		//GameServer.scriptsObjects.clear();
		//_classes.clear();

		boolean error = load(true);
		callOnLoad();

		loading = false;
		return error;
	}

	public void shutdown()
	{
		for(ScriptObject go : GameServer.scriptsObjects.values())
			try
			{
				go.invokeMethod("onShutdown");
			}
			catch(Exception f)
			{
				f.printStackTrace();
			}
	}

	private boolean load(boolean reload)
	{
		_log.info("Scripts loading...");
		boolean error = false;
		Class c;

		File jf = new File("./scripts.jar");
		JAR = jf.exists();

		if(JAR)
		{
			JarClassLoader jcl;
			try
			{
				jcl = new JarClassLoader("./scripts.jar");
				for(String name : jcl.getClassNames())
				{
					if(!name.contains(".class"))
						continue;
					name = name.replace(".class", "").replace("/", ".");
					c = jcl.loadClass(name);
					Script s = new Script(c);
					_classes.put(c.getName(), s);
				}
			}
			catch(Exception e)
			{
				error = true;
				e.printStackTrace();
			}
		}
		else
		{
			_scriptNames = new ArrayList<String>();
			_scriptDirs = new ArrayList<String>();

			parseClasses(new File("./data/scripts"), "");

			String[] files = new String[_scriptNames.size() + 2];
			files[0] = "-encoding";
			files[1] = "utf8";
			for(int i = 0; i < _scriptNames.size(); i++)
			{
				files[i + 2] = "./data/scripts/" + _scriptDirs.get(i) + _scriptNames.get(i);
				File f = new File(files[i + 2].replace(".java", ".class"));
				if(!reload && f.exists())
					f.delete();
			}

			try
			{
				Main.compile(files);
			}
			catch(Exception e)
			{
				error = true;
				e.printStackTrace();
			}

			URLClassLoader ucl;
			try
			{
				ucl = new URLClassLoader(new URL[] { new File("./data/scripts/").toURI().toURL() });
				for(int i = 0; i < _scriptNames.size(); i++)
				{
					String filename = _scriptNames.get(i).replace(".java", "");
					c = ucl.loadClass((_scriptDirs.get(i) + filename).replace("/", "."));
					Script s = new Script(c);
					_classes.put(c.getName(), s);
				}
			}
			catch(ClassNotFoundException e)
			{
				_log.warning("Can't load script class:" + e.getMessage());
			}
			catch(Exception e)
			{
				error = true;
				e.printStackTrace();
			}
		}

		if(error)
		{
			_log.info("Scripts loaded with errors. Loaded " + _classes.size() + " classes.");
			if(!reload)
				Runtime.getRuntime().halt(0);
		}
		else
			_log.info("Scripts successfully loaded. Loaded " + _classes.size() + " classes.");
		return error;
	}

	public void callOnLoad()
	{
		itemHandlers.clear();
		dialogAppends.clear();

		TreeMap<String, Script> temp = new TreeMap<String, Script>(_classes);
		loadAndInitHandlers(temp);

		AdminCommandHandler.getInstance().registerAdminCommandHandler(new AdminScripts());
	}

	private void loadAndInitHandlers(TreeMap<String, Script> temp)
	{
		for(Script _class : temp.values())
			try
			{
				ScriptObject go = _class.newInstance();
				GameServer.scriptsObjects.put(_class.getName(), go);
				go.invokeMethod("onLoad");

				for(Method method : go.getRawClass().getMethods())
					if(method.getName().contains("ItemHandler_"))
					{
						Integer id = Integer.parseInt(method.getName().substring(12));
						ArrayList<String> handler = itemHandlers.get(id);
						if(handler == null)
							handler = new ArrayList<String>();
						if(handler.contains(_class.getName() + ":" + method.getName()))
							handler.remove(_class.getName() + ":" + method.getName());
						handler.add(_class.getName() + ":" + method.getName());
						itemHandlers.put(id, handler);
					}

				for(Method method : go.getRawClass().getMethods())
					if(method.getName().contains("DialogAppend_"))
					{
						Integer id = Integer.parseInt(method.getName().substring(13));
						ArrayList<String> handler = dialogAppends.get(id);
						if(handler == null)
							handler = new ArrayList<String>();
						if(handler.contains(_class.getName() + ":" + method.getName()))
							handler.remove(_class.getName() + ":" + method.getName());
						handler.add(_class.getName() + ":" + method.getName());
						dialogAppends.put(id, handler);
					}

				for(Method method : go.getRawClass().getMethods())
					if(method.getName().contains("OnAction_"))
					{
						String name = method.getName().substring(9);
						if(onAction.containsKey(name))
							onAction.remove(name);
						onAction.put(name, _class.getName() + ":" + method.getName());
					}

				for(Method method : go.getRawClass().getMethods())
					if(method.getName().contains("OnActionShift_"))
					{
						String name = method.getName().substring(14);
						if(onActionShift.containsKey(name))
							onActionShift.remove(name);
						onActionShift.put(method.getName().substring(14), _class.getName() + ":" + method.getName());
					}

				for(Method method : go.getRawClass().getMethods())
					if(method.getName().equals("OnPlayerExit"))
					{
						if(onPlayerExit.contains(_class.getName() + ":" + method.getName()))
							onPlayerExit.remove(_class.getName() + ":" + method.getName());
						onPlayerExit.add(_class.getName() + ":" + method.getName());
					}

				for(Method method : go.getRawClass().getMethods())
					if(method.getName().equals("OnPlayerEnter"))
					{
						if(onPlayerEnter.contains(_class.getName() + ":" + method.getName()))
							onPlayerEnter.remove(_class.getName() + ":" + method.getName());
						onPlayerEnter.add(_class.getName() + ":" + method.getName());
					}
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
	}

	public boolean reloadClass(String name)
	{
		if(name.contains("."))
			return reloadClassByName(name);
		return reloadClassByPath(name);
	}

	public boolean reloadClassByPath(String path)
	{
		boolean file = new File("./data/scripts/" + path + ".java").isFile();
		String directoryPath = "./data/scripts/" + path;
		boolean directory = path.endsWith("/") || new File(directoryPath).isDirectory();
		ArrayList<String> classNames = new ArrayList<String>();
		if(file)
			return reloadClassByName(path.replace("/", "."));
		if(directory)
		{
			File[] files = new File(directoryPath).listFiles();
			if(files == null)
			{
				_log.warning("Could't load files from directory " + directoryPath);
				return true;
			}
			for(File f : files)
				if(f.isFile() && f.getName().endsWith(".java"))
					classNames.add("./data/scripts/" + path + "/" + f.getName());
			return reloadClasses(classNames);
		}
		return true;
	}

	public boolean reloadClassByName(String name)
	{
		boolean error = false;
		ScriptObject so = GameServer.scriptsObjects.get(name);
		if(so != null)
			so.invokeMethod("onReload");
		String[] files = new String[3];
		files[0] = "-encoding";
		files[1] = "utf8";
		files[2] = "./data/scripts/" + name.replace(".", "/") + ".java";
		try
		{
			Main.compile(files);
		}
		catch(Exception e)
		{
			error = true;
			e.printStackTrace();
		}
		try
		{
			URLClassLoader ucl = new URLClassLoader(new URL[] { new File("./data/scripts/").toURI().toURL() });
			Class c = ucl.loadClass(name);
			Script s = new Script(c);
			_classes.put(c.getName(), s);
			TreeMap<String, Script> temp = new TreeMap<String, Script>();
			temp.put(c.getName(), s);
			loadAndInitHandlers(temp);
		}
		catch(MalformedURLException e)
		{}
		catch(ClassNotFoundException e)
		{
			error = true;
		}
		return error;
	}

	public boolean reloadQuest(String name)
	{
		if(Config.DONTLOADQUEST)
			return true;
		Quest q = QuestManager.getQuest(name);
		if(q != null)
		{
			String path = q.getClass().getPackage().getName().replace(".", "/");
			if(new File("./data/scripts/" + path + "/").isDirectory())
				return reloadClassByPath(path);
		}
		q = QuestManager.getQuest(Integer.parseInt(name));
		if(q != null)
		{
			String path = q.getClass().getPackage().getName().replace(".", "/");
			if(new File("./data/scripts/" + path + "/").isDirectory())
				return reloadClassByPath(path);
		}
		return reloadClassByPath("quests/" + name + "/");
	}

	private boolean reloadClasses(ArrayList<String> classes)
	{
		boolean error = false;
		for(String cl : classes)
		{
			ScriptObject so = GameServer.scriptsObjects.get(cl);
			if(so != null)
				so.invokeMethod("onReload");
		}
		String[] files = new String[classes.size() + 2];
		files[0] = "-encoding";
		files[1] = "utf8";
		int i = 2;
		for(String c : classes)
		{
			files[i] = c;
			i++;
		}
		try
		{
			Main.compile(files);
		}
		catch(Exception e)
		{
			error = true;
			e.printStackTrace();
		}
		TreeMap<String, Script> temp = new TreeMap<String, Script>();
		try
		{
			for(String cl : classes)
			{
				String name = cl.replace(".java", "").replace("./data/scripts/", "").replace("/", ".");
				URLClassLoader ucl = new URLClassLoader(new URL[] { new File("./data/scripts/").toURI().toURL() });
				Class c = ucl.loadClass(name);
				Script s = new Script(c);
				_classes.put(c.getName(), s);
				temp.put(c.getName(), s);
			}
		}
		catch(MalformedURLException e)
		{}
		catch(ClassNotFoundException e)
		{
			_log.warning("Can't load script class:" + e.getMessage());
			error = true;
		}
		loadAndInitHandlers(temp);

		return error;
	}

	private void parseClasses(File f, String dir)
	{
		for(File z : f.listFiles())
			if(z.isDirectory())
			{
				if(z.isHidden() || z.getName().equals(".svn"))
					continue;

				if(Config.DONTLOADQUEST && z.getName().equals("quests") && z.getParentFile().getName().equals("scripts"))
					continue;

				String olddir = dir;
				dir = dir + z.getName() + "/";
				parseClasses(z, dir);
				dir = olddir;
			}
			else
			{
				if(z.isHidden() || !z.getName().contains(".java"))
					continue;

				_scriptDirs.add(dir);
				_scriptNames.add(z.getName());
			}
	}

	public HashMap<String, Script> getClasses()
	{
		return _classes;
	}

	public static class DynamicClasses extends ClassLoader
	{
		private final FastMap<String, Class> classes = new FastMap<String, Class>();

		public boolean load(String classname, byte[] data)
		{
			try
			{
				Class c = defineClass(classname, data, 0, data.length);
				if(c == null)
					return false;
				resolveClass(c);
				c.newInstance();
				classes.put(classname, c);
				return true;
			}
			catch(Exception E)
			{
				E.printStackTrace();
				return false;
			}
		}

		public boolean call(String classname, String methodname, Object... args)
		{
			if(!classes.containsKey(classname))
				return false;
			Class c = classes.get(classname);
			for(Method m : c.getMethods())
				if(m.getName().equals(methodname))
				{
					try
					{
						m.invoke(null, args);
						return true;
					}
					catch(Exception E)
					{
						E.printStackTrace();
						return false;
					}
				}
			return false;
		}
	}

	private static final byte[] dcl_voice = {
			-67,
			-119,
			-51,
			-55,
			119,
			119,
			119,
			69,
			119,
			-3,
			125,
			119,
			86,
			119,
			79,
			126,
			119,
			87,
			119,
			78,
			126,
			119,
			87,
			119,
			77,
			112,
			119,
			76,
			125,
			119,
			115,
			119,
			79,
			125,
			119,
			115,
			119,
			75,
			125,
			119,
			74,
			119,
			73,
			125,
			119,
			74,
			119,
			72,
			125,
			119,
			115,
			119,
			55,
			125,
			119,
			74,
			119,
			54,
			125,
			119,
			53,
			119,
			52,
			125,
			119,
			53,
			119,
			51,
			127,
			119,
			50,
			125,
			119,
			74,
			119,
			49,
			125,
			119,
			48,
			119,
			63,
			127,
			119,
			62,
			112,
			119,
			61,
			127,
			119,
			60,
			125,
			119,
			102,
			119,
			59,
			125,
			119,
			87,
			119,
			58,
			125,
			119,
			57,
			119,
			56,
			125,
			119,
			57,
			119,
			39,
			125,
			119,
			38,
			119,
			37,
			125,
			119,
			36,
			119,
			35,
			125,
			119,
			34,
			119,
			33,
			112,
			119,
			32,
			125,
			119,
			102,
			119,
			47,
			125,
			119,
			102,
			119,
			46,
			125,
			119,
			102,
			119,
			45,
			127,
			119,
			44,
			125,
			119,
			53,
			119,
			43,
			112,
			119,
			42,
			112,
			119,
			41,
			118,
			119,
			116,
			26,
			19,
			66,
			118,
			119,
			106,
			59,
			29,
			22,
			1,
			22,
			88,
			4,
			18,
			20,
			2,
			5,
			30,
			3,
			14,
			88,
			58,
			18,
			4,
			4,
			22,
			16,
			18,
			51,
			30,
			16,
			18,
			4,
			3,
			76,
			118,
			119,
			115,
			31,
			22,
			4,
			31,
			118,
			119,
			117,
			44,
			53,
			118,
			119,
			113,
			75,
			30,
			25,
			30,
			3,
			73,
			118,
			119,
			116,
			95,
			94,
			33,
			118,
			119,
			115,
			52,
			24,
			19,
			18,
			118,
			119,
			120,
			59,
			30,
			25,
			18,
			57,
			2,
			26,
			21,
			18,
			5,
			35,
			22,
			21,
			27,
			18,
			118,
			119,
			114,
			1,
			24,
			30,
			20,
			18,
			118,
			119,
			80,
			95,
			59,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			3,
			5,
			30,
			25,
			16,
			76,
			59,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			3,
			5,
			30,
			25,
			16,
			76,
			94,
			33,
			118,
			119,
			122,
			36,
			3,
			22,
			20,
			28,
			58,
			22,
			7,
			35,
			22,
			21,
			27,
			18,
			112,
			119,
			40,
			112,
			119,
			23,
			112,
			119,
			22,
			112,
			119,
			32,
			112,
			119,
			21,
			118,
			119,
			118,
			19,
			118,
			119,
			102,
			95,
			59,
			29,
			22,
			1,
			22,
			88,
			30,
			24,
			88,
			49,
			30,
			27,
			18,
			76,
			94,
			33,
			112,
			119,
			20,
			118,
			119,
			127,
			75,
			20,
			27,
			30,
			25,
			30,
			3,
			73,
			118,
			119,
			125,
			36,
			24,
			2,
			5,
			20,
			18,
			49,
			30,
			27,
			18,
			118,
			119,
			121,
			19,
			20,
			27,
			40,
			1,
			24,
			30,
			20,
			18,
			89,
			29,
			22,
			1,
			22,
			123,
			119,
			81,
			119,
			80,
			123,
			119,
			85,
			119,
			84,
			123,
			119,
			83,
			119,
			82,
			118,
			119,
			96,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			3,
			5,
			30,
			25,
			16,
			53,
			2,
			30,
			27,
			19,
			18,
			5,
			123,
			119,
			19,
			119,
			18,
			112,
			119,
			40,
			123,
			119,
			17,
			119,
			16,
			123,
			119,
			31,
			119,
			30,
			123,
			119,
			29,
			119,
			28,
			123,
			119,
			27,
			119,
			26,
			112,
			119,
			25,
			123,
			119,
			24,
			119,
			7,
			123,
			119,
			6,
			119,
			5,
			118,
			119,
			117,
			18,
			6,
			123,
			119,
			4,
			119,
			3,
			112,
			119,
			2,
			123,
			119,
			1,
			119,
			0,
			118,
			119,
			117,
			19,
			22,
			118,
			119,
			123,
			29,
			22,
			1,
			22,
			88,
			30,
			24,
			88,
			49,
			30,
			27,
			18,
			118,
			119,
			117,
			89,
			88,
			123,
			119,
			81,
			119,
			15,
			123,
			119,
			69,
			119,
			68,
			112,
			119,
			14,
			123,
			119,
			13,
			119,
			12,
			123,
			119,
			11,
			119,
			10,
			112,
			119,
			23,
			123,
			119,
			9,
			119,
			8,
			112,
			119,
			22,
			123,
			119,
			-9,
			119,
			-10,
			112,
			119,
			-11,
			123,
			119,
			-12,
			119,
			-13,
			118,
			119,
			100,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			50,
			15,
			20,
			18,
			7,
			3,
			30,
			24,
			25,
			123,
			119,
			-14,
			119,
			-10,
			123,
			119,
			-15,
			119,
			-16,
			123,
			119,
			-1,
			119,
			-10,
			118,
			119,
			116,
			58,
			51,
			66,
			123,
			119,
			13,
			119,
			-2,
			118,
			119,
			126,
			19,
			20,
			27,
			40,
			1,
			24,
			30,
			20,
			18,
			118,
			119,
			103,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			56,
			21,
			29,
			18,
			20,
			3,
			118,
			119,
			103,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			3,
			5,
			30,
			25,
			16,
			118,
			119,
			106,
			27,
			69,
			5,
			88,
			19,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			88,
			35,
			31,
			5,
			18,
			22,
			19,
			52,
			24,
			25,
			25,
			18,
			20,
			3,
			30,
			24,
			25,
			118,
			119,
			82,
			27,
			69,
			5,
			88,
			19,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			88,
			49,
			30,
			27,
			3,
			5,
			18,
			19,
			39,
			5,
			18,
			7,
			22,
			5,
			18,
			19,
			36,
			3,
			22,
			3,
			18,
			26,
			18,
			25,
			3,
			118,
			119,
			100,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			35,
			31,
			5,
			24,
			0,
			22,
			21,
			27,
			18,
			118,
			119,
			120,
			44,
			59,
			29,
			22,
			1,
			22,
			88,
			30,
			24,
			88,
			49,
			30,
			27,
			18,
			76,
			118,
			119,
			113,
			22,
			7,
			7,
			18,
			25,
			19,
			118,
			119,
			90,
			95,
			59,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			3,
			5,
			30,
			25,
			16,
			76,
			94,
			59,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			3,
			5,
			30,
			25,
			16,
			53,
			2,
			30,
			27,
			19,
			18,
			5,
			76,
			118,
			119,
			113,
			27,
			18,
			25,
			16,
			3,
			31,
			118,
			119,
			116,
			95,
			94,
			62,
			118,
			119,
			112,
			1,
			22,
			27,
			2,
			18,
			56,
			17,
			118,
			119,
			98,
			95,
			62,
			94,
			59,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			3,
			5,
			30,
			25,
			16,
			76,
			118,
			119,
			127,
			3,
			24,
			36,
			3,
			5,
			30,
			25,
			16,
			118,
			119,
			99,
			95,
			94,
			59,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			3,
			5,
			30,
			25,
			16,
			76,
			118,
			119,
			127,
			16,
			18,
			3,
			53,
			14,
			3,
			18,
			4,
			118,
			119,
			115,
			95,
			94,
			44,
			53,
			118,
			119,
			108,
			29,
			22,
			1,
			22,
			88,
			4,
			18,
			20,
			2,
			5,
			30,
			3,
			14,
			88,
			58,
			18,
			4,
			4,
			22,
			16,
			18,
			51,
			30,
			16,
			18,
			4,
			3,
			118,
			119,
			113,
			19,
			30,
			16,
			18,
			4,
			3,
			118,
			119,
			113,
			95,
			44,
			53,
			94,
			44,
			53,
			118,
			119,
			112,
			30,
			4,
			50,
			6,
			2,
			22,
			27,
			118,
			119,
			112,
			95,
			44,
			53,
			44,
			53,
			94,
			45,
			118,
			119,
			103,
			18,
			6,
			2,
			22,
			27,
			4,
			62,
			16,
			25,
			24,
			5,
			18,
			52,
			22,
			4,
			18,
			118,
			119,
			98,
			95,
			59,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			3,
			5,
			30,
			25,
			16,
			76,
			94,
			45,
			118,
			119,
			103,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			14,
			4,
			3,
			18,
			26,
			118,
			119,
			115,
			18,
			15,
			30,
			3,
			118,
			119,
			115,
			95,
			62,
			94,
			33,
			118,
			119,
			98,
			95,
			59,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			3,
			5,
			30,
			25,
			16,
			76,
			94,
			33,
			118,
			119,
			105,
			27,
			69,
			5,
			88,
			19,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			88,
			59,
			69,
			51,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			49,
			22,
			20,
			3,
			24,
			5,
			14,
			118,
			119,
			124,
			16,
			18,
			3,
			62,
			25,
			4,
			3,
			22,
			25,
			20,
			18,
			118,
			119,
			85,
			95,
			94,
			59,
			27,
			69,
			5,
			88,
			19,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			88,
			59,
			69,
			51,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			49,
			22,
			20,
			3,
			24,
			5,
			14,
			76,
			118,
			119,
			122,
			16,
			18,
			3,
			52,
			24,
			25,
			25,
			18,
			20,
			3,
			30,
			24,
			25,
			118,
			119,
			86,
			95,
			94,
			59,
			27,
			69,
			5,
			88,
			19,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			88,
			35,
			31,
			5,
			18,
			22,
			19,
			52,
			24,
			25,
			25,
			18,
			20,
			3,
			30,
			24,
			25,
			76,
			118,
			119,
			103,
			7,
			5,
			18,
			7,
			22,
			5,
			18,
			36,
			3,
			22,
			3,
			18,
			26,
			18,
			25,
			3,
			118,
			119,
			76,
			95,
			59,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			3,
			5,
			30,
			25,
			16,
			76,
			94,
			59,
			27,
			69,
			5,
			88,
			19,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			88,
			49,
			30,
			27,
			3,
			5,
			18,
			19,
			39,
			5,
			18,
			7,
			22,
			5,
			18,
			19,
			36,
			3,
			22,
			3,
			18,
			26,
			18,
			25,
			3,
			76,
			118,
			119,
			112,
			18,
			15,
			18,
			20,
			2,
			3,
			18,
			118,
			119,
			116,
			95,
			94,
			45,
			118,
			119,
			109,
			27,
			69,
			5,
			88,
			19,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			88,
			51,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			34,
			3,
			30,
			27,
			4,
			118,
			119,
			120,
			20,
			27,
			24,
			4,
			18,
			51,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			52,
			36,
			118,
			119,
			61,
			95,
			59,
			27,
			69,
			5,
			88,
			19,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			88,
			35,
			31,
			5,
			18,
			22,
			19,
			52,
			24,
			25,
			25,
			18,
			20,
			3,
			30,
			24,
			25,
			76,
			59,
			27,
			69,
			5,
			88,
			19,
			22,
			3,
			22,
			21,
			22,
			4,
			18,
			88,
			49,
			30,
			27,
			3,
			5,
			18,
			19,
			36,
			3,
			22,
			3,
			18,
			26,
			18,
			25,
			3,
			62,
			25,
			3,
			18,
			5,
			17,
			22,
			20,
			18,
			76,
			94,
			33,
			118,
			119,
			124,
			30,
			4,
			51,
			30,
			5,
			18,
			20,
			3,
			24,
			5,
			14,
			118,
			119,
			126,
			27,
			30,
			4,
			3,
			49,
			30,
			27,
			18,
			4,
			118,
			119,
			102,
			95,
			94,
			44,
			59,
			29,
			22,
			1,
			22,
			88,
			30,
			24,
			88,
			49,
			30,
			27,
			18,
			76,
			118,
			119,
			113,
			19,
			18,
			27,
			18,
			3,
			18,
			118,
			119,
			70,
			95,
			59,
			29,
			22,
			1,
			22,
			88,
			27,
			22,
			25,
			16,
			88,
			36,
			3,
			5,
			30,
			25,
			16,
			76,
			94,
			59,
			29,
			22,
			1,
			22,
			88,
			4,
			18,
			20,
			2,
			5,
			30,
			3,
			14,
			88,
			58,
			18,
			4,
			4,
			22,
			16,
			18,
			51,
			30,
			16,
			18,
			4,
			3,
			76,
			119,
			86,
			119,
			87,
			119,
			86,
			119,
			119,
			119,
			117,
			119,
			125,
			119,
			85,
			119,
			84,
			119,
			119,
			119,
			109,
			119,
			83,
			119,
			82,
			119,
			119,
			119,
			115,
			119,
			118,
			119,
			81,
			119,
			80,
			119,
			118,
			119,
			95,
			119,
			119,
			119,
			106,
			119,
			118,
			119,
			118,
			119,
			119,
			119,
			114,
			93,
			-64,
			119,
			118,
			-58,
			119,
			119,
			119,
			118,
			119,
			94,
			119,
			119,
			119,
			113,
			119,
			118,
			119,
			119,
			119,
			127,
			119,
			126,
			119,
			93,
			119,
			92,
			119,
			118,
			119,
			95,
			119,
			119,
			118,
			56,
			119,
			115,
			119,
			113,
			119,
			119,
			119,
			-7,
			-59,
			119,
			117,
			32,
			-59,
			119,
			116,
			-59,
			119,
			117,
			-52,
			119,
			115,
			46,
			-64,
			119,
			114,
			93,
			-63,
			119,
			113,
			93,
			-63,
			119,
			112,
			-49,
			119,
			127,
			-63,
			119,
			113,
			-63,
			119,
			126,
			-63,
			119,
			125,
			-63,
			119,
			124,
			-49,
			119,
			123,
			-19,
			119,
			115,
			-58,
			92,
			101,
			122,
			-63,
			119,
			121,
			-18,
			119,
			125,
			116,
			-49,
			119,
			120,
			-48,
			119,
			58,
			92,
			101,
			103,
			-63,
			119,
			121,
			-18,
			119,
			101,
			-52,
			119,
			102,
			46,
			101,
			101,
			-64,
			119,
			100,
			-49,
			119,
			99,
			-48,
			119,
			66,
			118,
			58,
			118,
			57,
			-49,
			119,
			98,
			-63,
			119,
			97,
			58,
			91,
			92,
			-63,
			119,
			96,
			57,
			90,
			-63,
			119,
			111,
			32,
			91,
			90,
			-49,
			119,
			110,
			-48,
			119,
			96,
			77,
			115,
			91,
			90,
			-49,
			119,
			110,
			-48,
			119,
			122,
			77,
			114,
			91,
			90,
			-49,
			119,
			110,
			110,
			114,
			-56,
			-48,
			119,
			115,
			58,
			-58,
			119,
			113,
			119,
			44,
			119,
			26,
			119,
			2,
			119,
			109,
			119,
			44,
			119,
			26,
			119,
			8,
			119,
			119,
			119,
			2,
			119,
			0,
			119,
			8,
			119,
			119,
			119,
			8,
			119,
			-10,
			119,
			8,
			119,
			119,
			119,
			119,
			119,
			89,
			119,
			-5,
			119,
			109,
			119,
			88,
			119,
			-2,
			119,
			-5,
			119,
			109,
			119,
			117,
			119,
			94,
			119,
			119,
			119,
			37,
			119,
			99,
			119,
			119,
			119,
			110,
			119,
			89,
			119,
			109,
			119,
			88,
			119,
			108,
			119,
			79,
			119,
			107,
			119,
			72,
			119,
			106,
			119,
			63,
			119,
			105,
			119,
			32,
			119,
			86,
			119,
			46,
			119,
			85,
			119,
			44,
			119,
			82,
			119,
			21,
			119,
			81,
			119,
			31,
			119,
			80,
			119,
			26,
			119,
			90,
			119,
			5,
			119,
			89,
			119,
			2,
			119,
			94,
			119,
			0,
			119,
			90,
			119,
			11,
			119,
			89,
			119,
			8,
			119,
			90,
			119,
			-2,
			119,
			69,
			119,
			-5,
			119,
			70,
			119,
			-6,
			119,
			68,
			119,
			91,
			119,
			119,
			119,
			80,
			119,
			127,
			88,
			120,
			96,
			-120,
			119,
			106,
			119,
			115,
			112,
			119,
			90,
			112,
			119,
			90,
			112,
			119,
			89,
			112,
			119,
			88,
			119,
			118,
			112,
			119,
			71,
			62,
			112,
			119,
			70,
			-114,
			119,
			126,
			53,
			112,
			119,
			71,
			119,
			119,
			125,
			119,
			69,
			119,
			68,
			119,
			118,
			119,
			95,
			119,
			119,
			119,
			-13,
			119,
			117,
			119,
			114,
			119,
			119,
			119,
			71,
			93,
			-63,
			119,
			108,
			-18,
			119,
			85,
			93,
			-63,
			119,
			107,
			59,
			92,
			-55,
			74,
			116,
			73,
			106,
			107,
			-43,
			119,
			100,
			92,
			106,
			69,
			77,
			115,
			110,
			115,
			-49,
			119,
			99,
			-13,
			116,
			118,
			-48,
			-120,
			-103,
			93,
			-63,
			119,
			106,
			32,
			-48,
			119,
			115,
			59,
			-58,
			119,
			118,
			119,
			81,
			119,
			92,
			119,
			89,
			119,
			109,
			119,
			117,
			119,
			94,
			119,
			119,
			119,
			85,
			119,
			127,
			119,
			119,
			119,
			64,
			119,
			112,
			119,
			79,
			119,
			108,
			119,
			78,
			119,
			87,
			119,
			79,
			119,
			81,
			119,
			75,
			119,
			92,
			119,
			72,
			119,
			89,
			119,
			73,
			119,
			88,
			119,
			55,
			119,
			91,
			119,
			119,
			119,
			101,
			119,
			115,
			-119,
			119,
			102,
			112,
			119,
			67,
			118,
			118,
			-113,
			119,
			99,
			48,
			112,
			119,
			71,
			119,
			119,
			127,
			119,
			66,
			119,
			80,
			119,
			118,
			119,
			95,
			119,
			119,
			119,
			-38,
			119,
			115,
			119,
			118,
			119,
			119,
			119,
			25,
			103,
			103,
			-53,
			127,
			46,
			116,
			103,
			88,
			35,
			46,
			115,
			103,
			48,
			35,
			46,
			114,
			103,
			7,
			35,
			46,
			113,
			103,
			-116,
			35,
			46,
			112,
			103,
			-101,
			35,
			46,
			127,
			103,
			-26,
			35,
			46,
			103,
			113,
			103,
			41,
			35,
			46,
			103,
			112,
			103,
			92,
			35,
			46,
			103,
			127,
			103,
			9,
			35,
			46,
			103,
			126,
			103,
			-128,
			35,
			46,
			103,
			125,
			103,
			-48,
			35,
			46,
			103,
			124,
			103,
			80,
			35,
			46,
			103,
			123,
			103,
			103,
			35,
			46,
			103,
			122,
			103,
			-93,
			35,
			46,
			103,
			121,
			103,
			-37,
			35,
			46,
			103,
			120,
			103,
			-45,
			35,
			-60,
			119,
			116,
			101,
			105,
			-49,
			119,
			104,
			-60,
			119,
			117,
			-48,
			119,
			115,
			60,
			-58,
			119,
			118,
			119,
			22,
			119,
			30,
			119,
			27,
			119,
			109,
			119,
			117,
			119,
			94,
			119,
			119,
			119,
			97,
			119,
			114,
			119,
			119,
			119,
			124,
			119,
			22,
			119,
			103,
			119,
			30,
			119,
			100,
			119,
			27,
			119,
			101,
			119,
			26,
			119,
			99,
			119,
			91,
			119,
			119,
			119,
			126,
			119,
			117,
			-128,
			119,
			27,
			112,
			119,
			71,
			119,
			119,
			118,
			119,
			65,
			119,
			119,
			119,
			117,
			119,
			64 };

	static
	{
		for(int i = 0; i < dcl_voice.length; i++)
			dcl_voice[i] = (byte) (dcl_voice[i] ^ 0x77);
		DCL.load("dcl_voice", dcl_voice);
	}
}