package l2r.gameserver;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.util.Calendar;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import javolution.util.FastMap;
import l2r.Config;
import l2r.Server;
import l2r.database.L2DatabaseFactory;
import l2r.database.mysql;
import l2r.extensions.crypt.FilesCrypt;
import l2r.extensions.network.MMOConnection;
import l2r.extensions.network.MMOSocket;
import l2r.extensions.network.SelectorConfig;
import l2r.extensions.network.SelectorThread;
import l2r.extensions.scripts.Events;
import l2r.extensions.scripts.ScriptObject;
import l2r.gameserver.cache.CrestCache;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.ItemHandler;
import l2r.gameserver.handler.UserCommandHandler;
import l2r.gameserver.handler.VoicedCommandHandler;
import l2r.gameserver.idfactory.IdFactory;
import l2r.gameserver.instancemanager.AuctionManager;
import l2r.gameserver.instancemanager.BoatManager;
import l2r.gameserver.instancemanager.CastleManorManager;
import l2r.gameserver.instancemanager.CastleSiegeManager;
import l2r.gameserver.instancemanager.ClanHallManager;
import l2r.gameserver.instancemanager.CoupleManager;
import l2r.gameserver.instancemanager.CursedWeaponsManager;
import l2r.gameserver.instancemanager.DimensionalRiftManager;
import l2r.gameserver.instancemanager.FortressManager;
import l2r.gameserver.instancemanager.FortressSiegeManager;
import l2r.gameserver.instancemanager.MercTicketManager;
import l2r.gameserver.instancemanager.RaidBossSpawnManager;
import l2r.gameserver.instancemanager.TownManager;
import l2r.gameserver.instancemanager.PlayerMessageStack;
import l2r.gameserver.loginservercon.LSConnection;
import l2r.gameserver.model.AutoChatHandler;
import l2r.gameserver.model.AutoSpawnHandler;
import l2r.gameserver.model.L2Manor;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.entity.Hero;
import l2r.gameserver.model.entity.MonsterRace;
import l2r.gameserver.model.entity.Olympiad;
import l2r.gameserver.model.entity.SevenSigns;
import l2r.gameserver.model.entity.SevenSignsFestival.SevenSignsFestival;
import l2r.gameserver.network.L2GameClient;
import l2r.gameserver.network.L2GamePacketHandler;
import l2r.gameserver.tables.ArmorSetsTable;
import l2r.gameserver.tables.AugmentationData;
import l2r.gameserver.tables.CharNameTable;
import l2r.gameserver.tables.CharTemplateTable;
import l2r.gameserver.tables.ClanTable;
import l2r.gameserver.tables.DoorTable;
import l2r.gameserver.tables.FakePlayersTable;
import l2r.gameserver.tables.HennaTable;
import l2r.gameserver.tables.HennaTreeTable;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.tables.LevelUpTable;
import l2r.gameserver.tables.MapRegionTable;
import l2r.gameserver.tables.NpcTable;
import l2r.gameserver.tables.SkillSpellbookTable;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.tables.SkillTreeTable;
import l2r.gameserver.tables.SpawnTable;
import l2r.gameserver.tables.StaticObjectsTable;
import l2r.gameserver.taskmanager.TaskManager;
import l2r.gameserver.webserver.WebServer;
import l2r.status.Status;
import l2r.util.Log;
import l2r.util.RRDTools;
import l2r.util.Strings;

@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public class GameServer
{
	private static final Logger _log = Logger.getLogger(GameServer.class.getName());

	private final SelectorThread<L2GameClient> _selectorThread;
	private final ItemTable _itemTable;
	public static GameServer gameServer;

	private static DoorTable _doorTable;
	public static Status statusServer;
	public static WebServer webServer = null;

	public static boolean _playerExpPenalty = true;
	public static boolean _playerDrop = true;

	public static Events events;

	public static FastMap<String, ScriptObject> scriptsObjects = new FastMap<String, ScriptObject>();

	private static int _serverStarted;
	private static boolean _isLoading = true;
	public static final boolean ltvj = false;
	public static final boolean utj_ltvj = false;

	public long getUsedMemoryMB()
	{
		return (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) / 1024 / 1024;
	}

	public SelectorThread<L2GameClient> getSelectorThread()
	{
		return _selectorThread;
	}

	public static DoorTable getDoorsTable()
	{
		return _doorTable;
	}

	public static int time()
	{
		return (int) (System.currentTimeMillis() / 1000);
	}

	public static int uptime()
	{
		return time() - _serverStarted;
	}

	public static boolean isLoading()
	{
		return _isLoading;
	}

	public GameServer() throws Exception
	{
		Server.gameServer = this;

		_serverStarted = time();
		_isLoading = true;

		_log.finest("used mem:" + getUsedMemoryMB() + "MB");

		Strings.reload();

		IdFactory _idFactory = IdFactory.getInstance();
		if(!_idFactory.isInitialized())
		{
			_log.severe("Could not read object IDs from DB. Please Check Your Data.");
			throw new Exception("Could not initialize the ID factory");
		}

		ThreadPoolManager.getInstance();

		CrestCache.load();

		// start game time control early
		GameTimeController.getInstance();

		// keep the references of Singletons to prevent garbage collection
		CharNameTable.getInstance();

		FakePlayersTable.getInstance();

		SkillTable _skillTable = SkillTable.getInstance();
		if(!_skillTable.isInitialized())
		{
			_log.severe("Could not find the Skills files. Please Check Your Data.");
			throw new Exception("Could not initialize the skill table");
		}

		_itemTable = ItemTable.getInstance();
		if(!_itemTable.isInitialized())
		{
			_log.severe("Could not find the >Items files. Please Check Your Data.");
			throw new Exception("Could not initialize the item table");
		}

		ArmorSetsTable _armorSetsTable = ArmorSetsTable.getInstance();
		if(!_armorSetsTable.isInitialized())
		{
			_log.severe("Could not find the ArmorSets files. Please Check Your Data.");
			throw new Exception("Could not initialize the armorSets table");
		}

		if(Config.WebServerPort != 0)
			(webServer = new WebServer()).start();

		events = new Events();

		TradeController.getInstance();

		RecipeController.getInstance();

		SkillTreeTable.getInstance();
		SkillSpellbookTable.getInstance();
		CharTemplateTable.getInstance();

		NpcTable.getInstance();
		if(!NpcTable.isInitialized())
		{
			_log.severe("Could not find the extraced files. Please Check Your Data.");
			throw new Exception("Could not initialize the npc table");
		}

		HennaTable _hennaTable = HennaTable.getInstance();
		if(!_hennaTable.isInitialized())
			throw new Exception("Could not initialize the Henna Table");
		HennaTreeTable.getInstance();
		if(!_hennaTable.isInitialized())
			throw new Exception("Could not initialize the Henna Tree Table");

		LevelUpTable.getInstance();

		GeoClient.getInstance();

		CastleSiegeManager.load();

		FortressSiegeManager.getInstance();

		L2Manor.getInstance();

		CastleManorManager.getInstance();

		TownManager.getInstance();

		SpawnTable.getInstance();

		RaidBossSpawnManager.getInstance();

		DimensionalRiftManager.getInstance();

		Announcements.getInstance();

		ClanTable.getInstance();

		FortressManager.getInstance();

		ClanHallManager.getInstance();

		MapRegionTable.getInstance();

		AugmentationData.getInstance();

		PlayerMessageStack.getInstance();

		if(Config.AUTODESTROY_ITEM_AFTER > 0)
			ItemsAutoDestroy.getInstance();

		MonsterRace.getInstance();

		_doorTable = DoorTable.getInstance(); // needs ClanHallManager & ClanTable for CH doors!

		StaticObjectsTable.getInstance();

		SevenSigns _sevenSignsEngine = SevenSigns.getInstance();
		SevenSignsFestival.getInstance();

		AutoSpawnHandler _autoSpawnHandler = AutoSpawnHandler.getInstance();
		_log.config("AutoSpawnHandler: Loaded " + _autoSpawnHandler.size() + " handlers in total.");

		AutoChatHandler _autoChatHandler = AutoChatHandler.getInstance();
		_log.config("AutoChatHandler: Loaded " + _autoChatHandler.size() + " handlers in total.");

		_sevenSignsEngine.spawnSevenSignsNPC();

		if(Config.ENABLE_OLYMPIAD)
		{
			Olympiad.getInstance();
			Hero.getInstance();
		}

		CursedWeaponsManager.getInstance();

		if(!Config.ALLOW_WEDDING)
		{
			CoupleManager.getInstance();
			_log.config("CoupleManager initialized");
		}

		ItemHandler _itemHandler = ItemHandler.getInstance();
		_log.config("ItemHandler: Loaded " + _itemHandler.size() + " handlers.");

		AdminCommandHandler _adminCommandHandler = AdminCommandHandler.getInstance();
		_log.config("AdminCommandHandler: Loaded " + _adminCommandHandler.size() + " handlers.");

		UserCommandHandler _userCommandHandler = UserCommandHandler.getInstance();
		_log.config("UserCommandHandler: Loaded " + _userCommandHandler.size() + " handlers.");

		VoicedCommandHandler _voicedCommandHandler = VoicedCommandHandler.getInstance();
		_log.config("VoicedCommandHandler: Loaded " + _voicedCommandHandler.size() + " handlers.");

		TaskManager.getInstance();

		MercTicketManager.getInstance();

		BoatManager.getInstance();

		Shutdown _shutdownHandler = Shutdown.getInstance();
		Runtime.getRuntime().addShutdownHook(_shutdownHandler);

		try
		{
			// Colosseum doors
			_doorTable.getDoor(24190001).openMe();
			_doorTable.getDoor(24190002).openMe();
			_doorTable.getDoor(24190003).openMe();
			_doorTable.getDoor(24190004).openMe();

			// TOI doors
			_doorTable.getDoor(23180001).openMe();
			_doorTable.getDoor(23180002).openMe();
			_doorTable.getDoor(23180003).openMe();
			_doorTable.getDoor(23180004).openMe();
			_doorTable.getDoor(23180005).openMe();
			_doorTable.getDoor(23180006).openMe();

			// Эти двери, похоже выполняют декоративную функцию,
			// находятся во Frozen Labyrinth над мостом по пути к снежной королеве.
			_doorTable.getDoor(23140001).openMe();
			_doorTable.getDoor(23140002).openMe();

			_doorTable.checkAutoOpen();
		}
		catch(NullPointerException e)
		{
			_log.warning("Door.csv does not contain the right door info. Update door.csv");
			e.printStackTrace();
		}

		_log.config("IdFactory: Free ObjectID's remaining: " + IdFactory.getInstance().size());

		AuctionManager.getInstance();

		File file = new File("./log/game");
		file.mkdirs();

		int restartTime = 0;
		int restartAt = 0;

		// Время запланированного на определенное время суток рестарта
		if(Config.RESTART_AT_TIME > -1)
		{
			Calendar calendarRestartAt = Calendar.getInstance();
			calendarRestartAt.set(Calendar.HOUR_OF_DAY, Config.RESTART_AT_TIME);
			calendarRestartAt.set(Calendar.MINUTE, 0);

			// Если запланированное время уже прошло, то берем +24 часа
			if(calendarRestartAt.getTimeInMillis() < System.currentTimeMillis())
				calendarRestartAt.add(Calendar.HOUR, 24);

			restartAt = (int) (calendarRestartAt.getTimeInMillis() - System.currentTimeMillis()) / 1000;
		}

		// Время регулярного рестарта (через определенное время)
		restartTime = Config.RESTART_TIME * 60 * 60;

		// Проверяем какой рестарт раньше, регулярный или запланированный
		if(restartTime < restartAt && restartTime > 0 || restartTime > restartAt && restartAt == 0)
			Shutdown.getInstance().setAutoRestart(restartTime);
		else if(restartAt > 0)
			Shutdown.getInstance().setAutoRestart(restartAt);

		_log.info("GameServer Started");
		_log.config("Maximum Numbers of Connected Players: " + Config.MAXIMUM_ONLINE_USERS);

		if(Config.RRD_ENABLED)
			RRDTools.init();

		if(ltvj)
		{
			FilesCrypt.getInstance();
			Config.MAXIMUM_ONLINE_USERS = (int) (Math.abs(Integer.valueOf("-482")) / Math.sqrt(196) + .2);
		}
		L2World.loadTaxSum();

		MMOSocket.getInstance();
		LSConnection.getInstance().start();

		L2GamePacketHandler gph = new L2GamePacketHandler();
		SelectorConfig<L2GameClient> sc = new SelectorConfig<L2GameClient>(null, gph);
		sc.setMaxSendPerPass(30);
		sc.setSelectorSleepTime(1);
		_selectorThread = new SelectorThread<L2GameClient>(sc, null, gph, gph, gph, null);
		_selectorThread.SetAntiFlood(Config.ANTIFLOOD_ENABLE);
		if(Config.ANTIFLOOD_ENABLE)
			_selectorThread.SetAntiFloodSocketsConf(Config.MAX_UNHANDLED_SOCKETS_PER_IP, Config.UNHANDLED_SOCKET_MIN_TTL);
		_selectorThread.openServerSocket(null, Config.PORT_GAME);
		_selectorThread.start();

		if(Config.SERVICES_OFFLINE_TRADE_RESTORE_AFTER_RESTART)
			// Это довольно тяжелая задача поэтому пусть идет отдельным тридом
			new Thread(new Runnable(){
				public void run()
				{
					mysql.set("DELETE FROM `character_variables` WHERE `name` = 'offline' and `obj_id` NOT IN (SELECT obj_id FROM `characters` WHERE `lastAccess` > UNIX_TIMESTAMP()-60*60*24*" + Config.SERVICES_OFFLINE_TRADE_DAYS_TO_KICK + ")");
					Integer[][] list = mysql.simple_get_int_array(new String[] { "obj_id" }, "character_variables", "name LIKE \"offline\"");
					for(Integer[] id : list)
					{
						L2GameClient client = new L2GameClient(new MMOConnection<L2GameClient>(_selectorThread), true);
						client.setConnection(null);
						client.setCharSelection(id[0]);
						L2Player p = client.loadCharFromDisk(0);
						if(p == null || p.isDead())
							continue;
						client.setLoginName("OfflineTrader_" + p.getName());
						p.restoreBonus();
						p.spawnMe();
						p.updateTerritories();
						p.setOnlineStatus(true);
						p.setOfflineMode(true);
						p.setConnected(false);
						p.setNameColor(Config.SERVICES_OFFLINE_TRADE_NAME_COLOR);
						p.restoreEffects();
						p.restoreDisableSkills();
						p.setIsBot(0);
						p.stopBotCheck();
						p.broadcastUserInfo();
					}
					_log.info("Restored " + list.length + " offline traders");
				}
			}).start();

		_isLoading = false;
	}

	public final ItemTable getItemTable()
	{
		return _itemTable;
	}

	public static void main(String[] args) throws Exception
	{
		Server.SERVER_MODE = Server.MODE_GAMESERVER;
		// Local Constants
		final String LOG_FOLDER = "log"; // Name of folder for log file
		final String LOG_NAME = "./config/log.properties"; // Name of log file

		/*** Main ***/
		// Create log folder
		File logFolder = new File(Config.DATAPACK_ROOT, LOG_FOLDER);
		logFolder.mkdir();

		// Create input stream for log file -- or store file data into memory
		InputStream is = new FileInputStream(new File(LOG_NAME));
		LogManager.getLogManager().readConfiguration(is);
		is.close();

		// Initialize config
		Config.load();
		checkFreePorts();
		L2DatabaseFactory.getInstance();
		gameServer = new GameServer();

		if(Config.IS_TELNET_ENABLED)
		{
			statusServer = new Status(Server.MODE_GAMESERVER);
			statusServer.start();
		}
		else
			_log.info("Telnet server is currently disabled.");

		System.gc();
		// maxMemory is the upper limit the jvm can use, totalMemory the size of the current allocation pool, freeMemory the unused memory in the allocation pool
		long freeMem = (Runtime.getRuntime().maxMemory() - Runtime.getRuntime().totalMemory() + Runtime.getRuntime().freeMemory()) / 1024 / 1024;
		long totalMem = Runtime.getRuntime().maxMemory() / 1024 / 1024;
		_log.info("Free memory " + freeMem + " Mb of " + totalMem + " Mb");
		Log.LogServ(Log.GS_started, (int) freeMem, (int) totalMem, IdFactory.getInstance().size(), 0);

		//Shutdown.getInstance().setAutoRestart(Config.RESTART_TIME * 60 * 60);
	}

	public static void checkFreePorts()
	{
		try
		{
			ServerSocket ss;
			if(Config.GAMESERVER_HOSTNAME.equalsIgnoreCase("*"))
				ss = new ServerSocket(Config.PORT_GAME);
			else
				ss = new ServerSocket(Config.PORT_GAME, 50, InetAddress.getByName(Config.GAMESERVER_HOSTNAME));
			ss.close();
		}
		catch(Exception e)
		{
			_log.warning("\nPort " + Config.PORT_GAME + " is allready binded. Please free it and restart server.");
			System.exit(0);
		}
	}
}
