package l2r.gameserver;

import java.sql.ResultSet;
import java.util.HashMap;
import java.util.Vector;
import java.util.logging.Logger;

import l2r.Config;
import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.instancemanager.ZoneManager;
import l2r.gameserver.model.Inventory;
import l2r.gameserver.model.L2ManufactureItem;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Recipe;
import l2r.gameserver.model.L2RecipeList;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.L2Zone;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.RecipeBookItemList;
import l2r.gameserver.serverpackets.RecipeItemMakeInfo;
import l2r.gameserver.serverpackets.RecipeShopItemInfo;
import l2r.gameserver.serverpackets.StatusUpdate;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.templates.L2EtcItem.EtcItemType;
import l2r.util.Rnd;

public class RecipeController
{
	protected static Logger _log = Logger.getLogger(RecipeController.class.getName());
	private static RecipeController _instance;

	private HashMap<Integer, L2RecipeList> _lists;

	/***************************************************************************/

	public static RecipeController getInstance()
	{
		if(_instance == null)
			_instance = new RecipeController();
		return _instance;
	}

	/***************************************************************************/

	public RecipeController()
	{
		_lists = new HashMap<Integer, L2RecipeList>();
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		FiltredPreparedStatement st2 = null;
		ResultSet list = null, rset2 = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT * FROM recipes");
			st2 = con.prepareStatement("SELECT * FROM `recitems` WHERE `rid`=?");
			list = statement.executeQuery();

			while(list.next())
			{
				Vector<L2Recipe> recipePartList = new Vector<L2Recipe>();

				boolean isDvarvenCraft = list.getBoolean("dwarven");
				String recipeName = list.getString("name");
				int id = list.getInt("id");
				short recipeId = list.getShort("recid");
				int level = list.getInt("lvl");
				short itemId = list.getShort("item");
				short count = list.getShort("q");
				int mpCost = list.getInt("mp");
				int successRate = list.getInt("success");
				long exp = list.getLong("exp");
				long sp = list.getLong("sp");

				//material
				st2.setInt(1, id);
				rset2 = st2.executeQuery();
				while(rset2.next())
				{
					int rpItemId = rset2.getInt("item");
					int quantity = rset2.getInt("q");
					L2Recipe rp = new L2Recipe(rpItemId, quantity);
					recipePartList.add(rp);
				}

				L2RecipeList recipeList = new L2RecipeList(id, level, recipeId, recipeName, successRate, mpCost, itemId, count, exp, sp, isDvarvenCraft);
				for(L2Recipe recipePart : recipePartList)
					recipeList.addRecipe(recipePart);
				_lists.put(new Integer(_lists.size()), recipeList);
			}

			_log.config("RecipeController: Loaded " + _lists.size() + " Recipes.");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseSR(st2, rset2);
			DatabaseUtils.closeDatabaseCSR(con, statement, list);
		}
	}

	/***************************************************************************/

	public int getRecipesCount()
	{
		return _lists.size();
	}

	/***************************************************************************/

	public L2RecipeList getRecipeList(int listId)
	{
		return _lists.get(listId);
	}

	/***************************************************************************/

	public L2RecipeList getRecipeByItemId(int itemId)
	{
		for(int i = 0; i < _lists.size(); i++)
		{
			L2RecipeList find = _lists.get(new Integer(i));
			if(find.getRecipeId() == itemId)
				return find;
		}
		return null;
	}

	/***************************************************************************/

	public void requestBookOpen(L2Player player, boolean isDwarvenCraft)
	{
		RecipeBookItemList response = new RecipeBookItemList(isDwarvenCraft, (int) player.getCurrentMp());
		if(isDwarvenCraft)
			response.setRecipes(player.getDwarvenRecipeBook());
		else
			response.setRecipes(player.getCommonRecipeBook());
		player.sendPacket(response);
	}

	/***************************************************************************/

	public void requestMakeItem(L2Player player, int recipeListId)
	{
		L2RecipeList recipeList = getRecipeList(recipeListId - 1);
		player.resetWaitSitTime();

		if(recipeList == null || recipeList.getRecipes().length == 0)
		{
			player.sendPacket(Msg.THE_RECIPE_IS_INCORRECT);
			return;
		}

		synchronized (player)
		{
			if(player.getCurrentMp() < recipeList.getMpCost())
			{
				player.sendPacket(Msg.NOT_ENOUGH_MP);
				player.sendPacket(new RecipeItemMakeInfo(recipeList.getId(), player, 0));
				return;
			}

			if(!player.findRecipe(recipeListId))
			{
				player.sendPacket(Msg.PLEASE_REGISTER_A_RECIPE);
				player.sendActionFailed();
				return;
			}
		}

		synchronized (player.getInventory())
		{
			L2Recipe[] recipes = recipeList.getRecipes();
			Inventory inventory = player.getInventory();
			for(L2Recipe recipe : recipes)
			{
				if(recipe.getQuantity() == 0)
					continue;

				if(Config.ALT_GAME_UNREGISTER_RECIPE && ItemTable.getInstance().getTemplate(recipe.getItemId()).getItemType() == EtcItemType.RECIPE)
				{
					L2RecipeList rp = RecipeController.getInstance().getRecipeByItemId(recipe.getItemId());
					if(player.findRecipe(rp))
						continue;
					player.sendPacket(Msg.NOT_ENOUGH_MATERIALS);
					player.sendPacket(new RecipeItemMakeInfo(recipeList.getId(), player, 0));
					return;
				}

				L2ItemInstance invItem = inventory.getItemByItemId(recipe.getItemId());

				if(invItem == null || recipe.getQuantity() > invItem.getIntegerLimitedCount())
				{
					player.sendPacket(Msg.NOT_ENOUGH_MATERIALS);
					player.sendPacket(new RecipeItemMakeInfo(recipeList.getId(), player, 0));
					return;
				}
			}

			player.reduceCurrentMp(recipeList.getMpCost(), null);

			for(L2Recipe recipe : recipes)
				if(recipe.getQuantity() != 0)
				{
					L2ItemInstance invItem = inventory.getItemByItemId(recipe.getItemId());
					if(Config.ALT_GAME_UNREGISTER_RECIPE && ItemTable.getInstance().getTemplate(recipe.getItemId()).getItemType() == EtcItemType.RECIPE)
						player.unregisterRecipe(RecipeController.getInstance().getRecipeByItemId(recipe.getItemId()).getId());
					else
						inventory.destroyItem(invItem, recipe.getQuantity(), false);
				}
		}

		L2ItemInstance createdItem = ItemTable.getInstance().createItem(recipeList.getItemId(), player.getObjectId(), player.getObjectId(), "RecipeController");
		createdItem.setCount(recipeList.getCount());

		SystemMessage sm;
		int success = 0;
		if(!Rnd.chance(recipeList.getSuccessRate()))
		{
			sm = new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(createdItem.getItemId());
			L2World.removeObject(createdItem);
		}
		else
		{
			if(Config.CRAFT_COUNTER)
				player.incrementCraftCounter(createdItem.getItemId(), new Integer(recipeList.getCount()));

			player.getInventory().addItem(createdItem);
			sm = new SystemMessage(SystemMessage.EARNED_S2_S1_s);
			sm.addItemName(createdItem.getItemId());
			sm.addNumber(recipeList.getCount());
			success++;
		}

		player.sendPacket(sm);
		if(Config.ALT_GAME_EXP_FOR_CRAFT)
			player.addExpAndSp((long) (recipeList.getExp() * Config.RATE_XP), (long) (recipeList.getSp() * Config.RATE_SP), true, false);
		StatusUpdate su = new StatusUpdate(player.getObjectId());
		su.addAttribute(StatusUpdate.CUR_LOAD, player.getCurrentLoad());
		su.addAttribute(StatusUpdate.CUR_MP, (int) player.getCurrentMp());
		player.sendPacket(su);
		player.sendPacket(new RecipeItemMakeInfo(recipeList.getId(), player, success));
	}

	/***************************************************************************/

	public void requestManufactureItem(L2Player player, L2Player employer, int recipeListId)
	{
		L2RecipeList recipeList = getRecipeList(recipeListId - 1);
		if(recipeList == null)
			return;

		player.resetWaitSitTime();
		int success = 0;

		player.sendMessage(new CustomMessage("l2r.gameserver.RecipeController.GotOrder", player).addString(recipeList.getRecipeName()));

		if(recipeList.getRecipes().length == 0)
		{
			player.sendMessage(new CustomMessage("l2r.gameserver.RecipeController.NoRecipe", player).addString(recipeList.getRecipeName()));
			employer.sendMessage(new CustomMessage("l2r.gameserver.RecipeController.NoRecipe", player).addString(recipeList.getRecipeName()));
			return;
		}

		synchronized (player)
		{
			if(player.getCurrentMp() < recipeList.getMpCost())
			{
				player.sendPacket(Msg.NOT_ENOUGH_MP);
				employer.sendPacket(Msg.NOT_ENOUGH_MP);
				employer.sendPacket(new RecipeShopItemInfo(player.getObjectId(), recipeListId, success));
				return;
			}

			if(!player.findRecipe(recipeListId))
			{
				player.sendPacket(Msg.PLEASE_REGISTER_A_RECIPE);
				player.sendActionFailed();
				return;
			}
		}

		int price = 0;
		for(L2ManufactureItem temp : player.getCreateList().getList())
			if(temp.getRecipeId() == recipeList.getId())
			{
				price = temp.getCost();
				break;
			}

		if(employer.getAdena() < price)
		{
			employer.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			employer.sendPacket(new RecipeShopItemInfo(player.getObjectId(), recipeListId, success));
			return;
		}

		synchronized (employer.getInventory())
		{
			L2Recipe[] recipes = recipeList.getRecipes();
			Inventory inventory = employer.getInventory();
			for(L2Recipe recipe : recipes)
			{
				if(recipe.getQuantity() == 0)
					continue;

				L2ItemInstance invItem = inventory.getItemByItemId(recipe.getItemId());

				if(invItem == null || recipe.getQuantity() > invItem.getIntegerLimitedCount())
				{
					employer.sendPacket(Msg.NOT_ENOUGH_MATERIALS);
					employer.sendPacket(new RecipeShopItemInfo(player.getObjectId(), recipeListId, success));
					return;
				}
			}

			player.reduceCurrentMp(recipeList.getMpCost(), null);

			for(L2Recipe recipe : recipes)
				if(recipe.getQuantity() != 0)
				{
					L2ItemInstance invItem = inventory.getItemByItemId(recipe.getItemId());
					inventory.destroyItem(invItem, recipe.getQuantity(), false);
				}
		}

		if(price > 0)
		{
			employer.reduceAdena(price, false);
			player.addAdena(price);

			int tax = (int) (price * Config.SERVICES_TRADE_TAX / 100);
			if(ZoneManager.getInstance().checkIfInZone(L2Zone.ZoneType.offshore, player.getX(), player.getY()))
				tax = (int) (price * Config.SERVICES_OFFSHORE_TRADE_TAX / 100);
			if(Config.SERVICES_TRADE_TAX_ONLY_OFFLINE && !player.isInOfflineMode())
				tax = 0;
			if(tax > 0)
			{
				player.reduceAdena(tax, false);
				L2World.addTax(tax);
				player.sendMessage(new CustomMessage("trade.HavePaidTax", player).addNumber(tax));
			}
		}

		L2ItemInstance createdItem = ItemTable.getInstance().createItem(recipeList.getItemId(), employer.getObjectId(), player.getObjectId(), "RecipeController");
		createdItem.setCount(recipeList.getCount());

		SystemMessage msgtoemployer;
		SystemMessage msgtomaster;
		if(!Rnd.chance(recipeList.getSuccessRate()))
		{
			msgtoemployer = new SystemMessage(SystemMessage.S1_HAS_FAILED_TO_CREATE_S2_AT_THE_PRICE_OF_S3_ADENA);
			msgtoemployer.addString(player.getName());
			msgtoemployer.addItemName(createdItem.getItemId());
			msgtoemployer.addNumber(price);
			msgtomaster = new SystemMessage(SystemMessage.THE_ATTEMPT_TO_CREATE_S2_FOR_S1_AT_THE_PRICE_OF_S3_ADENA_HAS_FAILED);
			msgtomaster.addString(employer.getName());
			msgtomaster.addItemName(createdItem.getItemId());
			msgtomaster.addNumber(price);
			L2World.removeObject(createdItem);
		}
		else
		{
			if(Config.CRAFT_COUNTER)
				player.incrementCraftCounter(createdItem.getItemId(), new Integer(recipeList.getCount()));

			employer.getInventory().addItem(createdItem);

			if(recipeList.getCount() > 1)
			{
				msgtoemployer = new SystemMessage(SystemMessage.S1_CREATED_S2_S3_AT_THE_PRICE_OF_S4_ADENA);
				msgtoemployer.addString(player.getName());
				msgtoemployer.addNumber(recipeList.getCount());
				msgtoemployer.addItemName(createdItem.getItemId());
				msgtoemployer.addNumber(price);
				msgtomaster = new SystemMessage(SystemMessage.S2_S3_HAVE_BEEN_SOLD_TO_S1_FOR_S4_ADENA);
				msgtomaster.addString(employer.getName());
				msgtomaster.addNumber(recipeList.getCount());
				msgtomaster.addItemName(createdItem.getItemId());
				msgtomaster.addNumber(price);
			}
			else
			{
				msgtoemployer = new SystemMessage(SystemMessage.S1_CREATED_S2_AFTER_RECEIVING_S3_ADENA);
				msgtoemployer.addString(player.getName());
				msgtoemployer.addItemName(createdItem.getItemId());
				msgtoemployer.addNumber(price);
				msgtomaster = new SystemMessage(SystemMessage.S2_IS_SOLD_TO_S1_AT_THE_PRICE_OF_S3_ADENA);
				msgtomaster.addString(employer.getName());
				msgtomaster.addItemName(createdItem.getItemId());
				msgtomaster.addNumber(price);
			}
			success++;
		}

		if(Config.ALT_GAME_EXP_FOR_CRAFT)
			player.addExpAndSp((long) (recipeList.getExp() * Config.RATE_XP), (long) (recipeList.getSp() * Config.RATE_SP), true, false);
		player.sendPacket(new StatusUpdate(player.getObjectId()).addAttribute(StatusUpdate.CUR_MP, (int) player.getCurrentMp()));
		player.sendPacket(msgtomaster);
		employer.sendPacket(msgtoemployer);
		employer.sendChanges();
		employer.sendPacket(new StatusUpdate(player.getObjectId()).addAttribute(StatusUpdate.CUR_LOAD, player.getCurrentLoad()));
		employer.sendPacket(new RecipeShopItemInfo(player.getObjectId(), recipeListId, success));
	}
}
