package l2r.gameserver.ai;

import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_ATTACK;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_CAST;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_IDLE;

import java.util.logging.Logger;

import l2r.Config;
import l2r.extensions.listeners.MethodCollection;
import l2r.extensions.listeners.engine.DefaultListenerEngine;
import l2r.extensions.listeners.engine.ListenerEngine;
import l2r.extensions.listeners.events.AbstractAI.AbstractAINotifyEvent;
import l2r.extensions.listeners.events.AbstractAI.AbstractAISetIntention;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2World;
import l2r.util.Location;

public abstract class AbstractAI
{
	protected static final Logger _log = Logger.getLogger(AbstractAI.class.getName());

	protected final L2Character _actor;

	protected CtrlIntention _intention = AI_INTENTION_IDLE;
	protected Object _intention_arg0 = null;
	protected Object _intention_arg1 = null;

	protected L2Character _attack_target;
	protected L2Skill _skill;

	public final double HEADINGS_IN_PI = 10430.378350470452724949566316381;

    protected AbstractAI(L2Character actor)
	{
		_actor = actor;
		if(isGlobalAI())
			L2World.addCustomMoveObject(actor);
	}

	public void changeIntention(CtrlIntention intention, Object arg0, Object arg1)
	{
		if(Config.DEBUG)
			_log.info("AbstractAI: " + _actor + ".changeIntention(" + intention + ", " + arg0 + ", " + arg1 + ")");

		_intention = intention;
		if(intention != AI_INTENTION_CAST && intention != AI_INTENTION_ATTACK)
		setAttackTarget(null);
	}

	public final void setIntention(CtrlIntention intention)
	{
		setIntention(intention, null, null);
	}

	public final void setIntention(CtrlIntention intention, Object arg0)
	{
		setIntention(intention, arg0, null);
	}

	public void setIntention(CtrlIntention intention, Object arg0, Object arg1)
	{
		if(intention != AI_INTENTION_CAST && intention != AI_INTENTION_ATTACK)
			setAttackTarget(null);
		if(!_actor.hasAI())
			return;

		if(!_actor.isVisible())
		{
			if(_intention == AI_INTENTION_IDLE)
				return;

			intention = AI_INTENTION_IDLE;
		}

		getListenerEngine().fireMethodInvoked(new AbstractAISetIntention(MethodCollection.AbstractAIsetIntention, this, new Object[] {
				intention,
				arg0,
				arg1 }));

		switch(intention)
		{
			case AI_INTENTION_IDLE:
				onIntentionIdle();
				break;
			case AI_INTENTION_ACTIVE:
				onIntentionActive();
				break;
			case AI_INTENTION_REST:
				onIntentionRest();
				break;
			case AI_INTENTION_ATTACK:
				onIntentionAttack((L2Character) arg0);
				break;
			case AI_INTENTION_CAST:
				onIntentionCast((L2Skill) arg0, (L2Character) arg1);
				break;
			case AI_INTENTION_PICK_UP:
				onIntentionPickUp((L2Object) arg0);
				break;
			case AI_INTENTION_INTERACT:
				onIntentionInteract((L2Object) arg0);
				break;
			case AI_INTENTION_FOLLOW:
				onIntentionFollow((L2Character) arg0, (Integer) arg1);
				break;
		}
	}

	public final void notifyEvent(CtrlEvent evt)
	{
		notifyEvent(evt, null, null);
	}

	public final void notifyEvent(CtrlEvent evt, Object arg0)
	{
		notifyEvent(evt, arg0, null);
	}

	public void notifyEvent(CtrlEvent evt, Object arg0, Object arg1)
	{
		if(!_actor.isVisible() || !_actor.hasAI())
			return;

		getListenerEngine().fireMethodInvoked(new AbstractAINotifyEvent(MethodCollection.AbstractAInotifyEvent, this, new Object[] { evt, arg0, arg1 }));

		switch(evt)
		{
			case EVT_THINK:
				onEvtThink();
				break;
			case EVT_ATTACKED:
				onEvtAttacked((L2Character) arg0, ((Number) arg1).intValue());
				break;
			case EVT_AGGRESSION:
				onEvtAggression((L2Character) arg0, ((Number) arg1).intValue());
				break;
			case EVT_READY_TO_ACT:
				onEvtReadyToAct();
				break;
			case EVT_ARRIVED:
				onEvtArrived();
				break;
			case EVT_ARRIVED_TARGET:
				onEvtArrivedTarget();
				break;
			case EVT_ARRIVED_BLOCKED:
				onEvtArrivedBlocked((Location) arg0);
				break;
			case EVT_FORGET_OBJECT:
				onEvtForgetObject((L2Object) arg0);
				break;
			case EVT_DEAD:
				onEvtDead();
				break;
			case EVT_FAKE_DEATH:
				onEvtFakeDeath();
				break;
			case EVT_FINISH_CASTING:
				onEvtFinishCasting();
				break;
			case EVT_SEE_SPELL:
				onEvtSeeSpell((L2Skill) arg0, (L2Character) arg1);
				break;
		}
	}

	protected void clientActionFailed()
	{
		if(_actor.isPlayer)
			_actor.sendActionFailed();
	}
	
	public void clientStopMoving(boolean validate)
	{
		L2Character actor = getActor();
		if(actor == null)
			return;
		actor.stopMove(validate);
	}
	public void clientStopMoving()
	{
		_actor.stopMove();
	}

	public L2Character getActor()
	{
		return _actor;
	}

	public CtrlIntention getIntention()
	{
		return _intention;
	}

	public void setAttackTarget(L2Character target)
	{
		_attack_target = target;
	}

	public L2Character getAttackTarget()
	{
		return _attack_target;
	}

	/** Означает, что AI всегда включен, независимо от состояния региона */
	public boolean isGlobalAI()
	{
		return false;
	}

	public void setGlobalAggro(int value)
	{}

	@Override
	public String toString()
	{
		return getL2ClassShortName() + " for " + _actor;
	}

	public String getL2ClassShortName()
	{
		if(getClass() != null && getClass().getName() != null)
			return getClass().getName().replaceAll("^.*\\.(.*?)$", "$1");
		return "";
	}

	protected abstract void onIntentionIdle();

	protected abstract void onIntentionActive();

	protected abstract void onIntentionRest();

	protected abstract void onIntentionAttack(L2Character target);

	protected abstract void onIntentionCast(L2Skill skill, L2Character target);

	protected abstract void onIntentionPickUp(L2Object item);

	protected abstract void onIntentionInteract(L2Object object);

	protected abstract void onEvtThink();

	protected abstract void onEvtAttacked(L2Character attacker, int damage);

	protected abstract void onEvtAggression(L2Character target, int aggro);

	protected abstract void onEvtReadyToAct();

	protected abstract void onEvtArrived();

	protected abstract void onEvtArrivedTarget();

	protected abstract void onEvtArrivedBlocked(Location blocked_at_pos);

	protected abstract void onEvtForgetObject(L2Object object);

	protected abstract void onEvtDead();

	protected abstract void onEvtFakeDeath();

	protected abstract void onEvtFinishCasting();

	protected abstract void onEvtSeeSpell(L2Skill skill, L2Character caster);

	protected abstract void onIntentionFollow(L2Character target, Integer offset);

	private ListenerEngine<AbstractAI> listenerEngine;

	public ListenerEngine<AbstractAI> getListenerEngine()
	{
		if(listenerEngine == null)
			listenerEngine = new DefaultListenerEngine<AbstractAI>(this);
		return listenerEngine;
	}
}
