package l2r.gameserver.ai;

import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Player;
import l2r.Config;

public class Guard extends Fighter implements Runnable
{
	private boolean _isMovingBack = false;

	public Guard(L2Character actor)
	{
		super(actor);
	}

	@Override
	protected boolean randomWalk()
	{
		return false;
	}

	@Override
	protected void thinkAttack()
	{
		if(_attack_target == null || (_attack_target.isDead() || _thisActor.getDistance(_attack_target) > Config.MAX_PURSUE_RANGE))
		{
			clearTasks();
			if(!_thisActor.isInRange(_thisActor.getSpawnedLoc(), MAX_PURSUE_RANGE))
				teleportHome();
			else
				returnHome();
			return;
		}
		super.thinkAttack();
	}

	@Override
	protected boolean thinkActive()
	{
		if(!_thisActor.isDead())
		{
			if(!_thisActor.isInRange(_thisActor.getSpawnedLoc(), MAX_PURSUE_RANGE))
			{
				clearTasks();
				teleportHome();
				return true;
			}
			if(!_isMovingBack) // Двигаемся на базу
			{
				clearTasks();
				_isMovingBack = true;
				returnHome();
				return false;
			}
			else if(_isMovingBack && !_thisActor.isMoving) //Пришли на базу
			{
				_isMovingBack = false;
				_thisActor.setHeading(_thisActor.getSpawnedLoc().h);
			}
		}

		if(super.thinkActive())
			return true;

		return false;
	}

	@Override
	public void checkAggression(L2Character target)
	{
		if(target.getKarma() <= 0)
			return;
		if(_intention != CtrlIntention.AI_INTENTION_ACTIVE)
			return;
		if(_globalAggro < 0)
			return;
		if(target.getHateList().get(_thisActor) == null && !_thisActor.isInRange(target, 600))
			return;
		if(Math.abs(target.getZ() - _actor.getZ()) > 400)
			return;
		if((target instanceof L2Playable) && isSilentMoveNotVisible((L2Playable) target))
			return;
		if(!GeoClient.getInstance().canSeeTarget(_actor, target))
			return;
		if(target.isPlayer && ((L2Player) target).getPlayerAccess().IsGM && ((L2Player) target).isInvisible())
			return;
		if((target.isSummon || target.isPet) && target.getOwner() != null)
			target.getOwner().addDamageHate(_thisActor, 0, 1);
		target.addDamageHate(_thisActor, 0, 2);
		startRunningTask(2000);
		setIntention(CtrlIntention.AI_INTENTION_ATTACK, target);
	}
}
