package l2r.gameserver.ai;

import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_ACTIVE;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_ATTACK;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_CAST;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_FOLLOW;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_IDLE;
import l2r.gameserver.ai.L2PlayableAI.nextAction;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.serverpackets.Die;
import l2r.util.Location;

public class L2CharacterAI extends AbstractAI
{
	public L2CharacterAI(L2Character actor)
	{
		super(actor);
	}

	@Override
	protected void onIntentionIdle()
	{
		clientStopMoving();
		changeIntention(AI_INTENTION_IDLE, null, null);
	}

	@Override
	protected void onIntentionActive()
	{
		clientStopMoving();
		changeIntention(AI_INTENTION_ACTIVE, null, null);
	}

	@Override
	protected void onIntentionAttack(L2Character target)
	{
		_attack_target = target;
		changeIntention(AI_INTENTION_ATTACK, target, null);
		onEvtThink();
	}

	@Override
	protected void onIntentionCast(L2Skill skill, L2Character target)
	{
		_skill = skill;
		_attack_target = target;
		changeIntention(AI_INTENTION_CAST, skill, target);
		onEvtThink();
	}

	@Override
	protected void onIntentionFollow(L2Character target, Integer offset)
	{
		changeIntention(AI_INTENTION_FOLLOW, target, offset);
		_actor.followToCharacter(target, offset, false);
	}

	@Override
	protected void onEvtArrivedBlocked(Location blocked_at_pos)
	{
		_actor.stopMove();
		onEvtThink();
	}

	@Override
	protected void onEvtForgetObject(L2Object object)
	{
		if(_actor == null || object == null)
			return;
		if(_attack_target == object || _actor.getTargetId() == object.getObjectId())
		{
			_attack_target = null;
			_actor.setTarget(null);
			setIntention(AI_INTENTION_ACTIVE, null, null);
		}
	}

	@Override
	protected void onEvtDead()
	{
		_actor.breakAttack();
		_actor.breakCast(true);
		_actor.stopMove();
		_actor.broadcastPacket(new Die(_actor));
		_intention = AI_INTENTION_IDLE;
		_attack_target = null;
	}

	@Override
	protected void onEvtFakeDeath()
	{
		clientStopMoving();
		_intention = AI_INTENTION_IDLE;
	}

	@Override
	protected void onEvtAttacked(L2Character attacker, int damage)
	{
		_actor.startAttackStanceTask();
		if(attacker != null)
			attacker.startAttackStanceTask();
	}
	public void Attack(L2Object target, boolean forceUse)
	{
		setIntention(AI_INTENTION_ATTACK, target);
	}

	public void Cast(L2Skill skill, L2Character target)
	{
		Cast(skill, target, false, false);
	}

	public void Cast(L2Skill skill, L2Character target, boolean forceUse, boolean dontMove)
	{
		setIntention(AI_INTENTION_ATTACK, target);
	}

	@SuppressWarnings("unused")
	public void Attack(L2Object target, boolean forceUse, boolean dontMove)
	{
		setIntention(AI_INTENTION_ATTACK, target);
	}
	@Override
	protected void onEvtThink()
	{}

	@Override
	protected void onEvtAggression(L2Character target, int aggro)
	{}

	@Override
	protected void onEvtFinishCasting()
	{}

	@Override
	protected void onIntentionInteract(L2Object object)
	{}

	@Override
	protected void onIntentionPickUp(L2Object item)
	{}

	@Override
	protected void onEvtReadyToAct()
	{}

	@Override
	protected void onEvtArrived()
	{}

	@Override
	protected void onEvtArrivedTarget()
	{}

	@Override
	protected void onIntentionRest()
	{}

	@Override
	protected void onEvtSeeSpell(L2Skill skill, L2Character caster)
	{}

	public void stopAITask()
	{}

	public void startAITask()
	{}

	public void setNextAction(nextAction action, Object arg0, Object arg1, boolean arg2, boolean arg3)
	{}

	public void clearTasks()
	{}
	
	public void clearNextAction()
	{}

	@SuppressWarnings("unused")
	public void teleportHome()
	{}

	public void checkAggression(L2Character target)
	{}

	public boolean isActive()
	{
		return true;
	}

	public boolean isGlobalAggro()
	{
		return true;
	}
}