package l2r.gameserver.ai;

import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_ACTIVE;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_ATTACK;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_CAST;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_FOLLOW;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_IDLE;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_INTERACT;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_PICK_UP;

import java.util.concurrent.ScheduledFuture;

import l2r.Config;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Effect;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Skill.NextAction;
import l2r.gameserver.model.L2Skill.SkillType;
import l2r.gameserver.serverpackets.MyTargetSelected;
import l2r.util.Location;

public class L2PlayableAI extends L2CharacterAI
{
	private boolean thinking = false; // to prevent recursive thinking
    private L2Skill _skill;

	private nextAction _nextAction;
	private Object _nextAction_arg0;
	private Object _nextAction_arg1;
	private boolean _nextAction_arg2;
	private boolean _nextAction_arg3;

	private boolean _forceUse;
	private boolean _dontMove;

	private ScheduledFuture<?> followTask;

	public L2PlayableAI(L2Playable actor)
	{
		super(actor);
	}

	public enum nextAction
	{
		ATTACK,
		CAST,
		MOVE,
		REST,
		PICKUP,
		INTERACT
	}
	
	@Override
	public void changeIntention(CtrlIntention intention, Object arg0, Object arg1)
	{
		super.changeIntention(intention, arg0, arg1);
		_intention_arg0 = arg0;
		_intention_arg1 = arg1;
	}

	@Override
	public void setIntention(CtrlIntention intention, Object arg0, Object arg1)
	{
		_intention_arg0 = arg0;
		_intention_arg1 = arg1;
		super.setIntention(intention, arg0, arg1);
	}

	@Override
	protected void onIntentionCast(L2Skill skill, L2Character target)
	{
        _skill = skill;
		super.onIntentionCast(skill, target);
	}

	@Override
	public void setNextAction(nextAction action, Object arg0, Object arg1, boolean arg2, boolean arg3)
	{
		_nextAction = action;
		_nextAction_arg0 = arg0;
		_nextAction_arg1 = arg1;
		_nextAction_arg2 = arg2;
		_nextAction_arg3 = arg3;
	}

	public boolean setNextIntention()
	{
		nextAction nextAction = _nextAction;
		Object nextAction_arg0 = _nextAction_arg0;
		Object nextAction_arg1 = _nextAction_arg1;
		boolean nextAction_arg2 = _nextAction_arg2;
		boolean nextAction_arg3 = _nextAction_arg3;

		if(nextAction == null)
			return false;

		L2Skill skill;
		L2Character target;
		L2Object object;

		switch(nextAction)
		{
			case ATTACK:
				if(nextAction_arg0 == null)
					return false;
				target = (L2Character) nextAction_arg0;
				_forceUse = nextAction_arg2;
				_dontMove = nextAction_arg3;
				clearNextAction();
				setIntention(AI_INTENTION_ATTACK, target);
				break;
			case CAST:
				if(nextAction_arg0 == null || nextAction_arg1 == null)
					return false;
				skill = (L2Skill) nextAction_arg0;
				target = (L2Character) nextAction_arg1;
				_forceUse = nextAction_arg2;
				_dontMove = nextAction_arg3;
				clearNextAction();
				if(skill.checkCondition(_actor, target, _forceUse, _dontMove, true))
				{
					if(skill.getNextAction() == NextAction.ATTACK && !_actor.equals(target))
					{
						setNextAction(nextAction.ATTACK, target, null, _forceUse, false);
						return setNextIntention();
					}
					return false;
				}
					setIntention(AI_INTENTION_CAST, skill, target);
				break;
			case MOVE:
				if(nextAction_arg0 == null || nextAction_arg1 == null)
					return false;
				Location loc = (Location) nextAction_arg0;
				Integer offset = (Integer) nextAction_arg1;
				clearNextAction();
				_actor.moveToLocation(loc, offset, nextAction_arg2);
				break;
			case REST:
				_actor.sitDown();
				break;
			case INTERACT:
				if(nextAction_arg0 == null)
					return false;
				object = (L2Object) nextAction_arg0;
				clearNextAction();
				onIntentionInteract(object);
				break;
			case PICKUP:
				if(nextAction_arg0 == null)
					return false;
				object = (L2Object) nextAction_arg0;
				clearNextAction();
				onIntentionPickUp(object);
				break;
			default:
				return false;
		}
		return true;
	}

	@Override
	public void clearNextAction()
	{
		_nextAction = null;
		_nextAction_arg0 = null;
		_nextAction_arg1 = null;
		_nextAction_arg2 = false;
		_nextAction_arg3 = false;
	}

	@Override
	protected void onEvtFinishCasting()
	{
		if(!setNextIntention())
			setIntention(AI_INTENTION_ACTIVE);
	}

	@Override
	protected void onEvtReadyToAct()
	{
		if(!setNextIntention())
			onEvtThink();
	}

	@Override
	protected void onEvtArrived()
	{
		if(!setNextIntention())
			if(_intention == AI_INTENTION_INTERACT || _intention == AI_INTENTION_PICK_UP)
				onEvtThink();
			else
				changeIntention(AI_INTENTION_ACTIVE, null, null);
	}

	@Override
	protected void onEvtArrivedTarget()
	{
		switch(_intention)
		{
			case AI_INTENTION_ATTACK:
				thinkAttack(false);
				break;
			case AI_INTENTION_CAST:
				thinkCast(false);
				break;
			case AI_INTENTION_FOLLOW:
				if(followTask != null)
					followTask.cancel(false);
				followTask = ThreadPoolManager.getInstance().scheduleAi(new ThinkFollow(), 1000, true);
				break;
		}
	}

	@Override
	protected void onEvtThink()
	{
		if(thinking || _actor.isActionsDisabled())
			return;

		if(Config.DEBUG)
			_log.warning("L2PlayableAI: onEvtThink -> Check intention");

		thinking = true;

		try
		{
			switch(_intention)
			{
				case AI_INTENTION_ATTACK:
					thinkAttack(true);
					break;
				case AI_INTENTION_CAST:
					thinkCast(true);
					break;
				case AI_INTENTION_PICK_UP:
					thinkPickUp();
					break;
				case AI_INTENTION_INTERACT:
					thinkInteract();
					break;
			}
		}
		catch(Exception e)
		{
			_log.warning("Exception onEvtThink(): " + e);
			e.printStackTrace();
		}
		finally
		{
			thinking = false;
		}
	}

	public class ThinkFollow implements Runnable
	{
		public void run()
		{
			followTask = null;
			if(_intention != AI_INTENTION_FOLLOW)
				return;
			L2Character target = (L2Character) _intention_arg0;
			Integer offset = (Integer) _intention_arg1;
			if(target == null || target.isAlikeDead() || _actor.getDistance(target) > 4000)
			{
				setIntention(CtrlIntention.AI_INTENTION_ACTIVE);
				return;
			}
			L2Player actorPlayer = _actor.getPlayer();
			if(actorPlayer == null || !actorPlayer.isConnected() || ((_actor.isPet || _actor.isSummon) && actorPlayer != target))
			{
				setIntention(CtrlIntention.AI_INTENTION_ACTIVE);
				return;
			}
			if(!_actor.isInRange(target, offset) && (!_actor.isFollow || _actor.getFollowTarget() != target))
			_actor.followToCharacter(target, offset, false);
			else
				followTask = ThreadPoolManager.getInstance().scheduleAi(this, 1000, true);
		}
	}

	@Override
	protected void onIntentionInteract(L2Object object)
	{
		if(_actor.isActionsDisabled())
		{
			setNextAction(nextAction.INTERACT, object, null, false, false);
			clientActionFailed();
			return;
		}

		clearNextAction();
		changeIntention(AI_INTENTION_INTERACT, object, null);
		onEvtThink();
	}

	protected void thinkInteract()
	{
		L2Object target = (L2Object) _intention_arg0;

		if(target == null)
		{
			setIntention(AI_INTENTION_ACTIVE);
			return;
		}

		int range = _actor.getMinDistance(target) + 20;

		if(_actor.isInRangeZ(target, range))
		{
			if(_actor.isPlayer)
				((L2Player) _actor).doInteract(target);
			setIntention(AI_INTENTION_IDLE);
		}
		else
		{
			_actor.moveToLocation(target.getLoc(), 10, true);
			setNextAction(nextAction.INTERACT, target, null, false, false);
		}
	}

	@Override
	protected void onIntentionPickUp(L2Object object)
	{
		if(_actor.isActionsDisabled())
		{
			setNextAction(nextAction.PICKUP, object, null, false, false);
			clientActionFailed();
			return;
		}

		clearNextAction();
		changeIntention(AI_INTENTION_PICK_UP, object, null);
		onEvtThink();
	}

	protected void thinkPickUp()
	{
		L2Object target = (L2Object) _intention_arg0;

		if(target == null)
		{
			setIntention(AI_INTENTION_ACTIVE);
			return;
		}

		if(_actor.isInRangeZ(target, 20))
		{
			if(_actor.isPlayer || _actor.isPet)
				_actor.doPickupItem(target);
			setIntention(AI_INTENTION_ACTIVE);
		}
		else
		{
			_actor.moveToLocation(target.getLoc(), 10, true);
			setNextAction(nextAction.PICKUP, target, null, false, false);
		}
	}

	protected void thinkAttack(boolean checkRange)
	{
		L2Player player = _actor.getPlayer();
		if(player == null)
		{
			setIntention(CtrlIntention.AI_INTENTION_ACTIVE);
			return;
		}
		
		if(_actor.isAttackingDisabled())
		{
			_actor.sendActionFailed();
			return;
		}

		if(_attack_target == null || _attack_target.isDead() || !(_forceUse ? _attack_target.isAttackable() : _attack_target.isAutoAttackable(getActor().getOwner())))
		{
			_actor.sendActionFailed();
			setIntention(AI_INTENTION_ACTIVE);
			return;
		}

		if(!checkRange)
		{
			clientStopMoving();
			_actor.doAttack(_attack_target);
			return;
		}

		int range = _actor.getPhysicalAttackRange() + _actor.getMinDistance(_attack_target);
		if(range < 10)
			range = 10;

		boolean canSee = GeoClient.getInstance().canSeeTarget(_actor, _attack_target);

		if(!canSee && (range > 200 || Math.abs(_actor.getZ() - _attack_target.getZ()) > 200))
		{
			_actor.sendPacket(Msg.CANNOT_SEE_TARGET);
			setIntention(AI_INTENTION_ACTIVE);
			_actor.sendActionFailed();
			return;
		}

		if(_actor.isFakeDeath())
			_actor.stopEffects(L2Effect.EffectType.FAKE_DEATH);

		if(_actor.isInRangeZ(_attack_target, range))
		{
			if(!canSee)
			{
				_actor.sendPacket(Msg.CANNOT_SEE_TARGET);
				setIntention(AI_INTENTION_ACTIVE);
				_actor.sendActionFailed();
				return;
			}

			clientStopMoving();
			_actor.doAttack(_attack_target);
		}
		else if(!_dontMove)//Don't move to char if shift pressed by vcart
			_actor.followToCharacter(_attack_target, range - 40, true);
		else
			_actor.sendActionFailed();
	}

	protected void thinkCast(boolean checkRange)
	{
		if(_skill.getSkillType() == SkillType.CRAFT || _skill.isToggle())
		{
			if(_skill.checkCondition(_actor, _attack_target, _forceUse, _dontMove, true))
				_actor.doCast(_skill, _attack_target, _forceUse);
			return;
		}

		if(_attack_target == null || _attack_target.isDead() != _skill.getCorpse() && !_skill.isNotTargetAoE())
		{
			setIntention(AI_INTENTION_ACTIVE);
			_actor.sendActionFailed();
			return;
		}

		if(!checkRange)
		{
			// Если скилл имеет следующее действие, назначим это действие после окончания действия скилла
			if(_skill.getNextAction() == NextAction.ATTACK && !_actor.equals(_attack_target))
				setNextAction(nextAction.ATTACK, _attack_target, null, _forceUse, false);
			else
				clearNextAction();

			if(!_skill.isToggle())
				clientStopMoving();

			if(_skill.checkCondition(_actor, _attack_target, _forceUse, _dontMove, true))
				_actor.doCast(_skill, _attack_target, _forceUse);
			else
			{
				setNextIntention();
				if(getIntention() == CtrlIntention.AI_INTENTION_ATTACK)
				thinkAttack(true);
			}

			return;
		}

		int range = _actor.getMagicalAttackRange(_skill) + _actor.getMinDistance(_attack_target);
		if(range < 10)
			range = 10;

		boolean canSee = GeoClient.getInstance().canSeeTarget(_actor, _attack_target);
		boolean noRangeSkill = _skill.getCastRange() == Integer.MAX_VALUE;

		if(!noRangeSkill && !canSee && (range > 200 || Math.abs(_actor.getZ() - _attack_target.getZ()) > 200))
		{
			_actor.sendPacket(Msg.CANNOT_SEE_TARGET);
			setIntention(AI_INTENTION_ACTIVE);
			_actor.sendActionFailed();
			return;
		}

		if(_actor.isFakeDeath())
			_actor.stopEffects(L2Effect.EffectType.FAKE_DEATH);

		if(_actor.isInRangeZ(_attack_target, range) || noRangeSkill)
		{
			if(!noRangeSkill && !canSee)
			{
				_actor.sendPacket(Msg.CANNOT_SEE_TARGET);
				setIntention(AI_INTENTION_ACTIVE);
				_actor.sendActionFailed();
				return;
			}

			// Если скилл имеет следующее действие, назначим это действие после окончания действия скилла
			if(_skill.getNextAction() == NextAction.ATTACK && !_actor.equals(_attack_target))
				setNextAction(nextAction.ATTACK, _attack_target, null, _forceUse, false);
			else
				clearNextAction();

			if(_skill.checkCondition(_actor, _attack_target, _forceUse, _dontMove, true))
            {
                clientStopMoving(false);
				_actor.doCast(_skill, _attack_target, _forceUse);
            }
			else
			{
				setNextIntention();
				if(getIntention() == CtrlIntention.AI_INTENTION_ATTACK)
					thinkAttack(true);
			}
		}
		else if(!_dontMove)
		{			
			if(!_actor.followToCharacter(_attack_target, range - 40, true))
			{
				_actor.sendPacket(Msg.YOUR_TARGET_IS_OUT_OF_RANGE);
				setIntention(AI_INTENTION_ACTIVE);
				_actor.sendActionFailed();
			}
		}
		else
		{
			_actor.sendPacket(Msg.YOUR_TARGET_IS_OUT_OF_RANGE);
			setIntention(AI_INTENTION_ACTIVE);
			_actor.sendActionFailed();
		}
	}

	@Override
	protected void onEvtDead()
	{
		clearNextAction();
		getActor().clearHateList();
		super.onEvtDead();
	}

	@Override
	protected void onEvtFakeDeath()
	{
		clearNextAction();
		super.onEvtFakeDeath();
	}

	@Override
	protected void onEvtAggression(L2Character target, int aggro)
	{
		if(aggro > 0)
			if(_actor.getTarget() != target && _actor.getAggressionTarget() == null)
			{

				_actor.setAggressionTarget(target);
				_actor.setTarget(target);


				clearNextAction();
				switch(getIntention())
				{
					case AI_INTENTION_ATTACK:
						setAttackTarget(target);
						break;
					case AI_INTENTION_CAST:
						L2Skill skill = _actor.getCastingSkill();
						if(skill == null)
						_skill = skill;
							switch(skill.getTargetType())
							{
								case TARGET_ONE:
								case TARGET_AREA:
								case TARGET_MULTIFACE:
									setAttackTarget(target);
									_actor.setCastingTarget(target);
									break;
							}
						break;
				}
				_actor.sendPacket(new MyTargetSelected(target.getObjectId(), 0));
			}
	}

	@Override
	public void Attack(L2Object target, boolean forceUse, boolean dontMove)
	{
		// Если не можем атаковать, то атаковать позже
		if(target.isCharacter &&(_actor.isActionsDisabled() || _actor.isAttackingDisabled()))
		{
			setNextAction(nextAction.ATTACK, target, null, forceUse, false);
			_actor.sendActionFailed();
			return;
		}

		_forceUse = forceUse;
		_dontMove = dontMove;
		clearNextAction();
		setIntention(AI_INTENTION_ATTACK, target);
	}

	@Override
	public void Cast(L2Skill skill, L2Character target, boolean forceUse, boolean dontMove)
	{
		// Если скилл альтернативного типа (например, бутылка на хп),
		// то он может использоваться во время каста других скиллов, или во время атаки, или на бегу.
		// Поэтому пропускаем дополнительные проверки.
		if(skill.altUse())
		{
			if(!_actor.isPotionsDisabled())
			{
				_actor.altUseSkill(skill, target);
			}
			else
				clientActionFailed();
		}
		else
		{
			// Если не можем кастовать, то использовать скилл позже
			if(_actor.isActionsDisabled())
			{
				if(!_actor.isSkillDisabled(skill.getId()))
					setNextAction(nextAction.CAST, skill, target, forceUse, dontMove);
				clientActionFailed();
			}
			else
			{
				//actor.stopMove(null);
				_forceUse = forceUse;
				_dontMove = dontMove;
				clearNextAction();
				setIntention(CtrlIntention.AI_INTENTION_CAST, skill, target);
			}
		}
	}

	@Override
	public L2Playable getActor()
	{
		return (L2Playable) super.getActor();
	}
	public boolean isForceCast()
	{
		return _forceUse;
	}
}