package l2r.gameserver.ai;

import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_IDLE;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_REST;
import l2r.Config;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;

public class L2PlayerAI extends L2PlayableAI
{
	public L2PlayerAI(L2Player actor)
	{
		super(actor);
	}

	@Override
	protected void onIntentionRest()
	{
		changeIntention(AI_INTENTION_REST, null, null);
		_attack_target = null;
		clientStopMoving();
	}

	@Override
	protected void onIntentionActive()
	{
		changeIntention(AI_INTENTION_IDLE, null, null);
	}

	@Override
	public void onIntentionInteract(L2Object object)
	{
		if(getActor().getSittingTask())
		{
			setNextAction(nextAction.INTERACT, object, null, false, false);
			return;
		}
		else if(getActor().isSitting())
		{
			getActor().sendPacket(Msg.YOU_CANNOT_MOVE_WHILE_SITTING);
			clientActionFailed();
			return;
		}
		super.onIntentionInteract(object);
	}

	@Override
	public void onIntentionPickUp(L2Object object)
	{
		if(getActor().getSittingTask())
		{
			setNextAction(nextAction.PICKUP, object, null, false, false);
			return;
		}
		else if(getActor().isSitting())
		{
			getActor().sendPacket(Msg.YOU_CANNOT_MOVE_WHILE_SITTING);
			clientActionFailed();
			return;
		}
		super.onIntentionPickUp(object);
	}

	@Override
	public void Attack(L2Object target, boolean forceUse, boolean dontMove)
	{
		if(getActor().getSittingTask())
		{
			setNextAction(nextAction.ATTACK, target, null, forceUse, dontMove);
			return;
		}
		else if(getActor().isSitting())
		{
			getActor().sendPacket(Msg.YOU_CANNOT_MOVE_WHILE_SITTING);
			clientActionFailed();
			return;
		}
		super.Attack(target, forceUse, dontMove);
	}

	@Override
	public void Cast(L2Skill skill, L2Character target, boolean forceUse, boolean dontMove)
	{
		if(!skill.altUse() && !(skill.getSkillType() == L2Skill.SkillType.CRAFT && Config.ALLOW_TALK_WHILE_SITTING))
			// Если в этот момент встаем, то использовать скилл когда встанем
			if(getActor().getSittingTask())
			{
				setNextAction(nextAction.CAST, skill, target, forceUse, dontMove);
				clientActionFailed();
				return;
			}
			// если сидим - скиллы нельзя использовать
			else if(getActor().isSitting())
			{
				getActor().sendPacket(Msg.YOU_CANNOT_MOVE_WHILE_SITTING);
				clientActionFailed();
				return;
			}
			else if(getActor().getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE && (skill.getSkillType() == L2Skill.SkillType.SUMMON || skill.getSkillType() == L2Skill.SkillType.SUMMON_ITEM))
			{
				_actor.sendPacket(Msg.YOU_CANNOT_SUMMON_DURING_A_TRADE_OR_WHILE_USING_THE_PRIVATE_SHOPS);
				clientActionFailed();
				return;
			}
		super.Cast(skill, target, forceUse, dontMove);
	}

	@Override
	protected void onEvtAttacked(L2Character attacker, int damage)
	{
		// notify the tamed beast of attacks
		if(getActor().getTrainedBeast() != null)
			getActor().getTrainedBeast().onOwnerGotAttacked(attacker);
		super.onEvtAttacked(attacker, damage);
	}

	@Override
	public L2Player getActor()
	{
		return (L2Player) super.getActor();
	}
}