package l2r.gameserver.ai;

import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_ATTACK;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Summon;

public class L2SummonAI extends L2PlayableAI
{
	public L2SummonAI(L2Summon actor)
	{
		super(actor);
	}

	@Override
	protected void onIntentionIdle()
	{
		getActor().stopMove();
		clearNextAction();
		onIntentionActive();
	}

	@Override
	protected void onIntentionActive()
	{
		if(!_actor.isVisible())
			return;

		clearNextAction();
		if(getActor().isPosessed())
		{
			getActor().setRunning();
			if(getIntention() != AI_INTENTION_ATTACK)
				setIntention(CtrlIntention.AI_INTENTION_ATTACK, getActor().getOwner(), null);
			return;
		}

		if(getActor().getFollowStatus())
			setIntention(CtrlIntention.AI_INTENTION_FOLLOW, getActor().getOwner(), 100);
		else
			super.onIntentionActive();
	}

	@Override
	protected void thinkAttack(boolean checkRange)
	{
		if(getActor().isPosessed())
			_attack_target = getActor().getOwner();

		if(_attack_target == null || !GeoClient.getInstance().canSeeTarget(_actor, _attack_target) || _attack_target.isDead())
			getActor().setFollowStatus(true);
		else
			super.thinkAttack(checkRange);
	}

	@Override
	protected void onEvtThink()
	{
		if(getActor().isPosessed())
		{
			_attack_target = getActor().getOwner();
			_intention = AI_INTENTION_ATTACK;
		}

		super.onEvtThink();
	}

	@Override
	protected void onEvtAttacked(L2Character attacker, int damage)
	{
		L2Summon actor = getActor();
		if(actor == null)
			return;
		if(attacker != null && actor.getPlayer().isDead() && !actor.isPosessed())
			Attack(attacker, false);
		super.onEvtAttacked(attacker, damage);
	}

	@Override
	protected void onEvtFinishCasting()
	{
		if(getActor().getFollowStatus())
			setIntention(CtrlIntention.AI_INTENTION_FOLLOW, getActor().getOwner(), 100);
		super.onEvtFinishCasting();
	}

	@Override
	public L2Summon getActor()
	{
		return (L2Summon) super.getActor();
	}
}