package l2r.gameserver.ai;

import l2r.gameserver.instancemanager.SiegeManager;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.instances.L2SiegeGuardInstance;
import l2r.util.Rnd;

public abstract class SiegeGuard extends DefaultAI
{
	protected L2SiegeGuardInstance _thisActor;

	public SiegeGuard(L2Character actor)
	{
		super(actor);
		_thisActor = (L2SiegeGuardInstance) actor;
		MAX_PATHFIND_FAILS = Integer.MAX_VALUE;
		MAX_ATTACK_TIMEOUT = Integer.MAX_VALUE;
		AI_TASK_DELAY = 1000;
	}

	@Override
	protected boolean randomWalk()
	{
		return false;
	}

	@Override
	protected boolean randomAnimation()
	{
		return false;
	}

	@Override
	protected boolean isSilentMoveNotVisible(L2Playable target)
	{
		// Осадные гварды могут видеть игроков в режиме Silent Move с вероятностью 1%
		return target.isSilentMoving() && Rnd.chance(90);
	}

	@Override
	public void checkAggression(L2Character target)
	{
		if(!(target instanceof L2Playable))
			return;
		if(!_thisActor.isAutoAttackable(target))
			return;
		super.checkAggression(target);
	}

	@Override
	protected void onEvtAttacked(L2Character attacker, int damage)
	{
		if(attacker != null && attacker.getPlayer() != null)
		{
			L2Clan clan = attacker.getPlayer().getClan();
			//TODO присвоить осаду гварду при спавне, чтобы избавиться от перебора
			if(clan != null && SiegeManager.getSiege(_thisActor) == clan.getSiege() && clan.isDefender())
				return;
		}
		super.onEvtAttacked(attacker, damage);
	}

	@Override
	protected void onEvtAggression(L2Character target, int aggro)
	{
		if(target != null && target.getPlayer() != null)
		{
			L2Clan clan = target.getPlayer().getClan();
			//TODO присвоить осаду гварду при спавне, чтобы избавиться от перебора
			if(clan != null && SiegeManager.getSiege(_thisActor) == clan.getSiege() && clan.isDefender())
				return;
		}
		super.onEvtAggression(target, aggro);
	}

	@Override
	protected boolean thinkActive()
	{
		if(_globalAggro < 0)
			_globalAggro++;
		else if(_globalAggro > 0)
			_globalAggro--;

		if(_thisActor.isDead())
			return true;

		if(_def_think)
		{
			doTask();
			return true;
		}

		_thisActor.setRunning();
		_thisActor.setAttackTimeout(Integer.MAX_VALUE);

		int homeX = _thisActor.getX() - _thisActor.getSpawnedLoc().getX();
		int homeY = _thisActor.getY() - _thisActor.getSpawnedLoc().getY();

		// Проверка на расстояние до точки спауна
		if(homeX * homeX + homeY * homeY > 100 * 100)
		{
			returnHome();
			return true;
		}

		return false;
	}

	@Override
	protected void thinkAttack()
	{
		_thisActor.setRunning();
		_thisActor.setAttackTimeout(Integer.MAX_VALUE);

		int homeX = _thisActor.getX() - _thisActor.getSpawnedLoc().getX();
		int homeY = _thisActor.getY() - _thisActor.getSpawnedLoc().getY();

		// Проверка на расстояние до точки спауна
		if(homeX * homeX + homeY * homeY > 1500 * 1500)
		{
			returnHome();
			return;
		}

		super.thinkAttack();
	}
}