package l2r.gameserver.ai;

import java.util.ArrayList;

import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Skill.SkillType;
import l2r.util.Rnd;

public class SiegeGuardPriest extends SiegeGuard
{
	public SiegeGuardPriest(L2Character actor)
	{
		super(actor);
		_isMobile = false;
	}

	@Override
	protected boolean createNewTask()
	{
		// Удаляем все задания
		clearTasks();

		L2Character _temp_attack_target = _attack_target;

		// Новая цель исходя из агрессивности
		L2Character hated = _thisActor.isConfused() ? _temp_attack_target : _thisActor.getMostHated();

		if(hated != null && hated != _thisActor)
		{
			_attack_target = hated;
			_temp_attack_target = hated;
		}
		else
		{
			returnHome();
			return false;
		}

		// Базовые параметры
		int phys_per = 25;
		int mdam_per = 50;

		ArrayList<L2Skill> d_skill = new ArrayList<L2Skill>();
		ArrayList<L2Skill> t_skill = new ArrayList<L2Skill>();
		L2Skill r_skill = null;

		double distance = _thisActor.getDistance(_temp_attack_target);
		double _def_mp = _thisActor.getCurrentMp();

		if(!Rnd.chance(phys_per))
		{
			int rnd_per = Rnd.get(100);

			// MDAM
			if(rnd_per < mdam_per && _mdam.length > 0)
			{
				t_skill.clear();
				for(L2Skill sk : _mdam)
				{
					if(sk == null || sk.getCastRange() > 200 && distance <= 200)
						continue;
					if(sk.getCastRange() <= 200 && distance > 200)
						continue;
					t_skill.add(sk);
				}
				if(t_skill.size() > 0)
					d_skill.add(t_skill.get(Rnd.get(t_skill.size())));
			}

			// HEAL
			if(_heal.length > 0)
			{
				t_skill.clear();
				for(L2Skill sk : _heal)
					t_skill.add(sk);
				if(t_skill.size() > 0)
					d_skill.add(t_skill.get(Rnd.get(t_skill.size())));
			}

			// Выбрать 1 скилл из полученных
			if(d_skill.size() > 0)
				r_skill = d_skill.get(Rnd.get(d_skill.size()));

			// Смена цели для HEAL скиллов
			if(r_skill != null && r_skill.getSkillType() == SkillType.HEAL)
				if(_thisActor.getMaxHp() - _thisActor.getCurrentHp() > r_skill.getPower())
					_temp_attack_target = _thisActor;
				else
					r_skill = null;
		}

		// Использовать скилл если можно, иначе атаковать
		if(r_skill != null && !(_thisActor.isMuted() && r_skill.isMagic() || _thisActor.isPMuted() && !r_skill.isMagic()) && _def_mp >= r_skill.getMpConsume() && !_thisActor.isSkillDisabled(r_skill.getId()))
		{
			// Проверка таргета
			if(r_skill.getTargetType() == L2Skill.SkillTargetType.TARGET_SELF)
				_temp_attack_target = _thisActor;
			if(r_skill.getTargetType() == L2Skill.SkillTargetType.TARGET_AURA)
				_temp_attack_target = _thisActor;

			// Добавить новое задание
			Task task = new Task();
			task.type = r_skill.isOffensive() ? TaskType.CAST : TaskType.BUFF;
			task.target = _temp_attack_target;
			task.skill = r_skill;
			_task_list.add(task);
			_def_think = true;
			return true;
		}

		// Добавить новое задание
		Task task = new Task();
		task.type = TaskType.ATTACK;
		task.target = _temp_attack_target;
		_task_list.add(task);
		_def_think = true;
		return true;
	}
}