package l2r.gameserver.clientpackets;

import l2r.Config;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.loginservercon.LSConnection;
import l2r.gameserver.loginservercon.SessionKey;
import l2r.gameserver.network.L2GameClient;

/**
 * This class ...
 *
 * @version $Revision: 1.9.2.3.2.4 $ $Date: 2005/03/27 15:29:30 $
 */
public class AuthLogin extends L2GameClientPacket
{
	private static String _C__08_AUTHLOGIN = "[C] 08 AuthLogin";
	//Format: cSddddd

	// loginName + keys must match what the loginserver used.
	private String _loginName;
	private int _playKey1;
	private int _playKey2;
	private int _loginKey1;
	private int _loginKey2;

	@Override
	public void readImpl()
	{
		_loginName = readS().toLowerCase();
		_playKey2 = readD();
		_playKey1 = readD();
		_loginKey1 = readD();
		_loginKey2 = readD();
		// ignore the rest
		_buf.clear();
	}

	@Override
	public void runImpl()
	{
		SessionKey key = new SessionKey(_loginKey1, _loginKey2, _playKey1, _playKey2);

		final L2GameClient client = getClient();
		client.setSessionId(key);
		client.setLoginName(_loginName);
		if(Config.GG_CHECK)
		{
			client.sendPacket(Msg.GameGuardQuery);
			ThreadPoolManager.getInstance().scheduleGeneral(new GGTest(client), 500);
		}
		LSConnection.getInstance().addWaitingClient(client);
	}

	/* (non-Javadoc)
	 * @see l2r.gameserver.clientpackets.ClientBasePacket#getType()
	 */
	@Override
	public String getType()
	{
		return _C__08_AUTHLOGIN;
	}

	private class GGTest implements Runnable
	{
		private final L2GameClient targetClient;

		private int attempts = 0;

		public GGTest(L2GameClient targetClient)
		{
			this.targetClient = targetClient;
		}

		public void run()
		{
			if(!targetClient.isGameGuardOk())
				if(attempts < 3)
				{
					targetClient.sendPacket(Msg.GameGuardQuery);
					attempts++;
					ThreadPoolManager.getInstance().scheduleGeneral(this, 500 * attempts);
				}
				else
					targetClient.closeNow(false);
		}
	}
}
