package l2r.gameserver.clientpackets;

import l2r.Config;
import l2r.gameserver.instancemanager.QuestManager;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2ShortCut;
import l2r.gameserver.model.L2SkillLearn;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.base.ClassId;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.network.L2GameClient;
import l2r.gameserver.serverpackets.CharCreateFail;
import l2r.gameserver.serverpackets.CharCreateOk;
import l2r.gameserver.serverpackets.CharacterSelectionInfo;
import l2r.gameserver.tables.CharNameTable;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.tables.SkillTreeTable;
import l2r.gameserver.templates.L2Item;
import l2r.gameserver.templates.L2PlayerTemplate;
import l2r.util.Util;

/**
 * This class ...
 *
 * @version $Revision: 1.9.2.3.2.8 $ $Date: 2005/03/27 15:29:30 $
 */
public class CharacterCreate extends L2GameClientPacket
{
	private static String _C__0B_CHARACTERCREATE = "[C] 0B CharacterCreate";

	// cSdddddddddddd
	private String _name;
	private int _sex;
	private int _classId;
	private int _hairStyle;
	private int _hairColor;
	private int _face;

	@Override
	public void readImpl()
	{
		_name = readS();
		readD(); // race
		_sex = readD();
		_classId = readD();
		readD(); // int
		readD(); // str
		readD(); // con
		readD(); // men
		readD(); // dex
		readD(); // wit
		_hairStyle = readD();
		_hairColor = readD();
		_face = readD();
	}

	@Override
	public void runImpl()
	{
		for(ClassId cid : ClassId.values())
			if(cid.getId() == _classId && cid.getLevel() != 1)
				return;
		if(CharNameTable.getInstance().accountCharNumber(getClient().getLoginName()) >= 8)
		{
			sendPacket(new CharCreateFail(CharCreateFail.REASON_TOO_MANY_CHARACTERS));
			return;
		}
		if(!Util.isMatchingRegexp(_name, Config.CNAME_TEMPLATE))
		{
			sendPacket(new CharCreateFail(CharCreateFail.REASON_16_ENG_CHARS));
			return;
		}
		else if(CharNameTable.getInstance().doesCharNameExist(_name))
		{
			sendPacket(new CharCreateFail(CharCreateFail.REASON_NAME_ALREADY_EXISTS));
			return;
		}

		L2Player newChar = L2Player.create((short) _classId, (byte) _sex, getClient().getLoginName(), _name, (byte) _hairStyle, (byte) _hairColor, (byte) _face);
		if(newChar == null)
			return;

		sendPacket(new CharCreateOk());

		initNewChar(getClient(), newChar);
	}

	private void initNewChar(L2GameClient client, L2Player newChar)
	{
		L2World.addObject(newChar);

		L2PlayerTemplate template = newChar.getTemplate();

		L2Player.restoreCharSubClasses(newChar);

		if(Config.STARTING_ADENA > 0)
			newChar.addAdena(Config.STARTING_ADENA);

		newChar.setXYZInvisible(template.spawnX, template.spawnY, template.spawnZ);
		newChar.setTitle("");

		// add attack, take, sit shortcut
		newChar.registerShortCut(new L2ShortCut(0, 0, 3, 2, -1));
		newChar.registerShortCut(new L2ShortCut(3, 0, 3, 5, -1));
		newChar.registerShortCut(new L2ShortCut(10, 0, 3, 0, -1));

		ItemTable itemTable = ItemTable.getInstance();
		for(L2Item i : template.getItems())
		{
			L2ItemInstance item = itemTable.createItem(i.getItemId(), newChar.getObjectId(), 0, "CharacterCreate[178]");
			newChar.getInventory().addItem(item);

			if(item.getItemId() == 5588) // tutorial book
				newChar.registerShortCut(new L2ShortCut(11, 0, 1, item.getObjectId(), -1));

			if(item.isEquipable())
				if(newChar.getActiveWeaponItem() == null || item.getItem().getType2() == L2Item.TYPE2_WEAPON)
					newChar.getInventory().equipItem(item);
		}

		for(L2SkillLearn skill : SkillTreeTable.getInstance().getAvailableSkills(newChar, newChar.getClassId()))
		{
			int skillId = skill.id;
			newChar.addSkill(SkillTable.getInstance().getInfo(skillId, skill.skillLevel), true);
			if(skillId == 1001 || skillId == 1177)
				newChar.registerShortCut(new L2ShortCut(1, 0, 2, skillId, 1));

			if(skillId == 1216)
				newChar.registerShortCut(new L2ShortCut(10, 0, 2, skillId, 1));
		}

		//startTutorialQuest(newChar);

		L2GameClient.saveCharToDisk(newChar);
		newChar.deleteMe(); // release the world of this character and it's inventory

		CharacterSelectionInfo cl = new CharacterSelectionInfo(client.getLoginName(), client.getSessionId().playOkID1);
		client.getConnection().sendPacket(cl);
		client.setCharSelection(cl.getCharInfo());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see l2r.gameserver.clientpackets.ClientBasePacket#getType()
	 */
	@Override
	public String getType()
	{
		return _C__0B_CHARACTERCREATE;
	}

	public static void startTutorialQuest(L2Player player)
	{
		Quest q = QuestManager.getQuest(255);
		if(q != null)
			q.newQuestState(player);
	}
}
