package l2r.gameserver.clientpackets;

import static l2r.gameserver.model.L2Zone.ZoneType.no_restart;

import java.io.File;
import java.sql.ResultSet;
import java.util.logging.Logger;

import l2r.Config;
import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.extensions.multilang.CustomMessage;
import l2r.extensions.scripts.Scripts;
import l2r.gameserver.Announcements;
import l2r.gameserver.GameTimeController;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.instancemanager.CastleManager;
import l2r.gameserver.instancemanager.CoupleManager;
import l2r.gameserver.instancemanager.CursedWeaponsManager;
import l2r.gameserver.instancemanager.DimensionalRiftManager;
import l2r.gameserver.instancemanager.QuestManager;
import l2r.gameserver.instancemanager.SiegeManager;
import l2r.gameserver.instancemanager.PlayerMessageStack;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.L2Zone.ZoneType;
import l2r.gameserver.model.entity.SevenSigns;
import l2r.gameserver.model.entity.siege.Siege;
import l2r.gameserver.model.entity.siege.SiegeUnit;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.serverpackets.*;
import l2r.gameserver.serverpackets.FriendList;
import l2r.gameserver.serverpackets.FriendStatus;
import l2r.gameserver.tables.MapRegionTable;
import l2r.gameserver.tables.SkillTable;
import l2r.util.Log;

/**
 * Enter World Packet Handler<p>
 * <p>
 * 0000: 11 <p>
 */
public class EnterWorld extends L2GameClientPacket
{
	private static String _C__03_ENTERWORLD = "[C] 03 EnterWorld";
	//Format1(EnterWorldPacket): cbddddbdcccccccccccccccccccc
	//Format2(EnterWorld): cS
	private static Logger _log = Logger.getLogger(EnterWorld.class.getName());

	@Override
	public void readImpl()
	{}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();

		if(activeChar == null)
		{
			getClient().closeNow(false);
			return;
		}

		if(activeChar.isGM() && !Config.SHOW_GM_LOGIN)
			activeChar.setInvisible(true);

		activeChar.spawnMe();
		activeChar.updateTerritories();

		//sendPacket(new UserInfo(activeChar));
		sendPacket(new HennaInfo(activeChar));
		sendPacket(new ItemList(activeChar, false));
		sendPacket(new ShortCutInit(activeChar));
		activeChar.getMacroses().sendUpdate();
		sendPacket(new ClientSetTime());
		if(Config.DAY_STATUS_FORCE_CLIENT_UPDATE)
			if(GameTimeController.getInstance().isNowNight())
				sendPacket(Msg.SunSet);
			else
				sendPacket(Msg.SunRise);
		sendPacket(new SkillList(activeChar));
		sendPacket(new SystemMessage(SystemMessage.WELCOME_TO_THE_WORLD_OF_LINEAGE_II));

		Announcements.getInstance().showAnnouncements(activeChar);

		//add char to online characters
		activeChar.setOnlineStatus(true);

		// Вызов всех хэндлеров, определенных в скриптах
		for(String handler : Scripts.onPlayerEnter)
		{
			String[] list = handler.split(":");
			String _class = list[0];
			String _method = list[1];
			activeChar.callScripts(_class, _method, new Object[] { activeChar });
		}

		SevenSigns.getInstance().sendCurrentPeriodMsg(activeChar);

		// send user info again .. just like the real client
		// sendPacket(new UserInfo(activeChar));

		if(activeChar.getClanId() != 0 && activeChar.getClan() != null)
		{
			notifyClanMembers(activeChar);
			sendPacket(new PledgeShowMemberListAll(activeChar.getClan(), activeChar));
			sendPacket(new PledgeShowInfoUpdate(activeChar.getClan()));
			sendPacket(new PledgeSkillList(activeChar.getClan()));

			for(SiegeUnit castle : CastleManager.getInstance().getCastles().values())
			{
				Siege siege = castle.getSiege();
				if(!siege.isInProgress())
					continue;
				if(siege.checkIsAttacker(activeChar.getClan()))
					activeChar.setSiegeState(1);
				else if(siege.checkIsDefender(activeChar.getClan()))
					activeChar.setSiegeState(2);
			}
		}

		// engage and notify Partner
		if(Config.ALLOW_WEDDING)
		{
			CoupleManager.getInstance().engage(activeChar);
			CoupleManager.getInstance().notifyPartner(activeChar);
		}

		Log.LogChar(activeChar, Log.EnterWorld, "");

		notifyFriends(activeChar, true);

		//loadTutorial(activeChar);

		sendPacket(new ExStorageMaxCount(activeChar));
		sendPacket(new QuestList(activeChar));

		if(Config.ENABLE_OLYMPIAD && activeChar.isInZoneOlympiad())
		{
			activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.EnterWorld.TeleportedReasonOlympiad", activeChar));
			activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown));
		}

		if(activeChar.isInZoneIncludeZ(no_restart))
		{
			long allowed_time = activeChar.getZone(no_restart).getRestartTime();
			long last_time = activeChar.getLastAccess();
			long curr_time = System.currentTimeMillis() / 1000;

			if(curr_time - last_time > allowed_time)
			{
				activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.EnterWorld.TeleportedReasonNoRestart", activeChar));
				activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown));
			}
		}

		if(!activeChar.isGM() && activeChar.isInZone(ZoneType.Siege))
		{
			Siege siege = SiegeManager.getSiege(activeChar.getX(), activeChar.getY());
			if(siege != null && !siege.checkIsDefender(activeChar.getClan()))
				if(siege.getHeadquarter(activeChar.getClan()) == null)
					activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown));
				else
					activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.Headquarter));
		}

		if(activeChar.isDead())
		{
			sendPacket(new UserInfo(activeChar));
			// no broadcast needed since the player will already spawn dead to others
			sendPacket(new Die(activeChar));
		}

		// activeChar.broadcastUserInfo(); уже шлется в restoreEffects()
		activeChar.restoreEffects();
		activeChar.restoreDisableSkills();

		CursedWeaponsManager.getInstance().checkPlayer(activeChar);

		// refresh player info
		sendPacket(new EtcStatusUpdate(activeChar));
		//activeChar.getInventory().refreshListeners();
		activeChar.checkHpMessages(activeChar.getMaxHp(), activeChar.getCurrentHp());
		activeChar.checkDayNightMessages();

		if(DimensionalRiftManager.getInstance().checkIfInRiftZone(activeChar.getLoc(), false))
			DimensionalRiftManager.getInstance().teleportToWaitingRoom(activeChar);

		sendPacket(new ExPCCafePointInfo(activeChar));

		if(Config.SHOW_HTML_WELCOME)
		{
			String welcomePath = "data/html/welcome.htm";
			File mainText = new File(Config.DATAPACK_ROOT, welcomePath); // Return the pathfile of the HTML file
			if(mainText.exists())
			{
				NpcHtmlMessage html = new NpcHtmlMessage(1);
				html.setFile(welcomePath);
				sendPacket(html);
			}
		}

		if(activeChar.isSitting())
			activeChar.sendPacket(new ChangeWaitType(activeChar, ChangeWaitType.WT_SITTING));
		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
			if(activeChar.getPrivateStoreType() == L2Player.STORE_PRIVATE_BUY)
				sendPacket(new PrivateStoreMsgBuy(activeChar));
			else if(activeChar.getPrivateStoreType() == L2Player.STORE_PRIVATE_SELL || activeChar.getPrivateStoreType() == L2Player.STORE_PRIVATE_SELL_PACKAGE)
				sendPacket(new PrivateStoreMsgSell(activeChar));
			else if(activeChar.getPrivateStoreType() == L2Player.STORE_PRIVATE_MANUFACTURE)
				sendPacket(new RecipeShopMsg(activeChar));

		activeChar.unsetVar("offline");

		// на всякий случай
		activeChar.sendActionFailed();

		if(activeChar.isGM() && Config.SAVE_GM_EFFECTS && activeChar.getPlayerAccess().CanUseGMCommand)
		{
			//silence
			if(activeChar.getVarB("gm_silence"))
			{
				activeChar.setMessageRefusal(true);
				activeChar.sendPacket(new SystemMessage(SystemMessage.MESSAGE_REFUSAL_MODE));
			}
			//invul
			if(activeChar.getVarB("gm_invul"))
			{
				activeChar.setIsInvul(true);
				activeChar.sendMessage(activeChar.getName() + " is now immortal.");
			}
			//gmspeed
			try
			{
				int var_gmspeed = Integer.parseInt(activeChar.getVar("gm_gmspeed"));
				if(var_gmspeed >= 1 && var_gmspeed <= 4)
					activeChar.doCast(SkillTable.getInstance().getInfo(7029, var_gmspeed), activeChar, true);
			}
			catch(Exception E)
			{}
		}

		PlayerMessageStack.getInstance().CheckMessages(activeChar);
	}

	public static void notifyFriends(L2Player cha, boolean login)
	{
		if(login)
			cha.sendPacket(new FriendList(cha, false));
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT friend_id from character_friends where char_id=?");
			statement.setInt(1, cha.getObjectId());
			rset = statement.executeQuery();
			while(rset.next())
			{
				int objectId = rset.getInt("friend_id");
				L2Player friend = (L2Player) L2World.findObject(objectId);
				if(friend != null)
					if(login)
					{
						friend.sendPacket(new SystemMessage(SystemMessage.S1_FRIEND_HAS_LOGGED_IN).addString(cha.getName()));
						friend.sendPacket(new FriendStatus(cha, true));
					}
					else
					{
						friend.sendPacket(new FriendStatus(cha, false));
						cha.sendPacket(new FriendList(cha, false));
					}
			}
		}
		catch(Exception e)
		{
			_log.warning("could not restore friend data:" + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	/**
	 * @param activeChar
	 */
	private void notifyClanMembers(L2Player activeChar)
	{
		L2Clan clan = activeChar.getClan();
		if(clan == null || clan.getClanMember(activeChar.getObjectId()) == null)
			return;

		clan.getClanMember(activeChar.getObjectId()).setPlayerInstance(activeChar);
		//if(activeChar.isClanLeader())
		//{
		//	if(activeChar.getClan().getHasHideout() != 0 && ClanHallManager.getInstance().getClanHall(activeChar.getClan().getHasHideout()).getNotPaid())
		//		activeChar.sendPacket(new SystemMessage(SystemMessage.THE_CLAN_HALL_FEE_IS_ONE_WEEK_OVERDUE_THEREFORE_THE_CLAN_HALL_OWNERSHIP_HAS_BEEN_REVOKED));
		//}

		int sponsor = activeChar.getSponsor();
		int apprentice = activeChar.getApprentice();
		SystemMessage msg = new SystemMessage(SystemMessage.CLAN_MEMBER_S1_HAS_LOGGED_INTO_GAME).addString(activeChar.getName());
		PledgeShowMemberListUpdate memberUpdate = new PledgeShowMemberListUpdate(activeChar);
		for(L2Player clanMember : clan.getOnlineMembers(activeChar.getName()))
		{
			clanMember.sendPacket(memberUpdate);
			if(clanMember.getObjectId() == sponsor)
				clanMember.sendPacket(new SystemMessage(SystemMessage.S1_YOUR_CLAN_ACADEMYS_APPRENTICE_HAS_LOGGED_IN).addString(activeChar.getName()));
			else if(clanMember.getObjectId() == apprentice)
				clanMember.sendPacket(new SystemMessage(SystemMessage.S1_YOUR_CLAN_ACADEMYS_SPONSOR_HAS_LOGGED_IN).addString(activeChar.getName()));
			else
				clanMember.sendPacket(msg);
		}
	}

	private void loadTutorial(L2Player player)
	{
		Quest q = QuestManager.getQuest(255);
		if(q != null)
			player.processQuestEvent(q.getName(), "UC");
	}

	@Override
	public String getType()
	{
		return _C__03_ENTERWORLD;
	}
}