package l2r.gameserver.clientpackets;

import java.util.logging.Logger;

import l2r.Config;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.network.L2GameClient;
import l2r.gameserver.serverpackets.KeyPacket;
import l2r.util.Protection;

/**
 * This class ...
 *
 * @version $Revision: 1.5.2.8.2.8 $ $Date: 2005/04/02 10:43:04 $
 * Пример пакета:
 * 00 EA 02 00 00 09 07 54 56 03 09 0B 01 07 02 54 54 56 07 00 02 55 56 00 51 00 53 57 04 07 55 08 54 01 07 01 53 00 56 55 56 01 06 05 04 51 03 08 51 08 51 56 04 54 06 55 08 02 09 51 56 01 53 06 55 04 53 00 56 56 53 01 09 02 09 01 51 54 51 09 55 56 09 03 04 07 05 55 04 06 55 04 06 09 04 51 01 08 08 06 05 52 06 04 01 07 54 03 06 52 55 06 55 55 51 01 02 04 54 03 55 54 01 57 51 55 05 52 05 54 07 51 51 55 07 02 53 53 00 52 05 52 07 01 54 00 03 05 05 08 06 05 05 06 03 00 0D 08 01 07 09 03 51 03 07 53 09 51 06 07 54 0A 50 56 02 52 04 05 55 51 02 53 00 08 54 04 52 56 06 02 09 00 08 03 53 56 01 05 00 55 06 08 56 04 0D 06 07 52 06 07 04 0A 06 01 04 54 04 00 05 02 04 54 00 09 52 53 05 04 01 04 05 05 01 52 51 52 0D 06 51 08 09 54 53 00 0D 01 02 03 54 53 01 05 03 08 56 54 07 02 54 0B 06 DC 4F 61 4F
 * где EA 02 00 00 = 746
 */
public class ProtocolVersion extends L2GameClientPacket
{
	private static String _C__00_PROTOCOLVERSION = "[C] 00 ProtocolVersion";
	static Logger _log = Logger.getLogger(ProtocolVersion.class.getName());

	private KeyPacket pk;
	private long _version;
	private byte[] _data;
	private static byte[] _xorB = { (byte) 0x4C, (byte) 0x32, (byte) 0x52, (byte) 0x2D, (byte) 0x44, (byte) 0x52, (byte) 0x69, (byte) 0x4E };

	/**
	 * packet type id 0x0E
	 * format:	cdbd
	 *
	 * @param rawPacket
	 */

	@Override
	public void readImpl()
	{
		if(_buf.remaining() < 4)
		{
			// Проверки рейтинга типа l2top.in.ua
			_client.closeNow(true);
			return;
		}
		L2GameClient _client = getClient();
		pk = new KeyPacket(_client.enableCrypt());
		_version = readD();
		if(_version == -2)
		{
			_client.closeNow(false);
			return;
		}
		try
		{
			_data = new byte[256];
			readB(_data);
			readD();
		}
		catch(Exception e)
		{
			if(_client.protect_used)
			{
				_log.info("Filed read ProtocolVersion! May be BOT or unprotected client! Client IP: " + _client.getIpAddr());
				_client.closeNow(true);
				return;
			}
		}
	}

	@Override
	public void runImpl()
	{
		if(_version == -2)
			return;
		L2GameClient _client = getClient();
		if(_version < Config.MIN_PROTOCOL_REVISION || _version > Config.MAX_PROTOCOL_REVISION)
		{
			_log.info("Client Protocol Revision: " + _version + ", client IP: " + _client.getIpAddr() + " not allowed. Supported protocols: from " + Config.MIN_PROTOCOL_REVISION + " to " + Config.MAX_PROTOCOL_REVISION + ". Closing connection.");
			_client.sendPacket(Msg.PROTOCOL_VERSION_IS_DIFFERENT_PLEASE_QUIT_THE_PROGRAM); //FIXME не работает
			_client.closeNow(false);
			return;
		}
		_client.setRevision((int) _version);
		if(Config.DEBUG)
			_log.fine("Client Protocol Revision is ok:" + _version);

		for(int i = 0; i < 256; i++)
			_data[i] ^= _xorB[i & 7];
		_client.client_lang = _data[255];
		if(_client.client_lang != 0 && _client.client_lang != 1)
			_client.client_lang = -1;
		if(_client.protect_used)
		{
			if(_client.client_lang == -1)
			{
				_log.info("ProtocolVersion lang wrong! May be BOT or unprotected client! Client IP: " + _client.getIpAddr());
				_client.closeNow(true);
				return;
			}
			_client.HWID = Protection.ExtractHWID(_data);
			if(_client.HWID == "")
			{
				_log.info("ProtocolVersion CRC Check Fail! May be BOT or unprotected client! Client IP: " + _client.getIpAddr());
				_client.closeNow(true);
				return;
			}
			if(L2GameClient.checkHWIDBanned(_client.HWID))
			{
				_log.info("Rejected banned HWID: " + _client.HWID + " from IP: " + _client.getIpAddr());
				_client.closeNow(true);
				return;
			}
		}
		_client.sendPacket(pk);
	}

	@Override
	public String getType()
	{
		return _C__00_PROTOCOLVERSION;
	}
}