package l2r.gameserver.clientpackets;

import l2r.Config;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2ShortCut;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2SkillLearn;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.instances.L2VillageMasterInstance;
import l2r.gameserver.serverpackets.AcquireSkillList;
import l2r.gameserver.serverpackets.ExStorageMaxCount;
import l2r.gameserver.serverpackets.ShortCutRegister;
import l2r.gameserver.serverpackets.SkillList;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.SkillSpellbookTable;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.tables.SkillTreeTable;
import l2r.util.Util;

public class RequestAquireSkill extends L2GameClientPacket
{
	private static String _C__6C_REQUESTACQUIRESKILL = "[C] 6C RequestAquireSkill";
	// format: cddd(d)
	private int _id;
	private byte _level;
	private int _skillType;

	@Override
	public void readImpl()
	{
		_id = readD();
		_level = (byte) readD();
		_skillType = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;
		L2NpcInstance trainer = activeChar.getLastNpc();
		if((trainer == null || activeChar.getDistance(trainer.getX(), trainer.getY()) > L2Character.INTERACTION_DISTANCE) && !activeChar.isGM())
			return;

		activeChar.setSkillLearningClassId(activeChar.getClassId());

		L2Skill skill = SkillTable.getInstance().getInfo(_id, _level);

		if(activeChar.getSkillLevel(_id) >= _level)
			return; // already knows the skill with this level

		if(_level > 1 && activeChar.getSkillLevel(_id) != _level - 1)
		{
			Util.handleIllegalPlayerAction(activeChar, "RequestAquireSkill[58]", "tried to increase skill " + _id + " level to " + _level + " while having it's level " + activeChar.getSkillLevel(_id), 2);
			return;
		}

		if(!(skill.isCommon() || SkillTreeTable.getInstance().isSkillPossible(activeChar, _id, _level)))
		{
			Util.handleIllegalPlayerAction(activeChar, "RequestAquireSkill[64]", "tried to learn skill " + _id + " while on class " + activeChar.getActiveClass(), 2);
			return;
		}

		L2SkillLearn SkillLearn = SkillTreeTable.getSkillLearn(_id, _level, activeChar.getClassId(), _skillType == AcquireSkillList.CLAN ? activeChar.getClan() : null);

		if(_skillType == AcquireSkillList.CLAN)
			learnClanSkill(skill, activeChar.getClan());
		else
		{
			int _requiredSp = SkillTreeTable.getInstance().getSkillCost(activeChar, skill);

			if(activeChar.getSp() >= _requiredSp || SkillLearn.common || SkillLearn.transformation)
			{
				Short spb_id = SkillSpellbookTable._skillSpellbooks.get(SkillSpellbookTable.hashCode(new int[] { skill.getId(), skill.getLevel() }));

				if(spb_id != null)
				{
					L2ItemInstance spb = activeChar.getInventory().findItemByItemId(spb_id.intValue());
					if(spb == null || spb.getIntegerLimitedCount() < SkillLearn.itemCount)
					{
						activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_ITEMS_TO_LEARN_SKILLS));
						return;
					}
					L2ItemInstance ri = activeChar.getInventory().destroyItem(spb, SkillLearn.itemCount, true);

					if(ri.getItemId() != 57)
						activeChar.sendPacket(new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(ri.getItemId()));
					else
						activeChar.sendPacket(new SystemMessage(SystemMessage.S1_ADENA_DISAPPEARED).addNumber(SkillLearn.itemCount));
					if(ri.getLastChange() == L2ItemInstance.REMOVED)
						L2World.removeObject(ri);
				}
				activeChar.addSkill(skill, true);
				if(!SkillLearn.common && !SkillLearn.transformation)
					activeChar.setSp(activeChar.getSp() - _requiredSp);
				activeChar.updateStats();
				activeChar.sendChanges();

				//update all the shortcuts to this skill
				if(_level > 1)
					for(L2ShortCut sc : activeChar.getAllShortCuts())
						if(sc.id == _id && sc.type == L2ShortCut.TYPE_SKILL)
						{
							L2ShortCut newsc = new L2ShortCut(sc.slot, sc.page, sc.type, sc.id, _level);
							activeChar.sendPacket(new ShortCutRegister(newsc));
							activeChar.registerShortCut(newsc);
						}
			}
			else
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_SP_TO_LEARN_SKILLS));
				return;
			}
		}

		if(SkillLearn.common)
			activeChar.sendPacket(new ExStorageMaxCount(activeChar));
		activeChar.sendPacket(new SkillList(activeChar));

		if(trainer != null)
			if(_skillType == AcquireSkillList.USUAL)
				trainer.showSkillList(activeChar);
			else if(_skillType == AcquireSkillList.FISHING)
				trainer.showFishingSkillList(activeChar);
			else if(_skillType == AcquireSkillList.CLAN)
				trainer.showClanSkillList(activeChar);
	}

	private void learnClanSkill(L2Skill skill, L2Clan clan)
	{
		L2Player player = getClient().getActiveChar();
		if(player == null || skill == null || clan == null)
			return;
		L2NpcInstance trainer = player.getLastNpc();
		if(trainer == null)
			return;
		if(!(trainer instanceof L2VillageMasterInstance))
		{
			System.out.println("RequestAquireSkill.learnClanSkill, trainer isn't L2VillageMasterInstance");
			System.out.println(trainer.getName() + "[" + trainer.getNpcId() + "] Loc: " + trainer.getLoc());
			return;
		}
		if(!player.isClanLeader())
		{
			player.sendPacket(new SystemMessage(SystemMessage.ONLY_THE_CLAN_LEADER_IS_ENABLED));
			return;
		}
		L2SkillLearn SkillLearn = SkillTreeTable.getSkillLearn(_id, _level, null, clan);
		int _requiredRep = SkillTreeTable.getInstance().getSkillRepCost(clan, skill);
		short itemId = 0;
		if(!Config.ALT_DISABLE_EGGS)
			itemId = SkillLearn.itemId;
		if(skill.getMinPledgeClass() <= clan.getLevel() && clan.getReputationScore() >= _requiredRep)
		{
			if(itemId > 0)
			{
				L2ItemInstance spb = player.getInventory().findItemByItemId(itemId);
				if(spb == null)
				{
					// Haven't spellbook
					player.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_ITEMS_TO_LEARN_SKILLS));
					return;
				}
				L2ItemInstance ri = player.getInventory().destroyItem(spb, SkillLearn.itemCount, true);
				player.sendPacket(new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(ri.getItemId()));
			}
			clan.incReputation(-_requiredRep, false, "AquireSkill");
			clan.addNewSkill(skill);
			player.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_EARNED_S1).addSkillName(_id, (short) _level));

			((L2VillageMasterInstance) trainer).showClanSkillWindow(player); //Maybe we shoud add a check here...
		}
		else
		{
			player.sendMessage("Your clan doesn't have enough reputation points to learn this skill");
			//sm = null;
			return;
		}

		//update all the shortcuts to this skill
		if(_level > 1)
			for(L2ShortCut sc : player.getAllShortCuts())
				if(sc.id == _id && sc.type == L2ShortCut.TYPE_SKILL)
				{
					L2ShortCut newsc = new L2ShortCut(sc.slot, sc.page, sc.type, sc.id, _level);
					player.sendPacket(new ShortCutRegister(newsc));
					player.registerShortCut(newsc);
				}

		clan.addAndShowSkillsToPlayer(player);
	}

	@Override
	public String getType()
	{
		return _C__6C_REQUESTACQUIRESKILL;
	}
}