package l2r.gameserver.clientpackets;

import java.util.Collection;
import java.util.logging.Logger;

import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.serverpackets.SystemMessage;

public class RequestBlock extends L2GameClientPacket
{
	private static String _C__A0_REQUESTBLOCK = "[C] A0 RequestBlock";
	// format: cd(S)
	private static Logger _log = Logger.getLogger(L2Player.class.getName());

	private final static int BLOCK = 0;
	private final static int UNBLOCK = 1;
	private final static int BLOCKLIST = 2;
	private final static int ALLBLOCK = 3;
	private final static int ALLUNBLOCK = 4;

	private Integer _type;
	private L2Player _target;

	@Override
	public void readImpl()
	{
		_type = readD(); //0x00 - block, 0x01 - unblock, 0x03 - allblock, 0x04 - allunblock

		if(_type == BLOCK || _type == UNBLOCK)
			_target = L2World.getPlayer(readS());
		else
			_target = null;
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		switch(_type)
		{
			case BLOCK:
				if(_target == null)
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_FAILED_TO_REGISTER_THE_USER_TO_YOUR_IGNORE_LIST));
					return;
				}
				if(_target.isGM())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_MAY_NOT_IMPOSE_A_BLOCK_ON_A_GM));
					return;
				}
				activeChar.sendPacket(new SystemMessage(SystemMessage.S1_HAS_BEEN_ADDED_TO_YOUR_IGNORE_LIST).addString(_target.getName()));
				_target.sendPacket(new SystemMessage(SystemMessage.S1__HAS_PLACED_YOU_ON_HIS_HER_IGNORE_LIST).addString(activeChar.getName()));
				activeChar.addToBlockList(_target);
				break;
			case UNBLOCK:
				if(_target == null)
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_FAILED_TO_DELETE_THE_CHARACTER));
					return;
				}
				activeChar.sendPacket(new SystemMessage(SystemMessage.S1_HAS_BEEN_REMOVED_FROM_YOUR_IGNORE_LIST).addString(_target.getName()));
				_target.sendMessage(activeChar.getName() + " has removed you from his/her Ignore List."); //В системных(619 == 620) мессагах ошибка ;)
				activeChar.removeFromBlockList(_target);
				break;
			case BLOCKLIST:
				Collection<String> blockList = activeChar.getBlockList();

				if(blockList != null)
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage._IGNORE_LIST_));

					for(String name : blockList)
						activeChar.sendMessage(name);

					activeChar.sendPacket(new SystemMessage(SystemMessage.__EQUALS__));
				}
				break;
			case ALLBLOCK:
				activeChar.setBlockAll(true);
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_ARE_NOW_BLOCKING_EVERYTHING));
				break;
			case ALLUNBLOCK:
				activeChar.setBlockAll(false);
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_ARE_NO_LONGER_BLOCKING_EVERYTHING));
				break;
			default:
				_log.info("Unknown 0x0a block type: " + _type);
		}
	}

	@Override
	public String getType()
	{
		return _C__A0_REQUESTBLOCK;
	}
}