package l2r.gameserver.clientpackets;

import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import l2r.gameserver.communitybbs.CommunityBoard;
import l2r.gameserver.handler.AdminCommandHandler;
import l2r.gameserver.handler.IAdminCommandHandler;
import l2r.gameserver.handler.IVoicedCommandHandler;
import l2r.gameserver.handler.VoicedCommandHandler;
import l2r.gameserver.model.BypassEngine;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.network.L2GameClient;
import l2r.gameserver.serverpackets.NpcHtmlMessage;
import l2r.util.Log;

public class RequestBypassToServer extends L2GameClientPacket
{
	private static String _C__21_REQUESTBYPASSTOSERVER = "[C] 21 RequestBypassToServer";
	//Format: cS
	private static Logger _log = Logger.getLogger(RequestBypassToServer.class.getName());
	private String _command;
	private int command_success;

	@Override
	public void readImpl()
	{
		_command = BypassEngine.decode(getClient().getActiveChar(), readS());
		if(_command != null)
			_command = _command.trim();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;
		command_success = 0;

		try
		{
			if(_command.startsWith("admin_"))
			{
				if(!(activeChar.getPlayerAccess().IsGM || activeChar.getPlayerAccess().CanUseGMCommand))
				{
					activeChar.illegalAction(activeChar.getName() + " user use adm command " + _command, 200);
					return;
				}
				IAdminCommandHandler ach = AdminCommandHandler.getInstance().getAdminCommandHandler(_command);

				if(ach != null)
				{
					try
					{
						command_success = ach.useAdminCommand(_command, activeChar) ? 1 : 0;
					}
					catch(Exception e)
					{
						e.printStackTrace();
					}
					Log.LogCommand(activeChar, Log.CMD_ADMH, _command, command_success);
				}
				else
					_log.warning("No handler registered for bypass '" + _command + "'");
			}
			else if(_command.equals("come_here") && activeChar.getPlayerAccess().IsGM)
				comeHere(getClient());
			else if(_command.startsWith("player_help "))
				playerHelp(activeChar, _command.substring(12));
			else if(_command.startsWith("scripts_"))
			{
				//System.out.println(_command);
				String command = _command.substring(8).trim();
				String[] word = command.split("\\s+");
				String[] args = command.substring(word[0].length()).trim().split("\\s+");
				String[] path = word[0].split(":");
				if(path.length != 2)
				{
					_log.warning("Bad Script bypass!");
					return;
				}

				HashMap<String, Object> variables = new HashMap<String, Object>();

				if(activeChar.getTarget() instanceof L2NpcInstance)
					variables.put("npc", activeChar.getTarget());
				else
					variables.put("npc", null);

				if(word.length == 1)
					activeChar.callScripts(path[0], path[1], new Object[] {}, variables);
				else
					activeChar.callScripts(path[0], path[1], new Object[] { args }, variables);
			}
			else if(_command.startsWith("user_"))
			{
				String command = _command.substring(5).trim();
				String word = (command.split("\\s+"))[0];
				String args = command.substring(word.length()).trim();
				IVoicedCommandHandler vch = VoicedCommandHandler.getInstance().getVoicedCommandHandler(word);

				if(vch != null)
					vch.useVoicedCommand(word, activeChar, args);
				else
					_log.warning("Unknow voiced command '" + word + "'");
			}
			else if(_command.startsWith("npc_"))
			{
				int endOfId = _command.indexOf('_', 5);
				String id;
				if(endOfId > 0)
					id = _command.substring(4, endOfId);
				else
					id = _command.substring(4);
				L2Object object = activeChar.getVisibleObject(Integer.parseInt(id));
				if(object != null && object instanceof L2NpcInstance && endOfId > 0 && activeChar.isInRange(object.getLoc(), L2Character.INTERACTION_DISTANCE))
					((L2NpcInstance) object).onBypassFeedback(activeChar, _command.substring(endOfId + 1));
			}
			// Navigate throught Manor windows
			else if(_command.startsWith("manor_menu_select?"))
			{
				L2Object object = activeChar.getTarget();
				if(object instanceof L2NpcInstance)
					((L2NpcInstance) object).onBypassFeedback(activeChar, _command);
			}
			else if(_command.startsWith("bbs_"))
				CommunityBoard.getInstance().handleCommands(getClient(), _command);
			else if(_command.startsWith("_bbs"))
				CommunityBoard.getInstance().handleCommands(getClient(), _command);
			else if(_command.startsWith("Quest "))
			{
				String p = _command.substring(6).trim();
				int idx = p.indexOf(' ');
				if(idx < 0)
					activeChar.processQuestEvent(p, "");
				else
					activeChar.processQuestEvent(p.substring(0, idx), p.substring(idx).trim());
			}
		}
		catch(Exception e)
		{
			String st = "Bad RequestBypassToServer: " + _command;
			if(activeChar.getTarget() instanceof L2NpcInstance)
				st = st + " via NPC #" + ((L2NpcInstance) activeChar.getTarget()).getNpcId();
			_log.log(Level.WARNING, st, e);
		}
	}

	/**
	 * @param client
	 */
	private void comeHere(L2GameClient client)
	{
		L2Object obj = client.getActiveChar().getTarget();
		if(obj instanceof L2NpcInstance)
		{
			L2NpcInstance temp = (L2NpcInstance) obj;
			L2Player activeChar = client.getActiveChar();
			temp.setTarget(activeChar);
			temp.moveToLocation(activeChar.getLoc(), 0, true);
		}
	}

	private void playerHelp(L2Player activeChar, String path)
	{
		String filename = "data/html/" + path;
		NpcHtmlMessage html = new NpcHtmlMessage(1);
		html.setFile(filename);
		activeChar.sendPacket(html);
	}

	@Override
	public String getType()
	{
		return _C__21_REQUESTBYPASSTOSERVER;
	}
}