package l2r.gameserver.clientpackets;

import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.ItemTable;
import l2r.util.Log;

public class RequestCrystallizeItem extends L2GameClientPacket
{
	private static String _C__2F_REQUESTDCRYSTALLIZEITEM = "[C] 2F RequestCrystallizeItem";
	//Format: cdd
	//private static Logger _log = Logger.getLogger(RequestCrystallizeItem.class.getName());

	//0, d, c, b, a, s ... 0 is for none
	public static short[] _crystalId = { 0, 1458, 1459, 1460, 1461, 1462 };

	private int _objectId;

	@Override
	public void readImpl()
	{
		_objectId = readD();
		readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();

		if(activeChar == null)
			return;

		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.WHILE_OPERATING_A_PRIVATE_STORE_OR_WORKSHOP_YOU_CANNOT_DISCARD_DESTROY_OR_TRADE_AN_ITEM));
			activeChar.sendActionFailed();
			return;
		}

		L2ItemInstance item = activeChar.getInventory().getItemByObjectId(_objectId);

		if(item == null || !item.canBeCrystallized(activeChar) || item.isWear())
		{
			activeChar.sendActionFailed();
			return;
		}

		// unequip if needed
		if(item.isEquipped())
		{
			activeChar.getInventory().unEquipItem(item);
			activeChar.broadcastUserInfo();
		}

		activeChar.getInventory().destroyItem(item, 1, true);

		// add crystals
		int crystalAmount = item.getItem().getCrystalCount();
		short crystalId = _crystalId[item.getItem().getCrystalType()];

		L2ItemInstance createditem = ItemTable.getInstance().createItem(crystalId, activeChar.getObjectId(), item.getItem().getItemId(), "RequestCrystallizeItem[83]");
		createditem.setCount(crystalAmount);
		L2ItemInstance addedItem = activeChar.getInventory().addItem(createditem);
		activeChar.sendPacket(new SystemMessage(SystemMessage.THE_ITEM_HAS_BEEN_SUCCESSFULLY_CRYSTALLIZED));
		activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S2_S1).addItemName(crystalId).addNumber(crystalAmount));

		Log.LogItem(activeChar, Log.CrystalizeItem, item);
		Log.LogItem(activeChar, Log.Sys_GetItem, addedItem);

		activeChar.updateStats();

		L2World.removeObject(item);
	}

	@Override
	public String getType()
	{
		return _C__2F_REQUESTDCRYSTALLIZEITEM;
	}
}
