package l2r.gameserver.clientpackets;

import l2r.gameserver.cache.Msg;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.ItemTable;
import l2r.util.Log;

public class RequestDestroyItem extends L2GameClientPacket
{

	private static String _C__5F_REQUESTDESTROYITEM = "[C] 5F RequestDestroyItem";

	private int _objectId;
	private int _count;

	/**
	 * packet type id 0x1f
	 *
	 * sample
	 *
	 * 59
	 * 0b 00 00 40		// object id
	 * 01 00 00 00		// count
	 *
	 *
	 * format:		cdd
	 * @param decrypt
	 */
	@Override
	public void readImpl()
	{
		_objectId = readD();
		_count = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		int count = _count;

		L2ItemInstance itemToRemove = activeChar.getInventory().getItemByObjectId(_objectId);

		if(itemToRemove == null || itemToRemove.isWear())
			return;

		if(count < 1)
		{
			activeChar.sendPacket(Msg.YOU_CANNOT_DESTROY_IT_BECAUSE_THE_NUMBER_IS_INCORRECT);
			return;
		}

		if(itemToRemove.isHeroItem())
		{
			activeChar.sendPacket(Msg.HERO_WEAPONS_CANNOT_BE_DESTROYED);
			return;
		}

		if(!itemToRemove.canBeDestroyed(activeChar))
		{
			activeChar.sendPacket(Msg.THIS_ITEM_CANNOT_BE_DISCARDED);
			return;
		}

		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			activeChar.sendPacket(Msg.WHILE_OPERATING_A_PRIVATE_STORE_OR_WORKSHOP_YOU_CANNOT_DISCARD_DESTROY_OR_TRADE_AN_ITEM);
			return;
		}

		if(activeChar.isFishing())
		{
			activeChar.sendPacket(Msg.YOU_CANNOT_DO_THAT_WHILE_FISHING);
			return;
		}

		if(activeChar.getPet() != null && activeChar.getPet().getControlItemId() == itemToRemove.getObjectId())
		{
			activeChar.sendPacket(Msg.THE_PET_HAS_BEEN_SUMMONED_AND_CANNOT_BE_DELETED);
			return;
		}

		if(activeChar.isActionsDisabled())
		{
			activeChar.sendActionFailed();
			return;
		}

		if(_count > itemToRemove.getIntegerLimitedCount())
			count = itemToRemove.getIntegerLimitedCount();

		if(itemToRemove.isEquipped())
		{
			activeChar.getInventory().unEquipItemInSlot(itemToRemove.getEquipSlot());
			activeChar.broadcastUserInfo();
		}

		if(itemToRemove.getItem().isCrystallizable())
		{
			int level = activeChar.getSkillLevel(L2Skill.SKILL_CRYSTALLIZE);
			if(!(level < 1 || itemToRemove.getItem().getCrystalType() > level))
			{
				activeChar.getInventory().destroyItem(itemToRemove, 1, true);

				// add crystals
				int crystalAmount = itemToRemove.getItem().getCrystalCount();
				short crystalId = RequestCrystallizeItem._crystalId[itemToRemove.getItem().getCrystalType()];

				L2ItemInstance createditem = ItemTable.getInstance().createItem(crystalId, activeChar.getObjectId(), itemToRemove.getItem().getItemId(), "RequestDestroyItem[130]");
				createditem.setCount(crystalAmount);
				L2ItemInstance addedItem = activeChar.getInventory().addItem(createditem);
				activeChar.sendPacket(Msg.THE_ITEM_HAS_BEEN_SUCCESSFULLY_CRYSTALLIZED);
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S2_S1).addItemName(crystalId).addNumber(crystalAmount));

				Log.LogItem(activeChar, Log.CrystalizeItem, itemToRemove);
				Log.LogItem(activeChar, Log.Sys_GetItem, addedItem);

				activeChar.updateStats();

				L2World.removeObject(itemToRemove);
				return;
			}
		}

		L2ItemInstance removedItem = activeChar.getInventory().destroyItem(_objectId, count, true);

		Log.LogItem(activeChar, Log.DeleteItem, removedItem);

		activeChar.sendChanges();

		if(!removedItem.isStackable() || count == 1)
			activeChar.sendPacket(new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(removedItem.getItemId()));
		else
			activeChar.sendPacket(new SystemMessage(SystemMessage.S2_S1_HAS_DISAPPEARED).addItemName(removedItem.getItemId()).addNumber(count));

		L2World.removeObject(removedItem);

		activeChar.updateStats();
	}

	@Override
	public String getType()
	{
		return _C__5F_REQUESTDESTROYITEM;
	}
}
