package l2r.gameserver.clientpackets;

import java.util.logging.Logger;

import l2r.Config;
import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.GmListTable;
import l2r.gameserver.templates.L2EtcItem.EtcItemType;
import l2r.util.Location;
import l2r.util.Log;

public class RequestDropItem extends L2GameClientPacket
{
	private static String _C__17_REQUESTDROPITEM = "[C] 17 RequestDropItem";
	private static Logger _log = Logger.getLogger(RequestDropItem.class.getName());

	private int _objectId;
	private int _count;
	private Location _loc = new Location(0, 0, 0);

	/**
	 * packet type id 0x17
	 * format:		cdd ddd
	 */
	@Override
	public void readImpl()
	{
		_objectId = readD();
		_count = readD();
		_loc.setX(readD());
		_loc.setY(readD());
		_loc.setZ(readD());
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null || activeChar.isDead())
			return;

		if(_count < 1 || _loc.getX() == 0 || _loc.getY() == 0 || _loc.getZ() == 0)
		{
			activeChar.sendActionFailed();
			return;
		}

		if(!Config.ALLOW_DISCARDITEM)
		{
			activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.RequestDropItem.Disallowed", activeChar));
			return;
		}

		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			activeChar.sendPacket(Msg.WHILE_OPERATING_A_PRIVATE_STORE_OR_WORKSHOP_YOU_CANNOT_DISCARD_DESTROY_OR_TRADE_AN_ITEM);
			return;
		}

		if(activeChar.isTransactionInProgress())
		{
			sendPacket(Msg.NOTHING_HAPPENED);
			return;
		}

		L2ItemInstance oldItem = activeChar.getInventory().getItemByObjectId(_objectId);
		if(oldItem == null)
		{
			_log.warning(activeChar.getName() + ":tried to drop an item that is not in the inventory ?!?:" + _objectId);
			return;
		}

		if(activeChar.isFishing())
		{
			activeChar.sendPacket(Msg.YOU_CANNOT_DO_THAT_WHILE_FISHING);
			return;
		}

		if(activeChar.isActionsDisabled() || activeChar.isSitting() || activeChar.isDropDisabled())
		{
			activeChar.sendActionFailed();
			return;
		}

		if(oldItem.getItemType() == EtcItemType.QUEST || !oldItem.canBeDropped(activeChar))
		{
			activeChar.sendPacket(Msg.THAT_ITEM_CANNOT_BE_DISCARDED);
			return;
		}

		if(oldItem.isWear())
		{
			activeChar.sendActionFailed();
			_log.warning(activeChar.getName() + ":tried to drop an item that is wear:" + _objectId);
			return;
		}

		int oldCount = oldItem.getIntegerLimitedCount();
		if(Config.DEBUG)
			_log.fine("requested drop item " + _objectId + "(" + oldCount + ") at " + _loc.toString());

		if(oldCount < _count)
		{
			activeChar.sendActionFailed();
			if(Config.DEBUG)
				_log.finest(activeChar.getObjectId() + ":player tried to drop more items than he has");
			return;
		}

		if(!activeChar.isInRangeSq(_loc, 22500) || Math.abs(_loc.getZ() - activeChar.getZ()) > 50)
		{
			if(Config.DEBUG)
				_log.finest(activeChar.getObjectId() + ": trying to drop too far away");
			activeChar.sendPacket(Msg.TOO_FAR_TO_DISCARD);
			return;
		}

		if(activeChar.getEnchantScroll() != null && oldItem.getObjectId() == activeChar.getEnchantScroll().getObjectId())
		{
			activeChar.sendActionFailed();
			if(Config.DEBUG)
				_log.finest(activeChar.getObjectId() + ":player tried to drop an enchant scroll he was using");
			return;
		}

		if(oldItem.isEquipped())
		{
			L2ItemInstance weapon = activeChar.getActiveWeaponInstance();
			L2ItemInstance[] unequipped = activeChar.getInventory().unEquipItemInBodySlotAndRecord(oldItem.getBodyPart(), oldItem);

			for(L2ItemInstance uneq : unequipped)
			{
				if(uneq == null)
					continue;

				SystemMessage sm;

				if(oldItem.getEnchantLevel() > 0)
				{
					sm = new SystemMessage(SystemMessage.EQUIPMENT_OF__S1_S2_HAS_BEEN_REMOVED);
					sm.addNumber(oldItem.getEnchantLevel());
					sm.addItemName(oldItem.getItemId());
				}
				else
					sm = new SystemMessage(SystemMessage.S1__HAS_BEEN_DISARMED).addItemName(oldItem.getItemId());

				activeChar.sendPacket(sm);

				if(weapon != null && uneq == weapon)
				{
					uneq.setChargedSpiritshot(L2ItemInstance.CHARGED_NONE);
					uneq.setChargedSoulshot(L2ItemInstance.CHARGED_NONE);
				}
			}

			activeChar.refreshExpertisePenalty();
			activeChar.broadcastUserInfo();
		}

		oldItem.setWhFlag(true);
		L2ItemInstance dropedItem = activeChar.getInventory().dropItem(_objectId, _count);
		oldItem.setWhFlag(false);

		if(dropedItem == null)
		{
			activeChar.sendActionFailed();
			return;
		}

		dropedItem.dropToTheGround(activeChar, _loc);

		activeChar.disableDrop(1000);

		Log.LogItem(activeChar, Log.Drop, dropedItem);

		if(dropedItem.getItemId() == 57 && dropedItem.getIntegerLimitedCount() >= 1000000)
		{
			String msg = "Character (" + activeChar.getName() + ") has dropped (" + dropedItem.getIntegerLimitedCount() + ") adena at " + _loc.toString();
			_log.warning(msg);
			GmListTable.broadcastMessageToGMs(msg);
		}

		activeChar.updateStats();
	}

	@Override
	public String getType()
	{
		return _C__17_REQUESTDROPITEM;
	}
}