package l2r.gameserver.clientpackets;

import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.base.L2EnchantSkillLearn;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.serverpackets.ExEnchantSkillInfo;
import l2r.gameserver.serverpackets.ExEnchantSkillList.EnchantSkillType;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.tables.SkillTreeTable;

public class RequestExEnchantSkillInfo extends L2GameClientPacket
{
	private static String _C__D0_0E_REQUESTEXENCHANTSKILLINFO = "[C] D0:0E RequestExEnchantSkillInfo";

	private static final int TYPE_NORMAL_ENCHANT = 0;
	private static final int TYPE_SAFE_ENCHANT = 1;
	private static final int TYPE_UNTRAIN_ENCHANT = 2;
	private static final int TYPE_CHANGE_ENCHANT = 3;

	private int _type;
	private int _skillId;
	private int _skillLvl;

	@Override
	public void readImpl()
	{
		_type = readH();
		_skillId = readD();
		_skillLvl = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(activeChar.getLevel() < 76 || activeChar.getClassId().getLevel() < 4)
		{
			activeChar.sendMessage("You must have 3rd class change quest completed.");
			return;
		}

		L2NpcInstance trainer = activeChar.getLastNpc();
		if(trainer == null || activeChar.getDistance(trainer) > L2Character.INTERACTION_DISTANCE && !activeChar.isGM())
		{
			activeChar.sendActionFailed();
			return;
		}

		if(_skillLvl > 100)
		{
			L2EnchantSkillLearn sl = SkillTreeTable.getSkillEnchant(_skillId, (short) _skillLvl);
			if(sl == null)
			{
				activeChar.sendMessage("Not found enchant info for this skill");
				return;
			}

			L2Skill skill = SkillTable.getInstance().getInfo(_skillId, SkillTreeTable.convertEnchantLevel(sl.getBaseLevel(), _skillLvl));

			if(skill == null || skill.getId() != _skillId)
			{
				activeChar.sendMessage("This skill doesn't yet have enchant info in Datapack");
				return;
			}

			if(activeChar.getSkillLevel(_skillId) != skill.getLevel())
			{
				activeChar.sendMessage("Skill not found");
				return;
			}
		}
		else if(activeChar.getSkillLevel(_skillId) != _skillLvl)
		{
			activeChar.sendMessage("Skill not found");
			return;
		}

		if(!(trainer.getTemplate().canTeach(activeChar.getClassId()) || trainer.getTemplate().canTeach(activeChar.getClassId().getParent(activeChar.getSex()))))
		{
			activeChar.sendMessage("Wrong teacher");
			return;
		}

		switch(_type)
		{
			case TYPE_NORMAL_ENCHANT:
				showEnchantInfo(activeChar, false);
				break;
			case TYPE_SAFE_ENCHANT:
				showEnchantInfo(activeChar, true);
				break;
			case TYPE_UNTRAIN_ENCHANT:
				showUntrainEnchantInfo();
				break;
			case TYPE_CHANGE_ENCHANT:
				showChangeEnchantInfo(activeChar);
				break;
			default:
				_log.severe("Unknown skill enchant type: " + _type);
				break;
		}
	}

	public void showEnchantInfo(L2Player activeChar, boolean isSafeEnchant)
	{
		ExEnchantSkillInfo asi = new ExEnchantSkillInfo(isSafeEnchant ? EnchantSkillType.SAFE : EnchantSkillType.NORMAL, _skillId);

		// skill already enchanted?
		if(_skillLvl > 100)
		{
			// get detail for next level
			L2EnchantSkillLearn esd = SkillTreeTable.getSkillEnchant(_skillId, (short) (_skillLvl + 1));

			// if it exists add it
			if(esd != null)
				asi.addEnchantSkillDetail(activeChar, esd);
		}
		else
			// not already enchanted
			for(L2EnchantSkillLearn esd : SkillTreeTable.getFirstEnchantsForSkill(_skillId))
				asi.addEnchantSkillDetail(activeChar, esd);

		sendPacket(asi);
	}

	public void showChangeEnchantInfo(L2Player activeChar)
	{
		ExEnchantSkillInfo asi = new ExEnchantSkillInfo(EnchantSkillType.CHANGE_ROUTE, _skillId);

		// skill already enchanted?
		if(_skillLvl > 100)
		{
			// get current enchant type
			L2EnchantSkillLearn esd = SkillTreeTable.getSkillEnchant(_skillId, (short) _skillLvl);

			if(esd != null)
				for(L2EnchantSkillLearn e : SkillTreeTable.getEnchantsForChange(_skillId, _skillLvl))
					// skip current route
					if(esd.getMinSkillLevel() != e.getMinSkillLevel())
						asi.addEnchantSkillDetail(activeChar, e);

			sendPacket(asi);
		}
		else
			_log.warning("Client: " + getClient() + " requested change route information for unenchanted skill");
	}

	public void showUntrainEnchantInfo()
	{
		ExEnchantSkillInfo asi = new ExEnchantSkillInfo(EnchantSkillType.UNTRAIN, _skillId);

		// skill already enchanted?
		if(_skillLvl > 100)
		{
			L2EnchantSkillLearn currentLevelDetail = SkillTreeTable.getSkillEnchant(_skillId, (short) _skillLvl);
			if(currentLevelDetail != null)
			{
				// no previous enchant level, return to original
				if(_skillLvl % 100 == 1)
					asi.addEnchantSkillDetail(currentLevelDetail.getBaseLevel(), 100, currentLevelDetail.getSpCost(), (int) currentLevelDetail.getExpCost());
				else
				{
					// get detail for previous level
					L2EnchantSkillLearn esd = SkillTreeTable.getSkillEnchant(_skillId, (short) (_skillLvl - 1));

					// if it exists add it
					if(esd != null)
						asi.addEnchantSkillDetail(esd.getLevel(), 100, currentLevelDetail.getSpCost(), (int) currentLevelDetail.getExpCost());
				}
			}
			else
				_log.warning("Client: " + getClient() + " tried to untrain enchanted skill, but server doesnt has data for his current skill enchantment level");

			sendPacket(asi);
		}
		else
			_log.warning("Client: " + getClient() + " requested untrain information for unenchanted skill");
	}

	@Override
	public String getType()
	{
		return _C__D0_0E_REQUESTEXENCHANTSKILLINFO;
	}
}