package l2r.gameserver.clientpackets;

import java.sql.ResultSet;
import java.util.logging.Level;
import java.util.logging.Logger;

import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.L2Player.TransactionType;
import l2r.gameserver.serverpackets.FriendAddRequest;
import l2r.gameserver.serverpackets.SystemMessage;

/**
 * This class ...
 *
 * @version $Revision: 1.3.4.2 $ $Date: 2005/03/27 15:29:30 $
 */
public class RequestFriendInvite extends L2GameClientPacket
{

	private static String _C__77_REQUESTFRIENDINVITE = "[C] 77 RequestFriendInvite";
	// format: cS
	private static Logger _log = Logger.getLogger(RequestFriendInvite.class.getName());

	private String _name;

	@Override
	public void readImpl()
	{
		_name = readS();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(activeChar.isTransactionInProgress())
		{
			activeChar.sendPacket(Msg.WAITING_FOR_ANOTHER_REPLY);
			return;
		}
		L2Player friend = L2World.getPlayer(_name);
		if(friend == null)
		{
			// Target is not found in the game.
			activeChar.sendPacket(Msg.THE_USER_WHO_REQUESTED_TO_BECOME_FRIENDS_IS_NOT_FOUND_IN_THE_GAME);
			return;
		}
		if(friend == activeChar)
		{
			// You cannot add yourself to your own friend list.
			activeChar.sendPacket(Msg.YOU_CANNOT_ADD_YOURSELF_TO_YOUR_OWN_FRIEND_LIST);
			return;
		}
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("select friend_id from character_friends where char_id=? and friend_id=?");
			statement.setInt(1, activeChar.getObjectId());
			statement.setInt(2, friend.getObjectId());
			rset = statement.executeQuery();
			if(rset.next())
				// Player already is in your friendlist
				sendPacket(new SystemMessage(SystemMessage.S1_IS_ALREADY_ON_YOUR_FRIEND_LIST).addString(_name));
			else if(!friend.isTransactionInProgress())
			{
				// requets to become friend
				friend.setTransactionRequester(activeChar, System.currentTimeMillis() + 10000);
				friend.setTransactionType(TransactionType.FRIEND);
				activeChar.setTransactionRequester(friend, System.currentTimeMillis() + 10000);
				activeChar.setTransactionType(TransactionType.FRIEND);
				sendPacket(new SystemMessage(SystemMessage.S1_HAS_REQUESTED_TO_BECOME_FRIENDS).addString(_name));
				friend.sendPacket(new FriendAddRequest(activeChar.getName()));
			}
			else
				sendPacket(new SystemMessage(SystemMessage.S1_IS_BUSY_PLEASE_TRY_AGAIN_LATER));
		}
		catch(Exception e)
		{
			_log.log(Level.WARNING, "could not add friend objectid: ", e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		// if(activeChar.isInParty() &&
		// activeChar.getParty().isLeader(activeChar))
		// activeChar.getParty().changePartyLeader(_name);
	}

	@Override
	public String getType()
	{
		return _C__77_REQUESTFRIENDINVITE;
	}
}
