package l2r.gameserver.clientpackets;

import java.util.logging.Logger;

import l2r.gameserver.cache.Msg;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.PcInventory;
import l2r.gameserver.model.PetInventory;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2PetInstance;
import l2r.gameserver.serverpackets.PetItemList;
import l2r.util.Log;

public class RequestGetItemFromPet extends L2GameClientPacket
{
	private static String REQUESTGETITEMFROMPET__C__8C = "[C] 8C RequestGetItemFromPet";
	private static Logger _log = Logger.getLogger(RequestGetItemFromPet.class.getName());

	private int _objectId;
	private int _amount;
	@SuppressWarnings("unused")
	private int _unknown;

	@Override
	public void readImpl()
	{
		_objectId = readD();
		_amount = readD();
		_unknown = readD();// = 0 for most trades
	}

	@Override
	public void runImpl()
	{
		if(_amount < 1)
			return;
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		L2PetInstance pet = (L2PetInstance) activeChar.getPet();
		if(pet == null)
		{
			activeChar.sendActionFailed();
			return;
		}

		PetInventory petInventory = pet.getInventory();
		PcInventory playerInventory = activeChar.getInventory();

		L2ItemInstance petItem = petInventory.getItemByObjectId(_objectId);

		if(petItem == null)
		{
			_log.warning("item requested from pet, but its not there.");
			return;
		}

		if(petItem.isEquipped())
		{
			activeChar.sendActionFailed();
			return;
		}

		long finalLoad = petItem.getItem().getWeight() * _amount;

		if(!activeChar.getInventory().validateWeight(finalLoad))
		{
			sendPacket(Msg.YOU_HAVE_EXCEEDED_THE_WEIGHT_LIMIT);
			return;
		}

		playerInventory.addItem(petInventory.dropItem(_objectId, _amount));

		activeChar.sendChanges();
		sendPacket(new PetItemList(pet));
		pet.broadcastPetInfo();

		Log.LogItem(activeChar, activeChar.getPet(), Log.GetItemFromPet, petItem);
	}

	@Override
	public String getType()
	{
		return REQUESTGETITEMFROMPET__C__8C;
	}
}