package l2r.gameserver.clientpackets;

import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.PcInventory;
import l2r.gameserver.model.PetInventory;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2PetInstance;
import l2r.gameserver.serverpackets.PetItemList;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.util.Log;

public class RequestGiveItemToPet extends L2GameClientPacket
{
	private static String REQUESTCIVEITEMTOPET__C__8B = "[C] 8B RequestGiveItemToPet";
	//private static Logger _log = Logger.getLogger(RequestGiveItemToPet.class.getName());

	private int _objectId;
	private int _amount;

	@Override
	public void readImpl()
	{
		_objectId = readD();
		_amount = readD();
	}

	@Override
	public void runImpl()
	{
		if(_amount < 1)
			return;
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		L2PetInstance pet = (L2PetInstance) activeChar.getPet();
		if(pet == null || pet.isDead())
		{
			sendPacket(new SystemMessage(SystemMessage.CANNOT_GIVE_ITEMS_TO_A_DEAD_PET));
			return;
		}

		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			sendPacket(new SystemMessage(SystemMessage.WHILE_OPERATING_A_PRIVATE_STORE_OR_WORKSHOP_YOU_CANNOT_DISCARD_DESTROY_OR_TRADE_AN_ITEM));
			return;
		}

		PetInventory petInventory = pet.getInventory();
		PcInventory playerInventory = activeChar.getInventory();

		L2ItemInstance playerItem = playerInventory.getItemByObjectId(_objectId);

		if(playerItem == null || playerItem.getObjectId() == pet.getControlItemId())
			return;

		if(pet.getInventory().getTotalWeight() + playerItem.getItem().getWeight() * _amount >= pet.getMaxLoad())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.EXCEEDED_PET_INVENTORYS_WEIGHT_LIMIT));
			return;
		}

		if(!playerItem.canBeDropped(activeChar))
		{
			activeChar.sendActionFailed();
			return;
		}

		if(_amount >= playerItem.getIntegerLimitedCount())
		{
			playerInventory.dropItem(_objectId, playerItem.getIntegerLimitedCount());
			petInventory.addItem(playerItem);
		}
		else
		{
			L2ItemInstance newPetItem = playerInventory.dropItem(_objectId, _amount);
			petInventory.addItem(newPetItem);
		}

		activeChar.sendChanges();
		sendPacket(new PetItemList(pet));
		pet.broadcastPetInfo();

		Log.LogItem(activeChar, pet, Log.GiveItemToPet, playerItem);
	}

	@Override
	public String getType()
	{
		return REQUESTCIVEITEMTOPET__C__8B;
	}
}