package l2r.gameserver.clientpackets;

import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Skill.NextAction;
import l2r.gameserver.tables.SkillTable;

public class RequestMagicSkillUse extends L2GameClientPacket
{
	private static String _C__39_REQUESTMAGICSKILLUSE = "[C] 39 RequestMagicSkillUse";

	private Integer _magicId;
	private boolean _ctrlPressed;
	private boolean _shiftPressed;

	/**
	 * packet type id 0x39
	 * format:		cddc
	 * @param rawPacket
	 */
	@Override
	public void readImpl()
	{
		_magicId = readD();
		_ctrlPressed = readD() != 0;
		_shiftPressed = readC() != 0;
	}

	@Override
	public void runImpl()
	{
		L2Player player = getClient().getActiveChar();
		if(player == null)
			return;

        if(player.isOutOfControl())
		{
			player.sendActionFailed();
			return;
		}

		L2Skill skill = SkillTable.getInstance().getInfo(_magicId, player.getSkillLevel(_magicId));
		if(skill != null)
		{
			if(skill.isPassive() || skill.isOnAttack() || skill.isOnMagicAttack() || skill.isOnUnderAttack())
				return;
				
			if(skill.isToggle())
				if(player.getEffectsBySkill(skill) != null)
				{
					player.stopEffect(skill.getId());
					player.sendActionFailed();
					return;
				}

			L2Character target = skill.getAimingTarget(player);

			if(skill.checkCondition(player, target, _ctrlPressed, _shiftPressed, true))
			{
				player.getAI().Cast(skill, target, _ctrlPressed, _shiftPressed);
			}
			else if(target != null && skill.getNextAction() == NextAction.ATTACK && !player.equals(target) && (_ctrlPressed ? target.isAttackable() : target.isAutoAttackable(player)))
				player.getAI().Attack(target, _ctrlPressed, _shiftPressed);
			else
				player.sendActionFailed();
		}
		else
			player.sendActionFailed();
	}

	@Override
	public String getType()
	{
		return _C__39_REQUESTMAGICSKILLUSE;
	}
}
