package l2r.gameserver.clientpackets;

import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2PetInstance;
import l2r.gameserver.serverpackets.PetItemList;
import l2r.gameserver.serverpackets.SystemMessage;

public class RequestPetUseItem extends L2GameClientPacket
{
	private static String _C__94_REQUESTPETUSEITEM = "[C] 94 RequestPetUseItem";

	private int _objectId;

	/**
	 * packet type id 0x94
	 * format:		cd
	 * @param decrypt
	 */
	@Override
	public void readImpl()
	{
		_objectId = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		L2PetInstance pet = (L2PetInstance) activeChar.getPet();
		if(pet == null)
			return;

		L2ItemInstance item = pet.getInventory().getItemByObjectId(_objectId);

		if(item == null || item.getIntegerLimitedCount() <= 0)
			return;

		int itemId = item.getItemId();

		if(activeChar.isAlikeDead() || pet.isDead())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.S1_CANNOT_BE_USED_DUE_TO_UNSUITABLE_TERMS).addItemName(item.getItemId()));
			return;
		}

		int petId = pet.getNpcId();

		if(item.isEquipable())
			if(petId == 12077 && item.getItem().isForWolf() || petId >= 12311 && petId <= 12313 && item.getItem().isForHatchling() || petId >= 12526 && petId <= 12528 && item.getItem().isForStrider() || petId == 16025 && item.getItem().isForGWolf())
			{
				if(item.isEquipped())
					pet.getInventory().unEquipItemInSlot(item.getEquipSlot());
				else
					pet.getInventory().equipItem(item);

				activeChar.sendPacket(new PetItemList(pet));
				pet.broadcastPetInfo();
				return;
			}

		// manual pet feeding
		if(petId == 12077 && itemId == 2515 || // Wolf
		petId == 12564 && itemId == 2515 || // Sin Eater
		petId >= 12311 && petId <= 12313 && itemId == 4038 || // Hatchling of Wind, Star, Twilight
		petId >= 12526 && petId <= 12528 && itemId == 5168 || // Wind, Star, Twilight Strider
		petId >= 12780 && petId <= 12782 && itemId == 7582 || // Baby Buffalo, Kookaburra, Cougar
		petId == 16025 && itemId == 9668 || // Great Wolf b
		petId == 16030 && itemId == 9668) // Great Wolf ab
		{
			feed(activeChar, pet, item);
			return;
		}

		activeChar.sendPacket(new SystemMessage(SystemMessage.ITEM_NOT_AVAILABLE_FOR_PETS));
		return;
	}

	private void feed(L2Player activeChar, L2PetInstance pet, L2ItemInstance item)
	{
		// единица еды добавляет 12% сытности.
		// Возможно у каждого пета свой процент, потом проставить. Saboteur
		if(pet.getCurrentFed() >= pet.getMaxFed())
		{
			//Not exists message to send in "systemmsg-e.txt"
			activeChar.sendActionFailed();
			return;
		}
		pet.removeItemFromInventory(item, 1, true);
		pet.setCurrentFed(pet.getCurrentFed() + (int) (pet.getMaxFed() * 0.12));
		if(pet.getCurrentFed() > pet.getMaxFed())
			pet.setCurrentFed(pet.getMaxFed());
		pet.broadcastStatusUpdate();
	}

	@Override
	public String getType()
	{
		return _C__94_REQUESTPETUSEITEM;
	}
}
