package l2r.gameserver.clientpackets;

import java.util.List;

import javolution.util.FastList;
import l2r.Config;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.instancemanager.CastleManorManager;
import l2r.gameserver.instancemanager.CastleManorManager.CropProcure;
import l2r.gameserver.model.L2Manor;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2ManorManagerInstance;
import l2r.gameserver.serverpackets.StatusUpdate;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.templates.L2Item;

@SuppressWarnings("unused")
public class RequestProcureCrop extends L2GameClientPacket
{
	private static final String _C__CF_REQUESTPROCURECROP = "[C] CF RequestProcureCrop";
	// format: cddb
	private int _listId;
	private int _count;
	private int[] _items;
	private List<CropProcure> _procureList = new FastList<CropProcure>();

	@Override
	protected void readImpl()
	{
		_listId = readD();
		_count = readD();
		if(_count * 12 > _buf.remaining() || _count > Short.MAX_VALUE || _count <= 0)
		{
			_count = 0;
			return;
		}
		_items = new int[_count * 2];
		for(int i = 0; i < _count; i++)
		{
			long servise = readD();
			int itemId = readD();
			_items[i * 2 + 0] = itemId;
			long cnt = readD();
			if(cnt > Integer.MAX_VALUE || cnt < 1)
			{
				_count = 0;
				_items = null;
				return;
			}
			_items[i * 2 + 1] = (int) cnt;
		}
	}

	@Override
	protected void runImpl()
	{
		L2Player player = getClient().getActiveChar();
		if(player == null)
			return;

		// Alt game - Karma punishment
		if(!Config.ALT_GAME_KARMA_PLAYER_CAN_SHOP && player.getKarma() > 0)
			return;

		L2Object target = player.getTarget();

		if(_count < 1)
		{
			player.sendActionFailed();
			return;
		}

		long subTotal = 0;
		int tax = 0;

		// Check for buylist validity and calculates summary values
		int slots = 0;
		int weight = 0;
		L2ManorManagerInstance manor = target != null && target instanceof L2ManorManagerInstance ? (L2ManorManagerInstance) target : null;

		for(int i = 0; i < _count; i++)
		{
			int itemId = _items[i * 2 + 0];
			int count = _items[i * 2 + 1];
			int price = 0;
			if(count < 0 || count > Integer.MAX_VALUE)
			{
				sendPacket(Msg.INCORRECT_ITEM_COUNT);
				return;
			}

			L2Item template = ItemTable.getInstance().getTemplate(L2Manor.getInstance().getRewardItem(itemId, manor.getCastle().getCrop(itemId, CastleManorManager.PERIOD_CURRENT).getReward()));
			weight += count * template.getWeight();

			if(!template.isStackable())
				slots += count;
			else if(player.getInventory().getItemByItemId(itemId) == null)
				slots++;
		}

		if(!player.getInventory().validateWeight(weight))
		{
			sendPacket(Msg.YOU_HAVE_EXCEEDED_THE_WEIGHT_LIMIT);
			return;
		}

		if(!player.getInventory().validateCapacity(slots))
		{
			sendPacket(Msg.YOUR_INVENTORY_IS_FULL);
			return;
		}

		// Proceed the purchase
		_procureList = manor.getCastle().getCropProcure(CastleManorManager.PERIOD_CURRENT);

		for(int i = 0; i < _count; i++)
		{
			int itemId = _items[i * 2 + 0];
			int count = _items[i * 2 + 1];
			if(count < 0)
				count = 0;

			int rewradItemId = L2Manor.getInstance().getRewardItem(itemId, manor.getCastle().getCrop(itemId, CastleManorManager.PERIOD_CURRENT).getReward());

			int rewradItemCount = L2Manor.getInstance().getRewardAmountPerCrop(manor.getCastle().getId(), itemId, manor.getCastle().getCropRewardType(itemId));

			rewradItemCount = count * rewradItemCount;

			// Add item to Inventory and adjust update packet
			L2ItemInstance item = player.getInventory().addItem(rewradItemId, rewradItemCount, 0, "Manor");
			L2ItemInstance iteme = player.getInventory().destroyItemByItemId(itemId, count, true);

			if(item == null || iteme == null)
				continue;

			// Send Char Buy Messages
			SystemMessage sm = new SystemMessage(SystemMessage.EARNED_S2_S1_s);
			sm.addItemName(rewradItemId);
			sm.addNumber(rewradItemCount);
			player.sendPacket(sm);
			sm = null;

			//manor.getCastle().setCropAmount(itemId, manor.getCastle().getCrop(itemId, CastleManorManager.PERIOD_CURRENT).getAmount() - count);
		}

		StatusUpdate su = new StatusUpdate(player.getObjectId());
		su.addAttribute(StatusUpdate.CUR_LOAD, player.getCurrentLoad());
		player.sendPacket(su);
	}

	@Override
	public String getType()
	{
		return _C__CF_REQUESTPROCURECROP;
	}
}
