package l2r.gameserver.clientpackets;

import l2r.Config;
import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.model.L2ManufactureItem;
import l2r.gameserver.model.L2ManufactureList;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.L2Zone;
import l2r.gameserver.serverpackets.RecipeShopMsg;
import l2r.gameserver.serverpackets.SystemMessage;

public class RequestRecipeShopListSet extends L2GameClientPacket
{
	private static String _C__BB_RequestRecipeShopListSet = "[C] BB RequestRecipeShopListSet";
	// format: cdb, b - array of (dd)
	private int _count;
	L2ManufactureList createList = new L2ManufactureList();

	@Override
	public void readImpl()
	{
		_count = readD();
		if(_count * 8 > _buf.remaining() || _count > Short.MAX_VALUE || _count < 0)
		{
			_count = 0;
			return;
		}
		for(int x = 0; x < _count; x++)
			createList.add(new L2ManufactureItem(readD(), readD()));
		_count = createList.size();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(activeChar.isMounted())
		{
			cancelStore(activeChar);
			return;
		}

		if(activeChar.getDuel() != null)
		{
			cancelStore(activeChar);
			return;
		}

		if(activeChar.isActionBlocked(L2Zone.BLOCKED_ACTION_PRIVATE_WORKSHOP))
		{
			sendPacket(new SystemMessage(SystemMessage.A_PRIVATE_WORKSHOP_MAY_NOT_BE_OPENED_IN_THIS_AREA));
			cancelStore(activeChar);
			return;
		}

		if(activeChar.getNoChannel() != 0)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_ARE_CURRENTLY_BANNED_FROM_ACTIVITIES_RELATED_TO_THE_PRIVATE_STORE_AND_PRIVATE_WORKSHOP));
			cancelStore(activeChar);
			return;
		}

		if(_count == 0 || activeChar.getCreateList() == null)
		{
			cancelStore(activeChar);
			return;
		}

		if(_count > Config.MAX_PVTCRAFT_SLOTS)
		{
			sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_EXCEEDED_THE_QUANTITY_THAT_CAN_BE_INPUTTED));
			cancelStore(activeChar);
			return;
		}

		if(Config.SERVICES_TRADE_ONLY_FAR && !activeChar.isInStoreMode())
		{
			boolean tradenear = false;
			for(L2Player player : L2World.getAroundPlayers(activeChar, Config.SERVICES_TRADE_RADIUS, Config.SERVICES_TRADE_RADIUS))
				if(player.isInStoreMode())
				{
					tradenear = true;
					break;
				}

			if(L2World.getAroundNpc(activeChar, Config.SERVICES_TRADE_RADIUS, Config.SERVICES_TRADE_RADIUS).size() > 0)
				tradenear = true;

			if(tradenear)
			{
				activeChar.sendMessage(new CustomMessage("trade.OtherTradersNear", activeChar));
				cancelStore(activeChar);
			}
		}

		createList.setStoreName(activeChar.getCreateList().getStoreName());
		activeChar.setCreateList(createList);

		activeChar.setPrivateStoreType(L2Player.STORE_PRIVATE_MANUFACTURE);
		activeChar.broadcastUserInfo();
		activeChar.broadcastPacket(new RecipeShopMsg(activeChar));
		activeChar.sitDown();
	}

	private void cancelStore(L2Player activeChar)
	{
		activeChar.setPrivateStoreType(L2Player.STORE_PRIVATE_NONE);
		activeChar.broadcastUserInfo();
		activeChar.getBuyList().clear();
		if(activeChar.isInOfflineMode() && Config.SERVICES_OFFLINE_TRADE_KICK_NOT_TRADING)
		{
			activeChar.setOfflineMode(false);
			activeChar.logout(false, false, true);
			activeChar.getNetConnection().disconnectOffline();
		}
	}

	@Override
	public String getType()
	{
		return _C__BB_RequestRecipeShopListSet;
	}
}