package l2r.gameserver.clientpackets;

import javolution.util.FastList;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Summon;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.instances.L2BoatInstance;
import l2r.gameserver.model.instances.L2DoorInstance;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.instances.L2StaticObjectInstance;
import l2r.gameserver.serverpackets.CharInfo;
import l2r.gameserver.serverpackets.NpcInfo;
import l2r.gameserver.serverpackets.Ride;
import l2r.gameserver.serverpackets.SpawnItem;
import l2r.gameserver.serverpackets.StaticObject;
import l2r.gameserver.serverpackets.UserInfo;
import l2r.gameserver.serverpackets.VehicleInfo;

public class RequestRecordInfo extends L2GameClientPacket
{
	@Override
	public void readImpl()
	{}

	@Override
	public void runImpl()
	{
		L2Player player = getClient().getActiveChar();

		if(player == null)
			return;

		player.sendPacket(new UserInfo(player));

		FastList<L2Object> objs = L2World.getAroundObjects(player);
		for(FastList.Node<L2Object> n = objs.head(), end = objs.tail(); (n = n.getNext()) != end;)
		{
			if(n == null)
				break;

			L2Object obj = n.getValue();

			if(obj instanceof L2NpcInstance)
				player.sendPacket(new NpcInfo((L2NpcInstance) obj, player));
			else if(obj instanceof L2Summon)
				((L2Summon) obj).broadcastPetInfo();
			else if(obj.isPlayer)
			{
				L2Player targetPlayer = (L2Player) obj;
				if(player.getObjectId() != targetPlayer.getObjectId() && !targetPlayer.isInvisible())
				{
					player.sendPacket(new CharInfo(targetPlayer));
					if(targetPlayer.isMounted())
						player.sendPacket(new Ride(targetPlayer));
				}
			}
			else if(obj instanceof L2DoorInstance)
				player.sendPacket(new StaticObject((L2DoorInstance) obj));
			else if(obj instanceof L2BoatInstance)
				player.sendPacket(new VehicleInfo((L2BoatInstance) obj));
			else if(obj instanceof L2ItemInstance)
				player.sendPacket(new SpawnItem((L2ItemInstance) obj));
			else if(obj instanceof L2StaticObjectInstance)
				player.sendPacket(new StaticObject((L2StaticObjectInstance) obj));
		}
	}

	/**
	 * just for information and debug purposes
	 * @return text for trace message
	 */
	@Override
	public String getType()
	{
		return "[C] CF RequestRecordInfo";
	}
}
