package l2r.gameserver.clientpackets;

import l2r.Config;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.ExVariationResult;
import l2r.gameserver.serverpackets.InventoryUpdate;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.serverpackets.UserInfo;
import l2r.gameserver.tables.AugmentationData;
import l2r.gameserver.templates.L2Item;

public final class RequestRefine extends L2GameClientPacket
{
	private static final String _C__D0_44_REQUESTREFINE = "[C] D0:44 RequestRefine";
	// format: (ch)dddd
	private int _targetItemObjId;
	private int _refinerItemObjId;
	private int _gemstoneItemObjId;
	@SuppressWarnings("unused")
	private int _gemstoneCount;

	@Override
	protected void readImpl()
	{
		_targetItemObjId = readD();
		_refinerItemObjId = readD();
		_gemstoneItemObjId = readD();
		_gemstoneCount = readD();
	}

	@Override
	protected void runImpl()
	{
		if(_gemstoneCount < 0)
			return;

		L2Player activeChar = getClient().getActiveChar();
		L2ItemInstance targetItem = activeChar.getInventory().getItemByObjectId(_targetItemObjId);
		L2ItemInstance refinerItem = activeChar.getInventory().getItemByObjectId(_refinerItemObjId);
		L2ItemInstance gemstoneItem = activeChar.getInventory().getItemByObjectId(_gemstoneItemObjId);

		if(targetItem == null || refinerItem == null || gemstoneItem == null || targetItem.getOwnerId() != activeChar.getObjectId() || refinerItem.getOwnerId() != activeChar.getObjectId() || gemstoneItem.getOwnerId() != activeChar.getObjectId() || activeChar.getLevel() < 46)
		{
			activeChar.sendPacket(new ExVariationResult(0, 0, 0));
			activeChar.sendPacket(new SystemMessage(SystemMessage.AUGMENTATION_FAILED_DUE_TO_INAPPROPRIATE_CONDITIONS));
			return;
		}

		if(TryAugmentItem(activeChar, targetItem, refinerItem, gemstoneItem))
		{
			int stat12 = 0x0000FFFF & targetItem.getAugmentation().getAugmentationId();
			int stat34 = targetItem.getAugmentation().getAugmentationId() >> 16;
			activeChar.sendPacket(new ExVariationResult(stat12, stat34, 1));
			activeChar.sendPacket(new SystemMessage(SystemMessage.THE_ITEM_WAS_SUCCESSFULLY_AUGMENTED));
		}
		else
		{
			activeChar.sendPacket(new ExVariationResult(0, 0, 0));
			activeChar.sendPacket(new SystemMessage(SystemMessage.AUGMENTATION_FAILED_DUE_TO_INAPPROPRIATE_CONDITIONS));
		}
	}

	boolean TryAugmentItem(L2Player player, L2ItemInstance targetItem, L2ItemInstance refinerItem, L2ItemInstance gemstoneItem)
	{
		if(targetItem.isAugmented() || targetItem.isWear() || targetItem.isHeroItem())
			return false;

		int itemGrade = targetItem.getItem().getItemGrade();
		int itemType = targetItem.getItem().getType2();
		int lifeStoneId = refinerItem.getItemId();
		int gemstoneItemId = gemstoneItem.getItemId();

		// is the refiner Item a life stone?
		if((lifeStoneId < 8723 || lifeStoneId > 8762) && (lifeStoneId < 9573 || lifeStoneId > 9576))
			return false;

		// must be a weapon, must be > d grade
		// TODO: can do better? : currently: using isdestroyable() as a check for hero / cursed weapons
		if(itemGrade < L2Item.CRYSTAL_C || itemType != L2Item.TYPE2_WEAPON && !Config.ALT_ALLOW_AUGMENT_ALL || !targetItem.isDestroyable())
			return false;

		// player must be able to use augmentation
		if(player.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE || player.isDead() || player.isParalyzed() || player.isFishing() || player.isSitting())
			return false;

		int modifyGemstoneCount = _gemstoneCount;
		int lifeStoneLevel = getLifeStoneLevel(lifeStoneId);
		int lifeStoneGrade = getLifeStoneGrade(lifeStoneId);
		switch(itemGrade)
		{
			case L2Item.CRYSTAL_C:
				if(player.getLevel() < 46 || gemstoneItemId != 2130)
					return false;
				modifyGemstoneCount = 20;
				break;
			case L2Item.CRYSTAL_B:
				if(player.getLevel() < 52 || gemstoneItemId != 2130)
					return false;
				modifyGemstoneCount = 30;
				break;
			case L2Item.CRYSTAL_A:
				if(player.getLevel() < 61 || gemstoneItemId != 2131)
					return false;
				modifyGemstoneCount = 20;
				break;
			case L2Item.CRYSTAL_S:
				if(player.getLevel() < 76 || gemstoneItemId != 2131)
					return false;
				modifyGemstoneCount = 25;
				break;
		}

		// check if the lifestone is appropriate for this player
		switch(lifeStoneLevel)
		{
			case 1:
				if(player.getLevel() < 46)
					return false;
				break;
			case 2:
				if(player.getLevel() < 49)
					return false;
				break;
			case 3:
				if(player.getLevel() < 52)
					return false;
				break;
			case 4:
				if(player.getLevel() < 55)
					return false;
				break;
			case 5:
				if(player.getLevel() < 58)
					return false;
				break;
			case 6:
				if(player.getLevel() < 61)
					return false;
				break;
			case 7:
				if(player.getLevel() < 64)
					return false;
				break;
			case 8:
				if(player.getLevel() < 67)
					return false;
				break;
			case 9:
				if(player.getLevel() < 70)
					return false;
				break;
			case 10:
				if(player.getLevel() < 76)
					return false;
				break;
			case 11:
				if(player.getLevel() < 80)
					return false;
				break;
		}

		if(gemstoneItem.getIntegerLimitedCount() < modifyGemstoneCount)
			return false;
		player.getInventory().destroyItem(_gemstoneItemObjId, modifyGemstoneCount, true);

		// consume the life stone
		player.getInventory().destroyItem(refinerItem, 1, true);

		// generate augmentation
		lifeStoneLevel = Math.min(lifeStoneLevel, 10); // инкрустация описана только до 10 левела
		targetItem.setAugmentation(AugmentationData.getInstance().generateRandomAugmentation(targetItem, lifeStoneLevel, lifeStoneGrade));

		// send an inventory update packet
		InventoryUpdate iu = new InventoryUpdate();
		iu.addModifiedItem(targetItem);
		player.sendPacket(iu);
		player.sendPacket(new UserInfo(player));

		return true;
	}

	private int getLifeStoneGrade(int itemId)
	{
		if(itemId < 9573)
		{
			itemId -= 8723;
			if(itemId < 10 || itemId == 850)
				return 0; // normal grade
			if(itemId < 20 || itemId == 851)
				return 1; // mid grade
			if(itemId < 30 || itemId == 852)
				return 2; // high grade
			return 3; // top grade
		}
		return itemId - 9573;
	}

	private int getLifeStoneLevel(int itemId)
	{
		if(itemId >= 9573 && itemId <= 9576)
			return 11;
		itemId -= 10 * getLifeStoneGrade(itemId);
		itemId -= 8722;
		return itemId;
	}

	@Override
	public String getType()
	{
		return _C__D0_44_REQUESTREFINE;
	}
}