package l2r.gameserver.clientpackets;

import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.model.L2Party;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.entity.SevenSignsFestival.SevenSignsFestival;
import l2r.gameserver.network.L2GameClient.GameClientState;
import l2r.gameserver.serverpackets.CharacterSelectionInfo;
import l2r.gameserver.serverpackets.RestartResponse;
import l2r.gameserver.serverpackets.SystemMessage;

public class RequestRestart extends L2GameClientPacket
{
	private static String _C__57_REQUESTRESTART = "[C] 57 RequestRestart";

	/**
	 * packet type id 0x57
	 * format:      c
	 * @param decrypt
	 */

	@Override
	public void readImpl()
	{}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();

		if(activeChar == null)
			return;

		if(activeChar.isInOlympiadMode())
		{
			activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.RequestRestart.Olympiad", activeChar));
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.inObserverMode())
		{
			activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.RequestRestart.Observer", activeChar));
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.isInCombat())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_RESTART_WHILE_IN_COMBAT));
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.isFishing())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_DO_ANYTHING_ELSE_WHILE_FISHING));
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.isBlocked())
		{
			activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.RequestRestart.OutOfControl", activeChar));
			activeChar.sendActionFailed();
			return;
		}

		// Prevent player from restarting if they are a festival participant
		// and it is in progress, otherwise notify party members that the player
		// is not longer a participant.
		if(activeChar.isFestivalParticipant())
		{
			if(SevenSignsFestival.getInstance().isFestivalInitialized())
			{
				activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.RequestRestart.Festival", activeChar));
				activeChar.sendActionFailed();
				return;
			}
			L2Party playerParty = activeChar.getParty();

			if(playerParty != null)
				playerParty.broadcastMessageToPartyMembers(activeChar.getName() + " has been removed from the upcoming festival.");
		}

		if(getClient() != null)
			getClient().setState(GameClientState.AUTHED);
		activeChar.logout(false, true, false);
		sendPacket(new RestartResponse());
		// send char list
		CharacterSelectionInfo cl = new CharacterSelectionInfo(getClient().getLoginName(), getClient().getSessionId().playOkID1);
		sendPacket(cl);
		getClient().setCharSelection(cl.getCharInfo());
	}

	@Override
	public String getType()
	{
		return _C__57_REQUESTRESTART;
	}
}
