package l2r.gameserver.clientpackets;

import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.instancemanager.SiegeManager;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Effect;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.entity.ClanHall;
import l2r.gameserver.model.entity.siege.Siege;
import l2r.gameserver.model.entity.siege.SiegeClan;
import l2r.gameserver.serverpackets.Die;
import l2r.gameserver.serverpackets.Revive;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.MapRegionTable;
import l2r.util.Location;
import l2r.util.Util;

/**
 * PointType:
 * 0 - To Village
 * 1 - ClanHall
 * 2 - Castle
 * 3 - Fortress
 * 4 - SiegeFlag
 * 5 - ResurectFixed
 */
public class RequestRestartPoint extends L2GameClientPacket
{
	private static String _C__7D_REQUESTRESTARTPOINT = "[C] 7D RequestRestartPoint";

	protected int requestedPointType;
	protected boolean continuation;

	/**
	 * packet type id 0x7D
	 * format:    cd
	 * @param decrypt
	 */
	@Override
	public void readImpl()
	{
		requestedPointType = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();

		if(activeChar == null)
			return;

		if(activeChar.isFakeDeath())
		{
			activeChar.stopEffects(L2Effect.EffectType.FAKE_DEATH);
			activeChar.broadcastPacket(new Revive(activeChar));
			return;
		}

		if(!activeChar.isDead() && !activeChar.getPlayerAccess().IsGM)
		{
			activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.RequestRestartPoint.Cheating", activeChar));
			Util.handleIllegalPlayerAction(activeChar, "RequestRestartPoint[55]", "Tried to use revive cheat", 1);
			return;
		}

		try
		{
			Location loc = null;

			if(activeChar.isFestivalParticipant())
				requestedPointType = 5;

			L2Clan clan = activeChar.getClan();
			ClanHall clanHall = activeChar.getClanHall();
			Siege siege = SiegeManager.getSiege(activeChar);

			switch(requestedPointType)
			{
				case 1:
					if(clan == null || clanHall == null)
					{
						if(clan == null)
							activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.RequestRestartPoint.ClanCheat", activeChar));
						if(clanHall == null)
							activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.RequestRestartPoint.HideoutCheat", activeChar));
						Util.handleIllegalPlayerAction(activeChar, "RequestRestartPoint[61]", "Tried clanhall respawn cheat", 0);
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown);
					}
					else
					{
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClanHall);
						if(clanHall.getFunction(ClanHall.FUNC_RESTORE_EXP) != null)
							activeChar.restoreExp(clanHall.getFunction(ClanHall.FUNC_RESTORE_EXP).getLvl());
					}
					break;
				case 2:
					boolean isInDefense = false;
					if(siege != null && siege.isInProgress() && siege.checkIsDefender(activeChar.getClan()))
						isInDefense = true;
					if((clan == null || clan.getHasCastle() == 0) && !isInDefense)
					{
						activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.RequestRestartPoint.Cheating", activeChar));
						Util.handleIllegalPlayerAction(activeChar, "RequestRestartPoint[79]", "Tried castle respawn cheat", 0);
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown);
					}
					else
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.Castle);
					break;
				case 3:
					if(clan == null || clan.getHasFortress() == 0)
					{
						activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.RequestRestartPoint.Cheating", activeChar));
						Util.handleIllegalPlayerAction(activeChar, "RequestRestartPoint[111]", "Tried fortress respawn cheat", 0);
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown);
					}
					else
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.Fortress);
					break;
				case 4:
					SiegeClan siegeClan = null;
					if(siege != null && siege.isInProgress())
						siegeClan = siege.getAttackerClan(activeChar.getClan());
					if(siegeClan == null || siegeClan.getHeadquarter() == null)
					{
						sendPacket(new SystemMessage(SystemMessage.IF_A_BASE_CAMP_DOES_NOT_EXIST_RESURRECTION_IS_NOT_POSSIBLE));
						sendPacket(new Die(activeChar));
						return;
					}
					loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.Headquarter);
					break;
				case 5:
					if(!activeChar.isFestivalParticipant() && !activeChar.getPlayerAccess().ResurectFixed)
					{
						activeChar.sendMessage(new CustomMessage("l2r.gameserver.clientpackets.RequestRestartPoint.Cheating", activeChar));
						Util.handleIllegalPlayerAction(activeChar, "RequestRestartPoint[103]", "Tried fixed respawn cheat", 0);
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown);
					}
					else
						loc = activeChar.getLoc();
					break;
				default:
					loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown);
					break;
			}

			activeChar.setIsPendingRevive(true);
			activeChar.teleToLocation(loc);
		}
		catch(Throwable e)
		{

		}
	}

	@Override
	public String getType()
	{
		return _C__7D_REQUESTRESTARTPOINT;
	}
}