package l2r.gameserver.clientpackets;

import l2r.Config;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.serverpackets.FriendRecvMsg;
import l2r.gameserver.serverpackets.SystemMessage;

import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

/**
 * Recieve Private (Friend) Message - 0xCC
 *
 * Format: c SS
 *
 * S: Message
 * S: Receiving Player
 *
 * @author Tempy
 *
 */
public class RequestSendFriendMsg extends L2GameClientPacket
{
	private static String _C__CC_REQUESTSENDMSG = "[C] CC RequestSendMsg";
	private static Logger _logChat = Logger.getLogger("chat");

	private String _message;
	private String _reciever;

	@Override
	public void readImpl()
	{
		_message = readS();
		_reciever = readS();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(activeChar.getNoChannel() != 0)
		{
			if(activeChar.getNoChannelRemained() > 0 || activeChar.getNoChannel() < 0)
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.CHATTING_IS_CURRENTLY_PROHIBITED_IF_YOU_TRY_TO_CHAT_BEFORE_THE_PROHIBITION_IS_REMOVED_THE_PROHIBITION_TIME_WILL_BECOME_EVEN_LONGER));
				return;
			}
			activeChar.updateNoChannel(0);
		}

		L2Player targetPlayer = L2World.getPlayer(_reciever);
		if(targetPlayer == null)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.TARGET_IS_NOT_FOUND_IN_THE_GAME));
			return;
		}

		if(Config.LOG_CHAT)
		{
			LogRecord record = new LogRecord(Level.INFO, _message);
			record.setLoggerName("chat");
			record.setParameters(new Object[] {
					"PRIV_MSG", "[" + activeChar.getName() + " to " + _reciever + "]" });

			_logChat.log(record);
		}

		FriendRecvMsg frm = new FriendRecvMsg(activeChar.getName(), _reciever, _message);
		targetPlayer.sendPacket(frm);
	}

	@Override
	public String getType()
	{
		return _C__CC_REQUESTSENDMSG;
	}
}