package l2r.gameserver.clientpackets;

import java.util.ArrayList;
import java.util.List;

import l2r.Config;
import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.extensions.listeners.PropertyCollection;
import l2r.extensions.multilang.CustomMessage;
import l2r.extensions.scripts.Scripts;
import l2r.gameserver.handler.IVoicedCommandHandler;
import l2r.gameserver.handler.VoicedCommandHandler;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.serverpackets.Say2;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.FakePlayersTable;
import l2r.gameserver.tables.GmListTable;
import l2r.gameserver.tables.MapRegionTable;
import l2r.status.GameStatusThread;
import l2r.status.Status;
import l2r.util.Calculator;
import l2r.util.Log;
import l2r.util.Util;

public class Say2C extends L2GameClientPacket
{
	private static String _C__49_SAY2C = "[C] 49 Say2C";

	public final static int ALL = 0;
	public final static int ALL_CHAT_RANGE = 1250; //Дальность белого чата
	public final static int SHOUT = 1; //!
	public final static int TELL = 2; //\"
	public final static int PARTY = 3; //#
	public final static int CLAN = 4; //@
	public final static int GM = 5;
	public final static int PETITION_PLAYER = 6; // used for petition
	public final static int PETITION_GM = 7; //* used for petition
	public final static int TRADE = 8; //+
	public final static int ALLIANCE = 9; //$
	public final static int ANNOUNCEMENT = 10;
	public final static int COMMANDCHANNEL_ALL = 15; //`` (pink) команды лидера СС
	public final static int COMMANDCHANNEL_COMMANDER = 16; //` (yellow) команды лидеров партий в СС
	public final static int HERO_VOICE = 17; //%
	public final static int CRITICAL_ANNOUNCEMENT = 18; //dark cyan
	public final static int[] BAN_CHAN = Config.BAN_CHANNEL_LIST;

	public static String[] chatNames = {
			"ALL	",
			"SHOUT",
			"TELL ",
			"PARTY",
			"CLAN ",
			"GM	 ",
			"PETITION_PLAYER",
			"PETITION_GM",
			"TRADE",
			"ALLIANCE",
			"ANNOUNCEMENT",
			"",
			"",
			"",
			"",
			"COMMANDCHANNEL_ALL",
			"COMMANDCHANNEL_COMMANDER",
			"HERO_VOICE",
			"CRITICAL_ANNOUNCEMENT" };

	protected static List<String> _banned = new ArrayList<String>();
	private String _text;
	private int _type;
	private String _target;

	/**
	 * packet type id 0x49
	 * format:		cSd (S)
	 * @param decrypt
	 */
	@Override
	public void readImpl()
	/*{
		_text = readS();
		_type = readD();
		_target = _type == TELL ? readS() : null;
	}*/
	{
		_text = readS();
		_text = _text.replaceAll("\\\\n", "");
		try
		{
			_type = readD();
		}
		catch(Exception e)
		{
			_type = 0;
		}
		_target = _type == TELL ? readS() : null;
	}

	@Override
	public void runImpl()
	{
		if(Config.DEBUG)
			_log.info("Say type:" + _type);

		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(_type > chatNames.length || _text == null || _text.length() == 0)
		{
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.isBotLevel() > 0 && Config.ALLOW_L2WALKER_CLIENT == Config.L2WalkerAllowed.Peace && (_type == ALL || _type == SHOUT))
		{
			activeChar.sendMessage("All and shout channels are not allowed for bots");
			return;
		}

		if(Config.LOG_TELNET)
		{
			String line_output;

			if(_type == TELL)
				line_output = chatNames[_type] + "[" + activeChar.getName() + " to " + _target + "] " + _text;
			else
				line_output = chatNames[_type] + "[" + activeChar.getName() + "] " + _text;
			telnet_output(line_output, _type);
		}

		if(_text.startsWith("."))
		{
			String fullcmd = _text.substring(1).trim();
			String command = fullcmd.split("\\s+")[0];
			String args = fullcmd.substring(command.length()).trim();

			if(command.length() > 0)
			{
				// then check for VoicedCommands
				IVoicedCommandHandler vch = VoicedCommandHandler.getInstance().getVoicedCommandHandler(command);
				if(vch != null)
				{
					vch.useVoicedCommand(command, activeChar, args);
					return;
				}
			}
			Scripts.DCL.call("dcl_voice", "voice", command, args);
			activeChar.sendMessage("Wrong command");
			return;
		}
		else if(_text.startsWith("==") || _text.startsWith("--"))
		{
			activeChar.sendMessage(Util.formatDouble(Calculator.eval(_text.substring(2).replace(",", ".")), "Wrong equation: result is not a number", false));
			return;
		}

		boolean globalchat = _type != ALLIANCE && _type != CLAN && _type != PARTY;
		boolean chan_banned = false;
		for(int i = 0; i <= Config.MAT_BAN_COUNT_CHANNELS; i++)
			if(_type == BAN_CHAN[i])
				chan_banned = true;
		if((globalchat || chan_banned) && activeChar.getNoChannel() != 0)
		{
			if(activeChar.getNoChannelRemained() > 0 || activeChar.getNoChannel() < 0)
			{
				if(activeChar.getNoChannel() > 0)
				{
					int timeRemained = Math.round(activeChar.getNoChannelRemained() / 60000);
					activeChar.sendMessage(new CustomMessage("common.ChatBanned", activeChar).addNumber(timeRemained));
				}
				else
					activeChar.sendMessage(new CustomMessage("common.ChatBannedPermanently", activeChar));
				activeChar.sendActionFailed();
				return;
			}
			activeChar.updateNoChannel(0);
		}

		if(globalchat)
			if(Config.MAT_REPLACE)
			{
				for(String pattern : Config.MAT_LIST)
					if(_text.matches(".*" + pattern + ".*"))
					{
						_text = Config.MAT_REPLACE_STRING;
						activeChar.sendActionFailed();
						continue;
					}
			}
			else if(Config.MAT_BANCHAT)
				for(String pattern : Config.MAT_LIST)
					if(_text.matches(".*" + pattern + ".*"))
					{
						activeChar.sendMessage("You are banned in all chats. Time to unban: " + Config.UNCHATBANTIME * 60 + "sec.");
						Log.add("" + activeChar + ": " + _text, "abuse");
						activeChar.updateNoChannel(Config.UNCHATBANTIME * 60000);
						activeChar.sendActionFailed();
						return;
					}

		if(Config.LOG_CHAT)
		{
			String _logchat = chatNames[_type] + " ";
			if(_type == TELL)
				_logchat += "[" + activeChar.getName() + " to " + _target + "] " + _text;
			else
				_logchat += "[" + activeChar.getName() + "] " + _text;
			Log.add(_logchat, "chat");
		}

		Say2 cs = new Say2(activeChar.getObjectId(), _type, activeChar.getName(), _text);
		int mapregion = MapRegionTable.getInstance().getMapRegion(activeChar.getX(), activeChar.getY());

		switch(_type)
		{
			case TELL:
				L2Player receiver = L2World.getPlayer(_target);
				if(receiver == null && Config.ALLOW_FAKE_PLAYERS && FakePlayersTable.getActiveFakePlayers().contains(_target))
				{
					cs = new Say2(activeChar.getObjectId(), _type, "->" + _target, _text);
					activeChar.sendPacket(cs);
					return;
				}
				else if(receiver != null && receiver.isInOfflineMode())
				{
					activeChar.sendMessage("The person is in offline trade mode");
					activeChar.sendActionFailed();
				}
				else if(receiver != null && !receiver.isInBlockList(activeChar) && !receiver.isBlockAll())
				{
					if(!receiver.getMessageRefusal())
					{
						receiver.sendPacket(cs);
						cs = new Say2(activeChar.getObjectId(), _type, "->" + receiver.getName(), _text);
						activeChar.sendPacket(cs);
					}
					else
						activeChar.sendPacket(new SystemMessage(SystemMessage.THE_PERSON_IS_IN_A_MESSAGE_REFUSAL_MODE));
				}
				else if(receiver == null)
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.S1_IS_NOT_LOGGED_IN).addString(_target));
					activeChar.sendActionFailed();
				}
				else
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_BEEN_BLOCKED_FROM_THE_CONTACT_YOU_SELECTED));
					activeChar.sendActionFailed();
				}
				break;
			case SHOUT:
				if(activeChar.isCursedWeaponEquipped())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.SHOUT_AND_TRADE_CHATING_CANNOT_BE_USED_SHILE_POSSESSING_A_CURSED_WEAPON));
					return;
				}
				if(activeChar.inObserverMode())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_CHAT_LOCALLY_WHILE_OBSERVING));
					return;
				}
				if(activeChar.getLevel() >= Config.GLOBAL_CHAT || activeChar.isGM())
				{
					for(L2Player player : L2World.getAllPlayers())
						if(!player.isInBlockList(activeChar) && !player.isBlockAll())
							player.sendPacket(cs);
				}
				else
				{
					if(Config.SHOUT_CHAT_MODE == 1)
					{
						for(L2Player player : L2World.getAroundPlayers(activeChar))
							if(!player.isInBlockList(activeChar) && !player.isBlockAll() && player != activeChar)
								player.sendPacket(cs);
					}
					else
						for(L2Player player : L2World.getAllPlayers())
							if(MapRegionTable.getInstance().getMapRegion(player.getX(), player.getY()) == mapregion && !player.isInBlockList(activeChar) && !player.isBlockAll() && player != activeChar)
								player.sendPacket(cs);
					activeChar.sendPacket(cs);
				}
				break;
			case TRADE:
				if(activeChar.isCursedWeaponEquipped())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.SHOUT_AND_TRADE_CHATING_CANNOT_BE_USED_SHILE_POSSESSING_A_CURSED_WEAPON));
					return;
				}
				if(activeChar.inObserverMode())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_CHAT_LOCALLY_WHILE_OBSERVING));
					return;
				}
				if(activeChar.getLevel() >= Config.GLOBAL_TRADE_CHAT || activeChar.isGM())
				{
					for(L2Player player : L2World.getAllPlayers())
						if(!player.isInBlockList(activeChar) && !player.isBlockAll())
							player.sendPacket(cs);
				}
				else
				{
					if(Config.TRADE_CHAT_MODE == 1)
					{
						for(L2Player player : L2World.getAroundPlayers(activeChar))
							if(!player.isInBlockList(activeChar) && !player.isBlockAll() && player != activeChar)
								player.sendPacket(cs);
					}
					else
						for(L2Player player : L2World.getAllPlayers())
							if(MapRegionTable.getInstance().getMapRegion(player.getX(), player.getY()) == mapregion && !player.isInBlockList(activeChar) && !player.isBlockAll() && player != activeChar)
								player.sendPacket(cs);
					activeChar.sendPacket(cs);
				}
				break;
			case ALL:
				if(activeChar.inObserverMode())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_CHAT_LOCALLY_WHILE_OBSERVING));
					return;
				}
				for(L2Player player : activeChar.getAroundPlayers(ALL_CHAT_RANGE))
					if(!player.isInBlockList(activeChar) && !player.isBlockAll() && player != activeChar)
						player.sendPacket(cs);
				activeChar.sendPacket(cs);
				break;
			case CLAN:
				if(activeChar.getClan() != null)
					activeChar.getClan().broadcastToOnlineMembers(cs);
				else
					activeChar.sendActionFailed();
				break;
			case ALLIANCE:
				if(activeChar.getClan() != null && activeChar.getClan().getAlliance() != null)
					activeChar.getClan().getAlliance().broadcastToOnlineMembers(cs);
				else
					activeChar.sendActionFailed();
				break;
			case PARTY:
				if(activeChar.isInParty())
					activeChar.getParty().broadcastToPartyMembers(cs);
				else
					activeChar.sendActionFailed();
				break;
			case COMMANDCHANNEL_ALL:
				if(!activeChar.isInParty() || !activeChar.getParty().isInCommandChannel())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_AUTHORITY_TO_USE_THE_COMMAND_CHANNEL));
					return;
				}
				if(activeChar.getParty().getCommandChannel().getChannelLeader() == activeChar)
					activeChar.getParty().getCommandChannel().broadcastToChannelMembers(cs);
				else
					activeChar.sendPacket(new SystemMessage(SystemMessage.ONLY_CHANNEL_OPENER_CAN_GIVE_ALL_COMMAND));
				break;
			case COMMANDCHANNEL_COMMANDER:
				if(!activeChar.isInParty() || !activeChar.getParty().isInCommandChannel())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_AUTHORITY_TO_USE_THE_COMMAND_CHANNEL));
					return;
				}
				if(activeChar.getParty().isLeader(activeChar))
					activeChar.getParty().getCommandChannel().broadcastToChannelMembers(cs);
				else
					activeChar.sendPacket(new SystemMessage(SystemMessage.ONLY_A_PARTY_LEADER_CAN_ACCESS_THE_COMMAND_CHANNEL));
				break;
			case HERO_VOICE:
				if(activeChar.isHero() || activeChar.getPlayerAccess().CanAnnounce)
				{
					// Ограничение только для героев, гм-мы пускай говорят.
					if(!activeChar.getPlayerAccess().CanAnnounce)
					{
						Long lastTime = (Long) activeChar.getProperty(PropertyCollection.HeroChatLaunched);
						long curTime = System.currentTimeMillis();
						if(lastTime != null && lastTime + 10000L > curTime)
						{
							activeChar.sendMessage("Hero chat is allowed once per 10 seconds.");
							return;
						}
						activeChar.addProperty(PropertyCollection.HeroChatLaunched, curTime);
					}

					for(L2Player player : L2World.getAllPlayers())
						if(!player.isInBlockList(activeChar) && !player.isBlockAll())
							player.sendPacket(cs);
				}
				break;
			case PETITION_PLAYER:
			case PETITION_GM:
				for(L2Player gm : GmListTable.getAllGMs())
					if(!gm.getMessageRefusal())
						gm.sendPacket(cs);
				break;
			default:
				_log.warning("Character " + activeChar.getName() + " used unknown chat type: " + _type + ". Cheater?");
		}
	}

	private void telnet_output(String _text, int type)
	{
		GameStatusThread tinstance = Status.telnetlist;

		while(tinstance != null)
		{
			if(type == TELL && tinstance.log_tell == 1)
				tinstance.write(_text);
			else if(tinstance.log_chat == 1)
				tinstance.write(_text);
			tinstance = tinstance.next;
		}
	}

	@Override
	public String getType()
	{
		return _C__49_SAY2C;
	}

	public class UnbanTask implements Runnable
	{
		private String _name;

		public UnbanTask(String Name)
		{
			_name = Name;
		}

		public void run()
		{
			L2Player plyr = L2World.getPlayer(_name);
			if(plyr != null)
			{
				plyr.setAccessLevel(0);
				plyr.sendMessage("Nochannel deactivated");
				Log.add("" + plyr + ": unbanchat online", "abuse");
			}
			else
			{
				setCharacterAccessLevel(_name, 0);
				Log.add("Player " + _name + ": unbanchat offline", "abuse");
			}

			_banned.remove(_name);
		}
	}

	public void setCharacterAccessLevel(String user, int banLevel)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			String stmt = "UPDATE characters SET characters.accesslevel = ? WHERE characters.char_name=?";
			statement = con.prepareStatement(stmt);
			statement.setInt(1, banLevel);
			statement.setString(2, user);
			statement.executeUpdate();
		}
		catch(Exception e)
		{
			_log.warning("Could not set accessLevl:" + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}
}