package l2r.gameserver.clientpackets;

import java.util.concurrent.ConcurrentLinkedQueue;

import l2r.Config;
import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.L2Zone;
import l2r.gameserver.model.TradeItem;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.PrivateStoreMsgSell;
import l2r.gameserver.serverpackets.SystemMessage;

/**
 * Это список вещей которые игрок хочет продать в создаваемом им приватном магазине
 *
 * Старое название SetPrivateStoreListSell
 */
public class SetPrivateStoreList extends L2GameClientPacket
{
	private static String _C__74_SETPRIVATESTORELISTSELL = "[C] 74 SetPrivateStoreListSell";
	//Format: cddb, b = array of (ddd)

	private int _count;
	@SuppressWarnings("unused")
	private boolean _package;
	private int[] _items; // count * 3

	@Override
	public void readImpl()
	{
		_package = readD() == 1;
		_count = readD();
		// Иначе нехватит памяти при создании массива.
		if(_count * 12 > _buf.remaining() || _count > Short.MAX_VALUE || _count <= 0)
		{
			_items = null;
			return;
		}
		_items = new int[_count * 3];
		for(int i = 0; i < _count; i++)
		{
			_items[i * 3 + 0] = readD(); //objectId
			_items[i * 3 + 1] = readD(); //count
			_items[i * 3 + 2] = readD(); //price
			if(_items[i * 3 + 1] < 0)
			{
				_items = null;
				break;
			}
		}
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(_items == null || _count <= 0)
		{
			cancelStore(activeChar);
			return;
		}

		if(activeChar.isMounted())
		{
			cancelStore(activeChar);
			return;
		}

		int maxSlots = activeChar.getTradeLimit();
		if(_count > maxSlots)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_EXCEEDED_THE_QUANTITY_THAT_CAN_BE_INPUTTED));
			cancelStore(activeChar);
			return;
		}

		if(activeChar.isActionBlocked(L2Zone.BLOCKED_ACTION_PRIVATE_STORE))
		{
			cancelStore(activeChar);
			return;
		}

		if(Config.SERVICES_TRADE_ONLY_FAR && !activeChar.isInStoreMode())
		{
			boolean tradenear = false;
			for(L2Player player : L2World.getAroundPlayers(activeChar, Config.SERVICES_TRADE_RADIUS, Config.SERVICES_TRADE_RADIUS))
				if(player.isInStoreMode())
				{
					tradenear = true;
					break;
				}

			if(L2World.getAroundNpc(activeChar, Config.SERVICES_TRADE_RADIUS, Config.SERVICES_TRADE_RADIUS).size() > 0)
				tradenear = true;

			if(tradenear)
			{
				activeChar.sendMessage(new CustomMessage("trade.OtherTradersNear", activeChar));
				cancelStore(activeChar);
			}
		}

		TradeItem temp;
		ConcurrentLinkedQueue<TradeItem> listsell = new ConcurrentLinkedQueue<TradeItem>();

		int count = _count;
		for(int x = 0; x < _count; x++)
		{
			int objectId = _items[x * 3 + 0];
			int cnt = _items[x * 3 + 1];
			int price = _items[x * 3 + 2];

			L2ItemInstance itemToSell = activeChar.getInventory().getItemByObjectId(objectId);

			if(cnt < 1 || itemToSell == null || itemToSell.isWear() || !itemToSell.canBeTraded(activeChar))
			{
				count--;
				continue;
			}

			// If player sells the enchant scroll he is using, deactivate it
			if(activeChar.getEnchantScroll() != null && itemToSell.getObjectId() == activeChar.getEnchantScroll().getObjectId())
				activeChar.setEnchantScroll(null);

			if(cnt > itemToSell.getIntegerLimitedCount())
				cnt = itemToSell.getIntegerLimitedCount();

			temp = new TradeItem();
			temp.setObjectId(objectId);
			temp.setCount(cnt);
			temp.setOwnersPrice(price);
			temp.setItemId(itemToSell.getItemId());
			temp.setEnchantLevel(itemToSell.getEnchantLevel());
			listsell.add(temp);
		}

		if(count != 0)
		{
			activeChar.setSellList(listsell);
			activeChar.setPrivateStoreType(_package ? L2Player.STORE_PRIVATE_SELL_PACKAGE : L2Player.STORE_PRIVATE_SELL);
			activeChar.broadcastUserInfo();
			activeChar.broadcastPacket(new PrivateStoreMsgSell(activeChar));
			activeChar.sitDown();
		}
		else
			cancelStore(activeChar);
	}

	private void cancelStore(L2Player activeChar)
	{
		activeChar.setPrivateStoreType(L2Player.STORE_PRIVATE_NONE);
		activeChar.broadcastUserInfo();
		activeChar.getBuyList().clear();
		if(activeChar.isInOfflineMode() && Config.SERVICES_OFFLINE_TRADE_KICK_NOT_TRADING)
		{
			activeChar.setOfflineMode(false);
			activeChar.logout(false, false, true);
			activeChar.getNetConnection().disconnectOffline();
		}
	}

	@Override
	public String getType()
	{
		return _C__74_SETPRIVATESTORELISTSELL;
	}
}