package l2r.gameserver.clientpackets;

import java.util.concurrent.ConcurrentLinkedQueue;

import l2r.Config;
import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.TradeItem;
import l2r.gameserver.model.L2Zone;
import l2r.gameserver.serverpackets.ChangeWaitType;
import l2r.gameserver.serverpackets.PrivateStoreMsgBuy;
import l2r.gameserver.serverpackets.SystemMessage;

public class SetPrivateStoreListBuy extends L2GameClientPacket
{
	private static String _C__91_SETPRIVATESTORELISTBUY = "[C] 91 SetPrivateStoreListBuy";
	// format: cdb, b - array of (dhhdd)
	private int _count;
	private int[] _items; // count * 3

	@Override
	public void readImpl()
	{
		_count = readD();
		if(_count * 16 > _buf.remaining() || _count > Short.MAX_VALUE || _count <= 0)
		{
			_items = null;
			return;
		}
		_items = new int[_count * 3];
		for(int i = 0; i < _count; i++)
		{
			_items[i * 3 + 0] = readD(); //item id
			readH();// don't know what this is
			readH();
			_items[i * 3 + 1] = readD(); //count
			_items[i * 3 + 2] = readD(); //price
			if(_items[i * 3 + 1] < 0)
			{
				_items = null;
				break;
			}
		}
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(_items == null)
		{
			cancelStore(activeChar);
			return;
		}

		if(activeChar.isMounted())
		{
			cancelStore(activeChar);
			return;
		}

		int maxSlots = activeChar.getTradeLimit();
		if(_count > maxSlots)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_EXCEEDED_THE_QUANTITY_THAT_CAN_BE_INPUTTED));
			cancelStore(activeChar);
			return;
		}

		if(activeChar.isActionBlocked(L2Zone.BLOCKED_ACTION_PRIVATE_STORE))
		{
			cancelStore(activeChar);
			return;
		}

		if(Config.SERVICES_TRADE_ONLY_FAR && !activeChar.isInStoreMode())
		{
			boolean tradenear = false;
			for(L2Player player : L2World.getAroundPlayers(activeChar, Config.SERVICES_TRADE_RADIUS, Config.SERVICES_TRADE_RADIUS))
				if(player.isInStoreMode())
				{
					tradenear = true;
					break;
				}

			if(L2World.getAroundNpc(activeChar, Config.SERVICES_TRADE_RADIUS, Config.SERVICES_TRADE_RADIUS).size() > 0)
				tradenear = true;

			if(tradenear)
			{
				activeChar.sendMessage(new CustomMessage("trade.OtherTradersNear", activeChar));
				cancelStore(activeChar);
			}
		}

		TradeItem temp;
		ConcurrentLinkedQueue<TradeItem> listbuy = new ConcurrentLinkedQueue<TradeItem>();
		int totalCost = 0;

		for(int x = 0; x < _count; x++)
		{
			if(_items[x * 3 + 1] < 1)
			{
				_count--;
				continue;
			}
			temp = new TradeItem();
			temp.setItemId(_items[x * 3 + 0]);
			temp.setCount(_items[x * 3 + 1]);
			temp.setOwnersPrice(_items[x * 3 + 2]);
			totalCost += temp.getOwnersPrice() * temp.getCount();
			if(temp.getOwnersPrice() < 0 || temp.getCount() < 0)
			{
				cancelStore(activeChar);
				return;
			}
			if(temp.getOwnersPrice() > activeChar.getAdena())
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.THE_PURCHASE_PRICE_IS_HIGHER_THAN_THE_AMOUNT_OF_MONEY_THAT_YOU_HAVE_AND_SO_YOU_CANNOT_OPEN_A_PERSONAL_STORE));
				cancelStore(activeChar);
				return;
			}
			listbuy.add(temp);
		}

		if(_count != 0)
		{
			activeChar.setBuyList(listbuy);
			activeChar.setPrivateStoreType(L2Player.STORE_PRIVATE_BUY);
			activeChar.broadcastPacket(new ChangeWaitType(activeChar, ChangeWaitType.WT_SITTING));
			activeChar.broadcastUserInfo();
			activeChar.broadcastPacket(new PrivateStoreMsgBuy(activeChar));
			activeChar.sitDown();
			return;
		}

		cancelStore(activeChar);
	}

	private void cancelStore(L2Player activeChar)
	{
		activeChar.setPrivateStoreType(L2Player.STORE_PRIVATE_NONE);
		activeChar.broadcastUserInfo();
		activeChar.getBuyList().clear();
		if(activeChar.isInOfflineMode() && Config.SERVICES_OFFLINE_TRADE_KICK_NOT_TRADING)
		{
			activeChar.setOfflineMode(false);
			activeChar.logout(false, false, true);
			activeChar.getNetConnection().disconnectOffline();
		}
	}

	@Override
	public String getType()
	{
		return _C__91_SETPRIVATESTORELISTBUY;
	}
}