package l2r.gameserver.clientpackets;

import l2r.Config;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Zone.ZoneType;
import l2r.gameserver.serverpackets.CharMoveToLocation;
import l2r.gameserver.serverpackets.ValidateLocation;
import l2r.gameserver.tables.MapRegionTable;
import l2r.util.Location;

public class ValidatePosition extends L2GameClientPacket
{
	private static String _C__48_VALIDATEPOSITION = "[C] 48 ValidatePosition";

	private Location _loc = new Location(0, 0, 0);
	@SuppressWarnings("unused")
	private int _data;
	private double _diff;
	private Location _lastClientPosition;
	private Location _lastServerPosition;

	/**
	 * packet type id 0x48
	 * format:		cddddd
	 */
	@Override
	public void readImpl()
	{
		_loc.setX(readD());
		_loc.setY(readD());
		_loc.setZ(readD());
		_loc.h = readD();
		_data = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(activeChar.isTeleporting() || activeChar.inObserverMode())
			return;

		_lastClientPosition = activeChar.getLastClientPosition();
		_lastServerPosition = activeChar.getLastServerPosition();

		if(_lastClientPosition == null)
			_lastClientPosition = activeChar.getLoc();
		if(_lastServerPosition == null)
			_lastServerPosition = activeChar.getLoc();

		if(activeChar.getX() == 0 && activeChar.getY() == 0 && activeChar.getZ() == 0)
		{
			correctPosition(activeChar);
			_loc = activeChar.getLoc();
		}

		int dz = _lastClientPosition.getZ() - _loc.getZ();

		if(Config.DAMAGE_FROM_FALLING && dz > 333 && !activeChar.isInZone(ZoneType.water))
			activeChar.falling(dz);

		_diff = activeChar.getDistance(_loc.getX(), _loc.getY());

		if(activeChar.isInBoat())
			activeChar.setXYZ(activeChar.getBoat().getX(), activeChar.getBoat().getY(), activeChar.getBoat().getZ());
		else if(activeChar.isFlying() || activeChar.isInWater())
			activeChar.setXYZ(_loc.getX(), _loc.getY(), _loc.getZ());
		else if(_loc.getZ() < -15000 || _loc.getZ() > 15000)
			correctPosition(activeChar);
		else if(_diff > 1000)
			activeChar.broadcastPacket(new ValidateLocation(activeChar));
		else if(_diff > activeChar.getMoveSpeed())
			if(activeChar.isMoving)
				activeChar.broadcastPacket(new CharMoveToLocation(activeChar));
			else
				activeChar.broadcastPacket(new ValidateLocation(activeChar));

		if(activeChar.getPet() != null && !activeChar.getPet().isInRange())
			activeChar.getPet().teleportToOwner();

		activeChar.setLastClientPosition(_loc.setH(activeChar.getHeading()));
		activeChar.setLastServerPosition(activeChar.getLoc());
	}

	private void correctPosition(L2Player activeChar)
	{
		if(_lastServerPosition.getX() != 0 && _lastServerPosition.getY() != 0 && _lastServerPosition.getZ() != 0)
		{
			if(GeoClient.getInstance().getNSWE(_lastServerPosition.getX(), _lastServerPosition.getY(), _lastServerPosition.getZ()) == 15)
			{
				activeChar.setXYZ(_lastServerPosition.getX(), _lastServerPosition.getY(), GeoClient.getInstance().getHeight(_lastServerPosition));
				activeChar.stopMove();
				activeChar.broadcastPacket(new ValidateLocation(activeChar));
			}
			else
				activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown));
		}
		else if(_lastClientPosition.getX() != 0 && _lastClientPosition.getY() != 0 && _lastClientPosition.getZ() != 0)
		{
			if(GeoClient.getInstance().getNSWE(_lastClientPosition.getX(), _lastClientPosition.getY(), _lastClientPosition.getZ()) == 15)
			{
				activeChar.setXYZ(_lastClientPosition.getX(), _lastClientPosition.getY(), GeoClient.getInstance().getHeight(_lastClientPosition));
				activeChar.stopMove();
				activeChar.broadcastPacket(new ValidateLocation(activeChar));
			}
			else
				activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown));
		}
		else
			activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown));
	}

	@Override
	public String getType()
	{
		return _C__48_VALIDATEPOSITION;
	}
}