package l2r.gameserver.geodata;

import java.util.Vector;

import l2r.gameserver.geodata.network.GeoConfig;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.util.Location;
import l2r.Config;

/**
 * @Author: Diamond
 * @Date: 20/5/2007
 * @Time: 9:57:48
 */
public class GeoMove
{
	private final GeoEngine engine;
	private static final Vector<Location> emptyTargetRecorder = new Vector<Location>();

	public GeoMove(GeoEngine engine)
	{
		this.engine = engine;
	}

	public Vector<Location> findPath(int x, int y, int z, Location target, boolean playable, boolean showTrace)
	{
		if(Math.abs(z - target.getZ()) > 256)
			return emptyTargetRecorder;

		z = engine.getHeight(x, y, z);
		target.setZ(engine.getHeight(target));
		PathFind n = new PathFind(x, y, z, target.getX(), target.getY(), target.getZ(), playable, engine);

		if(n.getPath() == null || n.getPath().isEmpty())
			return emptyTargetRecorder;

		Vector<Location> targetRecorder = new Vector<Location>();

		// добавляем первую точку в список (начальная позиция чара)
		targetRecorder.add(new Location(x, y, z));

		for(PathFindBuffers.GeoNode p : n.getPath())
			targetRecorder.add(new Location(p._x, p._y, p._z).geo2world());

		// добавляем последнюю точку в список (цель)
		targetRecorder.add(target);

		if(GeoConfig.PATH_CLEAN)
			pathClean(targetRecorder);
/*Not work logging whith remote
		if(showTrace && obj.isPlayer && obj.getPlayer().getVarB("trace"))
		{
			L2Player player = obj.getPlayer();
			ExShowTrace trace = new ExShowTrace();
			int i = 0;
			for(Location loc : targetRecorder)
			{
				i++;
				if(i == 1 || i == targetRecorder.size())
					continue;
				trace.addTrace(loc.getX(), loc.getY(), loc.getZ() + 15, 30000);
			}
			player.sendPacket(trace);
		}

		if(obj.isPlayer && Config.GEODATA_DEBUG)
		{
			System.out.println("Path: -----------------------------------------------");
			for(Location l : targetRecorder)
				System.out.println("Path: " + l.getX() + ", " + l.getY() + ", " + l.getZ());
		}
*/
		return targetRecorder;
	}

	/**
	 * Очищает путь от ненужных точек.
	 *
	 * @param path путь который следует очистить
	 */
	private void pathClean(Vector<Location> path)
	{
		int size = path.size();
		if(size > 2)
		{
			for(int i = 2; i < size; i++)
			{
				Location p3 = path.elementAt(i);
				Location p2 = path.elementAt(i - 1);
				Location p1 = path.elementAt(i - 2);
				if(p1.equals(p2) || p3.equals(p2) || IsPointInLine(p1.getX(), p1.getY(), p3.getX(), p3.getY(), p2))
				{
					path.remove(i - 1);
					size--;
					i = Math.max(2, i - 2);
				}
			}
		}

		for(int current = 0; current < path.size() - 2; current++)
		{
			for(int sub = current + 2; sub < path.size(); sub++)
			{
				Location one = path.elementAt(current);
				Location two = path.elementAt(sub);
				if(!one.equals(two) && !engine.canMoveWithCollision(one.getX(), one.getY(), one.getZ(), two.getX(), two.getY(), two.getZ()))
					continue;
				for(; current + 1 < sub; sub--)
					path.remove(current + 1);
			}
		}
	}

	private static boolean IsPointInLine(int x1, int y1, int x2, int y2, Location P)
	{
		if(x1 == x2 && x2 == P.getX() || y1 == y2 && y2 == P.getY())
			return true;
		return (x1 - P.getX()) * (y1 - P.getY()) == (P.getX() - x2) * (P.getY() - y2);
	}

}
