package l2r.gameserver.geodata;

import java.util.ArrayList;
import l2r.Config;
import l2r.gameserver.geodata.network.GeoConfig;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.tables.ItemTable;
import l2r.util.Location;

/**
 * @Author: Diamond
 * @Date: 20/5/2007
 * @Time: 9:57:48
 */
public class PathFind
{
	private static final byte NSWE_NONE = 0, EAST = 1, WEST = 2, SOUTH = 4, NORTH = 8, NSWE_ALL = 15;
	private final GeoEngine engine;
	private ArrayList<PathFindBuffers.GeoNode> path;
	private PathFindBuffers.PathFindBuffer buff;
	private static ArrayList<L2ItemInstance> debugItems;

	public PathFind(int x, int y, int z, int destX, int destY, int destZ, boolean isPlayable, GeoEngine engine)
	{
		this.engine = engine;
		Location startpoint = GeoConfig.PATHFIND_BOOST == 0 ? new Location(x, y, z) : engine.moveCheckWithCollision(x, y, z, destX, destY, true);
		Location native_endpoint = new Location(destX, destY, destZ);
		Location endpoint = GeoConfig.PATHFIND_BOOST == 2 && Math.abs(destZ - z) <= 200 ? engine.moveCheckBackwardWithCollision(destX, destY, destZ, startpoint.getX(), startpoint.getY(), true) : native_endpoint.clone();
		startpoint.world2geo();
		native_endpoint.world2geo();
		endpoint.world2geo();
		startpoint.setZ(engine.NgetHeight(startpoint.getX(), startpoint.getY(), startpoint.getZ()));
		endpoint.setZ(engine.NgetHeight(endpoint.getX(), endpoint.getY(), endpoint.getZ()));
		int xdiff = Math.abs(endpoint.getX() - startpoint.getX());
		int ydiff = Math.abs(endpoint.getY() - startpoint.getY());
//		if(xdiff == 0 && ydiff == 0)//Fixed By Flare at rev 2559 by Return SoulKeeperFix see rev 70
//		Please do not uncomment this if block, it makes possible to walk throught walls(c)Death 
//		{
//			if(Math.abs(endpoint.getZ() - startpoint.getZ()) < 32)
//			{
//				path = new ArrayList<PathFindBuffers.GeoNode>();
//				path.add(0, PathFindBuffers.GeoNode.initNode(startpoint, 0, null));
//			}
//			return;
//		}
		if((buff = PathFindBuffers.alloc(64 + Math.max(xdiff, ydiff), isPlayable, startpoint, endpoint, native_endpoint)) != null)
		{
			path = findPath();
			//            if(Config.GEODATA_DEBUG && buff.isPlayer)                
			//            	debugPath((L2Player)obj, buff, path);
			buff.free();
		}
	}

	private PathFindBuffers.GeoNode getBestOpenNode()
	{
		PathFindBuffers.GeoNode bestNodeLink = null;
		PathFindBuffers.GeoNode oldNode = buff.firstNode;
		PathFindBuffers.GeoNode nextNode = buff.firstNode.link;

		while(nextNode != null)
		{
			if(bestNodeLink == null || nextNode.score < bestNodeLink.link.score)
				bestNodeLink = oldNode;
			oldNode = nextNode;
			nextNode = nextNode.link;
		}

		if(bestNodeLink != null)
		{
			PathFindBuffers.GeoNode bestNode = bestNodeLink.link;
			bestNodeLink.link.closed = true;
			bestNodeLink.link = bestNode.link;
			if(bestNode == buff.currentNode)
				buff.currentNode = bestNodeLink;
			return bestNode;
		}

		return null;
	}

	private ArrayList<PathFindBuffers.GeoNode> tracePath(PathFindBuffers.GeoNode f)
	{
		ArrayList<PathFindBuffers.GeoNode> nodes = new ArrayList<PathFindBuffers.GeoNode>();
		do
		{
			nodes.add(0, f);
			f = f.parent;
		} while(f.parent != null);
		return nodes;
	}

	public ArrayList<PathFindBuffers.GeoNode> findPath()
	{
		buff.firstNode = PathFindBuffers.GeoNode.initNode(buff, buff.startpoint.getX() - buff.offsetX, buff.startpoint.getY() - buff.offsetY, buff.startpoint, 0, null);
		PathFindBuffers.GeoNode nextNode = buff.firstNode, finish = null;
		buff.firstNode.closed = true;
		int i = buff.info.maxIterations;
		while(nextNode != null && i-- > 0)
		{
			if((finish = handleNode(nextNode)) != null)
				return tracePath(finish);
			nextNode = getBestOpenNode();
		}

		return null;
	}

	public PathFindBuffers.GeoNode handleNode(PathFindBuffers.GeoNode node)
	{
		PathFindBuffers.GeoNode result = null;
		int clX = node._x;
		int clY = node._y;
		int clZ = node._z;
		byte nswe = engine.NgetNSWE(clX, clY, clZ);
		if((nswe & EAST) == EAST)
		{
			result = getNeighbour(clX + 1, clY, node);
			if(result != null)
				return result;
		}
		if((nswe & WEST) == WEST)
		{
			result = getNeighbour(clX - 1, clY, node);
			if(result != null)
				return result;
		}
		if((nswe & SOUTH) == SOUTH)
		{
			result = getNeighbour(clX, clY + 1, node);
			if(result != null)
				return result;
		}
		if((nswe & NORTH) == NORTH)
			result = getNeighbour(clX, clY - 1, node);
		return result;
	}

	public PathFindBuffers.GeoNode getNeighbour(int x, int y, PathFindBuffers.GeoNode from)
	{
		int nX = x - buff.offsetX, nY = y - buff.offsetY;

		if(nX >= buff.info.MapSize || nX < 0 || nY >= buff.info.MapSize || nY < 0)
			return null;

		boolean isOldNull = PathFindBuffers.GeoNode.isNull(buff.nodes[nX][nY]);

		if(!isOldNull && buff.nodes[nX][nY].closed)
			return null;

		PathFindBuffers.GeoNode n = ((isOldNull) ? PathFindBuffers.GeoNode.initNode(buff, nX, nY, x, y, from._z, GeoConfig.WEIGHT0, from) : buff.tempNode.reuse(buff.nodes[nX][nY], from));

		int height = Math.abs(n._z - from._z);

		if(height > GeoConfig.MAX_Z_DIFF || n._nswe == NSWE_NONE)
			return null;

		double weight = 0.0D;

		if(n._nswe != NSWE_ALL || height > 16)
			weight = GeoConfig.WEIGHT1;

		if(buff.isPlayer || GeoConfig.SIMPLE_PATHFIND_FOR_MOBS)
		{
			getHeightAndNSWE(x + 1, y, n._z);
			if(buff.hNSWE[1] != NSWE_ALL || Math.abs(n._z - buff.hNSWE[0]) > 16)
				weight += GeoConfig.WEIGHT2;
			getHeightAndNSWE(x - 1, y, n._z);
			if(buff.hNSWE[1] != NSWE_ALL || Math.abs(n._z - buff.hNSWE[0]) > 16)
				weight += GeoConfig.WEIGHT2;
			getHeightAndNSWE(x, y + 1, n._z);
			if(buff.hNSWE[1] != NSWE_ALL || Math.abs(n._z - buff.hNSWE[0]) > 16)
				weight += GeoConfig.WEIGHT2;
			getHeightAndNSWE(x, y - 1, n._z);
			if(buff.hNSWE[1] != NSWE_ALL || Math.abs(n._z - buff.hNSWE[0]) > 16)
				weight += GeoConfig.WEIGHT2;
		}

		int diffx = buff.endpoint.getX() - x;
		int diffy = buff.endpoint.getY() - y;
		int dz = Math.abs(buff.endpoint.getZ() - n._z);

		n.moveCost += from.moveCost + weight;
		n.score = n.moveCost + Math.sqrt(diffx * diffx + diffy * diffy + (dz * dz) / 256);
		if(x == buff.endpoint.getX() && y == buff.endpoint.getY() && dz < 64)
			return n;
		if(isOldNull)
		{
			if(buff.currentNode == null)
				buff.firstNode.link = n;
			else
				buff.currentNode.link = n;
			buff.currentNode = n;
		}
		else if(n.moveCost < buff.nodes[nX][nY].moveCost)
			buff.nodes[nX][nY].copy(n);
		return null;
	}

	private void getHeightAndNSWE(int x, int y, short z)
	{
		int nX = x - buff.offsetX;
		int nY = y - buff.offsetY;
		if(nX >= buff.info.MapSize || nX < 0 || nY >= buff.info.MapSize || nY < 0)
		{
			buff.hNSWE[1] = NSWE_NONE;
			return;
		}
		PathFindBuffers.GeoNode n = buff.nodes[nX][nY];

		if(n == null)
			n = PathFindBuffers.GeoNode.initNodeGeo(buff, nX, nY, x, y, z);

		//        buff.hNSWE[0] = n._z;
		//        buff.hNSWE[1] = n._nswe;               
	}

	public ArrayList<PathFindBuffers.GeoNode> getPath()
	{
		return path;
	}

	private static void addDebugItem(int item_id, int item_count, Location loc)
	{
		L2ItemInstance item = ItemTable.getInstance().createItem(item_id, 0, 0, "debugItem");
		if(item_count > 1)
			item.setCount(item_count);
		debugItems.add(item);
		item.dropMe(null, loc);
	}

	protected static void debugPath(L2Player player, PathFindBuffers.PathFindBuffer pf_buff, ArrayList _path)
	{
		if(!player.isGM())
			return;
		double dist = pf_buff.startpoint.clone().geo2world().distance(pf_buff.endpoint.clone().geo2world());

		System.out.println(String.format("Path for %s, distance: %.0f, size: %s", new Object[] {
				player.getName(),
				Double.valueOf(dist),
				_path != null ? Integer.valueOf(_path.size()) : "null" }));

		if(debugItems == null)
		{
			debugItems = new ArrayList<L2ItemInstance>();
		}
		else
		{
			for(L2ItemInstance item : debugItems)
				item.deleteMe();

			debugItems.clear();
		}

		for(int i = 0; i < pf_buff.nodes.length; i++)
		{
			for(int j = 0; j < pf_buff.nodes[i].length; j++)
				if(!PathFindBuffers.GeoNode.isNull(pf_buff.nodes[i][j]) && pf_buff.nodes[i][j] != pf_buff.firstNode && (_path != null || !pf_buff.nodes[i][j].closed))
					addDebugItem(pf_buff.nodes[i][j].closed ? 65 : 57, (int) pf_buff.nodes[i][j].moveCost, (new Location(pf_buff.nodes[i][j]._x, pf_buff.nodes[i][j]._y, pf_buff.nodes[i][j]._z)).geo2world());

		}

		addDebugItem(3433, 1, pf_buff.startpoint.geo2world());
		addDebugItem(3436, 1, pf_buff.endpoint.geo2world());
	}
}
