package l2r.gameserver.geodata;

import java.util.logging.Logger;

import l2r.gameserver.model.L2World;

/**
 * <b>для закрытия двери нужно</b>:
 * <p/>
 * 1). Расставить вдоль двери непроходимости<br>
 * 2). В середине у квадратов поднять высоту на 80
 * <p/>
 * <b>для открытия</b>:<br>
 * 1). Расставить вокруг двери проходимости<br>
 * 2). в середине поставить высоту равною позиции двери
 * <p/>
 * http://www.pics.net.ua/images/304050geodoors.jpg
 * <p/>
 * User: Death
 * Date: 15 груд 2007
 * Time: 12:29:59
 */
public class RemoteGeoEngine extends GeoEngine
{
	private static final Logger log = Logger.getLogger(RemoteGeoEngine.class.getName());

	private static final int geodataSizeX = L2World.WORLD_SIZE_X;
	private static final int geodataSizeY = L2World.WORLD_SIZE_Y;

	private byte[][][][] localGeo = new byte[geodataSizeX][geodataSizeY][][];

	@Override
	protected byte[] getGeoBlockFromGeoCoords(int geoX, int geoY)
	{
		int ix = geoX >> 11;
		int iy = geoY >> 11;

		if(ix < 0 || ix >= geodataSizeX || iy < 0 || iy >= geodataSizeY)
			return null;

		int blockX = getBlock(geoX);
		int blockY = getBlock(geoY);
		int blockIndex = getBlockIndex(blockX, blockY);

		byte[] localBlock = getLocalBlock(ix, iy, blockIndex);
		if(localBlock != null)
			return localBlock;

		byte[][] region = geodata[ix][iy];

		if(region == null)
			return null;

		return region[blockIndex];
	}

	private byte[] getLocalBlock(int ix, int iy, int blockIndex)
	{
		byte[][] region = localGeo[ix][iy];

		if(region == null)
			return null;

		return region[blockIndex];
	}

	/**
	 * Копирует блок с основной геодаты в локальную для дальнейших модификаций.<BR>
	 * Если блок локальной геодаты уже скопирован, то еше раз копироватся не будет.
	 *
	 * @param ix		 регион x
	 * @param iy		 регион y
	 * @param blockIndex индекс блока в регионе
	 */
	@Override
	protected void copyBlock(int ix, int iy, int blockIndex)
	{
		byte[][] region = localGeo[ix][iy];

		if(region == null)
		{
			region = new byte[65536][];
			localGeo[ix][iy] = region;
		}

		byte[] block = region[blockIndex];

		if(block == null)
		{
			byte[][] superRegion = geodata[ix][iy];

			if(superRegion == null || superRegion.length == 0)
				return;

			byte[] superBlock = superRegion[blockIndex];

			if(superBlock == null)
				return;

			byte blockType = superBlock[0];

			// Делаем опирацию с блоком в зависимости от типа блока
			switch(blockType)
			{
				// для дверей даный тип блока не подходит, преобразовываем в COMPLEX
				case BLOCKTYPE_FLAT:
					block = new byte[129];
					block[0] = BLOCKTYPE_COMPLEX;
					for(int i = 1; i < 129; i += 2)
					{
						block[i] = superBlock[1];
						block[i + 1] = superBlock[2];
					}
					region[blockIndex] = block;
					break;
				// комплекс и мультилевел подходят для дверй
				case BLOCKTYPE_COMPLEX:
				case BLOCKTYPE_MULTILEVEL:
					block = new byte[superBlock.length];
					System.arraycopy(superBlock, 0, block, 0, superBlock.length);
					region[blockIndex] = block;
					break;
				default:
					log.severe("GeoEngine: Invalid BlockType");
			}
		}
	}
}