package l2r.gameserver.instancemanager;

import java.sql.ResultSet;
import java.util.HashMap;

import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Zone;
import l2r.gameserver.model.entity.CHSiege;
import l2r.gameserver.model.instances.L2CHSiegeDoorInstance;
import l2r.gameserver.model.instances.L2DoorInstance;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.tables.ClanTable;
import l2r.gameserver.tables.DoorTable;

public class CHSiegeManager
{
	private HashMap<Integer, CHSiege> _chsieges = new HashMap<Integer, CHSiege>();

	public static class MonsterLocation
	{
		private int _id;
		private int _x;
		private int _y;
		private int _z;
		private int _countMin;
		private int _countMax;
		private int _heading;
		private int _locId;

		public MonsterLocation(int id, int x, int y, int z, int heading, int locId, int countMin, int countMax)
		{
			_id = id;
			_x = x;
			_y = y;
			_z = z;
			_heading = heading;
			_countMin = countMin;
			_countMax = countMax;
			_locId = locId;
		}

		public int getId()
		{
			return _id;
		}

		public int getX()
		{
			return _x;
		}

		public int getY()
		{
			return _y;
		}

		public int getZ()
		{
			return _z;
		}

		public int getCountMin()
		{
			return _countMin;
		}

		public int getCountMax()
		{
			return _countMax;
		}

		public int getHeading()
		{
			return _heading;
		}

		public int getLocId()
		{
			return _locId;
		}
	}

	private static CHSiegeManager instance;

	public static CHSiegeManager getInstance()
	{
		if(instance == null)
			instance = new CHSiegeManager();
		return instance;
	}

	public CHSiegeManager()
	{}

	public void spawnDoors(int[] doors, boolean siege)
	{
		for(int element : doors)
		{
			L2DoorInstance door = DoorTable.getInstance().getDoor(element);
			if(siege)
			{
				L2CHSiegeDoorInstance sdoor = new L2CHSiegeDoorInstance(door.getObjectId(), door.getTemplate(), door.getDoorId(), door.getDoorName(), false, true);
				sdoor.setDestroyable(true);
				sdoor.setCurrentHpMp(door.getMaxHp(), door.getMaxMp());
				sdoor.setOpen(false);
				door.decayMe();
				DoorTable.getInstance().removeDoor(door.getDoorId());
				sdoor.spawnMe(door.getLoc());
				DoorTable.getInstance().putDoor(sdoor.getDoorId(), sdoor);
			}
			else
			{
				L2DoorInstance sdoor = new L2DoorInstance(door.getObjectId(), door.getTemplate(), door.getDoorId(), door.getDoorName(), false, true);
				sdoor.setCurrentHpMp(door.getMaxHp(), door.getMaxMp());
				sdoor.setOpen(false);
				door.decayMe();
				DoorTable.getInstance().removeDoor(door.getDoorId());
				sdoor.spawnMe(door.getLoc());
				DoorTable.getInstance().putDoor(sdoor.getDoorId(), sdoor);
			}
		}
	}

	public boolean registerCHSiege(int id, CHSiege s)
	{
		if(_chsieges.containsKey(id))
			_chsieges.remove(id);
		_chsieges.put(id, s);
		loadAttackers(id);
		loadSiegeTime(id);
		if(ZoneManager.getInstance().getZoneByIndex(L2Zone.ZoneType.Siege, id, false) != null)
			ZoneManager.getInstance().getZoneByIndex(L2Zone.ZoneType.Siege, id, false).setActive(false);
		return true;
	}

	public boolean isRegistered(L2Clan clan)
	{
		for(CHSiege siege : _chsieges.values())
			if(siege.isRegistered(clan))
				return true;
		return false;
	}

	public CHSiege getSiege(int id)
	{
		return _chsieges.get(id);
	}

	public CHSiege getSiege(L2NpcInstance npc)
	{
		return _chsieges.get(npc.getSpawn().getSiegeId());
	}

	public CHSiege getSiege(L2Clan clan)
	{
		for(int i : _chsieges.keySet())
		{
			CHSiege siege = CHSiegeManager.getInstance().getSiege(i);
			if(siege.getAttackers().contains(clan))
				return siege;
		}
		return null;
	}

	public void loadAttackers(int id)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT clan_id FROM chs_clans where ch_id=?");
			statement.setInt(1, id);
			rset = statement.executeQuery();
			while(rset.next())
				getSiege(id).addAttacker(ClanTable.getInstance().getClan(rset.getInt("clan_id")), false);
		}
		catch(Exception e)
		{
			System.out.println("Exception: CHSiege:loadAttackers() Id:" + id + " : " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public void loadSiegeTime(int id)
	{
		long starttime = 0;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT starttime FROM ch_sieges where ch_id=?");
			statement.setInt(1, id);
			rset = statement.executeQuery();
			if(rset.next())
				starttime = rset.getLong("starttime");
			DatabaseUtils.closeStatement(statement);
			if(starttime < System.currentTimeMillis() / 1000)
			{
				starttime = 0;
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("DELETE FROM chs_clans where ch_id=?");
				statement.setInt(1, id);
				statement.executeUpdate();
				DatabaseUtils.closeStatement(statement);
				CHSiegeManager.getInstance().getSiege(id).clearAttackers();
				statement = con.prepareStatement("DELETE FROM ch_sieges where ch_id=?");
				statement.setInt(1, id);
				statement.executeUpdate();
				DatabaseUtils.closeStatement(statement);
			}
			if(starttime == 0)
				starttime = getSiege(id).setNewStartTime(false);
		}
		catch(Exception e)
		{
			System.out.println("Exception: CHSiege:loadSiegeTime() Id:" + id + " : " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
		getSiege(id).setRegistrationAvailable(true);
		getSiege(id).setStartTime(starttime);
	}
}
