package l2r.gameserver.instancemanager;

import java.util.HashMap;
import java.util.logging.Logger;

import javolution.util.FastList;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Zone;
import l2r.gameserver.model.L2Zone.ZoneType;
import l2r.gameserver.model.entity.Castle;
import l2r.gameserver.model.entity.siege.SiegeUnit;

public class CastleManager
{
	protected static Logger _log = Logger.getLogger(CastleManager.class.getName());

	private static CastleManager _instance;
	private HashMap<Integer, Castle> _castles;

	public static CastleManager getInstance()
	{
		if(_instance == null)
			_instance = new CastleManager();
		return _instance;
	}

	public CastleManager()
	{
		load();
	}

	private void load()
	{
		FastList<L2Zone> zones = ZoneManager.getInstance().getZoneByType(ZoneType.Castle);
		if(zones.size() == 0)
			_log.info("Not found zones for Castles!!!");
		else
			for(L2Zone zone : zones)
				getCastles().put(zone.getIndex(), new Castle(zone.getIndex()));
		_log.info("Loaded: " + getCastles().size() + " castles.");
	}

	/**
	 * Возвращает замок, соответствующий индексу.
	 */
	public Castle getCastleByIndex(int index)
	{
		for(Castle castle : getCastles().values())
		{
			if(castle != null && castle.getId() == index)
				return castle;
		}
		return null;
	}

	/**
	 * Находит замок по имени. Если такого замка нет - возвращает null.
	 */
	public Castle getCastleByName(String name)
	{
		int index = getCastleIndexByName(name);
		if(index > 0)
			return getCastles().get(index);
		return null;
	}

	/**
	 * Если координаты принадлежат зоне какого-либо замка, возвращает этот замок.
	 * Иначе возвращает null.
	 */
	public Castle getCastleByObject(L2Object activeObject)
	{
		return getCastleByCoord(activeObject.getX(), activeObject.getY());
	}

	/**
	 * Если обьект находится в зоне какого-либо замка, возвращает этот замок.
	 * Иначе возвращает null.
	 */
	public Castle getCastleByCoord(int x, int y)
	{
		int index = getCastleIndexByCoord(x, y);
		if(index > 0)
			return getCastles().get(index);
		return null;
	}

	/**
	 * Если обьект находится в зоне какого-либо замка, возвращает индекс этого замка.
	 * Иначе возвращает -1.
	 */
	public int getCastleIndex(L2Object activeObject)
	{
		return getCastleIndexByCoord(activeObject.getX(), activeObject.getY());
	}

	/**
	 * Если координаты принадлежат зоне какого-либо замка, возвращает индекс этого замка.
	 * Иначе возвращает -1.
	 */
	public int getCastleIndexByCoord(int x, int y)
	{
		SiegeUnit castle;
		for(int i = 1; i <= getCastles().size(); i++)
		{
			castle = getCastles().get(i);
			if(castle != null && castle.checkIfInZone(x, y))
				return i;
		}
		return -1;
	}

	/**
	 * Находит замок по имени, без учета регистра.
	 * Если не найден - возвращает -1.
	 */
	public int getCastleIndexByName(String name)
	{
		SiegeUnit castle;
		for(int i = 1; i <= getCastles().size(); i++)
		{
			castle = getCastles().get(i);
			if(castle != null && castle.getName().equalsIgnoreCase(name.trim()))
				return i;
		}
		return -1;
	}

	/**
	 * Возвращает список, содержащий все замки.
	 */
	public HashMap<Integer, Castle> getCastles()
	{
		if(_castles == null)
			_castles = new HashMap<Integer, Castle>();
		return _castles;
	}
}
