package l2r.gameserver.instancemanager;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.logging.Logger;

import javolution.util.FastList;
import javolution.util.FastMap;
import l2r.Config;
import l2r.gameserver.idfactory.IdFactory;
import l2r.gameserver.model.entity.Castle;
import l2r.gameserver.model.entity.siege.SiegeSpawn;
import l2r.gameserver.model.instances.L2ArtefactInstance;
import l2r.gameserver.model.instances.L2ControlTowerInstance;
import l2r.gameserver.tables.NpcTable;

public class CastleSiegeManager extends SiegeManager
{
	private static Logger _log = Logger.getLogger(CastleSiegeManager.class.getName());

	private static FastMap<Integer, FastList<SiegeSpawn>> _controlTowerSpawnList;
	private static FastMap<Integer, FastList<SiegeSpawn>> _artefactSpawnList;

	public static void load()
	{
		try
		{
			InputStream is = new FileInputStream(new File(Config.SIEGE_CASTLE_CONFIGURATION_FILE));
			Properties siegeSettings = new Properties();
			siegeSettings.load(is);
			is.close();

			// Siege setting
			int controlTowerLosePenalty = Integer.decode(siegeSettings.getProperty("CTLossPenalty", "20000"));
			int defenderRespawnDelay = Integer.decode(siegeSettings.getProperty("DefenderRespawn", "20000"));
			int siegeClanMinLevel = Integer.decode(siegeSettings.getProperty("SiegeClanMinLevel", "4"));
			int siegeLength = Integer.decode(siegeSettings.getProperty("SiegeLength", "120"));

			// Siege spawns settings
			_controlTowerSpawnList = new FastMap<Integer, FastList<SiegeSpawn>>();
			_artefactSpawnList = new FastMap<Integer, FastList<SiegeSpawn>>();

			for(Castle castle : CastleManager.getInstance().getCastles().values())
			{
				FastList<SiegeSpawn> _controlTowersSpawns = new FastList<SiegeSpawn>();

				for(int i = 1; i < 0xFF; i++)
				{
					String _spawnParams = siegeSettings.getProperty(castle.getName() + "ControlTower" + Integer.toString(i), "");

					if(_spawnParams.length() == 0)
						break;

					StringTokenizer st = new StringTokenizer(_spawnParams.trim(), ",");

					try
					{
						int x = Integer.parseInt(st.nextToken());
						int y = Integer.parseInt(st.nextToken());
						int z = Integer.parseInt(st.nextToken());
						int npc_id = Integer.parseInt(st.nextToken());
						int hp = Integer.parseInt(st.nextToken());

						_controlTowersSpawns.add(new SiegeSpawn(castle.getId(), x, y, z, 0, npc_id, hp));
					}
					catch(Exception e)
					{
						_log.warning("Error while loading control tower(s) for " + castle.getName() + " castle.");
					}
				}

				FastList<SiegeSpawn> _artefactSpawns = new FastList<SiegeSpawn>();

				for(int i = 1; i < 0xFF; i++)
				{
					String _spawnParams = siegeSettings.getProperty(castle.getName() + "Artefact" + Integer.toString(i), "");

					if(_spawnParams.length() == 0)
						break;

					StringTokenizer st = new StringTokenizer(_spawnParams.trim(), ",");

					try
					{
						int x = Integer.parseInt(st.nextToken());
						int y = Integer.parseInt(st.nextToken());
						int z = Integer.parseInt(st.nextToken());
						int heading = Integer.parseInt(st.nextToken());
						int npc_id = Integer.parseInt(st.nextToken());

						_artefactSpawns.add(new SiegeSpawn(castle.getId(), x, y, z, heading, npc_id));
					}
					catch(Exception e)
					{
						_log.warning("Error while loading artefact(s) for " + castle.getName() + " castle.");
					}
				}

				_controlTowerSpawnList.put(castle.getId(), _controlTowersSpawns);
				_artefactSpawnList.put(castle.getId(), _artefactSpawns);

				castle.getSiege().setControlTowerLosePenalty(controlTowerLosePenalty);
				castle.getSiege().setDefenderRespawnDelay(defenderRespawnDelay);
				castle.getSiege().setSiegeClanMinLevel(siegeClanMinLevel);
				castle.getSiege().setSiegeLength(siegeLength);

				castle.getSiege().startAutoTask();
				spawnArtifacts(castle);
				spawnControlTowers(castle);

				castle.getSiege().getZone().setActive(false);
			}
		}
		catch(Exception e)
		{
			System.err.println("Error while loading siege data.");
			e.printStackTrace();
		}
	}

	public static FastList<SiegeSpawn> getControlTowerSpawnList(int _castleId)
	{
		if(_controlTowerSpawnList.containsKey(_castleId))
			return _controlTowerSpawnList.get(_castleId);
		return null;
	}

	public static void spawnArtifacts(Castle castle)
	{
		for(SiegeSpawn _sp : _artefactSpawnList.get(castle.getId()))
		{
			L2ArtefactInstance art = new L2ArtefactInstance(IdFactory.getInstance().getNextId(), NpcTable.getTemplate(_sp.getNpcId()));
			art.setCurrentHpMp(art.getMaxHp(), art.getMaxMp());
			art.setHeading(_sp.getLoc().h);
			art.spawnMe(_sp.getLoc().changeZ(50));
			castle.getSiege().addArtifact(art);
		}
	}

	/**
	 * Spawn control tower.
	 * @param Сastle
	 */
	public static void spawnControlTowers(Castle castle)
	{
		for(SiegeSpawn _sp : getControlTowerSpawnList(castle.getId()))
		{
			L2ControlTowerInstance tower = new L2ControlTowerInstance(IdFactory.getInstance().getNextId(), NpcTable.getTemplate(_sp.getNpcId()), castle.getSiege());
			tower.setCurrentHpMp(tower.getMaxHp(), tower.getMaxMp());
			tower.setHeading(_sp.getLoc().h);
			tower.spawnMe(_sp.getLoc());
			castle.getSiege().addControlTower(tower);
		}
	}
}