package l2r.gameserver.instancemanager;

import java.util.HashSet;
import java.util.logging.Logger;

import l2r.gameserver.model.L2Spawn;
import l2r.gameserver.model.entity.SevenSigns;

public class CatacombSpawnManager
{
	private static Logger _log = Logger.getLogger(CatacombSpawnManager.class.getName());

	private static CatacombSpawnManager _instance;
	private static HashSet<L2Spawn> _dawnMobs = new HashSet<L2Spawn>();
	private static HashSet<L2Spawn> _duskMobs = new HashSet<L2Spawn>();

	private static int _currentState = 0; // 0 = Undefined, 1 = Dawn, 2 = Dusk

	public static CatacombSpawnManager getInstance()
	{
		if(_instance == null)
			_instance = new CatacombSpawnManager();

		return _instance;
	}

	public void addDawnMob(L2Spawn spawnDat)
	{
		_dawnMobs.add(spawnDat);
	}

	public void addDuskMob(L2Spawn spawnDat)
	{
		_duskMobs.add(spawnDat);
	}

	public void changeMode(int mode)
	{
		if(_currentState == mode)
			return;

		_currentState = mode;

		switch(mode)
		{
			case 0: // all spawns
				deleteMobs(_dawnMobs);
				deleteMobs(_duskMobs);
				spawnMobs(_duskMobs);
				spawnMobs(_dawnMobs);
				break;
			case 1: // dusk spawns
				deleteMobs(_dawnMobs);
				deleteMobs(_duskMobs);
				spawnMobs(_duskMobs);
				break;
			case 2: // dawn spawns
				deleteMobs(_dawnMobs);
				deleteMobs(_duskMobs);
				spawnMobs(_dawnMobs);
				break;
			default:
				_log.warning("DayNightSpawnManager: Wrong mode sent");
				break;
		}
	}

	public void notifyChangeMode()
	{
		if(SevenSigns.getInstance().getCurrentPeriod() == SevenSigns.PERIOD_SEAL_VALIDATION)
			changeMode(SevenSigns.getInstance().getCabalHighestScore());
		else
			changeMode(0);
	}

	public void cleanUp()
	{
		deleteMobs(_duskMobs);
		deleteMobs(_dawnMobs);

		_duskMobs.clear();
		_dawnMobs.clear();
	}

	public void spawnMobs(HashSet<L2Spawn> mobsSpawnsList)
	{
		for(L2Spawn spawnDat : mobsSpawnsList)
		{
			if(_currentState == 0)
				spawnDat.restoreAmount();
			else
				spawnDat.setAmount(spawnDat.getAmount() * 2);

			spawnDat.init();
		}
	}

	public static void deleteMobs(HashSet<L2Spawn> mobsSpawnsList)
	{
		for(L2Spawn spawnDat : mobsSpawnsList)
			spawnDat.despawnAll();
	}
}