package l2r.gameserver.instancemanager;

import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.entity.Castle;
import l2r.gameserver.model.entity.Fortress;
import l2r.gameserver.model.entity.siege.Siege;
import l2r.gameserver.model.entity.siege.SiegeUnit;
import l2r.gameserver.tables.SkillTable;

public abstract class SiegeManager
{
	public static void addSiegeSkills(L2Player character)
	{
		character.addSkill(SkillTable.getInstance().getInfo(246, 1), false);
		character.addSkill(SkillTable.getInstance().getInfo(247, 1), false);
	}

	public static void removeSiegeSkills(L2Player character)
	{
		character.removeSkill(SkillTable.getInstance().getInfo(246, 1), false);
		character.removeSkill(SkillTable.getInstance().getInfo(247, 1), false);
	}

	public static boolean getCanRide()
	{
		for(Castle castle : CastleManager.getInstance().getCastles().values())
			if(castle != null && (castle.getSiege().isInProgress() || castle.getSiege().getTimeRemaining() <= 7200))
				return false;
		for(Fortress fortress : FortressManager.getInstance().getFortresses().values())
			if(fortress != null && (fortress.getSiege().isInProgress() || fortress.getSiege().getTimeRemaining() <= 7200))
				return false;
		return true;
	}

	public static SiegeUnit getSiegeUnitByObject(L2Object activeObject)
	{
		return getSiegeUnitByCoord(activeObject.getX(), activeObject.getY());
	}

	public static SiegeUnit getSiegeUnitByCoord(int x, int y)
	{
		for(Castle castle : CastleManager.getInstance().getCastles().values())
			if(castle.checkIfInZone(x, y))
				return castle;
		for(Fortress fortress : FortressManager.getInstance().getFortresses().values())
			if(fortress.checkIfInZone(x, y))
				return fortress;
		return null;
	}

	public static Siege getSiege(L2Object activeObject)
	{
		return getSiege(activeObject.getX(), activeObject.getY());
	}

	public static Siege getSiege(int x, int y)
	{
		for(Castle castle : CastleManager.getInstance().getCastles().values())
			if(castle.getSiege().checkIfInZone(x, y))
				return castle.getSiege();
		for(Fortress fortress : FortressManager.getInstance().getFortresses().values())
			if(fortress.getSiege().checkIfInZone(x, y))
				return fortress.getSiege();
		return null;
	}
}