package l2r.gameserver.loginservercon.lspackets;

import java.util.logging.Logger;

import javolution.util.FastList;
import l2r.Config;
import l2r.gameserver.loginservercon.AttLS;
import l2r.gameserver.loginservercon.Attribute;
import l2r.gameserver.loginservercon.gspackets.PlayerInGame;
import l2r.gameserver.loginservercon.gspackets.ServerStatus;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.tables.FakePlayersTable;

public class AuthResponse extends LoginServerBasePacket
{
	private static final Logger log = Logger.getLogger(AuthResponse.class.getName());

	private int _serverId;
	private String _serverName;

	public AuthResponse(byte[] decrypt, AttLS loginServer)
	{
		super(decrypt, loginServer);
	}

	@Override
	public void read()
	{
		_serverId = readC();
		_serverName = readS();
		getLoginServer().setLicenseShown(readC() == 1);

		log.info("Registered on login as Server " + _serverId + " : " + _serverName);

		FastList<Attribute> attributes = FastList.newInstance();

		attributes.add(new Attribute(Attribute.SERVER_LIST_SQUARE_BRACKET, Config.SERVER_LIST_BRACKET ? Attribute.ON : Attribute.OFF));
		attributes.add(new Attribute(Attribute.SERVER_LIST_CLOCK, Config.SERVER_LIST_CLOCK ? Attribute.ON : Attribute.OFF));
		attributes.add(new Attribute(Attribute.TEST_SERVER, Config.SERVER_LIST_TESTSERVER ? Attribute.ON : Attribute.OFF));
		attributes.add(new Attribute(Attribute.SERVER_LIST_STATUS, Config.SERVER_GMONLY ? Attribute.STATUS_GM_ONLY : Attribute.STATUS_AUTO));

		sendPacket(new ServerStatus(attributes));

		if(L2World.getAllPlayers().size() > 0)
		{
			FastList<String> playerList = FastList.newInstance();
			for(L2Player player : L2World.getAllPlayers())
			{
				if(player.isInOfflineMode())
					continue;
				playerList.add(player.getAccountName());
				getLoginServer().getCon().addAccountInGame(player.getNetConnection());
			}
			for(String name : FakePlayersTable.getActiveFakePlayers())
				playerList.add(name + name.hashCode());
			PlayerInGame pig = new PlayerInGame(playerList);
			sendPacket(pig);
		}
	}

	/**
	 * @return Returns the serverId.
	 */
	public int getServerId()
	{
		return _serverId;
	}

	/**
	 * @return Returns the serverName.
	 */
	public String getServerName()
	{
		return _serverName;
	}
}
