package l2r.gameserver.model;

import java.security.MessageDigest;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import l2r.Config;
import l2r.gameserver.ThreadPoolManager;
import l2r.util.Rnd;
import l2r.util.Strings;
import l2r.util.Util;

public class BypassEngine
{
	private static final Logger _log = Logger.getLogger(BypassEngine.class.getName());

	static
	{
		// Периодическая чистка от хлама
		ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new PurgeTask(), Config.PURGE_BYPASS_TASK_FREQUENCY * 1000, Config.PURGE_BYPASS_TASK_FREQUENCY * 1000);
		_log.info("Purge task for BypassEngine started for every " + Util.formatTime(Config.PURGE_BYPASS_TASK_FREQUENCY) + ".");
	}

	private static class PurgeTask implements Runnable
	{
		public void run()
		{
			for(String s : bypassMap.keySet())
			{
				BypassStorage bs = bypassMap.get(s);

				if(bs == null || bs.getPlayer() == null)
					bypassMap.remove(s);
			}
		}
	}

	//	private static final int MAX_RECORDS = Config.MAXIMUM_ONLINE_USERS*10;
	private static final int HASH_LENGTH = 16;

	private static final ConcurrentHashMap<String, BypassStorage> bypassMap = new ConcurrentHashMap<String, BypassStorage>();

	private static final Pattern p = Pattern.compile("\"(bypass +-h +)(.+?)\"");

	public static String encode(L2Player player, String html)
	{
		Matcher m = p.matcher(html);
		StringBuffer sb = new StringBuffer();
		while(m.find())
		{
			// System.out.print("Found bypass '");
			String bypass = m.group(2);
			// System.out.println(bypass+"'");

			try
			{
				String code = bypass;
				String params = "";
				boolean use_params = false;

				int pos = bypass.indexOf(" $");
				if(pos >= 0)
				{
					code = bypass.substring(0, pos);
					params = bypass.substring(pos).replace("$", "\\$");
					// _log.info("Found params '"+params+"' with code '"+code+"' in bypass '"+bypass+"'");
					use_params = true;
				}

				String hash = Strings.bytesToString(MessageDigest.getInstance("MD5").digest(("" + Rnd.get()).getBytes("UTF-8"))).substring(0, HASH_LENGTH);
				// _log.info("Created hash '"+hash+"'");
				bypassMap.put(hash, new BypassStorage(player, code, use_params));
				m.appendReplacement(sb, "\"bypass -h " + hash + params + "\"");
				// _log.info("Hash appened.");
			}
			catch(Exception e)
			{
				_log.warning("Exception " + e);
				e.printStackTrace();
			}
		}
		m.appendTail(sb);

		// _log.info("Encript '"+html+"' to '"+sb.toString()+"'");

		return sb.toString();
	}

	public static String decode(L2Player player, String bypass)
	{
		if(bypass.equals("") || bypass.length() == 0)
			return "";

		if(Strings.matches(bypass, "^(bbs_|_mrsl|_clbbs|_friend|_mm|_mail|_bbs|_diary|friendlist|friendmail|manor_menu_select|_match).*", Pattern.DOTALL))
			return bypass;

		if(bypass.length() < HASH_LENGTH)
			return errRet(player, "Too short bypass: '" + bypass + "'", 1);

		String hash = bypass.substring(0, HASH_LENGTH);
		// _log.warning("BypassEngine: '" + hash + "'");
		BypassStorage bs = bypassMap.get(hash);
		if(bs == null)
		{
			if(hash.matches("[0-9a-f]{" + HASH_LENGTH + "}"))
				return "";
			return errRet(player, "Not found hash '" + hash + "' in bypass '" + bypass + "'", 1);
		}

		if(!bs.isUseParams() && !hash.equals(bypass))
			return errRet(player, "Try use bypass with wrong parameters '" + bypass + "' with command '" + bs.getCode() + "'", 1);
		if(bs.getPlayer() != player)
			return errRet(player, "Wrong " + (player != null ? "" : "null player") + " try execute '" + bs.getCode() + "' over stored bypass '" + bypass + "'", 1);

		String code = bs.getCode();

		if(bs.isUseParams())
			code += bypass.substring(HASH_LENGTH);

		bypassMap.remove(hash);

		clean(player);

		// _log.info("Decript bypass '"+bypass+"' with hash '"+hash+"' to '"+code+"'");

		return code;
	}

	private static String errRet(L2Player player, String errMsg, Integer isBug)
	{
		if(player != null && !player.isGM())
		{
			_log.warning("BypassEngine: " + player + " " + errMsg);
			Util.handleIllegalPlayerAction(player, "BypassEngine", errMsg, isBug);
		}
		else
			_log.warning("Hm... Illegal action '" + errMsg + "' for null player or GM");
		return "may_be_hack_detected";
	}

	private static void clean(L2Player caller)
	{
		long now = System.currentTimeMillis();
		for(Object element : bypassMap.keySet())
		{
			String hash = (String) element;
			BypassStorage bs = bypassMap.get(hash);
			if(bs.getTime() <= now || bs.getPlayer() == caller)
				bypassMap.remove(hash);
		}
	}
}