package l2r.gameserver.model;

import javolution.util.FastList;
import l2r.Config;
import l2r.gameserver.model.base.Experience;
import l2r.gameserver.model.base.ItemToDrop;
import l2r.gameserver.model.instances.L2ItemInstance.ItemClass;
import l2r.util.Rnd;

public class L2DropGroup
{
	private short _id;
	private int _chance;
	private boolean _isAdena = false;
	private boolean _fixedQty = false;
	private FastList<L2DropData> _items = new FastList<L2DropData>();

	public L2DropGroup(short id, int chance)
	{
		_id = id;
		_chance = chance;
	}

	public short getId()
	{
		return _id;
	}

	public boolean fixedQty()
	{
		return _fixedQty;
	}

	public void addDropItem(L2DropData item)
	{
		if(item.getItem().isAdena())
			_isAdena = true;
		if(item.getItem().getItemClass() == ItemClass.SPELLBOOKS || item.getItem().isEquipment())
			_fixedQty = true;
		_items.add(item);
	}

	public FastList<L2DropData> getDropItems(boolean copy)
	{
		if(!copy)
			return _items;
		FastList<L2DropData> temp = new FastList<L2DropData>();
		temp.addAll(_items);
		for(L2DropData i : _items)
			temp.add(i);
		return temp;
	}

	/**
	 * Эта функция выбирает одну вещь из группы
	 * Используется в основном механизме рассчета дропа
	 */
	public ItemToDrop roll(int diff, boolean isSpoil, boolean isRaidBoss, L2Player player)
	{
		float rate, adenarate;
		if(!isRaidBoss)
		{
			rate = isSpoil ? Config.RATE_DROP_SPOIL * player.getRateSpoil() : Config.RATE_DROP_ITEMS * player.getRateItems();
			adenarate = Config.RATE_DROP_ADENA * player.getRateAdena();
		}
		else
		{
			adenarate = Config.RATE_DROP_RAIDBOSS * player.getRateAdena();
			rate = Config.RATE_DROP_RAIDBOSS * player.getRateItems();
		}

		// Считаем шанс группы
		float calcChance = _chance * (_isAdena ? 1 : rate);

		// Поправка на глублко синих мобов
		if(Config.DEEPBLUE_DROP_RULES && diff > 1)
		{
			if(isRaidBoss) // С рейдов на 9+ левелов ниже вообще ничего не падает
				return null;
			calcChance *= Experience.penaltyModifier(diff - 1, 9);
			adenarate *= Experience.penaltyModifier(diff - 1, 9); // не совсем понятно - для адены штраф применяется дважды?
		}

		int dropmult = 1;
		// Если шанс оказался больше 100%
		if(calcChance > L2Drop.MAX_CHANCE && !_fixedQty)
			if(calcChance % L2Drop.MAX_CHANCE == 0) // если кратен 100% то тупо умножаем количество
				dropmult = (int) (calcChance / L2Drop.MAX_CHANCE);
			else
			{ // иначе балансируем
				dropmult = (int) Math.ceil(calcChance / L2Drop.MAX_CHANCE); // множитель равен шанс / 100% округление вверх
				calcChance = calcChance / dropmult; // шанс равен шанс / множитель
				// в результате получаем увеличение количества и уменьшение шанса, при этом шанс не падает ниже 50%
			}

		if(Rnd.get(1, L2Drop.MAX_CHANCE) > calcChance)
			return null;

		// перебираем все вещи в группе и првоеряем шанс
		int chance = Rnd.get(1, L2Drop.MAX_CHANCE);
		for(L2DropData i : _items)
		{
			if(chance > i.getChance())
			{
				chance -= i.getChance();
				continue;
			}

			ItemToDrop t = new ItemToDrop(i.getItemId());
			// С вещью определились, считаем количество
			// если это адена то умножаем на рейт адены
			// если нет - на модификатор
			float mult = _isAdena ? adenarate : dropmult;

			if(i.getMinDrop() >= i.getMaxDrop())
				t.count = (int) (i.getMinDrop() * mult);
			else
				t.count = Rnd.get((int) (i.getMinDrop() * mult), (int) (i.getMaxDrop() * mult));

			t.isAdena = _isAdena;
			return t;
		}
		// can it be?
		return null;
	}

	public int getChance()
	{
		return _chance;
	}

	public boolean isAdena()
	{
		return _isAdena;
	}
}