package l2r.gameserver.model;

import static l2r.gameserver.model.L2Zone.ZoneType.Siege;
import static l2r.gameserver.model.L2Zone.ZoneType.peace_zone;
import javolution.util.FastList;
import l2r.Config;
import l2r.extensions.listeners.MethodCollection;
import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.GameTimeController;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.ai.CtrlEvent;
import l2r.gameserver.ai.CtrlIntention;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.instancemanager.SiegeManager;
import l2r.gameserver.model.L2Skill.SkillType;
import l2r.gameserver.model.L2Zone.ZoneType;
import l2r.gameserver.model.entity.Duel;
import l2r.gameserver.model.entity.Olympiad;
import l2r.gameserver.model.entity.siege.Siege;
import l2r.gameserver.model.instances.L2ArtefactInstance;
import l2r.gameserver.model.instances.L2DoorInstance;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.instances.L2SiegeGuardInstance;
import l2r.gameserver.serverpackets.Attack;
import l2r.gameserver.skills.Stats;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.templates.L2CharTemplate;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.L2Weapon.WeaponType;
import l2r.util.Rnd;

/**
 * This class represents all Playable characters in the world.
 *
 * L2PlayableInstance:
 *  * L2Player
 *  * L2Summon
 *
 */
@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public abstract class L2Playable extends L2Character
{
	/**
	 * this если Player или owner если Summon
	 */
	public L2Player _owner;
	private boolean _isSilentMoving = false;

	private int _checkAggroTimestamp;

	public L2Playable(int objectId, L2CharTemplate template, L2Player owner)
	{
		super(objectId, template);
		if(owner == null)
			_owner = (L2Player) this;
		else
			_owner = owner;
	}

	@Override
	public boolean isAttackable()
	{
		return true;
	}

	public abstract Inventory getInventory();

	/**
	 * Проверяет, выставлять ли PvP флаг для игрока.<BR><BR>
	 */
	@Override
	public boolean checkPvP(final L2Character target, L2Skill skill)
	{
		if(target == null || _owner == null || target == this || target == _owner || target == _owner.getPet() || _owner.getKarma() > 0)
			return false;

		if(skill != null && skill.altUse())
			return false;

		// Проверка на дуэли... Мэмбэры одной дуэли не флагаются
		if(getDuel() != null && getDuel() == target.getDuel())
			return false;

		if(isInZone(peace_zone) || target.isInZone(peace_zone) || isInZoneBattle() || target.isInZoneBattle())
			return false;
		if(isInZone(Siege) && target.isInZone(Siege))
			return false;
		if(skill == null || skill.isOffensive())
		{
			if(target.getKarma() > 0)
				return false;
			else if(target instanceof L2Playable)
				return true;
		}
		else if(target.getPvpFlag() > 0 || target.getKarma() > 0 || target.isMonster)
			return true;

		return false;
	}

	/**
	 * Проверяет, можно ли атаковать цель (для физ атак)
	 */
	public boolean checkAttack(L2Character target)
	{
		if(target == null || target.isDead())
		{
			getOwner().sendPacket(Msg.INCORRECT_TARGET);
			return false;
		}

		if(!isInRange(target, 6000))
		{
			getOwner().sendPacket(Msg.YOUR_TARGET_IS_OUT_OF_RANGE);
			return false;
		}

		if(target instanceof L2DoorInstance && !((L2DoorInstance) target).isAttackable(this))
		{
			getOwner().sendPacket(Msg.INCORRECT_TARGET);
			return false;
		}

		if(getOwner().inObserverMode())
		{
			getOwner().sendMessage(new CustomMessage("ru.l2f.gameserver.model.L2Playable.OutOfControl.ObserverNoAttack", getOwner()));
			return false;
		}

		if(!GeoClient.getInstance().canSeeTarget(this, target) || getReflection() != target.getReflection())
		{
			getOwner().sendPacket(Msg.CANNOT_SEE_TARGET);
			return false;
		}

		if(target instanceof L2Playable)
		{
			// Нельзя атаковать того, кто находится на арене, если ты сам не на арене
			if(isInZoneBattle() != target.isInZoneBattle())
			{
				getOwner().sendPacket(Msg.INCORRECT_TARGET);
				return false;
			}

			//В режиме олимп обсерва - нельзя атаковать
			if(getOwner().inObserverMode() || (target instanceof L2Player && target.getOwner().inObserverMode()))
				return false;

			//Если сам находишся на олимпе - то атаковать можно только того кто с тобой на олимпе
			if(getOwner().isInOlympiadMode())
			{
				if(target.getPlayer() == null)
					return false;
				if(!target.getPlayer().isInOlympiadMode())
					return false;
				if(getOwner().getOlympiadGameId() != target.getPlayer().getOlympiadGameId())
					return false;
			}
			// Если цель либо атакующий находится в мирной зоне - атаковать нельзя
			if((isInZonePeace() || target.isInZonePeace()) && !getOwner().getPlayerAccess().PeaceAttack)
				if(!Config.ALT_GAME_KARMAPLAYERCANBEKILLEDINPEACEZONE || getKarma() <= 0 && target.getKarma() <= 0)
				{
					getOwner().sendPacket(Msg.YOU_CANNOT_ATTACK_THE_TARGET_IN_THE_PEACE_ZONE);
					return false;
				}

		}

		return true;
	}

	@Override
	public void doAttack(L2Character target)
	{
		if(isAttackingNow())
		{
			_owner.sendActionFailed();
			return;
		}

		if(!checkAttack(target))
		{
			getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE, null, null);
			_owner.sendActionFailed();
			return;
		}

		// Прерывать дуэли если цель не дуэлянт
		if(getDuel() != null && target.getDuel() != getDuel())
			getDuel().setDuelState(_owner, Duel.DuelState.Interrupted);

		// Get the active weapon item corresponding to the active weapon instance (always equipped in the right hand)
		L2Weapon weaponItem = getActiveWeaponItem();

		if(weaponItem != null && (weaponItem.getItemType() == WeaponType.BOW))
		{
			double bowMpConsume = weaponItem.getMpConsume();

			// cheap shot SA
			double chance = calcStat(Stats.MP_USE_BOW_CHANCE, 0., null, null);
			if(chance > 0 && chance > Rnd.get())
				bowMpConsume = calcStat(Stats.MP_USE_BOW, bowMpConsume, null, null);

			if(_currentMp < bowMpConsume)
			{
				getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE, null, null);
				_owner.sendPacket(Msg.NOT_ENOUGH_MP);
				_owner.sendActionFailed();
				return;
			}

			reduceCurrentMp(bowMpConsume, null);

			if(!_owner.checkAndEquipArrows())
			{
				getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE, null, null);
				_owner.sendPacket(Msg.NOT_ENOUGH_ARROWS);
				_owner.sendActionFailed();
				return;
			}
		}

		// Verify if SoulShot are charged
		boolean wasSSCharged = getChargedSoulShot();

		fireMethodInvoked(MethodCollection.onStartAttack, new Object[] { target });

		// Get the Attack Speed of the L2Character (delay (in milliseconds) before next attack)
		int sAtk = Math.max(calculateAttackSpeed(), 200) - 50;

		// Get the Attack Reuse Delay of the L2Weapon
		int reuse = (int) (weaponItem != null ? weaponItem.getAttackReuseDelay() * getReuseModifier(target) * 26 / Math.sqrt(getPAtkSpd()) : 0) - 50;
		if(reuse > 0)
			reuse = Math.max(reuse, sAtk);

		_attackEndTime = sAtk + System.currentTimeMillis();
		_attackReuseEndTime = reuse + System.currentTimeMillis();

		// Create a Server->Client packet Attack
		Attack attack = new Attack(this, wasSSCharged);

		boolean hitted;

		_isAttackAborted = false;

		setHeading(target, true);

		// Select the type of attack to start
		if(weaponItem == null)
			hitted = doAttackHitSimple(attack, target, weaponItem, 1., isSummon || isPet, sAtk);
		else
			switch(weaponItem.getItemType())
			{
				case BOW:
					hitted = doAttackHitByBow(attack, target, sAtk, reuse, weaponItem);
					break;
				case POLE:
					if(calcStat(Stats.POLE_ATTACK_ANGLE, 0, null, null) == -1)
						hitted = doAttackHitSimple(attack, target, weaponItem, 1., true, sAtk);
					else
						hitted = doAttackHitByPole(attack, weaponItem, sAtk);
					break;
				case DUAL:
				case DUALFIST:
					hitted = doAttackHitByDual(attack, target, weaponItem, sAtk);
					break;
				default:
					hitted = doAttackHitSimple(attack, target, weaponItem, 1., true, sAtk);
			}

		if(hitted)
			// if hitted by a cursed weapon, Cp is reduced to 0
			// if a cursed weapon is hitted by a Hero, Cp is reduced to 0
			if(isPlayer && target.isPlayer && !target.isInvul())
				if(isCursedWeaponEquipped() || isHero() && target.isCursedWeaponEquipped())
					target.setCurrentCp(0);

		if(attack.hasHits())
			broadcastPacket(attack);

		// Для кайтинга с луком
		if(reuse > 0)
			ThreadPoolManager.getInstance().scheduleAi(new NotifyAITask(CtrlEvent.EVT_READY_TO_ACT), sAtk + 50, isPlayer || isPet || isSummon);
		ThreadPoolManager.getInstance().scheduleAi(new NotifyAITask(CtrlEvent.EVT_READY_TO_ACT), Math.max(reuse, sAtk) + 50, isPlayer || isPet || isSummon);
	}

	@Override
	protected void onHitTimer(L2Character target, int damage, boolean crit, boolean miss, boolean soulshot, boolean shld, boolean unchargeSS)
	{
		if(target instanceof L2Playable && isInZoneBattle() != target.isInZoneBattle())
		{
			if(_owner != null)
			{
				_owner.sendPacket(Msg.INCORRECT_TARGET);
				_owner.sendActionFailed();
			}
			return;
		}

		super.onHitTimer(target, damage, crit, miss, soulshot, shld, unchargeSS);
	}

	@Override
	public boolean isAutoAttackable(final L2Character attacker)
	{
		if(_owner == null || attacker == this || attacker == _owner || isDead())
			return false;

		if(_owner.isInOlympiadMode() && !_owner.isOlympiadCompStart())
			return false;

		// Автоатака на дуэлях, только враг и только если он еше не проиграл.
		if(getDuel() != null && attacker.getDuel() == getDuel())
		{
			//Тут не может быть ClassCastException у attacker, т.к. L2Character.getDuel() всегда возвращает null
			L2Player enemy = attacker.isPlayer ? (L2Player) attacker : ((L2Summon) attacker).getOwner();

			if(enemy.getDuel().getTeamForPlayer(enemy) != getDuel().getTeamForPlayer(_owner) && getDuel().getDuelState(_owner) == Duel.DuelState.Fighting && getDuel().getDuelState(enemy) == Duel.DuelState.Fighting)
				return true;
		}

		// TODO: check for friendly mobs
		if(attacker.isMonster)
			return true;
		final L2Player pcAttacker = attacker.getPlayer();
		final L2Clan clan1 = _owner.getClan();
		if(pcAttacker != null)
		{
			if(_owner.getParty() != null && _owner.getParty().getPartyMembers().contains(pcAttacker))
				return false;
			// Если цель либо атакующий находится в мирной зоне - атаковать нельзя
			if((isInZonePeace() || pcAttacker.isInZonePeace()) && !_owner.getPlayerAccess().PeaceAttack)
				if(!Config.ALT_GAME_KARMAPLAYERCANBEKILLEDINPEACEZONE || getKarma() <= 0 && pcAttacker.getKarma() <= 0)
					return false;
			if(isInZoneBattle() && attacker.isInZoneBattle())
				return true;
			final L2Clan clan2 = pcAttacker.getClan();
			if(clan1 != null && clan2 != null)
			{
				if(clan1.getClanId() == clan2.getClanId())
					return false;
				if(pcAttacker.atMutualWarWith(_owner))
					return true;
			}
			if(isInZone(ZoneType.Siege) && attacker.isInZone(ZoneType.Siege))
			{
				if(clan1 == null || clan2 == null)
					return true;
				if(!clan1.isDefender() && !clan1.isAttacker())
					return true;
				if(!clan2.isDefender() && !clan2.isAttacker())
					return true;
				if(clan1.isDefender() && clan2.isAttacker() || clan2.isDefender() && clan1.isAttacker())
					return true;
			}
			if(_owner.getKarma() > 0 || _owner.getPvpFlag() != 0)
				return true;
		}
		else if(attacker instanceof L2SiegeGuardInstance)
			if(clan1 != null)
			{
				Siege siege = SiegeManager.getSiege(this);
				return siege != null && siege.checkIsAttacker(clan1);
			}
		return false;
	}

	@Override
	public int getKarma()
	{
		if(_owner == null)
			return 0;
		return _owner.getKarma();
	}

	@Override
	public void callSkill(L2Skill skill, FastList<L2Character> targets)
	{
		FastList<L2Character> toRemove = new FastList<L2Character>();

		if(!skill.altUse() && !skill.getSkillType().equals(SkillType.BEAST_FEED))
			for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
			{
				if(n == null)
					break;
				L2Character target = n.getValue();

				if(target.isInvul() && skill.isOffensive() && !(target instanceof L2ArtefactInstance))
					toRemove.add(target);

				if(!skill.isOffensive())
				{
					if(target instanceof L2Playable)
					{
						int aggro = skill.getEffectPoint() != 0 ? skill.getEffectPoint() : Math.max(1, (int) skill.getPower());
						for(L2NpcInstance monster : getHateList().keySet())
							if(monster != null && !monster.isDead())
								if(monster.isRaid && _owner.getLevel() > monster.getLevel() + Config.RAID_MAX_LEVEL_DIFF)
								{
									final L2Skill revengeSkill = SkillTable.getInstance().getInfo(4515, 1);
									revengeSkill.getEffects(target, _owner, false);
									if(this instanceof L2Summon)
										revengeSkill.getEffects(target, this, false);
								}
								else if(monster.hasAI())
								{
									monster.getAI().notifyEvent(CtrlEvent.EVT_SEE_SPELL, skill, this);
									monster.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, this, aggro);
								}
					}
				}
				else if(target instanceof L2NpcInstance)
					// mobs will hate on debuff
					if(target.isRaid && _owner.getLevel() > target.getLevel() + Config.RAID_MAX_LEVEL_DIFF)
					{
						L2Skill revengeSkill = SkillTable.getInstance().getInfo(4515, 1);
						revengeSkill.getEffects(target, _owner, false);
						if(this instanceof L2Summon)
							revengeSkill.getEffects(target, this, false);
					}
					else if(target.hasAI())
					{
						target.getAI().notifyEvent(CtrlEvent.EVT_SEE_SPELL, skill, this);
						if(!skill.isAI())
						{
							target.getAI().notifyEvent(CtrlEvent.EVT_ATTACKED, this, 0);
							target.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, this, skill.getEffectPoint() != 0 ? skill.getEffectPoint() : 1);
						}
					}
				// Check for PvP Flagging / Drawing Aggro
				if(checkPvP(target, skill))
					startPvPFlag(target);
			}

		for(L2Character cha : toRemove)
			targets.remove(cha);

		super.callSkill(skill, targets);
	}

	@Override
	public void teleToLocation(int x, int y, int z)
	{
		super.teleToLocation(x, y, z);
		clearHateList();
	}

	@Override
	public void updatePosition(int currentGameTick)
	{
		super.updatePosition(currentGameTick);

		if((currentGameTick - _checkAggroTimestamp) * GameTimeController.MILLIS_IN_TICK > 1000)
		{
			_checkAggroTimestamp = currentGameTick;

			if(_owner == null)
				return;
			if(_owner.getNonAggroTime() > System.currentTimeMillis())
				return;
			if(isAlikeDead())
				return;
			if(isInvul() || !isVisible())
				return;
			if(getCurrentRegion() == null)
				return;
			if(getAI().getIntention() == CtrlIntention.AI_INTENTION_FOLLOW && !isPlayer)
				return;

			for(L2WorldRegion region : getCurrentRegion().getNeighbors())
				if(region != null && region.getObjectsSize() > 0)
					for(L2NpcInstance obj : region.getNpcsList(getReflection()))
						if(obj != null && obj.hasAI())
							obj.getAI().checkAggression(this);
		}
	}

	@Override
	public L2Player getOwner()
	{
		return _owner;
	}

	/**
	 * Set the Silent Moving mode Flag.<BR><BR>
	 */
	public void setSilentMoving(final boolean flag)
	{
		_isSilentMoving = flag;
	}

	/**
	 * @return True if the Silent Moving mode is active.<BR><BR>
	 */
	public boolean isSilentMoving()
	{
		return _isSilentMoving;
	}
}
