package l2r.gameserver.model;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javolution.util.FastList;
import l2r.Config;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.instancemanager.SiegeManager;
import l2r.gameserver.model.L2Zone.ZoneType;
import l2r.gameserver.model.base.ClassId;
import l2r.gameserver.model.entity.siege.Siege;
import l2r.gameserver.model.instances.L2ArtefactInstance;
import l2r.gameserver.model.instances.L2ChestInstance;
import l2r.gameserver.model.instances.L2ControlTowerInstance;
import l2r.gameserver.model.instances.L2DecoyInstance;
import l2r.gameserver.model.instances.L2DoorInstance;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.instances.L2PetInstance;
import l2r.gameserver.model.instances.L2StaticObjectInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.serverpackets.FlyToLocation.FlyType;
import l2r.gameserver.skills.Env;
import l2r.gameserver.skills.Formulas;
import l2r.gameserver.skills.Stats;
import l2r.gameserver.skills.conditions.Condition;
import l2r.gameserver.skills.effects.EffectTemplate;
import l2r.gameserver.skills.funcs.Func;
import l2r.gameserver.skills.funcs.FuncTemplate;
import l2r.gameserver.skills.skillclasses.AIeffects;
import l2r.gameserver.skills.skillclasses.Aggression;
import l2r.gameserver.skills.skillclasses.Balance;
import l2r.gameserver.skills.skillclasses.BeastFeed;
import l2r.gameserver.skills.skillclasses.CPDam;
import l2r.gameserver.skills.skillclasses.Call;
import l2r.gameserver.skills.skillclasses.Cancel;
import l2r.gameserver.skills.skillclasses.Charge;
import l2r.gameserver.skills.skillclasses.ChargeDmg;
import l2r.gameserver.skills.skillclasses.CombatPointHeal;
import l2r.gameserver.skills.skillclasses.Continuous;
import l2r.gameserver.skills.skillclasses.Craft;
import l2r.gameserver.skills.skillclasses.DeathPenalty;
import l2r.gameserver.skills.skillclasses.Default;
import l2r.gameserver.skills.skillclasses.DeleteHate;
import l2r.gameserver.skills.skillclasses.DeleteHateOfMe;
import l2r.gameserver.skills.skillclasses.DestroySummon;
import l2r.gameserver.skills.skillclasses.Disablers;
import l2r.gameserver.skills.skillclasses.Drain;
import l2r.gameserver.skills.skillclasses.DrainSoul;
import l2r.gameserver.skills.skillclasses.Effect;
import l2r.gameserver.skills.skillclasses.Elemental;
import l2r.gameserver.skills.skillclasses.FatalBlow;
import l2r.gameserver.skills.skillclasses.Fishing;
import l2r.gameserver.skills.skillclasses.Harvesting;
import l2r.gameserver.skills.skillclasses.Heal;
import l2r.gameserver.skills.skillclasses.HealPercent;
import l2r.gameserver.skills.skillclasses.LethalShot;
import l2r.gameserver.skills.skillclasses.MDam;
import l2r.gameserver.skills.skillclasses.ManaDam;
import l2r.gameserver.skills.skillclasses.ManaHeal;
import l2r.gameserver.skills.skillclasses.ManaHealPercent;
import l2r.gameserver.skills.skillclasses.NegateEffects;
import l2r.gameserver.skills.skillclasses.NegateStats;
import l2r.gameserver.skills.skillclasses.PDam;
import l2r.gameserver.skills.skillclasses.Recall;
import l2r.gameserver.skills.skillclasses.ReelingPumping;
import l2r.gameserver.skills.skillclasses.Resurrect;
import l2r.gameserver.skills.skillclasses.Ride;
import l2r.gameserver.skills.skillclasses.SPHeal;
import l2r.gameserver.skills.skillclasses.Seed;
import l2r.gameserver.skills.skillclasses.ShiftAggression;
import l2r.gameserver.skills.skillclasses.SiegeFlag;
import l2r.gameserver.skills.skillclasses.Sowing;
import l2r.gameserver.skills.skillclasses.Spoil;
import l2r.gameserver.skills.skillclasses.Summon;
import l2r.gameserver.skills.skillclasses.SummonItem;
import l2r.gameserver.skills.skillclasses.Sweep;
import l2r.gameserver.skills.skillclasses.TakeCastle;
import l2r.gameserver.skills.skillclasses.TakeFortress;
import l2r.gameserver.skills.skillclasses.TeleportNpc;
import l2r.gameserver.skills.skillclasses.Toggle;
import l2r.gameserver.skills.skillclasses.Unlock;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.templates.StatsSet;
import l2r.gameserver.templates.L2Weapon.WeaponType;

public abstract class L2Skill
{
	protected static Logger _log = Logger.getLogger(L2Skill.class.getName());

	public static final Integer SKILL_CUBIC_MASTERY = 143;
	public static final Integer SKILL_CRAFTING = 172;
	public static final Integer SKILL_CRYSTALLIZE = 248;
	public static final Integer SKILL_TRANSFER_PAIN = 1262;
	public static final Integer WEAPON_MAGIC_MASTERY1 = 249;
	public static final Integer WEAPON_MAGIC_MASTERY2 = 250;
	public static final Integer SKILL_FISHING_MASTERY = 1315;
	public static final Integer POLEARM_MASTERY = 216;
	public static final Integer BLINDING_BLOW = 321;
	public static final int BLUFF_SKILL_ID = 358;

	//save vs
	public final static int SAVEVS_INT = 1;
	public final static int SAVEVS_WIT = 2;
	public final static int SAVEVS_MEN = 3;
	public final static int SAVEVS_CON = 4;
	public final static int SAVEVS_DEX = 5;
	public final static int SAVEVS_STR = 6;

	private static final Func[] _emptyFunctionSet = new Func[0];

	// these two build the primary key
	protected final Integer _id;
	protected final Short _level;

	/** Identifier for a skill that client can't display */
	private final Integer _displayId;
	private Short _displayLevel;

	// not needed, just for easier debug
	private final String _name;

	private final SkillOpType _operateType;
	private final boolean _magic;
	private final int _mpConsume1;
	private final int _mpConsume2;
	private final int _hpConsume;
	protected final int[] _itemConsume;
	protected final int[] _itemConsumeId;
	protected final boolean _isItemHandler;
	private final boolean _isCommon;
	private final boolean _isSaveable;
	private final boolean _isValidateable;
	private final int _castRange;

	private final int _matak;
	private final boolean _useSS;

	protected final int _reuseGroupId;
	static final HashMap<Integer, ArrayList<L2Skill>> _reuseGroups = new HashMap<Integer, ArrayList<L2Skill>>();

    public String getIcon()
	{
		return null;
	}


    public static class AddedSkill
	{
		public int id;
		public int level;

		public AddedSkill(int id, int level)
		{
			this.id = id;
			this.level = level;
		}

		public L2Skill getSkill()
		{
			return SkillTable.getInstance().getInfo(id, level);
		}
	}

	private AddedSkill[] _addedSkills;

	// all times in milliseconds
	private final int _hitTime;
	private final int _coolTime;
	private final int _skillInterruptTime;
	private final long _reuseDelay;

	protected final SkillType _skillType;
	protected final SkillTargetType _targetType;
	protected final double _power;
	protected final int _effectPoint;
	protected final int _skillRadius;

	private final boolean _undeadOnly;
	private final boolean _corpse;
	private final boolean _altUse;
	private final Element _element;
	private int _savevs;
	private final int _activateRate;
	private final int _levelModifier;
	private final byte _magicLevel;
	private final boolean _cancelable;
	private final boolean _shieldignore;
	private final boolean _critical;
	private final boolean _overhit;
	private final int _weaponsAllowed;
	private final boolean _isOffensive;
	private final boolean _isPvm;

	private final int _forceId;

	private final FlyType _flyType;
	private final int _flyRadius;

	private final ArrayList<ClassId> _canLearn; // which classes can learn
	private final ArrayList<Integer> _teachers; // which NPC teaches

	protected Condition _preCondition;
	protected FuncTemplate[] _funcTemplates;
	protected EffectTemplate[] _effectTemplates;

	private final int _minPledgeClass;
	private final int _minRank;

	private final boolean _isSuicideAttack;
	private final boolean _isSkillTimePermanent;
	private NextAction _nextAction;

	private final int _chance;

	private final int _symbolId;

	private final int _negateSkill;
	private final int _negatePower;

	protected final int _numCharges;
	protected final int _condCharges;

	private final int _npcId;

	protected final boolean _behind;

	private final String _toString;

	public static enum SkillOpType
	{
		OP_PASSIVE,
		OP_ACTIVE,
		OP_TOGGLE,
		OP_ON_ATTACK,
		OP_ON_CRIT,
		OP_ON_MAGIC_ATTACK,
		OP_ON_UNDER_ATTACK
	}

	public static enum NextAction
	{
		DEFAULT,
		NONE,
		ATTACK,
		CAST,
		MOVE
	}

	//elements
	public static enum Element
	{
		FIRE,
		WATER,
		WIND,
		EARTH,
		SACRED,
		UNHOLY,
		NONE
	}

	public static enum SkillTargetType
	{
		TARGET_NONE,
		TARGET_SELF,
		TARGET_ONE,
		TARGET_PARTY,
		TARGET_PARTY_ONE,
		TARGET_CLAN,
		TARGET_ALLY,
		TARGET_PET,
		TARGET_OWNER,
		TARGET_ENEMY_PET,
		TARGET_AREA,
		TARGET_AURA,
		TARGET_CORPSE,
		TARGET_MULTIFACE,
		TARGET_CORPSE_PLAYER,
		TARGET_ITEM,
		TARGET_AREA_AIM_CORPSE,
		TARGET_UNLOCKABLE,
		TARGET_CHEST,
		TARGET_HOLY,
		TARGET_SIEGE,
		TARGET_FLAGPOLE
	}

	public static enum SkillType
	{
		AGGRESSION(Aggression.class),
		AIEFFECTS(AIeffects.class),
		BALANCE(Balance.class),
		BLEED(Continuous.class),
		BUFF(Continuous.class),
		CALL(Call.class),
		CANCEL(Cancel.class),
		CHARGE(Charge.class),
		CHARGEDAM(ChargeDmg.class),
		COMBATPOINTHEAL(CombatPointHeal.class),
		CONT(Toggle.class),
		CPHOT(Continuous.class),
		CRAFT(Craft.class),
		DEATH_PENALTY(DeathPenalty.class),
		DANCE(Continuous.class),
		DEBUFF(Continuous.class),
		DOT(Continuous.class),
		DRAIN(Drain.class),
		DRAIN_SOUL(DrainSoul.class),
		ENCHANT_ARMOR,
		ENCHANT_WEAPON,
		FATALBLOW(FatalBlow.class),
		FEED_PET,
		FISHING(Fishing.class),
		HEAL(Heal.class),
		HEAL_PERCENT(HealPercent.class),
		MANAHEAL_PERCENT(ManaHealPercent.class),
		HOT(Continuous.class),
		LETHAL_SHOT(LethalShot.class),
		LUCK,
		MANAHEAL(ManaHeal.class),
		MDAM(MDam.class),
		MDAM_ELEMENTAL(Elemental.class),
		MDOT(Continuous.class),
		MPHOT(Continuous.class),
		MUTE(Disablers.class),
		NEGATE_EFFECTS(NegateEffects.class),
		NEGATE_STATS(NegateStats.class),
		PARALYZE(Disablers.class),
		PASSIVE,
		PDAM(PDam.class),
		CPDAM(CPDam.class),
		POISON(Continuous.class),
		PUMPING(ReelingPumping.class),
		RECALL(Recall.class),
		REELING(ReelingPumping.class),
		RESURRECT(Resurrect.class),
		ROOT(Disablers.class),
		SEED(Seed.class),
		SIEGEFLAG(SiegeFlag.class),
		SLEEP(Disablers.class),
		SONG(Continuous.class),
		SOULSHOT,
		SPIRITSHOT,
		SPHEAL(SPHeal.class),
		SPOIL(Spoil.class),
		SOWING(Sowing.class),
		HARVESTING(Harvesting.class),
		STUN(Disablers.class),
		SUMMON(Summon.class),
		RIDE(Ride.class),
		DESTROY_SUMMON(DestroySummon.class),
		SUMMON_ITEM(SummonItem.class),
		SWEEP(Sweep.class),
		TAKECASTLE(TakeCastle.class),
		TAKEFORT(TakeFortress.class),
		TELEPORT_NPC(TeleportNpc.class),
		UNLOCK(Unlock.class),
		WATCHER_GAZE(Continuous.class),
		BEAST_FEED(BeastFeed.class),
		MANADAM(ManaDam.class),
		FORCE_BUFF(Continuous.class),
		DELETE_HATE(DeleteHate.class),
		DELETE_HATE_OF_ME(DeleteHateOfMe.class),
		SHIFT_AGGRESSION(ShiftAggression.class),
		DISCORD(Continuous.class),

		// effect based
		EFFECT(Effect.class),
		HARDCODED(Effect.class),
		// unimplemented
		NOTDONE;

		private final Class<? extends L2Skill> clazz;

		public L2Skill makeSkill(StatsSet set)
		{
			try
			{
				Constructor<? extends L2Skill> c = clazz.getConstructor(StatsSet.class);
				return c.newInstance(set);
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new RuntimeException(e);
			}
		}

		private SkillType()
		{
			clazz = Default.class;
		}

		private SkillType(Class<? extends L2Skill> clazz)
		{
			this.clazz = clazz;
		}
	}

	public final boolean isPvpSkill()
	{
		switch(_skillType)
		{
			case BLEED:
			case CANCEL:
			case AGGRESSION:
			case DEBUFF:
			case DOT:
			case MDOT:
			case MUTE:
			case PARALYZE:
			case POISON:
			case ROOT:
			case SLEEP:
			case MANADAM:
			case DESTROY_SUMMON:
			case NEGATE_STATS:
			case DELETE_HATE:
			case DELETE_HATE_OF_ME:
				return true;
			default:
				return false;
		}
	}

	public boolean isOffensive()
	{
		if(_isOffensive)
			return _isOffensive;

		switch(_skillType)
		{
			case AGGRESSION:
			case AIEFFECTS:
			case BLEED:
			case CANCEL:
			case CHARGEDAM:
			case DEBUFF:
			case DOT:
			case DRAIN:
			case DRAIN_SOUL:
			case FATALBLOW:
			case LETHAL_SHOT:
			case MANADAM:
			case MDAM:
			case MDAM_ELEMENTAL:
			case MDOT:
			case MUTE:
			case PARALYZE:
			case PDAM:
			case CPDAM:
			case POISON:
			case ROOT:
			case SLEEP:
			case SOULSHOT:
			case SPIRITSHOT:
			case SPOIL:
			case STUN:
			case SWEEP:
			case HARVESTING:
			case TELEPORT_NPC:
			case SOWING:
			case DELETE_HATE:
			case DELETE_HATE_OF_ME:
			case SHIFT_AGGRESSION:
			case DESTROY_SUMMON:
			case DISCORD:
			case UNLOCK:
				return true;
			default:
				return false;
		}
	}

	/**
	 * Работают только против npc
	 */
	public boolean isPvM()
	{
		if(_isPvm)
			return _isPvm;

		switch(_skillType)
		{
			case DISCORD:
				return true;
			default:
				return false;
		}
	}

	/**
	 * Такие скиллы не аггрят цель, и не флагают чара, но являются "плохими"
	 */
	public boolean isAI()
	{
		switch(_skillType)
		{
			case AGGRESSION:
			case AIEFFECTS:
			case SOWING:
			case DELETE_HATE:
			case DELETE_HATE_OF_ME:
			case SHIFT_AGGRESSION:
				return true;
			default:
				return false;
		}
	}

	public boolean isAoE()
	{
		switch(_targetType)
		{
			case TARGET_AREA:
			case TARGET_AREA_AIM_CORPSE:
			case TARGET_AURA:
			case TARGET_MULTIFACE:
				return true;
			default:
				return false;
		}
	}

	public boolean isNotTargetAoE()
	{
		switch(_targetType)
		{
			case TARGET_AURA:
			case TARGET_MULTIFACE:
			case TARGET_ALLY:
			case TARGET_CLAN:
			case TARGET_PARTY:
				return true;
			default:
				return false;
		}
	}

	public boolean oneTarget()
	{
		switch(_targetType)
		{
			case TARGET_CORPSE:
			case TARGET_CORPSE_PLAYER:
			case TARGET_HOLY:
			case TARGET_ITEM:
			case TARGET_NONE:
			case TARGET_ONE:
			case TARGET_PARTY_ONE:
			case TARGET_PET:
			case TARGET_OWNER:
			case TARGET_ENEMY_PET:
			case TARGET_SELF:
			case TARGET_UNLOCKABLE:
			case TARGET_CHEST:
			case TARGET_SIEGE:
			case TARGET_FLAGPOLE:
				return true;
			default:
				return false;
		}
	}

	/**
	 * Внимание!!! У наследников вручную надо поменять тип на public
	 * @param set парамерты скилла
	 */
	protected L2Skill(StatsSet set)
	{
		_id = set.getInteger("skill_id");
		_level = set.getShort("level");
		_displayId = set.getInteger("displayId", _id);
		_displayLevel = set.getShort("displayLevel", _level);
		_name = set.getString("name");
		_operateType = set.getEnum("operateType", SkillOpType.class);
		_magic = set.getBool("isMagic", false);
		_altUse = set.getBool("altUse", false);
		_mpConsume1 = set.getInteger("mpConsume1", 0);
		_mpConsume2 = set.getInteger("mpConsume2", 0);
		_hpConsume = set.getInteger("hpConsume", 0);
		_matak = set.getInteger("mAtk", 0);
		_useSS = set.getBool("useSS", true);

		_forceId = set.getInteger("forceId", 0);

		//ItemConsume quick hack
		String s1 = set.getString("itemConsumeCount", "");
		String s2 = set.getString("itemConsumeId", "");

		if(s1.length() == 0)
			_itemConsume = new int[] { 0 };
		else
		{
			String[] s = s1.split(" ");
			_itemConsume = new int[s.length];
			for(int i = 0; i < s.length; i++)
				_itemConsume[i] = Integer.parseInt(s[i]);
		}

		if(s2.length() == 0)
			_itemConsumeId = new int[] { 0 };
		else
		{
			String[] s = s2.split(" ");
			_itemConsumeId = new int[s.length];
			for(int i = 0; i < s.length; i++)
				_itemConsumeId[i] = Integer.parseInt(s[i]);
		}

		_isItemHandler = set.getBool("isHandler", false);
		_reuseGroupId = set.getInteger("reuseGroup", 0);
		if(_reuseGroupId > 0)
		{
			if(_reuseGroups.get(_reuseGroupId) == null)
				_reuseGroups.put(_reuseGroupId, new ArrayList<L2Skill>());
			_reuseGroups.get(_reuseGroupId).add(this);
		}
		_isCommon = set.getBool("isCommon", false);
		_isSaveable = set.getBool("isSaveable", true);
		_isValidateable = set.getBool("isValidateable", !_name.contains("Item Skill"));
		_castRange = set.getInteger("castRange", 0) == -2 ? Integer.MAX_VALUE : set.getInteger("castRange", 0);
		_hitTime = set.getInteger("hitTime", 0);
		_coolTime = set.getInteger("coolTime", 0);
		_skillInterruptTime = set.getInteger("skillInterruptTime", _hitTime * 3 / 4);
		_reuseDelay = set.getLong("reuseDelay", 0);
		_skillRadius = set.getInteger("skillRadius", 80);
		_targetType = set.getEnum("target", SkillTargetType.class);
		_undeadOnly = set.getBool("undeadOnly", false);
		_corpse = set.getBool("corpse", false);
		_power = set.getDouble("power", 0.);
		_effectPoint = set.getInteger("effectPoint", 0);
		_nextAction = NextAction.valueOf(set.getString("nextAction", "DEFAULT").toUpperCase());
		_skillType = set.getEnum("skillType", SkillType.class);
		_isSuicideAttack = set.getBool("isSuicideAttack", false);
		_isSkillTimePermanent = set.getBool("isSkillTimePermanent", false);
		_element = Element.valueOf(set.getString("element", "NONE").toUpperCase());
		_savevs = set.getInteger("save", 0);
		_activateRate = set.getInteger("activateRate", -1);
		_levelModifier = set.getInteger("levelModifier", 1);
		_magicLevel = set.getByte("magicLevel", (byte) 0);
		_cancelable = set.getBool("cancelable", true);
		_shieldignore = set.getBool("shieldignore", false);
		_critical = set.getBool("critical", false);
		_overhit = set.getBool("overHit", false);
		_weaponsAllowed = set.getInteger("weaponsAllowed", 0);
		_minPledgeClass = set.getInteger("minPledgeClass", 0);
		_minRank = set.getInteger("minRank", 0);
		_isOffensive = set.getBool("isOffensive", false);
		_isPvm = set.getBool("isPvm", false);
		_chance = set.getInteger("chance", 100);
		_behind = set.getBool("behind", false);
		_symbolId = set.getInteger("symbolId", 0);
		_npcId = set.getInteger("npcId", 0);

		_flyType = FlyType.valueOf(set.getString("flyType", "NONE").toUpperCase());
		_flyRadius = set.getInteger("flyRadius", 200);

		_negateSkill = set.getInteger("negateSkill", 0);
		_negatePower = set.getInteger("negatePower", Integer.MAX_VALUE);

		_numCharges = set.getInteger("num_charges", 0);
		_condCharges = set.getInteger("cond_charges", 0);

		StringTokenizer st = new StringTokenizer(set.getString("addSkills", ""), ";");
		if(st.hasMoreTokens())
		{
			_addedSkills = new AddedSkill[st.countTokens() / 2];
			int i = 0;
			while(st.hasMoreTokens())
			{
				_addedSkills[i] = new AddedSkill(Integer.valueOf(st.nextToken()), Integer.valueOf(st.nextToken()));
				i++;
			}
		}

		_toString = _name + "[id=" + _id + ",lvl=" + _level + "]";

		if(_nextAction == NextAction.DEFAULT)
			switch(_skillType)
			{
				case PDAM:
				case CPDAM:
				case FATALBLOW:
				case LETHAL_SHOT:
				case CHARGEDAM:
				case SPOIL:
				case SOWING:
				case STUN:
				case DRAIN_SOUL:
					_nextAction = NextAction.ATTACK;
					break;
				default:
					_nextAction = NextAction.NONE;
			}

		if(_savevs == 0)
			switch(_skillType)
			{
				case BLEED:
				case DOT:
				case MDOT:
				case FATALBLOW:
				case LETHAL_SHOT:
				case PDAM:
				case CPDAM:
				case POISON:
				case STUN:
					_savevs = SAVEVS_CON;
					break;
				case CANCEL:
				case MANADAM:
				case DEBUFF:
				case MDAM:
				case MDAM_ELEMENTAL:
				case MUTE:
				case PARALYZE:
				case ROOT:
				case SLEEP:
					_savevs = SAVEVS_MEN;
					break;
			}

		String canLearn = set.getString("canLearn", null);
		if(canLearn == null)
			_canLearn = null;
		else
		{
			_canLearn = new ArrayList<ClassId>();
			st = new StringTokenizer(canLearn, " \r\n\t,;");
			while(st.hasMoreTokens())
			{
				String cls = st.nextToken();
				try
				{
					_canLearn.add(ClassId.valueOf(cls));
				}
				catch(Throwable t)
				{
					_log.log(Level.SEVERE, "Bad class " + cls + " to learn skill", t);
				}
			}
		}

		String teachers = set.getString("teachers", null);
		if(teachers == null)
			_teachers = null;
		else
		{
			_teachers = new ArrayList<Integer>();
			st = new StringTokenizer(teachers, " \r\n\t,;");
			while(st.hasMoreTokens())
			{
				String npcid = st.nextToken();
				try
				{
					_teachers.add(Integer.parseInt(npcid));
				}
				catch(Throwable t)
				{
					_log.log(Level.SEVERE, "Bad teacher id " + npcid + " to teach skill", t);
				}
			}
		}
	}

	public abstract void useSkill(L2Character activeChar, FastList<L2Character> targets);

	public final boolean altUse()
	{
		return _altUse;
	}

	public final SkillType getSkillType()
	{
		return _skillType;
	}

	public final int getSavevs()
	{
		return _savevs;
	}

	public final int getActivateRate()
	{
		return _activateRate;
	}

	public final int getLevelModifier()
	{
		return _levelModifier;
	}

	public final byte getMagicLevel()
	{
		return _magicLevel;
	}

	public final boolean isCancelable()
	{
		return _cancelable;
	}

	public final boolean getShieldIgnore()
	{
		return _shieldignore;
	}

	public final boolean isCritical()
	{
		return _critical;
	}

	public final Element getElement()
	{
		return _element;
	}

	public final SkillTargetType getTargetType()
	{
		return _targetType;
	}

	public final boolean isOverhit()
	{
		return _overhit;
	}

	public final boolean isSuicideAttack()
	{
		return _isSuicideAttack;
	}

	/**
	 * На некоторые скиллы и хендлеры предметов скорости каста/атаки не влияет
	 */
	public final boolean isSkillTimePermanent()
	{
		return _isSkillTimePermanent || _isItemHandler;
	}

	/**
	 * Return the power of the skill.<BR><BR>
	 */
	public final double getPower()
	{
		return _power;
	}

	/**
	 * @return Returns the castRange.
	 */
	public final int getCastRange()
	{
		return _castRange;
	}

	/**
	 * @return Returns the hpConsume.
	 */
	public final int getHpConsume()
	{
		return _hpConsume;
	}

	/**
	 * @return Returns the id.
	 */
	public final Integer getId()
	{
		return _id;
	}

	public final Integer getDisplayId()
	{
		return _displayId;
	}

	public Short getDisplayLevel()
	{
		return _displayLevel;
	}

	public void setDisplayLevel(Short lvl)
	{
		_displayLevel = lvl;
	}

	public int getMinPledgeClass()
	{
		return _minPledgeClass;
	}

	/**
	 * @return Returns the itemConsume.
	 */
	public final int[] getItemConsume()
	{
		return _itemConsume;
	}

	/**
	 * @return Returns the itemConsumeId.
	 */
	public final int[] getItemConsumeId()
	{
		return _itemConsumeId;
	}

	public final boolean isHandler()
	{
		return _isItemHandler;
	}

	/**
	 * Является ли скилл общим
	 */
	public final boolean isCommon()
	{
		return _isCommon;
	}

	public final int getReuseGroupId()
	{
		return _reuseGroupId;
	}

	public final ArrayList<L2Skill> getReuseGroup()
	{
		return _reuseGroups.get(_reuseGroupId);
	}

	/**
	 * @return Returns the level.
	 */
	public final short getLevel()
	{
		return _level;
	}

	/**
	 * @return Returns true if skill is magic.
	 */
	public final boolean isMagic()
	{
		return _magic;
	}

	/**
	 * @return Returns the mpConsume as _mpConsume1 + _mpConsume2.
	 */
	public final int getMpConsume()
	{
		return _mpConsume1 + _mpConsume2;
	}

	/**
	 * @return Returns the mpConsume1.
	 */
	public final int getMpConsume1()
	{
		return _mpConsume1;
	}

	/**
	 * @return Returns the mpConsume2.
	 */
	public final int getMpConsume2()
	{
		return _mpConsume2;
	}

	/**
	 * @return Returns the name.
	 */
	public final String getName()
	{
		return _name;
	}

	/**
	 * @return Returns the reuseDelay.
	 */
	public final long getReuseDelay()
	{
		return _reuseDelay;
	}

	public final int getHitTime()
	{
		return _hitTime;
	}

	public final int getCoolTime()
	{
		return _coolTime;
	}

	public final int getSkillInterruptTime()
	{
		return _skillInterruptTime;
	}

	public final int getSkillRadius()
	{
		return _skillRadius;
	}

	public final boolean isActive()
	{
		return _operateType == SkillOpType.OP_ACTIVE;
	}

	public final boolean isPassive()
	{
		return _operateType == SkillOpType.OP_PASSIVE;
	}

	public final boolean isToggle()
	{
		return _operateType == SkillOpType.OP_TOGGLE;
	}

	public final boolean isOnAttack()
	{
		return _operateType == SkillOpType.OP_ON_ATTACK;
	}

	public final boolean isOnCrit()
	{
		return _operateType == SkillOpType.OP_ON_CRIT;
	}

	public final boolean isOnMagicAttack()
	{
		return _operateType == SkillOpType.OP_ON_MAGIC_ATTACK;
	}

	public final boolean isOnUnderAttack()
	{
		return _operateType == SkillOpType.OP_ON_UNDER_ATTACK;
	}

	public final int getWeaponsAllowed()
	{
		return _weaponsAllowed;
	}

	public final boolean getCanLearn(ClassId cls)
	{
		return _canLearn == null || _canLearn.contains(cls);
	}

	public final boolean canTeachBy(int npcId)
	{
		return _teachers == null || _teachers.contains(npcId);
	}

	public final boolean getWeaponDependancy(L2Character activeChar)
	{
		if(_weaponsAllowed == 0)
			return true;

		if(activeChar.getActiveWeaponInstance() != null && activeChar.getActiveWeaponItem() != null)
			if((activeChar.getActiveWeaponItem().getItemType().mask() & _weaponsAllowed) != 0)
				return true;

		if(activeChar.getSecondaryWeaponInstance() != null && activeChar.getSecondaryWeaponItem() != null)
			if((activeChar.getSecondaryWeaponItem().getItemType().mask() & _weaponsAllowed) != 0)
				return true;

		StringBuffer skillmsg = new StringBuffer();
		skillmsg.append(_name);
		skillmsg.append(" can only be used with weapons of type ");
		for(WeaponType wt : WeaponType.values())
			if((wt.mask() & _weaponsAllowed) != 0)
				skillmsg.append(wt).append('/');
		skillmsg.setCharAt(skillmsg.length() - 1, '.');
		activeChar.sendMessage(skillmsg.toString());

		return false;
	}

	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, @SuppressWarnings("unused")
	boolean dontMove, boolean first)
	{
		L2Player player = activeChar.getPlayer();

		if(activeChar.isDead())
			return false;

		if(!getWeaponDependancy(activeChar))
			return false;

		if(first && activeChar.isSkillDisabled(_id))
		{
			if(Config.ALT_SHOW_REUSE_MSG && activeChar.isPlayer)
				activeChar.sendPacket(new SystemMessage(SystemMessage.S1_IS_NOT_AVAILABLE_AT_THIS_TIME_BEING_PREPARED_FOR_REUSE).addSkillName(_id, _displayLevel));
			return false;
		}

		if(first && activeChar.getCurrentMp() < (isMagic() ? activeChar.calcStat(Stats.MP_MAGIC_SKILL_CONSUME, _mpConsume1 + _mpConsume2, null, this) : activeChar.calcStat(Stats.MP_PHYSICAL_SKILL_CONSUME, _mpConsume1 + _mpConsume2, null, this)))
		{
			activeChar.sendPacket(Msg.NOT_ENOUGH_MP);
			return false;
		}

		if(activeChar.getCurrentHp() < _hpConsume + 1)
		{
			activeChar.sendPacket(Msg.NOT_ENOUGH_HP);
			return false;
		}

		if(!(_isItemHandler || _altUse) && (isMagic() && activeChar.isMuted() || !isMagic() && activeChar.isPMuted()))
			return false;

		if(activeChar.getIncreasedForce() < _condCharges || activeChar.getIncreasedForce() < _numCharges)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOUR_FORCE_HAS_REACHED_MAXIMUM_CAPACITY_));
			return false;
		}

		if(player != null)
		{
			if((_isItemHandler || _altUse) && player.isInOlympiadMode())
			{
				player.sendPacket(Msg.THIS_ITEM_IS_NOT_AVAILABLE_FOR_THE_OLYMPIAD_EVENT);
				return false;
			}

			if(player.isInBoat())
				return false;

			if(player.inObserverMode())
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.OBSERVERS_CANNOT_PARTICIPATE));
				return false;
			}

			if(player.isBot() && getSkillType() == SkillType.CRAFT && !Config.ALLOW_CRAFT_BOT)
			{
				player.sendMessage("Crafting is disabled for bots");
				return false;
			}

			// If summon siege golem, hog cannon, Swoop Cannon, check its ok to place the flag
			if(_id == 13 || _id == 299 || _id == 448)
			{
				SystemMessage sm = null;
				Siege siege = SiegeManager.getSiege(player);
				if(siege == null)
					sm = new SystemMessage(SystemMessage.YOU_MAY_NOT_SUMMON_FROM_YOUR_CURRENT_LOCATION);
				else if(!siege.isInProgress())
					sm = new SystemMessage(SystemMessage.YOU_ARE_NOT_IN_SIEGE);
				else if(player.getClanId() != 0 && siege.getAttackerClan(player.getClanId()) == null)
					sm = new SystemMessage(SystemMessage.OBSERVATION_IS_ONLY_POSSIBLE_DURING_A_SIEGE);
				if(sm != null)
				{
					player.sendPacket(sm);
					return false;
				}
			}

			if(first && _itemConsume[0] > 0)
				for(int i = 0; i < _itemConsume.length; i++)
				{
					L2ItemInstance requiredItems = player.getInventory().findItemByItemId(_itemConsumeId[i]);
					if(requiredItems == null || requiredItems.getCount() < _itemConsume[i])
					{
						player.sendPacket(Msg.INCORRECT_ITEM_COUNT);
						return false;
					}
				}

			if(player.isFishing() && _id != 1312 && _id != 1313 && _id != 1314)
			{
				player.sendPacket(Msg.ONLY_FISHING_SKILLS_ARE_AVAILABLE);
				return false;
			}
		}

		SystemMessage msg = checkTarget(activeChar, target, forceUse, first);
		if(msg != null && activeChar.getPlayer() != null)
		{
			activeChar.getPlayer().sendPacket(msg);
			return false;
		}

		if(_preCondition == null)
			return true;

		Env env = new Env();
		env.character = activeChar;
		env.skill = this;
		env.target = target;

		if(!_preCondition.test(env))
		{
			String cond_msg = _preCondition.getMessage();
			if(cond_msg != null)
				activeChar.sendMessage(cond_msg);
			return false;
		}

		return true;
	}

	public SystemMessage checkTarget(L2Character activeChar, L2Character target, boolean forceUse, boolean first)
	{
		if(target == activeChar && isNotTargetAoE())
			return null;
		if(target == null || isOffensive() && target == activeChar)
			return Msg.TARGET_IS_INCORRECT;
		// Попадает ли цель в радиус действия в конце каста
		if(!first && target != activeChar && getCastRange() > 0 && getCastRange() != Integer.MAX_VALUE && !activeChar.isInRange(target.getLoc(), getCastRange() + (getCastRange() < 200 ? 400 : 500)))
			return Msg.YOUR_TARGET_IS_OUT_OF_RANGE;
		// Конусообразные скиллы
		if(target != activeChar && _targetType == SkillTargetType.TARGET_MULTIFACE && (_behind ? activeChar.isInFront(target, 120) : !activeChar.isInFront(target, 60)))
			return Msg.YOUR_TARGET_IS_OUT_OF_RANGE;
		// Проверка на каст по трупу
		if(target.isDead() != _corpse && _targetType != SkillTargetType.TARGET_AREA_AIM_CORPSE || _undeadOnly && !target.isUndead())
			return Msg.INCORRECT_TARGET;
		if(_targetType != SkillTargetType.TARGET_UNLOCKABLE && target instanceof L2DoorInstance && !((L2DoorInstance) target).isAttackable(activeChar))
			return Msg.INCORRECT_TARGET;
		// Для различных бутылок, и для скилла кормления, дальнейшие проверки ненужны
		if(_altUse || _skillType.equals(SkillType.BEAST_FEED))
			return null;
		if(activeChar instanceof L2Playable)
		{
			L2Player player = activeChar.getPlayer();
			if(player == null)
				return Msg.TARGET_IS_INCORRECT;
			if(target instanceof L2Playable)
			{
				if(isPvM())
					return Msg.TARGET_IS_INCORRECT;
				L2Player pcTarget = target.getPlayer();
				if(pcTarget == null)
					return Msg.TARGET_IS_INCORRECT;
				if(pcTarget.isInOlympiadMode() && player.getOlympiadGameId() != pcTarget.getOlympiadGameId())
					return Msg.TARGET_IS_INCORRECT;
				if(isOffensive())
				{
					if(player.isInOlympiadMode() && !player.isOlympiadCompStart())
						return Msg.TARGET_IS_INCORRECT;
					if(isAoE() && getCastRange() < Integer.MAX_VALUE && !GeoClient.getInstance().canSeeTarget(activeChar, target))
						return Msg.CANNOT_SEE_TARGET;
					if(activeChar.isInZoneBattle() != target.isInZoneBattle() || activeChar.isInZonePeace() || target.isInZonePeace())
						return Msg.YOU_CANNOT_ATTACK_THE_TARGET_IN_THE_PEACE_ZONE;
					if(activeChar.isInZoneBattle())
					{
						if(!forceUse && player.getParty() != null && player.getParty() == pcTarget.getParty())
							return Msg.INCORRECT_TARGET;
					}
					else
					{
						if(isPvpSkill() || !forceUse || isAoE())
						{
							if(player.getParty() != null && player.getParty() == pcTarget.getParty())
								return Msg.INCORRECT_TARGET;
							if(player.getClanId() != 0 && player.getClanId() == pcTarget.getClanId())
								return Msg.INCORRECT_TARGET;
							if(player.getAllyId() != 0 && player.getAllyId() == pcTarget.getAllyId())
								return Msg.INCORRECT_TARGET;
						}

						if(player.atMutualWarWith(pcTarget))
							return null;
						if(player.getDuel() != null && target.getDuel() != player.getDuel())
							return Msg.INCORRECT_TARGET;
						if(player.getDuel() != null && target.getDuel() != null && target.getDuel() == player.getDuel())
							return null;
						if(pcTarget.getPvpFlag() != 0)
							return null;
						if(pcTarget.getKarma() > 0)
							return null;
						if(forceUse && !isPvpSkill())
							return null;

						if(activeChar.isInZone(ZoneType.Siege) && target.isInZone(ZoneType.Siege))
						{
							L2Clan clan1 = player.getClan();
							L2Clan clan2 = pcTarget.getClan();
							if(clan1 == null || clan2 == null)
								return null;
							if(!clan1.isDefender() && !clan1.isAttacker())
								return null;
							if(!clan2.isDefender() && !clan2.isAttacker())
								return null;
							if(clan1.isDefender() && clan2.isAttacker() || clan2.isDefender() && clan1.isAttacker())
								return null;
						}

						return Msg.INCORRECT_TARGET;
					}
				}
				else
				{
					if(forceUse)
						return null;
					if(target == player)
						return null;
					if(player.isInOlympiadMode() && !player.isOlympiadGameStart() && !player.isOlympiadCompStart())
						return Msg.INCORRECT_TARGET;
					if(!activeChar.isInZoneBattle() && target.isInZoneBattle())
						return Msg.INCORRECT_TARGET;
					if(player.getParty() != null && player.getParty() == pcTarget.getParty())
						return null;
					if(player.getClanId() != 0 && player.getClanId() == pcTarget.getClanId())
						return null;
					if(player.getAllyId() != 0 && player.getAllyId() == pcTarget.getAllyId())
						return null;

					if(player.getDuel() != null && target.getDuel() != player.getDuel())
						return Msg.INCORRECT_TARGET;
					if(player.getDuel() != null && target.getDuel() != null && target.getDuel() == player.getDuel())
						return Msg.INCORRECT_TARGET;
					if(player.atMutualWarWith(pcTarget))
						return Msg.INCORRECT_TARGET;
					if(pcTarget.getPvpFlag() != 0)
						return Msg.INCORRECT_TARGET;
					if(pcTarget.getKarma() > 0)
						return Msg.INCORRECT_TARGET;
				}
				return null;
			}
			if(!forceUse && !isOffensive() && target.isAutoAttackable(activeChar))
				return Msg.INCORRECT_TARGET;
			if(!forceUse && isOffensive() && !target.isAutoAttackable(activeChar))
				return Msg.INCORRECT_TARGET;
			return null;
		}
		if(activeChar.isMonster && getCastRange() < Integer.MAX_VALUE && !GeoClient.getInstance().canSeeTarget(activeChar, target))
			return Msg.CANNOT_SEE_TARGET;
		return null;
	}

	public final L2Character getAimingTarget(L2Character activeChar)
	{
		L2Object obj = activeChar.getTarget();
		L2Character target = obj == null || !obj.isCharacter ? null : (L2Character) obj;
		switch(_targetType)
		{
			case TARGET_ALLY:
			case TARGET_CLAN:
			case TARGET_PARTY:
			case TARGET_SELF:
				return activeChar;
			case TARGET_AURA:
			case TARGET_MULTIFACE:
				return activeChar;
			case TARGET_HOLY:
				return activeChar.isPlayer && target instanceof L2ArtefactInstance ? target : null;
			case TARGET_FLAGPOLE:
				return activeChar.isPlayer && target instanceof L2StaticObjectInstance && ((L2StaticObjectInstance) target).getType() == 3 ? target : null;
			case TARGET_UNLOCKABLE:
				return target instanceof L2DoorInstance || target instanceof L2ChestInstance ? target : null;
			case TARGET_CHEST:
				return target instanceof L2ChestInstance ? target : null;
			case TARGET_PET:
				target = activeChar.getPet() != null ? activeChar.getPet() : null;
				return target != null && target.isDead() == _corpse ? target : null;
			case TARGET_OWNER:
				if(activeChar.isPet)
					target = ((L2PetInstance) activeChar).getOwner();
				else
					return null;
				return target != null && target.isDead() == _corpse ? target : null;
			case TARGET_ENEMY_PET:
				if(target == null || target == activeChar.getPet() || !(target instanceof L2Summon))
					return null;
				return target;
			case TARGET_ONE:
				return target != null && target.isDead() == _corpse && !(target == activeChar && isOffensive()) && (!_undeadOnly || target.isUndead()) ? target : null;
			case TARGET_PARTY_ONE:
				if(target == null)
					return null;
				// self or self pet.
				if(target.getPlayer() != null && target.getPlayer().equals(activeChar))
					return target;
				// party member or party member pet.
				if(target.getPlayer() != null && activeChar.getPlayer() != null && activeChar.getPlayer().getParty() != null && activeChar.getPlayer().getParty().containsMember(target.getPlayer()) && target.isDead() == _corpse && !(target == activeChar && isOffensive()) && (!_undeadOnly || target.isUndead()))
					return target;
				return null;
			case TARGET_AREA:
				return target != null && target != activeChar && !target.isDead() && target.isAttackable() ? target : null;
			case TARGET_AREA_AIM_CORPSE:
				return target != null && target.isDead() ? target : null;
			case TARGET_CORPSE:
				return target instanceof L2NpcInstance && target.isDead() ? target : null;
			case TARGET_CORPSE_PLAYER:
				return target instanceof L2Playable && target.isDead() ? target : null;
			case TARGET_SIEGE:
				return target != null && !target.isDead() && (target instanceof L2DoorInstance || target instanceof L2ControlTowerInstance) ? target : null;
			default:
				activeChar.sendMessage("Target type of skill is not currently handled");
				return null;
		}
	}

	public FastList<L2Character> getTargets(L2Character activeChar, L2Character aimingTarget, boolean forceUse)
	{
		FastList<L2Character> targets = new FastList<L2Character>();

		if(oneTarget())
		{
			targets.add(aimingTarget);
			return targets;
		}

		switch(_targetType)
		{
			case TARGET_AREA:
			{
				if(aimingTarget.isDead() == _corpse && (!_undeadOnly || aimingTarget.isUndead()))
					targets.add(aimingTarget);
				addTargetsToList(targets, aimingTarget, activeChar, forceUse);
				break;
			}
			case TARGET_AREA_AIM_CORPSE:
			case TARGET_AURA:
			case TARGET_MULTIFACE:
			{
				addTargetsToList(targets, activeChar, activeChar, forceUse);
				break;
			}
			case TARGET_PARTY:
			{
				L2Player player = activeChar.getPlayer();
				L2Party party = player.getParty();
				if(party == null)
				{
					if(!_corpse)
						targets.add(player);
					L2Summon pet = player.getPet();
					if(pet != null && player.isInRange(pet, _skillRadius) && pet.isDead() == _corpse)
						targets.add(pet);
					break;
				}
				ArrayList<L2Player> party_list = party.getPartyMembers();
				for(L2Player pl : party_list)
				{
					if(player.isInRange(pl, _skillRadius) && pl.isDead() == _corpse)
						targets.add(pl);
					L2Summon pet = pl.getPet();
					if(pet != null && player.isInRange(pet, _skillRadius) && pet.isDead() == _corpse)
						targets.add(pet);
				}
				break;
			}
			case TARGET_CLAN:
			{
				L2Player[] target_list;
				L2Player player = activeChar.getPlayer();
				if(player.getClanId() != 0)
				{
					target_list = player.getClan().getOnlineMembers(null);
					for(L2Player pl : target_list)
					{
						if(player.isInRange(pl, _skillRadius) && pl.isDead() == _corpse)
							targets.add(pl);
						L2Summon pet = pl.getPet();
						if(pet != null && player.isInRange(pet, _skillRadius) && pet.isDead() == _corpse)
							targets.add(pet);
					}
				}
				else
				{
					if(!_corpse)
						targets.add(player);
					L2Summon pet = player.getPet();
					if(pet != null && pet.isDead() == _corpse)
						targets.add(pet);
				}
				break;
			}
			case TARGET_ALLY:
			{
				L2Player[] target_list;
				L2Player player = activeChar.getPlayer();
				if(player.getClanId() != 0)
				{
					if(player.getAllyId() == 0)
						target_list = player.getClan().getOnlineMembers(null);
					else
						target_list = player.getAlliance().getOnlineMembers(null);
					for(L2Player pl : target_list)
					{
						if(player.isInRange(pl, _skillRadius) && pl.isDead() == _corpse)
							targets.add(pl);
						L2Summon pet = pl.getPet();
						if(pet != null && player.isInRange(pet, _skillRadius) && pet.isDead() == _corpse)
							targets.add(pet);
					}
				}
				else
				{
					if(!_corpse)
						targets.add(player);
					L2Summon pet = player.getPet();
					if(pet != null && pet.isDead() == _corpse)
						targets.add(pet);
				}
				break;
			}
		}
		return targets;
	}

	private void addTargetsToList(FastList<L2Character> targets, L2Character aimingTarget, L2Character activeChar, boolean forceUse)
	{
		int count = 0;
		FastList<L2Character> temp = aimingTarget.getKnownCharacters(_skillRadius, 400);
		for(FastList.Node<L2Character> n = temp.head(), end = temp.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(activeChar == target)
				continue;
			if(checkTarget(activeChar, target, forceUse, false) != null)
				continue;
			if(!(activeChar instanceof L2DecoyInstance) && activeChar instanceof L2NpcInstance && target instanceof L2NpcInstance)
				continue;
			if(target instanceof L2NpcInstance)
				if(count < 20)
					count++;
				else
					continue;
			targets.add(target);
		}
	}

	public final Func[] getStatFuncs(L2Character player)
	{
		if(_funcTemplates == null)
			return _emptyFunctionSet;
		ArrayList<Func> funcs = new ArrayList<Func>();
		for(FuncTemplate t : _funcTemplates)
		{
			Env env = new Env();
			env.character = player;
			env.skill = this;
			Func f = t.getFunc(env, this); // skill is owner
			if(f != null)
				funcs.add(f);
		}
		if(funcs.size() == 0)
			return _emptyFunctionSet;
		return funcs.toArray(new Func[funcs.size()]);
	}

	public final boolean getEffects(L2Character effector, L2Character effected, boolean calcChance)
	{
		if(isPassive() || _effectTemplates == null || _effectTemplates.length == 0)
			return false;

		// Mystic Immunity Makes a target temporarily immune to buffs/debuffs
		if(effected.getEffectsBySkillId(1411) != null)
			return false;

		// No effect on invulnerable characters unless they cast it themselves.
		if(effector != effected && effected.isInvul())
			return false;

		// No effect on doors/walls
		if(effected instanceof L2DoorInstance)
			return false;

		boolean x = false;

		for(EffectTemplate et : _effectTemplates)
		{
			L2Character target = et._applyOnCaster ? effector : effected;

			//in C4+ sleep, root, etc has no effect if the same effect already in progress
			if(!et.isOverlay() && target.getEffectByResistType(et._resistType) != null)
				continue;

			if(isBlockedByChar(target, et))
				continue;

			Env env = new Env(effector, target, this);

			if(calcChance)
			{
				env.value = _activateRate;
				if(!Formulas.calcSkillSuccess(env, et._resistType))
					continue;
			}

			x = true;
			L2Effect e = et.getEffect(env);
			if(e != null)
				if(e._count == 1 && e.getPeriod() == 0)
				{
					// Эффекты однократного действия не шедулятся а применяются немедленно
					// Как правило это побочные эффекты для скиллов моментального действия
					e.onStart();
					e.onActionTime();
					e.onExit();
				}
				else
					target.addEffect(e);
		}

		if(x)
		{
			if(calcChance)
				effector.sendPacket(new SystemMessage(SystemMessage.S1_HAS_SUCCEEDED).addSkillName(_id, _displayLevel));
		}
		else if(calcChance)
			effector.sendPacket(new SystemMessage(SystemMessage.S1_HAS_FAILED).addSkillName(_id, _displayLevel));

		return x;
	}

	public boolean isBlockedByChar(L2Character effected, EffectTemplate et)
	{
		if(et._stats == null)
			return false;
		for(Stats stat : et._stats)
			if(effected.checkBlockedStat(stat))
				return true;
		return false;
	}

	public final void attach(FuncTemplate f)
	{
		if(_funcTemplates == null)
			_funcTemplates = new FuncTemplate[] { f };
		else
		{
			int len = _funcTemplates.length;
			FuncTemplate[] tmp = new FuncTemplate[len + 1];
			System.arraycopy(_funcTemplates, 0, tmp, 0, len);
			tmp[len] = f;
			_funcTemplates = tmp;
		}
	}

	public final void attach(EffectTemplate effect)
	{
		if(_effectTemplates == null)
			_effectTemplates = new EffectTemplate[] { effect };
		else
		{
			int len = _effectTemplates.length;
			EffectTemplate[] tmp = new EffectTemplate[len + 1];
			System.arraycopy(_effectTemplates, 0, tmp, 0, len);
			tmp[len] = effect;
			_effectTemplates = tmp;
		}
	}

	public final void attach(Condition c)
	{
		_preCondition = c;
	}

	@Override
	public String toString()
	{
		return _toString;
	}

	public int getMinRank()
	{
		return _minRank;
	}

	public int getEffectPoint()
	{
		return _effectPoint;
	}

	public NextAction getNextAction()
	{
		return _nextAction;
	}

	public boolean getCorpse()
	{
		return _corpse;
	}

	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = 1;
		result = prime * result + (_displayId == null ? 0 : _displayId.hashCode());
		result = prime * result + (_displayLevel == null ? 0 : _displayLevel.hashCode());
		result = prime * result + (_id == null ? 0 : _id.hashCode());
		result = prime * result + (_level == null ? 0 : _level.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj)
	{
		if(this == obj)
			return true;
		if(obj == null)
			return false;
		if(getClass() != obj.getClass())
			return false;
		final L2Skill other = (L2Skill) obj;
		if(_displayId == null)
		{
			if(other._displayId != null)
				return false;
		}
		else if(!_displayId.equals(other._displayId))
			return false;
		if(_displayLevel == null)
		{
			if(other._displayLevel != null)
				return false;
		}
		else if(!_displayLevel.equals(other._displayLevel))
			return false;
		if(_id == null)
		{
			if(other._id != null)
				return false;
		}
		else if(!_id.equals(other._id))
			return false;
		if(_level == null)
		{
			if(other._level != null)
				return false;
		}
		else if(!_level.equals(other._level))
			return false;
		return true;
	}

	public EffectTemplate[] getEffectTemplates()
	{
		return _effectTemplates;
	}

	public boolean isSaveable()
	{
		return _isSaveable;
	}

	/**
	 * Скилл подлежит удалению из ярлыков при валидейте, в случае отсутсвия данного скила у чара
	 * @return true, если скилл подлежит удалению
	 */
	public boolean isValidateable()
	{
		return _isValidateable;
	}

	public int getForceId()
	{
		return _forceId;
	}

	public int getChance()
	{
		return _chance;
	}

	public int getSymbolId()
	{
		return _symbolId;
	}

	public AddedSkill[] getAddedSkills()
	{
		if(_addedSkills == null)
			return new AddedSkill[0];
		return _addedSkills;
	}

	public L2Skill getFirstAddedSkill()
	{
		if(_addedSkills == null)
			return null;
		return _addedSkills[0].getSkill();
	}

	public int getNpcId()
	{
		return _npcId;
	}

	public int getNegateSkill()
	{
		return _negateSkill;
	}

	public int getNegatePower()
	{
		return _negatePower;
	}

	public int getNumCharges()
	{
		return _numCharges;
	}

	public int getCondCharges()
	{
		return _condCharges;
	}

	public int getMatak()
	{
		return _matak;
	}

	/**
	 * Может ли скилл тратить шоты, для хендлеров всегда false
	 */
	public boolean isSSPossible()
	{
		return _useSS && !_isItemHandler;
	}

	public FlyType getFlyType()
	{
		return _flyType;
	}

	public int getFlyRadius()
	{
		return _flyRadius;
	}
}
