package l2r.gameserver.model;

import javolution.util.FastList;
import l2r.Config;
import l2r.extensions.scripts.Events;
import l2r.gameserver.ai.CtrlIntention;
import l2r.gameserver.ai.L2SummonAI;
import l2r.gameserver.model.L2Zone.ZoneType;
import l2r.gameserver.model.base.Experience;
import l2r.gameserver.model.entity.Duel;
import l2r.gameserver.model.instances.L2CubicInstance;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.MyTargetSelected;
import l2r.gameserver.serverpackets.NpcInfo;
import l2r.gameserver.serverpackets.PartySpelled;
import l2r.gameserver.serverpackets.PetDelete;
import l2r.gameserver.serverpackets.PetInfo;
import l2r.gameserver.serverpackets.PetStatusShow;
import l2r.gameserver.serverpackets.PetStatusUpdate;
import l2r.gameserver.serverpackets.StatusUpdate;
import l2r.gameserver.taskmanager.DecayTaskManager;
import l2r.gameserver.templates.L2NpcTemplate;
import l2r.gameserver.templates.L2Weapon;
import l2r.util.Location;
import l2r.util.Rnd;

public abstract class L2Summon extends L2Playable
{
	//private static final Logger _log = Logger.getLogger(L2Summon.class.getName());

	protected long _exp = 0;
	protected int _sp = 0;
	private int _attackRange = 36; //Melee range
	private boolean _follow = true;
	private L2Character _lastTarget = null;
	private int _maxLoad;
	private boolean _posessed = false;
	private boolean _showSpawnAnimation = true;

	private boolean _ssCharged = false;
	private int _spsCharged = 0;

	public static final int SIEGE_GOLEM_ID = 14737;
	public static final int SIEGE_CANNON_ID = 14768;
	public static final int SWOOP_CANNON_ID = 14839;

	private static final int SUMMON_DISAPPEAR_RANGE = 2500;

	public L2Summon(final int objectId, final L2NpcTemplate template, final L2Player owner)
	{
		super(objectId, template, owner);
		_owner = owner;
		setXYZInvisible(owner.getX() + Rnd.get(-100, 100), owner.getY() + Rnd.get(-100, 100), owner.getZ());
	}

	@Override
	public void spawnMe()
	{
		super.spawnMe();
		onSpawn();
	}

	@Override
	public L2SummonAI getAI()
	{
		if(_ai == null)
			_ai = new L2SummonAI(this);
		return (L2SummonAI) _ai;
	}

	@Override
	public L2NpcTemplate getTemplate()
	{
		return (L2NpcTemplate) _template;
	}

	@Override
	public boolean isUndead()
	{
		return getTemplate().isUndead();
	}

	// this defines the action buttons, 1 for Summon, 2 for Pets
	public abstract int getSummonType();

	@Override
	public void updateAbnormalEffect()
	{
		broadcastPetInfo();
	}

	/**
	 * @return Returns the mountable.
	 */
	public boolean isMountable()
	{
		return false;
	}

	@Override
	public void onAction(final L2Player player)
	{
		if(Events.onAction(player, this))
			return;

		// Check if the L2Player is confused
		if(player.isConfused() || player.isBlocked())
			player.sendActionFailed();

		if(player.getTarget() != this)
		{
			// Set the target of the player
			player.setTarget(this);
			// The color to display in the select window is White
			player.sendPacket(new MyTargetSelected(getObjectId(), 0));

			//if(isSummon)
			//{
			StatusUpdate su = new StatusUpdate(getObjectId());
			su.addAttribute(StatusUpdate.CUR_HP, (int) getCurrentHp());
			su.addAttribute(StatusUpdate.MAX_HP, getMaxHp());
			//su.addAttribute(StatusUpdate.CUR_MP, (int) getCurrentMp());
			//su.addAttribute(StatusUpdate.MAX_MP, getMaxMp());
			player.sendPacket(su);
			//}
		}
		else if(player == _owner)
		{
			player.sendPacket(new PetStatusShow(this));
			player.sendActionFailed();
		}
		else if(isAutoAttackable(player))
		{
			// Player with lvl < 21 can't attack a cursed weapon holder
			// And a cursed weapon holder  can't attack players with lvl < 21
			if(_owner.isCursedWeaponEquipped() && player.getLevel() < 21 || player.isCursedWeaponEquipped() && _owner.getLevel() < 21)
				player.sendActionFailed();
			else
				player.getAI().Attack(this, false);
		}
		else if(player != _owner)
			player.getAI().setIntention(CtrlIntention.AI_INTENTION_FOLLOW, this, 100);
		else
			sendActionFailed();
	}

	public L2Character getLastTarget()
	{
		return _lastTarget;
	}

	public void setLastTarget(final L2Character lastTarget)
	{
		_lastTarget = lastTarget;
	}

	public long getExpForThisLevel()
	{
		if(getLevel() >= Experience.LEVEL.length)
			return 0;
		return Experience.LEVEL[getLevel()];
	}

	public long getExpForNextLevel()
	{
		if(getLevel() + 1 >= Experience.LEVEL.length)
			return 0;
		return Experience.LEVEL[getLevel() + 1];
	}

	@Override
	public final L2Player getOwner()
	{
		return _owner;
	}

	@Override
	public int getNpcId()
	{
		return getTemplate().npcId;
	}

	public final long getExp()
	{
		return _exp;
	}

	public final void setExp(final long exp)
	{
		_exp = exp;
	}

	public final int getSp()
	{
		return _sp;
	}

	public void setSp(final int sp)
	{
		_sp = sp;
	}

	public int getMaxLoad()
	{
		return _maxLoad;
	}

	public void setMaxLoad(final int maxLoad)
	{
		_maxLoad = maxLoad;
	}

	public abstract int getCurrentFed();

	public abstract int getMaxFed();

	public void followOwner()
	{
		setFollowStatus(true);
	}

	@Override
	public synchronized void doDie(L2Character killer)
	{
		super.doDie(killer);

		if(killer == null || killer == _owner || killer.getObjectId() == _objectId || isInZoneBattle() || killer.isInZoneBattle())
			return;

		if(killer instanceof L2Summon)
			killer = killer.getOwner();

		if(killer == null || _owner == null)
			return;

		if(killer.isPlayer)
		{
			L2Player pk = (L2Player) killer;

			if(isInZone(ZoneType.Siege))
				return;

			if(_owner.getPvpFlag() > 0 || _owner.atMutualWarWith(pk))
				pk.setPvpKills(pk.getPvpKills() + 1);
			else if((getDuel() == null || getDuel() != pk.getDuel()) && getKarma() <= 0)
			{
				int pkCountMulti = Math.max(pk.getPkKills() / 2, 1);
				pk.increaseKarma(Config.KARMA_MIN_KARMA * pkCountMulti);
			}

			// Send a Server->Client UserInfo packet to attacker with its PvP Kills Counter
			pk.sendChanges();
		}
	}

	public void stopDecay()
	{
		DecayTaskManager.getInstance().cancelDecayTask(this);
	}

	@Override
	public void onDecay()
	{
		deleteMe();
	}

	@Override
	public void broadcastStatusUpdate()
	{
		super.broadcastStatusUpdate();
		if(_owner != null)
		{
			if(isVisible())
				_owner.sendPacket(new PetStatusUpdate(this));
		}
	}

	public void deleteMe()
	{
		stopHpMpRegeneration();
		stopMove();
		giveAllToOwner();
		decayMe();
		detachAI();
		if(_owner != null)
		{
			if(_owner.getParty() != null)
			_owner.sendPacket(new PetDelete(getObjectId(), 2));
			_owner.setPet(null);
		}
		setTarget(null);
		L2World.removeObject(this);
		_owner = null;
	}

	public synchronized void unSummon()
	{
		deleteMe();
	}

	public int getAttackRange()
	{
		return _attackRange;
	}

	public void setAttackRange(int range)
	{
		if(range < 36)
			range = 36;
		_attackRange = range;
	}

	// TODO пока не используется
	Location _lastFollowPosition = null;

	@Override
	public void setFollowStatus(final boolean state)
	{
		_follow = state;
		if(_follow)
		{
			_lastFollowPosition = null;
			getAI().setIntention(CtrlIntention.AI_INTENTION_FOLLOW, _owner, 100);
		}
		else
		{
			_lastFollowPosition = getLoc();
			getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);
		}
	}

	public boolean getFollowStatus()
	{
		return _follow;
	}

	public Location getLastFollowPosition()
	{
		return _lastFollowPosition;
	}

	@Override
	public void updateEffectIcons()
	{
		FastList<L2Player> players = L2World.getAroundPlayers(this);
		for(FastList.Node<L2Player> n = players.head(), end = players.tail(); (n = n.getNext()) != end;)
		{
			L2Player player = n.getValue();
			if(player != _owner)
				player.sendPacket(new NpcInfo(this, player, _showSpawnAnimation));
			else
				player.sendPacket(new PetInfo(this, _showSpawnAnimation));
		}

		PartySpelled ps = new PartySpelled(this, true);

		if(_owner != null)
			if(_owner.getParty() != null)
				_owner.getParty().broadcastToPartyMembers(ps);
			else
				_owner.sendPacket(ps);
	}

	/**
	 * @return Returns the showSpawnAnimation.
	 */
	public boolean isShowSpawnAnimation()
	{
		return _showSpawnAnimation;
	}

	/**
	 * Sets showSpawnAnimation.
	 */
	public void setShowSpawnAnimation(boolean showSpawnAnimation)
	{
		_showSpawnAnimation = showSpawnAnimation;
	}

	public int getControlItemId()
	{
		return 0;
	}

	public L2Weapon getActiveWeapon()
	{
		return null;
	}

	@Override
	public PetInventory getInventory()
	{
		return null;
	}

	@Override
	public void doPickupItem(final L2Object object)
	{}

	public void giveAllToOwner()
	{}

	/**
	 * Return null.<BR><BR>
	 */
	@Override
	public L2ItemInstance getActiveWeaponInstance()
	{
		return null;
	}

	@Override
	public L2Weapon getActiveWeaponItem()
	{
		return null;
	}

	@Override
	public L2ItemInstance getSecondaryWeaponInstance()
	{
		return null;
	}

	@Override
	public L2Weapon getSecondaryWeaponItem()
	{
		return null;
	}

	/**
	 * @return the L2Party object of its L2Player owner or null.<BR><BR>
	 */
	public L2Party getParty()
	{
		return _owner.getParty();
	}

	public boolean isInParty()
	{
		return _owner.getParty() != null;
	}

	public abstract void sendDamageMsg(int damage);

	public abstract void sendCriticalHitMsg();

	@Override
	public boolean unChargeShots(final boolean spirit)
	{
		if(getOwner() == null)
			return false;

		if(spirit && _spsCharged != 0)
		{
			_spsCharged = 0;
			getOwner().AutoShot();
			return true;
		}

		if(_ssCharged)
		{
			_ssCharged = false;
			getOwner().AutoShot();
			return true;
		}

		getOwner().AutoShot();
		return false;
	}

	@Override
	public boolean getChargedSoulShot()
	{
		return _ssCharged;
	}

	@Override
	public int getChargedSpiritShot()
	{
		return _spsCharged;
	}

	public void chargeSoulShot()
	{
		_ssCharged = true;
	}

	public void chargeSpiritShot(final int state)
	{
		_spsCharged = state;
	}

	public int getSoulshotConsumeCount()
	{
		return getLevel() / 27 + 1;
	}

	public int getSpiritshotConsumeCount()
	{
		return getLevel() / 58 + 1;
	}

	@Override
	public void doAttack(final L2Character target)
	{
		super.doAttack(target);

		for(final L2CubicInstance cubic : _owner._cubics.values())
			if(cubic.getId() != L2CubicInstance.LIFE_CUBIC)
				cubic.doAction(target);
	}

	@Override
	public void doCast(final L2Skill skill, final L2Character target, boolean forceUse)
	{
		super.doCast(skill, target, forceUse);

		if(skill.isOffensive() && target != null)
		{
			for(final L2CubicInstance cubic : _owner._cubics.values())
				if(cubic.getId() != L2CubicInstance.LIFE_CUBIC)
					cubic.doAction(target);
		}
	}

	public boolean isPosessed()
	{
		return _posessed;
	}

	public void setPossessed(final boolean possessed)
	{
		_posessed = possessed;
	}

	public boolean isInRange()
	{
		if(_owner == null)
			return false;
		return getDistance(_owner) < SUMMON_DISAPPEAR_RANGE;
	}

	public void teleportToOwner()
	{
		if(_owner == null)
			return;
		stopMove();
		setXYZ(_owner.getX() + Rnd.get(-100, 100), _owner.getY() + Rnd.get(-100, 100), _owner.getZ());
		setFollowStatus(true);
		updateEffectIcons();
	}

	public void broadcastPetInfo()
	{
		// После PetInfo нужно обязательно обновлять иконки бафов (они затираются).
		// Поэтому броадкаст для удобства совмещен с updateEffectIcons()
		updateEffectIcons();
	}

	@Override
	public void startPvPFlag(L2Character target)
	{
		if(_owner == null)
			return;
		_owner.startPvPFlag(target);
	}

	@Override
	public int getPvpFlag()
	{
		if(_owner == null)
			return 0;
		return _owner.getPvpFlag();
	}

	@Override
	public Duel getDuel()
	{
		if(_owner == null)
			return null;
		return _owner.getDuel();
	}

	@Override
	public int getTeam()
	{
		if(_owner == null)
			return 0;
		return _owner.getTeam();
	}

	/**
	 * Делает броадкаст для пета
	 */
	@Override
	public void broadcastUserInfo()
	{
		broadcastPetInfo();
	}
}