package l2r.gameserver.model;

import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;

import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2ItemInstance.ItemLocation;
import l2r.gameserver.tables.ItemTable;
import l2r.util.Log;

public class PcInventory extends Inventory
{

	private final L2Player _owner;

	public PcInventory(L2Player owner)
	{
		_owner = owner;
	}

	@Override
	public L2Player getOwner()
	{
		return _owner;
	}

	@Override
	protected ItemLocation getBaseLocation()
	{
		return ItemLocation.INVENTORY;
	}

	@Override
	protected ItemLocation getEquipLocation()
	{
		return ItemLocation.PAPERDOLL;
	}

	public int getAdena()
	{
		L2ItemInstance _adena = getItemByItemId(57);
		if(_adena == null)
			return 0;
		return _adena.getIntegerLimitedCount();
	}

	/**
	 * Get all augmented items
	 */
	public ArrayList<L2ItemInstance> getAugmentedItems()
	{
		ArrayList<L2ItemInstance> list = new ArrayList<L2ItemInstance>();
		for(L2ItemInstance item : _items)
			if(item != null && item.isAugmented())
				list.add(item);
		return list;
	}

	/**
	 * Добавляет адену игроку.<BR><BR>
	 * @param adena - сколько адены дать
	 * @return L2ItemInstance - новое количество адены
	 */
	public L2ItemInstance addAdena(long adena)
	{
		L2ItemInstance _adena = ItemTable.getInstance().createItem(57, 0, 0, "PcInventory.addAdena");
		_adena.setCount(adena);
		_adena = addItem(_adena);
		Log.LogItem(_owner, Log.Sys_GetItem, _adena);
		return _adena;
	}

	public L2ItemInstance reduceAdena(long adena)
	{
		return destroyItemByItemId(57, adena, true);
	}

	public static int[][] restoreVisibleInventory(int objectId)
	{
		int[][] paperdoll = new int[25][3];
		ThreadConnection con = null;
		FiltredPreparedStatement statement2 = null;
		ResultSet invdata = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement2 = con.prepareStatement("SELECT object_id,item_id,loc_data,enchant_level FROM items WHERE owner_id=? AND loc='PAPERDOLL'");
			statement2.setInt(1, objectId);
			invdata = statement2.executeQuery();

			while(invdata.next())
			{
				int slot = invdata.getInt("loc_data");
				paperdoll[slot][0] = invdata.getInt("object_id");
				paperdoll[slot][1] = invdata.getInt("item_id");
				paperdoll[slot][2] = invdata.getInt("enchant_level");
			}
		}
		catch(Exception e)
		{
			_log.log(Level.WARNING, "could not restore inventory:", e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement2, invdata);
		}
		return paperdoll;
	}

	public boolean validateCapacity(L2ItemInstance item)
	{
		int slots = _items.size();

		if(!(item.isStackable() && findItemByItemId(item.getItemId()) != null))
			slots++;

		return validateCapacity(slots);
	}

	public boolean validateCapacity(List<L2ItemInstance> items)
	{
		int slots = _items.size();

		for(L2ItemInstance item : items)
			if(!(item.isStackable() && findItemByItemId(item.getItemId()) != null))
				slots++;

		return validateCapacity(slots);
	}

	public boolean validateCapacity(int slots)
	{
		return slots <= _owner.getInventoryLimit();
	}

	public short slotsLeft()
	{
		short slots = (short) (_owner.getInventoryLimit() - _items.size());
		return slots > 0 ? slots : 0;
	}

	public boolean validateWeight(L2ItemInstance item)
	{
		long weight = item.getItem().getWeight() * item.getIntegerLimitedCount();
		return validateWeight(weight);
	}

	public boolean validateWeight(long weight)
	{
		return getTotalWeight() + weight <= _owner.getMaxLoad();
	}
}