package l2r.gameserver.model.base;

import l2r.gameserver.model.L2Player;

public final class L2EnchantSkillLearn
{
	// these two build the primary key
	private final int _id;
	private final int _level;

	// not needed, just for easier debug
	private final String _name;
	private final String _type;

	private final int _spCost;
	private final int _baseLvl;
	private final int _minSkillLevel;
	private final long _exp;

	public L2EnchantSkillLearn(int id, int lvl, String name, String type, int minSkillLvl, int baseLvl, int cost)
	{
		_id = id;
		_level = lvl;
		_baseLvl = baseLvl;
		_minSkillLevel = minSkillLvl;
		_name = name.intern();
		_type = type.intern();
		_spCost = cost;
		_exp = cost * 10;
	}

	/**
	 * @return Returns the id.
	 */
	public int getId()
	{
		return _id;
	}

	/**
	 * @return Returns the level.
	 */
	public int getLevel()
	{
		return _level;
	}

	/**
	 * @return Returns the minLevel.
	 */
	public int getBaseLevel()
	{
		return _baseLvl;
	}

	/**
	 * @return Returns the minSkillLevel.
	 */
	public int getMinSkillLevel()
	{
		return _minSkillLevel;
	}

	/**
	 * @return Returns the name.
	 */
	public String getName()
	{
		return _name;
	}

	/**
	 * @return Returns the spCost.
	 */
	public int getSpCost()
	{
		return _spCost;
	}

	public long getExpCost()
	{
		return _exp;
	}

	private static final int[][] _chance = {
	//    76  77  78  79  80  81
			{ 82, 92, 97, 98, 99, 99, 99, 99, 99, 99 }, // 1
			{ 80, 90, 95, 97, 98, 99, 99, 99, 99, 99 }, // 2
			{ 78, 88, 93, 96, 98, 99, 99, 99, 99, 99 }, // 3
			{ 40, 82, 92, 97, 98, 99, 99, 99, 99, 99 }, // 4
			{ 30, 80, 90, 95, 97, 98, 99, 99, 99, 99 }, // 5
			{ 20, 78, 88, 93, 96, 97, 99, 99, 99, 99 }, // 6
			{ 14, 40, 82, 92, 96, 97, 99, 99, 99, 99 }, // 7
			{ 10, 30, 80, 90, 95, 96, 99, 99, 99, 99 }, // 8
			{ 06, 20, 78, 88, 93, 94, 99, 99, 99, 99 }, // 9
			{ 06, 14, 40, 82, 92, 93, 99, 99, 99, 99 }, // 10
			{ 02, 10, 30, 80, 90, 91, 95, 95, 95, 95 }, // 11
			{ 02, 06, 20, 78, 88, 89, 90, 90, 90, 90 }, // 12
			{ 02, 02, 14, 40, 82, 83, 85, 85, 85, 85 }, // 13
			{ 01, 02, 10, 30, 80, 81, 85, 85, 85, 85 }, // 14
			{ 01, 02, 06, 20, 78, 79, 80, 80, 80, 80 }, // 15
			{ 01, 01, 02, 10, 30, 31, 35, 35, 35, 35 }, // 16
			{ 01, 01, 02, 06, 20, 21, 25, 25, 25, 25 }, // 17
			{ 01, 01, 02, 02, 14, 15, 15, 15, 15, 15 }, // 18
			{ 01, 01, 01, 02, 10, 11, 12, 12, 12, 12 }, // 19
			{ 01, 01, 01, 02, 06, 07, 05, 05, 05, 05 }, // 20
			{ 01, 01, 01, 01, 02, 03, 03, 03, 05, 05 }, // 21
			{ 01, 01, 01, 01, 02, 03, 03, 03, 03, 05 }, // 22
			{ 01, 01, 01, 01, 02, 03, 03, 03, 03, 03 }, // 23
			{ 01, 01, 01, 01, 01, 01, 02, 02, 02, 02 }, // 24
			{ 01, 01, 01, 01, 01, 01, 01, 01, 02, 02 }, // 25
			{ 01, 01, 01, 01, 01, 01, 01, 01, 01, 02 }, // 26
			{ 01, 01, 01, 01, 01, 01, 01, 01, 01, 01 }, // 27
			{ 01, 01, 01, 01, 01, 01, 01, 01, 01, 01 }, // 28
			{ 01, 01, 01, 01, 01, 01, 01, 01, 01, 01 }, // 29
			{ 01, 01, 01, 01, 01, 01, 01, 01, 01, 01 }, // 30
	};

	/**
	 * Шанс успешной заточки
	 */
	public int getRate(L2Player ply)
	{
		return _chance[_level % 100][ply.getLevel() - 76];
	}

	public String getType()
	{
		return _type;
	}

	@Override
	public int hashCode()
	{
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + _id;
		result = PRIME * result + _level;
		return result;
	}

	@Override
	public boolean equals(Object obj)
	{
		if(this == obj)
			return true;
		if(obj == null)
			return false;
		if(getClass() != obj.getClass())
			return false;
		if(!(obj instanceof L2EnchantSkillLearn))
			return false;
		L2EnchantSkillLearn other = (L2EnchantSkillLearn) obj;
		return getId() == other.getId() && getLevel() == other.getLevel();
	}
}