package l2r.gameserver.model.entity;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.concurrent.ScheduledFuture;

import javolution.util.FastList;
import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.instancemanager.CHSiegeManager;
import l2r.gameserver.instancemanager.ClanHallManager;
import l2r.gameserver.instancemanager.ZoneManager;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2ClanMember;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Spawn;
import l2r.gameserver.model.L2Zone;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.MapRegionTable;
import l2r.gameserver.tables.NpcTable;
import l2r.gameserver.tables.SpawnTable;
import l2r.gameserver.templates.L2NpcTemplate;
import l2r.util.Rnd;

public abstract class CHSiege
{
	protected int _id = 0;
	private static CHSiege _instance;
	private long _startTime;
	private boolean _progress = false;
	private ArrayList<L2Clan> _attackers = new ArrayList<L2Clan>();
	private boolean _registration;
	private Calendar startDate = null;
	private ArrayList<CHSiegeManager.MonsterLocation> _monsters = new ArrayList<CHSiegeManager.MonsterLocation>();
	private ArrayList<L2Spawn> _spawns = new ArrayList<L2Spawn>();
	@SuppressWarnings("unchecked")
	public ScheduledFuture _starttask = null;
	@SuppressWarnings("unchecked")
	public ScheduledFuture _stoptask = null;
	public int[] _doors;

	public CHSiege()
	{
		_instance = this;
	}

	public static CHSiege getInstance()
	{
		return _instance;
	}

	public void register()
	{
		if(_id > 0)
		{
			if(!CHSiegeManager.getInstance().registerCHSiege(_id, this))
				return;
			spawnMessenger();
			if(getStartTime() > 0)
			{
				long delay = getStartTime() * 1000 - System.currentTimeMillis();
				startTask(delay);
			}
			else
			{
				setInProgress(true);
				spawnMonsters();
				spawnBosses();
			}
		}
	}

	public long getStartTime()
	{
		return _startTime;
	}

	public void setStartTime(long startTime)
	{
		_startTime = startTime;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("UPDATE ch_sieges SET starttime=? where ch_id=?");
			statement.setLong(1, _startTime);
			statement.setInt(2, _id);
			statement.executeUpdate();
		}
		catch(SQLException e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public int getId()
	{
		return _id;
	}

	public boolean isInProgress()
	{
		return _progress;
	}

	public void setInProgress(boolean progress)
	{
		_progress = progress;
	}

	public void addAttacker(L2Clan clan, boolean dbupdate)
	{
		_attackers.add(clan);
		if(!dbupdate)
			return;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("INSERT INTO chs_clans(ch_id,clan_id) VALUES(?,?)");
			statement.setInt(1, _id);
			statement.setInt(2, clan.getClanId());
			statement.executeUpdate();
		}
		catch(Exception e)
		{
			System.out.println("Exception: CHSiege:AddAttacker(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public void removeAttacker(L2Clan clan)
	{
		_attackers.remove(clan);
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM chs_clans WHERE ch_id = ? AND clan_id = ?");
			statement.setInt(1, _id);
			statement.setInt(2, clan.getClanId());
			statement.executeUpdate();
		}
		catch(Exception e)
		{
			System.out.println("Exception: CHSiege:RemoveAttacker(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public ArrayList<L2Clan> getAttackers()
	{
		return _attackers;
	}

	public boolean isRegistered(L2Clan clan)
	{
		return _attackers.contains(clan);
	}

	public void clearAttackers()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM chs_clans WHERE ch_id = ?");
			statement.setInt(1, _id);
			statement.executeUpdate();
		}
		catch(Exception e)
		{
			System.out.println("Exception: DCSiege:clearAttackers(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
		_attackers.clear();
	}

	public boolean isRegistrationAvailable()
	{
		return _registration;
	}

	public void setRegistrationAvailable(boolean registration)
	{
		_registration = registration;
	}

	public long setNewStartTime(boolean update)
	{
		if(startDate == null)
			return 0;
		while(startDate.getTimeInMillis() < System.currentTimeMillis())
			startDate.add(Calendar.DAY_OF_MONTH, 7);
		_startTime = (int) (startDate.getTimeInMillis() / 1000);
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			if(update)
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("UPDATE ch_sieges SET starttime=? where ch_id=?");
				statement.setLong(1, _startTime);
				statement.setInt(2, _id);
				statement.executeUpdate();
			}
			else
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("INSERT INTO ch_sieges(ch_id, starttime) VALUES(?, ?)");
				statement.setInt(1, _id);
				statement.setLong(2, _startTime);
				statement.executeUpdate();
			}
			statement = con.prepareStatement("DELETE FROM chs_clans where ch_id=?");
			statement.setInt(1, _id);
			statement.executeUpdate();
		}
		catch(Exception e)
		{
			System.out.println("Exception: DCSiege:setNewStartTime: " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
		return _startTime;
	}

	public void setStartDate(Calendar c)
	{
		startDate = c;
	}

	public void addMonster(int id, int x, int y, int z)
	{
		_monsters.add(new CHSiegeManager.MonsterLocation(id, x, y, z, 0, 0, 1, 1));
	}

	public void addMonster(int id, int x, int y, int z, int count)
	{
		_monsters.add(new CHSiegeManager.MonsterLocation(id, x, y, z, 0, 0, count, count));
	}

	public void addMonster(int id, int x, int y, int z, int heading, int count)
	{
		_monsters.add(new CHSiegeManager.MonsterLocation(id, x, y, z, heading, 0, count, count));
	}

	public void addMonster(int id, int x, int y, int z, int heading, int countMin, int countMax)
	{
		_monsters.add(new CHSiegeManager.MonsterLocation(id, x, y, z, heading, 0, countMin, countMax));
	}

	public void addMonster(int id, int x, int y, int z, int heading, int locId, int countMin, int countMax)
	{
		_monsters.add(new CHSiegeManager.MonsterLocation(id, x, y, z, heading, locId, countMin, countMax));
	}

	public void spawnMonsters()
	{
		for(CHSiegeManager.MonsterLocation ml : _monsters)
			try
			{
				// Создаем новый темплэйт, чтобы не менять старый
				L2NpcTemplate template = NpcTable.getTemplate(ml.getId());
				L2Spawn sp = new L2Spawn(template);
				if(ml.getLocId() > 0)
					sp.setLocation(ml.getLocId());
				else
				{
					sp.setLocx(ml.getX());
					sp.setLocy(ml.getY());
					sp.setLocz(ml.getZ());
				}
				sp.setHeading(ml.getHeading());
				sp.setAmount(Rnd.get(ml.getCountMin(), ml.getCountMax()));
				int time = Rnd.get(3) + 2;
				sp.setRespawnDelay(time * 60 * 1000);
				sp.setSiegeId(_id);
				SpawnTable.getInstance().addNewSpawn(sp, false, null);
				sp.init();
				_spawns.add(sp);
			}
			catch(ClassNotFoundException e)
			{
				e.printStackTrace();
			}
	}

	public abstract void spawnMessenger();

	public abstract void spawnBosses();

	public void addSpawn(L2Spawn spawn)
	{
		_spawns.add(spawn);
	}

	public void clearSpawns()
	{
		for(L2Spawn sp : _spawns)
		{
			sp.stopRespawn();
			sp.getLastSpawn().deleteMe();
			SpawnTable.getInstance().deleteSpawn(sp, false);
		}
		_spawns.clear();
	}

	public void startTask(long delay)
	{
		_starttask = ThreadPoolManager.getInstance().scheduleGeneral(new startSiege(), delay);
	}

	public void stopTask(L2Player player, long delay)
	{
		_stoptask = ThreadPoolManager.getInstance().scheduleGeneral(new stopSiege(player), delay);
	}

	private class startSiege implements Runnable
	{
		public void run()
		{
			if(getAttackers().isEmpty())
			{
				stopTask(null, 10);
				return;
			}

			if(ClanHallManager.getInstance().getClanHall(_id).getOwnerId() > 0)
				ClanHallManager.getInstance().getClanHall(_id).setOwner(null);
			L2Zone zone = ZoneManager.getInstance().getZoneByIndex(L2Zone.ZoneType.Siege, _id, false);
			teleportPlayers(zone);
			setInProgress(true);
			CHSiegeManager.getInstance().spawnDoors(_doors, true);
			spawnMonsters();
			spawnBosses();
			zone.setActive(true);
			stopTask(null, 60 * 60 * 1000);
			for(L2Clan attacker : getAttackers())
				attacker.broadcastToOnlineMembers(new SystemMessage(SystemMessage.THE_SIEGE_OF_THE_CLAN_HALL_HAS_BEGUN));
		}
	}

	private void teleportPlayers(L2Zone zone)
	{
		List<L2Player> players = new FastList<L2Player>();
		for(L2Player player : getPlayersInZone(zone))
			if(player.getClan() != null && getAttackers().contains(player.getClan()))
				players.add(player);

		for(L2Player player : players)
			if(player != null && !player.isGM())
				player.teleToLocation(zone.getSpawn());
	}

	public List<L2Player> getPlayersInZone(L2Zone zone)
	{
		List<L2Player> players = new FastList<L2Player>();
		for(L2Object object : zone.getObjects())
			if(object.isPlayer)
				players.add((L2Player) object);
		return players;
	}

	public class stopSiege implements Runnable
	{
		private L2Player _player;

		public stopSiege(L2Player player)
		{
			_player = player;
		}

		public void run()
		{
			if(!isInProgress())
				return;

			L2Zone zone = ZoneManager.getInstance().getZoneByIndex(L2Zone.ZoneType.Siege, _id, false);

			if(_player != null)
			{
				//TODO уточнить кол-во поинтов
				_player.getClan().broadcastToOnlineMembers(new SystemMessage(SystemMessage.THE_SIEGE_OF_THE_CLAN_HALL_IS_FINISHED));
				if(ClanHallManager.getInstance().getClanHall(_id).getOwnerId() == 0)
					_player.getClan().broadcastToOnlineMembers(new SystemMessage(SystemMessage.YOUR_CLAN_NEWLY_ACQUIRED_CONTESTED_CLAN_HALL_HAS_ADDED_S1_POINTS_TO_YOUR_CLAN_REPUTATION_SCORE).addNumber(500));
				else
				{
					_player.getClan().broadcastToOnlineMembers(new SystemMessage(SystemMessage.YOUR_CLAN_HAS_CAPTURED_YOUR_OPPONENT_CONTESTED_CLAN_HALL_S1_POINTS_HAVE_BEEN_DEDUCTED_FROM_YOUR_OPPONENT_CLAN_REPUTATION_SCORE).addNumber(500));
					ClanHallManager.getInstance().getClanHall(_id).getOwner().broadcastToOnlineMembers(new SystemMessage(SystemMessage.AN_OPPOSING_CLAN_HAS_CAPTURED_YOUR_CLAN_CONTESTED_CLAN_HALL_S1_POINTS_HAVE_BEEN_DEDUCTED_FROM_YOUR_CLAN_REPUTATION_SCORE).addNumber(300));
					ClanHallManager.getInstance().getClanHall(_id).getOwner().incReputation(-300, false, "CHSiege");
				}
				_player.getClan().incReputation(500, true, "CHSiege");

				for(L2Clan attacker : getAttackers())
					if(_player.getClan() != attacker)
					{
						attacker.broadcastToOnlineMembers(new SystemMessage(SystemMessage.THE_SIEGE_OF_THE_CLAN_HALL_IS_FINISHED));
						attacker.broadcastToOnlineMembers(new SystemMessage(SystemMessage.AFTER_LOSING_THE_CONTESTED_CLAN_HALL_300_POINTS_HAVE_BEEN_DEDUCTED_FROM_YOUR_CLAN_REPUTATION_SCORE));
						attacker.incReputation(-300, false, "CHSiege");
						for(L2ClanMember pl : attacker.getMembers())
							if(pl.getPlayer() != null)
								if(zone.checkIfInZone(pl.getPlayer().getX(), pl.getPlayer().getY()))
									pl.getPlayer().teleToLocation(MapRegionTable.getInstance().getTeleToLocation(pl.getPlayer(), MapRegionTable.TeleportWhereType.ClosestTown));
					}
				ClanHallManager.getInstance().getClanHall(_id).setOwner(_player.getClan());
			}

			CHSiegeManager.getInstance().spawnDoors(_doors, false);
			clearSpawns();

			setNewStartTime(true);
			long delay = getStartTime() * 1000 - System.currentTimeMillis();
			setInProgress(false);
			clearAttackers();
			if(ClanHallManager.getInstance().getClanHall(_id).getOwnerId() != 0)
				addAttacker(ClanHallManager.getInstance().getClanHall(_id).getOwner(), true);
			zone.setActive(false);
			startTask(delay);
		}
	}

	public void startPreparingSiege()
	{
		for(L2Clan attacker : getAttackers())
			attacker.broadcastToOnlineMembers(new SystemMessage(SystemMessage.REGISTRATION_FOR_THE_CLAN_HALL_SIEGE_IS_CLOSED));
		setRegistrationAvailable(false);
		startTask(300);
	}

	public void stop(L2Player player)
	{
		if(!isInProgress())
			return;

		if(player != null)
		{
			//TODO уточнить кол-во поинтов
			player.getClan().broadcastToOnlineMembers(new SystemMessage(SystemMessage.THE_SIEGE_OF_THE_CLAN_HALL_IS_FINISHED));
			if(ClanHallManager.getInstance().getClanHall(_id).getOwnerId() == 0)
				player.getClan().broadcastToOnlineMembers(new SystemMessage(SystemMessage.YOUR_CLAN_NEWLY_ACQUIRED_CONTESTED_CLAN_HALL_HAS_ADDED_S1_POINTS_TO_YOUR_CLAN_REPUTATION_SCORE).addNumber(500));
			else
			{
				player.getClan().broadcastToOnlineMembers(new SystemMessage(SystemMessage.YOUR_CLAN_HAS_CAPTURED_YOUR_OPPONENT_CONTESTED_CLAN_HALL_S1_POINTS_HAVE_BEEN_DEDUCTED_FROM_YOUR_OPPONENT_CLAN_REPUTATION_SCORE).addNumber(500));
				ClanHallManager.getInstance().getClanHall(_id).getOwner().broadcastToOnlineMembers(new SystemMessage(SystemMessage.AN_OPPOSING_CLAN_HAS_CAPTURED_YOUR_CLAN_CONTESTED_CLAN_HALL_S1_POINTS_HAVE_BEEN_DEDUCTED_FROM_YOUR_CLAN_REPUTATION_SCORE).addNumber(300));
				ClanHallManager.getInstance().getClanHall(_id).getOwner().incReputation(-300, false, "CHSiege");
			}
			player.getClan().incReputation(500, true, "CHSiege");

			for(L2Clan attacker : getAttackers())
				if(player.getClan() != attacker)
				{
					attacker.broadcastToOnlineMembers(new SystemMessage(SystemMessage.THE_SIEGE_OF_THE_CLAN_HALL_IS_FINISHED));
					attacker.broadcastToOnlineMembers(new SystemMessage(SystemMessage.AFTER_LOSING_THE_CONTESTED_CLAN_HALL_300_POINTS_HAVE_BEEN_DEDUCTED_FROM_YOUR_CLAN_REPUTATION_SCORE));
					attacker.incReputation(-300, false, "CHSiege");
				}
			ClanHallManager.getInstance().getClanHall(_id).setOwner(player.getClan());
		}

		clearSpawns();

		long delay = 7 * 24 * 60 * 60 * 1000 - System.currentTimeMillis();
		setStartTime(System.currentTimeMillis() + delay);
		setInProgress(false);
		clearAttackers();
		if(ClanHallManager.getInstance().getClanHall(_id).getOwnerId() != 0)
			addAttacker(ClanHallManager.getInstance().getClanHall(_id).getOwner(), true);
	}
}
