package l2r.gameserver.model.entity;

import java.sql.ResultSet;
import java.util.logging.Logger;

import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.gameserver.instancemanager.SiegeGuardManager;
import l2r.gameserver.instancemanager.ZoneManager;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Zone.ZoneType;
import l2r.gameserver.model.entity.siege.SiegeUnit;
import l2r.gameserver.model.entity.siege.fortress.FortressSiege;
import l2r.gameserver.serverpackets.PlaySound;
import l2r.gameserver.serverpackets.PledgeShowInfoUpdate;

public class Fortress extends SiegeUnit
{
	protected static Logger _log = Logger.getLogger(Fortress.class.getName());

	private FortressSiege _siege = null;
	private int _siegeDate = 0;
	private int _lastSiegeDate = 0;

	public Fortress(int fortressId)
	{
		super(fortressId);
	}

	@Override
	public FortressSiege getSiege()
	{
		if(_siege == null)
			_siege = new FortressSiege(this);
		return _siege;
	}

	/**
	 * Возращает дату предстоящей осады
	 * @return дата осады в unixtime
	 */
	public int getSiegeDate()
	{
		return _siegeDate;
	}

	/**
	 * Возвращает дату последней осады
	 * @return дата осады в unixtime
	 */
	@Override
	public int getLastSiegeDate()
	{
		return _lastSiegeDate;
	}

	@Override
	public void setLastSiegeDate(int time)
	{
		_lastSiegeDate = time;
	}

	@Override
	public int getSiegeDayOfWeek()
	{
		return -1;
	}

	@Override
	public int getSiegeHourOfDay()
	{
		return -1;
	}

	@Override
	public void changeOwner(L2Clan clan)
	{
		if(clan != null)
			clan.setHasFortress(getId()); // Set has fortress flag for new owner

		updateOwnerInDB(clan); // Update in database

		SiegeGuardManager.removeMercsFromDb(_id);
		removeUpgrade();

		if(clan != null && getSiege().isInProgress()) // If siege in progress
			getSiege().midVictory(); // Mid victory phase of siege
	}

	@Override
	protected void loadData()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rs = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement("SELECT * FROM `forts` WHERE `id` = ?");
			statement.setInt(1, _id);
			rs = statement.executeQuery();

			while(rs.next())
			{
				_name = rs.getString("name");
				_siegeDate = rs.getInt("siegeDate");
				_lastSiegeDate = rs.getInt("lastSiegeDate");
			}
			DatabaseUtils.closeDatabaseSR(statement, rs);

			statement = con.prepareStatement("SELECT `clan_id` FROM `clan_data` WHERE hasFortress = ?");
			statement.setInt(1, _id);
			rs = statement.executeQuery();

			while(rs.next())
				_ownerId = rs.getInt("clan_id");

			_zone = ZoneManager.getInstance().getZoneByIndex(ZoneType.Fortress, getId(), true);
		}
		catch(Exception e)
		{
			System.out.println("Exception: Fortress.load(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rs);
		}

		if(getOwnerId() == 0) // this should never happen, but one never knows ;)
			return;
	}

	private void updateOwnerInDB(L2Clan clan)
	{
		_ownerId = clan == null ? 0 : clan.getClanId(); // Update owner id property

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("UPDATE clan_data SET hasFortress=0 WHERE hasFortress=? LIMIT 1");
			statement.setInt(1, getId());
			statement.execute();
			DatabaseUtils.closeStatement(statement);
			statement = null;

			if(clan != null)
			{
				statement = con.prepareStatement("UPDATE clan_data SET hasFortress=? WHERE clan_id=? LIMIT 1");
				statement.setInt(1, getId());
				statement.setInt(2, getOwnerId());
				statement.execute();

				clan.broadcastToOnlineMembers(new PledgeShowInfoUpdate(clan));
				clan.broadcastToOnlineMembers(new PlaySound("Siege_Victory"));
			}
		}
		catch(Exception e)
		{
			_log.warning("Exception: updateOwnerInDB(L2Clan clan): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}
}