package l2r.gameserver.model.entity;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javolution.util.FastMap;
import l2r.Config;
import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.PledgeShowInfoUpdate;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.ClanTable;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.templates.L2Item;
import l2r.gameserver.templates.StatsSet;

public class Hero
{
	private static Logger _log = Logger.getLogger(Hero.class.getName());

	private static Hero _instance;
	private static final String GET_HEROES = "SELECT * FROM heroes WHERE played = 1";
	private static final String GET_ALL_HEROES = "SELECT * FROM heroes";
	private static final String UPDATE_ALL = "UPDATE heroes SET played = 0";
	private static final String INSERT_HERO = "INSERT INTO heroes VALUES (?,?,?,?,?)";
	private static final String UPDATE_HERO = "UPDATE heroes SET count = ?, played = ?" + " WHERE char_id = ?";
	private static final String GET_CLAN_ALLY = "SELECT c.clanid, cd.ally_id AS allyId FROM characters AS c LEFT JOIN clan_data AS cd ON c.clanid=cd.clan_id" + " WHERE c.obj_Id = ?";
	private static final String GET_CLAN_NAME = "SELECT clan_name FROM clan_data WHERE clan_id = (SELECT clanid FROM characters WHERE char_name = ?)";

	private static Map<Integer, StatsSet> _heroes;
	private static Map<Integer, StatsSet> _completeHeroes;

	public static final String COUNT = "count";
	public static final String PLAYED = "played";
	public static final String CLAN_NAME = "clan_name";
	public static final String CLAN_CREST = "clan_crest";
	public static final String ALLY_NAME = "ally_name";
	public static final String ALLY_CREST = "ally_crest";

	public static Hero getInstance()
	{
		if(_instance == null)
			_instance = new Hero();
		return _instance;
	}

	public Hero()
	{
		init();
	}

	private void init()
	{
		_heroes = new FastMap<Integer, StatsSet>();
		_completeHeroes = new FastMap<Integer, StatsSet>();

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		FiltredPreparedStatement statement2 = null;
		ResultSet rset = null, rset2 = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement(GET_HEROES);
			statement2 = con.prepareStatement(GET_CLAN_ALLY);
			rset = statement.executeQuery();

			while(rset.next())
			{
				StatsSet hero = new StatsSet();
				int charId = rset.getInt(Olympiad.CHAR_ID);
				hero.set(Olympiad.CHAR_NAME, rset.getString(Olympiad.CHAR_NAME));
				hero.set(Olympiad.CLASS_ID, rset.getInt(Olympiad.CLASS_ID));
				hero.set(COUNT, rset.getInt(COUNT));
				hero.set(PLAYED, rset.getInt(PLAYED));

				statement2.setInt(1, charId);
				rset2 = statement2.executeQuery();

				if(rset2.next())
				{
					int clanId = rset2.getInt("clanid");
					int allyId = rset2.getInt("allyId");

					String clanName = "";
					String allyName = "";
					int clanCrest = 0;
					int allyCrest = 0;

					if(clanId > 0)
					{
						clanName = ClanTable.getInstance().getClan(clanId).getName();
						clanCrest = ClanTable.getInstance().getClan(clanId).getCrestId();
					}

					if(allyId > 0)
					{
						allyName = ClanTable.getInstance().getAlliance(allyId).getAllyName();
						allyCrest = ClanTable.getInstance().getAlliance(allyId).getAllyCrestId();
					}

					hero.set(CLAN_CREST, clanCrest);
					hero.set(CLAN_NAME, clanName);
					hero.set(ALLY_CREST, allyCrest);
					hero.set(ALLY_NAME, allyName);
				}
				DatabaseUtils.closeResultSet(rset2);
				rset2 = null;
				_heroes.put(charId, hero);
			}
			DatabaseUtils.closeDatabaseSR(statement2, rset2);
			DatabaseUtils.closeDatabaseSR(statement, rset);

			statement = con.prepareStatement(GET_ALL_HEROES);
			statement2 = con.prepareStatement(GET_CLAN_ALLY);

			rset = statement.executeQuery();
			while(rset.next())
			{
				StatsSet hero = new StatsSet();
				int charId = rset.getInt(Olympiad.CHAR_ID);
				hero.set(Olympiad.CHAR_NAME, rset.getString(Olympiad.CHAR_NAME));
				hero.set(Olympiad.CLASS_ID, rset.getInt(Olympiad.CLASS_ID));
				hero.set(COUNT, rset.getInt(COUNT));
				hero.set(PLAYED, rset.getInt(PLAYED));

				statement2.setInt(1, charId);
				rset2 = statement2.executeQuery();

				if(rset2.next())
				{
					int clanId = rset2.getInt("clanid");
					int allyId = rset2.getInt("allyId");

					String clanName = "";
					String allyName = "";
					int clanCrest = 0;
					int allyCrest = 0;

					if(clanId > 0)
					{
						L2Clan clan = ClanTable.getInstance().getClan(clanId);
						if(clan != null)
						{
							clanName = clan.getName();
							clanCrest = clan.getCrestId();
						}
						else
							_log.warning("null clan for id:" + clanId);
					}

					if(allyId > 0)
					{
						allyName = ClanTable.getInstance().getAlliance(allyId).getAllyName();
						allyCrest = ClanTable.getInstance().getAlliance(allyId).getAllyCrestId();
					}

					hero.set(CLAN_CREST, clanCrest);
					hero.set(CLAN_NAME, clanName);
					hero.set(ALLY_CREST, allyCrest);
					hero.set(ALLY_NAME, allyName);
				}
				DatabaseUtils.closeResultSet(rset2);
				_completeHeroes.put(charId, hero);
			}
		}
		catch(SQLException e)
		{
			_log.warning("Hero System: Couldnt load Heroes");
		}
		finally
		{
			DatabaseUtils.closeDatabaseSR(statement2, rset2);
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		_log.info("Hero System: Loaded " + _heroes.size() + " Heroes.");
		_log.info("Hero System: Loaded " + _completeHeroes.size() + " all time Heroes.");
	}

	public Map<Integer, StatsSet> getHeroes()
	{
		return _heroes;
	}

	public synchronized void clearHeroes()
	{
		updateHeroes(true);

		if(_heroes.size() != 0)
			for(StatsSet hero : _heroes.values())
			{
				String name = hero.getString(Olympiad.CHAR_NAME);

				L2Player player = L2World.getPlayer(name);

				if(player != null)
				{
					player.setHero(false);

					player.getInventory().unEquipItemInBodySlot(L2Item.SLOT_L_HAND, null);
					player.getInventory().unEquipItemInBodySlot(L2Item.SLOT_R_HAND, null);
					player.getInventory().unEquipItemInBodySlot(L2Item.SLOT_LR_HAND, null);
					player.getInventory().unEquipItemInBodySlot(L2Item.SLOT_HAIR, null);
					player.getInventory().unEquipItemInBodySlot(L2Item.SLOT_HAIRALL, null);
					player.getInventory().unEquipItemInBodySlot(L2Item.SLOT_DHAIR, null);

					for(L2ItemInstance item : player.getInventory().getItems())
					{
						if(item == null)
							continue;
						if(item.isHeroItem())
							player.getInventory().destroyItem(item, 1, true);
					}

					player.broadcastUserInfo();
				}
			}

		_heroes.clear();
	}

	public synchronized boolean computeNewHeroes(List<StatsSet> newHeroes)
	{
		if(newHeroes.size() == 0)
			return true;

		Map<Integer, StatsSet> heroes = new FastMap<Integer, StatsSet>();
		boolean error = false;

		for(StatsSet hero : newHeroes)
		{
			int charId = hero.getInteger(Olympiad.CHAR_ID);

			if(_completeHeroes != null && _completeHeroes.containsKey(charId))
			{
				StatsSet oldHero = _completeHeroes.get(charId);
				int count = oldHero.getInteger(COUNT);
				oldHero.set(COUNT, count + 1);
				oldHero.set(PLAYED, 1);

				heroes.put(charId, oldHero);
			}
			else
			{
				StatsSet newHero = new StatsSet();
				newHero.set(Olympiad.CHAR_NAME, hero.getString(Olympiad.CHAR_NAME));
				newHero.set(Olympiad.CLASS_ID, hero.getInteger(Olympiad.CLASS_ID));
				newHero.set(COUNT, 1);
				newHero.set(PLAYED, 1);

				heroes.put(charId, newHero);
			}
		}

		_heroes.putAll(heroes);
		heroes.clear();

		updateHeroes(false);

		for(StatsSet hero : _heroes.values())
		{
			//StatsSet hero = _heroes.get(heroId);
			String name = hero.getString(Olympiad.CHAR_NAME);

			L2Player player = L2World.getPlayer(name);

			if(player != null)
			{
				player.addSkill(SkillTable.getInstance().getInfo(395, 1));
				player.addSkill(SkillTable.getInstance().getInfo(396, 1));
				player.addSkill(SkillTable.getInstance().getInfo(1374, 1));
				player.addSkill(SkillTable.getInstance().getInfo(1375, 1));
				player.addSkill(SkillTable.getInstance().getInfo(1376, 1));
				player.setHero(true);
				if(player.getClan() != null && player.getClan().getLevel() >= 5)
				{
					player.getClan().incReputation(1000, true, "Hero");
					player.getClan().broadcastToOnlineMembers(new PledgeShowInfoUpdate(player.getClan()));
					player.getClan().broadcastToOtherOnlineMembers(new SystemMessage(SystemMessage.CLAN_MEMBER_S1_WAS_NAMED_A_HERO_2S_POINTS_HAVE_BEEN_ADDED_TO_YOUR_CLAN_REPUTATION_SCORE).addString(name).addNumber(Math.round(1000 * Config.RATE_CLAN_REP_SCORE)), player);
				}
				else
					player.broadcastUserInfo();
			}
			else
			{
				ThreadConnection con = null;
				FiltredPreparedStatement statement = null;
				ResultSet rset = null;

				try
				{
					con = L2DatabaseFactory.getInstance().getConnection();
					statement = con.prepareStatement(GET_CLAN_NAME);
					statement.setString(1, name);
					rset = statement.executeQuery();
					if(rset.next())
					{
						String clanName = rset.getString("clan_name");
						if(clanName != null)
						{
							L2Clan clan = ClanTable.getInstance().getClanByName(clanName);
							if(clan != null)
							{
								clan.incReputation(1000, true, "Hero");
								clan.broadcastToOnlineMembers(new PledgeShowInfoUpdate(clan));
								SystemMessage sm = new SystemMessage(SystemMessage.CLAN_MEMBER_S1_WAS_NAMED_A_HERO_2S_POINTS_HAVE_BEEN_ADDED_TO_YOUR_CLAN_REPUTATION_SCORE);
								sm.addString(name);
								sm.addNumber(Math.round(1000 * Config.RATE_CLAN_REP_SCORE));
								clan.broadcastToOnlineMembers(sm);
							}
						}
					}

					DatabaseUtils.closeDatabaseSR(statement, rset);
				}
				catch(Exception e)
				{
					_log.warning("could not get clan name of " + name + ": " + e);
					error = true;
				}
				finally
				{
					DatabaseUtils.closeDatabaseCSR(con, statement, rset);
				}
			}
		}

		return error;
	}

	public void updateHeroes(boolean setDefault)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			if(setDefault)
			{
				statement = con.prepareStatement(UPDATE_ALL);
				statement.execute();
				return;
			}

			for(Integer heroId : _heroes.keySet())
			{
				StatsSet hero = _heroes.get(heroId);

				if(_completeHeroes == null || !_completeHeroes.containsKey(heroId))
				{
					statement = con.prepareStatement(INSERT_HERO);
					statement.setInt(1, heroId);
					statement.setString(2, hero.getString(Olympiad.CHAR_NAME));
					statement.setInt(3, hero.getInteger(Olympiad.CLASS_ID));
					statement.setInt(4, hero.getInteger(COUNT));
					statement.setInt(5, hero.getInteger(PLAYED));
					statement.execute();

					DatabaseUtils.closeStatement(statement);

					int clanId = 0;
					int allyId = 0;
					if(L2World.findObject(heroId) != null)
					{
						clanId = ((L2Player) L2World.findObject(heroId)).getClanId();
						allyId = ((L2Player) L2World.findObject(heroId)).getAllyId();
					}
					else
					{
						FiltredPreparedStatement statement2 = con.prepareStatement(GET_CLAN_ALLY);
						statement2.setInt(1, heroId);
						ResultSet rset2 = statement2.executeQuery();
						if(rset2.next())
						{
							clanId = rset2.getInt("clanid");
							allyId = rset2.getInt("allyId");
						}
						DatabaseUtils.closeDatabaseSR(statement2, rset2);
					}

					String clanName = "";
					String allyName = "";
					int clanCrest = 0;
					int allyCrest = 0;

					if(clanId > 0)
					{
						clanName = ClanTable.getInstance().getClan(clanId).getName();
						clanCrest = ClanTable.getInstance().getClan(clanId).getCrestId();
					}

					if(allyId > 0)
					{
						allyName = ClanTable.getInstance().getAlliance(allyId).getAllyName();
						allyCrest = ClanTable.getInstance().getAlliance(allyId).getAllyCrestId();
					}

					hero.set(CLAN_CREST, clanCrest);
					hero.set(CLAN_NAME, clanName);
					hero.set(ALLY_CREST, allyCrest);
					hero.set(ALLY_NAME, allyName);

					_heroes.remove(heroId);
					_heroes.put(heroId, hero);

					_completeHeroes.put(heroId, hero);
				}
				else
				{
					statement = con.prepareStatement(UPDATE_HERO);
					statement.setInt(1, hero.getInteger(COUNT));
					statement.setInt(2, hero.getInteger(PLAYED));
					statement.setInt(3, heroId);
					statement.execute();
					DatabaseUtils.closeStatement(statement);
				}
			}
		}
		catch(SQLException e)
		{
			_log.warning("Hero System: Couldnt update Heroes");
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}
}