package l2r.gameserver.model.entity.siege;

import java.sql.ResultSet;

import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.gameserver.model.L2Clan;

public abstract class SiegeDatabase
{
	protected Siege _siege;

	public SiegeDatabase(Siege siege)
	{
		_siege = siege;
	}

	public abstract void saveSiegeDate();

	public void saveLastSiegeDate()
	{}

	/**
	 * Return true if the clan is registered or owner of a castle<BR><BR>
	 * @param clan The L2Clan of the player
	 */
	public static boolean checkIsRegistered(L2Clan clan, int unitid)
	{
		if(clan == null)
			return false;

		if(unitid > 0 && clan.getHasCastle() == unitid)
			return true;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		boolean register = false;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT clan_id FROM siege_clans where clan_id=?" + (unitid == 0 ? "" : " and unit_id=?"));
			statement.setInt(1, clan.getClanId());
			if(unitid != 0)
				statement.setInt(2, unitid);
			rset = statement.executeQuery();
			if(rset.next())
				register = true;
		}
		catch(Exception e)
		{
			System.out.println("Exception: checkIsRegistered(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		return register;
	}

	public void clearSiegeClan()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM siege_clans WHERE unit_id=?");
			statement.setInt(1, _siege.getSiegeUnit().getId());
			statement.execute();
			DatabaseUtils.closeStatement(statement);
			if(_siege.getSiegeUnit().getOwnerId() > 0)
			{
				statement = con.prepareStatement("DELETE FROM siege_clans WHERE clan_id=?");
				statement.setInt(1, _siege.getSiegeUnit().getOwnerId());
				statement.execute();
			}

			_siege.getAttackerClans().clear();
			_siege.getDefenderClans().clear();
			_siege.getDefenderWaitingClans().clear();
		}
		catch(Exception e)
		{
			System.out.println("Exception: clearSiegeClan(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public void clearSiegeWaitingClan()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM siege_clans WHERE unit_id=? and type = 2");
			statement.setInt(1, _siege.getSiegeUnit().getId());
			statement.execute();

			_siege.getDefenderWaitingClans().clear();
		}
		catch(Exception e)
		{
			System.out.println("Exception: clearSiegeWaitingClan(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public void removeSiegeClan(int clanId)
	{
		if(clanId <= 0)
			return;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM siege_clans WHERE unit_id=? and clan_id=?");
			statement.setInt(1, _siege.getSiegeUnit().getId());
			statement.setInt(2, clanId);
			statement.execute();
			loadSiegeClan();
		}
		catch(Exception e)
		{
			System.out.println("Exception: removeSiegeClan(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public void loadSiegeClan()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			_siege.getAttackerClans().clear();
			_siege.getDefenderClans().clear();
			_siege.getDefenderWaitingClans().clear();
			// Add castle owner as defender
			if(_siege.getSiegeUnit().getOwnerId() > 0)
				_siege.addDefender(_siege.getSiegeUnit().getOwnerId(), SiegeClanType.OWNER);
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT clan_id, type FROM siege_clans where unit_id = ?");
			statement.setInt(1, _siege.getSiegeUnit().getId());
			rset = statement.executeQuery();
			int typeId;
			while(rset.next())
			{
				typeId = rset.getInt("type");
				if(typeId == 0)
					_siege.addDefender(rset.getInt("clan_id"), SiegeClanType.DEFENDER);
				else if(typeId == 1)
					_siege.addAttacker(rset.getInt("clan_id"));
				else if(typeId == 2)
					_siege.addDefenderWaiting(rset.getInt("clan_id"));
			}
		}
		catch(Exception e)
		{
			System.out.println("Exception: loadSiegeClan(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public void saveSiegeClan(L2Clan clan, int typeId, @SuppressWarnings("unused")
	boolean isUpdateRegistration)
	{
		if(clan == null || clan.getHasCastle() > 0)
			return;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("REPLACE INTO siege_clans (clan_id,unit_id,type,castle_owner) values (?,?,?,0)");
			statement.setInt(1, clan.getClanId());
			statement.setInt(2, _siege.getSiegeUnit().getId());
			statement.setInt(3, typeId);
			statement.execute();

			if(typeId == 0 || typeId == -1)
				_siege.addDefender(clan.getClanId(), SiegeClanType.DEFENDER);
			else if(typeId == 1)
				_siege.addAttacker(clan.getClanId());
			else if(typeId == 2)
				_siege.addDefenderWaiting(clan.getClanId());
		}
		catch(Exception e)
		{
			System.out.println("Exception: saveSiegeClan: " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}
}