package l2r.gameserver.model.entity.siege.fortress;

import java.util.logging.Logger;

import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.instancemanager.FortressSiegeManager;
import l2r.gameserver.serverpackets.ItemList;
import l2r.gameserver.serverpackets.SkillList;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.tables.SkillTable;
import l2r.util.Location;

public class CombatFlag
{
	protected static Logger _log = Logger.getLogger(CombatFlag.class.getName());

	public L2Player _player = null;
	public int playerId = 0;
	public L2ItemInstance _item = null;
	private Location _location;
	public L2ItemInstance itemInstance;
	public int _itemId;

	@SuppressWarnings("unused")
	private int _heading;
	@SuppressWarnings("unused")
	private int _fortId;

	public CombatFlag(int fort_id, int x, int y, int z, int heading, int item_id)
	{
		_fortId = fort_id;
		_location = new Location(x, y, z, heading);
		_heading = heading;
		_itemId = item_id;
	}

	public synchronized void spawnMe()
	{
		// Init the dropped L2ItemInstance and add it in the world as a visible object at the position where mob was last
		itemInstance = ItemTable.getInstance().createItem(_itemId, 0, 0, "Combat");
		itemInstance.spawnMe(_location);
	}

	public synchronized void unSpawnMe()
	{
		if(_player != null)
			dropIt();
		if(itemInstance != null)
			itemInstance.decayMe();
	}

	public void activate(L2Player player, L2ItemInstance item)
	{
		// Player holding it data
		_player = player;
		playerId = player.getObjectId();
		itemInstance = null;

		// Add skill
		giveSkill();

		// Equip with the weapon
		_item = item;
		_player.getInventory().equipItem(_item);
		SystemMessage sm = new SystemMessage(SystemMessage.YOU_HAVE_EQUIPPED_YOUR_S1);
		sm.addItemName(_item.getItemId());
		_player.sendPacket(sm);

		_player.sendPacket(new ItemList(_player, false));

		// Refresh player stats
		_player.broadcastUserInfo();
		_player.setCombatFlagEquipped(true);
	}

	public void checkPlayer(L2Player player, L2ItemInstance item)
	{
		if(player == null || item == null)
			return;

		if(FortressSiegeManager.getInstance().checkIfCanPickup(player))
			activate(player, item);
		else
		{
			// wtf? how you get it?
			_log.warning("FortressManager: " + player + " tried to obtain " + item + " in wrong way");
			player.getInventory().destroyItem(item, item.getCount(), true);
		}
	}

	public void dropIt()
	{
		// Reset player stats
		_player.setCombatFlagEquipped(false);
		removeSkill();
		_player.getInventory().destroyItem(_item, 1, false);
		_item = null;
		_player.broadcastUserInfo();
		_player = null;
		playerId = 0;
	}

	public void giveSkill()
	{
		_player.addSkill(SkillTable.getInstance().getInfo(3318, 1), false);
		_player.addSkill(SkillTable.getInstance().getInfo(3358, 1), false);
		_player.sendPacket(new SkillList(_player));
	}

	public void removeSkill()
	{
		_player.removeSkill(SkillTable.getInstance().getInfo(3318, 1), false);
		_player.removeSkill(SkillTable.getInstance().getInfo(3358, 1), false);
		_player.sendPacket(new SkillList(_player));
	}
}