package l2r.gameserver.model.entity.siege.fortress;

import java.util.Calendar;
import java.util.concurrent.ScheduledFuture;

import javolution.util.FastList;
import javolution.util.FastMap;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.idfactory.IdFactory;
import l2r.gameserver.instancemanager.FortressSiegeManager;
import l2r.gameserver.instancemanager.MercTicketManager;
import l2r.gameserver.instancemanager.SiegeGuardManager;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.entity.Fortress;
import l2r.gameserver.model.entity.siege.Siege;
import l2r.gameserver.model.entity.siege.SiegeClan;
import l2r.gameserver.model.entity.siege.SiegeClanType;
import l2r.gameserver.model.entity.siege.SiegeEndTask;
import l2r.gameserver.model.entity.siege.SiegeSpawn;
import l2r.gameserver.model.entity.siege.SiegeStartTask;
import l2r.gameserver.model.entity.siege.TeleportWhoType;
import l2r.gameserver.model.instances.L2CommanderInstance;
import l2r.gameserver.model.instances.L2DoorInstance;
import l2r.gameserver.model.instances.L2StaticObjectInstance;
import l2r.gameserver.serverpackets.FortressSiegeInfo;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.ClanTable;
import l2r.gameserver.tables.MapRegionTable;
import l2r.gameserver.tables.NpcTable;

public class FortressSiege extends Siege
{
	private FastList<L2CommanderInstance> _commanders = new FastList<L2CommanderInstance>();
	private FastList<L2StaticObjectInstance> _flagPoles = new FastList<L2StaticObjectInstance>();
	private FastMap<Integer, Integer> _engrave = new FastMap<Integer, Integer>();

	private ScheduledFuture _commanderRespawnTask = null;

	public FortressSiege(Fortress siegeUnit)
	{
		super(siegeUnit);
		_database = new FortressSiegeDatabase(this);
		_siegeGuardManager = new SiegeGuardManager(getSiegeUnit());
	}

	@Override
	public void startSiege()
	{
		if(!_isInProgress)
		{
			if(getAttackerClans().size() <= 0)
			{
				if(getSiegeUnit().getOwnerId() <= 0)
					announceToPlayer(new SystemMessage(SystemMessage.THE_SIEGE_OF_S1_HAS_BEEN_CANCELED_DUE_TO_LACK_OF_INTEREST).addString(getSiegeUnit().getName()), false);
				else
					announceToPlayer(new SystemMessage(SystemMessage.S1S_SIEGE_WAS_CANCELED_BECAUSE_THERE_WERE_NO_CLANS_THAT_PARTICIPATED).addString(getSiegeUnit().getName()), false);
				return;
			}

			getZone().setActive(true);

			_isInProgress = true; // Flag so that same siege instance cannot be started again
			_ownerBeforeStart = getSiegeUnit().getOwnerId();

			_database.loadSiegeClan(); // Load siege clan from db
			updatePlayerSiegeStateFlags(false);
			teleportPlayer(TeleportWhoType.Attacker, MapRegionTable.TeleportWhereType.ClosestTown); // Teleport to the closest town
			teleportPlayer(TeleportWhoType.Spectator, MapRegionTable.TeleportWhereType.ClosestTown); // Teleport to the closest town

			spawnCommanders(); // Spawn commander
			spawnFlags(); // Spawn flags
			getSiegeUnit().spawnDoor(); // Spawn door
			getSiegeGuardManager().spawnSiegeGuard(); // Spawn siege guard
			defendersUpdate(false); // Add defenders to list
			attackersUpdate(false); // Add attackers to list
			MercTicketManager.getInstance().deleteTickets(getSiegeUnit().getId()); // remove the tickets from the ground
			_defenderRespawnPenalty = 0; // Reset respawn delay

			// Schedule a task to prepare auto siege end
			_siegeEndDate = Calendar.getInstance();
			_siegeEndDate.add(Calendar.MINUTE, getSiegeLength());
			ThreadPoolManager.getInstance().scheduleGeneral(new SiegeEndTask(this), 1000); // Prepare auto end task

			announceToPlayer(new SystemMessage(SystemMessage.THE_TEMPORARY_ALLIANCE_OF_THE_CASTLE_ATTACKER_TEAM_IS_IN_EFFECT_IT_WILL_BE_DISSOLVED_WHEN_THE_CASTLE_LORD_IS_REPLACED), false);
			announceToPlayer(new SystemMessage(SystemMessage.THE_SIEGE_OF_S1_HAS_STARTED).addString(getSiegeUnit().getName()), false);
		}
	}

	@Override
	public void midVictory()
	{
		// Если осада закончилась
		if(!isInProgress() || getSiegeUnit().getOwnerId() <= 0)
			return;

		// Поменять местами атакующих и защитников
		for(SiegeClan sc : getDefenderClans().values())
			if(sc != null)
			{
				removeDefender(sc);
				addAttacker(sc);
			}

		SiegeClan sc_newowner = getAttackerClan(getSiegeUnit().getOwnerId());
		removeAttacker(sc_newowner);
		addDefender(sc_newowner, SiegeClanType.OWNER);
		endSiege();
	}

	@Override
	public void endSiege()
	{
		getZone().setActive(false);

		if(isInProgress())
		{
			announceToPlayer(new SystemMessage(SystemMessage.THE_SIEGE_OF_S1_HAS_FINISHED).addString(getSiegeUnit().getName()), false);

			if(getSiegeUnit().getOwnerId() <= 0)
				announceToPlayer(new SystemMessage(SystemMessage.THE_SIEGE_OF_S1_HAS_ENDED_IN_A_DRAW).addString(getSiegeUnit().getName()), false);
			else
			{
				L2Clan oldOwner = null;
				if(_ownerBeforeStart != 0)
					oldOwner = ClanTable.getInstance().getClan(_ownerBeforeStart);
				L2Clan newOwner = ClanTable.getInstance().getClan(getSiegeUnit().getOwnerId());

				if(oldOwner == null)
				{ // castle was taken over from scratch
					if(newOwner.getLevel() >= 5)
						newOwner.broadcastToOnlineMembers(new SystemMessage(SystemMessage.SINCE_YOUR_CLAN_EMERGED_VICTORIOUS_FROM_THE_SIEGE_S1_POINTS_HAVE_BEEN_ADDED_TO_YOUR_CLAN_REPUTATION_SCORE).addNumber(newOwner.incReputation(1000, true, "CastleSiege")));
				}
				else if(newOwner.equals(oldOwner))
				{ // castle was defended
					if(newOwner.getLevel() >= 5)
						newOwner.broadcastToOnlineMembers(new SystemMessage(SystemMessage.SINCE_YOUR_CLAN_EMERGED_VICTORIOUS_FROM_THE_SIEGE_S1_POINTS_HAVE_BEEN_ADDED_TO_YOUR_CLAN_REPUTATION_SCORE).addNumber(newOwner.incReputation(500, true, "CastleSiege")));
				}
				else
				{ // castle was taken over by another clan
					announceToPlayer(new SystemMessage(SystemMessage.CLAN_S1_IS_VICTORIOUS_OVER_S2S_CASTLE_SIEGE).addString(newOwner.getName()).addString(getSiegeUnit().getName()), false);
					if(newOwner.getLevel() >= 5)
						newOwner.broadcastToOnlineMembers(new SystemMessage(SystemMessage.SINCE_YOUR_CLAN_EMERGED_VICTORIOUS_FROM_THE_SIEGE_S1_POINTS_HAVE_BEEN_ADDED_TO_YOUR_CLAN_REPUTATION_SCORE).addNumber(newOwner.incReputation(1000, true, "CastleSiege")));
					if(oldOwner.getLevel() >= 5)
						oldOwner.broadcastToOnlineMembers(new SystemMessage(SystemMessage.YOUR_CLAN_HAS_FAILED_TO_DEFEND_THE_CASTLE_S1_POINTS_HAVE_BEEN_DEDUCTED_FROM_YOUR_CLAN_REPUTATION_SCORE).addNumber(-oldOwner.incReputation(-2000, true, "CastleSiege")));
				}
			}

			removeHeadquarters();
			unSpawnFlags();
			teleportPlayer(TeleportWhoType.Attacker, MapRegionTable.TeleportWhereType.ClosestTown); // Teleport to the closest town
			teleportPlayer(TeleportWhoType.Spectator, MapRegionTable.TeleportWhereType.ClosestTown); // Teleport to the closest town
			removeSiegeSummons();
			_isInProgress = false; // Flag so that siege instance can be started
			updatePlayerSiegeStateFlags(true);
			saveSiege(); // Save castle specific data
			_database.clearSiegeClan(); // Clear siege clan from db
			unspawnCommanders(); // Remove commander from this fort
			getSiegeGuardManager().unspawnSiegeGuard(); // Remove all spawned siege guard from this castle
			SiegeGuardManager.removeMercsFromDb(getSiegeUnit().getId());
			getSiegeUnit().spawnDoor(); // Respawn door to castle
			defendersUpdate(true);
			attackersUpdate(true);
		}
	}

	@Override
	public void Engrave(L2Clan clan, int objId)
	{
		_engrave.put(objId, clan.getClanId());
		if(_engrave.size() >= _flagPoles.size())
		{
			boolean rst = true;
			for(int id : _engrave.values())
				if(id != clan.getClanId())
					rst = false;
			if(rst)
			{
				_engrave.clear();
				getSiegeUnit().changeOwner(clan);
			}
		}
	}

	/**
	 * Start the auto tasks<BR><BR>
	 */
	@Override
	public void startAutoTask()
	{
		correctSiegeDateTime();
		System.out.println("Siege of " + getSiegeUnit().getName() + ": " + _siegeDate.getTime());
		_database.loadSiegeClan();
		// Schedule registration end
		_siegeRegistrationEndDate = Calendar.getInstance();
		_siegeRegistrationEndDate.setTimeInMillis(_siegeDate.getTimeInMillis());
		_siegeRegistrationEndDate.add(Calendar.MINUTE, -10);
		// Schedule siege auto start
		ThreadPoolManager.getInstance().scheduleGeneral(new SiegeStartTask(this), 1000);
	}

	/** Set the date for the next siege. */
	@Override
	protected void setNextSiegeDate()
	{
		if(_siegeDate.getTimeInMillis() < Calendar.getInstance().getTimeInMillis())
		{
			_siegeDate.add(Calendar.HOUR_OF_DAY, 4); // Set next siege date if siege has passed
			_isRegistrationOver = false; // Allow registration for next siege
			if(_siegeDate.getTimeInMillis() < Calendar.getInstance().getTimeInMillis())
				setNextSiegeDate(); // Re-run again if still in the pass
		}
	}

	@Override
	protected void correctSiegeDateTime()
	{
		boolean corrected = false;
		if(_siegeDate.getTimeInMillis() < Calendar.getInstance().getTimeInMillis())
		{
			corrected = true;
			setNextSiegeDate();
		}
		_siegeDate.set(Calendar.MINUTE, 0);
		if(corrected)
			_database.saveSiegeDate();
	}

	/** Display list of registered clans */
	@Override
	public void listRegisterClan(L2Player player)
	{
		player.sendPacket(new FortressSiegeInfo((Fortress) getSiegeUnit()));
	}

	/** Один из командиров убит */
	public void killedCommander(L2CommanderInstance ct)
	{
		_commanders.remove(ct);
		if(_commanders.size() == 1)
			operateGuardDoors(true);
		if(_commanders.size() == 0)
			operateCommandCenterDoors(true);
		if(_commanderRespawnTask == null)
			_commanderRespawnTask = ThreadPoolManager.getInstance().scheduleGeneral(new CommanderRespawnTask(), 600000);
	}

	private class CommanderRespawnTask implements Runnable
	{
		public void run()
		{
			unspawnCommanders();
			spawnCommanders();
			_commanderRespawnTask = null;
		}
	}

	private void unspawnCommanders()
	{
		for(L2CommanderInstance commander : _commanders)
			if(commander != null)
				commander.decayMe();
		_commanders.clear();
	}

	private void spawnCommanders()
	{
		for(SiegeSpawn sp : FortressSiegeManager.getCommanderSpawnList(getSiegeUnit().getId()))
		{
			L2CommanderInstance commander = new L2CommanderInstance(IdFactory.getInstance().getNextId(), NpcTable.getTemplate(sp.getNpcId()));
			commander.setCurrentHpMp(commander.getMaxHp(), commander.getMaxMp());
			commander.setXYZInvisible(sp.getLoc().getX(), sp.getLoc().getY(), GeoClient.getInstance().getHeight(sp.getLoc()));
			commander.setSpawnedLoc(commander.getLoc());
			commander.setHeading(sp.getLoc().h);
			commander.spawnMe();
			_commanders.add(commander);
		}
	}

	public void operateGuardDoors(boolean open)
	{
		for(Integer doorId : FortressSiegeManager.getGuardDoors(getSiegeUnit().getId()))
		{
			L2DoorInstance door = getSiegeUnit().getDoor(doorId);
			if(door != null)
				if(open)
					door.openMe();
				else
					door.closeMe();
		}
	}

	public void operateCommandCenterDoors(boolean open)
	{
		for(Integer doorId : FortressSiegeManager.getCommandCenterDoors(getSiegeUnit().getId()))
		{
			L2DoorInstance door = getSiegeUnit().getDoor(doorId);
			if(door != null)
				if(open)
					door.openMe();
				else
					door.closeMe();
		}
	}

	public void addFlagPole(L2StaticObjectInstance art)
	{
		_flagPoles.add(art);
	}
	
	public final boolean getIsInProgress()
	{
		return _isInProgress;
	}

	private void spawnFlags()
	{
		for(CombatFlag cf : FortressSiegeManager.getFlagsList(getSiegeUnit().getId()))
			if(cf != null)
				cf.spawnMe();
	}

	private void unSpawnFlags()
	{
		for(CombatFlag cf : FortressSiegeManager.getFlagsList(getSiegeUnit().getId()))
			if(cf != null)
				cf.unSpawnMe();
	}
}