package l2r.gameserver.model.instances;

import java.util.concurrent.ScheduledFuture;

import javolution.util.FastList;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Summon;
import l2r.gameserver.model.entity.SevenSigns;
import l2r.gameserver.serverpackets.MagicSkillUse;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.templates.L2NpcTemplate;

public class L2CabaleBufferInstance extends L2NpcInstance
{
	@SuppressWarnings("unchecked")
	ScheduledFuture aiTask;

	class CabalaAI implements Runnable
	{
		L2CabaleBufferInstance _caster;

		CabalaAI(L2CabaleBufferInstance caster)
		{
			_caster = caster;
		}

		public void run()
		{
			int winningCabal = SevenSigns.getInstance().getCabalHighestScore();

			if(winningCabal == SevenSigns.CABAL_NULL)
				return;

			int losingCabal = SevenSigns.CABAL_NULL;

			if(winningCabal == SevenSigns.CABAL_DAWN)
				losingCabal = SevenSigns.CABAL_DUSK;
			else if(winningCabal == SevenSigns.CABAL_DUSK)
				losingCabal = SevenSigns.CABAL_DAWN;

			/**
			 * For each known player in range, cast either the positive or negative buff.
			 * <BR>
			 * The stats affected depend on the player type, either a fighter or a mystic.
			 * <BR>
			 * Curse of Destruction (Loser)
			 *  - Fighters: -25% Accuracy, -25% Effect Resistance
			 *  - Mystics: -25% Casting Speed, -25% Effect Resistance
			 *
			 * Blessing of Prophecy (Winner)
			 *  - Fighters: +25% Max Load, +25% Effect Resistance
			 *  - Mystics: +25% Magic Cancel Resist, +25% Effect Resistance
			 */
			FastList<L2Player> players = getAroundPlayers(900);
			for(FastList.Node<L2Player> n = players.head(), end = players.tail(); (n = n.getNext()) != end;)
			{
				L2Player player = n.getValue();
				int playerCabal = SevenSigns.getInstance().getPlayerCabal(player);
				if(playerCabal == winningCabal && _caster.getNpcId() == SevenSigns.ORATOR_NPC_ID)
					if(player.isMageClass())
						handleCast(player, 4365);
					else
						handleCast(player, 4364);
				else if(playerCabal == losingCabal && _caster.getNpcId() == SevenSigns.PREACHER_NPC_ID)
					if(player.isMageClass())
						handleCast(player, 4362);
					else
						handleCast(player, 4361);
			}
		}

		private void handleCast(L2Player player, int skillId)
		{
			L2Skill skill = SkillTable.getInstance().getInfo(skillId, 2);
			if(player.getEffectsBySkill(skill) == null && GeoClient.getInstance().canSeeTarget(_caster, player))
			{
				skill.getEffects(_caster, player, false);
				broadcastPacket(new MagicSkillUse(_caster, player, skill.getId(), 2, skill.getHitTime(), 0));
			}

			L2Summon summon = player.getPet();
			if(summon != null && summon.getEffectsBySkill(skill) == null && summon.isInRangeZ(_caster, 900) && GeoClient.getInstance().canSeeTarget(_caster, summon))
			{
				skill.getEffects(_caster, summon, false);
				broadcastPacket(new MagicSkillUse(_caster, summon, skill.getId(), 2, skill.getHitTime(), 0));
			}
		}
	}

	public L2CabaleBufferInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
		aiTask = ThreadPoolManager.getInstance().scheduleAiAtFixedRate(new CabalaAI(this), 3000, 3000);
	}

	@Override
	public void deleteMe()
	{
		if(aiTask != null)
			aiTask.cancel(true);

		super.deleteMe();
	}

	@Override
	public boolean hasRandomAnimation()
	{
		return false;
	}
}