package l2r.gameserver.model.instances;

import javolution.util.FastList;
import l2r.Config;
import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.ai.CtrlEvent;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.base.ItemToDrop;
import l2r.gameserver.tables.NpcTable;
import l2r.gameserver.templates.L2NpcTemplate;
import l2r.util.Rnd;

public class L2ChestInstance extends L2MonsterInstance
{
	private boolean _fake;

	public L2ChestInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public void onSpawn()
	{
		_fake = !Rnd.chance(Config.ALT_TRUE_CHESTS);
		super.onSpawn();
	}

	public void onOpen(L2Player opener)
	{
		if(_fake)
		{
			opener.sendMessage(new CustomMessage("l2r.gameserver.model.instances.L2ChestInstance.Fake", opener));
			getAI().notifyEvent(CtrlEvent.EVT_ATTACKED, opener, 100);
		}
		else
		{
			setSpoiled(false);
			if(NpcTable.getTemplate(getNpcId() - 3536) != null && NpcTable.getTemplate(getNpcId() - 3536).getDropData() != null)
			{
				final FastList<ItemToDrop> drops = NpcTable.getTemplate(getNpcId() - 3536).getDropData().rollDrop(0, false, opener);
				for(final ItemToDrop drop : drops)
					dropItem(opener, drop.itemId, drop.count);
			}
			doDie(opener);
		}
	}

	@Override
	public void reduceCurrentHp(final double damage, final L2Character attacker, final boolean awake, final boolean standUp, boolean directHp)
	{
		if(_fake)
			super.reduceCurrentHp(damage, attacker, awake, standUp, directHp);
		else
			doDie(attacker);
	}

	public boolean isFake()
	{
		return _fake;
	}
}