package l2r.gameserver.model.instances;

import java.util.concurrent.ScheduledFuture;
import java.util.logging.Logger;

import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.serverpackets.MyTargetSelected;
import l2r.gameserver.tables.NpcTable;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.templates.L2NpcTemplate;

public class L2DecoyInstance extends L2NpcInstance
{
	protected static final Logger log = Logger.getLogger(L2DecoyInstance.class.getName());

	private L2Player _owner;

	private int _lifeTime;
	private int _timeRemaining;

	private ScheduledFuture _decoyLifeTask;
	private ScheduledFuture _hateSpam;

	public L2DecoyInstance(int objectId, L2NpcTemplate template, L2Player owner, int lifeTime)
	{
		super(objectId, template);

		_owner = owner;
		_lifeTime = lifeTime;
		_timeRemaining = _lifeTime;
		int skilllevel = getNpcId() - 13070;
		_decoyLifeTask = ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new DecoyLifetime(), 1000, 1000);
		_hateSpam = ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new HateSpam(SkillTable.getInstance().getInfo(5272, skilllevel)), 2000, 5000);
	}

	@Override
	public void doDie(L2Character killer)
	{
		super.doDie(killer);
		if(_hateSpam != null)
		{
			_hateSpam.cancel(true);
			_hateSpam = null;
		}
		_lifeTime = 0;
	}

	class DecoyLifetime implements Runnable
	{
		public void run()
		{
			try
			{
				double newTimeRemaining;
				decTimeRemaining(1000);
				newTimeRemaining = getTimeRemaining();
				if(newTimeRemaining < 0)
					unSummon();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	class HateSpam implements Runnable
	{
		private L2Skill _skill;

		HateSpam(L2Skill skill)
		{
			_skill = skill;
		}

		public void run()
		{
			try
			{
				setTarget(L2DecoyInstance.this);
				doCast(_skill, L2DecoyInstance.this, true);
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	public void unSummon()
	{
		if(_decoyLifeTask != null)
		{
			_decoyLifeTask.cancel(true);
			_decoyLifeTask = null;
		}
		if(_hateSpam != null)
		{
			_hateSpam.cancel(true);
			_hateSpam = null;
		}
		deleteMe();
	}

	public void decTimeRemaining(int value)
	{
		_timeRemaining -= value;
	}

	public int getTimeRemaining()
	{
		return _timeRemaining;
	}

	public int getLifeTime()
	{
		return _lifeTime;
	}

	@Override
	public L2Player getOwner()
	{
		return _owner;
	}

	@Override
	public boolean isAutoAttackable(L2Character attacker)
	{
		return attacker != _owner && _owner.isAutoAttackable(attacker);
	}

	@Override
	public void deleteMe()
	{
		decayMe();
		_owner.setDecoy(null);
	}

	@Override
	public void onAction(L2Player player)
	{
		if(player.isConfused() || player.isBlocked())
		{
			player.sendActionFailed();
			return;
		}

		if(player.getTarget() != this)
		{
			player.setTarget(this);
			player.sendPacket(new MyTargetSelected(getObjectId(), 0));
		}
		else if(isAutoAttackable(player))
			player.getAI().Attack(this, false);
		else
			sendActionFailed();
	}

	@Override
	public float getColRadius()
	{
		return getOwner().getBaseTemplate().collisionRadius;
	}

	@Override
	public float getColHeight()
	{
		return getOwner().getBaseTemplate().collisionHeight;
	}
}