package l2r.gameserver.model.instances;

import java.util.StringTokenizer;
import java.util.logging.Logger;

import l2r.Config;
import l2r.gameserver.TradeController;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2TradeList;
import l2r.gameserver.model.entity.SevenSigns;
import l2r.gameserver.serverpackets.BuyList;
import l2r.gameserver.serverpackets.NpcHtmlMessage;
import l2r.gameserver.templates.L2NpcTemplate;

public final class L2MercManagerInstance extends L2NpcInstance
{
	private static Logger _log = Logger.getLogger(L2MercManagerInstance.class.getName());

	private static int Cond_All_False = 0;
	private static int Cond_Busy_Because_Of_Siege = 1;
	private static int Cond_Owner = 2;

	public L2MercManagerInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public void onBypassFeedback(L2Player player, String command)
	{
		player.sendActionFailed();

		if(!isInRange(player, INTERACTION_DISTANCE))
			return;

		int condition = validateCondition(player);
		if(condition <= Cond_All_False || condition == Cond_Busy_Because_Of_Siege)
			return;

		if(condition == Cond_Owner)
		{
			StringTokenizer st = new StringTokenizer(command, " ");
			String actualCommand = st.nextToken(); // Get actual command

			String val = "";
			if(st.countTokens() >= 1)
				val = st.nextToken();

			if(actualCommand.equalsIgnoreCase("hire"))
			{
				if(val.equals(""))
					return;

				showBuyWindow(player, Integer.parseInt(val));
			}
			else
				super.onBypassFeedback(player, command);
		}
	}

	private void showBuyWindow(L2Player player, int val)
	{
		player.tempInvetoryDisable();
		if(Config.DEBUG)
			_log.fine("Showing buylist");
		L2TradeList list = TradeController.getInstance().getBuyList(val);
		if(list != null && list.getNpcId().equals(String.valueOf(getNpcId())))
		{
			BuyList bl = new BuyList(list, player.getAdena(), 0);
			player.sendPacket(bl);
		}
		else
		{
			_log.warning("possible client hacker: " + player.getName() + " attempting to buy from GM shop! < Ban him!");
			_log.warning("buylist id:" + val);
		}
	}

	@Override
	public void showChatWindow(L2Player player, int val)
	{
		String filename = "data/html/mercmanager/mercmanager-no.htm";
		int condition = validateCondition(player);
		if(condition == Cond_Busy_Because_Of_Siege)
			filename = "data/html/mercmanager/mercmanager-busy.htm"; // Busy because of siege
		else if(condition == Cond_Owner)
			if(SevenSigns.getInstance().getCurrentPeriod() == SevenSigns.PERIOD_SEAL_VALIDATION)
			{
				if(SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_STRIFE) == SevenSigns.CABAL_DAWN)
					filename = "data/html/mercmanager/mercmanager_dawn.htm";
				else if(SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_STRIFE) == SevenSigns.CABAL_DUSK)
					filename = "data/html/mercmanager/mercmanager_dusk.htm";
				else
					filename = "data/html/mercmanager/mercmanager.htm";
			}
			else
				filename = "data/html/mercmanager/mercmanager_nohire.htm";
		player.sendPacket(new NpcHtmlMessage(player, this, filename, val));
	}

	private int validateCondition(L2Player player)
	{
		if(getCastle() != null && getCastle().getId() > 0)
			if(player.getClan() != null)
				if(getCastle().getSiege().isInProgress())
					return Cond_Busy_Because_Of_Siege; // Busy because of siege
				else if(getCastle().getOwnerId() == player.getClanId() // Clan owns castle
						&& (player.getClanPrivileges() & L2Clan.CP_CS_MERCENARIES) == L2Clan.CP_CS_MERCENARIES) // has merc rights
					return Cond_Owner; // Owner

		return Cond_All_False;
	}
}