package l2r.gameserver.model.instances;

import java.io.File;
import java.util.StringTokenizer;
import java.util.logging.Logger;

import l2r.Config;
import l2r.gameserver.TradeController;
import l2r.gameserver.model.L2Multisell;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2TradeList;
import l2r.gameserver.serverpackets.BuyList;
import l2r.gameserver.serverpackets.SellList;
import l2r.gameserver.serverpackets.ShopPreviewList;
import l2r.gameserver.templates.L2NpcTemplate;

@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public class L2MerchantInstance extends L2NpcInstance
{
	private static Logger _log = Logger.getLogger(L2MerchantInstance.class.getName());

	public L2MerchantInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public String getHtmlPath(int npcId, int val)
	{
		String pom;
		if(val == 0)
			pom = "" + npcId;
		else
			pom = npcId + "-" + val;
		File file = new File(Config.DATAPACK_ROOT, "data/html/merchant/" + pom + ".htm");
		if(file.exists())
			return "data/html/merchant/" + pom + ".htm";
		return "data/html/teleporter/" + pom + ".htm";
	}

	private void showWearWindow(L2Player player, int val)
	{
		if(!player.getPlayerAccess().UseShop)
			return;

		player.tempInvetoryDisable();
		if(Config.DEBUG)
			_log.fine("Showing wearlist");
		L2TradeList list = TradeController.getInstance().getBuyList(val);

		if(list != null)
		{
			ShopPreviewList bl = new ShopPreviewList(list, player.getAdena(), player.expertiseIndex);
			player.sendPacket(bl);
		}
		else
		{
			_log.warning("no buylist with id:" + val);
			player.sendActionFailed();
		}
	}

	protected void showBuyWindow(L2Player player, int val)
	{
		if(!player.getPlayerAccess().UseShop)
			return;

		double taxRate = 0;

		if(getCastle() != null)
			taxRate = getCastle().getTaxRate();

		player.tempInvetoryDisable();
		if(Config.DEBUG)
			_log.fine("Showing buylist");
		L2TradeList list = TradeController.getInstance().getBuyList(val);
		if(list != null && list.getNpcId().equals(String.valueOf(getNpcId())))
		{
			BuyList bl = new BuyList(list, player.getAdena(), taxRate);
			player.sendPacket(bl);
		}
		else
		{
			_log.warning("possible client hacker: " + player.getName() + " attempting to buy from GM shop! < Ban him!");
			_log.warning("buylist id:" + val);
		}
	}

	private void showSellWindow(L2Player player)
	{
		if(!player.getPlayerAccess().UseShop)
			return;

		if(Config.DEBUG)
			_log.fine("Showing selllist");
		SellList sl = new SellList(player);
		player.sendPacket(sl);
	}

	@Override
	public void onBypassFeedback(L2Player player, String command)
	{
		StringTokenizer st = new StringTokenizer(command, " ");
		String actualCommand = st.nextToken(); // Get actual command

		if(actualCommand.equalsIgnoreCase("Buy"))
		{
			if(st.countTokens() < 1)
				return;
			int val = Integer.parseInt(st.nextToken());
			showBuyWindow(player, val);
		}
		else if(actualCommand.equalsIgnoreCase("Sell"))
			showSellWindow(player);
		else if(actualCommand.equalsIgnoreCase("Wear") && Config.WEAR_TEST_ENABLED)
		{
			if(st.countTokens() < 1)
				return;
			int val = Integer.parseInt(st.nextToken());
			showWearWindow(player, val);
		}
		else if(actualCommand.equalsIgnoreCase("Multisell"))
		{
			if(st.countTokens() < 1)
				return;
			int val = Integer.parseInt(st.nextToken());
			L2Multisell.getInstance().SeparateAndSend(val, player, getCastle() != null ? getCastle().getTaxRate() : 0);
		}
		else
			super.onBypassFeedback(player, command);
	}
}
