package l2r.gameserver.model.instances;

import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_ACTIVE;
import static l2r.gameserver.ai.CtrlIntention.AI_INTENTION_ATTACK;

import java.io.File;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Logger;

import javolution.text.TextBuilder;
import javolution.util.FastList;
import l2r.gameserver.instancemanager.ClanHallManager;
import l2r.gameserver.instancemanager.FortressManager;
import l2r.gameserver.model.entity.ClanHall;
import l2r.gameserver.model.entity.Fortress;
import l2r.Config;
import l2r.extensions.multilang.CustomMessage;
import l2r.extensions.scripts.Events;
import l2r.extensions.scripts.Functions;
import l2r.extensions.scripts.Scripts;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.ai.CtrlEvent;
import l2r.gameserver.ai.CtrlIntention;
import l2r.gameserver.ai.L2CharacterAI;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.idfactory.IdFactory;
import l2r.gameserver.instancemanager.DimensionalRiftManager;
import l2r.gameserver.instancemanager.QuestManager;
import l2r.gameserver.instancemanager.TownManager;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Multisell;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2SkillLearn;
import l2r.gameserver.model.L2Spawn;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.L2WorldRegion;
import l2r.gameserver.model.L2Zone.ZoneType;
import l2r.gameserver.model.base.ClassId;
import l2r.gameserver.model.entity.Castle;
import l2r.gameserver.model.entity.Olympiad;
import l2r.gameserver.model.entity.SevenSigns;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.gameserver.serverpackets.AcquireSkillList;
import l2r.gameserver.serverpackets.ExEnchantSkillList;
import l2r.gameserver.serverpackets.MagicSkillUse;
import l2r.gameserver.serverpackets.MyTargetSelected;
import l2r.gameserver.serverpackets.NpcHtmlMessage;
import l2r.gameserver.serverpackets.NpcInfo;
import l2r.gameserver.serverpackets.RadarControl;
import l2r.gameserver.serverpackets.SocialAction;
import l2r.gameserver.serverpackets.StatusUpdate;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.serverpackets.ValidateLocation;
import l2r.gameserver.serverpackets.ExEnchantSkillList.EnchantSkillType;
import l2r.gameserver.skills.Stats;
import l2r.gameserver.tables.ClanTable;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.tables.SkillTreeTable;
import l2r.gameserver.taskmanager.DecayTaskManager;
import l2r.gameserver.templates.L2Item;
import l2r.gameserver.templates.L2NpcTemplate;
import l2r.gameserver.templates.L2Weapon;
import l2r.util.Location;
import l2r.util.Log;
import l2r.util.Rnd;

public class L2NpcInstance extends L2Character
{
	public class NotifyFaction implements Runnable
	{
		private final L2Character _attacker;

		NotifyFaction(L2Character attacker)
		{
			_attacker = attacker;
		}

		public void run()
		{
			if(_attacker == null)
				return;
			try
			{
				String faction_id = getFactionId();
				if(faction_id == null)
					return;
				for(L2NpcInstance npc : ActiveFriendTargets(true, false))
				{
					if(npc == null || !npc.getAI().isGlobalAggro() || !faction_id.equalsIgnoreCase(npc.getFactionId()))
						continue;
					if(npc.getName().contains("Lilim") && !getName().contains("Lilim") || npc.getName().contains("Nephilim") && !getName().contains("Nephilim") || npc.getName().contains("Lith") && !getName().contains("Lith") || npc.getName().contains("Gigant") && !getName().contains("Gigant"))
						continue;
					if(Math.abs(_attacker.getZ() - npc.getZ()) > 400)
						continue;
					if(!isInRange(npc, npc.getFactionRange()))
						continue;
					if(!GeoClient.getInstance().canSeeTarget(L2NpcInstance.this, npc))
						continue;
					npc.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, _attacker, 1);
				}
			}
			catch(Throwable t)
			{
				t.printStackTrace();
			}
		}
	}

	private long lastFactionNotifyTime = 0;
	private int _personalAggroRange = -1;

	private int _currentLHandId;
	private int _currentRHandId;

	private double _currentCollisionRadius;
	private double _currentCollisionHeight;

	/** Нужно для отображения анимайии спауна, используется в пакете NpcInfo **/
	private boolean _showSpawnAnimation = true;

	public void callFriends(L2Character attacker)
	{
		if(isMonster)
		{
			L2MonsterInstance master = (L2MonsterInstance) this;
			if(this instanceof L2MinionInstance)
			{
				master = ((L2MinionInstance) this).getLeader();
				if(!master.isInCombat() && !master.isDead())
					master.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, attacker, Rnd.get(1, 100));
			}
			else
				master.callMinionsToAssist(attacker);
		}

		// call friend's
		if(System.currentTimeMillis() - lastFactionNotifyTime > 10000)
		{
			ThreadPoolManager.getInstance().scheduleAi(new NotifyFaction(attacker), 100, false);
			lastFactionNotifyTime = System.currentTimeMillis();
		}
	}

	public FastList<L2NpcInstance> ActiveFriendTargets(boolean active, boolean attack)
	{
		FastList<L2NpcInstance> ActiveFriends = new FastList<L2NpcInstance>();
		L2WorldRegion region = L2World.getRegion(this);
		if(region != null && region.getObjectsSize() > 0)
			for(L2NpcInstance obj : region.getNpcsList(getReflection()))
				if(obj != null)
					if(obj.hasAI())
						if(!obj.isDead() && (attack && obj.getAI().getIntention() == AI_INTENTION_ATTACK || active && obj.getAI().getIntention() == AI_INTENTION_ACTIVE))
							if(GeoClient.getInstance().canSeeTarget(this, obj))
								ActiveFriends.add(obj);
		return ActiveFriends;
	}

	private static final Logger _log = Logger.getLogger(L2NpcInstance.class.getName());
	private final ClassId[] _classesToTeach;

	/** The delay after witch the attacked is stopped */
	private long _attack_timeout;
	private Location _spawnedLoc = new Location(0, 0, 0);

	@SuppressWarnings("unchecked")
	private Constructor _ai_constructor;

	@Override
	public synchronized L2CharacterAI getAI()
	{
		if(_ai == null)
		{
			if(_ai_constructor != null)
				try
				{
					_ai = (L2CharacterAI) _ai_constructor.newInstance(new Object[] { this });
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			if(_ai == null)
				_ai = new L2CharacterAI(this);
		}
		return _ai;
	}

	public void setAttackTimeout(long time)
	{
		_attack_timeout = time;
	}

	public long getAttackTimeout()
	{
		return _attack_timeout;
	}

	/**
	 * Return the position of the spawned point.<BR><BR>
	 */
	public Location getSpawnedLoc()
	{
		return _spawnedLoc;
	}

	public void setSpawnedLoc(Location loc)
	{
		_spawnedLoc = loc;
	}

	public L2NpcInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);

		if(template == null)
		{
			_log.warning("No template for Npc. Please check your datapack is setup correctly.");
			throw new IllegalArgumentException();
		}

		_classesToTeach = template.getTeachInfo();
		setName(template.name);
		setTitle(template.title);

		String implementationName = template.ai_type;

		try
		{
			if(!implementationName.equalsIgnoreCase("npc"))
				_ai_constructor = Class.forName("l2r.gameserver.ai." + implementationName).getConstructors()[0];
		}
		catch(Exception e)
		{
			try
			{
				_ai_constructor = Scripts.getInstance().getClasses().get("ai." + implementationName).getRawClass().getConstructors()[0];
			}
			catch(Exception e1)
			{
				_log.warning("AI type " + template.ai_type + " not found!");
				e1.printStackTrace();
			}
		}

		// Т.к. у нас не создаются инстансы при каждом спавне, то все ок.
		if(hasRandomAnimation() && !(this instanceof L2MonsterInstance)) // для монстров через AI
			startRandomAnimation();

		// инициализация параметров оружия
		_currentLHandId = getTemplate().lhand;
		_currentRHandId = getTemplate().rhand;
		// инициализация коллизий
		_currentCollisionHeight = getTemplate().collisionHeight;
		_currentCollisionRadius = getTemplate().collisionRadius;
	}

	public int getRightHandItem()
	{
		return _currentRHandId;
	}

	public int getLeftHandItem()
	{
		return _currentLHandId;
	}

	public void setLHandId(int newWeaponId)
	{
		_currentLHandId = newWeaponId;
	}

	public void setRHandId(int newWeaponId)
	{
		_currentRHandId = newWeaponId;
	}

	/**
	 * @return Returns the zOffset.
	 */
	public double getCollisionHeight()
	{
		return _currentCollisionHeight;
	}

	/**
	 * @param offset The zOffset to set.
	 */
	public void setCollisionHeight(double offset)
	{
		_currentCollisionHeight = offset;
	}

	/**
	 * @return Returns the collisionRadius.
	 */
	public double getCollisionRadius()
	{
		return _currentCollisionRadius;
	}

	/**
	 * @param collisionRadius The collisionRadius to set.
	 */
	public void setCollisionRadius(double collisionRadius)
	{
		_currentCollisionRadius = collisionRadius;
	}

	/**
	 * Kill the L2NpcInstance (the corpse disappeared after 7 seconds), distribute rewards (EXP, SP, Drops...) and notify Quest Engine.<BR><BR>
	 */
	@Override
	public void doDie(L2Character killer)
	{
		setDecayed(false);
		DecayTaskManager.getInstance().addDecayTask(this);

		// установка параметров оружия и коллизий по умолчанию
		_currentLHandId = getTemplate().lhand;
		_currentRHandId = getTemplate().rhand;
		_currentCollisionHeight = getTemplate().collisionHeight;
		_currentCollisionRadius = getTemplate().collisionRadius;

		super.doDie(killer);
	}

	public class AggroInfo
	{
		public L2Playable attacker;
		public int hate;
		public int damage;

		AggroInfo(L2Playable attacker)
		{
			this.attacker = attacker;
		}
	}

	public ConcurrentHashMap<L2Playable, AggroInfo> getAggroList()
	{
		ConcurrentHashMap<L2Playable, AggroInfo> temp = new ConcurrentHashMap<L2Playable, AggroInfo>();
		for(L2Playable playable : L2World.getAroundPlayables(this))
			if(playable != null)
			{
				HateInfo hateInfo = playable.getHateList().get(this);
				if(hateInfo != null)
				{
					AggroInfo aggroInfo = new AggroInfo(playable);
					aggroInfo.hate = hateInfo.hate;
					aggroInfo.damage = hateInfo.damage;
					temp.put(playable, aggroInfo);
				}
			}
		return temp;
	}

	public int getAggroListSize()
	{
		return getAggroList().size();
	}

	public void clearAggroList()
	{
		for(L2Playable playable : L2World.getAroundPlayables(this))
			if(playable != null)
				playable.removeFromHatelist(this);
	}

	public L2Character getMostHated()
	{
		L2Character target = getAI().getAttackTarget();
		if(target != null && target.isNpc && target.isVisible() && !target.isDead() && target.isInRange(this, 2000))
			return target;

		ConcurrentHashMap<L2Playable, AggroInfo> aggroList = getAggroList();

		ConcurrentHashMap<L2Playable, AggroInfo> activeList = new ConcurrentHashMap<L2Playable, AggroInfo>();
		ConcurrentHashMap<L2Playable, AggroInfo> passiveList = new ConcurrentHashMap<L2Playable, AggroInfo>();

		for(AggroInfo ai : aggroList.values())
		{
			L2Playable cha = ai.attacker;
			if(cha != null)
				if(cha.isStunned() || cha.isSleeping() || cha.isParalyzed() || isAfraid() || isBlocked())
					passiveList.put(cha, ai);
				else
					activeList.put(cha, ai);
		}

		if(activeList.size() > 0)
			aggroList = activeList;
		else
			aggroList = passiveList;

		AggroInfo mosthated = null;

		for(AggroInfo ai : aggroList.values())
			if(mosthated == null)
				mosthated = ai;
			else if(mosthated.hate < ai.hate)
				mosthated = ai;

		return mosthated != null && mosthated.hate > 0 ? mosthated.attacker : null;
	}

	public boolean noTarget()
	{
		return getAggroList().size() == 0;
	}

	public void dropItem(L2Player lastAttacker, int itemId, int itemCount)
	{
		if(itemCount == 0 || lastAttacker == null)
			return;

		if(Config.DROP_COUNTER)
			lastAttacker.incrementDropCounter(itemId, itemCount);

		L2ItemInstance item;

		for(int i = 1; i <= itemCount; i++)
		{
			item = ItemTable.getInstance().createItem(itemId, lastAttacker.getObjectId(), getNpcId(), "L2NpcInstance.dropItem");

			// Set the Item quantity dropped if L2ItemInstance is stackable
			if(item.isStackable())
			{
				i = itemCount; // Set so loop won't happent again
				item.setCount(itemCount); // Set item count
			}
			if(Config.DEBUG)
				_log.fine("Item id to drop: " + itemId + " amount: " + item.getCount());

			if(isRaid)
			{
				SystemMessage sm = new SystemMessage(SystemMessage.S1_DIED_AND_DROPPED_S3_S2);
				sm.addString(getName());
				sm.addItemName(itemId);
				sm.addNumber(item.getCount());
				broadcastPacket(sm);
			}

			// Check if the autoLoot mode is active
			if(Config.AUTO_LOOT)
				if(!Config.AUTO_LOOT_HERBS && item.isHerb())
					item.dropToTheGround(lastAttacker, this);
				else
					lastAttacker.doAutoLoot(item, this);
			else
				item.dropToTheGround(lastAttacker, this);
		}
	}

	public void dropItem(L2Player lastAttacker, L2ItemInstance item)
	{
		if(item.getCount() == 0)
			return;

		if(Config.DEBUG)
			_log.fine("Item id to drop: " + item + " amount: " + item.getCount());

		if(isRaid)
		{
			final SystemMessage sm = new SystemMessage(SystemMessage.S1_DIED_AND_DROPPED_S3_S2);
			sm.addString(getName());
			sm.addItemName(item.getItemId());
			sm.addNumber(item.getCount());
			broadcastPacket(sm);
		}

		// Check if the autoLoot mode is active
		if(Config.AUTO_LOOT)
			lastAttacker.doAutoLoot(item, this); // Give this or these Item(s) to the L2Player that has killed the L2NpcInstance
		else
			item.dropToTheGround(lastAttacker, this);
	}

	public L2ItemInstance getActiveWeapon()
	{
		return null;
	}

	@Override
	public boolean isAttackable()
	{
		return true;
	}

	@Override
	public boolean isAutoAttackable(L2Character attacker)
	{
		return false;
	}

	@Override
	public void onSpawn()
	{
		setDecayed(false);
		_showSpawnAnimation = false;
	}

	@Override
	public L2NpcTemplate getTemplate()
	{
		return (L2NpcTemplate) _template;
	}

	@Override
	public int getNpcId()
	{
		return getTemplate().npcId;
	}

	protected boolean _unAggred = false;

	public void setUnAggred(boolean state)
	{
		_unAggred = state;
	}

	/**
	 * Return True if the L2NpcInstance is aggressive (ex : L2MonsterInstance in function of aggroRange).<BR><BR>
	 */
	public boolean isAggressive()
	{
		return getAggroRange() > 0;
	}

	public int getAggroRange()
	{
		if(_unAggred)
			return 0;

		if(_personalAggroRange >= 0)
			return _personalAggroRange;

		return getTemplate().aggroRange;
	}

	/**
	 * Устанавливает данному npc новый aggroRange.
	 * Если установленый aggroRange < 0, то будет братся аггрорейндж с темплейта.
	 * @param aggroRange новый agrroRange
	 */
	public void setAggroRange(int aggroRange)
	{
		_personalAggroRange = aggroRange;
	}

	public int getFactionRange()
	{
		return getTemplate().factionRange;
	}

	public final String getFactionId()
	{
		return getTemplate().factionId;
	}

	public long getExpReward()
	{
		return (long) calcStat(Stats.EXP_SP, getTemplate().revardExp * (isRaid ? Config.RATE_DROP_RAIDBOSS : Config.RATE_XP), null, null);
	}

	public long getSpReward()
	{
		return (long) calcStat(Stats.EXP_SP, getTemplate().revardSp * (isRaid ? Config.RATE_DROP_RAIDBOSS : Config.RATE_SP), null, null);
	}

	/**
	 * Remove PROPERLY the L2NpcInstance from the world.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Remove the L2NpcInstance from the world and update its spawn object </li>
	 * <li>Remove L2Object object from _allObjects of L2World </li><BR><BR>
	 *
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : This method DOESN'T SEND Server->Client packets to players</B></FONT><BR><BR>
	 *
	 */
	public void deleteMe()
	{
		decayMe();
		detachAI();
		L2World.removeObject(this);
	}

	private L2Spawn _spawn;

	public L2Spawn getSpawn()
	{
		return _spawn;
	}

	public void setSpawn(L2Spawn spawn)
	{
		_spawn = spawn;
	}

	@Override
	public synchronized void onDecay()
	{
		if(isDecayed())
			return;

		setDecayed(true);

		// Remove the L2NpcInstance from the world when the decay task is launched
		super.onDecay();

		// Decrease its spawn counter
		if(_spawn != null)
			_spawn.decreaseCount(this);
	}

	private boolean _isDecayed = false;

	/**
	 * Set the decayed state of this L2NpcInstance<BR><BR>
	 */
	public final void setDecayed(boolean mode)
	{
		_isDecayed = mode;
	}

	/**
	 * Return the decayed status of this L2NpcInstance<BR><BR>
	 */
	public final boolean isDecayed()
	{
		return _isDecayed;
	}

	public void endDecayTask()
	{
		DecayTaskManager.getInstance().cancelDecayTask(this);
		onDecay();
	}

	/**
	 * Return True if this L2NpcInstance is undead in function of the L2NpcTemplate.<BR><BR>
	 */
	@Override
	public boolean isUndead()
	{
		return getTemplate().isUndead();
	}

	/**
	 * Return the Level of this L2NpcInstance contained in the L2NpcTemplate.<BR><BR>
	 */
	@Override
	public byte getLevel()
	{
		return getTemplate().level;
	}

	/**
	 * Return null (regular NPCs don't have weapons instancies).<BR><BR>
	 */
	@Override
	public L2ItemInstance getActiveWeaponInstance()
	{
		// regular NPCs dont have weapons instancies
		return null;
	}

	/**
	 * Return the weapon item equipped in the right hand of the L2NpcInstance or null.<BR><BR>
	 */
	@Override
	public L2Weapon getActiveWeaponItem()
	{
		// Get the weapon identifier equipped in the right hand of the L2NpcInstance
		int weaponId = getTemplate().rhand;

		if(weaponId < 1)
			return null;

		// Get the weapon item equipped in the right hand of the L2NpcInstance
		L2Item item = ItemTable.getInstance().getTemplate(getTemplate().rhand);

		if(!(item instanceof L2Weapon))
			return null;

		return (L2Weapon) item;
	}

	/**
	 * Return null (regular NPCs don't have weapons instances).<BR><BR>
	 */
	@Override
	public L2ItemInstance getSecondaryWeaponInstance()
	{
		// regular NPCs dont have weapons instances
		return null;
	}

	/**
	 * Return the weapon item equipped in the left hand of the L2NpcInstance or null.<BR><BR>
	 */
	@Override
	public L2Weapon getSecondaryWeaponItem()
	{
		// Get the weapon identifier equipped in the right hand of the L2NpcInstance
		int weaponId = getTemplate().lhand;

		if(weaponId < 1)
			return null;

		// Get the weapon item equipped in the right hand of the L2NpcInstance
		L2Item item = ItemTable.getInstance().getTemplate(getTemplate().lhand);

		if(!(item instanceof L2Weapon))
			return null;

		return (L2Weapon) item;
	}

	/**
	 * Send a packet NpcInfo with state of abnormal effect to all visible L2Player<BR><BR>
	 */
	@Override
	public void updateAbnormalEffect()
	{
		for(L2Player _cha : L2World.getAroundPlayers(this))
			_cha.sendPacket(new NpcInfo(this, _cha));
	}

	@Override
	public void onActionShift(L2Player player)
	{
		if(this != player.getTarget())
		{
			if(Config.DEBUG)
				_log.fine("new target selected:" + getObjectId());
			player.setTarget(this);
			player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));
			if(isAutoAttackable(player))
			{
				StatusUpdate su = new StatusUpdate(getObjectId());
				su.addAttribute(StatusUpdate.CUR_HP, (int) getCurrentHp());
				su.addAttribute(StatusUpdate.MAX_HP, getMaxHp());
				player.sendPacket(su);
			}
			player.sendPacket(new ValidateLocation(this));
			player.sendActionFailed();
			Events.onActionShift(player, this);
			return;
		}

		player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));

		if(Events.onActionShift(player, this))
			return;

		player.sendActionFailed();
	}

	// У NPC всегда 2
	public void onRandomAnimation()
	{
		broadcastPacket(new SocialAction(getObjectId(), 2));
	}

	public void startRandomAnimation()
	{
		int interval = 1000 * Rnd.get(Config.MIN_NPC_ANIMATION, Config.MAX_NPC_ANIMATION);
		ThreadPoolManager.getInstance().scheduleGeneral(new RandomAnimationTask(), interval);
	}

	public class RandomAnimationTask implements Runnable
	{
		private final int interval;

		public RandomAnimationTask()
		{
			interval = 1000 * Rnd.get(Config.MIN_NPC_ANIMATION, Config.MAX_NPC_ANIMATION);
		}

		public void run()
		{
			if(!isDead() && !isMoving)
				onRandomAnimation();

			ThreadPoolManager.getInstance().scheduleGeneral(this, interval);
		}
	}

	@Override
	public boolean hasRandomAnimation()
	{
		return Config.MAX_NPC_ANIMATION > 0;
	}

	@Override
	public boolean isInvul()
	{
		return true;
	}

	public Castle getCastle()
	{
		if(Config.SERVICES_OFFSHORE_NO_CASTLE_TAX && isInZone(ZoneType.offshore))
			return null;
		return TownManager.getInstance().getClosestTown(this).getCastle();
	}

	private int _fortressId = -1;

	/** Return the Fortress this L2NpcInstance belongs to. */
	public Fortress getFortress()
	{
		if(_fortressId < 0)
			_fortressId = FortressManager.getInstance().findNearestFortressIndex(getX(), getY(), 10000);
		return FortressManager.getInstance().getFortressByIndex(_fortressId);
	}

	private int _clanHallId = -1;

	/** Return the L2ClanHall this L2NpcInstance belongs to. */
	public ClanHall getClanHall()
	{
		if(_clanHallId < 0)
			_clanHallId = ClanHallManager.getInstance().findNearestClanHallIndex(getX(), getY(), 1500);
		return ClanHallManager.getInstance().getClanHall(_clanHallId);
	}

	private long _lastSocialAction;

	@Override
	public void onAction(L2Player player)
	{
		if(this != player.getTarget())
		{
			if(Config.DEBUG)
				_log.fine("new target selected:" + getObjectId());
			player.setTarget(this);
			player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));
			if(isAutoAttackable(player))
			{
				StatusUpdate su = new StatusUpdate(getObjectId());
				su.addAttribute(StatusUpdate.CUR_HP, (int) getCurrentHp());
				su.addAttribute(StatusUpdate.MAX_HP, getMaxHp());
				player.sendPacket(su);
			}
			player.sendPacket(new ValidateLocation(this));
			player.sendActionFailed();
			return;
		}

		if(Events.onAction(player, this))
			return;

		if(isAutoAttackable(player))
		{
			player.getAI().Attack(this, false);
			return;
		}

		if(!isInRange(player, INTERACTION_DISTANCE))
		{
			if(player.getAI().getIntention() != CtrlIntention.AI_INTENTION_INTERACT)
				player.getAI().setIntention(CtrlIntention.AI_INTENTION_INTERACT, this, null);
			return;
		}

		if(!Config.ALT_GAME_KARMA_PLAYER_CAN_SHOP && player.getKarma() > 0 && !player.isGM())
		{
			player.sendActionFailed();
			return;
		}

		// С NPC нельзя разговаривать мертвым и сидя
		if(!Config.ALLOW_TALK_WHILE_SITTING && player.isSitting() || player.isAlikeDead())
			return;

		if(System.currentTimeMillis() - _lastSocialAction > 10000)
		{
			broadcastPacket(new SocialAction(getObjectId(), 2));
			_lastSocialAction = System.currentTimeMillis();
		}

		player.sendActionFailed();

		if(_isBusy)
			showBusyWindow(player);
		else
		{
			Quest[] qlst = getTemplate().getEventQuests(Quest.QuestEventType.NPC_FIRST_TALK);
			if(qlst == null || qlst.length == 0 || player.getQuestState(qlst[0].getName()) == null || player.getQuestState(qlst[0].getName()).isCompleted() || !qlst[0].notifyFirstTalk(this, player))
				showChatWindow(player, 0);
		}
	}

	public void showQuestWindow(L2Player player, String questId)
	{
		if(!player.isQuestContinuationPossible())
			return;

		try
		{
			// Get the state of the selected quest
			QuestState qs = player.getQuestState(questId);
			if(qs != null)
			{
				if(qs.isCompleted())
				{
					Functions.show(new CustomMessage("quests.QuestAlreadyCompleted", player), player);
					return;
				}
				if(qs.getQuest().notifyTalk(this, qs))
					return;
			}
			else
			{
				Quest q = QuestManager.getQuest(questId);
				if(q != null)
				{
					// check for start point
					Quest[] qlst = getTemplate().getEventQuests(Quest.QuestEventType.QUEST_START);
					if(qlst != null && qlst.length > 0)
						for(Quest element : qlst)
							if(element == q)
							{
								qs = q.newQuestState(player);
								if(qs.getQuest().notifyTalk(this, qs))
									return;
								break;
							}
				}
			}

			String content = "<html><body>You are either not on a quest that involves this NPC, or you don't meet this NPC's minimum quest requirements.</body></html>";
			NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
			html.setHtml(content);
			player.sendPacket(html);
		}
		catch(Exception e)
		{
			_log.warning("problem with npc text " + e);
			e.printStackTrace();
		}

		player.sendActionFailed();
	}

	/**
	 * Open a quest or chat window on client with the text of the L2NpcInstance in function of the command.<BR><BR>
	 *
	 * <B><U> Example of use </U> :</B><BR><BR>
	 * <li> Client packet : RequestBypassToServer</li><BR><BR>
	 *
	 * @param command The command string received from client
	 *
	 */
	public void onBypassFeedback(L2Player player, String command)
	{
		if(!isInRange(player, INTERACTION_DISTANCE))
		{
			player.getAI().setIntention(CtrlIntention.AI_INTENTION_ATTACK, this);
			player.sendActionFailed();
		}
		else
			try
			{
				if(command.equalsIgnoreCase("TerritoryStatus"))
				{
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/merchant/territorystatus.htm");
					html.replace("%npcname%", getName());

					if(getCastle() != null && getCastle().getId() > 0)
					{
						html.replace("%castlename%", getCastle().getName());
						html.replace("%taxpercent%", String.valueOf(getCastle().getTaxPercent()));

						if(getCastle().getOwnerId() > 0)
						{
							L2Clan clan = ClanTable.getInstance().getClan(getCastle().getOwnerId());
							if(clan != null)
							{
								html.replace("%clanname%", clan.getName());
								html.replace("%clanleadername%", clan.getLeaderName());
							}
							else
							{
								html.replace("%clanname%", "unexistant clan");
								html.replace("%clanleadername%", "None");
							}
						}
						else
						{
							html.replace("%clanname%", "NPC");
							html.replace("%clanleadername%", "None");
						}
					}
					else
					{
						html.replace("%castlename%", "Open");
						html.replace("%taxpercent%", "0");

						html.replace("%clanname%", "No");
						html.replace("%clanleadername%", getName());
					}

					player.sendPacket(html);
				}
				else if(command.equalsIgnoreCase("Exchange"))
				{
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/merchant/exchange.htm");
					html.replace("%npcname%", getName());
					player.sendPacket(html);
				}
				else if(command.startsWith("Quest"))
				{
					String quest = command.substring(5).trim();
					if(quest.length() == 0)
						showQuestWindow(player);
					else
						showQuestWindow(player, quest);
				}
				else if(command.startsWith("Chat"))
					try
					{
						int val = Integer.parseInt(command.substring(5));
						showChatWindow(player, val);
					}
					catch(NumberFormatException nfe)
					{
						String filename = command.substring(5).trim();
						if(filename.length() == 0)
							showChatWindow(player, "data/html/npcdefault.htm");
						else
							showChatWindow(player, filename);
					}
				else if(command.startsWith("Loto"))
				{
					int val = Integer.parseInt(command.substring(5));
					if(val == 0)
						// new loto ticket
						for(int i = 0; i < 5; i++)
							player.setLoto(i, 0);
					showLotoWindow(player, val);
				}
				else if(command.startsWith("CPRecovery"))
					makeCPRecovery(player);
				else if(command.startsWith("NpcLocationInfo"))
				{
					int val = Integer.parseInt(command.substring(16));
					L2NpcInstance npc = L2World.findNpcByNpcId(val);
					// Убираем флажок на карте и стрелку на компасе
					player.sendPacket(new RadarControl(2, 2, npc.getLoc()));
					// Ставим флажок на карте и стрелку на компасе
					player.sendPacket(new RadarControl(0, 1, npc.getLoc()));
				}
				else if(command.startsWith("SupportMagic"))
					makeSupportMagic(player);
				else if(command.startsWith("ProtectionBlessing"))
				{
					// Не выдаём блессиг протекшена ПКшникам.
					if(player.getKarma() > 0)
						return;
					if(player.getLevel() > 39 || player.getClassId().getLevel() >= 2)
					{
						String content = "<html><body>Blessing of protection not available for characters whose level more than 39 or completed second class transfer.</body></html>";
						NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
						html.setHtml(content);
						player.sendPacket(html);
						return;
					}
					doCast(SkillTable.getInstance().getInfo(5182, 1), player, true);
				}
				else if(command.startsWith("Multisell") || command.startsWith("multisell"))
				{
					String listId = command.substring(9).trim();
					L2Multisell.getInstance().SeparateAndSend(Integer.parseInt(listId), player, getCastle() != null ? getCastle().getTaxRate() : 0);
				}
				else if(command.startsWith("EnterRift"))
				{
					StringTokenizer st = new StringTokenizer(command);
					st.nextToken(); //no need for "enterRift"

					Byte b1 = Byte.parseByte(st.nextToken()); //type

					DimensionalRiftManager.getInstance().start(player, b1, this);
				}
				else if(command.startsWith("ChangeRiftRoom"))
				{
					if(player.isInParty() && player.getParty().isInDimensionalRift())
						player.getParty().getDimensionalRift().manualTeleport(player, this);
					else
					{
						DimensionalRiftManager.getInstance().showHtmlFile(player, "data/html/rift/Cheater.htm", this);
						if(!player.isGM())
							_log.warning("Player " + player.getName() + "(" + player.getObjectId() + ") was cheating in dimension rift area!");
					}
				}
				else if(command.startsWith("ExitRift"))
				{
					if(player.isInParty() && player.getParty().isInDimensionalRift())
						player.getParty().getDimensionalRift().manualExitRift(player, this);
					else
					{
						DimensionalRiftManager.getInstance().showHtmlFile(player, "data/html/rift/Cheater.htm", this);
						if(!player.isGM())
							_log.warning("Player " + player.getName() + "(" + player.getObjectId() + ") was cheating in dimension rift area!");
					}
				}
				else if(command.equalsIgnoreCase("SkillList"))
					showSkillList(player);
				else if(command.equalsIgnoreCase("ClanSkillList"))
					showClanSkillList(player);
				else if(command.equalsIgnoreCase("FishingSkillList"))
					showFishingSkillList(player);
				else if(command.equalsIgnoreCase("EnchantSkillManage"))
					showChatWindow(player, "data/html/enchant_skill_manage.htm");
				else if(command.equalsIgnoreCase("EnchantSkillList"))
					showEnchantSkillList(player, false);
				else if(command.startsWith("SafeEnchantSkillList"))
					showEnchantSkillList(player, true);
				else if(command.startsWith("ChangeEnchantSkillList"))
					showEnchantChangeSkillList(player);
				else if(command.startsWith("UntrainEnchantSkillList"))
					showEnchantUntrainSkillList(player);
				else if(command.startsWith("Augment"))
				{
					int cmdChoice = Integer.parseInt(command.substring(8, 9).trim());
					if(cmdChoice == 1)
					{
						player.sendPacket(new SystemMessage(SystemMessage.SELECT_THE_ITEM_TO_BE_AUGMENTED));
						player.sendPacket(Msg.ExShowVariationMakeWindow);
					}
					else if(cmdChoice == 2)
					{
						player.sendPacket(new SystemMessage(SystemMessage.SELECT_THE_ITEM_FROM_WHICH_YOU_WISH_TO_REMOVE_AUGMENTATION));
						player.sendPacket(Msg.ExShowVariationCancelWindow);
					}
				}
				else if(command.startsWith("Link"))
					showChatWindow(player, "data/html/" + command.substring(5));
			}
			catch(StringIndexOutOfBoundsException sioobe)
			{
				_log.info("Incorrect htm bypass! npcId=" + getTemplate().npcId + " command=[" + command + "]");
			}
			catch(NumberFormatException nfe)
			{
				_log.info("Invalid bypass to Server command parameter! npcId=" + getTemplate().npcId + " command=[" + command + "]");
			}
	}

	public void showQuestWindow(L2Player player)
	{
		// collect awaiting quests and start points
		ArrayList<Quest> options = new ArrayList<Quest>();

		ArrayList<QuestState> awaits = player.getQuestsForTalk(getTemplate().npcId);
		Quest[] starts = getTemplate().getEventQuests(Quest.QuestEventType.QUEST_START);

		if(awaits != null)
			for(QuestState x : awaits)
				if(!options.contains(x.getQuest()))
					if(x.getQuest().getQuestIntId() > 0 && x.getQuest().getQuestIntId() < 1000)
						options.add(x.getQuest());

		if(starts != null)
			for(Quest x : starts)
				if(!options.contains(x))
					if(x.getQuestIntId() > 0 && x.getQuestIntId() < 1000)
						options.add(x);

		// Display a QuestChooseWindow (if several quests are available) or QuestWindow
		if(options.size() > 1)
			showQuestChooseWindow(player, options.toArray(new Quest[options.size()]));
		else if(options.size() == 1)
			showQuestWindow(player, options.get(0).getName());
		else
			showQuestWindow(player, "");
	}

	public void showQuestChooseWindow(L2Player player, Quest[] quests)
	{
		StringBuffer sb = new StringBuffer();

		sb.append("<html><body><title>Talk about:</title><br>");

		for(Quest q : quests)
			if(player.getQuestState(q.getName()) == null)
				sb.append("<a action=\"bypass -h npc_").append(getObjectId()).append("_Quest ").append(q.getName()).append("\">[").append(q.getDescr()).append("]</a><br>");
			else if(player.getQuestState(q.getName()).isCompleted())
				sb.append("<a action=\"bypass -h npc_").append(getObjectId()).append("_Quest ").append(q.getName()).append("\">[").append(q.getDescr()).append(" - completed").append("]</a><br>");
			else
				sb.append("<a action=\"bypass -h npc_").append(getObjectId()).append("_Quest ").append(q.getName()).append("\">[").append(q.getDescr()).append(" - in progress").append("]</a><br>");

		sb.append("</body></html>");

		NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
		html.setHtml(sb.toString());
		player.sendPacket(html);
	}

	public void showChatWindow(L2Player player, int val)
	{
		int npcId = getTemplate().npcId;

		/* For use with Seven Signs implementation */
		String filename = SevenSigns.SEVEN_SIGNS_HTML_PATH;
		int sealAvariceOwner = SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_AVARICE);
		int sealGnosisOwner = SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_GNOSIS);
		int playerCabal = SevenSigns.getInstance().getPlayerCabal(player);
		boolean isSealValidationPeriod = SevenSigns.getInstance().isSealValidationPeriod();
		int compWinner = SevenSigns.getInstance().getCabalHighestScore();

		switch(npcId)
		{
			case 31095: //
			case 31096: //
			case 31097: //
			case 31098: // Enter Necropolises
			case 31099: //
			case 31100: //
			case 31101: //
			case 31102: //
				if(isSealValidationPeriod)
				{
					if(sealAvariceOwner != compWinner)
					{
						player.sendMessage("Seal of avarice wasn't taken!");
						filename += "necro_no.htm";
					}
					else if(playerCabal != compWinner)
						switch(compWinner)
						{
							case SevenSigns.CABAL_DAWN:
								player.sendPacket(new SystemMessage(SystemMessage.CAN_BE_USED_ONLY_BY_THE_LORDS_OF_DAWN));
								filename += "necro_no.htm";
								break;
							case SevenSigns.CABAL_DUSK:
								player.sendPacket(new SystemMessage(SystemMessage.CAN_BE_USED_ONLY_BY_THE_REVOLUTIONARIES_OF_DUSK));
								filename += "necro_no.htm";
								break;
							case SevenSigns.CABAL_NULL:
								filename = getHtmlPath(npcId, val); // do the default!
								break;
						}
					else
						filename = getHtmlPath(npcId, val); // do the default!
				}
				else if(playerCabal == SevenSigns.CABAL_NULL)
					filename += "necro_no.htm";
				else
					filename = getHtmlPath(npcId, val); // do the default!
				break;
			case 31114: //
			case 31115: //
			case 31116: // Enter Catacombs
			case 31117: //
			case 31118: //
			case 31119: //
				if(isSealValidationPeriod)
				{
					if(sealGnosisOwner != compWinner)
					{
						player.sendMessage("Seal of gnosis wasn't taken!");
						filename += "cata_no.htm";
					}
					else if(playerCabal != compWinner)
						switch(compWinner)
						{
							case SevenSigns.CABAL_DAWN:
								player.sendPacket(new SystemMessage(SystemMessage.CAN_BE_USED_ONLY_BY_THE_LORDS_OF_DAWN));
								filename += "cata_no.htm";
								break;
							case SevenSigns.CABAL_DUSK:
								player.sendPacket(new SystemMessage(SystemMessage.CAN_BE_USED_ONLY_BY_THE_REVOLUTIONARIES_OF_DUSK));
								filename += "cata_no.htm";
								break;
							case SevenSigns.CABAL_NULL:
								filename = getHtmlPath(npcId, val); // do the default!
								break;
						}
					else
						filename = getHtmlPath(npcId, val); // do the default!
				}
				else if(playerCabal == SevenSigns.CABAL_NULL)
					filename += "cata_no.htm";
				else
					filename = getHtmlPath(npcId, val); // do the default!
				break;
			case 31111: // Gatekeeper Spirit (Disciples)
				if(playerCabal == sealAvariceOwner && playerCabal == compWinner)
					switch(sealAvariceOwner)
					{
						case SevenSigns.CABAL_DAWN:
							filename += "spirit_dawn.htm";
							break;
						case SevenSigns.CABAL_DUSK:
							filename += "spirit_dusk.htm";
							break;
						case SevenSigns.CABAL_NULL:
							filename += "spirit_null.htm";
							break;
					}
				else
					filename += "spirit_null.htm";
				break;
			case 31112: // Gatekeeper Spirit (Disciples)
				filename += "spirit_exit.htm";
				break;
			case 31688:
				if(player.isNoble())
					filename = Olympiad.OLYMPIAD_HTML_FILE + "noble_main.htm";
				else
					filename = getHtmlPath(npcId, val);
				break;
			case 31690:
			case 31769:
			case 31770: // Monument of Heroes
			case 31771:
			case 31772:
				if(player.isHero())
					filename = Olympiad.OLYMPIAD_HTML_FILE + "hero_main.htm";
				else
					filename = getHtmlPath(npcId, val);
				break;
			default:
				if(npcId >= 31093 && npcId <= 31094 || npcId >= 31172 && npcId <= 31201 || npcId >= 31239 && npcId <= 31254)
					return;
				// Get the text of the selected HTML file in function of the npcId and of the page number
				filename = getHtmlPath(npcId, val);
				break;
		}

		player.sendPacket(new NpcHtmlMessage(player, this, filename, val));
	}

	public void showChatWindow(L2Player player, String filename)
	{
		player.sendPacket(new NpcHtmlMessage(player, this, filename, 0));
	}

	public String getHtmlPath(int npcId, int val)
	{
		String pom;
		if(val == 0)
			pom = "" + npcId;
		else
			pom = npcId + "-" + val;
		String temp = "data/html/default/" + pom + ".htm";
		File mainText = new File(temp);
		if(mainText.exists())
			return temp;

		temp = "data/html/trainer/" + pom + ".htm";
		mainText = new File(temp);
		if(mainText.exists())
			return temp;

		// If the file is not found, the standard message "I have nothing to say to you" is returned
		return "data/html/npcdefault.htm";
	}

	// 0 - buy ticket window
	// 1-20 - buttons
	// 22 - lottery ticket window
	// 21 - selected ticket with 5 numbers
	public void showLotoWindow(L2Player player, int val)
	{
		int npcId = getTemplate().npcId;
		String filename;
		SystemMessage sm;
		NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
		//if loto
		if(val == 0)
		{
			filename = getHtmlPath(npcId, 1);
			html.setFile(filename);
		}
		else if(val != 21)
		{
			filename = getHtmlPath(npcId, 5);
			html.setFile(filename);
			int count = 0;
			int found = 0;
			// counting buttons and unsetting button if found
			for(int i = 0; i < 5; i++)
				if(player.getLoto(i) == val)
				{
					//unsetting button
					player.setLoto(i, 0);
					found = 1;
				}
				else if(player.getLoto(i) > 0)
					count++;
			//if not rearched limit 5 and not unseted value
			if(count < 5 && found == 0 && val < 21)
				for(int i = 0; i < 5; i++)
					if(player.getLoto(i) == 0)
					{
						player.setLoto(i, val);
						break;
					}
			//setting pusshed buttons
			count = 0;
			for(int i = 0; i < 5; i++)
				if(player.getLoto(i) > 0)
				{
					count++;
					String button = String.valueOf(player.getLoto(i));
					if(player.getLoto(i) < 10)
						button = "0" + button;
					String search = "fore=\"L2UI.lottoNum" + button + "\" back=\"L2UI.lottoNum" + button + "a_check\"";
					String replace = "fore=\"L2UI.lottoNum" + button + "a_check\" back=\"L2UI.lottoNum" + button + "\"";
					html.replace(search, replace);
				}
			if(count == 5)
			{
				String search = "0\">Return";
				String replace = "21\">The winner selected the numbers above.";
				html.replace(search, replace);
			}
		}
		else
		{
			int price = 2000;
			int lotonumber = 1;
			int enchant = 0;
			int type2 = 0;
			for(int i = 0; i < 5; i++)
			{
				if(player.getLoto(i) == 0)
					return;
				if(player.getLoto(i) < 17)
					enchant += Math.pow(2, player.getLoto(i) - 1);
				else
					type2 += Math.pow(2, player.getLoto(i) - 17);
			}
			if(player.getAdena() < price)
			{
				player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
				return;
			}
			player.reduceAdena(price, true);
			sm = new SystemMessage(SystemMessage.ACQUIRED__S1_S2);
			sm.addNumber(lotonumber);
			sm.addItemName(4442);
			player.sendPacket(sm);
			L2ItemInstance item = new L2ItemInstance(IdFactory.getInstance().getNextId(), (short) 4442, player.getObjectId(), npcId, "showLotoWindow");
			item.setCustomType1(lotonumber);
			item.setEnchantLevel(enchant);
			item.setCustomType2(type2);
			player.getInventory().addItem(item);
			Log.LogItem(player, Log.BuyItem, item);
			player.getInventory().findItemByItemId(57);
			html.setHtml("<html><body>Lottery Ticket Seller:<br>Thank you for playing the lottery<br>The winners will be announced at ??? 19<br><center><a action=\"bypass -h npc_%objectId%_Chat 0\">Back</a></center></body></html>");
		}
		player.sendPacket(html);
		player.sendActionFailed();
	}

	public void makeCPRecovery(L2Player player)
	{
		if(getNpcId() != 31225 && getNpcId() != 31226)
			return;
		int neededmoney = 100;
		int currentmoney = player.getAdena();
		if(neededmoney > currentmoney)
		{
			player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			return;
		}
		player.reduceAdena(neededmoney, true);
		player.setCurrentCp(getCurrentCp() + 5000);
		player.sendPacket(new SystemMessage(SystemMessage.S1_CPS_WILL_BE_RESTORED).addString(player.getName()));
	}

	public void makeSupportMagic(L2Player player)
	{
		// Prevent a cursed weapon weilder of being buffed
		if(player.isCursedWeaponEquipped())
			return;
		int lvl = player.getLevel();
		L2Skill skill;
		if(lvl < 8)
		{
			showChatWindow(player, "data/html/default/newbie_nosupport8.htm");
			return;
		}
		if(lvl > 39)
		{
			showChatWindow(player, "data/html/default/newbie_nosupport39.htm");
			return;
		}
		int time = 0;
		//windwalk
		if(lvl >= 8 && lvl <= 39)
		{
			skill = SkillTable.getInstance().getInfo(4322, 1);
			time += 1000;
			ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
		}
		//shield
		if(lvl >= 11 && lvl <= 38)
		{
			skill = SkillTable.getInstance().getInfo(4323, 1);
			time += 1000;
			ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
		}
		//lifecubic
		if(lvl >= 16 && lvl <= 34)
		{
			skill = SkillTable.getInstance().getInfo(4338, 1);
			time += 1000;
			ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
		}
		if(!player.isMageClass())
		{
			//blessofbody
			if(lvl >= 12 && lvl <= 37)
			{
				skill = SkillTable.getInstance().getInfo(4324, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
			//vampirerage
			if(lvl >= 13 && lvl <= 36)
			{
				skill = SkillTable.getInstance().getInfo(4325, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
			//regeneration
			if(lvl >= 14 && lvl <= 35)
			{
				skill = SkillTable.getInstance().getInfo(4326, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
			//haste
			if(lvl >= 15 && lvl <= 34)
			{
				skill = SkillTable.getInstance().getInfo(4327, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
		}
		else
		{
			//blessthesoul
			if(lvl >= 12 && lvl <= 37)
			{
				skill = SkillTable.getInstance().getInfo(4328, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
			//acumen
			if(lvl >= 13 && lvl <= 36)
			{
				skill = SkillTable.getInstance().getInfo(4329, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
			//concentration
			if(lvl >= 14 && lvl <= 35)
			{
				skill = SkillTable.getInstance().getInfo(4330, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
			//empower
			if(lvl >= 15 && lvl <= 34)
			{
				skill = SkillTable.getInstance().getInfo(4331, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
		}
	}

	protected class doBuff implements Runnable
	{
		L2Character _actor;
		L2Skill _skill;
		L2Player _target;

		doBuff(L2Character actor, L2Skill skill, L2Player target)
		{
			_actor = actor;
			_skill = skill;
			_target = target;
		}

		public void run()
		{
			_actor.doCast(_skill, _target, true);
		}
	}

	private boolean _isBusy;
	private String _busyMessage = "";

	/**
	 * Return the busy status of this L2NpcInstance.<BR><BR>
	 */
	public final boolean isBusy()
	{
		return _isBusy;
	}

	/**
	 * Set the busy status of this L2NpcInstance.<BR><BR>
	 */
	public void setBusy(boolean isBusy)
	{
		_isBusy = isBusy;
	}

	/**
	 * Return the busy message of this L2NpcInstance.<BR><BR>
	 */
	public final String getBusyMessage()
	{
		return _busyMessage;
	}

	/**
	 * Set the busy message of this L2NpcInstance.<BR><BR>
	 */
	public void setBusyMessage(String message)
	{
		_busyMessage = message;
	}

	public void showBusyWindow(L2Player player)
	{
		NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
		html.setFile("data/html/npcbusy.htm");
		html.replace("%npcname%", getName());
		html.replace("%playername%", player.getName());
		html.replace("%busymessage%", _busyMessage);
		player.sendPacket(html);
	}

	/**
	 * this displays SkillList to the player.
	 * @param player
	 */
	public void showSkillList(L2Player player)
	{
		if(Config.DEBUG)
			_log.fine("SkillList activated on: " + getObjectId());

		ClassId classId = player.getClassId();

		if(classId == null)
			return;

		int npcId = getTemplate().npcId;

		if(_classesToTeach == null)
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			TextBuilder sb = new TextBuilder();
			sb.append("<html><head><body>");
			sb.append("I cannot teach you. My class list is empty.<br> Ask admin to fix it. Need add my npcid and classes to skill_learn.sql.<br>NpcId:" + npcId + ", Your classId:" + player.getClassId().getId() + "<br>");
			sb.append("</body></html>");
			html.setHtml(sb.toString());
			player.sendPacket(html);

			return;
		}

		if(!(getTemplate().canTeach(classId) || getTemplate().canTeach(classId.getParent(player.getSex()))))
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			TextBuilder sb = new TextBuilder();
			sb.append("<html><head><body>");
			sb.append(new CustomMessage("l2r.gameserver.model.instances.L2NpcInstance.WrongTeacherClass", player));
			sb.append("</body></html>");
			html.setHtml(sb.toString());
			player.sendPacket(html);

			return;
		}

		AcquireSkillList asl = new AcquireSkillList(AcquireSkillList.USUAL);
		int counts = 0;

		ArrayList<L2SkillLearn> skills = SkillTreeTable.getInstance().getAvailableSkills(player, classId);
		for(L2SkillLearn s : skills)
		{
			L2Skill sk = SkillTable.getInstance().getInfo(s.getId(), s.getLevel());
			if(sk == null || !sk.getCanLearn(player.getClassId()) || !sk.canTeachBy(npcId))
				continue;
			int cost = SkillTreeTable.getInstance().getSkillCost(player, sk);
			counts++;
			asl.addSkill(s.getId(), s.getLevel(), s.getLevel(), cost, 0);
		}

		if(counts == 0)
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			int minlevel = SkillTreeTable.getInstance().getMinLevelForNewSkill(player, classId);

			if(minlevel > 0)
			{
				SystemMessage sm = new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ANY_FURTHER_SKILLS_TO_LEARN__COME_BACK_WHEN_YOU_HAVE_REACHED_LEVEL_S1);
				sm.addNumber(minlevel);
				player.sendPacket(sm);
			}
			else
			{
				TextBuilder sb = new TextBuilder();
				sb.append("<html><head><body>");
				sb.append("You've learned all skills for your class.");
				sb.append("</body></html>");
				html.setHtml(sb.toString());
				player.sendPacket(html);
			}
		}
		else
			player.sendPacket(asl);

		player.sendActionFailed();
	}

	public void showEnchantSkillList(L2Player player, boolean isSafeEnchant)
	{
		if(!enchantChecks(player))
			return;
		ArrayList<L2Skill> skills = SkillTreeTable.getInstance().getSkillsToEnchant(player);
		ExEnchantSkillList esl = new ExEnchantSkillList(isSafeEnchant ? EnchantSkillType.SAFE : EnchantSkillType.NORMAL);
		int counts = 0;
		for(L2Skill s : skills)
		{
			counts++;
			esl.addSkill(s.getId(), s.getDisplayLevel());
		}
		if(counts == 0)
			player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NO_SKILL_THAT_ENABLES_ENCHANT));
		else
			player.sendPacket(esl);
	}

	public void showEnchantChangeSkillList(L2Player player)
	{
		if(!enchantChecks(player))
			return;
		ExEnchantSkillList esl = new ExEnchantSkillList(EnchantSkillType.CHANGE_ROUTE);
		int counts = 0;
		for(L2Skill s : player.getAllSkills())
		{
			if(s.getDisplayLevel() < 100)
				continue;
			counts++;
			esl.addSkill(s.getId(), s.getDisplayLevel());
		}
		if(counts == 0)
			player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NO_SKILL_THAT_ENABLES_ENCHANT));
		else
			player.sendPacket(esl);
	}

	public void showEnchantUntrainSkillList(L2Player player)
	{
		if(!enchantChecks(player))
			return;
		ExEnchantSkillList esl = new ExEnchantSkillList(EnchantSkillType.UNTRAIN);
		int counts = 0;
		for(L2Skill s : player.getAllSkills())
		{
			if(s.getDisplayLevel() < 100)
				continue;
			counts++;
			esl.addSkill(s.getId(), s.getDisplayLevel());
		}
		if(counts == 0)
			player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NO_SKILL_THAT_ENABLES_ENCHANT));
		else
			player.sendPacket(esl);
	}

	private boolean enchantChecks(L2Player player)
	{

		int npcId = getTemplate().npcId;

		if(_classesToTeach == null)
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			TextBuilder sb = new TextBuilder();
			sb.append("<html><head><body>");
			sb.append("I cannot teach you. My class list is empty.<br> Ask admin to fix it. Need add my npcid and classes to skill_learn.sql.<br>NpcId:" + npcId + ", Your classId:" + player.getClassId().getId() + "<br>");
			sb.append("</body></html>");
			html.setHtml(sb.toString());
			player.sendPacket(html);
			return false;
		}

		if(!(getTemplate().canTeach(player.getClassId()) || getTemplate().canTeach(player.getClassId().getParent(player.getSex()))))
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			TextBuilder sb = new TextBuilder();
			sb.append("<html><head><body>");
			sb.append(new CustomMessage("l2r.gameserver.model.instances.L2NpcInstance.WrongTeacherClass", player));
			sb.append("</body></html>");
			html.setHtml(sb.toString());
			player.sendPacket(html);
			return false;
		}

		if(player.getClassId().getLevel() < 4)
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			TextBuilder sb = new TextBuilder();
			sb.append("<html><head><body>");
			sb.append("You must have 3rd class change quest completed.");
			sb.append("</body></html>");
			html.setHtml(sb.toString());
			player.sendPacket(html);
			return false;
		}

		if(player.getLevel() < 76)
		{
			player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NO_SKILL_THAT_ENABLES_ENCHANT));
			return false;
		}

		return true;
	}

	public void showFishingSkillList(L2Player player)
	{
		if(Config.DEBUG)
			_log.fine("SkillList activated on: " + getObjectId());

		AcquireSkillList asl = new AcquireSkillList(AcquireSkillList.FISHING);
		int counts = 0;

		L2SkillLearn[] skills = SkillTreeTable.getInstance().getAvailableFishingSkills(player);
		for(L2SkillLearn s : skills)
		{
			L2Skill sk = SkillTable.getInstance().getInfo(s.getId(), s.getLevel());
			if(sk == null)
				continue;
			int cost = SkillTreeTable.getInstance().getSkillCost(player, sk);
			counts++;
			asl.addSkill(s.getId(), s.getLevel(), s.getLevel(), cost, 0);
		}

		if(counts == 0)
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			TextBuilder sb = new TextBuilder();
			sb.append("<html><head><body>");
			sb.append("You've learned all skills.");
			sb.append("</body></html>");
			html.setHtml(sb.toString());
			player.sendPacket(html);
		}
		else
			player.sendPacket(asl);

		player.sendActionFailed();
	}

	public void showClanSkillList(L2Player player)
	{
		if(Config.DEBUG)
			_log.fine("SkillList activated on: " + getObjectId());

		if(player.getClan() == null || !player.isClanLeader())
		{
			player.sendPacket(new SystemMessage(SystemMessage.ONLY_THE_CLAN_LEADER_IS_ENABLED));
			player.sendActionFailed();
			return;
		}

		AcquireSkillList asl = new AcquireSkillList(AcquireSkillList.CLAN);
		int counts = 0;

		L2SkillLearn[] skills = SkillTreeTable.getInstance().getAvailableClanSkills(player.getClan());
		for(L2SkillLearn s : skills)
		{
			L2Skill sk = SkillTable.getInstance().getInfo(s.getId(), s.getLevel());
			if(sk == null)
				continue;
			int cost = s.getRepCost();
			counts++;
			asl.addSkill(s.getId(), s.getLevel(), s.getLevel(), cost, 0);
		}

		if(counts == 0)
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			html.setHtml("<html><head><body>You've learned all skills.</body></html>");
			player.sendPacket(html);
		}
		else
			player.sendPacket(asl);

		player.sendActionFailed();
	}

	/**
	 * Возвращает режим NPC: свежезаспавненный или нормальное состояние
	 * @return true, если NPC свежезаспавненный
	 */
	public boolean isShowSpawnAnimation()
	{
		return _showSpawnAnimation;
	}

	@Override
	public boolean getChargedSoulShot()
	{
		switch(getTemplate().shots)
		{
			case SOUL:
			case SOUL_SPIRIT:
			case SOUL_BSPIRIT:
				return true;
			default:
				return false;
		}
	}

	@Override
	public int getChargedSpiritShot()
	{
		switch(getTemplate().shots)
		{
			case SPIRIT:
			case SOUL_SPIRIT:
				return 1;
			case BSPIRIT:
			case SOUL_BSPIRIT:
				return 2;
			default:
				return 0;
		}
	}

	@Override
	public boolean unChargeShots(boolean spirit)
	{
		broadcastPacket(new MagicSkillUse(this, spirit ? 2061 : 2039, 1, 0, 0));
		return true;
	}

	@Override
	public float getColRadius()
	{
		return (float) getCollisionRadius();
	}

	@Override
	public float getColHeight()
	{
		return (float) getCollisionHeight();
	}
}