package l2r.gameserver.model.instances;

import java.sql.ResultSet;
import java.util.concurrent.Future;

import l2r.Config;
import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.idfactory.IdFactory;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.PetInventory;
import l2r.gameserver.tables.PetDataTable;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.templates.L2NpcTemplate;
import l2r.util.Rnd;

public final class L2PetBabyInstance extends L2PetInstance
{
	@SuppressWarnings("unchecked")
	private Future _healTask;
	float _expPenalty;
	boolean _thinking = false;

	protected L2PetBabyInstance _pet;

	public L2PetBabyInstance(int objectId, L2NpcTemplate template, L2Player owner, L2ItemInstance control)
	{
		super(objectId, template, owner, control);
		_pet = this;
	}

	class HealTask implements Runnable
	{
		public void run()
		{
			if(_thinking)
				return;
			_thinking = true;

			int maxHp = getOwner().getMaxHp();
			double curHP = getOwner().getCurrentHp();
			L2Skill skill = null;
			try
			{
				if(!getOwner().isDead() && getOwner() != null)
				{
					if(Rnd.chance(25))
					{
						if(curHP <= maxHp * .95 && curHP >= maxHp * .2)
							skill = SkillTable.getInstance().getInfo(4717, getSkillLevel());
					}
					else if(curHP < maxHp * .2)
						skill = SkillTable.getInstance().getInfo(4718, getSkillLevel());
					if(skill != null)
					{
						boolean follow = _pet.getFollowStatus();
						_pet.setTarget(_owner);
						_pet.getAI().Cast(skill, _owner, false, false);
						_pet.setFollowStatus(follow);
					}
				}
			}
			catch(Throwable e)
			{
				if(Config.DEBUG)
					_log.warning("Pet [#" + getObjectId() + "] a heal task error has occurred: " + e);
			}
			_thinking = false;
		}
	}

	public synchronized void stopHealTask()
	{
		if(_healTask != null)
		{
			_healTask.cancel(false);
			_healTask = null;
			if(Config.DEBUG)
				_log.warning("Pet [#" + getObjectId() + "] Heal task stop");
		}
	}

	public synchronized void startHealTask()
	{
		if(_healTask != null)
			stopHealTask();

		if(_healTask == null && !isDead())
			_healTask = ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new HealTask(), 3000, 1000);
	}

	public static L2PetBabyInstance spawnPet(L2NpcTemplate template, L2Player owner, L2ItemInstance control)
	{
		return restore(control, template, owner);
	}

	@Override
	public synchronized void doDie(L2Character killer)
	{
		stopHealTask();
		super.doDie(killer);
	}

	@Override
	public void doRevive()
	{
		super.doRevive();
		startHealTask();
	}

	private static L2PetBabyInstance restore(L2ItemInstance control, L2NpcTemplate template, L2Player owner)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT objId, name, level, curHp, curMp, exp, sp, fed FROM pets WHERE item_obj_id=?");
			statement.setInt(1, control.getObjectId());
			rset = statement.executeQuery();
			if(!rset.next())
				return new L2PetBabyInstance(IdFactory.getInstance().getNextId(), template, owner, control);
			L2PetBabyInstance pet = new L2PetBabyInstance(rset.getInt("objId"), template, owner, control);
			pet.setExpPenalty((float) .1); // baby pets get 10% flat rate while fighting or not. Exp. pen. done at owner Exp. calc..
			pet.setRespawned(true);
			pet.setName(rset.getString("name"));
			pet.setLevel(rset.getByte("level"));
			pet.setCurrentHpMp(rset.getDouble("curHp"), rset.getInt("curMp"));
			pet.setCurrentCp(pet.getMaxCp());
			pet.setExp(rset.getInt("exp"));
			pet.setSp(rset.getInt("sp"));
			pet.setCurrentFed(rset.getInt("fed"));

			pet._data = PetDataTable.getInstance().getInfo(pet.getTemplate().npcId, pet.getLevel());
			pet._inventory = new PetInventory(pet);

			return pet;
		}
		catch(Exception e)
		{
			_log.warning("could not restore PetBaby data: " + e);
			e.printStackTrace();
			return null;
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public void setExpPenalty(float expPenalty)
	{
		_expPenalty = expPenalty;
	}

	public float getExpPenalty()
	{
		return _expPenalty;
	}

	@Override
	public synchronized void unSummon()
	{
		stopHealTask();
		super.unSummon();
	}

	public int getSkillLevel()
	{
		int lvl = getLevel();
		return lvl > 70 ? 7 + (lvl - 70) / 5 : lvl / 10;
	}
}