package l2r.gameserver.model.inventory.listeners;

import l2r.gameserver.model.Inventory;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.serverpackets.SkillList;
import l2r.gameserver.skills.Formulas;
import l2r.gameserver.templates.L2Item;
import l2r.gameserver.templates.L2Weapon;

/**
 * Добавление\удалениe скилов, прописанных предметам в sql или в xml.
 */
public final class ItemSkillsListener implements PaperdollListener
{
	Inventory _inv;

	public ItemSkillsListener(Inventory inv)
	{
		_inv = inv;
	}

	public void notifyUnequipped(int slot, L2ItemInstance item)
	{
		if(!_inv.isRefreshingListeners() && item.isShadowItem())
			item.notifyEquipped(false);

		L2Player player;

		if(_inv.getOwner().isPlayer)
			player = (L2Player) _inv.getOwner();
		else
			return;

		L2Skill[] itemSkills = null;
		L2Skill enchant4Skill = null;

		L2Item it = item.getItem();

		itemSkills = it.getAttachedSkills();

		if(it instanceof L2Weapon)
			enchant4Skill = ((L2Weapon) it).getEnchant4Skill();

		if(itemSkills != null)
			for(L2Skill itemSkill : itemSkills)
			{
				if(!itemSkill.isPassive())
					player.removeSkillFromShortCut(itemSkill.getId());
				player.removeSkill(itemSkill, false);
			}

		if(enchant4Skill != null)
			player.removeSkill(enchant4Skill, false);

		if(itemSkills != null || enchant4Skill != null)
		{
			player.sendPacket(new SkillList(player));
			player.updateStats();
		}
	}

	public void notifyEquipped(int slot, L2ItemInstance item)
	{
		if(!_inv.isRefreshingListeners() && item.isShadowItem())
			item.notifyEquipped(true);

		L2Player player;
		if(_inv.getOwner().isPlayer)
			player = (L2Player) _inv.getOwner();
		else
			return;

		L2Skill[] itemSkills = null;
		L2Skill enchant4Skill = null;

		L2Item it = item.getItem();

		itemSkills = it.getAttachedSkills();

		if(it instanceof L2Weapon && item.getEnchantLevel() >= 4)
			enchant4Skill = ((L2Weapon) it).getEnchant4Skill();

		if(itemSkills != null)
			for(L2Skill itemSkill : itemSkills)
			{
				player.addSkill(itemSkill, false);
				if(!itemSkill.isPassive())
				{
					long reuseDelay = Formulas.calcSkillReuseDelay(player, itemSkill);
					if(reuseDelay > 0)
					{
						player.disableSkill(itemSkill.getId(), reuseDelay);
						player.addSkillTimeStamp(itemSkill.getId(), reuseDelay);
					}
				}
			}

		if(enchant4Skill != null)
			player.addSkill(enchant4Skill, false);

		if(itemSkills != null || enchant4Skill != null)
		{
			player.sendPacket(new SkillList(player));
			player.updateStats();
		}
	}
}