package l2r.gameserver.model.quest;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javolution.util.FastList;
import javolution.util.FastMap;
import l2r.Config;
import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.gameserver.instancemanager.QuestManager;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.serverpackets.NpcHtmlMessage;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.tables.NpcTable;
import l2r.gameserver.templates.L2Item;
import l2r.gameserver.templates.L2NpcTemplate;
import l2r.util.Files;
import l2r.util.Rnd;

public class Quest
{
	public static String SOUND_ITEMGET = "ItemSound.quest_itemget";
	public static String SOUND_ACCEPT = "ItemSound.quest_accept";
	public static String SOUND_MIDDLE = "ItemSound.quest_middle";
	public static String SOUND_FINISH = "ItemSound.quest_finish";
	public static String SOUND_GIVEUP = "ItemSound.quest_giveup";
	public static String SOUND_TUTORIAL = "ItemSound.quest_tutorial";
	public static String SOUND_JACKPOT = "ItemSound.quest_jackpot";
	public static String SOUND_HORROR2 = "SkillSound5.horror_02";
	public static String SOUND_BEFORE_BATTLE = "Itemsound.quest_before_battle";
	public static String SOUND_FANFARE_MIDDLE = "ItemSound.quest_fanfare_middle";
	public static String SOUND_FANFARE2 = "ItemSound.quest_fanfare_2";
	public static String SOUND_BROKEN_KEY = "ItemSound2.broken_key";
	public static String SOUND_ENCHANT_SUCESS = "ItemSound3.sys_enchant_sucess";
	public static String SOUND_ENCHANT_FAILED = "ItemSound3.sys_enchant_failed";
	public static String SOUND_ED_CHIMES05 = "AmdSound.ed_chimes_05";
	public static String SOUND_ARMOR_WOOD_3 = "ItemSound.armor_wood_3";
	public static String SOUND_ITEM_DROP_EQUIP_ARMOR_CLOTH = "ItemSound.item_drop_equip_armor_cloth";

	public static class Drop
	{
		public int condition;
		public int maxcount;
		public int chance;

		public FastList<Short> itemList = new FastList<Short>();

		public Drop(Integer _condition, Integer _maxcount, Integer _chance)
		{
			condition = _condition;
			maxcount = _maxcount;
			chance = _chance;
		}

		public Drop addItem(Short item)
		{
			itemList.add(item);
			return this;
		}
	}

	public static class DropChance
	{
		private final int item_id, chance;
		private int base_count = 0;

		public DropChance(int _item_id, int _chance)
		{
			item_id = _item_id;
			while(_chance > 100)
			{
				_chance -= 100;
				base_count++;
			}
			chance = _chance;
		}

		public int getItemId()
		{
			return item_id;
		}

		public int getChance()
		{
			return chance;
		}

		public boolean Chance()
		{
			return Rnd.chance(chance);
		}

		/*
		 * С учетом рейтов
		 */
		public boolean Chance(L2Player player)
		{
			return Rnd.chance(chance * Config.RATE_QUESTS_REWARD * player.getBonus().RATE_QUESTS_REWARD);
		}

		public int getBaseCount()
		{
			return base_count;
		}

		/*
		 * С учетом рейтов
		 */
		public int getBaseCount(L2Player player)
		{
			return base_count * (int) (Config.RATE_QUESTS_REWARD * player.getBonus().RATE_QUESTS_REWARD);
		}

		public int getRewardCount()
		{
			return getBaseCount() + (Chance() ? 1 : 0);
		}

		/*
		 * С учетом рейтов
		 */
		public int getRewardCount(L2Player player)
		{
			return getBaseCount(player) + (Chance(player) ? 1 : 0);
		}
	}

	public static enum QuestEventType
	{
		MOB_TARGETED_BY_SKILL(true), // onSkillUse action triggered when a character uses a skill on a mob
		MOBGOTATTACKED(true), // onAttack action triggered when a mob attacked by someone
		MOBKILLED(true), // onKill action triggered when a mob killed.
		QUEST_START(true), // onTalk action from start npcs
		QUEST_TALK(true), // onTalk action from npcs participating in a quest
		NPC_FIRST_TALK(false);

		// control whether this event type is allowed for the same npc template
		// in multiple quests
		// or if the npc must be registered in at most one quest for the
		// specified event
		private boolean _allowMultipleRegistration;

		QuestEventType(boolean allowMultipleRegistration)
		{
			_allowMultipleRegistration = allowMultipleRegistration;
		}

		public boolean isMultipleRegistrationAllowed()
		{
			return _allowMultipleRegistration;
		}
	}

	protected static Logger _log = Logger.getLogger(Quest.class.getName());

	/** HashMap containing events from String value of the event */
	private static Map<String, Quest> _allEventsS = new FastMap<String, Quest>();

	/** HashMap containing lists of timers from the name of the timer */
	private static Map<String, FastList<QuestTimer>> _allEventTimers = new FastMap<String, FastList<QuestTimer>>();

	private ArrayList<Integer> _questitems = new ArrayList<Integer>();

	/**
	 * Этот метод для регистрации квестовых вещей, которые будут удалены
	 * при прекращении квеста, независимо от того, был он закончен или
	 * прерван. <strong>Добавлять сюда награды нельзя</strong>.
	 */
	public void addQuestItem(int[] ids)
	{
		for(int id : ids)
			if(id != 0)
				addQuestItem(id);
	}

	/**
	 * Этот метод для регистрации квестовых вещей, которые будут удалены
	 * при прекращении квеста, независимо от того, был он закончен или
	 * прерван. <strong>Добавлять сюда награды нельзя</strong>.
	 */
	public void addQuestItem(int id)
	{
		L2Item i = null;
		try
		{
			i = ItemTable.getInstance().getTemplate(id);
		}
		catch(Exception e)
		{
			System.out.println("Warning: unknown item " + i + " (" + id + ") in quest drop in " + getName());
		}

		if(i == null || i.getType2() != L2Item.TYPE2_QUEST)
			System.out.println("Warning: non-quest item " + i + " (" + id + ") in quest drop in " + getName());

		if(_questitems.contains(id))
			System.out.println("Warning: " + i + " (" + id + ") multiple times in quest drop in " + getName());

		_questitems.add(id);
	}

	public ArrayList<Integer> getItems()
	{
		return _questitems;
	}

	/**
	 * Update informations regarding quest in database.<BR>
	 * <U><I>Actions :</I></U><BR>
	 * <LI>Get ID state of the quest recorded in object qs</LI>
	 * <LI>Save in database the ID state (with or without the star) for the variable called "&lt;state&gt;" of the quest</LI>
	 * @param qs : QuestState
	 */
	public static void updateQuestInDb(QuestState qs)
	{
		updateQuestVarInDb(qs, "<state>", qs.getStateId());
	}

	/**
	 * Insert in the database the quest for the player.
	 * @param qs : QuestState pointing out the state of the quest
	 * @param var : String designating the name of the variable for the quest
	 * @param value : String designating the value of the variable for the quest
	 */
	public static void updateQuestVarInDb(QuestState qs, String var, String value)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("REPLACE INTO character_quests (char_id,name,var,value) VALUES (?,?,?,?)");
			statement.setInt(1, qs.getPlayer().getObjectId());
			statement.setString(2, qs.getQuest().getName());
			statement.setString(3, var);
			statement.setString(4, value);
			statement.executeUpdate();
		}
		catch(Exception e)
		{
			_log.log(Level.WARNING, "could not insert char quest:", e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/**
	 * Delete the player's quest from database.
	 * @param qs : QuestState pointing out the player's quest
	 */
	public static void deleteQuestInDb(QuestState qs)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM character_quests WHERE char_id=? AND name=?");
			statement.setInt(1, qs.getPlayer().getObjectId());
			statement.setString(2, qs.getQuest().getName());
			statement.executeUpdate();
		}
		catch(Exception e)
		{
			_log.log(Level.WARNING, "could not delete char quest:", e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/**
	 * Delete a variable of player's quest from the database.
	 * @param qs : object QuestState pointing out the player's quest
	 * @param var : String designating the variable characterizing the quest
	 */
	public static void deleteQuestVarInDb(QuestState qs, String var)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM character_quests WHERE char_id=? AND name=? AND var=?");
			statement.setInt(1, qs.getPlayer().getObjectId());
			statement.setString(2, qs.getQuest().getName());
			statement.setString(3, var);
			statement.executeUpdate();
		}
		catch(Exception e)
		{
			_log.log(Level.WARNING, "could not delete char quest:", e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/**
	 * Return collection view of the values contains in the allEventS
	 * @return Collection<Quest>
	 */
	public static Collection<Quest> findAllEvents()
	{
		return _allEventsS.values();
	}

	/**
	 * Add quests to the L2PCInstance of the player.<BR><BR>
	 * <U><I>Action : </U></I><BR>
	 * Add state of quests, drops and variables for quests in the HashMap _quest of L2Player
	 * @param player : Player who is entering the world
	 */
	public static void playerEnter(L2Player player)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		FiltredPreparedStatement invalidQuestData = null;
		FiltredPreparedStatement invalidQuestDataVar = null;
		ResultSet rset = null;
		try
		{
			// Get list of quests owned by the player from database
			con = L2DatabaseFactory.getInstance().getConnection();

			invalidQuestData = con.prepareStatement("DELETE FROM character_quests WHERE char_id=? and name=?");
			invalidQuestDataVar = con.prepareStatement("delete FROM character_quests WHERE char_id=? and name=? and var=?");
			statement = con.prepareStatement("SELECT name,value FROM character_quests WHERE char_id=? AND var=?");
			statement.setInt(1, player.getObjectId());
			statement.setString(2, "<state>");
			rset = statement.executeQuery();
			while(rset.next())
			{
				// Get ID of the quest and ID of its state
				String questId = rset.getString("name");
				String stateId = rset.getString("value");

				if(stateId.equalsIgnoreCase("Start")) // невзятый квест
				{
					invalidQuestData.setInt(1, player.getObjectId());
					invalidQuestData.setString(2, questId);
					invalidQuestData.executeUpdate();
					continue;
				}

				// Search quest associated with the ID
				Quest q = QuestManager.getQuest(questId);
				if(q == null)
				{
					if(Config.AUTODELETE_INVALID_QUEST_DATA)
					{
						invalidQuestData.setInt(1, player.getObjectId());
						invalidQuestData.setString(2, questId);
						invalidQuestData.executeUpdate();
					}
					else
						_log.warning("Unknown quest " + questId + " for player " + player.getName());
					continue;
				}

				// Create an object State containing the state of the quest
				State state = q._states.get(stateId);
				if(state == null)
				{
					if(Config.AUTODELETE_INVALID_QUEST_DATA)
					{
						invalidQuestData.setInt(1, player.getObjectId());
						invalidQuestData.setString(2, questId);
						invalidQuestData.executeUpdate();
					}
					else
						_log.warning("Unknown state " + state + " in quest " + questId + " for player " + player.getName());
					continue;
				}
				// Create a new QuestState for the player that will be added to the player's list of quests
				new QuestState(q, player, state);
			}
			invalidQuestData.close();
			DatabaseUtils.closeDatabaseSR(statement, rset);

			// Get list of quests owned by the player from the DB in order to add variables used in the quest.
			statement = con.prepareStatement("SELECT name,var,value FROM character_quests WHERE char_id=?");
			statement.setInt(1, player.getObjectId());
			rset = statement.executeQuery();
			while(rset.next())
			{
				String questId = rset.getString("name");
				String var = rset.getString("var");
				String value = rset.getString("value");
				// Get the QuestState saved in the loop before
				QuestState qs = player.getQuestState(questId);
				if(qs == null)
				{
					if(Config.AUTODELETE_INVALID_QUEST_DATA)
					{
						invalidQuestDataVar.setInt(1, player.getObjectId());
						invalidQuestDataVar.setString(2, questId);
						invalidQuestDataVar.setString(3, var);
						invalidQuestDataVar.executeUpdate();
					}
					else
						_log.warning("Lost variable " + var + " in quest " + questId + " for player " + player.getName());
					continue;
				}
				// Add parameter to the quest
				qs.setInternal(var, value);
			}
		}
		catch(Exception e)
		{
			_log.log(Level.WARNING, "could not insert char quest:", e);
		}
		finally
		{
			DatabaseUtils.closeStatement(invalidQuestData);
			DatabaseUtils.closeStatement(invalidQuestDataVar);
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		// events
		for(String name : _allEventsS.keySet())
			player.processQuestEvent(name, "enter");
	}

	protected final String _descr;

	protected final String _name;

	protected final boolean _party;

	protected final int _questId;

	protected State CREATED;
	protected State STARTED;
	protected State COMPLETED;

	@Deprecated
	protected State MIDDLE;

	protected Map<String, State> _states = new FastMap<String, State>();

	/**
	 * (Constructor)Add values to class variables and put the quest in HashMaps.
	 * @param questId : int pointing out the ID of the quest
	 * @param name : String corresponding to the name of the quest
	 * @param descr : String for the description of the quest
	 * @param party : boolean for the party quest drop
	 */
	public Quest(int questId, String name, String descr, boolean party)
	{
		_questId = questId;
		_name = name;
		_descr = descr;
		_party = party;

		CREATED = new State("Start", this);
		STARTED = new State("Started", this);
		COMPLETED = new State("Completed", this);

		MIDDLE = new State("Middle", this);

		_initialState = CREATED;

		if(questId != 0)
			QuestManager.addQuest(this);
		else
			_allEventsS.put(name, this);
	}

	/**
	 * Add this quest to the list of quests that the passed mob will respond to
	 * for Attack Events.<BR>
	 * <BR>
	 *
	 * @param attackId
	 * @return int : attackId
	 */
	public L2NpcTemplate addAttackId(int attackId)
	{
		return addEventId(attackId, Quest.QuestEventType.MOBGOTATTACKED);
	}

	/**
	 * Add this quest to the list of quests that the passed mob will respond to
	 * for the specified Event type.<BR>
	 * <BR>
	 *
	 * @param npcId : id of the NPC to register
	 * @param eventType : type of event being registered
	 * @return int : npcId
	 */
	public L2NpcTemplate addEventId(int npcId, QuestEventType eventType)
	{
		try
		{
			L2NpcTemplate t = NpcTable.getTemplate(npcId);
			if(t != null)
				t.addQuestEvent(eventType, this);
			return t;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return null;
		}
	}

	public void addKillId(int[] killIds)
	{
		for(int killid : killIds)
			addEventId(killid, Quest.QuestEventType.MOBKILLED);
	}

	/**
	 * Add this quest to the list of quests that the passed mob will respond to
	 * for Kill Events.<BR>
	 * <BR>
	 *
	 * @param killId
	 * @return int : killId
	 */
	public L2NpcTemplate addKillId(int killId)
	{
		return addEventId(killId, Quest.QuestEventType.MOBKILLED);
	}

	/**
	 * Add this quest to the list of quests that the passed npc will respond to
	 * for Skill-Use Events.<BR>
	 * <BR>
	 *
	 * @param npcId : ID of the NPC
	 * @return int : ID of the NPC
	 */
	public L2NpcTemplate addSkillUseId(int npcId)
	{
		return addEventId(npcId, Quest.QuestEventType.MOB_TARGETED_BY_SKILL);
	}

	/**
	 * Add the quest to the NPC's startQuest
	 * Вызывает addTalkId
	 *
	 * @param npcId
	 * @return L2NpcTemplate : Start NPC
	 */
	public L2NpcTemplate addStartNpc(int npcId)
	{
		addTalkId(npcId);
		return addEventId(npcId, Quest.QuestEventType.QUEST_START);
	}

	/**
	 * Add a state to the quest
	 * @param state
	 * @return state added
	 */
	public State addState(State state)
	{
		_states.put(state.getName(), state);
		return state;
	}

	/**
	* Add the quest to the NPC's first-talk (default action dialog)
	*
	* @param npcId
	 * @return L2NpcTemplate : Start NPC
	 */
	public L2NpcTemplate addFirstTalkId(int npcId)
	{
		return addEventId(npcId, Quest.QuestEventType.NPC_FIRST_TALK);
	}

	public void addFirstTalkId(int[] npcIds)
	{
		for(int npcId : npcIds)
			addFirstTalkId(npcId);
	}

	/**
	 * Add this quest to the list of quests that the passed npc will respond to
	 * for Talk Events.<BR>
	 * <BR>
	 *
	 * @param talkId : ID of the NPC
	 * @return int : ID of the NPC
	 */
	public L2NpcTemplate addTalkId(int talkId)
	{
		return addEventId(talkId, Quest.QuestEventType.QUEST_TALK);
	}

	public void addTalkId(int[] talkIds)
	{
		for(int talkId : talkIds)
			addTalkId(talkId);
	}

	public void cancelQuestTimer(String name, L2NpcInstance npc, L2Player player)
	{
		QuestTimer timer = getQuestTimer(name, npc, player);
		if(timer != null)
			timer.cancel();
	}

	/**
	 * Return description of the quest
	 * @return String
	 */
	public String getDescr()
	{
		return _descr;
	}

	/**
	 * Return name of the quest
	 * @return String
	 */
	public String getName()
	{
		return _name;
	}

	/**
	 * Return ID of the quest
	 * @return int
	 */
	public int getQuestIntId()
	{
		return _questId;
	}

	public QuestTimer getQuestTimer(String name, L2NpcInstance npc, L2Player player)
	{
		if(_allEventTimers.get(name) == null)
			return null;
		for(QuestTimer timer : _allEventTimers.get(name))
			if(timer.isMatch(this, name, npc, player))
				return timer;
		return null;
	}

	public FastList<QuestTimer> getQuestTimers(String name)
	{
		return _allEventTimers.get(name);
	}

	/**
	 * Return party state of quest
	 * @return String
	 */
	public boolean isParty()
	{
		return _party;
	}

	/**
	 * Add a new QuestState to the database and return it.
	 * @param player
	 * @return QuestState : QuestState created
	 */
	public QuestState newQuestState(L2Player player)
	{
		QuestState qs = new QuestState(this, player, getInitialState());
		Quest.updateQuestInDb(qs);
		return qs;
	}

	@Deprecated
	protected State _initialState;

	@Deprecated
	public State getInitialState()
	{
		return _initialState;
	}

	@Deprecated
	public void setInitialState(State state)
	{
		_initialState = state;
	}

	public void notifyAttack(L2NpcInstance npc, QuestState qs)
	{
		String res = null;
		try
		{
			res = onAttack(npc, qs);
		}
		catch(Exception e)
		{
			showError(qs.getPlayer(), e);
			return;
		}
		showResult(qs.getPlayer(), res);
	}

	public void notifyDeath(L2NpcInstance killer, L2Character victim, QuestState qs)
	{
		String res = null;
		try
		{
			res = onDeath(killer, victim, qs);
		}
		catch(Exception e)
		{
			showError(qs.getPlayer(), e);
			return;
		}
		showResult(qs.getPlayer(), res);
	}

	public void notifyEvent(String event, QuestState qs)
	{
		String res = null;
		try
		{
			res = onEvent(event, qs);
		}
		catch(Exception e)
		{
			showError(qs.getPlayer(), e);
			return;
		}
		showResult(qs.getPlayer(), res);
	}

	public void notifyKill(L2NpcInstance npc, QuestState qs)
	{
		String res = null;
		try
		{
			res = onKill(npc, qs);
		}
		catch(Exception e)
		{
			showError(qs.getPlayer(), e);
			return;
		}
		showResult(qs.getPlayer(), res);
	}

	/**
	 * Override the default NPC dialogs when a quest defines this for the given NPC
	 */
	public final boolean notifyFirstTalk(L2NpcInstance npc, L2Player player)
	{
		String res = null;
		try
		{
			res = onFirstTalk(npc, player);
		}
		catch(Exception e)
		{
			showError(player, e);
			return true;
		}
		// if the quest returns text to display, display it. Otherwise, use the default npc text.
		return showResult(player, res);
	}

	public boolean notifyTalk(L2NpcInstance npc, QuestState qs)
	{
		String res = null;
		try
		{
			res = onTalk(npc, qs);
		}
		catch(Exception e)
		{
			showError(qs.getPlayer(), e);
			return true;
		}
		return showResult(qs.getPlayer(), res);
	}

	@SuppressWarnings("unused")
	public String onAttack(L2NpcInstance npc, QuestState qs)
	{
		return null;
	}

	@SuppressWarnings("unused")
	public String onDeath(L2NpcInstance killer, L2Character victim, QuestState qs)
	{
		return null;
	}

	@SuppressWarnings("unused")
	public String onEvent(String event, QuestState qs)
	{
		return null;
	}

	@SuppressWarnings("unused")
	public String onKill(L2NpcInstance npc, QuestState qs)
	{
		return null;
	}

	@SuppressWarnings("unused")
	public String onFirstTalk(L2NpcInstance npc, L2Player player)
	{
		return null;
	}

	@SuppressWarnings("unused")
	public String onTalk(L2NpcInstance npc, QuestState qs)
	{
		return null;
	}

	public void removeQuestTimer(QuestTimer timer)
	{
		if(timer == null)
			return;
		FastList<QuestTimer> timers = getQuestTimers(timer.getName());
		if(timers == null)
			return;
		timers.remove(timer);
	}

	/**
	 * Show message error to player who has an access level greater than 0
	 * @param player : L2PcInstance
	 * @param t : Throwable
	 */
	private void showError(L2Player player, Throwable t)
	{
		_log.log(Level.WARNING, "", t);
		if(player.getPlayerAccess().IsGM)
		{
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw);
			t.printStackTrace(pw);
			pw.close();
			String res = "<html><body><title>Script error</title>" + sw.toString() + "</body></html>";
			showResult(player, res);
		}
	}

	/**
	 * Show HTML file to client
	 * @param fileName
	 * @return String : message sent to client
	 */
	public String showHtmlFile(L2Player player, String fileName)
	{
		return showHtmlFile(player, fileName, null, (String[]) null); //(String[]) - затык для вызова корректного метода.
	}

	public String showHtmlFile(L2Player player, String fileName, String toReplace, String replaceWith)
	{
		return showHtmlFile(player, fileName, new String[] { toReplace }, new String[] { replaceWith });
	}

	public String showHtmlFile(L2Player player, String fileName, String toReplace[], String replaceWith[])
	{
		String content = null;

		// for scripts
		if(fileName.contains("/"))
			content = Files.read(fileName, player);
		else
		{
			String _path = getClass().toString();
			_path = _path.substring(6, _path.lastIndexOf(".")) + ".";
			content = Files.read("data/scripts/" + _path.replace(".", "/") + fileName, player);
		}

		if(content == null)
			content = "Can't find file '" + fileName + "'";

		if(player != null && player.getTarget() != null)
			content = content.replaceAll("%objectId%", String.valueOf(player.getTarget().getObjectId()));

		// Make a replacement inside before sending html to client
		if(toReplace != null && replaceWith != null && toReplace.length == replaceWith.length)
			for(int i = 0; i < toReplace.length; i++)
				content = content.replaceAll(toReplace[i], replaceWith[i]);

		// Send message to client if message not empty
		if(content != null && player != null)
		{
			NpcHtmlMessage npcReply = new NpcHtmlMessage(5);
			npcReply.setHtml(content);
			player.sendPacket(npcReply);
		}

		return content;
	}

	/**
	 * Show a message to player.<BR><BR>
	 * <U><I>Concept : </I></U><BR>
	 * 3 cases are managed according to the value of the parameter "res" :<BR>
	 * <LI><U>"res" ends with string ".html" :</U> an HTML is opened in order to be shown in a dialog box</LI>
	 * <LI><U>"res" starts with "<html>" :</U> the message hold in "res" is shown in a dialog box</LI>
	 * <LI><U>otherwise :</U> the message hold in "res" is shown in chat box</LI>
	 * @param qs : QuestState
	 * @param res : String pointing out the message to show at the player
	 * @return boolean
	 */
	private boolean showResult(L2Player player, String res)
	{
		if(res == null)
			return true;
		if(res.isEmpty())
			return false;
		if(res.startsWith("no_quest") || res.equalsIgnoreCase("noquest") || res.equalsIgnoreCase("no-quest"))
			showHtmlFile(player, "data/html/no-quest.htm");
		else if(res.equalsIgnoreCase("completed"))
			showHtmlFile(player, "data/html/completed-quest.htm");
		else if(res.endsWith(".htm"))
			showHtmlFile(player, res);
		else
		{
			NpcHtmlMessage npcReply = new NpcHtmlMessage(5);
			npcReply.setHtml(res);
			player.sendPacket(npcReply);
		}
		return true;
	}

	/**
	 * Add a timer to the quest, if it doesn't exist already
	 * @param name: name of the timer (also passed back as "event" in onAdvEvent)
	 * @param time: time in ms for when to fire the timer
	 * @param npc: npc associated with this timer (can be null)
	 * @param player: player associated with this timer (can be null)
	 */
	public void startQuestTimer(String name, long time, L2NpcInstance npc, L2Player player)
	{
		// Add quest timer if timer doesn't already exist
		FastList<QuestTimer> timers = getQuestTimers(name);
		if(timers == null)
		{
			timers = new FastList<QuestTimer>();
			timers.add(new QuestTimer(this, name, time, npc, player));
			_allEventTimers.put(name, timers);
		}
		// a timer with this name exists, but may not be for the same set of npc and player
		else // if there exists a timer with this name, allow the timer only if the [npc, player] set is unique
		// nulls act as wildcards
		if(getQuestTimer(name, npc, player) == null)
			timers.add(new QuestTimer(this, name, time, npc, player));
	}

	protected String str(long i)
	{
		return String.valueOf(i);
	}

	protected boolean isdigit(String s)
	{
		try
		{
			Integer.parseInt(s);
		}
		catch(NumberFormatException e)
		{
			return false;
		}
		return true;
	}
}