package l2r.gameserver.model.quest;

import java.util.List;

import javolution.util.FastList;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2ItemInstance;

public class QuestDropInfo
{
	public class DropInfo
	{
		public long dropItemCount;
		public int dropItemID;
		public int dropItemObjID;

		/**
		 * Constructor of DropInfo that initialzes class variables
		 * @param itemID : int designating the ID of the item
		 * @param itemCount : int designating the quantity of items needed for quest
		 * @param itemObjID : int designating the ID of the item in the inventory of the player
		 */
		public DropInfo(int itemID, long itemCount, int itemObjID)
		{
			dropItemID = itemID;
			dropItemObjID = itemObjID;
			dropItemCount = itemCount;
		}
	}

	public List<DropInfo> dropList;

	/**
	 * Add informations for dropped items in the inventory of the player.
	 * @param pcInstance : L2Player designating the player
	 * @param questName : String designating the name of the quest
	 */
	public QuestDropInfo(L2Player pcInstance, String questName)
	{
		// Get the QuestState and the State from the name of the quest
		QuestState questState = pcInstance.getQuestState(questName);
		dropList = new FastList<DropInfo>();
		if(questState == null)
			return;

		if(questState.getQuest().getItems() != null)
			for(Integer dropID : questState.getQuest().getItems())
			{
				L2ItemInstance dropObj = pcInstance.getInventory().getItemByItemId(dropID);
				if(dropObj == null) //If player doesn't have this quest item(doesn't kill need npc? or other) then skip it
					continue;
				long dropCount = questState.getQuestItemsCount(dropID);
				int dropObjID = pcInstance.getInventory().getItemByItemId(dropID).getObjectId();
				// Add info for the dropped item in the player's inventory
				dropList.add(new DropInfo(dropID, dropCount, dropObjID));
			}
	}
}
