package l2r.gameserver.model.quest;

import java.util.List;

import javolution.util.FastList;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.model.L2Player;

public class QuestStateManager
{
	public class ScheduleTimerTask implements Runnable
	{
		public void run()
		{
			try
			{
				cleanUp();
				ThreadPoolManager.getInstance().scheduleGeneral(new ScheduleTimerTask(), 60000);
			}
			catch(Throwable t)
			{}
		}
	}

	private static QuestStateManager _Instance;

	public static QuestStateManager getInstance()
	{
		if(_Instance == null)
			_Instance = new QuestStateManager();
		return _Instance;
	}

	private List<QuestState> _QuestStates = new FastList<QuestState>();

	public QuestStateManager()
	{
		ThreadPoolManager.getInstance().scheduleGeneral(new ScheduleTimerTask(), 60000);
	}

	/**
	 * Add QuestState for the specified player instance
	 */
	public void addQuestState(Quest quest, L2Player player, State state)
	{
		QuestState qs = getQuestState(player);
		if(qs == null)
			new QuestState(quest, player, state);
	}

	/**
	 * Remove all QuestState for all player instance that does not exist
	 */
	public void cleanUp()
	{
		for(int i = getQuestStates().size() - 1; i >= 0; i--)
			if(getQuestStates().get(i).getPlayer() == null)
				getQuestStates().remove(i);
	}

	/**
	 * Return QuestState for specified player instance
	 */
	public QuestState getQuestState(L2Player player)
	{
		for(int i = 0; i < getQuestStates().size(); i++)
			if(getQuestStates().get(i).getPlayer() != null && getQuestStates().get(i).getPlayer().getObjectId() == player.getObjectId())
				return getQuestStates().get(i);

		return null;
	}

	/**
	 * Return all QuestState
	 */
	public List<QuestState> getQuestStates()
	{
		if(_QuestStates == null)
			_QuestStates = new FastList<QuestState>();
		return _QuestStates;
	}
}
