package l2r.gameserver.serverpackets;

import java.util.ArrayList;

import javolution.util.FastList;
import l2r.gameserver.model.L2TradeList;
import l2r.gameserver.model.instances.L2ItemInstance;

public class BuyList extends L2GameServerPacket
{
	private static final String _S__07_BUYLIST = "[S] 07 BuyList";
	private int _listId;
	private FastList<L2ItemInstance> _list;
	private int _money;
	private double _TaxRate = 0;

	public BuyList(L2TradeList list, int currentMoney)
	{
		_listId = list.getListId();
		_list = list.getItems();
		_money = currentMoney;
	}

	public BuyList(L2TradeList list, int currentMoney, double taxRate)
	{
		_listId = list.getListId();
		_list = list.getItems();
		_money = currentMoney;
		_TaxRate = taxRate;
	}

	public BuyList(ArrayList<L2ItemInstance> lst, int listId, int currentMoney)
	{
		_listId = listId;
		_list = new FastList<L2ItemInstance>(lst);
		_money = currentMoney;
	}

	@Override
	protected final void writeImpl()
	{
		writeC(0x11);
		writeD(_money); // current money
		writeD(_listId);

		writeH(_list.size());

		for(FastList.Node<L2ItemInstance> n = _list.head(), end = _list.tail(); (n = n.getNext()) != end;)
		{
			L2ItemInstance item = n.getValue();
			if(item.getCountToSell() == 0 && item.getMaxCountToSell() != 0)
				continue;

			writeH(item.getItem().getType1()); // item type1
			writeD(item.getObjectId());
			writeD(item.getItemId());

			// А не пора ли обновить количество лимитированных предметов в трейд листе?
			if(item.getCountToSell() < item.getMaxCountToSell() && item.getLastRechargeTime() + item.getRechargeTime() < System.currentTimeMillis() / 60000)
			{
				item.setLastRechargeTime((int) (System.currentTimeMillis() / 60000));
				item.setCountToSell(item.getMaxCountToSell());
			}

			writeD(item.getCountToSell()); // max amount of items that a player can buy at a time (with this itemid)
			writeH(item.getItem().getType2()); // item type2
			writeH(item.getCustomType1()); // ?
			writeD(item.getBodyPart()); // rev 415  slot    0006-lr.ear  0008-neck  0030-lr.finger  0040-head  0080-??  0100-l.hand  0200-gloves  0400-chest  0800-pants  1000-feet  2000-??  4000-r.hand  8000-r.hand
			writeH(item.getEnchantLevel()); // enchant level
			writeH(item.getCustomType2()); // ?
			writeH(0x00); // unknown
			writeD((int) (item.getPriceToSell() * (1 + _TaxRate)));
		}
	}

	@Override
	public String getType()
	{
		return _S__07_BUYLIST;
	}
}