package l2r.gameserver.serverpackets;

import java.util.Calendar;
import java.util.logging.Logger;

import l2r.gameserver.model.L2Alliance;
import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.entity.ClanHall;
import l2r.gameserver.model.entity.siege.SiegeUnit;
import l2r.gameserver.tables.ClanTable;

/**
 * Shows the Siege Info<BR>
 * <BR>
 * packet type id 0xc9<BR>
 * format: cdddSSdSdd<BR>
 * <BR>
 * c = c9<BR>
 * d = CastleID<BR>
 * d = Show Owner Controls (0x00 default || >=0x02(mask?) owner)<BR>
 * d = Owner ClanID<BR>
 * S = Owner ClanName<BR>
 * S = Owner Clan LeaderName<BR>
 * d = Owner AllyID<BR>
 * S = Owner AllyName<BR>
 * d = current time (seconds)<BR>
 * d = Siege time (seconds) (0 for selectable)<BR>
 * d = (UNKNOW) Siege Time Select Related
 */
public class CastleSiegeInfo extends L2GameServerPacket
{
	private static final String _S__C9_SIEGEINFO = "[S] C9 SiegeInfo";
	private static Logger _log = Logger.getLogger(CastleSiegeInfo.class.getName());
	private SiegeUnit _Castle;
	private ClanHall _ClanHall;
	private long startTime;
	private L2Player activeChar;
	private int _id;
	private int is_owner;
	private int _owner;
	private String owner_name;
	private String leader_name;
	private String ally_name;
	private int ally_id;

	public CastleSiegeInfo(SiegeUnit castle)
	{
		_Castle = castle;
	}

	public CastleSiegeInfo(ClanHall clanhall, long time)
	{
		_ClanHall = clanhall;
		startTime = time;
	}

	@Override
	final public void runImpl()
	{
		activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		owner_name = "";
		leader_name = "";
		ally_name = "";
		ally_id = 0;

		if(_Castle != null)
		{
			_id = _Castle.getId();
			is_owner = _Castle.getOwnerId() == activeChar.getClanId() && activeChar.isClanLeader() ? 0x01 : 0x00;
			_owner = _Castle.getOwnerId();
			if(_Castle.getOwnerId() > 0)
			{
				L2Clan owner = ClanTable.getInstance().getClan(_Castle.getOwnerId());
				if(owner != null)
				{
					owner_name = owner.getName();
					leader_name = owner.getLeaderName();
					if(owner.getAllyId() != 0)
					{
						L2Alliance alliance = ClanTable.getInstance().getAlliance(owner.getAllyId());
						ally_id = alliance.getAllyId();
						ally_name = alliance.getAllyName();
					}
				}
				else
					_log.warning("Null owner for castle: " + _Castle.getName());
			}
		}
		else
		{
			_id = _ClanHall.getId();
			is_owner = _ClanHall.getOwnerId() == activeChar.getClanId() && activeChar.isClanLeader() ? 0x01 : 0x00;
			_owner = _ClanHall.getOwnerId();
			if(_ClanHall.getOwnerId() > 0)
			{
				L2Clan owner = ClanTable.getInstance().getClan(_ClanHall.getOwnerId());
				if(owner != null)
				{
					owner_name = owner.getName();
					leader_name = owner.getLeaderName();
					if(owner.getAllyId() != 0)
					{
						L2Alliance alliance = ClanTable.getInstance().getAlliance(owner.getAllyId());
						ally_id = alliance.getAllyId();
						ally_name = alliance.getAllyName();
					}
				}
				else
					_log.warning("Null owner for clanhall: " + _ClanHall.getName());
			}
		}

		if(_Castle != null)
			startTime = (int) (_Castle.getSiege().getSiegeDate().getTimeInMillis() / 1000);
	}

	@Override
	protected final void writeImpl()
	{
		if(activeChar == null)
			return;

		writeC(0xC9);
		writeD(_id);
		writeD(is_owner);
		writeD(_owner);

		writeS(owner_name); // Clan Name
		writeS(leader_name); // Clan Leader Name
		writeD(ally_id); // Ally ID
		writeS(ally_name); // Ally Name
		writeD((int) (Calendar.getInstance().getTimeInMillis() / 1000));
		writeD((int) startTime);
		writeD(0x00); //number of choices?
	}

	@Override
	public String getType()
	{
		return _S__C9_SIEGEINFO;
	}
}