package l2r.gameserver.serverpackets;

import java.util.HashMap;
import java.util.logging.Logger;

import l2r.Config;
import l2r.gameserver.instancemanager.CursedWeaponsManager;
import l2r.gameserver.model.Inventory;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2CubicInstance;
import l2r.util.Location;

public class CharInfo extends L2GameServerPacket
{
	private static final Logger _log = Logger.getLogger(CharInfo.class.getName());

	private static final String _S__03_CHARINFO = "[S] 03 CharInfo";
	private L2Player _cha;
	private Inventory _inv;
	private int _mAtkSpd, _pAtkSpd;
	private int _runSpd, _walkSpd, _flRunSpd, _flWalkSpd, _flyRunSpd, _flyWalkSpd, _swimRunSpd, _swimWalkSpd;
	private float _moveMultiplier;
	private Location _loc, _fishLoc;
	private String _name, _title;
	private int _objId, _race, _sex, base_class, pvp_flag, karma, rec_have, rec_left;
	private float speed_move, speed_atack, col_radius, col_height;
	private int hair_style, hair_color, face, abnormal_effect;
	private int clan_id, clan_crest_id, large_clan_crest_id, ally_id, ally_crest_id, class_id;
	private byte _sit, _run, _combat, _dead, _invis, private_store, _enchant;
	private byte _team, _noble, _hero, _fishing, mount_type;
	private int plg_class, pledge_type, clan_rep_score, cw_level, mount_id;
	private int _nameColor, title_color;
	private HashMap<Integer, L2CubicInstance> cubics;
	private boolean can_writeImpl = false;

	protected boolean logHandled()
	{
		return true;
	}

	public CharInfo(L2Player cha)
	{
		if(cha == null || cha.isInvisible())
			return;
		_cha = cha;
		_inv = cha.getInventory();
		_mAtkSpd = _cha.getMAtkSpd();
		_pAtkSpd = _cha.getPAtkSpd();
		_moveMultiplier = _cha.getMovementSpeedMultiplier();
		_runSpd = (int) (_cha.getRunSpeed() / _moveMultiplier);
		_walkSpd = (int) (_cha.getWalkSpeed() / _moveMultiplier);
		_flRunSpd = _runSpd;
		_flWalkSpd = _walkSpd;
		_swimRunSpd = _cha.getSwimSpeed();
		_swimWalkSpd = _cha.getSwimSpeed();
	}

	@Override
	final public void runImpl()
	{
		if(_cha == null || _cha.isInvisible() || _cha.isDeleting())
			return;

		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(activeChar.equals(_cha))
		{
			_log.severe("You cant send CharInfo about his character to active user!!!");
			return;
		}

		if(_cha.isPolymorphed())
		{
			activeChar.sendPacket(new NpcInfoPoly(_cha, activeChar));
			return;
		}

		if(_cha.isCursedWeaponEquipped())
		{
			_name = _cha.getName();
			_title = "";
			clan_id = 0;
			clan_crest_id = 0;
			ally_id = 0;
			ally_crest_id = 0;
			large_clan_crest_id = 0;
			cw_level = CursedWeaponsManager.getInstance().getLevel(_cha.getCursedWeaponEquippedId());
		}
		else
		{
			_name = _cha.getName();
			_title = _cha.getTitle();
			clan_id = _cha.getClanId();
			clan_crest_id = _cha.getClanCrestId();
			ally_id = _cha.getAllyId();
			ally_crest_id = _cha.getAllyCrestId();
			large_clan_crest_id = _cha.getClanCrestLargeId();
			cw_level = 0;
		}

		if(_cha.isMounted())
		{
			_enchant = 0;
			mount_id = _cha.getMountNpcId() + 1000000;
			mount_type = (byte) _cha.getMountType();
		}
		else
		{
			_enchant = (byte) _cha.getEnchantEffect();
			mount_id = 0;
			mount_type = 0;
		}

		_inv = _cha.getInventory();
		_mAtkSpd = _cha.getMAtkSpd();
		_pAtkSpd = _cha.getPAtkSpd();
		_moveMultiplier = _cha.getMovementSpeedMultiplier();
		_runSpd = (int) (_cha.getRunSpeed() / _moveMultiplier);
		_walkSpd = (int) (_cha.getWalkSpeed() / _moveMultiplier);
		_flRunSpd = _runSpd;
		_flWalkSpd = _walkSpd;
		_swimWalkSpd = _cha.getSwimSpeed();
        _swimRunSpd = _cha.getSwimSpeed();
		_loc = _cha.getLoc();
		_objId = _cha.getObjectId();
		_race = _cha.getBaseTemplate().race.ordinal();
		_sex = _cha.getSex();
		base_class = _cha.getBaseClass();
		pvp_flag = _cha.getPvpFlag();
		karma = _cha.getKarma();
		speed_move = _cha.getMovementSpeedMultiplier();
		speed_atack = _cha.getAttackSpeedMultiplier();
		col_radius = _cha.getColRadius();
		col_height = _cha.getColHeight();
		hair_style = _cha.getHairStyle();
		hair_color = _cha.getHairColor();
		face = _cha.getFace();
		if(clan_id > 0 && _cha.getClan() != null)
			clan_rep_score = _cha.getClan().getReputationScore();
		else
			clan_rep_score = 0;
		_sit = _cha.isSitting() ? (byte) 0 : (byte) 1; // standing = 1 sitting = 0
		_run = _cha.isRunning() ? (byte) 1 : (byte) 0; // running = 1 walking = 0
		_combat = _cha.isInCombat() ? (byte) 1 : (byte) 0;
		_dead = _cha.isAlikeDead() ? (byte) 1 : (byte) 0;
		_invis = _cha.isInvisible() ? (byte) 1 : (byte) 0; // invisible = 1 visible = 0
		private_store = (byte) _cha.getPrivateStoreType(); // 1 - sellshop
		cubics = _cha.getCubics();
		abnormal_effect = _cha.getAbnormalEffect();
		rec_left = _cha.getRecomLeft();
		rec_have = _cha.getPlayerAccess().IsGM ? 0 : _cha.getRecomHave();
		class_id = _cha.getClassId().getId();
		_team = (byte) _cha.getTeam(); // team circle around feet 1 = Blue, 2 = red
		_noble = _cha.isNoble() ? (byte) 1 : (byte) 0; // 0x01: symbol on char menu ctrl+I
		_hero = _cha.isHero() || _cha.isGM() && Config.GM_HERO_AURA ? (byte) 1 : (byte) 0; // 0x01: Hero Aura
		_fishing = _cha.isFishing() ? (byte) 1 : (byte) 0;
		_fishLoc = _cha.getFishLoc();
		_nameColor = _cha.getNameColor(); // New C5
		plg_class = _cha.getPledgeClass();
		pledge_type = _cha.getPledgeType();
		title_color = _cha.getTitleColor();
		can_writeImpl = true;
	}

	@Override
	protected final void writeImpl()
	{
		if(!can_writeImpl)
			return;

		writeC(0x03);
		writeD(_loc.getX());
		writeD(_loc.getY());
		writeD(_loc.getZ());
		writeD(_loc.h); //?
		writeD(_objId);
		writeS(_name);
		writeD(_race);
		writeD(_sex);
		writeD(base_class);

		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_DHAIR));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_HEAD));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_RHAND));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_LHAND));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_GLOVES));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_CHEST));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_LEGS));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_FEET));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_BACK));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_LRHAND));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_HAIR));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_FACE));

			//KamaeL packets
			if(getClient().getRevision() >= 817)
			{
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
			}

		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_DHAIR));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_HEAD));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_RHAND));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_LHAND));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_GLOVES));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_CHEST));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_LEGS));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_FEET));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_BACK));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_LRHAND));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_HAIR));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_FACE));

			//KamaeL packets
			if(getClient().getRevision() >= 817)
			{
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
			}

		writeD(pvp_flag);
		writeD(karma);

		writeD(_mAtkSpd);
		writeD(_pAtkSpd);

		writeD(pvp_flag);
		writeD(karma);

		writeD(_runSpd);
		writeD(_walkSpd);
		writeD(_swimWalkSpd/* 0x32 */); // swimspeed
		writeD(_swimRunSpd/* 0x32 */); // swimspeed
		writeD(_flRunSpd);
		writeD(_flWalkSpd);
		writeD(_flyRunSpd);
		writeD(_flyWalkSpd);
		writeF(speed_move); // _cha.getProperMultiplier()
		writeF(speed_atack); // _cha.getAttackSpeedMultiplier()
		writeF(col_radius);
		writeF(col_height);
		writeD(hair_style);
		writeD(hair_color);
		writeD(face);
		writeS(_title);
		writeD(clan_id);
		writeD(clan_crest_id);
		writeD(ally_id);
		writeD(ally_crest_id);

		writeD(0);

		writeC(_sit);
		writeC(_run);
		writeC(_combat);
		writeC(_dead);
		writeC(_invis);
		writeC(mount_type); // 1-on Strider, 2-on Wyvern, 3-on Great Wolf, 0-no mount
		writeC(private_store);

		if(cubics == null)
			writeH(0);
		else
		{
			writeH(cubics.size());
			for(int id : cubics.keySet())
				writeH(id);
		}

		writeC(0x00); // find party members
		writeD(abnormal_effect);
		writeC(rec_left);
		writeH(rec_have);
		writeD(mount_id);
		writeD(class_id);
		writeD(0); //?
		writeC(_enchant);

		writeC(_team);
		writeD(large_clan_crest_id);
		writeC(_noble);
		writeC(_hero);

		writeC(_fishing);
		writeD(_fishLoc.getX());
		writeD(_fishLoc.getY());
		writeD(_fishLoc.getZ());

		writeD(_nameColor);
		writeD(_loc.h);
		writeD(plg_class);
		writeD(pledge_type);
		writeD(title_color);
		writeD(cw_level);
		writeD(clan_rep_score);
	}

	@Override
	public String getType()
	{
		return _S__03_CHARINFO + ": " + _cha;
	}
}