package l2r.gameserver.serverpackets;

import javolution.util.FastList;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.base.L2EnchantSkillLearn;
import l2r.gameserver.serverpackets.ExEnchantSkillList.EnchantSkillType;
import l2r.gameserver.tables.SkillTreeTable;

public class ExEnchantSkillInfo extends L2GameServerPacket
{
	private static final String _S__FE_2A_EXENCHANTSKILLINFO = "[S] FE:2a ExEnchantSkillInfo";
	private FastList<SkillEnchantDetailElement> _routes;

	private final int _id;
	private final EnchantSkillType _type;
	private final int _xpSpCostMultiplier;

	public ExEnchantSkillInfo(EnchantSkillType type, int id)
	{
		_routes = new FastList<SkillEnchantDetailElement>();
		_id = id;
		_type = type;
		_xpSpCostMultiplier = type == EnchantSkillType.SAFE ? SkillTreeTable.SAFE_ENCHANT_COST_MULTIPLIER : SkillTreeTable.NORMAL_ENCHANT_COST_MULTIPLIER;
	}

	static class SkillEnchantDetailElement
	{
		public final int _level;
		public final int _rate;
		public final int _spCost;
		public final int _expCost;

		public SkillEnchantDetailElement(int level, int rate, int spCost, int expCost)
		{
			_level = level;
			_rate = rate;
			_spCost = spCost;
			_expCost = expCost;
		}

		public SkillEnchantDetailElement(L2Player cha, L2EnchantSkillLearn esd)
		{
			this(esd.getLevel(), esd.getRate(cha), esd.getSpCost(), (int) esd.getExpCost());
		}

		public SkillEnchantDetailElement(int rate, L2EnchantSkillLearn esd)
		{
			this(esd.getLevel(), rate, esd.getSpCost(), (int) esd.getExpCost());
		}
	}

	public void addEnchantSkillDetail(L2Player cha, L2EnchantSkillLearn esd)
	{
		_routes.add(new SkillEnchantDetailElement(cha, esd));
	}

	public void addEnchantSkillDetail(int rate, L2EnchantSkillLearn esd)
	{
		_routes.add(new SkillEnchantDetailElement(rate, esd));
	}

	public void addEnchantSkillDetail(int level, int rate, int spCost, int expCost)
	{
		_routes.add(new SkillEnchantDetailElement(level, rate, spCost, expCost));
	}

	@Override
	protected void writeImpl()
	{
		writeC(EXTENDED_PACKET);
		writeH(0x18);

		writeD(_type.ordinal()); // safe enchant
		writeD(_routes.size());

		for(SkillEnchantDetailElement sede : _routes)
		{
			writeD(_id);
			writeD(sede._level);
			writeD(sede._rate);
			writeD(sede._spCost * _xpSpCostMultiplier);
			writeQ(sede._expCost * _xpSpCostMultiplier);
			writeD(0); // required item count
			writeD(0); // req type?
			writeD(0); // required itemId
			writeD(0); // ?
		}
	}

	@Override
	public String getType()
	{
		return _S__FE_2A_EXENCHANTSKILLINFO;
	}
}