package l2r.gameserver.serverpackets;

import l2r.gameserver.model.L2CommandChannel;
import l2r.gameserver.model.L2Party;

/**
 * Update a Command Channel
 * @author SYS
 * 
 * Format:
 * ch sddd[sdd]
 * Пример пакеты с оффа:
 * FE 30 00 
 * 50 00 6C 00 61 00 74 00 69 00 6E 00 6F 00 00 00 - имя лидера СС (Platino)
 * 00 00 00 00 - ? Looting type
 * 67 00 00 00 - общее число человек в СС (103)
 * 0C 00 00 00 - общее число партий в СС (12)
 * 50 00 6C 00 61 00 74 00 69 00 6E 00 6F 00 00 00 - лидер партии 1 (Platino)
 * 96 AB 40 49 - ObjId пати лидера 1
 * 09 00 00 00 - количество мемберов в пати 1
 * 46 00 61 00 74 00 48 00 69 00 70 00 73 00 00 00 - лидер партии 2 (FatHips)
 * 57 50 20 4E - ObjId пати лидера  2
 * 09 00 00 00 - количество мемберов в пати 2
 * и т.п.
 */
public class ExMPCCUpdate extends L2GameServerPacket
{

	private static final String _S__FE_30_EXMPCCUPDATE = "[S] FE:30 ExMPCCUpdate";
	private L2CommandChannel _channel;

	/**
	 * @param L2CommandChannel CommandChannel
	 */
	public ExMPCCUpdate(L2CommandChannel channel)
	{
		if(channel == null)
			return;
		_channel = channel;
	}

	@Override
	protected void writeImpl()
	{
		writeC(EXTENDED_PACKET);
		writeH(0x30);
		writeS(_channel.getChannelLeader().getName()); // имя лидера CC
		writeD(0); // Looting type?
		writeD(_channel.getMemberCount()); // общее число человек в СС
		writeD(_channel.getParties().size()); // общее число партий в СС
		for(L2Party party : _channel.getParties())
		{
			writeS(party.getPartyLeader().getName()); // имя лидера партии
			writeD(party.getPartyLeaderOID()); // ObjId пати лидера
			writeD(party.getMemberCount()); // количество мемберов в пати
		}
	}

	@Override
	public String getType()
	{
		return _S__FE_30_EXMPCCUPDATE;
	}
}