package l2r.gameserver.serverpackets;

import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.entity.Fortress;
import l2r.gameserver.model.entity.siege.SiegeClan;
import l2r.gameserver.tables.ClanTable;

/**
 * Populates the Siege Attacker List in the SiegeInfo Window<BR>
 * <BR>
 * packet type id 0xca<BR>
 * format: cddddddd + dSSdddSSd<BR>
 * <BR>
 * c = ca<BR>
 * d = FortID<BR>
 * d = unknow (0x00)<BR>
 * d = unknow (0x01)<BR>
 * d = unknow (0x00)<BR>
 * d = Number of Attackers Clans?<BR>
 * d = Number of Attackers Clans<BR>
 * { //repeats<BR>
 * d = ClanID<BR>
 * S = ClanName<BR>
 * S = ClanLeaderName<BR>
 * d = ClanCrestID<BR>
 * d = signed time (seconds)<BR>
 * d = AllyID<BR>
 * S = AllyName<BR>
 * S = AllyLeaderName<BR>
 * d = AllyCrestID<BR>
 */
public final class FortSiegeAttackerList extends L2GameServerPacket
{
	private static final String _S__CA_SiegeAttackerList = "[S] ca SiegeAttackerList";

	private Fortress _fort;

	public FortSiegeAttackerList(Fortress fort)
	{
		_fort = fort;
	}

	@Override
	protected final void writeImpl()
	{
		writeC(0xca);
		writeD(_fort.getId());
		writeD(0x00); // 0
		writeD(0x01); // 1
		writeD(0x00); // 0
		int size = _fort.getSiege().getAttackerClans().size();
		if(size > 0)
		{
			L2Clan clan;

			writeD(size);
			writeD(size);
			for(SiegeClan siegeclan : _fort.getSiege().getAttackerClans().values())
			{
				clan = ClanTable.getInstance().getClan(siegeclan.getClanId());
				if(clan == null)
					continue;

				writeD(clan.getClanId());
				writeS(clan.getName());
				writeS(clan.getLeaderName());
				writeD(clan.getCrestId());
				writeD(0x00); // signed time (seconds)
				writeD(clan.getAllyId());
				writeS(clan.getAlliance() != null ? clan.getAlliance().getAllyName() : "");
				writeS(""); // AllyLeaderName
				writeD(clan.getAlliance() != null ? clan.getAlliance().getAllyCrestId() : 0);
			}
		}
		else
		{
			writeD(0x00);
			writeD(0x00);
		}
	}

	@Override
	public String getType()
	{
		return _S__CA_SiegeAttackerList;
	}
}