package l2r.gameserver.serverpackets;

import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.entity.Fortress;
import l2r.gameserver.model.entity.siege.SiegeClan;
import l2r.gameserver.tables.ClanTable;

/**
 * Populates the Siege Defender List in the SiegeInfo Window<BR>
 * <BR>
 * packet type id 0xcb<BR>
 * format: cddddddd + dSSdddSSd<BR>
 * <BR>
 * c = 0xcb<BR>
 * d = FortID<BR>
 * d = unknow (0x00)<BR>
 * d = unknow (0x01)<BR>
 * d = unknow (0x00)<BR>
 * d = Number of Defending Clans?<BR>
 * d = Number of Defending Clans<BR>
 * { //repeats<BR>
 * d = ClanID<BR>
 * S = ClanName<BR>
 * S = ClanLeaderName<BR>
 * d = ClanCrestID<BR>
 * d = signed time (seconds)<BR>
 * d = Type -> Owner = 0x01 || Waiting = 0x02 || Accepted = 0x03<BR>
 * d = AllyID<BR>
 * S = AllyName<BR>
 * S = AllyLeaderName<BR>
 * d = AllyCrestID<BR>
 */
public final class FortSiegeDefenderList extends L2GameServerPacket
{
	private static final String _S__CA_SiegeDefenderList = "[S] cb SiegeDefenderList";

	private Fortress _fort;

	public FortSiegeDefenderList(Fortress fort)
	{
		_fort = fort;
	}

	@Override
	protected final void writeImpl()
	{
		writeC(0xcb);
		writeD(_fort.getId());
		writeD(0x00); // 0
		writeD(0x01); // 1
		writeD(0x00); // 0
		int size = _fort.getSiege().getDefenderClans().size() + _fort.getSiege().getDefenderWaitingClans().size();
		if(size > 0)
		{
			L2Clan clan;

			writeD(size);
			writeD(size);
			// Listing the Lord and the approved clans
			for(SiegeClan siegeclan : _fort.getSiege().getDefenderClans().values())
			{
				clan = ClanTable.getInstance().getClan(siegeclan.getClanId());
				if(clan == null)
					continue;

				writeD(clan.getClanId());
				writeS(clan.getName());
				writeS(clan.getLeaderName());
				writeD(clan.getCrestId());
				writeD(0x00); // signed time (seconds)
				switch(siegeclan.getType())
				{
					case OWNER:
						writeD(0x01); // owner
						break;
					case DEFENDER_PENDING:
						writeD(0x02); // approved
						break;
					case DEFENDER:
						writeD(0x03); // waiting approved
						break;
					default:
						writeD(0x00);
						break;
				}
				writeD(clan.getAllyId());
				writeS(clan.getAlliance() != null ? clan.getAlliance().getAllyName() : "");
				writeS(clan.getAlliance() != null ? clan.getAlliance().getAllyLeaderName() : "");
				writeD(clan.getAlliance() != null ? clan.getAlliance().getAllyCrestId() : 0);
			}
			for(SiegeClan siegeclan : _fort.getSiege().getDefenderWaitingClans().values())
			{
				clan = ClanTable.getInstance().getClan(siegeclan.getClanId());
				writeD(clan.getClanId());
				writeS(clan.getName());
				writeS(clan.getLeaderName());
				writeD(clan.getCrestId());
				writeD(0x00); // signed time (seconds)
				writeD(0x02); // waiting approval
				writeD(clan.getAllyId());
				writeS(clan.getAlliance() != null ? clan.getAlliance().getAllyName() : "");
				writeS(clan.getAlliance() != null ? clan.getAlliance().getAllyLeaderName() : "");
				writeD(clan.getAlliance() != null ? clan.getAlliance().getAllyCrestId() : 0);
			}
		}
		else
		{
			writeD(0x00);
			writeD(0x00);
		}
	}

	@Override
	public String getType()
	{
		return _S__CA_SiegeDefenderList;
	}
}