package l2r.gameserver.serverpackets;

import java.util.Calendar;
import java.util.logging.Logger;

import l2r.gameserver.model.L2Clan;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.entity.Fortress;
import l2r.gameserver.tables.ClanTable;

/**
 * Shows the Siege Info<BR>
 * <BR>
 * packet type id 0xc9<BR>
 * format: cdddSSdSdd<BR>
 * <BR>
 * c = c9<BR>
 * d = FortID<BR>
 * d = Show Owner Controls (0x00 default || >=0x02(mask?) owner)<BR>
 * d = Owner ClanID<BR>
 * S = Owner ClanName<BR>
 * S = Owner Clan LeaderName<BR>
 * d = Owner AllyID<BR>
 * S = Owner AllyName<BR>
 * d = current time (seconds)<BR>
 * d = Siege time (seconds) (0 for selectable)<BR>
 * d = (UNKNOW) Siege Time Select Related?
 */
public class FortressSiegeInfo extends L2GameServerPacket
{
	private static final String _S__C9_SIEGEINFO = "[S] c9 SiegeInfo";
	private static Logger _log = Logger.getLogger(FortressSiegeInfo.class.getName());
	private Fortress _fort;

	public FortressSiegeInfo(Fortress fort)
	{
		_fort = fort;
	}

	@Override
	protected final void writeImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		writeC(0xc9);
		writeD(_fort.getId());
		writeD(_fort.getOwnerId() == activeChar.getClanId() && activeChar.isClanLeader() ? 0x01 : 0x00);
		writeD(_fort.getOwnerId());
		if(_fort.getOwnerId() > 0)
		{
			L2Clan owner = ClanTable.getInstance().getClan(_fort.getOwnerId());
			if(owner != null)
			{
				writeS(owner.getName()); // Clan Name
				writeS(owner.getLeaderName()); // Clan Leader Name
				writeD(owner.getAllyId()); // Ally ID
				writeS(owner.getAlliance() != null ? owner.getAlliance().getAllyName() : ""); // Ally Name
			}
			else
				_log.warning("Null owner for fort: " + _fort.getName());
		}
		else
		{
			writeS("NPC"); // Clan Name
			writeS(""); // Clan Leader Name
			writeD(0); // Ally ID
			writeS(""); // Ally Name
		}

		writeD((int) (Calendar.getInstance().getTimeInMillis() / 1000));
		writeD((int) (_fort.getSiege().getSiegeDate().getTimeInMillis() / 1000));
		writeD(0x00); //number of choices?
	}

	@Override
	public String getType()
	{
		return _S__C9_SIEGEINFO;
	}
}