package l2r.gameserver.serverpackets;

import javolution.util.FastList;
import l2r.Config;
import l2r.gameserver.model.L2Multisell.MultiSellListContainer;
import l2r.gameserver.model.base.MultiSellEntry;
import l2r.gameserver.model.base.MultiSellIngredient;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.templates.L2Item;

public class MultiSellList extends L2GameServerPacket
{
	private static final String _S__D0_MULTISELLLIST = "[S] D0 MultiSellList";

	protected int _page;
	protected int _finished;

	private int _listId;
	private FastList<MultiSellEntry> _possiblelist = new FastList<MultiSellEntry>();

	public MultiSellList(MultiSellListContainer list, int page, int finished)
	{
		_possiblelist = list.getEntries();
		_listId = list.getListId();
		_page = page;
		_finished = finished;
	}

	@Override
	protected final void writeImpl()
	{
		// ddddd (dchddddddddddhh (ddhdhdddddddddd)(dhdhdddddddddd))
		writeC(0xD0);
		writeD(_listId); // list id
		writeD(_page); // page
		writeD(_finished); // finished
		writeD(Config.MULTISELL_SIZE); // size of pages
		writeD(_possiblelist != null ? _possiblelist.size() : 0); //list lenght

		if(_possiblelist == null)
			return;

		for(FastList.Node<MultiSellEntry> n = _possiblelist.head(), end = _possiblelist.tail(); (n = n.getNext()) != end;)
		{
			MultiSellEntry ent = n.getValue();
			writeD(ent.getEntryId());
			writeC(ent.getProduction().get(0).isStackable() ? 1 : 0); // stackable?
			writeH(0x00); // unknown
			writeD(0x00); // инкрустация
			writeD(0x00); // инкрустация
			writeH(ent.getProduction().size());
			writeH(ent.getIngredients().size());

			for(MultiSellIngredient prod : ent.getProduction())
			{
				L2Item template = ItemTable.getInstance().getTemplate(prod.getItemId());
				if(template == null)
					continue;
				writeD(prod.getItemId());
				writeD(template.getBodyPart());
				writeH(template.getType2());
				writeD(prod.getItemCount());
				writeH(prod.getItemEnchant());
				writeD(0x00); // инкрустация
				writeD(0x00); // инкрустация
			}

			for(FastList.Node<MultiSellIngredient> sn = ent.getIngredients().head(), send = ent.getIngredients().tail(); (sn = sn.getNext()) != send;)
			{
				MultiSellIngredient i = sn.getValue();
				int itemId = i.getItemId();
				final L2Item item = itemId != L2Item.ITEM_ID_CLAN_REPUTATION_SCORE && itemId != L2Item.ITEM_ID_PC_BANG_POINTS ? ItemTable.getInstance().getTemplate(i.getItemId()) : null;
				writeD(itemId); //ID
				writeH(itemId != L2Item.ITEM_ID_CLAN_REPUTATION_SCORE && itemId != L2Item.ITEM_ID_PC_BANG_POINTS ? item.getType2() : 0xffff);
				writeD(i.getItemCount()); //Count
				writeH((itemId != L2Item.ITEM_ID_CLAN_REPUTATION_SCORE && itemId != L2Item.ITEM_ID_PC_BANG_POINTS ? item.getType2() : 0x00) <= L2Item.TYPE2_ACCESSORY ? i.getItemEnchant() : 0); //Enchant Level
				writeD(0x00); // инкрустация
				writeD(0x00); // инкрустация
			}
		}
	}

	@Override
	public String getType()
	{
		return _S__D0_MULTISELLLIST;
	}
}