package l2r.gameserver.serverpackets;

import java.util.ArrayList;
import java.util.logging.Logger;

import l2r.extensions.scripts.Scripts;
import l2r.gameserver.model.BypassEngine;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.entity.SevenSignsFestival.SevenSignsFestival;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.util.Files;
import l2r.util.Strings;

/**
 * the HTML parser in the client knowns these standard and non-standard tags and attributes
 * VOLUMN
 * UNKNOWN
 * UL
 * U
 * TT
 * TR
 * TITLE
 * TEXTCODE
 * TEXTAREA
 * TD
 * TABLE
 * SUP
 * SUB
 * STRIKE
 * SPIN
 * SELECT
 * RIGHT
 * PRE
 * P
 * OPTION
 * OL
 * MULTIEDIT
 * LI
 * LEFT
 * INPUT
 * IMG
 * I
 * HTML
 * H7
 * H6
 * H5
 * H4
 * H3
 * H2
 * H1
 * FONT
 * EXTEND
 * EDIT
 * COMMENT
 * COMBOBOX
 * CENTER
 * BUTTON
 * BR
 * BODY
 * BAR
 * ADDRESS
 * A
 * SEL
 * LIST
 * VAR
 * FORE
 * READONL
 * ROWS
 * VALIGN
 * FIXWIDTH
 * BORDERCOLORLI
 * BORDERCOLORDA
 * BORDERCOLOR
 * BORDER
 * BGCOLOR
 * BACKGROUND
 * ALIGN
 * VALU
 * READONLY
 * MULTIPLE
 * SELECTED
 * TYP
 * TYPE
 * MAXLENGTH
 * CHECKED
 * SRC
 * Y
 * X
 * QUERYDELAY
 * NOSCROLLBAR
 * IMGSRC
 * B
 * FG
 * SIZE
 * FACE
 * COLOR
 * DEFFON
 * DEFFIXEDFONT
 * WIDTH
 * VALUE
 * TOOLTIP
 * NAME
 * MIN
 * MAX
 * HEIGHT
 * DISABLED
 * ALIGN
 * MSG
 * LINK
 * HREF
 * ACTION
 */public class NpcHtmlMessage extends L2GameServerPacket
{

	private static final Logger _log = Logger.getLogger(NpcHtmlMessage.class.getName());
	// d S
	// d is usually 0, S is the html text starting with <html> and ending with </html>
	//
	private static final String _S__0F_NPCHTMLMESSAGE = "[S] 0F NpcHtmlMessage";
	private int _npcObjId;
	private String _html;
	private String _file = null;
	private ArrayList<String> replaces = new ArrayList<String>();

	public NpcHtmlMessage(L2Player player, L2NpcInstance npc, String filename, Integer val)
	{
		_npcObjId = npc.getObjectId();

		player.setLastNpc(npc);
		setFile(filename);

		// Добавить в конец странички текст, определенный в скриптах.
		ArrayList<String> appends = Scripts.dialogAppends.get(npc.getNpcId());
		String replaces = "";
		if(appends != null)
			for(String append : appends)
			{
				String[] list = append.split(":");
				String _class = list[0];
				String _method = list[1];
				Object obj = player.callScripts(_class, _method, new Object[] { val });
				if(obj == null)
					return;
				replaces += (String) obj;
			}

		if(!replaces.equals(""))
			replace("</body>", "\n" + Strings.bbParse(replaces) + "</body>");

		replace("%npcId%", String.valueOf(npc.getNpcId()));
		replace("%npcname%", npc.getName());
		replace("%festivalMins%", SevenSignsFestival.getInstance().getTimeToNextFestivalStr());
	}

	public NpcHtmlMessage(int npcObjId)
	{
		_npcObjId = npcObjId;
	}

	public final NpcHtmlMessage setHtml(String text)
	{
		if(!text.contains("<html>"))
			text = "<html><body>" + text + "</body></html>"; //<title>Message:</title> <br><br><br>
		_html = text;
		return this;
	}

	public final NpcHtmlMessage setFile(String file)
	{
		_file = file;
		return this;
	}

	private void setFile()
	{
		String content = loadHtml(_file, getClient().getActiveChar());
		if(content == null)
			content = _file;
		setHtml(content);
	}

	protected String loadHtml(String name, L2Player player)
	{
		return Files.read(name, player);
	}

	protected String html_load(String name, String lang)
	{
		String content = Files.read(name, lang);
		if(content == null)
			content = "Can't find file'" + name + "'";
		return content;
	}

	public void replace(String pattern, String value)
	{
		replaces.add(pattern);
		replaces.add(value);
	}

	@Override
	final public void runImpl()
	{}

	@Override
	protected final void writeImpl()
	{
		if(_file != null)
			setFile();

		L2Player player = getClient().getActiveChar();

		if(player == null)
			return;

		for(int i = 0; i < replaces.size(); i += 2)
			_html = _html.replaceAll(replaces.get(i), replaces.get(i + 1));

		_html = _html.replaceAll("%objectId%", String.valueOf(_npcObjId));

		if(player.getVarB("translit"))
			_html = Strings.translit(_html);

		_html = BypassEngine.encode(player, _html);

		if(_html.length() > 8192)
		{
			_log.severe("To long html, client crash prevented. File: '" + _file + "', Character: " + player.getName());
			_html = "<html><body><center>Sorry, to long html.</center></body></html>";
		}

		writeC(0x0F);

		writeD(_npcObjId);
		writeS(_html);
		writeD(0x00);
	}

	@Override
	public String getType()
	{
		return _S__0F_NPCHTMLMESSAGE;
	}
}
