package l2r.gameserver.serverpackets;

import java.util.HashMap;

import l2r.Config;
import l2r.gameserver.instancemanager.CursedWeaponsManager;
import l2r.gameserver.model.Inventory;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Summon;
import l2r.gameserver.model.instances.L2CubicInstance;
import l2r.gameserver.model.instances.L2DecoyInstance;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.util.Location;

public class NpcInfo extends L2GameServerPacket
{
	//   ddddddddddddddddddffffdddcccccSSddd dddddc
	private static final String _S__0c_NPCINFO = "[S] 0c NpcInfo";
	private boolean can_writeImpl = false;
	private L2Character _cha;
	private int _npcObjId, _npcId, running, incombat, dead, team;
	private int _runSpd, _walkSpd, _mAtkSpd, _pAtkSpd, _rhand, _lhand;
	private int karma, pvp_flag, _abnormalEffect, clan_crest_id, ally_crest_id;
	private double colHeight, colRadius;
	//private int _swimRunSpd, _swimWalkSpd, _flRunSpd, _flWalkSpd, _flyRunSpd, _flyWalkSpd;
	private boolean _isAttackable;
	private Location _loc, decoy_fishLoc;
	private String _name = "";
	private String _title = "";
	private boolean _isShowSpawnAnimation = false;

	private Inventory decoy_inv;
	private int decoy_race, decoy_sex, decoy_base_class, decoy_clan_id, decoy_ally_id;
	private int decoy_noble, decoy_hair_style, decoy_hair_color, decoy_face, decoy_sitting;
	private int decoy_invis, decoy_rec_have, decoy_rec_left, decoy_class_id, decoy_large_clan_crest_id;
	private int decoy_enchant, decoy_PledgeClass, decoy_pledge_type;
	private int decoy_NameColor, decoy_TitleColor, decoy_Transformation, decoy_Agathion;
	private int decoy_hero, decoy_mount_id, decoy_swimSpd, decoy_cw_level, decoy_clan_rep_score;
	private byte decoy_mount_type, decoy_private_store, decoy_fishing;
	private double decoy_move_speed, decoy_attack_speed;
	private HashMap<Integer, L2CubicInstance> decoy_cubics;

	public NpcInfo(L2NpcInstance cha, L2Character attacker)
	{
		if(cha == null)
			return;

		_cha = cha;
		_npcId = cha.getTemplate().displayId != 0 ? cha.getTemplate().displayId : cha.getTemplate().npcId;
		_isAttackable = cha.isAutoAttackable(attacker);
		_rhand = cha.getRightHandItem();
		_lhand = cha.getLeftHandItem();
		if(Config.SERVER_SIDE_NPC_NAME || cha.getTemplate().displayId != 0)
			_name = cha.getName();
		if(Config.SERVER_SIDE_NPC_TITLE || cha.getTemplate().displayId != 0)
			_title = _title + cha.getTitle();
		_isShowSpawnAnimation = cha.isShowSpawnAnimation();
		can_writeImpl = true;
	}

	public NpcInfo(L2Summon cha, L2Character attacker)
	{
		if(cha == null)
			return;
		if(cha.getOwner() != null && cha.getOwner().isInvisible())
			return;

		_cha = cha;
		_npcId = cha.getTemplate().npcId;
		_isAttackable = cha.isAutoAttackable(attacker); //(cha.getKarma() > 0);
		_rhand = 0;
		_lhand = 0;
		if(Config.SERVER_SIDE_NPC_NAME || cha.isPet)
			_name = _cha.getName();
		_title = cha.getTitle();
		can_writeImpl = true;
	}

	public NpcInfo(L2Summon cha, L2Character attacker, boolean isShowSpawnAnimation)
	{
		this(cha, attacker);
		_isShowSpawnAnimation = isShowSpawnAnimation;
	}

	@Override
	final public void runImpl()
	{
		if(!can_writeImpl)
			return;

		colHeight = _cha.getColHeight();
		colRadius = _cha.getColRadius();
		_npcObjId = _cha.getObjectId();
		_loc = _cha.getLoc();
		_mAtkSpd = _cha.getMAtkSpd();
		clan_crest_id = _cha.getClanCrestId();
		ally_crest_id = _cha.getAllyCrestId();

		if(_cha instanceof L2DecoyInstance)
			runImpl_Decoy();
		else
		{
			_runSpd = _cha.getRunSpeed();
			_walkSpd = _cha.getWalkSpeed();
			karma = _cha.getKarma();
			pvp_flag = _cha.getPvpFlag();
			_pAtkSpd = _cha.getPAtkSpd();
			running = _cha.isRunning() ? 1 : 0;
			incombat = _cha.isInCombat() ? 1 : 0;
			dead = _cha.isAlikeDead() ? 1 : 0;
			_abnormalEffect = _cha.getAbnormalEffect();
			team = _cha.getTeam();
		}
	}

	@Override
	protected final void writeImpl()
	{
		if(!can_writeImpl)
			return;

		if(_cha instanceof L2DecoyInstance)
		{
			writeImpl_Decoy();
			return;
		}

		writeC(0x16);
		//ddddddddddddddddddffffdddcccccSSddddddddccffdddd
		writeD(_npcObjId);
		writeD(_npcId + 1000000); // npctype id c4
		writeD(_isAttackable ? 1 : 0);
		writeD(_loc.getX());
		writeD(_loc.getY());
		writeD(_loc.getZ());
		writeD(_loc.h);
		writeD(0x00);
		writeD(_mAtkSpd);
		writeD(_pAtkSpd);
		writeD(_runSpd);
		writeD(_walkSpd);
		writeD(_runSpd /*_swimRunSpd*//*0x32*/); // swimspeed
		writeD(_walkSpd/*_swimWalkSpd*//*0x32*/); // swimspeed
		writeD(_runSpd/*_flRunSpd*/);
		writeD(_walkSpd/*_flWalkSpd*/);
		writeD(_runSpd/*_flyRunSpd*/);
		writeD(_walkSpd/*_flyWalkSpd*/);
		writeF(1.1); // Interlude
		writeF(1/*_cha.getAttackSpeedMultiplier()*/);
		writeF(colRadius);
		writeF(colHeight);
		writeD(_rhand); // right hand weapon
		writeD(0);
		writeD(_lhand); // left hand weapon
		writeC(1); // name above char 1=true ... ??
		writeC(running);
		writeC(incombat);
		writeC(dead);
		writeC(_isShowSpawnAnimation ? 2 : 0); // invisible ?? 0=false  1=true   2=summoned (only works if model has a summon animation)
		writeS(_name);
		writeS(_title);
		writeD(0); // как-то связано с тайтлом, если не 0 скрывать?
		writeD(pvp_flag);
		writeD(karma); // hmm karma ??
		writeD(_abnormalEffect); // C2
		writeD(0); // clan id (клиентом не используется, но требуется для показа значка)
		writeD(clan_crest_id); // clan crest id
		writeD(0); // ally id (клиентом не используется, но требуется для показа значка)
		writeD(ally_crest_id); // ally crest id
		writeC(0000); // C2
		writeC(team); // team aura 1-blue, 2-red
		writeF(colRadius);
		writeF(colHeight);
		writeD(0x00); // C4
	}

	private void runImpl_Decoy()
	{
		L2Player cha_owner = _cha.getOwner();
		_runSpd = cha_owner.getRunSpeed();
		_walkSpd = cha_owner.getWalkSpeed();
		karma = cha_owner.getKarma();
		pvp_flag = cha_owner.getPvpFlag();
		_pAtkSpd = cha_owner.getPAtkSpd();
		running = cha_owner.isRunning() ? 1 : 0;
		incombat = cha_owner.isInCombat() ? 1 : 0;
		dead = cha_owner.isAlikeDead() ? 1 : 0;
		_abnormalEffect = cha_owner.getAbnormalEffect();
		team = cha_owner.getTeam();

		if(cha_owner.isCursedWeaponEquipped())
		{
			_name = cha_owner.getName();
			_title = "";
			clan_crest_id = 0;
			ally_crest_id = 0;
			decoy_clan_id = 0;
			decoy_ally_id = 0;
			decoy_large_clan_crest_id = 0;
			decoy_cw_level = CursedWeaponsManager.getInstance().getLevel(cha_owner.getCursedWeaponEquippedId());
		}
		else
		{
			_name = cha_owner.getName();
			_title = cha_owner.getTitle();
			clan_crest_id = cha_owner.getClanCrestId();
			ally_crest_id = cha_owner.getAllyCrestId();
			decoy_clan_id = cha_owner.getClanId();
			decoy_ally_id = cha_owner.getAllyId();
			decoy_large_clan_crest_id = cha_owner.getClanCrestLargeId();
			decoy_cw_level = 0;
		}

		if(cha_owner.isMounted())
		{
			decoy_enchant = 0;
			decoy_mount_id = cha_owner.getMountNpcId() + 1000000;
			decoy_mount_type = (byte) cha_owner.getMountType();
		}
		else
		{
			decoy_enchant = (byte) cha_owner.getEnchantEffect();
			decoy_mount_id = 0;
			decoy_mount_type = 0;
		}

		if(decoy_clan_id > 0 && cha_owner.getClan() != null)
			decoy_clan_rep_score = cha_owner.getClan().getReputationScore();
		else
			decoy_clan_rep_score = 0;

		decoy_fishing = cha_owner.isFishing() ? (byte) 1 : (byte) 0;
		decoy_fishLoc = cha_owner.getFishLoc();
		decoy_swimSpd = cha_owner.getSwimSpeed();
		decoy_private_store = (byte) cha_owner.getPrivateStoreType(); // 1 - sellshop
		decoy_inv = cha_owner.getInventory();
		decoy_race = cha_owner.getBaseTemplate().race.ordinal();
		decoy_sex = cha_owner.getSex();
		decoy_base_class = cha_owner.getBaseClass();
		decoy_move_speed = cha_owner.getMovementSpeedMultiplier();
		decoy_attack_speed = cha_owner.getAttackSpeedMultiplier();
		decoy_hair_style = cha_owner.getHairStyle();
		decoy_hair_color = cha_owner.getHairColor();
		decoy_face = cha_owner.getFace();
		decoy_sitting = cha_owner.isSitting() ? 0 : 1;
		decoy_invis = cha_owner.isInvisible() ? 1 : 0;
		decoy_cubics = cha_owner.getCubics();
		decoy_rec_left = cha_owner.getRecomLeft();
		decoy_rec_have = cha_owner.getPlayerAccess().IsGM ? 0 : cha_owner.getRecomHave();
		decoy_class_id = cha_owner.getClassId().getId();
		decoy_noble = cha_owner.isNoble() ? 1 : 0;
		decoy_hero = cha_owner.isHero() || cha_owner.isGM() && Config.GM_HERO_AURA ? 1 : 0; // 0x01: Hero Aura
		decoy_NameColor = cha_owner.getNameColor();
		decoy_PledgeClass = cha_owner.getPledgeClass();
		decoy_pledge_type = cha_owner.getPledgeType();
		decoy_TitleColor = cha_owner.getTitleColor();
	}

	private void writeImpl_Decoy()
	{
		writeC(0x03);
		writeD(_loc.getX());
		writeD(_loc.getY());
		writeD(_loc.getZ());
		writeD(_loc.h);
		writeD(_npcObjId);
		writeS(_name);
		writeD(decoy_race);
		writeD(decoy_sex);
		writeD(decoy_base_class);
		writeD(decoy_inv.getPaperdollItemId(Inventory.PAPERDOLL_DHAIR));
		writeD(decoy_inv.getPaperdollItemId(Inventory.PAPERDOLL_HEAD));
		writeD(decoy_inv.getPaperdollItemId(Inventory.PAPERDOLL_RHAND));
		writeD(decoy_inv.getPaperdollItemId(Inventory.PAPERDOLL_LHAND));
		writeD(decoy_inv.getPaperdollItemId(Inventory.PAPERDOLL_GLOVES));
		writeD(decoy_inv.getPaperdollItemId(Inventory.PAPERDOLL_CHEST));
		writeD(decoy_inv.getPaperdollItemId(Inventory.PAPERDOLL_LEGS));
		writeD(decoy_inv.getPaperdollItemId(Inventory.PAPERDOLL_FEET));
		writeD(decoy_inv.getPaperdollItemId(Inventory.PAPERDOLL_BACK));
		writeD(decoy_inv.getPaperdollItemId(Inventory.PAPERDOLL_LRHAND));
		writeD(decoy_inv.getPaperdollItemId(Inventory.PAPERDOLL_HAIR));
		writeD(decoy_inv.getPaperdollItemId(Inventory.PAPERDOLL_FACE));

		writeD(decoy_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_DHAIR));
		writeD(decoy_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_HEAD));
		writeD(decoy_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_RHAND));
		writeD(decoy_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_LHAND));
		writeD(decoy_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_GLOVES));
		writeD(decoy_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_CHEST));
		writeD(decoy_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_LEGS));
		writeD(decoy_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_FEET));
		writeD(decoy_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_BACK));
		writeD(decoy_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_LRHAND));
		writeD(decoy_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_HAIR));
		writeD(decoy_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_FACE));
		writeD(pvp_flag);
		writeD(karma);
		writeD(_mAtkSpd);
		writeD(_pAtkSpd);
		writeD(pvp_flag);
		writeD(karma);
		writeD(_runSpd);
		writeD(_walkSpd);
		writeD(decoy_swimSpd); // swimspeed
		writeD(decoy_swimSpd); // swimspeed
		writeD(_runSpd/*_flRunSpd*/);
		writeD(_walkSpd/*_flWalkSpd*/);
		writeD(_runSpd/*_flyRunSpd*/);
		writeD(_walkSpd/*_flyWalkSpd*/);
		writeF(decoy_move_speed);
		writeF(decoy_attack_speed);
		writeF(colRadius);
		writeF(colHeight);
		writeD(decoy_hair_style);
		writeD(decoy_hair_color);
		writeD(decoy_face);
		writeS(_title);
		writeD(decoy_clan_id);
		writeD(clan_crest_id);
		writeD(decoy_ally_id);
		writeD(ally_crest_id);
		writeD(0);
		writeC(decoy_sitting);
		writeC(running);
		writeC(incombat);
		writeC(dead);
		writeC(decoy_invis);
		writeC(decoy_mount_type);
		writeC(decoy_private_store);

		if(decoy_cubics == null)
			writeH(0);
		else
		{
			writeH(decoy_cubics.size());
			for(int id : decoy_cubics.keySet())
				writeH(id);
		}

		writeC(0x00); // find party members
		writeD(_abnormalEffect);
		writeC(decoy_rec_left);
		writeH(decoy_rec_have);
		writeD(decoy_mount_id);
		writeD(decoy_class_id);
		writeD(0); // ?
		writeC(decoy_enchant);
		writeC(team);
		writeD(decoy_large_clan_crest_id);
		writeC(decoy_noble);
		writeC(decoy_hero);
		writeC(decoy_fishing);
		writeD(decoy_fishLoc.getX());
		writeD(decoy_fishLoc.getY());
		writeD(decoy_fishLoc.getZ());
		writeD(decoy_NameColor);
		writeD(_loc.h);
		writeD(decoy_PledgeClass);
		writeD(decoy_pledge_type);
		writeD(decoy_TitleColor);
		writeD(decoy_cw_level);
		writeD(decoy_clan_rep_score);
	}

	@Override
	public String getType()
	{
		return _S__0c_NPCINFO + (_cha != null ? " about " + _cha : "");
	}
}