package l2r.gameserver.serverpackets;

import java.util.ArrayList;

import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.templates.L2Item;

/**
 * Fotmat:
 * d char object id
 * d количество адены
 * d количество предметов
 * [hdddhhdhhhddddddddd]
 *
 * Пример с оффа(828 протокол):
 * 0000: d2 97 e7 07 00 41 06 00 00 04 00 00 00 04 00 70    .....A.........p
 * 0010: 28 54 41 39 00 00 00 41 06 00 00 04 00 00 00 00    (TA9...A........
 * 0020: 00 00 00 00 00 00 00 00 00 f1 ca f8 06 fe ff ff    ................
 * 0030: ff 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00    ................
 * 0040: 00 00 00 00 00 00 00 00 00 00 00 00 00 04 00 b2    ................
 * 0050: 78 63 41 b4 21 00 00 01 00 00 00 05 00 00 00 00    xcA.!...........
 * 0060: 00 00 00 00 00 00 00 00 00 d9 fa e2 06 fe ff ff    ................
 * 0070: ff 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00    ................
 * 0080: 00 00 00 00 00 00 00 00 00 00 00 00 00 04 00 94    ................
 * 0090: 80 59 41 af 21 00 00 01 00 00 00 05 00 00 00 00    .YA.!...........
 * 00a0: 00 00 00 00 00 00 00 00 00 d8 fa e2 06 fe ff ff    ................
 * 00b0: ff 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00    ................
 * 00c0: 00 00 00 00 00 00 00 00 00 00 00 00 00 04 00 53    ...............S
 * 00d0: 8d 25 41 ae 21 00 00 02 00 00 00 05 00 00 00 00    .%A.!...........
 * 00e0: 00 00 00 00 00 00 00 00 00 88 b7 e2 06 fe ff ff    ................
 * 00f0: ff 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00    ................
 * 0100: 00 00 00 00 00 00 00 00 00 00 00 00 00             .............
 */
public class PackageSendableList extends L2GameServerPacket
{
	private static final String _S__d2_PACKAGESENDABLELIST = "[S] d2 PackageSendableList";
	private int player_obj_id, char_adena;
	private ArrayList<L2ItemInstance> _itemslist = new ArrayList<L2ItemInstance>();

	public PackageSendableList(L2Player cha, int playerObjId)
	{
		player_obj_id = playerObjId;
		char_adena = cha.getAdena();
		for(L2ItemInstance item : cha.getInventory().getItems())
			if(!item.isEquipped() && item.getItem().getType2() != L2Item.TYPE2_QUEST && item.canBeStored(cha))
				_itemslist.add(item);
	}

	@Override
	protected final void writeImpl()
	{
		if(player_obj_id == 0)
			return;

		writeC(0xC3);
		writeD(player_obj_id);
		writeD(char_adena);
		writeD(_itemslist.size());
		for(L2ItemInstance temp : _itemslist)
		{
			L2Item item = temp.getItem();
			writeH(item.getType1());
			writeD(temp.getObjectId());
			writeD(temp.getItemId());
			writeD(temp.getIntegerLimitedCount());
			writeH(item.getType2());
			writeH(temp.getCustomType1());
			writeD(temp.getBodyPart());
			writeH(temp.getEnchantLevel());
			writeH(temp.getCustomType2());
			writeH(0x00); // ?
			writeD(temp.getObjectId()); // some item identifier later used by client to answer (see RequestPackageSend) not item id nor object id maybe some freight system id??
		}
	}

	@Override
	public String getType()
	{
		return _S__d2_PACKAGESENDABLELIST;
	}
}