package l2r.gameserver.serverpackets;

import java.util.concurrent.ConcurrentLinkedQueue;

import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2TradeList;
import l2r.gameserver.model.TradeItem;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.templates.L2Item;

public class PrivateStoreManageList extends L2GameServerPacket
{
	private static final String _S__A0_PRIVATESTOREMANAGELISTSELL = "[S] A0 PrivateStoreManageListSell";
	private int seller_id, seller_adena;
	private boolean _package = false;
	private ConcurrentLinkedQueue<TradeItem> _sellList;
	private ConcurrentLinkedQueue<TradeItem> _haveList;

	/**
	 * Окно управления личным магазином покупки
	 * @param seller
	 */
	public PrivateStoreManageList(L2Player seller, boolean pkg)
	{
		seller_id = seller.getObjectId();
		seller_adena = seller.getAdena();
		_package = pkg;

		// Проверяем список вещей в инвентаре, если вещь остутствует - убираем из списка продажи
		_sellList = new ConcurrentLinkedQueue<TradeItem>();
		for(TradeItem i : seller.getSellList())
		{
			L2ItemInstance inst = seller.getInventory().getItemByObjectId(i.getObjectId());
			if(i.getCount() <= 0 || inst == null || inst.isWear())
				continue;
			if(inst.getIntegerLimitedCount() < i.getCount())
				i.setCount(inst.getIntegerLimitedCount());
			_sellList.add(i);
		}

		L2TradeList _list = new L2TradeList(0);
		// Строим список вещей, годных для продажи имеющихся в инвентаре
		for(L2ItemInstance item : seller.getInventory().getItemsList())
			if(!item.isEquipped() && item.getItem().getType2() != L2Item.TYPE2_QUEST && item.getItem().getType2() != L2Item.TYPE2_MONEY && item.canBeTraded(seller))
				_list.addItem(item);

		_haveList = new ConcurrentLinkedQueue<TradeItem>();

		// Делаем список для собственно передачи с учетом количества
		for(L2ItemInstance item : _list.getItems())
		{
			TradeItem ti = new TradeItem();
			ti.setObjectId(item.getObjectId());
			ti.setItemId(item.getItemId());
			ti.setCount(item.getIntegerLimitedCount());
			ti.setEnchantLevel(item.getEnchantLevel());
			_haveList.add(ti);
		}

		//Убираем совпадения между списками, в сумме оба списка должны совпадать с содержимым инвентаря
		if(_sellList.size() > 0)
			for(TradeItem itemOnSell : _sellList)
			{
				_haveList.remove(itemOnSell);
				boolean added = false;
				for(TradeItem itemInInv : _haveList)
					if(itemInInv.getObjectId() == itemOnSell.getObjectId())
					{
						added = true;
						itemOnSell.setCount(Math.min(itemOnSell.getCount(), itemInInv.getCount()));
						if(itemOnSell.getCount() == itemInInv.getCount())
							_haveList.remove(itemInInv);
						else if(itemOnSell.getCount() > 0)
							itemInInv.setCount(itemInInv.getCount() - itemOnSell.getCount());
						else
							_sellList.remove(itemOnSell);
						break;
					}
				if(!added)
					_sellList.remove(itemOnSell);
			}
	}

	@Override
	protected final void writeImpl()
	{
		writeC(0x9A);
		//section 1
		writeD(seller_id);
		writeD(_package ? 1 : 0);
		writeD(seller_adena);

		//Список имеющихся вещей
		int count = _haveList.size();
		writeD(count);
		for(TradeItem temp : _haveList)
		{
			L2Item tempItem = ItemTable.getInstance().getTemplate(temp.getItemId());
			writeD(tempItem.getType2());
			writeD(temp.getObjectId());
			writeD(temp.getItemId());
			writeD(temp.getCount());
			writeH(0);
			writeH(temp.getEnchantLevel());//enchant lvl
			writeH(0);
			writeD(tempItem.getBodyPart());
			writeD(tempItem.getReferencePrice()); //store price
		}

		//Список вещей уже поставленых на продажу
		count = _sellList.size();
		writeD(count);
		for(TradeItem temp2 : _sellList)
		{
			L2Item tempItem = ItemTable.getInstance().getTemplate(temp2.getItemId());
			writeD(tempItem.getType2());
			writeD(temp2.getObjectId());
			writeD(temp2.getItemId());
			writeD(temp2.getCount());
			writeH(0);
			writeH(temp2.getEnchantLevel());//enchant lvl
			writeH(0);
			writeD(tempItem.getBodyPart());
			writeD(temp2.getOwnersPrice());//your price
			writeD(temp2.getStorePrice()); //store price
		}
	}

	@Override
	public String getType()
	{
		return _S__A0_PRIVATESTOREMANAGELISTSELL;
	}
}