package l2r.gameserver.serverpackets;

import javolution.util.FastList;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2TradeList;
import l2r.gameserver.model.TradeItem;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.templates.L2Item;

public class PrivateStoreManageListBuy extends L2GameServerPacket
{
	private static final String _S__BD_PRIVATESTOREMANAGELISTBUY = "[S] BD PrivateStoreManageListBuy";
	private FastList<BuyItemInfo> buylist = new FastList<BuyItemInfo>();
	private int buyer_id, buyer_adena;
	private L2TradeList _list;

	/**
	 * Окно управления личным магазином продажи
	 * @param buyer
	 */
	public PrivateStoreManageListBuy(L2Player buyer)
	{
		buyer_id = buyer.getObjectId();
		buyer_adena = buyer.getAdena();

		int _id, count, store_price, body_part, type2, owner_price;
		for(TradeItem e : buyer.getBuyList())
		{
			_id = e.getItemId();
			L2Item tempItem = ItemTable.getInstance().getTemplate(_id);
			if(tempItem == null)
				continue;

			count = e.getCount();
			store_price = e.getStorePrice();
			body_part = tempItem.getBodyPart();
			type2 = tempItem.getType2();
			owner_price = e.getOwnersPrice();
			buylist.add(new BuyItemInfo(_id, count, store_price, body_part, type2, owner_price));
		}

		_list = new L2TradeList(0);
		for(L2ItemInstance item : buyer.getInventory().getItems())
			if(item.getItem().getType2() != L2Item.TYPE2_QUEST && item.getItem().getType2() != L2Item.TYPE2_MONEY && item.canBeTraded(buyer))
			{
				for(TradeItem ti : buyer.getBuyList())
					if(ti.getItemId() == item.getItemId())
						continue;
				_list.addItem(item);
			}
	}

	@Override
	protected final void writeImpl()
	{
		writeC(0xB7);
		//section 1
		writeD(buyer_id);
		writeD(buyer_adena);

		//section2
		writeD(_list.getItems().size());//for potential sells
		for(L2ItemInstance temp : _list.getItems())
		{
			writeD(temp.getItemId());
			writeH(0); //show enchant lvl as 0, as you can't buy enchanted weapons
			writeD(temp.getIntegerLimitedCount());
			writeD(temp.getPriceToSell());
			writeH(0);
			writeD(temp.getBodyPart());
			writeH(temp.getItem().getType2());
		}

		//section 3
		writeD(buylist.size());//count for any items already added for sell
		for(BuyItemInfo e : buylist)
		{
			writeD(e._id);
			writeH(0);
			writeD(e.count);
			writeD(e.store_price);
			writeH(0);
			writeD(e.body_part);
			writeH(e.type2);
			writeD(e.owner_price);//your price
			writeD(e.store_price);//fixed store price
			writeD(-2); // attack element (-2 - none)
			writeD(0x00); // attack element value
			writeD(0x00); // водная стихия (fire pdef)
			writeD(0x00); // огненная стихия (water pdef)
			writeD(0x00); // земляная стихия (wind pdef)
			writeD(0x00); // воздушная стихия (earth pdef)
			writeD(0x00); // темная стихия (holy pdef)
			writeD(0x00); // светлая стихия (dark pdef)
		}
	}

	@Override
	public String getType()
	{
		return _S__BD_PRIVATESTOREMANAGELISTBUY;
	}

	static class BuyItemInfo
	{
		public int _id, count, store_price, body_part, type2, owner_price;

		public BuyItemInfo(int __id, int _count, int _store_price, int _body_part, int _type2, int _owner_price)
		{
			_id = __id;
			count = _count;
			store_price = _store_price;
			body_part = _body_part;
			type2 = _type2;
			owner_price = _owner_price;
		}
	}
}