package l2r.gameserver.serverpackets;

import java.util.ArrayList;
import java.util.List;

import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestDropInfo;
import l2r.gameserver.model.quest.QuestState;

/**
 *
 * sample forrev 377:
 *
 * 98
 * 05 00 		number of quests
 * ff 00 00 00
 * 0a 01 00 00
 * 39 01 00 00
 * 04 01 00 00
 * a2 00 00 00
 *
 * 04 00 		number of quest items
 *
 * 85 45 13 40 	item obj id
 * 36 05 00 00 	item id
 * 02 00 00 00 	count
 * 00 00 		?? bodyslot
 *
 * 23 bd 12 40
 * 86 04 00 00
 * 0a 00 00 00
 * 00 00
 *
 * 1f bd 12 40
 * 5a 04 00 00
 * 09 00 00 00
 * 00 00
 *
 * 1b bd 12 40
 * 5b 04 00 00
 * 39 00 00 00
 * 00 00                                                 .
 *
 * format h (d) h (dddh)   rev 377
 * format h (dd) h (dddd)  rev 417
 */
public class QuestList extends L2GameServerPacket
{
	private static final String _S__86_QUESTLIST = "[S] 86 QuestList";
	private Quest[] _quests;
	private L2Player _player;

	public QuestList(L2Player player)
	{
		if(player == null)
			return;
		_quests = player.getAllActiveQuests();
		_player = player;
	}

	@Override
	protected final void writeImpl()
	{
		/**
		 * This text was wrote by XaKa
		 * QuestList packet structure:
		 * {
		 * 		1 byte - 0x80
		 * 		2 byte - Number of Quests
		 * 		for Quest in AvailibleQuests
		 * 		{
		 * 			4 byte - Quest.ID
		 * 			4 byte - 1
		 * 		}
		 * 		2 byte - Number of All Quests Item
		 * 		for Item in AllQuestsItem
		 * 		{
		 * 			4 byte - Item.ObjID
		 * 			4 byte - Item.ID
		 * 			4 byte - Item.Count
		 * 			4 byte - 5
		 * 		}
		 */
		if(_player == null)
			return;

		if(_quests == null || _quests.length == 0)
		{
			writeC(0x80);
			writeH(0);
			writeH(0);
			return;
		}

		writeC(0x80);
		writeH(_quests.length);
		for(Quest q : _quests)
		{
			writeD(q.getQuestIntId());
			QuestState qs = _player.getQuestState(q.getName());
			if(qs == null)
			{
				writeD(0);
				continue;
			}
			writeD(qs.getInt("cond")); // stage of quest progress
		}

		//Prepare info about all quests
		List<QuestDropInfo> questDrops = new ArrayList<QuestDropInfo>();
		int FullCountDropItems = 0;
		for(Quest q : _quests)
		{
			QuestDropInfo newQDrop = new QuestDropInfo(_player, q.getName());
			//Calculate full count drop items
			FullCountDropItems += newQDrop.dropList.size();

			questDrops.add(newQDrop);
		}

		writeH(FullCountDropItems);
		for(QuestDropInfo currQDropInfo : questDrops)
			for(QuestDropInfo.DropInfo itemDropInfo : currQDropInfo.dropList)
			{
				writeD(itemDropInfo.dropItemObjID);
				writeD(itemDropInfo.dropItemID);
				writeD((int) itemDropInfo.dropItemCount);
				writeD(5); //<<<<---- what is that?
			}
	}

	@Override
	public String getType()
	{
		return _S__86_QUESTLIST;
	}
}