package l2r.gameserver.skills;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilderFactory;

import l2r.gameserver.GameServer;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.skills.conditions.Condition;
import l2r.gameserver.skills.conditions.ConditionForceBuff;
import l2r.gameserver.skills.conditions.ConditionGameTime;
import l2r.gameserver.skills.conditions.ConditionHasSkill;
import l2r.gameserver.skills.conditions.ConditionLogicAnd;
import l2r.gameserver.skills.conditions.ConditionLogicNot;
import l2r.gameserver.skills.conditions.ConditionLogicOr;
import l2r.gameserver.skills.conditions.ConditionPlayerMaxLevel;
import l2r.gameserver.skills.conditions.ConditionPlayerMaxPK;
import l2r.gameserver.skills.conditions.ConditionPlayerMinHp;
import l2r.gameserver.skills.conditions.ConditionPlayerMinLevel;
import l2r.gameserver.skills.conditions.ConditionPlayerPercentHp;
import l2r.gameserver.skills.conditions.ConditionPlayerPercentMp;
import l2r.gameserver.skills.conditions.ConditionPlayerRace;
import l2r.gameserver.skills.conditions.ConditionPlayerRiding;
import l2r.gameserver.skills.conditions.ConditionPlayerState;
import l2r.gameserver.skills.conditions.ConditionSlotItemId;
import l2r.gameserver.skills.conditions.ConditionTargetAggro;
import l2r.gameserver.skills.conditions.ConditionTargetCastleDoor;
import l2r.gameserver.skills.conditions.ConditionTargetDirection;
import l2r.gameserver.skills.conditions.ConditionTargetHasBuffId;
import l2r.gameserver.skills.conditions.ConditionTargetMob;
import l2r.gameserver.skills.conditions.ConditionTargetPlayable;
import l2r.gameserver.skills.conditions.ConditionTargetPlayerRace;
import l2r.gameserver.skills.conditions.ConditionTargetRace;
import l2r.gameserver.skills.conditions.ConditionUsingArmor;
import l2r.gameserver.skills.conditions.ConditionUsingItemType;
import l2r.gameserver.skills.conditions.ConditionUsingSkill;
import l2r.gameserver.skills.conditions.ConditionGameTime.CheckGameTime;
import l2r.gameserver.skills.conditions.ConditionPlayerRiding.CheckPlayerRiding;
import l2r.gameserver.skills.conditions.ConditionPlayerState.CheckPlayerState;
import l2r.gameserver.skills.conditions.ConditionTargetDirection.TargetDirection;
import l2r.gameserver.skills.effects.EffectTemplate;
import l2r.gameserver.skills.funcs.FuncTemplate;
import l2r.gameserver.skills.funcs.Lambda;
import l2r.gameserver.skills.funcs.LambdaCalc;
import l2r.gameserver.skills.funcs.LambdaConst;
import l2r.gameserver.skills.funcs.LambdaConstPercent;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.templates.L2Item;
import l2r.gameserver.templates.StatsSet;
import l2r.gameserver.templates.L2Armor.ArmorType;
import l2r.gameserver.templates.L2Weapon.WeaponType;
import l2r.util.Files;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

abstract class DocumentBase
{
	static Logger _log = Logger.getLogger(DocumentBase.class.getName());

	private File file;
	protected HashMap<String, Number[]> tables;

	DocumentBase(File file)
	{
		this.file = file;
		tables = new HashMap<String, Number[]>();
	}

	Document parse()
	{
		Document doc;
		try
		{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setValidating(false);
			factory.setIgnoringComments(true);
			if(GameServer.ltvj)
			{
				String content = Files.read(file);
				ByteArrayInputStream bais = new ByteArrayInputStream(content.getBytes());
				doc = factory.newDocumentBuilder().parse(bais);
			}
			else
				doc = factory.newDocumentBuilder().parse(file);
		}
		catch(Exception e)
		{
			_log.log(Level.SEVERE, "Error loading file " + file, e);
			return null;
		}
		try
		{
			parseDocument(doc);
		}
		catch(Exception e)
		{
			_log.log(Level.SEVERE, "Error in file " + file, e);
			return null;
		}
		return doc;
	}

	protected abstract void parseDocument(Document doc);

	protected abstract StatsSet getStatsSet();

	protected abstract Number getTableValue(String name);

	protected abstract Number getTableValue(String name, int idx);

	protected void resetTable()
	{
		tables = new HashMap<String, Number[]>();
	}

	protected void setTable(String name, Number[] table)
	{
		tables.put(name, table);
	}

	protected void parseTemplate(Node n, Object template)
	{
		Condition condition = null;
		n = n.getFirstChild();
		if(n == null)
			return;
		if("cond".equalsIgnoreCase(n.getNodeName()))
		{
			condition = parseCondition(n.getFirstChild(), template);
			Node msg = n.getAttributes().getNamedItem("msg");
			if(condition != null && msg != null)
				condition.setMessage(msg.getNodeValue());
			n = n.getNextSibling();
		}
		for(; n != null; n = n.getNextSibling())
		{
			String nodeName = n.getNodeName();
			if("add".equalsIgnoreCase(nodeName))
				attachFunc(n, template, "Add", condition);
			else if("sub".equalsIgnoreCase(nodeName))
				attachFunc(n, template, "Sub", condition);
			else if("mul".equalsIgnoreCase(nodeName))
				attachFunc(n, template, "Mul", condition);
			else if("div".equalsIgnoreCase(nodeName))
				attachFunc(n, template, "Div", condition);
			else if("set".equalsIgnoreCase(nodeName))
				attachFunc(n, template, "Set", condition);
			else if("enchant".equalsIgnoreCase(nodeName))
				attachFunc(n, template, "Enchant", condition);
			else if("effect".equalsIgnoreCase(nodeName))
			{
				if(template instanceof EffectTemplate)
					throw new RuntimeException("Nested effects");
				attachEffect(n, template, condition);
			}
			else if("skill".equalsIgnoreCase(nodeName))
				attachSkill(n, (L2Item) template, condition);
		}
	}

	protected void attachSkill(Node n, L2Item template, @SuppressWarnings("unused")
	Condition condition)
	{
		NamedNodeMap attrs = n.getAttributes();
		int skillId = Short.valueOf(attrs.getNamedItem("id").getNodeValue());
		int skillLevel = Byte.valueOf(attrs.getNamedItem("level").getNodeValue());
		byte chance = getNumber(attrs.getNamedItem("chance").getNodeValue(), template).byteValue();
		String action = attrs.getNamedItem("action").getNodeValue();
		L2Skill skill = SkillTable.getInstance().getInfo(skillId, skillLevel);
		if("crit".equalsIgnoreCase(action))
			template.attachSkillOnAction(skill, chance, true);
		else if("magic".equalsIgnoreCase(action))
			template.attachSkillOnAction(skill, chance, false);
		else if("add".equalsIgnoreCase(action))
			template.attachSkill(skill);
		else
			throw new NoSuchElementException("Unsupported action type for weapon-attached skill");
	}

	protected void attachFunc(Node n, Object template, String name, Condition attachCond)
	{
		Stats stat = Stats.valueOfXml(n.getAttributes().getNamedItem("stat").getNodeValue());
		String order = n.getAttributes().getNamedItem("order").getNodeValue();
		Lambda lambda = getLambda(n, template);
		int ord = getNumber(order, template).intValue();
		Condition applyCond = parseCondition(n.getFirstChild(), template);
		FuncTemplate ft = new FuncTemplate(attachCond, applyCond, name, stat, ord, lambda);
		if(template instanceof L2Item)
			((L2Item) template).attachFunction(ft);
		else if(template instanceof L2Skill)
			((L2Skill) template).attach(ft);
		else if(template instanceof EffectTemplate)
			((EffectTemplate) template).attach(ft);
	}

	protected void attachLambdaFunc(Node n, Object template, LambdaCalc calc)
	{
		String name = n.getNodeName();
		StringBuffer sb = new StringBuffer(name);
		sb.setCharAt(0, Character.toUpperCase(name.charAt(0)));
		name = sb.toString();
		Lambda lambda = getLambda(n, template);
		FuncTemplate ft = new FuncTemplate(null, null, name, null, calc._funcs.length, lambda);
		calc.addFunc(ft.getFunc(new Env(), calc));
	}

	protected void attachEffect(Node n, Object template, Condition attachCond)
	{
		NamedNodeMap attrs = n.getAttributes();
		String name = attrs.getNamedItem("name").getNodeValue();
		long time = 1;
		int count = 1;

		if(attrs.getNamedItem("count") != null)
			count = getNumber(attrs.getNamedItem("count").getNodeValue(), template).intValue();
		if(attrs.getNamedItem("time") != null)
			time = (long) (getNumber(attrs.getNamedItem("time").getNodeValue(), template).doubleValue() * 1000);

		Lambda lambda = getLambda(n, template);
		Condition applyCond = parseCondition(n.getFirstChild(), template);
		int abnormal = 0;

		if(attrs.getNamedItem("abnormal") != null)
		{
			String abn = attrs.getNamedItem("abnormal").getNodeValue();
			if(abn.equals("poison"))
				abnormal = L2Character.ABNORMAL_EFFECT_POISON;
			if(abn.equals("bleeding"))
				abnormal = L2Character.ABNORMAL_EFFECT_BLEEDING;
			if(abn.equals("flame"))
				abnormal = L2Character.ABNORMAL_EFFECT_FLAME;
			if(abn.equals("bighead"))
				abnormal = L2Character.ABNORMAL_EFFECT_BIG_HEAD;
			if(abn.equals("shadow"))
				abnormal = L2Character.ABNORMAL_EFFECT_SILENT_MOVE;
		}

		int stackOrder = 0;
		String stackType = "none";

		if(attrs.getNamedItem("stackType") != null)
			stackType = attrs.getNamedItem("stackType").getNodeValue();
		if(attrs.getNamedItem("stackOrder") != null)
			stackOrder = getNumber(attrs.getNamedItem("stackOrder").getNodeValue(), template).intValue();

		boolean applyOnCaster = false;
		if(attrs.getNamedItem("applyOnCaster") != null)
			applyOnCaster = Boolean.valueOf(attrs.getNamedItem("applyOnCaster").getNodeValue());

		EffectTemplate lt = new EffectTemplate(attachCond, applyCond, name, lambda, count, time, abnormal, stackType, stackOrder, applyOnCaster);

		parseTemplate(n, lt);

		if(template instanceof L2Skill)
			((L2Skill) template).attach(lt);
	}

	protected Condition parseCondition(Node n, Object template)
	{
		while(n != null && n.getNodeType() != Node.ELEMENT_NODE)
			n = n.getNextSibling();
		if(n == null)
			return null;
		if("and".equalsIgnoreCase(n.getNodeName()))
			return parseLogicAnd(n, template);
		if("or".equalsIgnoreCase(n.getNodeName()))
			return parseLogicOr(n, template);
		if("not".equalsIgnoreCase(n.getNodeName()))
			return parseLogicNot(n, template);
		if("player".equalsIgnoreCase(n.getNodeName()))
			return parsePlayerCondition(n);
		if("target".equalsIgnoreCase(n.getNodeName()))
			return parseTargetCondition(n, template);
		if("has".equalsIgnoreCase(n.getNodeName()))
			return parseHasCondition(n);
		if("using".equalsIgnoreCase(n.getNodeName()))
			return parseUsingCondition(n);
		if("game".equalsIgnoreCase(n.getNodeName()))
			return parseGameCondition(n);
		return null;
	}

	protected Condition parseLogicAnd(Node n, Object template)
	{
		ConditionLogicAnd cond = new ConditionLogicAnd();
		for(n = n.getFirstChild(); n != null; n = n.getNextSibling())
			if(n.getNodeType() == Node.ELEMENT_NODE)
				cond.add(parseCondition(n, template));
		if(cond._conditions == null || cond._conditions.length == 0)
			_log.severe("Empty <and> condition in " + file);
		return cond;
	}

	protected Condition parseLogicOr(Node n, Object template)
	{
		ConditionLogicOr cond = new ConditionLogicOr();
		for(n = n.getFirstChild(); n != null; n = n.getNextSibling())
			if(n.getNodeType() == Node.ELEMENT_NODE)
				cond.add(parseCondition(n, template));
		if(cond._conditions == null || cond._conditions.length == 0)
			_log.severe("Empty <or> condition in " + file);
		return cond;
	}

	protected Condition parseLogicNot(Node n, Object template)
	{
		for(n = n.getFirstChild(); n != null; n = n.getNextSibling())
			if(n.getNodeType() == Node.ELEMENT_NODE)
				return new ConditionLogicNot(parseCondition(n, template));
		_log.severe("Empty <not> condition in " + file);
		return null;
	}

	protected Condition parsePlayerCondition(Node n)
	{
		int[] forces = new int[2];
		Condition cond = null;
		NamedNodeMap attrs = n.getAttributes();
		for(int i = 0; i < attrs.getLength(); i++)
		{
			Node a = attrs.item(i);
			String nodeName = a.getNodeName();
			if("race".equalsIgnoreCase(nodeName))
				cond = joinAnd(cond, new ConditionPlayerRace(a.getNodeValue()));
			else if("minLevel".equalsIgnoreCase(nodeName))
			{
				int lvl = getNumber(a.getNodeValue(), null).intValue();
				cond = joinAnd(cond, new ConditionPlayerMinLevel(lvl));
			}
			else if("maxLevel".equalsIgnoreCase(nodeName))
			{
				int lvl = getNumber(a.getNodeValue(), null).intValue();
				cond = joinAnd(cond, new ConditionPlayerMaxLevel(lvl));
			}
			else if("maxPK".equalsIgnoreCase(nodeName))
			{
				int pk = getNumber(a.getNodeValue(), null).intValue();
				cond = joinAnd(cond, new ConditionPlayerMaxPK(pk));
			}
			else if("resting".equalsIgnoreCase(nodeName))
			{
				boolean val = Boolean.valueOf(a.getNodeValue());
				cond = joinAnd(cond, new ConditionPlayerState(CheckPlayerState.RESTING, val));
			}
			else if("moving".equalsIgnoreCase(nodeName))
			{
				boolean val = Boolean.valueOf(a.getNodeValue());
				cond = joinAnd(cond, new ConditionPlayerState(CheckPlayerState.MOVING, val));
			}
			else if("running".equalsIgnoreCase(nodeName))
			{
				boolean val = Boolean.valueOf(a.getNodeValue());
				cond = joinAnd(cond, new ConditionPlayerState(CheckPlayerState.RUNNING, val));
			}
			else if("standing".equalsIgnoreCase(nodeName))
			{
				boolean val = Boolean.valueOf(a.getNodeValue());
				cond = joinAnd(cond, new ConditionPlayerState(CheckPlayerState.STANDING, val));
			}
			else if("flying".equalsIgnoreCase(a.getNodeName()))
			{
				boolean val = Boolean.valueOf(a.getNodeValue());
				cond = joinAnd(cond, new ConditionPlayerState(CheckPlayerState.FLYING, val));
			}
			else if("percentHP".equalsIgnoreCase(nodeName))
			{
				int percentHP = getNumber(a.getNodeValue(), null).intValue();
				cond = joinAnd(cond, new ConditionPlayerPercentHp(percentHP));
			}
			else if("minHP".equalsIgnoreCase(nodeName))
			{
				int minHP = getNumber(a.getNodeValue(), null).intValue();
				cond = joinAnd(cond, new ConditionPlayerMinHp(minHP));
			}
			else if("percentMP".equalsIgnoreCase(nodeName))
			{
				int percentMP = getNumber(a.getNodeValue(), null).intValue();
				cond = joinAnd(cond, new ConditionPlayerPercentMp(percentMP));
			}
			else if("riding".equalsIgnoreCase(nodeName))
			{
				String riding = a.getNodeValue();
				if("strider".equalsIgnoreCase(riding))
					cond = joinAnd(cond, new ConditionPlayerRiding(CheckPlayerRiding.STRIDER));
				else if("wyvern".equalsIgnoreCase(riding))
					cond = joinAnd(cond, new ConditionPlayerRiding(CheckPlayerRiding.WYVERN));
				else if("none".equalsIgnoreCase(riding))
					cond = joinAnd(cond, new ConditionPlayerRiding(CheckPlayerRiding.NONE));
			}
			else if("battle_force".equalsIgnoreCase(a.getNodeName()))
				forces[0] = getNumber(a.getNodeValue(), null).intValue();
			else if("spell_force".equalsIgnoreCase(a.getNodeName()))
				forces[1] = getNumber(a.getNodeValue(), null).intValue();
		}

		if(forces[0] + forces[1] > 0)
			cond = joinAnd(cond, new ConditionForceBuff(forces));

		if(cond == null)
			_log.severe("Unrecognized <player> condition in " + file);
		return cond;
	}

	protected Condition parseTargetCondition(Node n, @SuppressWarnings("unused")
	Object template)
	{
		Condition cond = null;
		NamedNodeMap attrs = n.getAttributes();
		for(int i = 0; i < attrs.getLength(); i++)
		{
			Node a = attrs.item(i);
			String nodeName = a.getNodeName();
			if("aggro".equalsIgnoreCase(nodeName))
			{
				boolean val = Boolean.valueOf(a.getNodeValue());
				cond = joinAnd(cond, new ConditionTargetAggro(val));
			}
			else if("pvp".equalsIgnoreCase(nodeName))
			{
				boolean val = Boolean.valueOf(a.getNodeValue());
				cond = joinAnd(cond, new ConditionTargetPlayable(val));
			}
			else if("mob".equalsIgnoreCase(nodeName))
			{
				boolean val = Boolean.valueOf(a.getNodeValue());
				cond = joinAnd(cond, new ConditionTargetMob(val));
			}
			else if("race".equalsIgnoreCase(nodeName))
				cond = joinAnd(cond, new ConditionTargetRace(a.getNodeValue()));
			else if("playerRace".equalsIgnoreCase(nodeName))
				cond = joinAnd(cond, new ConditionTargetPlayerRace(a.getNodeValue()));
			else if("castledoor".equalsIgnoreCase(nodeName))
			{
				boolean val = Boolean.valueOf(a.getNodeValue());
				cond = joinAnd(cond, new ConditionTargetCastleDoor(val));
			}
			else if("direction".equalsIgnoreCase(nodeName))
			{
				String val = a.getNodeValue();
				if("behind".equalsIgnoreCase(val))
					cond = joinAnd(cond, new ConditionTargetDirection(TargetDirection.BEHIND));
				if("front".equalsIgnoreCase(val))
					cond = joinAnd(cond, new ConditionTargetDirection(TargetDirection.FRONT));
				if("side".equalsIgnoreCase(val))
					cond = joinAnd(cond, new ConditionTargetDirection(TargetDirection.FRONT));
			}
			else if("hasBuffId".equalsIgnoreCase(nodeName))
			{
				Integer id = getNumber(a.getNodeValue().trim(), null).intValue();
				cond = joinAnd(cond, new ConditionTargetHasBuffId(id));
			}
		}
		if(cond == null)
			_log.severe("Unrecognized <target> condition in " + file);
		return cond;
	}

	protected Condition parseUsingCondition(Node n)
	{
		Condition cond = null;
		NamedNodeMap attrs = n.getAttributes();
		for(int i = 0; i < attrs.getLength(); i++)
		{
			Node a = attrs.item(i);
			String nodeName = a.getNodeName();
			String nodeValue = a.getNodeValue();
			if("kind".equalsIgnoreCase(nodeName))
			{
				int mask = 0;
				StringTokenizer st = new StringTokenizer(nodeValue, ",");
				while(st.hasMoreTokens())
				{
					String item = st.nextToken().trim();
					for(WeaponType wt : WeaponType.values())
						if(wt.toString().equalsIgnoreCase(item))
						{
							mask |= wt.mask();
							break;
						}
				}
				cond = joinAnd(cond, new ConditionUsingItemType(mask));
			}
			else if("armor".equalsIgnoreCase(nodeName))
			{
				ArmorType armor = ArmorType.valueOf(nodeValue.toUpperCase());
				cond = joinAnd(cond, new ConditionUsingArmor(armor));
			}
			else if("skill".equalsIgnoreCase(nodeName))
			{
				int id = Integer.parseInt(nodeValue);
				cond = joinAnd(cond, new ConditionUsingSkill(id));
			}
			else if("slotitem".equalsIgnoreCase(nodeName))
			{
				StringTokenizer st = new StringTokenizer(nodeValue, ";");
				int id = Integer.parseInt(st.nextToken().trim());
				short slot = Short.parseShort(st.nextToken().trim());
				int enchant = 0;
				if(st.hasMoreTokens())
					enchant = Integer.parseInt(st.nextToken().trim());
				cond = joinAnd(cond, new ConditionSlotItemId(slot, id, enchant));
			}
			else if("direction".equalsIgnoreCase(nodeName))
			{
				TargetDirection Direction = TargetDirection.valueOf(nodeValue.toUpperCase());
				cond = joinAnd(cond, new ConditionTargetDirection(Direction));
			}
		}
		if(cond == null)
			_log.severe("Unrecognized <using> condition in " + file);
		return cond;
	}

	protected Condition parseHasCondition(Node n)
	{
		Condition cond = null;
		NamedNodeMap attrs = n.getAttributes();
		for(int i = 0; i < attrs.getLength(); i++)
		{
			Node a = attrs.item(i);
			String nodeName = a.getNodeName();
			String nodeValue = a.getNodeValue();
			if("skill".equalsIgnoreCase(nodeName))
			{
				StringTokenizer st = new StringTokenizer(nodeValue, ";");
				Integer id = getNumber(st.nextToken().trim(), null).intValue();
				short level = getNumber(st.nextToken().trim(), null).shortValue();
				cond = joinAnd(cond, new ConditionHasSkill(id, level));
			}
		}
		if(cond == null)
			_log.severe("Unrecognized <has> condition in " + file);
		return cond;
	}

	protected Condition parseGameCondition(Node n)
	{
		Condition cond = null;
		NamedNodeMap attrs = n.getAttributes();
		for(int i = 0; i < attrs.getLength(); i++)
		{
			Node a = attrs.item(i);
			if("night".equalsIgnoreCase(a.getNodeName()))
			{
				boolean val = Boolean.valueOf(a.getNodeValue());
				cond = joinAnd(cond, new ConditionGameTime(CheckGameTime.NIGHT, val));
			}
		}
		if(cond == null)
			_log.severe("Unrecognized <game> condition in " + file);
		return cond;
	}

	protected void parseTable(Node n)
	{
		NamedNodeMap attrs = n.getAttributes();
		String name = attrs.getNamedItem("name").getNodeValue();
		if(name.charAt(0) != '#')
			throw new IllegalArgumentException("Table name must start with #");
		StringTokenizer data = new StringTokenizer(n.getFirstChild().getNodeValue());
		ArrayList<String> array = new ArrayList<String>();
		while(data.hasMoreTokens())
			array.add(data.nextToken());
		Number[] res = new Number[array.size()];
		for(int i = 0; i < array.size(); i++)
			res[i] = getNumber(array.get(i), null);
		setTable(name, res);
	}

	protected void parseBeanSet(Node n, StatsSet set, Integer level)
	{
		String name = n.getAttributes().getNamedItem("name").getNodeValue().trim();
		String value = n.getAttributes().getNamedItem("val").getNodeValue().trim();
		char ch = value.length() == 0 ? ' ' : value.charAt(0);
		if((ch == '#' || ch == '-' || Character.isDigit(ch)) && !value.contains(" ") && !value.contains(";"))
			set.set(name, String.valueOf(getNumber(value, level)));
		else
			set.set(name, value);
	}

	protected Lambda getLambda(Node n, Object template)
	{
		Node nval = n.getAttributes().getNamedItem("val");
		if(nval != null)
		{
			String val = nval.getNodeValue();
			if(val.charAt(0) == '#') // table by level
			{
				if(val.charAt(val.length() - 1) == '%')
				{
					String newVal = val.substring(0, val.length() - 1).trim();
					return new LambdaConstPercent(getTableValue(newVal).doubleValue());
				}
				return new LambdaConst(getTableValue(val.trim()).doubleValue());
			}

			if(val.charAt(val.length() - 1) == '%')
			{
				String newVal = val.substring(0, val.length() - 1);
				return new LambdaConstPercent(Double.parseDouble(newVal));
			}

			return new LambdaConst(Double.parseDouble(val));
		}

		LambdaCalc calc = new LambdaCalc();
		n = n.getFirstChild();
		while(n != null && n.getNodeType() != Node.ELEMENT_NODE)
			n = n.getNextSibling();
		if(n == null || !"val".equals(n.getNodeName()))
			throw new IllegalArgumentException("Value not specified");

		for(n = n.getFirstChild(); n != null; n = n.getNextSibling())
		{
			if(n.getNodeType() != Node.ELEMENT_NODE)
				continue;
			attachLambdaFunc(n, template, calc);
		}
		return calc;
	}

	protected Number getNumber(String value, Object template)
	{
		if(value.charAt(0) == '#')
			if(template == null || template instanceof L2Skill)
				return getTableValue(value);
			else if(template instanceof Integer)
				return getTableValue(value, (Integer) template);
			else
				throw new IllegalStateException();
		if(value.indexOf('.') == -1)
		{
			int radix = 10;
			if(value.length() > 2 && value.substring(0, 2).equalsIgnoreCase("0x"))
			{
				value = value.substring(2);
				radix = 16;
			}
			return Integer.valueOf(value, radix);
		}
		return Double.valueOf(value);
	}

	protected Condition joinAnd(Condition cond, Condition c)
	{
		if(cond == null)
			return c;
		if(cond instanceof ConditionLogicAnd)
		{
			((ConditionLogicAnd) cond).add(c);
			return cond;
		}
		ConditionLogicAnd and = new ConditionLogicAnd();
		and.add(cond);
		and.add(c);
		return and;
	}
}