package l2r.gameserver.skills;

import java.io.File;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import l2r.gameserver.Item;
import l2r.gameserver.templates.L2Armor;
import l2r.gameserver.templates.L2EtcItem;
import l2r.gameserver.templates.L2Item;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.StatsSet;
import l2r.gameserver.templates.L2Armor.ArmorType;
import l2r.gameserver.templates.L2EtcItem.EtcItemType;
import l2r.gameserver.templates.L2Weapon.WeaponType;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

final class DocumentItem extends DocumentBase
{
	private Item currentItem = null;
	private List<L2Item> itemsInFile = new LinkedList<L2Item>();
	private Map<Integer, Item> itemData = new HashMap<Integer, Item>();

	/**
	 * @param armorData
	 * @param f
	 */
	public DocumentItem(HashMap<Integer, Item> itemData, File file)
	{
		super(file);
		this.itemData = itemData;
	}

	/**
	 * @param item
	 */
	private void setCurrentItem(Item item)
	{
		currentItem = item;
	}

	@Override
	protected StatsSet getStatsSet()
	{
		return currentItem.set;
	}

	@Override
	protected Number getTableValue(String name)
	{
		return tables.get(name)[currentItem.currentLevel];
	}

	@Override
	protected Number getTableValue(String name, int idx)
	{
		return tables.get(name)[idx - 1];
	}

	@Override
	protected void parseDocument(Document doc)
	{
		for(Node n = doc.getFirstChild(); n != null; n = n.getNextSibling())
			if("list".equalsIgnoreCase(n.getNodeName()))
			{

				for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
					if("item".equalsIgnoreCase(d.getNodeName()))
					{
						setCurrentItem(new Item());
						parseItem(d);
						itemsInFile.add(currentItem.item);
						resetTable();
					}
			}
			else if("item".equalsIgnoreCase(n.getNodeName()))
			{
				setCurrentItem(new Item());
				parseItem(n);
				itemsInFile.add(currentItem.item);
			}
	}

	protected void parseItem(Node n)
	{
		int itemId = Integer.parseInt(n.getAttributes().getNamedItem("id").getNodeValue());
		String itemName = n.getAttributes().getNamedItem("name").getNodeValue();

		currentItem.id = itemId;
		currentItem.name = itemName;

		if(itemData.get(currentItem.id) == null)
		{
			_log.warning("Item " + itemId + "(" + itemName + ") not found in .xml templates");
			return;
		}

		currentItem.set = itemData.get(currentItem.id).set;
		currentItem.type = itemData.get(currentItem.id).type;

		Node first = n.getFirstChild();
		for(n = first; n != null; n = n.getNextSibling())
			if("table".equalsIgnoreCase(n.getNodeName()))
				parseTable(n);
		for(n = first; n != null; n = n.getNextSibling())
			if("set".equalsIgnoreCase(n.getNodeName()))
				parseBeanSet(n, itemData.get(currentItem.id).set, 1);
		for(n = first; n != null; n = n.getNextSibling())
			if("for".equalsIgnoreCase(n.getNodeName()))
			{
				makeItem();
				parseTemplate(n, currentItem.item);
			}
	}

	private void makeItem()
	{
		if(currentItem.item != null)
			return;
		if(currentItem.type instanceof ArmorType)
			currentItem.item = new L2Armor((ArmorType) currentItem.type, currentItem.set);
		else if(currentItem.type instanceof WeaponType)
			currentItem.item = new L2Weapon((WeaponType) currentItem.type, currentItem.set);
		else if(currentItem.type instanceof EtcItemType)
			currentItem.item = new L2EtcItem((EtcItemType) currentItem.type, currentItem.set);
		else
			throw new Error("Unknown item type " + currentItem.type);
	}

	public List<L2Item> getItemList()
	{
		return itemsInFile;
	}
}