package l2r.gameserver.skills;

import java.io.File;
import java.io.IOException;
import java.util.TreeMap;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import l2r.Config;
import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.instancemanager.ClanHallManager;
import l2r.gameserver.model.Inventory;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Effect;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Summon;
import l2r.gameserver.model.base.Race;
import l2r.gameserver.model.entity.ClanHall;
import l2r.gameserver.model.entity.SevenSigns;
import l2r.gameserver.model.instances.L2DoorInstance;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.instances.L2RaidBossInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.skills.conditions.ConditionPlayerState;
import l2r.gameserver.skills.conditions.ConditionPlayerState.CheckPlayerState;
import l2r.gameserver.skills.effects.ResistType;
import l2r.gameserver.skills.funcs.Func;
import l2r.gameserver.templates.L2PlayerTemplate;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.L2Armor.ArmorType;
import l2r.gameserver.templates.L2Weapon.WeaponType;
import l2r.util.Rnd;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

public class Formulas
{
	/** Regen Task period */
	protected static final Logger _log = Logger.getLogger(L2Character.class.getName());
	private static final int HP_REGENERATE_PERIOD = 3000; // 3 secs

	public static int MAX_STAT_VALUE = 100;

	public static final double[] WITbonus = new double[MAX_STAT_VALUE];
	public static final double[] MENbonus = new double[MAX_STAT_VALUE];
	public static final double[] INTbonus = new double[MAX_STAT_VALUE];
	public static final double[] STRbonus = new double[MAX_STAT_VALUE];
	public static final double[] DEXbonus = new double[MAX_STAT_VALUE];
	public static final double[] CONbonus = new double[MAX_STAT_VALUE];

	static
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(false);
		factory.setIgnoringComments(true);
		File file = new File(Config.AttributeBonusFile);
		Document doc = null;

		try
		{
			doc = factory.newDocumentBuilder().parse(file);
		}
		catch(SAXException e)
		{
			e.printStackTrace();
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
		catch(ParserConfigurationException e)
		{
			e.printStackTrace();
		}

		int i;
		double val;

		if(doc != null)
			for(Node z = doc.getFirstChild(); z != null; z = z.getNextSibling())
				for(Node n = z.getFirstChild(); n != null; n = n.getNextSibling())
				{
					if(n.getNodeName().equalsIgnoreCase("str_bonus"))
						for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
						{
							String node = d.getNodeName();
							if(node.equalsIgnoreCase("set"))
							{
								i = Integer.valueOf(d.getAttributes().getNamedItem("attribute").getNodeValue());
								val = Integer.valueOf(d.getAttributes().getNamedItem("val").getNodeValue());
								STRbonus[i] = (100 + val) / 100;
							}
						}
					if(n.getNodeName().equalsIgnoreCase("int_bonus"))
						for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
						{
							String node = d.getNodeName();
							if(node.equalsIgnoreCase("set"))
							{
								i = Integer.valueOf(d.getAttributes().getNamedItem("attribute").getNodeValue());
								val = Integer.valueOf(d.getAttributes().getNamedItem("val").getNodeValue());
								INTbonus[i] = (100 + val) / 100;
							}
						}
					if(n.getNodeName().equalsIgnoreCase("con_bonus"))
						for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
						{
							String node = d.getNodeName();
							if(node.equalsIgnoreCase("set"))
							{
								i = Integer.valueOf(d.getAttributes().getNamedItem("attribute").getNodeValue());
								val = Integer.valueOf(d.getAttributes().getNamedItem("val").getNodeValue());
								CONbonus[i] = (100 + val) / 100;
							}
						}
					if(n.getNodeName().equalsIgnoreCase("men_bonus"))
						for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
						{
							String node = d.getNodeName();
							if(node.equalsIgnoreCase("set"))
							{
								i = Integer.valueOf(d.getAttributes().getNamedItem("attribute").getNodeValue());
								val = Integer.valueOf(d.getAttributes().getNamedItem("val").getNodeValue());
								MENbonus[i] = (100 + val) / 100;
							}
						}
					if(n.getNodeName().equalsIgnoreCase("dex_bonus"))
						for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
						{
							String node = d.getNodeName();
							if(node.equalsIgnoreCase("set"))
							{
								i = Integer.valueOf(d.getAttributes().getNamedItem("attribute").getNodeValue());
								val = Integer.valueOf(d.getAttributes().getNamedItem("val").getNodeValue());
								DEXbonus[i] = (100 + val) / 100;
							}
						}
					if(n.getNodeName().equalsIgnoreCase("wit_bonus"))
						for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
						{
							String node = d.getNodeName();
							if(node.equalsIgnoreCase("set"))
							{
								i = Integer.valueOf(d.getAttributes().getNamedItem("attribute").getNodeValue());
								val = Integer.valueOf(d.getAttributes().getNamedItem("val").getNodeValue());
								WITbonus[i] = (100 + val) / 100;
							}
						}

				}
	}

	private static class FuncMultRegenResting extends Func
	{
		static final FuncMultRegenResting[] _instance = new FuncMultRegenResting[Stats.NUM_STATS];

		static Func getInstance(Stats stat)
		{
			int pos = stat.ordinal();
			if(_instance[pos] == null)
				_instance[pos] = new FuncMultRegenResting(stat);
			return _instance[pos];
		}

		private FuncMultRegenResting(Stats stat)
		{
			super(stat, 0x20, null);
			setCondition(new ConditionPlayerState(CheckPlayerState.RESTING, true));
		}

		@Override
		public void calc(Env env)
		{
			if(!_cond.test(env))
				return;

			if(env.character.isPlayer && env.character.getLevel() <= 40 && ((L2Player) env.character).getClassId().getLevel() < 3)
				env.value *= 6;
			else
				env.value *= 1.5;
		}
	}

	private static class FuncMultRegenStanding extends Func
	{
		static final FuncMultRegenStanding[] _instance = new FuncMultRegenStanding[Stats.NUM_STATS];

		static Func getInstance(Stats stat)
		{
			int pos = stat.ordinal();
			if(_instance[pos] == null)
				_instance[pos] = new FuncMultRegenStanding(stat);
			return _instance[pos];
		}

		private FuncMultRegenStanding(Stats stat)
		{
			super(stat, 0x20, null);
			setCondition(new ConditionPlayerState(CheckPlayerState.STANDING, true));
		}

		@Override
		public void calc(Env env)
		{
			if(!_cond.test(env))
				return;

			env.value *= 1.1;
		}
	}

	private static class FuncMultRegenRunning extends Func
	{
		static final FuncMultRegenRunning[] _instance = new FuncMultRegenRunning[Stats.NUM_STATS];

		static Func getInstance(Stats stat)
		{
			int pos = stat.ordinal();
			if(_instance[pos] == null)
				_instance[pos] = new FuncMultRegenRunning(stat);
			return _instance[pos];
		}

		private FuncMultRegenRunning(Stats stat)
		{
			super(stat, 0x20, null);
			setCondition(new ConditionPlayerState(CheckPlayerState.RUNNING, true));
		}

		@Override
		public void calc(Env env)
		{
			if(!_cond.test(env))
				return;
			env.value *= 0.7;
		}
	}

	private static class FuncPAtkMod extends Func
	{
		static final FuncPAtkMod _instance = new FuncPAtkMod();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncPAtkMod()
		{
			super(Stats.POWER_ATTACK, 0x20, null);
		}

		@Override
		public void calc(Env env)
		{
			env.value *= STRbonus[env.character.getSTR()] * env.character.getLevelMod();
		}
	}

	private static class FuncMAtkMod extends Func
	{
		static final FuncMAtkMod _instance = new FuncMAtkMod();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncMAtkMod()
		{
			super(Stats.MAGIC_ATTACK, 0x20, null);
		}

		@Override
		public void calc(Env env)
		{
			//{Wpn*(lvlbn^2)*[(1+INTbn)^2]+Msty}
			double ib = INTbonus[env.character.getINT()];
			double lvlb = env.character.getLevelMod();
			env.value *= lvlb * lvlb * ib * ib;
		}
	}

	private static class FuncPDefMod extends Func
	{
		static final FuncPDefMod _instance = new FuncPDefMod();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncPDefMod()
		{
			super(Stats.POWER_DEFENCE, 0x20, null);
		}

		@Override
		public void calc(Env env)
		{
			env.value *= env.character.getLevelMod();
		}
	}

	private static class FuncMDefMod extends Func
	{
		static final FuncMDefMod _instance = new FuncMDefMod();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncMDefMod()
		{
			super(Stats.MAGIC_DEFENCE, 0x20, null);
		}

		@Override
		public void calc(Env env)
		{
			env.value *= MENbonus[env.character.getMEN()] * env.character.getLevelMod();
		}
	}

	private static class FuncAttackRange extends Func
	{
		static final FuncAttackRange _instance = new FuncAttackRange();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncAttackRange()
		{
			super(Stats.POWER_ATTACK_RANGE, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			L2Weapon weapon = env.character.getActiveWeaponItem();
			if(weapon != null)
				if(weapon.getItemType() == WeaponType.BOW)
					env.value += 460;
				else if(weapon.getItemType() == WeaponType.CROSSBOW)
					env.value += 350;
				else if(weapon.getItemType() == WeaponType.POLE)
					env.value += 40;
		}
	}

	private static class FuncAtkAccuracy extends Func
	{
		static final FuncAtkAccuracy _instance = new FuncAtkAccuracy();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncAtkAccuracy()
		{
			super(Stats.ACCURACY_COMBAT, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			//[Square(DEX)]*6 + lvl + weapon hitbonus;
			env.value += Math.sqrt(env.character.getDEX()) * 6 + env.character.getLevel();
			if(env.character.isSummon)
				env.value += 5;
		}
	}

	private static class FuncAtkEvasion extends Func
	{
		static final FuncAtkEvasion _instance = new FuncAtkEvasion();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncAtkEvasion()
		{
			super(Stats.EVASION_RATE, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			//[Square(DEX)]*6 + lvl;
			env.value += Math.sqrt(env.character.getDEX()) * 6 + env.character.getLevel();
		}
	}

	private static class FuncAtkCritical extends Func
	{
		static final FuncAtkCritical _instance = new FuncAtkCritical();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncAtkCritical()
		{
			super(Stats.CRITICAL_RATE_STATIC, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			env.value *= DEXbonus[env.character.getDEX()];
			if(env.character instanceof L2Summon)
				env.value = env.character.getTemplate().baseCritRate;
			else if(env.character.getActiveWeaponInstance() == null)
				env.value = 40;
			else
				env.value *= 10;
		}
	}

	private static class FuncAtkCriticalPercents extends Func
	{
		static final FuncAtkCriticalPercents _instance = new FuncAtkCriticalPercents();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncAtkCriticalPercents()
		{
			super(Stats.CRITICAL_RATE_STATIC, 0x30, null);
		}

		@Override
		public void calc(Env env)
		{
			env.value += env.value * env.character.calcStat(Stats.CRITICAL_RATE, 0, env.target, env.skill) / 100.;
		}
	}

	private static class FuncMAtkCriticalPercents extends Func
	{
		static final FuncMAtkCriticalPercents _instance = new FuncMAtkCriticalPercents();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncMAtkCriticalPercents()
		{
			super(Stats.MCRITICAL_RATE, 0x30, null);
		}

		@Override
		public void calc(Env env)
		{
			env.value *= WITbonus[env.character.getWIT()];
		}
	}

	private static class FuncAtkFinalCritical extends Func
	{
		static final FuncAtkFinalCritical _instance = new FuncAtkFinalCritical();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncAtkFinalCritical()
		{
			super(Stats.CRITICAL_RATE_STATIC, 0xFF, null);
		}

		@Override
		public void calc(Env env)
		{
			if(env.value > 500)
				env.value = 500;
		}
	}

	/**
	 * надо уточнить какие типы брони как влияют
	 */
	private static class FuncAtkCriticalDmgRcpt extends Func
	{
		static final FuncAtkCriticalDmgRcpt _instance = new FuncAtkCriticalDmgRcpt();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncAtkCriticalDmgRcpt()
		{
			super(Stats.CRIT_DAMAGE_RECEPTIVE, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			L2ItemInstance i = ((L2Player) env.character).getInventory().getPaperdollItem(Inventory.PAPERDOLL_CHEST);
			if(i != null && i.getItemType() == ArmorType.HEAVY)
				env.value -= 20;
		}
	}

	/**
	 * надо уточнить какие типы брони как влияют
	 */
	private static class FuncAtkCriticalChanceRcpt extends Func
	{
		static final FuncAtkCriticalChanceRcpt _instance = new FuncAtkCriticalChanceRcpt();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncAtkCriticalChanceRcpt()
		{
			super(Stats.CRIT_CHANCE_RECEPTIVE, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			L2ItemInstance i = ((L2Player) env.character).getInventory().getPaperdollItem(Inventory.PAPERDOLL_CHEST);
			if(i != null && i.getItemType() == ArmorType.LIGHT)
				env.value -= 20;
		}
	}

	private static class FuncMoveSpeed extends Func
	{
		static final FuncMoveSpeed _instance = new FuncMoveSpeed();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncMoveSpeed()
		{
			super(Stats.RUN_SPEED, 0x20, null);
		}

		@Override
		public void calc(Env env)
		{
			env.value *= DEXbonus[env.character.getDEX()];
		}
	}

	private static class FuncPAtkSpeed extends Func
	{
		static final FuncPAtkSpeed _instance = new FuncPAtkSpeed();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncPAtkSpeed()
		{
			super(Stats.POWER_ATTACK_SPEED, 0x20, null);
		}

		@Override
		public void calc(Env env)
		{
			env.value *= DEXbonus[env.character.getDEX()];
		}
	}

	private static class FuncMAtkSpeed extends Func
	{
		static final FuncMAtkSpeed _instance = new FuncMAtkSpeed();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncMAtkSpeed()
		{
			super(Stats.MAGIC_ATTACK_SPEED, 0x20, null);
		}

		@Override
		public void calc(Env env)
		{
			env.value *= WITbonus[env.character.getWIT()];
		}
	}

	private static class FuncHennaSTR extends Func
	{
		static final FuncHennaSTR _instance = new FuncHennaSTR();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncHennaSTR()
		{
			super(Stats.STAT_STR, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			L2Player pc = (L2Player) env.character;
			if(pc != null)
				env.value = Math.max(1, env.value + pc.getHennaStatSTR());
		}
	}

	private static class FuncHennaDEX extends Func
	{
		static final FuncHennaDEX _instance = new FuncHennaDEX();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncHennaDEX()
		{
			super(Stats.STAT_DEX, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			L2Player pc = (L2Player) env.character;
			if(pc != null)
				env.value = Math.max(1, env.value + pc.getHennaStatDEX());
		}
	}

	private static class FuncHennaINT extends Func
	{
		static final FuncHennaINT _instance = new FuncHennaINT();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncHennaINT()
		{
			super(Stats.STAT_INT, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			L2Player pc = (L2Player) env.character;
			if(pc != null)
				env.value = Math.max(1, env.value + pc.getHennaStatINT());
		}
	}

	private static class FuncHennaMEN extends Func
	{
		static final FuncHennaMEN _instance = new FuncHennaMEN();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncHennaMEN()
		{
			super(Stats.STAT_MEN, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			L2Player pc = (L2Player) env.character;
			if(pc != null)
				env.value = Math.max(1, env.value + pc.getHennaStatMEN());
		}
	}

	private static class FuncHennaCON extends Func
	{
		static final FuncHennaCON _instance = new FuncHennaCON();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncHennaCON()
		{
			super(Stats.STAT_CON, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			L2Player pc = (L2Player) env.character;
			if(pc != null)
				env.value = Math.max(1, env.value + pc.getHennaStatCON());
		}
	}

	private static class FuncHennaWIT extends Func
	{
		static final FuncHennaWIT _instance = new FuncHennaWIT();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncHennaWIT()
		{
			super(Stats.STAT_WIT, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			L2Player pc = (L2Player) env.character;
			if(pc != null)
				env.value = Math.max(1, env.value + pc.getHennaStatWIT());
		}
	}

	private static class FuncMaxHpAdd extends Func
	{
		static final FuncMaxHpAdd _instance = new FuncMaxHpAdd();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncMaxHpAdd()
		{
			super(Stats.MAX_HP, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			L2PlayerTemplate t = (L2PlayerTemplate) env.character.getTemplate();
			int lvl = env.character.getLevel() - t.classBaseLevel;
			double hpmod = t.lvlHpMod * lvl;
			double hpmax = (t.lvlHpAdd + hpmod) * lvl;
			double hpmin = t.lvlHpAdd * lvl + hpmod;
			env.value += (hpmax + hpmin) / 2;
		}
	}

	private static class FuncMaxHpMul extends Func
	{
		static final FuncMaxHpMul _instance = new FuncMaxHpMul();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncMaxHpMul()
		{
			super(Stats.MAX_HP, 0x20, null);
		}

		@Override
		public void calc(Env env)
		{
			env.value *= CONbonus[env.character.getCON()];
		}
	}

	private static class FuncMaxCpAdd extends Func
	{
		static final FuncMaxCpAdd _instance = new FuncMaxCpAdd();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncMaxCpAdd()
		{
			super(Stats.MAX_CP, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			L2PlayerTemplate t = (L2PlayerTemplate) env.character.getTemplate();
			int lvl = env.character.getLevel() - t.classBaseLevel;
			double cpmod = t.lvlCpMod * lvl;
			double cpmax = (t.lvlCpAdd + cpmod) * lvl;
			double cpmin = t.lvlCpAdd * lvl + cpmod;
			env.value += (cpmax + cpmin) / 2;
		}
	}

	private static class FuncMaxCpMul extends Func
	{
		static final FuncMaxCpMul _instance = new FuncMaxCpMul();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncMaxCpMul()
		{
			super(Stats.MAX_CP, 0x20, null);
		}

		@Override
		public void calc(Env env)
		{
			double cpSSmod = 1;
			int sealOwnedBy = SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_STRIFE);
			int playerCabal = SevenSigns.getInstance().getPlayerCabal((L2Player) env.character);

			if(sealOwnedBy != SevenSigns.CABAL_NULL)
				if(playerCabal == sealOwnedBy)
					cpSSmod = 1.1;
				else
					cpSSmod = 0.9;

			env.value *= CONbonus[env.character.getCON()] * cpSSmod;
		}
	}

	private static class FuncMaxMpAdd extends Func
	{
		static final FuncMaxMpAdd _instance = new FuncMaxMpAdd();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncMaxMpAdd()
		{
			super(Stats.MAX_MP, 0x10, null);
		}

		@Override
		public void calc(Env env)
		{
			L2PlayerTemplate t = (L2PlayerTemplate) env.character.getTemplate();
			int lvl = env.character.getLevel() - t.classBaseLevel;
			double mpmod = t.lvlMpMod * lvl;
			double mpmax = (t.lvlMpAdd + mpmod) * lvl;
			double mpmin = t.lvlMpAdd * lvl + mpmod;
			env.value += (mpmax + mpmin) / 2;
		}
	}

	private static class FuncMaxMpMul extends Func
	{
		static final FuncMaxMpMul _instance = new FuncMaxMpMul();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncMaxMpMul()
		{
			super(Stats.MAX_MP, 0x20, null);
		}

		@Override
		public void calc(Env env)
		{
			env.value *= MENbonus[env.character.getMEN()];
		}
	}

	private static class FuncFatalBlow extends Func
	{
		private static final FuncFatalBlow _instance = new FuncFatalBlow();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncFatalBlow()
		{
			super(Stats.FATALBLOW_RATE, 0x30, null);
		}

		@Override
		public void calc(Env env)
		{
			if(!env.target.isInCombat())
				env.value *= 1.41;

			int head = env.character.getHeadingTo(env.target, true);

			if(head <= 10430 || head >= 55105)
				env.value *= 1.41;
			else if(head <= 21000 || head >= 44500)
				env.value *= 1.19;

			if(env.target.isPlayer)
			{
				L2Player pcTarget = (L2Player) env.target;
				if(pcTarget.isWearingArmor(ArmorType.HEAVY))
					env.value *= 0.6;
				else if(pcTarget.isWearingArmor(ArmorType.MAGIC))
					env.value *= 1.19;
			}
		}
	}

	private static class FuncPhysicalDamageResists extends Func
	{
		private static final FuncPhysicalDamageResists _instance = new FuncPhysicalDamageResists();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncPhysicalDamageResists()
		{
			super(Stats.PHYSICAL_DAMAGE, 0x30, null);
		}

		@Override
		public void calc(Env env)
		{
			if(env.target instanceof L2RaidBossInstance && Math.abs(env.character.getLevel() - env.target.getLevel()) > Config.RAID_MAX_LEVEL_DIFF)
			{
				env.value = 1;
				return;
			}

			L2Weapon weapon = env.character.getActiveWeaponItem();
			if(weapon != null)
				switch(weapon.getItemType())
				{
					case BOW:
					case CROSSBOW:
						env.value *= 0.01 * env.target.calcStat(Stats.BOW_WPN_RECEPTIVE, 100, null, null);
						break;
					case BLUNT:
						env.value *= 0.01 * env.target.calcStat(Stats.BLUNT_WPN_RECEPTIVE, 100, null, null);
						break;
					case RAPIER:
					case DAGGER:
						env.value *= 0.01 * env.target.calcStat(Stats.DAGGER_WPN_RECEPTIVE, 100, null, null);
						break;
					case DUAL:
						env.value *= 0.01 * env.target.calcStat(Stats.DUAL_WPN_RECEPTIVE, 100, null, null);
						break;
					case BIGSWORD:
					case SWORD:
					case ANCIENTSWORD:
						env.value *= 0.01 * env.target.calcStat(Stats.SWORD_WPN_RECEPTIVE, 100, null, null);
						break;
					case POLE:
						env.value *= 0.01 * env.target.calcStat(Stats.POLE_WPN_RECEPTIVE, 100, null, null);
						break;
					case DUALFIST:
						env.value *= 0.01 * env.target.calcStat(Stats.FIST_WPN_RECEPTIVE, 100, null, null);
						break;
				}

			env.value = calcDamageResists(env.skill, env.character, env.target, env.value);
		}
	}

	private static class FuncMagicDamageResists extends Func
	{
		private static final FuncMagicDamageResists _instance = new FuncMagicDamageResists();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncMagicDamageResists()
		{
			super(Stats.MAGIC_DAMAGE, 0x30, null);
		}

		@Override
		public void calc(Env env)
		{
			if(env.target instanceof L2RaidBossInstance && Math.abs(env.character.getLevel() - env.target.getLevel()) > Config.RAID_MAX_LEVEL_DIFF)
			{
				env.value = 1;
				return;
			}

			env.value = calcDamageResists(env.skill, env.character, env.target, env.value);
		}
	}

	private static class FuncInventory extends Func
	{
		private static final FuncInventory _instance = new FuncInventory();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncInventory()
		{
			super(Stats.INVENTORY_LIMIT, 0x01, null);
		}

		@Override
		public void calc(Env env)
		{
			L2Player _cha = (L2Player) env.character;
			if(_cha.isGM())
				env.value = Config.INVENTORY_MAXIMUM_GM;
			else if(_cha.getTemplate().race == Race.dwarf)
				env.value = Config.INVENTORY_MAXIMUM_DWARF;
			else
				env.value = Config.INVENTORY_MAXIMUM_NO_DWARF;
		}
	}

	private static class FuncWarehouse extends Func
	{
		private static final FuncWarehouse _instance = new FuncWarehouse();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncWarehouse()
		{
			super(Stats.STORAGE_LIMIT, 0x01, null);
		}

		@Override
		public void calc(Env env)
		{
			L2Player _cha = (L2Player) env.character;
			if(_cha.getTemplate().race == Race.dwarf)
				env.value = Config.WAREHOUSE_SLOTS_DWARF;
			else
				env.value = Config.WAREHOUSE_SLOTS_NO_DWARF;
		}
	}

	private static class FuncTradeLimit extends Func
	{
		private static final FuncTradeLimit _instance = new FuncTradeLimit();

		static Func getInstance()
		{
			return _instance;
		}

		private FuncTradeLimit()
		{
			super(Stats.TRADE_LIMIT, 0x01, null);
		}

		@Override
		public void calc(Env env)
		{
			L2Player _cha = (L2Player) env.character;
			if(_cha.getRace() == Race.dwarf)
				env.value = Config.MAX_PVTSTORE_SLOTS_DWARF;
			else
				env.value = Config.MAX_PVTSTORE_SLOTS_OTHER;
		}
	}

	/**
	 * Return the period between 2 regenerations task (3s for L2Character, 5 min for L2DoorInstance).<BR><BR>
	 */
	public static int getRegeneratePeriod(L2Character cha)
	{
		if(cha instanceof L2DoorInstance)
			return HP_REGENERATE_PERIOD * 100; // 5 mins

		return HP_REGENERATE_PERIOD; // 3s
	}

	/**
	 * Add basics Func objects to L2Player and L2Summon.<BR><BR>
	 *
	 * <B><U> Concept</U> :</B><BR><BR>
	 * A calculator is created to manage and dynamically calculate the effect of a character property (ex : MAX_HP, REGENERATE_HP_RATE...).
	 * In fact, each calculator is a table of Func object in which each Func represents a mathematic function : <BR><BR>
	 *
	 * FuncAtkAccuracy -> Math.sqrt(_player.getDEX())*6+_player.getLevel()<BR><BR>
	 *
	 * @param cha L2Player or L2Summon that must obtain basic Func objects
	 */
	public static void addFuncsToNewCharacter(L2Character cha)
	{
		if(cha.isPlayer)
		{
			cha.addStatFunc(FuncMultRegenResting.getInstance(Stats.REGENERATE_CP_RATE));
			cha.addStatFunc(FuncMultRegenStanding.getInstance(Stats.REGENERATE_CP_RATE));
			cha.addStatFunc(FuncMultRegenRunning.getInstance(Stats.REGENERATE_CP_RATE));
			cha.addStatFunc(FuncMultRegenResting.getInstance(Stats.REGENERATE_HP_RATE));
			cha.addStatFunc(FuncMultRegenStanding.getInstance(Stats.REGENERATE_HP_RATE));
			cha.addStatFunc(FuncMultRegenRunning.getInstance(Stats.REGENERATE_HP_RATE));
			cha.addStatFunc(FuncMultRegenResting.getInstance(Stats.REGENERATE_MP_RATE));
			cha.addStatFunc(FuncMultRegenStanding.getInstance(Stats.REGENERATE_MP_RATE));
			cha.addStatFunc(FuncMultRegenRunning.getInstance(Stats.REGENERATE_MP_RATE));

			cha.addStatFunc(FuncMaxCpAdd.getInstance());
			cha.addStatFunc(FuncMaxHpAdd.getInstance());
			cha.addStatFunc(FuncMaxMpAdd.getInstance());

			cha.addStatFunc(FuncMaxCpMul.getInstance());
			cha.addStatFunc(FuncMaxHpMul.getInstance());
			cha.addStatFunc(FuncMaxMpMul.getInstance());

			cha.addStatFunc(FuncAttackRange.getInstance());

			cha.addStatFunc(FuncMoveSpeed.getInstance());

			cha.addStatFunc(FuncHennaSTR.getInstance());
			cha.addStatFunc(FuncHennaDEX.getInstance());
			cha.addStatFunc(FuncHennaINT.getInstance());
			cha.addStatFunc(FuncHennaMEN.getInstance());
			cha.addStatFunc(FuncHennaCON.getInstance());
			cha.addStatFunc(FuncHennaWIT.getInstance());

			cha.addStatFunc(FuncInventory.getInstance());
			cha.addStatFunc(FuncWarehouse.getInstance());
			cha.addStatFunc(FuncTradeLimit.getInstance());

			cha.addStatFunc(FuncFatalBlow.getInstance());

			cha.addStatFunc(FuncPAtkMod.getInstance());
			cha.addStatFunc(FuncMAtkMod.getInstance());
			cha.addStatFunc(FuncPDefMod.getInstance());
			cha.addStatFunc(FuncMDefMod.getInstance());
			cha.addStatFunc(FuncAtkCriticalDmgRcpt.getInstance());
			cha.addStatFunc(FuncAtkCriticalChanceRcpt.getInstance());

			cha.addStatFunc(FuncPAtkSpeed.getInstance());
			cha.addStatFunc(FuncMAtkSpeed.getInstance());
		}

		cha.addStatFunc(FuncAtkCritical.getInstance());
		cha.addStatFunc(FuncAtkCriticalPercents.getInstance());
		cha.addStatFunc(FuncMAtkCriticalPercents.getInstance());
		cha.addStatFunc(FuncAtkFinalCritical.getInstance());
		cha.addStatFunc(FuncAtkAccuracy.getInstance());
		cha.addStatFunc(FuncAtkEvasion.getInstance());

		cha.addStatFunc(FuncPhysicalDamageResists.getInstance());
		cha.addStatFunc(FuncMagicDamageResists.getInstance());
	}

	/**
	 * Calculate the HP regen rate (base + modifiers).<BR><BR>
	 */
	public static double calcHpRegen(L2Character cha)
	{
		double init;
		if(cha.isPlayer)
			init = (cha.getLevel() <= 10 ? 1.95 + cha.getLevel() / 20. : 1.4 + cha.getLevel() / 10.) * cha.getLevelMod() * CONbonus[cha.getCON()];
		else
			init = cha.getTemplate().baseHpReg * CONbonus[cha.getCON()];

		if(cha instanceof L2Playable)
		{
			L2Player player = cha.getPlayer();
			if(player != null && player.getClan() != null && player.isInClanHall())
			{
				int clanHallIndex = player.getClan().getHasHideout();
				if(clanHallIndex > 0)
				{
					ClanHall clansHall = ClanHallManager.getInstance().getClanHall(clanHallIndex);
					if(clansHall != null)
						if(clansHall.getFunction(ClanHall.FUNC_RESTORE_HP) != null)
							init *= 1. + clansHall.getFunction(ClanHall.FUNC_RESTORE_HP).getLvl() / 100.;
				}
			}
		}

		return cha.calcStat(Stats.REGENERATE_HP_RATE, init, null, null);
	}

	/**
	 * Calculate the MP regen rate (base + modifiers).<BR><BR>
	 */
	public static double calcMpRegen(L2Character cha)
	{
		double init;
		if(cha.isPlayer)
			init = (.87 + cha.getLevel() * .03) * cha.getLevelMod() * MENbonus[cha.getMEN()];
		else
			init = cha.getTemplate().baseMpReg * MENbonus[cha.getMEN()];

		if(cha instanceof L2Playable)
		{
			L2Player player = cha.getPlayer();
			if(player != null && player.getClan() != null && player.isInClanHall())
			{
				int clanHallIndex = player.getClan().getHasHideout();
				if(clanHallIndex > 0)
				{
					ClanHall clansHall = ClanHallManager.getInstance().getClanHall(clanHallIndex);
					if(clansHall != null)
						if(clansHall.getFunction(ClanHall.FUNC_RESTORE_MP) != null)
							init *= 1. + clansHall.getFunction(ClanHall.FUNC_RESTORE_MP).getLvl() / 100.;
				}
			}
		}

		return cha.calcStat(Stats.REGENERATE_MP_RATE, init, null, null);
	}

	/**
	 * Calculate the CP regen rate (base + modifiers).<BR><BR>
	 */
	public static double calcCpRegen(L2Character cha)
	{
		//double init = ((0.08 * cha.getLevel() + cha.getTemplate().baseCpReg) * (0.6 + CONbonus[cha.getCON()])) * (Config.CP_REGEN_MULTIPLIER / 100);
		double init = (1.5 + cha.getLevel() / 10) * cha.getLevelMod() * CONbonus[cha.getCON()];
		return cha.calcStat(Stats.REGENERATE_CP_RATE, init, null, null);
	}

	/** Calculated damage caused by ATTACK of attacker on target,
	 * called separatly for each weapon, if dual-weapon is used.
	 *
	 * @param attacker player or NPC that makes ATTACK
	 * @param target player or NPC, target of ATTACK
	 * @param miss one of ATTACK_XXX constants
	 * @param crit if the ATTACK have critical success
	 * @param dual if dual weapon is used
	 * @param ss if weapon item was changes by soulshot
	 * @return damage points
	 */
	public static double calcPhysDam(L2Character attacker, L2Character target, L2Skill skill, boolean shld, boolean crit, boolean dual, boolean ss)
	{
		double damage = attacker.getPAtk(target);
		double defence = target.getPDef(attacker);

		if(shld)
			defence += target.getShldDef();

		if(defence == 0)
			defence = 1;

		if(skill != null)
			damage += skill.getPower();
		else if(dual)
			damage /= 1.5;

		if(ss)
			damage *= 2.;

		if(skill != null && skill.isCritical() && calcCrit(attacker, target, attacker.getTemplate().baseCritRate * STRbonus[attacker.getSTR()] * 0.01 * target.calcStat(Stats.CRIT_CHANCE_RECEPTIVE, 100, null, null)))
		{
			// Для скиллов сообщение посылается здесь, для обычной атаки в другом месте.
			attacker.sendPacket(Msg.CRITICAL_HIT);
			// Для скилловых критов, повреждения просто удваиваются, бафы не влияют.
			damage += (6.2 * attacker.calcStat(Stats.CRITICAL_DAMAGE_STATIC, 0, target, null) + damage) * 0.01 * target.calcStat(Stats.CRIT_DAMAGE_RECEPTIVE, 100, null, null);
		}
		else if(crit)
		{
			double rcpt = 0.01 * target.calcStat(Stats.CRIT_DAMAGE_RECEPTIVE, 100, null, null);
			damage += damage * rcpt;
			damage = attacker.calcStat(Stats.CRITICAL_DAMAGE, damage, target, skill);
			damage += 6.2 * attacker.calcStat(Stats.CRITICAL_DAMAGE_STATIC, 0, target, null) * rcpt;
		}

		damage *= 70. / defence;
		damage *= 1 + (Rnd.get() * attacker.getRandomDamage() * 2 - attacker.getRandomDamage()) / 100;
		damage = attacker.calcStat(Stats.PHYSICAL_DAMAGE, damage, target, skill);

		// In C5 summons make 10 % less dmg in PvP.
		if(attacker.isSummon && target.isPlayer)
			damage *= 0.9;
		if(shld && Rnd.chance(5))
			damage = 1;
		// Тут проверяем только если skill != null, т.к. L2Character.onHitTimer не обсчитывает дамаг.
		if(skill != null)
		{
			if(shld)
				if(damage == 1)
					target.sendPacket(Msg.YOUR_EXCELLENT_SHIELD_DEFENSE_WAS_A_SUCCESS);
				else
					target.sendPacket(Msg.SHIELD_DEFENSE_HAS_SUCCEEDED);
			byte mLevel = skill.getMagicLevel() == 0 ? attacker.getLevel() : skill.getMagicLevel();
			int levelDiff = Math.max(1, target.getLevel() - mLevel);

			if(levelDiff > 6)
			{
				double miss1chance = 10 * levelDiff;
				double miss2chance = levelDiff;

				if(miss1chance > Rnd.get(1000))
					damage /= 2;

				if(miss2chance > Rnd.get(1000))
					damage = 1;
			}

			if(target.isStunned() && calcStunBreak(crit))
				target.stopEffects(L2Effect.EffectType.STUN);

			if(calcCastBreak(target, crit))
				target.breakCast(false);
		}

		return damage < 1 ? 1. : damage;
	}

	public static boolean calcStunBreak(boolean crit)
	{
		return Rnd.chance(crit ? 75 : 10);
	}

	public static double calcBlowDamage(L2Character attacker, L2Character target, L2Skill skill, boolean shld, boolean ss)
	{
		//dmg = (([P.Atk] * [SS] + [Skill Power]) * [CD Mult] + [Skill Mult] * [CD Static] ) * 70 / [P.Def]
		//6.2 is a skill multiplier for CRITICAL_DAMAGE_STATIC
		double damage = attacker.getPAtk(target);
		double defence = target.getPDef(attacker);

		if(shld)
			defence += target.getShldDef();
		if(defence == 0)
			defence = 1;

		damage = attacker.calcStat(Stats.CRITICAL_DAMAGE, damage + skill.getPower(), target, skill);

		if(ss)
			damage *= 2.;

		damage += 6.2 * attacker.calcStat(Stats.CRITICAL_DAMAGE_STATIC, 0, target, null);
		damage *= 70. / defence;
		damage *= 1 + (Rnd.get() * attacker.getRandomDamage() * 2 - attacker.getRandomDamage()) / 100;
		damage = attacker.calcStat(Stats.PHYSICAL_DAMAGE, damage, target, skill);

		// Exellent shield defence, 5% на прохождение
		if(shld)
			if(Rnd.chance(5))
			{
				damage = 1;
				target.sendPacket(Msg.YOUR_EXCELLENT_SHIELD_DEFENSE_WAS_A_SUCCESS);
			}
			else
				target.sendPacket(Msg.SHIELD_DEFENSE_HAS_SUCCEEDED);

		if(target.isStunned() && calcStunBreak(true))
			target.stopEffects(L2Effect.EffectType.STUN);

		if(calcCastBreak(target, true))
			target.breakCast(false);

		return damage < 1 ? 1. : damage;
	}

	public static double calcMagicDam(L2Character attacker, L2Character target, L2Skill skill, boolean shield, int sps)
	{
		double mAtk = attacker.getMAtk(target, skill);

		if(sps == 2)
			mAtk *= 4;
		else if(sps == 1)
			mAtk *= 2;

		double mdef = target.getMDef(null, skill);

		if(shield)
			mdef += target.getShldDef();
		if(mdef == 0)
			mdef = 1;

		double damage = 91 * skill.getPower() * Math.sqrt(mAtk) / mdef;

		boolean crit = calcMCrit(attacker.getCriticalMagic(target, skill));

		if(crit)
		{
			damage *= 4.;
			attacker.sendPacket(new SystemMessage(SystemMessage.MAGIC_CRITICAL_HIT));
		}

		damage = attacker.calcStat(Stats.MAGIC_DAMAGE, damage, target, skill);

		if(shield)
		{
			if(Rnd.chance(5))
			{
				damage = 1;
				target.sendPacket(Msg.YOUR_EXCELLENT_SHIELD_DEFENSE_WAS_A_SUCCESS);
			}
			else
				target.sendPacket(Msg.SHIELD_DEFENSE_HAS_SUCCEEDED);
			attacker.sendMessage("Spell deflected");
		}

		byte mLevel = skill.getMagicLevel() == 0 ? attacker.getLevel() : skill.getMagicLevel();
		double levelDiff = Math.max(1, target.getLevel() - mLevel);

		if(levelDiff > 6)
		{
			boolean failed = false;

			if(Rnd.chance(levelDiff))
			{
				damage /= 2;
				failed = true;
			}

			if(Rnd.chance(.1 * levelDiff))
			{
				damage = 1;
				failed = true;
			}

			if(failed)
				attacker.sendPacket(Msg.ATTACK_FAILED);
		}

		if(calcCastBreak(target, crit))
			target.breakCast(false);

		return damage;
	}

	/** Returns true in case of fatal blow success */
	public static boolean calcBlow(L2Character activeChar, L2Character target)
	{
		return Rnd.chance((int) activeChar.calcStat(Stats.FATALBLOW_RATE, activeChar.getDEX(), target, null));
	}

	/** Returns true in case of critical hit */
	public static boolean calcCrit(L2Character attacker, L2Character target, double rate)
	{
		if(attacker.isToSideOfTarget(target))
			rate *= 1.5;
		return Rnd.get() * 1000 <= rate;
	}

	/** Returns true in case of magic critical hit */
	public static boolean calcMCrit(double mRate)
	{
		// floating point random gives more accuracy calculation, because argument also floating point
		return Rnd.get() * 1000 <= mRate;
	}

	/** Returns true in case when ATTACK is canceled due to hit */
	public static boolean calcCastBreak(L2Character target, boolean crit)
	{
		return !(!target.isCastingNow() || target instanceof L2RaidBossInstance) && Rnd.chance((int) target.calcStat(Stats.CAST_INTERRUPT, crit ? 75 : 10, null, null));
	}

	/** Calculate delay (in milliseconds) before next ATTACK */
	public static int calcPAtkSpd(double rate)
	{
		return (int) (500000 / rate); // в миллисекундах поэтому 500*1000
	}

	/** Calculate delay (in milliseconds) for skills cast */
	public static int calcMAtkSpd(L2Character attacker, @SuppressWarnings("unused")
	L2Character target, L2Skill skill, double skillTime)
	{
		if(skill.isMagic())
			return (int) (skillTime * 333 / Math.max(attacker.getMAtkSpd(), 1));
		return (int) (skillTime * 333 / Math.max(attacker.getPAtkSpd(), 1));
	}

	/** Calculate reuse delay (in milliseconds) for skills */
	public static long calcSkillReuseDelay(L2Character actor, L2Skill skill)
	{
		long reuseDelay = skill.getReuseDelay();
		if(!(actor.isPlayer || actor instanceof L2Summon) && reuseDelay == 0)
			reuseDelay = skill.getHitTime() + 1000;
		if(skill.isSkillTimePermanent())
			return reuseDelay;
		if(actor.getSkillMastery(skill.getId()) == 1)
		{
			actor.removeSkillMastery(skill.getId());
			return 0;
		}
		if(skill.isMagic())
			return (long) (reuseDelay * actor.getMReuseRate(skill) * 333 / Math.max(actor.getMAtkSpd(), 1));
		return (long) actor.calcStat(Stats.PHYSIC_REUSE_RATE, reuseDelay * 333 / Math.max(actor.getPAtkSpd(), 1), null, skill);
	}

	/** Returns true if hit missed (target evaded) */
	public static boolean calcHitMiss(L2Character attacker, L2Character target)
	{
		int chanceToHit = 88 + 2 * (attacker.getAccuracy() - target.getEvasionRate(attacker));

		chanceToHit = Math.max(chanceToHit, 28);
		chanceToHit = Math.min(chanceToHit, 98);

		if(attacker.isBehindTarget(target))
			chanceToHit *= 1.2;
		if(attacker.isToSideOfTarget(target))
			chanceToHit *= 1.1;

		return !Rnd.chance(chanceToHit);
	}

	/** Returns true if shield defence successfull */
	public static boolean calcShldUse(L2Character attacker, L2Character target)
	{
		double shldRate = 0.;

		if(target.calcStat(Stats.SHIELD_RATE, target.getTemplate().baseShldRate, attacker, null) == 0)
			return false;

		int angle = (int) target.calcStat(Stats.SHIELD_ANGLE, 60, null, null);

		if(!target.isInFront(attacker, angle))
			return false;

		if(attacker.getActiveWeaponItem() != null)
			if(attacker.getActiveWeaponItem().getItemType() == WeaponType.BOW || attacker.getActiveWeaponItem().getItemType() == WeaponType.CROSSBOW)
				shldRate = 30.;
			else if(attacker.getActiveWeaponItem().getItemType() == WeaponType.DAGGER)
				shldRate = 12.;

		if(target.isPlayer)
		{
			L2ItemInstance shld = ((L2Player) target).getInventory().getPaperdollItem(Inventory.PAPERDOLL_LHAND);
			if(shld != null && shld.getItemType() == WeaponType.NONE)
				shldRate = DEXbonus[target.getDEX()] * target.calcStat(Stats.SHIELD_RATE, shldRate, attacker, null);
			else
				return false;
		}
		else if(target instanceof L2NpcInstance)
			shldRate = target.calcStat(Stats.SHIELD_RATE, shldRate, attacker, null);

		return Rnd.chance((int) shldRate);
	}

	public static double calcSavevsDependence(int save, L2Character cha)
	{
		try
		{
			switch(save)
			{
				case L2Skill.SAVEVS_INT:
					return INTbonus[cha.getINT()];
				case L2Skill.SAVEVS_WIT:
					return WITbonus[cha.getWIT()];
				case L2Skill.SAVEVS_MEN:
					return MENbonus[cha.getMEN()];
				case L2Skill.SAVEVS_CON:
					return CONbonus[cha.getCON()];
				case L2Skill.SAVEVS_DEX:
					return DEXbonus[cha.getDEX()];
				case L2Skill.SAVEVS_STR:
					return STRbonus[cha.getSTR()];
			}
		}
		catch(ArrayIndexOutOfBoundsException e)
		{
			_log.warning("Failed calc savevs on char " + cha + " with save-stat " + save);
			e.printStackTrace();
		}
		return 1.;
	}

	public static boolean calcSkillSuccess(Env env, ResistType effect)
	{
		if(env.value == -1)
			return true;

		if(env.target instanceof L2DoorInstance)
			return false;

		env.value /= calcSavevsDependence(env.skill.getSavevs(), env.target);

		switch(effect)
		{
			case BLEED:
				env.value *= 0.01 * env.character.calcStat(Stats.BLEED_POWER, 100, null, null);
				env.value *= 0.01 * env.target.calcStat(Stats.BLEED_RECEPTIVE, 100, null, null);
				break;
			case CANCEL:
				env.value *= 0.01 * env.character.calcStat(Stats.CANCEL_POWER, 100, null, null);
				env.value *= 0.01 * env.target.calcStat(Stats.CANCEL_RECEPTIVE, 100, null, null);
				break;
			case DEBUFF:
				env.value *= 0.01 * env.character.calcStat(Stats.DEBUFF_POWER, 100, null, null);
				env.value *= 0.01 * env.target.calcStat(Stats.DEBUFF_RECEPTIVE, 100, null, null);
				break;
			case MENTAL:
				env.value *= 0.01 * env.character.calcStat(Stats.MENTAL_POWER, 100, null, null);
				env.value *= 0.01 * env.target.calcStat(Stats.MENTAL_RECEPTIVE, 100, null, null);
				break;
			case PARALYZE:
				env.value *= 0.01 * env.character.calcStat(Stats.PARALYZE_POWER, 100, null, null);
				env.value *= 0.01 * env.target.calcStat(Stats.PARALYZE_RECEPTIVE, 100, null, null);
				break;
			case MUTE:
				env.value *= 0.01 * env.character.calcStat(Stats.SILENCE_POWER, 100, null, null);
				env.value *= 0.01 * env.target.calcStat(Stats.SILENCE_RECEPTIVE, 100, null, null);
				break;
			case POISON:
				env.value *= 0.01 * env.character.calcStat(Stats.POISON_POWER, 100, null, null);
				env.value *= 0.01 * env.target.calcStat(Stats.POISON_RECEPTIVE, 100, null, null);
				break;
			case ROOT:
				env.value *= 0.01 * env.character.calcStat(Stats.ROOT_POWER, 100, null, null);
				env.value *= 0.01 * env.target.calcStat(Stats.ROOT_RECEPTIVE, 100, null, null);
				break;
			case SLEEP:
				env.value *= 0.01 * env.character.calcStat(Stats.SLEEP_POWER, 100, null, null);
				env.value *= 0.01 * env.target.calcStat(Stats.SLEEP_RECEPTIVE, 100, null, null);
				break;
			case STUN:
				env.value *= 0.01 * env.character.calcStat(Stats.STUN_POWER, 100, null, null);
				env.value *= 0.01 * env.target.calcStat(Stats.STUN_RECEPTIVE, 100, null, null);
				break;
			case DEATH:
				env.value *= 0.01 * env.target.calcStat(Stats.DEATH_RECEPTIVE, 100, null, null);
				break;
		}

		byte mLevel = env.skill.getMagicLevel() == 0 ? env.character.getLevel() : env.skill.getMagicLevel();
		double levelmod = (double) mLevel / env.target.getLevel();

		if(env.skill.getLevelModifier() > 1)
			if(levelmod > 1)
				levelmod = (levelmod - 1) * env.skill.getLevelModifier() + 1;
			else if(levelmod < 1)
				levelmod = 1 - (1 - levelmod) * env.skill.getLevelModifier();
		env.value *= Math.max(levelmod, 0);

		if(env.skill.isMagic())
		{
			int mdef = env.target.getMDef(null, env.skill);
			if(mdef == 0)
				mdef = 1;

			if(Config.ALT_SKILLS_CHANCES)
				env.value *= 15 * Math.sqrt(env.character.getMAtkSps(env.target, env.skill)) / mdef;
			else
			{
				double matak = env.character.getMAtkSps(env.target, env.skill);
				env.value *= Math.pow(matak, .35) * Math.pow(Math.log1p(matak), 2.) / mdef;
			}
		}

		env.value = env.character.calcStat(Stats.ACTIVATE_RATE, env.value, env.target, env.skill);

		env.value = Math.max(Math.min(env.value, Config.SKILLS_CHANCE_CAP), 1);

		if(Config.SKILLS_SHOW_CHANCE && env.character.isPlayer)
			env.character.sendMessage(new CustomMessage("l2r.gameserver.skills.Formulas.Chance", env.character).addString(env.skill.getName()).addNumber((int) env.value));
		return Rnd.chance((int) env.value);
	}

	public static boolean calcSkillSuccess(L2Character player, L2Character target, L2Skill skill)
	{
		Env env = new Env();
		env.character = player;
		env.target = target;
		env.skill = skill;
		env.value = skill.getActivateRate();
		ResistType rt;
		switch(skill.getSkillType())
		{
			case CANCEL:
			case NEGATE_EFFECTS:
			case NEGATE_STATS:
				rt = ResistType.CANCEL;
				break;
			case FATALBLOW:
			case LETHAL_SHOT:
				rt = ResistType.DEATH;
				break;
			default:
				rt = ResistType.NONE;
		}
		return calcSkillSuccess(env, rt);
	}

	public static boolean calculateUnlockChance(L2Skill skill)
	{
		int level = skill.getLevel();
		int chance = 0;
		switch(level)
		{
			case 1:
				chance = 30;
				break;

			case 2:
				chance = 50;
				break;

			case 3:
				chance = 75;
				break;

			case 4:
			case 5:
			case 6:
			case 7:
			case 8:
			case 9:
			case 10:
			case 11:
			case 12:
			case 13:
			case 14:
				chance = 100;
				break;
		}

		return Rnd.chance((int) (chance * 1.2));
	}

	public static void calcSkillMastery(L2Skill skill, L2Character activeChar)
	{
		//Skill id 330 for fighters, 331 for mages
		//Actually only GM can have 2 skill masteries, so let's make them more lucky ^^
		if(activeChar.getSkillLevel(331) > 0 && activeChar.getINT() >= Rnd.get(1000) || activeChar.getSkillLevel(330) > 0 && activeChar.getSTR() >= Rnd.get(1000))
		{
			//byte mastery level, 0 = no skill mastery, 1 = no reuseTime, 2 = buff duration*2
			byte masteryLevel;
			L2Skill.SkillType type = skill.getSkillType();
			if(type == L2Skill.SkillType.BUFF || type == L2Skill.SkillType.SONG || type == L2Skill.SkillType.DANCE || type == L2Skill.SkillType.HOT || type == L2Skill.SkillType.HEAL_PERCENT || type == L2Skill.SkillType.SEED) //Hope i didn't forget skills to multiply their time
				masteryLevel = 2;
			else
				masteryLevel = 1;
			if(masteryLevel > 0)
				activeChar.setSkillMastery(skill.getId(), masteryLevel);
		}
	}

	public static double calcDamageResists(L2Skill skill, L2Character attacker, L2Character defender, double value)
	{
		int fire_attack = (int) attacker.calcStat(Stats.ATTACK_ELEMENT_FIRE, 0, null, null);
		int water_attack = (int) attacker.calcStat(Stats.ATTACK_ELEMENT_WATER, 0, null, null);
		int wind_attack = (int) attacker.calcStat(Stats.ATTACK_ELEMENT_WIND, 0, null, null);
		int earth_attack = (int) attacker.calcStat(Stats.ATTACK_ELEMENT_EARTH, 0, null, null);
		int sacred_attack = (int) attacker.calcStat(Stats.ATTACK_ELEMENT_SACRED, 0, null, null);
		int unholy_attack = (int) attacker.calcStat(Stats.ATTACK_ELEMENT_UNHOLY, 0, null, null);

		if(skill != null)
			switch(skill.getElement())
			{
				case FIRE:
					return applyDefense(defender, Stats.FIRE_RECEPTIVE, fire_attack, value);
				case WATER:
					return applyDefense(defender, Stats.WATER_RECEPTIVE, water_attack, value);
				case WIND:
					return applyDefense(defender, Stats.WIND_RECEPTIVE, wind_attack, value);
				case EARTH:
					return applyDefense(defender, Stats.EARTH_RECEPTIVE, earth_attack, value);
				case SACRED:
					return applyDefense(defender, Stats.SACRED_RECEPTIVE, sacred_attack, value);
				case UNHOLY:
					return applyDefense(defender, Stats.UNHOLY_RECEPTIVE, unholy_attack, value);
			}

		if(fire_attack == 0 && water_attack == 0 && earth_attack == 0 && wind_attack == 0 && unholy_attack == 0 && sacred_attack == 0)
			return value;

		TreeMap<Integer, Stats> sort_attibutes = new TreeMap<Integer, Stats>();
		sort_attibutes.put(fire_attack, Stats.FIRE_RECEPTIVE);
		sort_attibutes.put(water_attack, Stats.WATER_RECEPTIVE);
		sort_attibutes.put(wind_attack, Stats.WIND_RECEPTIVE);
		sort_attibutes.put(earth_attack, Stats.EARTH_RECEPTIVE);
		sort_attibutes.put(sacred_attack, Stats.SACRED_RECEPTIVE);
		sort_attibutes.put(unholy_attack, Stats.UNHOLY_RECEPTIVE);

		int attack = sort_attibutes.lastEntry().getKey();
		Stats defence_type = sort_attibutes.lastEntry().getValue();

		return applyDefense(defender, defence_type, attack, value);
	}

	public static double applyDefense(L2Character defender, Stats defence_type, int attack, double value)
	{
		int defense = 100 - (int) defender.calcStat(defence_type, 100, null, null);

		if(defense == 0)
			return value;

		defense = Math.min(100, defense);
		defense = Math.max(-100, defense);

		if(defense > 0)
		{
			defense = defense - attack + 10;
			defense = Math.max(0, defense);
			value *= 1. - defense / 400.;
		}
		else
		{
			defense = defense - attack - 10;
			defense = Math.max(-100, defense);
			value *= 1. - defense / 400.;
		}

		return value;
	}

	public static int[] calcAttackElement(L2Character attacker)
	{
		int fire_attack = (int) attacker.calcStat(Stats.ATTACK_ELEMENT_FIRE, 0, null, null);
		int water_attack = (int) attacker.calcStat(Stats.ATTACK_ELEMENT_WATER, 0, null, null);
		int wind_attack = (int) attacker.calcStat(Stats.ATTACK_ELEMENT_WIND, 0, null, null);
		int earth_attack = (int) attacker.calcStat(Stats.ATTACK_ELEMENT_EARTH, 0, null, null);
		int sacred_attack = (int) attacker.calcStat(Stats.ATTACK_ELEMENT_SACRED, 0, null, null);
		int unholy_attack = (int) attacker.calcStat(Stats.ATTACK_ELEMENT_UNHOLY, 0, null, null);

		if(fire_attack == 0 && water_attack == 0 && earth_attack == 0 && wind_attack == 0 && unholy_attack == 0 && sacred_attack == 0)
			return null;

		TreeMap<Integer, Stats> sort_attibutes = new TreeMap<Integer, Stats>();
		sort_attibutes.put(fire_attack, Stats.ATTACK_ELEMENT_FIRE);
		sort_attibutes.put(water_attack, Stats.ATTACK_ELEMENT_WATER);
		sort_attibutes.put(wind_attack, Stats.ATTACK_ELEMENT_WIND);
		sort_attibutes.put(earth_attack, Stats.ATTACK_ELEMENT_EARTH);
		sort_attibutes.put(sacred_attack, Stats.ATTACK_ELEMENT_SACRED);
		sort_attibutes.put(unholy_attack, Stats.ATTACK_ELEMENT_UNHOLY);

		int element = 0;
		switch(sort_attibutes.lastEntry().getValue())
		{
			case ATTACK_ELEMENT_FIRE:
				element = 0;
				break;
			case ATTACK_ELEMENT_WATER:
				element = 1;
				break;
			case ATTACK_ELEMENT_WIND:
				element = 2;
				break;
			case ATTACK_ELEMENT_EARTH:
				element = 3;
				break;
			case ATTACK_ELEMENT_SACRED:
				element = 4;
				break;
			case ATTACK_ELEMENT_UNHOLY:
				element = 5;
				break;
		}

		return new int[] { element, sort_attibutes.lastEntry().getKey() };
	}
}
