package l2r.gameserver.skills;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Logger;

import l2r.gameserver.Item;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.templates.L2Armor;
import l2r.gameserver.templates.L2EtcItem;
import l2r.gameserver.templates.L2Item;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.L2EtcItem.EtcItemType;

public class SkillsEngine
{

	protected static Logger _log = Logger.getLogger(SkillsEngine.class.getName());

	private static final SkillsEngine _instance = new SkillsEngine();

	private List<File> _armorFiles;
	private List<File> _weaponFiles;
	private List<File> _etcitemFiles;
	private List<File> _skillFiles;

	public static SkillsEngine getInstance()
	{
		return _instance;
	}

	private SkillsEngine()
	{
		_armorFiles = new LinkedList<File>();
		_weaponFiles = new LinkedList<File>();
		_skillFiles = new LinkedList<File>();
		_etcitemFiles = new LinkedList<File>();
		//hashFiles("data/stats/etcitem", _etcitemFiles);
		hashFiles("data/stats/armor", _armorFiles);
		hashFiles("data/stats/weapon", _weaponFiles);
		hashFiles("data/stats/skills", _skillFiles);
	}

	private void hashFiles(String dirname, List<File> hash)
	{
		File dir = new File(dirname);
		if(!dir.exists())
		{
			_log.config("Dir " + dir.getAbsolutePath() + " not exists");
			return;
		}
		File[] files = dir.listFiles();
		for(File f : files)
			if(f.getName().endsWith(".xml"))
				hash.add(f);
	}

	public List<L2Skill> loadSkills(File file)
	{
		if(file == null)
		{
			_log.config("SkillsEngine: File not found!");
			return null;
		}
		DocumentSkill doc = new DocumentSkill(file);
		doc.parse();
		return doc.getSkills();
	}

	public void loadAllSkills(L2Skill[] allSkills, ArrayList<L2Skill> pledgeSkills)
	{
		int count = 0;
		for(File file : _skillFiles)
		{
			List<L2Skill> s = loadSkills(file);
			if(s == null)
				continue;
			for(L2Skill skill : s)
			{
				//if(!allSkills.containsKey(skill.getId() * 200 + 1) && skill.getActivateRate() > -1 && skill.getEffectTemplates() == null)
				//	_log.warning("Skill " + skill.getId() + " has ActivateRate without Effect!");
				allSkills[skill.getId() * 300 + skill.getLevel()] = skill;
				if(skill.getMinPledgeClass() > 0)
					pledgeSkills.add(skill);
				count++;
			}
		}
		_log.info("SkillsEngine: Loaded " + count + " Skill templates from XML files.");
	}

	public List<L2Armor> loadArmors(HashMap<Integer, Item> armorData)
	{
		List<L2Armor> list = new LinkedList<L2Armor>();
		for(L2Item item : loadData(armorData, _armorFiles))
			list.add((L2Armor) item);
		return list;
	}

	public List<L2Weapon> loadWeapons(HashMap<Integer, Item> weaponData)
	{
		List<L2Weapon> list = new LinkedList<L2Weapon>();
		for(L2Item item : loadData(weaponData, _weaponFiles))
			list.add((L2Weapon) item);
		return list;
	}

	public List<L2EtcItem> loadItems(HashMap<Integer, Item> itemData)
	{
		List<L2EtcItem> list = new LinkedList<L2EtcItem>();
		for(L2Item item : loadData(itemData, _etcitemFiles))
			list.add((L2EtcItem) item);
		if(list.size() == 0)
			for(Item item : itemData.values())
				list.add(new L2EtcItem((EtcItemType) item.type, item.set));
		return list;
	}

	public List<L2Item> loadData(HashMap<Integer, Item> itemData, List<File> files)
	{
		List<L2Item> list = new LinkedList<L2Item>();
		for(File f : files)
		{
			DocumentItem document = new DocumentItem(itemData, f);
			document.parse();
			list.addAll(document.getItemList());
		}
		return list;
	}
}