package l2r.gameserver.skills.effects;

import java.util.ArrayList;

import l2r.gameserver.ai.CtrlIntention;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Effect;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Summon;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.skills.Env;
import l2r.util.Rnd;

@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public class EffectDiscord extends L2Effect
{
	public EffectDiscord(Env env, EffectTemplate template)
	{
		super(env, template);
	}

	@Override
	public EffectType getEffectType()
	{
		return EffectType.CONFUSE_MOB_ONLY;
	}

	/** Notify started */
	@Override
	public void onStart()
	{
		if(!(_effected instanceof L2NpcInstance))
		{
			getEffector().sendPacket(Msg.TARGET_IS_INCORRECT);
			exit();
			return;
		}

		if(_effected.isFearImmune())
		{
			getEffector().sendPacket(Msg.TARGET_IS_INCORRECT);
			exit();
			return;
		}

		// Discord нельзя наложить на осадных саммонов
		if(_effected.getNpcId() == L2Summon.SIEGE_GOLEM_ID || _effected.getNpcId() == L2Summon.SIEGE_CANNON_ID || _effected.getNpcId() == L2Summon.SWOOP_CANNON_ID)
		{
			getEffector().sendPacket(Msg.TARGET_IS_INCORRECT);
			exit();
			return;
		}

		if(_effected.isInZonePeace())
		{
			getEffector().sendPacket(Msg.YOU_CANNOT_ATTACK_IN_THE_PEACE_ZONE);
			exit();
			return;
		}

		_effected.startConfused();
		onActionTime();
	}

	/** Notify exited */
	@Override
	public void onExit()
	{
		getEffected().stopConfused();
	}

	@Override
	public boolean onActionTime()
	{
		ArrayList<L2Character> targetList = new ArrayList<L2Character>();

		for(L2Character character : getEffected().getKnownCharacters(900))
			if(character instanceof L2NpcInstance && character != getEffected())
				targetList.add(character);
		// if there is no target, exit function
		if(targetList.size() == 0)
			return true;

		// Choosing randomly a new target
		L2Object target = targetList.get(Rnd.get(targetList.size()));

		// Attacking the target
		getEffected().setTarget(target);
		getEffected().getAI().setIntention(CtrlIntention.AI_INTENTION_ATTACK, target, null);

		return true;
	}
}