package l2r.gameserver.skills.effects;

import l2r.gameserver.ai.CtrlIntention;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.model.L2Effect;
import l2r.gameserver.model.L2Summon;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.skills.Env;
import l2r.util.Location;
import l2r.util.Rnd;

final class EffectFear extends L2Effect
{
	public static final int FEAR_RANGE = 500;

	public EffectFear(Env env, EffectTemplate template)
	{
		super(env, template);
	}

	@Override
	public EffectType getEffectType()
	{
		return EffectType.FEAR;
	}

	/** Notify started */
	@Override
	public void onStart()
	{
		if(_effected.isFearImmune())
		{
			getEffector().sendPacket(Msg.TARGET_IS_INCORRECT);
			exit();
			return;
		}

		// Fear нельзя наложить на осадных саммонов
		if(_effected.getNpcId() == L2Summon.SIEGE_GOLEM_ID || _effected.getNpcId() == L2Summon.SIEGE_CANNON_ID || _effected.getNpcId() == L2Summon.SWOOP_CANNON_ID)
		{
			getEffector().sendPacket(Msg.TARGET_IS_INCORRECT);
			exit();
			return;
		}

		if(_effected.isInZonePeace())
		{
			getEffector().sendPacket(Msg.YOU_CANNOT_ATTACK_IN_THE_PEACE_ZONE);
			exit();
			return;
		}

		if((_effected.isSummon || _effected.isPet) && _effected.getOwner() == getEffector())
		{
			exit();
			return;
		}

		_effected.startFear();

		onActionTime();
	}

	/** Notify exited */
	@Override
	public void onExit()
	{
		_effected.stopFear();
		_effected.getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE);
	}

	@Override
	public boolean onActionTime()
	{
		Location pos = Rnd.coordsRandomize(_effected.getLoc(), FEAR_RANGE, FEAR_RANGE);
		Location loc = GeoClient.getInstance().moveCheck(_effected.getX(), _effected.getY(), _effected.getZ(), pos.getX(), pos.getY(), pos.getZ());

		_effected.setRunning();
		_effected.moveToLocation(loc, 0, false);
		_effected.sendMessage("You can feel Fears's effect");

		return true;
	}
}