package l2r.gameserver.skills.effects;

import java.util.logging.Logger;

import l2r.gameserver.model.L2Effect;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2RoundTerritoryWithSkill;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Spawn;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.skills.Env;
import l2r.gameserver.tables.NpcTable;
import l2r.gameserver.templates.L2NpcTemplate;
import l2r.util.Location;

final class EffectSymbol extends L2Effect
{
	private static final Logger log = Logger.getLogger(EffectSymbol.class.getName());

	L2RoundTerritoryWithSkill territory;
	L2NpcInstance symbol;

	public EffectSymbol(Env env, EffectTemplate template)
	{
		super(env, template);
	}

	@Override
	public EffectType getEffectType()
	{
		return EffectType.SYMBOL;
	}

	/** Notify started */
	@Override
	public void onStart()
	{
		if(getSkill().getTargetType() != L2Skill.SkillTargetType.TARGET_SELF)
		{
			log.severe("Symbol skill with target != self, id = " + getSkill().getId());
			exit();
			return;
		}

		L2Skill skill = getSkill().getFirstAddedSkill();
		if(skill == null)
		{
			log.severe("Not implemented symbol skill, id = " + getSkill().getId());
			exit();
			return;
		}

		territory = new L2RoundTerritoryWithSkill("Symbol" + _effected.getObjectId(), _effected.getX(), _effected.getY(), _skill.getSkillRadius(), _effected.getZ() - 600, _effected.getZ() + 600, _effector, skill);
		L2World.addTerritory(territory);
		_effected.block();

		L2NpcTemplate template = NpcTable.getTemplate(_skill.getSymbolId());

		try
		{
			Location loc = _effected.getLoc();
			if(_effected.isPlayer && ((L2Player) _effected).getGroundSkillLoc() != null)
			{
				loc = ((L2Player) _effected).getGroundSkillLoc();
				((L2Player) _effected).setGroundSkillLoc(null);
			}

			L2Spawn spawn = new L2Spawn(template);
			spawn.setLoc(loc);
			spawn.setAmount(1);
			spawn.init();
			spawn.stopRespawn();
			symbol = spawn.getLastSpawn();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	/** Notify exited */
	@Override
	public void onExit()
	{
		L2World.removeTerritory(territory);
		_effected.unblock();
		symbol.deleteMe();
	}

	@Override
	public boolean onActionTime()
	{
		// just stop this effect
		return false;
	}
}