package l2r.gameserver.skills.funcs;

import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.skills.Env;
import l2r.gameserver.skills.Stats;
import l2r.gameserver.templates.L2Item;
import l2r.gameserver.templates.L2Weapon.WeaponType;

public class FuncEnchant extends Func
{
	private final Stats _stat;

	public FuncEnchant(Stats stat, int order, Object owner, @SuppressWarnings("unused")
	Lambda lambda)
	{
		super(stat, order, owner);
		_stat = stat;
	}

	@Override
	public void calc(Env env)
	{
		if(_cond != null && !_cond.test(env))
			return;
		L2ItemInstance item = (L2ItemInstance) _funcOwner;

		if(item.getItem().getCrystalType() == L2Item.CRYSTAL_NONE)
			return;

		int enchant = item.getEnchantLevel();
		int overenchant = Math.max(0, enchant - 3);

		if(_stat == Stats.MAGIC_DEFENCE || _stat == Stats.POWER_DEFENCE)
		{
			env.value += enchant + overenchant * 3;
			return;
		}

		if(_stat == Stats.MAGIC_ATTACK)
		{
			switch(item.getItem().getCrystalType())
			{
				case L2Item.CRYSTAL_S:
					env.value += 4 * (enchant + overenchant);
					break;
				case L2Item.CRYSTAL_A:
					env.value += 3 * (enchant + overenchant);
					break;
				case L2Item.CRYSTAL_B:
					env.value += 3 * (enchant + overenchant);
					break;
				case L2Item.CRYSTAL_C:
					env.value += 3 * (enchant + overenchant);
					break;
				case L2Item.CRYSTAL_D:
					env.value += 2 * (enchant + overenchant);
					break;
			}
			return;
		}

		Enum itemType = item.getItemType();
		switch(item.getItem().getCrystalType())
		{
			case L2Item.CRYSTAL_S:
				if(itemType == WeaponType.BOW)
					env.value += 10 * (enchant + overenchant);
				else if(itemType == WeaponType.DUALFIST || itemType == WeaponType.DUAL || itemType == WeaponType.BIGSWORD || itemType == WeaponType.SWORD || itemType == WeaponType.RAPIER || itemType == WeaponType.ANCIENTSWORD && item.getItem().getBodyPart() == 16384)
					env.value += 6 * (enchant + overenchant);
				else
					env.value += 5 * (enchant + overenchant);
				break;
			case L2Item.CRYSTAL_A:
				if(itemType == WeaponType.BOW)
					env.value += 8 * (enchant + overenchant);
				else if(itemType == WeaponType.DUALFIST || itemType == WeaponType.DUAL || itemType == WeaponType.BIGSWORD || itemType == WeaponType.SWORD || itemType == WeaponType.RAPIER || itemType == WeaponType.ANCIENTSWORD && item.getBodyPart() == 16384)
					env.value += 5 * (enchant + overenchant);
				else
					env.value += 4 * (enchant + overenchant);
				break;
			case L2Item.CRYSTAL_B:
				if(itemType == WeaponType.BOW)
					env.value += 6 * (enchant + overenchant);
				else if(itemType == WeaponType.DUALFIST || itemType == WeaponType.DUAL || itemType == WeaponType.BIGSWORD || itemType == WeaponType.SWORD || itemType == WeaponType.RAPIER || itemType == WeaponType.ANCIENTSWORD && item.getItem().getBodyPart() == 16384)
					env.value += 4 * (enchant + overenchant);
				else
					env.value += 3 * (enchant + overenchant);
				break;
			case L2Item.CRYSTAL_C:
				if(itemType == WeaponType.BOW)
					env.value += 6 * (enchant + overenchant);
				else if(itemType == WeaponType.DUALFIST || itemType == WeaponType.DUAL || itemType == WeaponType.BIGSWORD || itemType == WeaponType.SWORD || itemType == WeaponType.RAPIER || itemType == WeaponType.ANCIENTSWORD && item.getItem().getBodyPart() == 16384)
					env.value += 4 * (enchant + overenchant);
				else
					env.value += 3 * (enchant + overenchant);
				break;
			case L2Item.CRYSTAL_D:
				if(itemType == WeaponType.BOW)
					env.value += 4 * (enchant + overenchant);
				else
					env.value += 2 * (enchant + overenchant);
				break;
		}
		return;
	}
}
