package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.gameserver.ai.CtrlEvent;
import l2r.gameserver.ai.CtrlIntention;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.instances.L2DecoyInstance;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.StatsSet;
import l2r.util.Rnd;

public class Aggression extends L2Skill
{
	private final boolean _unaggring;

	public Aggression(StatsSet set)
	{
		super(set);
		_unaggring = set.getBool("unaggroing", false);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		int effect = _effectPoint;

		if(isSSPossible() && activeChar.unChargeShots(isMagic()))
			effect *= 2;

		L2Weapon weapon = activeChar.getActiveWeaponItem();
		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();

			if(!(target instanceof L2NpcInstance || target instanceof L2Playable))
				continue;

			if(_unaggring)
			{
				if(target instanceof L2NpcInstance)
				{
					if(activeChar instanceof L2Playable)
						((L2Playable) activeChar).addDamageHate((L2NpcInstance) target, 0, -effect);
				}
				else if(target instanceof L2Playable && Rnd.chance(80))
					target.setTarget(null);
			}
			else if(activeChar instanceof L2DecoyInstance)
			{
				if(((L2DecoyInstance) activeChar).getOwner() == target)
					return;
				target.getAI().setIntention(CtrlIntention.AI_INTENTION_ATTACK, activeChar);
			}
			else
				target.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, activeChar, effect);

			if(weapon != null)
				weapon.getEffect(false, activeChar, target, isOffensive());

			getEffects(activeChar, target, getActivateRate() > 0);
		}
	}
}
