package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.serverpackets.MagicSkillUse;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.skills.Formulas;
import l2r.gameserver.templates.StatsSet;

public class Charge extends L2Skill
{
	private int _charges;

	public Charge(StatsSet set)
	{
		super(set);
		_charges = set.getInteger("charges", getLevel());
	}

	@Override
	public boolean checkCondition(final L2Character activeChar, final L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(!activeChar.isPlayer)
			return false;

		L2Player player = (L2Player) activeChar;

		//Камушки можно юзать даже если заряд > 7, остальное только если заряд < уровень скила
		if(getId() != 2165 && player.getIncreasedForce() >= _charges)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOUR_FORCE_HAS_REACHED_MAXIMUM_CAPACITY_));
			return false;
		}
		else if(getId() == 2165)
			player.sendPacket(new MagicSkillUse(player, player, 2165, 1, 0, 0));

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(!activeChar.isPlayer)
			return;

		boolean ss = activeChar.getChargedSoulShot() && isSSPossible();
		if(ss && getTargetType() != SkillTargetType.TARGET_SELF)
			activeChar.unChargeShots(false);

		for(L2Character target : targets)
		{
			if(target == activeChar)
				continue;

			boolean shld = !getShieldIgnore() && Formulas.calcShldUse(activeChar, target);

			double damage;

			// В 345, 346 скиллах урон = ударом двух мечей дуалов(кастетов)
			if(getId() == 345 || getId() == 346)
				damage = 2 * Formulas.calcPhysDam(activeChar, target, this, shld, false, true, ss);
			else
				damage = Formulas.calcPhysDam(activeChar, target, this, shld, false, false, ss);

			if(damage < 1)
				damage = 1;

			target.reduceCurrentHp(damage, activeChar, true, true, false);

			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber((int) damage));
		}

		chargePlayer((L2Player) activeChar, getId());
	}

	public void chargePlayer(L2Player player, Integer skillId)
	{
		switch(skillId)
		{
			case 8:
			case 50:
			case 345:
			case 346:
			case 570:
				player.setIncreasedForce(player.getIncreasedForce() + 1);
				createRunnable(player);
				break;
			case 2165:
				if(player.getIncreasedForce() < _charges)
				{
					player.setIncreasedForce(player.getIncreasedForce() + 1);
					createRunnable(player);
				}
				else
					player.sendPacket(new SystemMessage(SystemMessage.YOUR_FORCE_HAS_REACHED_MAXIMUM_CAPACITY_));
				break;
			default:
		}
	}

	public void createRunnable(L2Player player)
	{
		Runnable r = new ChargeTimer(player);
		ThreadPoolManager.getInstance().scheduleGeneral(r, 600000);
		player._lastChargeRunnable = r;
	}

	public class ChargeTimer implements Runnable
	{
		private L2Player _player;

		public ChargeTimer(L2Player player)
		{
			_player = player;
		}

		public void run()
		{
			if(_player._lastChargeRunnable != this)
				return;

			_player.setIncreasedForce(0);
			_player._lastChargeRunnable = null;
		}
	}
}