package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.gameserver.ai.CtrlEvent;
import l2r.gameserver.cache.Msg;
import l2r.util.Rnd;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Party;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.templates.StatsSet;

import java.util.ArrayList;

public class Confusion extends L2Skill
{
	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(activeChar == null || target == null)
			return false;

		if(activeChar.isInZonePeace())
		{
			activeChar.sendPacket(Msg.YOU_CANNOT_ATTACK_IN_THE_PEACE_ZONE);
			return false;
		}

		if(target.isFearImmune())
		{
			activeChar.sendPacket(Msg.TARGET_IS_INCORRECT);
			return false;
		}

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	public Confusion(StatsSet set)
	{
		super(set);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(!isHandler())
			activeChar.unChargeShots(isMagic());
		
		int radius = getCastRange() > 0 ? getCastRange() : getSkillRadius();
		
		L2Party party = ((L2Player) activeChar).getParty();
		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			ArrayList<L2Character> chars = new ArrayList<L2Character>();
			if(target.isPlayer)
			{
				for(L2Character ch : target.getKnownCharacters(radius))
					if(ch instanceof L2Playable && ch != activeChar)
						chars.add(ch);

				if(chars.size() > 0)
				{
					target.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, chars.get(Rnd.get(chars.size())), _effectPoint);
					getEffects(activeChar, target, getActivateRate() > 0);
				}
			}
			else
			{
				if(party != null)
					for(L2Player player : party.getPartyMembers())
						if(player.isInRange(target, 1250) && player != activeChar)
							chars.add(player);

				if(chars.isEmpty())
				{
					FastList<L2Character> knownlist = target.getKnownCharacters(radius);
					for(FastList.Node<L2Character> sn = knownlist.head(), send = knownlist.tail(); (sn = sn.getNext()) != send;)
					{
						L2Character ch = sn.getValue();
						if(ch instanceof L2Playable && ch != activeChar)
							chars.add(ch);
					}
				}

				if(chars.size() > 0)
				{
					target.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, chars.get(Rnd.get(chars.size())), _effectPoint);
					getEffects(activeChar, target, getActivateRate() > 0);
				}
			}
		}
	}
}
