package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.Config;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.skills.Stats;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.StatsSet;
import l2r.util.Rnd;

public class Continuous extends L2Skill
{
	private int _lethal1;
	private int _lethal2;
	private final String _aborts;

	public Continuous(StatsSet set)
	{
		super(set);
		_lethal1 = set.getInteger("lethal1", 0);
		_lethal2 = set.getInteger("lethal2", 0);
		_aborts = set.getString("abortType", "");
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(_numCharges > 0)
			activeChar.setIncreasedForce(activeChar.getIncreasedForce() - _numCharges);

		L2Weapon weapon = activeChar.getActiveWeaponItem();
		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			// Player holding a cursed weapon can't be buffed and can't buff
			if(getSkillType() == L2Skill.SkillType.BUFF && target != activeChar)
				if(target == null || target.isCursedWeaponEquipped())
					continue;
				else if(activeChar.isCursedWeaponEquipped())
					continue;

			if(isOffensive() && target.checkReflectSkill(this))
				target = activeChar;

			double mult = 0.01 * target.calcStat(Stats.DEATH_RECEPTIVE, 100, null, null);
			_lethal1 *= mult;
			_lethal2 *= mult;

			if(_lethal1 > 0 && Rnd.chance(_lethal1))
			{
				if(target.isPlayer)
				{
					target.reduceCurrentHp(target.getCurrentCp(), activeChar, true, true, false);
					target.sendPacket(Msg.YOU_HAVE_BEEN_STRUCK_BY_THE_INSTANT_KILL_SKILL);
				}
				else if(target instanceof L2NpcInstance && !target.isLethalImmune() && target.getMaxHp() <= 50000)
					target.reduceCurrentHp(target.getCurrentHp() / 2, activeChar, true, true, false);

				activeChar.sendPacket(Msg.INSTANT_KILL);
			}

			if(_lethal2 > 0 && Rnd.chance(_lethal2))
			{
				if(target.isPlayer)
				{
					target.reduceCurrentHp(target.getCurrentHp() + target.getCurrentCp() - 1, activeChar, true, true, false);
					target.sendPacket(Msg.YOU_HAVE_BEEN_STRUCK_BY_THE_INSTANT_KILL_SKILL);
				}
				else if(target instanceof L2NpcInstance && !target.isLethalImmune() && target.getMaxHp() <= 50000)
					target.reduceCurrentHp(target.getCurrentHp(), activeChar, true, true, false);

				activeChar.sendPacket(Msg.INSTANT_KILL);
			}

			if(weapon != null)
				weapon.getEffect(false, activeChar, target, isOffensive());

			if(_aborts.contains("attack"))
				activeChar.abortAttack();

			if(_aborts.contains("cast"))
				activeChar.abortCast();

			getEffects(activeChar, target, getActivateRate() > 0);
		}

		if(isMagic() && _id < 2000 && isSSPossible())
			if(!(Config.SAVING_SPS && _skillType == SkillType.BUFF))
				activeChar.unChargeShots(true);
	}
}