package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.gameserver.ai.CtrlIntention;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.skills.Formulas;
import l2r.gameserver.templates.StatsSet;
import l2r.util.Rnd;

public class DeleteHateOfMe extends L2Skill
{
	private final boolean _cancelTarget;
	private final boolean _cancelSelfTarget;

	public DeleteHateOfMe(StatsSet set)
	{
		super(set);
		_cancelTarget = set.getBool("cancelTarget", false);
		_cancelSelfTarget = set.getBool("cancelSelfTarget", false);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();

			boolean success = _id == BLUFF_SKILL_ID ? false : Rnd.chance(getActivateRate());

			if(_id == BLUFF_SKILL_ID ? Formulas.calcSkillSuccess(activeChar, target, this) : success)
			{
				if(target instanceof L2NpcInstance)
				{
					L2NpcInstance npc = (L2NpcInstance) target;
					activeChar.removeFromHatelist(npc);
					npc.setAttackTimeout(Integer.MAX_VALUE);
					npc.getAI().setAttackTarget(null);
					if(npc.noTarget())
						npc.getAI().setGlobalAggro(-10);
					npc.getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE);
				}
				else if(_cancelTarget)
					target.setTarget(null);

				if(_cancelSelfTarget)
					activeChar.setTarget(null);
			}

			// Для Bluff шанс прохождения эффекта скила расчитывается как для шоковых атак, причем отдельно
			if(success || _id == BLUFF_SKILL_ID)
				getEffects(activeChar, target, _id == BLUFF_SKILL_ID);
		}
	}
}