package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Summon;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.serverpackets.StopMove;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.skills.Formulas;
import l2r.gameserver.skills.Stats;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.StatsSet;
import l2r.util.Location;
import l2r.util.Rnd;

public class FatalBlow extends L2Skill
{
	public final static float INSTANT_CHANCE = 0.10f;
	public final static float INSTANT_CHANCE_PVP = 0.05f;

	private final boolean _onCrit;
	private final boolean _instant;
	private int _lethal1;
	private int _lethal2;
	private final boolean _directHp;
	private final boolean _turner;

	public FatalBlow(StatsSet set)
	{
		super(set);
		_onCrit = set.getBool("onCrit", false);
		_instant = set.getBool("instant", false);
		_lethal1 = set.getInteger("lethal1", 0);
		_lethal2 = set.getInteger("lethal2", 0);
		_directHp = set.getBool("directHp", false);
		_turner = set.getBool("turner", false);
	}

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(_behind && !activeChar.isBehindTarget())
			return false;
		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		boolean ss = activeChar.getChargedSoulShot() && isSSPossible();
		if(ss)
			activeChar.unChargeShots(false);
		L2Weapon weapon = activeChar.getActiveWeaponItem();

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(target.isDead())
				continue;

			if(_turner && !target.isInvul())
			{
				int posX = target.getX();
				int posY = target.getY();
				int posZ = target.getZ();
				int signx = -1;
				int signy = -1;
				if(target.getX() > target.getX())
					signx = 1;
				if(target.getY() > target.getY())
					signy = 1;

				target.broadcastPacket(new StopMove(target));
				target.setHeading(target, false);

				if(!target.isMonster)
					target.setTarget(null);
				target.setRunning();

				Location loc = GeoClient.getInstance().moveCheck(target.getX(), target.getY(), target.getZ(), posX + signx * 40, posY + signy * 40, posZ);
				target.moveToLocation(loc, 0, false);

				target.sendMessage("You can feel Turner's effect");
			}

			double CHANCE = INSTANT_CHANCE;
			double CHANCE_PVP = INSTANT_CHANCE_PVP;

			double mult = 0.01 * target.calcStat(Stats.DEATH_RECEPTIVE, 100, null, null);

			CHANCE *= mult;
			CHANCE_PVP *= mult;
			_lethal1 *= mult;
			_lethal2 *= mult;

			if(_onCrit && !Formulas.calcBlow(activeChar, target))
			{
				activeChar.sendPacket(Msg.MISSED_TARGET);
				continue;
			}

			target.doCounterAttack(this, activeChar);

			if(target.checkReflectSkill(this))
				target = activeChar;

			boolean shld = Formulas.calcShldUse(activeChar, target);

			double damage = Formulas.calcBlowDamage(activeChar, target, this, shld, ss);

			if(_instant && !target.isLethalImmune() && (target instanceof L2Playable ? CHANCE_PVP : CHANCE) > Rnd.get())
			{
				damage = target.getCurrentCp() + target.getCurrentHp();
				if(target.isPlayer)
					target.sendPacket(Msg.YOU_HAVE_BEEN_STRUCK_BY_THE_INSTANT_KILL_SKILL);
				activeChar.sendPacket(Msg.INSTANT_KILL);
			}

			if(_lethal1 > 0 && Rnd.chance(_lethal1))
			{
				if(target.isPlayer)
				{
					damage += target.getCurrentCp();
					target.sendPacket(Msg.YOU_HAVE_BEEN_STRUCK_BY_THE_INSTANT_KILL_SKILL);
				}
				else if(target instanceof L2NpcInstance)
					if(target.isLethalImmune() || target.getMaxHp() > 50000)
						damage *= 2;
					else
						damage += target.getCurrentHp() / 2;

				activeChar.sendPacket(Msg.INSTANT_KILL);
			}

			if(_lethal2 > 0 && Rnd.chance(_lethal2))
			{
				if(target.isPlayer)
				{
					damage = target.getCurrentHp() + target.getCurrentCp() - 1 > damage ? target.getCurrentHp() + target.getCurrentCp() - 1 : damage;
					target.sendPacket(Msg.YOU_HAVE_BEEN_STRUCK_BY_THE_INSTANT_KILL_SKILL);
				}
				else if(target instanceof L2NpcInstance)
					if(target.isLethalImmune() || target.getMaxHp() > 50000)
						damage *= 2;
					else
						damage = target.getCurrentHp() - 1 > damage ? target.getCurrentHp() - 1 : damage;

				activeChar.sendPacket(Msg.INSTANT_KILL);
			}

			if(damage < 1)
				damage = 1;

			target.reduceCurrentHp(damage, activeChar, true, true, _directHp);

			if(activeChar.isPlayer)
			{
				activeChar.sendPacket(Msg.CRITICAL_HIT);
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber((int) damage));
			}
			else if(activeChar instanceof L2Summon)
			{
				L2Summon summon = (L2Summon) activeChar;
				summon.sendCriticalHitMsg();
				summon.sendDamageMsg((int) damage);
			}

			if(weapon != null)
				weapon.getEffect(true, activeChar, target, true);
			if(!_id.equals(BLINDING_BLOW))
				getEffects(activeChar, target, getActivateRate() > 0);
		}
	}
}