package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Summon;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.skills.Formulas;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.StatsSet;
import l2r.util.Rnd;

public class MDam extends L2Skill
{
	public static final double DEATHLINK_MULTIPLIER = 1.8;

	private final boolean _deathlink;
	private final boolean _cancelTarget;
	private final boolean _basedOnTargetDebuff;

	public MDam(StatsSet set)
	{
		super(set);
		_deathlink = set.getBool("deathlink", false);
		_cancelTarget = set.getBool("cancelTarget", false);
		_basedOnTargetDebuff = set.getBool("basedOnTargetDebuff", false);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		double deathlinkMult = DEATHLINK_MULTIPLIER * (1. - activeChar.getCurrentHp() / activeChar.getMaxHp());
		int sps = isSSPossible() ? activeChar.getChargedSpiritShot() : 0;
		L2Weapon weapon = activeChar.getActiveWeaponItem();

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(target.isDead())
				continue;

			if(target.checkReflectSkill(this))
				target = activeChar;

			if(_cancelTarget && !target.isNpc && Rnd.chance(50))
				target.setTarget(null);

			boolean shield = Formulas.calcShldUse(activeChar, target);
			double damage = Formulas.calcMagicDam(activeChar, target, this, shield, sps);

			if(_deathlink)
				damage *= deathlinkMult;

			if(_basedOnTargetDebuff)
				damage *= 1 + 0.05 * target.getAllEffects().size();

			if(damage < 1)
				damage = 1;

			if(activeChar.isPlayer)
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber((int) damage));
			else if(activeChar instanceof L2Summon)
				((L2Summon) activeChar).sendDamageMsg((int) damage);

			target.reduceCurrentHp(damage, activeChar, true, true, false);

			getEffects(activeChar, target, getActivateRate() > 0);
			if(weapon != null)
				weapon.getEffect(false, activeChar, target, isOffensive());
		}
		if(sps != 0)
			activeChar.unChargeShots(true);

		if(isSuicideAttack())
			activeChar.doDie(null);
	}
}