package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.Config;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.skills.Stats;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.StatsSet;

public class ManaHeal extends L2Skill
{
	public ManaHeal(StatsSet set)
	{
		super(set);
	}

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(!isHandler() && getTargetType() != SkillTargetType.TARGET_SELF && (target == null || target.getSkillLevel(_id) > 0))
		{
			activeChar.sendPacket(Msg.TARGET_IS_INCORRECT);
			return false;
		}

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		double mp = _power + 0.1 * _power * Math.sqrt(activeChar.getMAtk(null, null) / 333);

		int sps = isSSPossible() ? activeChar.getChargedSpiritShot() : 0;
		if(sps > 0)
		{
			if(Config.MANAHEAL_SPS_BONUS)
				mp *= sps == 2 ? 1.5 : 1.3;
			activeChar.unChargeShots(true);
		}
		L2Weapon weapon = activeChar.getActiveWeaponItem();

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			double newMp = mp * target.calcStat(Stats.MANAHEAL_EFFECTIVNESS, 100, null, null) / 100;

			// Обработка разницы в левелах при речардже. Учитывыется разница уровня скилла и уровня цели.
			// 1013 = id скилла recharge. Для сервиторов не проверено убавление маны, пока оставлено так как есть.
			int diff = target.getLevel() - getMagicLevel();
			if(getId() == 1013 && diff > 5)
				if(diff < 20)
					newMp = newMp / 100 * (100 - diff * 5);
				else
					newMp = 0;

			if(newMp == 0)
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.S1_HAS_FAILED).addSkillName(_id, getDisplayLevel()));
				if(weapon != null)
					weapon.getEffect(false, activeChar, target, isOffensive());
				getEffects(activeChar, target, getActivateRate() > 0);
				continue;
			}

			target.setCurrentMp(newMp + target.getCurrentMp());
			if(target.isPlayer)
				target.sendPacket(new SystemMessage(SystemMessage.S1_MPS_HAVE_BEEN_RESTORED).addNumber((int) newMp));
			if(activeChar != target)
				activeChar.sendPacket(new SystemMessage(SystemMessage.XS2S_MP_HAS_BEEN_RESTORED_BY_S1).addNumber((int) newMp).addString(target.getName()));
			if(weapon != null)
				weapon.getEffect(false, activeChar, target, isOffensive());
			getEffects(activeChar, target, getActivateRate() > 0);
		}
	}
}
