package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Effect;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.StatsSet;

public class NegateEffects extends L2Skill
{
	private final SkillType[] _negateEffects;
	private final int _negatePower;

	public NegateEffects(StatsSet set)
	{
		super(set);

		String[] negateEffectsString = set.getString("negateEffects", "").split(" ");
		_negateEffects = new SkillType[negateEffectsString.length];
		for(int i = 0; i < negateEffectsString.length; i++)
			_negateEffects[i] = Enum.valueOf(SkillType.class, negateEffectsString[i].toUpperCase());

		_negatePower = set.getInteger("negatePower", Integer.MAX_VALUE);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(_numCharges > 0)
			activeChar.setIncreasedForce(activeChar.getIncreasedForce() - _numCharges);

		if(isSSPossible())
			activeChar.unChargeShots(isMagic());

		L2Weapon weapon = activeChar.getActiveWeaponItem();

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();

			for(SkillType stat : _negateEffects)
				negateEffectAtPower(target, stat, _negatePower);

			if(weapon != null)
				weapon.getEffect(false, activeChar, target, isOffensive());

			getEffects(activeChar, target, getActivateRate() > 0);
		}
	}

	private void negateEffectAtPower(L2Character target, SkillType type, double power)
	{
		for(L2Effect e : target.getAllEffects())
		{
			L2Skill skill = e.getSkill();
			if(skill.getSkillType() == type && skill.isCancelable() && e.getStackOrder() <= power)
				e.exit();
		}
	}
}