package l2r.gameserver.skills.skillclasses;

import java.util.concurrent.ConcurrentLinkedQueue;

import javolution.util.FastList;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Effect;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.skills.Formulas;
import l2r.gameserver.skills.Stats;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.StatsSet;

public class NegateStats extends L2Skill
{
	private final FastList<Stats> _negateStats;
	private final boolean _negateOffensive;

	public NegateStats(StatsSet set)
	{
		super(set);

		String[] negateStats = set.getString("negateStats", "").split(" ");
		_negateStats = new FastList<Stats>(negateStats.length);
		for(String stat : negateStats)
			_negateStats.add(Stats.valueOfXml(stat));

		_negateOffensive = set.getBool("negateDebuffs");
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(isSSPossible())
			activeChar.unChargeShots(isMagic());
		L2Weapon weapon = activeChar.getActiveWeaponItem();
		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(!_negateOffensive && !Formulas.calcSkillSuccess(activeChar, target, this))
				continue;

			ConcurrentLinkedQueue<L2Effect> effects = target.getAllEffects();
			for(Stats stat : _negateStats)
				for(L2Effect e : effects)
				{
					L2Skill skill = e.getSkill();
					if(skill.isOffensive() == _negateOffensive && e.containsStat(stat) && skill.isCancelable())
					{
						target.sendPacket(new SystemMessage(SystemMessage.THE_EFFECT_OF_S1_HAS_BEEN_REMOVED).addSkillName(e.getSkill().getId(), e.getSkill().getDisplayLevel()));
						e.exit();
					}
				}

			getEffects(activeChar, target, getActivateRate() > 0);
			if(weapon != null)
				weapon.getEffect(false, activeChar, target, isOffensive());
		}
	}

	@Override
	public boolean isOffensive()
	{
		return !_negateOffensive;
	}

	public FastList<Stats> getNegateStats()
	{
		return _negateStats;
	}
}