package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Summon;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.skills.Formulas;
import l2r.gameserver.templates.StatsSet;
import l2r.util.Rnd;

public class PDam extends L2Skill
{
	public static final double DEATHLINK_PDAM_MULTIPLIER = 1.8;

	private final boolean _deathlink;
	private final boolean _cancelTarget;
	private final boolean _skillInterrupt;

	public PDam(StatsSet set)
	{
		super(set);
		_deathlink = set.getBool("deathlink", false);
		_cancelTarget = set.getBool("cancelTarget", false);
		_skillInterrupt = set.getBool("skillInterrupt", false);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		boolean ss = activeChar.getChargedSoulShot() && isSSPossible();
		if(ss)
			activeChar.unChargeShots(false);

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(target.isDead())
				continue;

			target.doCounterAttack(this, activeChar);

			if(target.checkReflectSkill(this))
				target = activeChar;

			boolean shld = !getShieldIgnore() && Formulas.calcShldUse(activeChar, target);

			double damage = Formulas.calcPhysDam(activeChar, target, this, shld, false, false, ss);

			if(damage != 1)
			{
				if(_deathlink)
					damage *= DEATHLINK_PDAM_MULTIPLIER * (1. - activeChar.getCurrentHp() / activeChar.getMaxHp());
				if(activeChar.isPlayer)
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber((int) damage));
				else if(activeChar instanceof L2Summon)
					((L2Summon) activeChar).sendDamageMsg((int) damage);
			}

			if(damage < 1)
				damage = 1;

			target.reduceCurrentHp(damage, activeChar, true, true, false);

			if(_cancelTarget && !target.isNpc && Rnd.chance(50))
				target.setTarget(null);

			if(_skillInterrupt)
			{
				if(target.getCastingSkill() != null && !target.getCastingSkill().isMagic() && !target.isRaid)
					target.abortCast();
				target.abortAttack();
			}

			getEffects(activeChar, target, getActivateRate() > 0);

			if(isSuicideAttack())
				activeChar.doDie(null);
		}
	}
}