package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.instancemanager.TownManager;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Zone.ZoneType;
import l2r.gameserver.serverpackets.StopMove;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.MapRegionTable;
import l2r.gameserver.templates.StatsSet;

public class Recall extends L2Skill
{
	private final int _townId;
	private final boolean _clanhall;
	private final boolean _castle;

	public Recall(StatsSet set)
	{
		super(set);
		_townId = set.getInteger("townId", 0);
		_clanhall = set.getBool("clanhall", false);
		_castle = set.getBool("castle", false);
	}

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if((_clanhall || _castle) && getHitTime() == 0) // BSOE в кланхолл/замок работает только при наличии оного
		{
			L2Player player = (L2Player) activeChar;
			if(player.getClan() == null || player.getClan().getHasHideout() == 0)
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.S1_CANNOT_BE_USED_DUE_TO_UNSUITABLE_TERMS).addItemName(_itemConsumeId[0]));
				return false;
			}
		}

		if(activeChar.isPlayer)
		{
			L2Player p = (L2Player) activeChar;
			if(p.getDuel() != null)
			{
				activeChar.sendMessage(new CustomMessage("common.RecallInDuel", activeChar));
				return false;
			}
		}

		if(activeChar.isInZone(ZoneType.no_escape))
		{
			activeChar.sendMessage(new CustomMessage("l2r.gameserver.skills.skillclasses.Recall.Here", activeChar));
			return false;
		}

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(isSSPossible())
			activeChar.unChargeShots(true);
		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			L2Player pcTarget = target.getPlayer();
			if(pcTarget == null)
				continue;

			if(pcTarget.isFestivalParticipant())
			{
				activeChar.sendMessage(new CustomMessage("l2r.gameserver.skills.skillclasses.Recall.Festival", activeChar));
				continue;
			}
			if(pcTarget.isInOlympiadMode())
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.THIS_SKILL_IS_NOT_AVAILABLE_FOR_THE_OLYMPIAD_EVENT));
				return;
			}
			if(pcTarget.getDuel() != null)
			{
				activeChar.sendMessage(new CustomMessage("common.RecallInDuel", activeChar));
				return;
			}

			target.abortAttack();
			target.abortCast();
			target.sendActionFailed();
			target.broadcastPacket(new StopMove(target));

			if(_isItemHandler)
			{
				//TODO: переделать SOE по TownId на SOE по Loc_id
				if(_itemConsumeId[0] == 7125) // floran
				{
					pcTarget.teleToLocation(17144, 170156, -3502);
					continue;
				}
				if(_itemConsumeId[0] == 7127) // hardin's academy
				{
					pcTarget.teleToLocation(105918, 109759, -3207);
					continue;
				}
				if(_itemConsumeId[0] == 7130) // ivory
				{
					pcTarget.teleToLocation(85475, 16087, -3672);
					continue;
				}
				if(_itemConsumeId[0] == 7618)
				{
					pcTarget.teleToLocation(149864, -81062, -5618);
					continue;
				}
				if(_itemConsumeId[0] == 7619)
				{
					pcTarget.teleToLocation(108275, -53785, -2524);
					continue;
				}
				if(_townId > 0) // SoE: Town
				{
					pcTarget.teleToLocation(TownManager.getInstance().getTown(_townId).getSpawn());
					continue;
				}
				if(_castle) // SoE: Castle
				{
					pcTarget.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(pcTarget, MapRegionTable.TeleportWhereType.Castle));
					continue;
				}
				if(_clanhall) // SoE: Clanhall
				{
					pcTarget.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(pcTarget, MapRegionTable.TeleportWhereType.ClanHall));
					continue;
				}
			}

			if(target.isInZone(ZoneType.battle_zone) && target.getZone(ZoneType.battle_zone).getRestartPoints() != null)
			{
				target.teleToLocation(target.getZone(ZoneType.battle_zone).getSpawn());
				continue;
			}

			if(target.isInZone(ZoneType.peace_zone) && target.getZone(ZoneType.peace_zone).getRestartPoints() != null)
			{
				target.teleToLocation(target.getZone(ZoneType.peace_zone).getSpawn());
				continue;
			}

			target.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(target, MapRegionTable.TeleportWhereType.ClosestTown));
		}
	}
}