package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.Config;
import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Manor;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2MonsterInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.templates.StatsSet;
import l2r.util.Rnd;

public class Sowing extends L2Skill
{
	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		// Проверки в хэндлере
		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	public Sowing(StatsSet set)
	{
		super(set);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(!activeChar.isPlayer)
			return;

		L2Player player = (L2Player) activeChar;
		int seed_id = player.getUseSeed();
		// remove seed from inventory
		L2ItemInstance seedItem = player.getInventory().findItemByItemId(seed_id);
		if(seedItem != null)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(seed_id));
			player.getInventory().destroyItem(seedItem, 1, true);
		}

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			L2MonsterInstance monster = (L2MonsterInstance) target;
			if(monster.isSeeded())
				continue;

			// обработка
			double SuccessRate = Config.MANOR_SOWING_BASIC_SUCCESS;

			double diffPlayerTarget = Math.abs(activeChar.getLevel() - target.getLevel());
			double diffSeedTarget = Math.abs(L2Manor.getInstance().getSeedLevel(seed_id) - target.getLevel());

			// Штраф, на разницу уровней между мобом и игроком
			// 5% на каждый уровень при разнице >5 - по умолчанию
			if(diffPlayerTarget > Config.MANOR_DIFF_PLAYER_TARGET)
				SuccessRate -= (diffPlayerTarget - Config.MANOR_DIFF_PLAYER_TARGET) * Config.MANOR_DIFF_PLAYER_TARGET_PENALTY;

			// Штраф, на разницу уровней между семечкой и мобом
			// 5% на каждый уровень при разнице >5 - по умолчанию
			if(diffSeedTarget > Config.MANOR_DIFF_SEED_TARGET)
				SuccessRate -= (diffSeedTarget - Config.MANOR_DIFF_SEED_TARGET) * Config.MANOR_DIFF_SEED_TARGET_PENALTY;

			if(ItemTable.getInstance().getTemplate(seed_id).isAltSeed())
			{
				SuccessRate *= Config.MANOR_SOWING_ALT_BASIC_SUCCESS / Config.MANOR_SOWING_BASIC_SUCCESS;
				SuccessRate = 100 - 1. / player.getBonus().LUCK_MULTIPLIER * (100 - SuccessRate);
			}

			// Минимальный шанс успеха всегда 1%
			if(SuccessRate < 1)
				SuccessRate = 1;

			if(Config.SKILLS_SHOW_CHANCE)
				activeChar.sendMessage(new CustomMessage("l2r.gameserver.skills.skillclasses.Sowing.Chance", activeChar).addNumber((int) SuccessRate));

			if(Rnd.chance((int) SuccessRate))
			{
				monster.setSeeded((short) seed_id, player);
				activeChar.sendPacket(new SystemMessage(SystemMessage.THE_SEED_WAS_SUCCESSFULLY_SOWN));
			}
			else
				activeChar.sendPacket(new SystemMessage(SystemMessage.THE_SEED_WAS_NOT_SOWN));
		}
	}
}