package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.Config;
import l2r.extensions.multilang.CustomMessage;
import l2r.gameserver.ai.CtrlEvent;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.instances.L2MonsterInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.skills.Formulas;
import l2r.gameserver.templates.StatsSet;
import l2r.util.Rnd;

public class Spoil extends L2Skill
{
	private final boolean _crush;

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(target == null)
			return false;
		if(!target.isMonster)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.IT_IS_A_CHARACTER_THAT_CANNOT_BE_SPOILED));
			return false;
		}
		if(((L2MonsterInstance) target).isSpoiled())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.ALREADY_SPOILED));
			return false;
		}
		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	public Spoil(StatsSet set)
	{
		super(set);
		_crush = set.getBool("crush", false);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(!activeChar.isPlayer)
			return;
		boolean ss = activeChar.getChargedSoulShot();
		if(ss && _crush)
			activeChar.unChargeShots(false);

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(!target.isMonster || target.isDead())
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.IT_IS_A_CHARACTER_THAT_CANNOT_BE_SPOILED));
				continue;
			}

			if(_crush)
			{
				boolean shld = !getShieldIgnore() && Formulas.calcShldUse(activeChar, target);
				double damage = Formulas.calcPhysDam(activeChar, target, this, shld, false, false, ss);
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber((int) damage));
				target.reduceCurrentHp(damage, activeChar, true, true, false);
			}

			L2MonsterInstance monster = (L2MonsterInstance) target;
			if(monster.isSpoiled())
				continue;

			boolean success;
			if(!Config.ALT_SPOIL_FORMULA)
			{
				int monsterLevel = monster.getLevel();
				int modifier = Math.abs(monsterLevel - activeChar.getLevel());
				double rateOfSpoil = Config.BASE_SPOIL_RATE;

				if(modifier > 8)
					rateOfSpoil = rateOfSpoil - rateOfSpoil * (modifier - 8) * 9 / 100;

				rateOfSpoil = rateOfSpoil * getMagicLevel() / monsterLevel;

				if(rateOfSpoil < Config.MINIMUM_SPOIL_RATE)
					rateOfSpoil = Config.MINIMUM_SPOIL_RATE;
				else if(rateOfSpoil > 99.)
					rateOfSpoil = 99.;

				activeChar.sendMessage(new CustomMessage("l2r.gameserver.skills.skillclasses.Spoil.Chance", activeChar).addNumber((int) rateOfSpoil));
				success = Rnd.chance((int) rateOfSpoil);
			}
			else
				success = Formulas.calcSkillSuccess(activeChar, target, this);

			if(success)
			{
				monster.setSpoiled(true);
				activeChar.sendPacket(new SystemMessage(SystemMessage.THE_SPOIL_CONDITION_HAS_BEEN_ACTIVATED));
			}
			else
				activeChar.sendPacket(new SystemMessage(SystemMessage.S1_HAS_FAILED).addSkillName(_id, getDisplayLevel()));

			target.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, activeChar, _effectPoint);
			getEffects(activeChar, target, getActivateRate() > 0);
		}
	}
}