package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.idfactory.IdFactory;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.base.Experience;
import l2r.gameserver.model.instances.L2DecoyInstance;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.instances.L2SummonInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.skills.Stats;
import l2r.gameserver.skills.funcs.FuncAdd;
import l2r.gameserver.skills.funcs.LambdaConst;
import l2r.gameserver.tables.NpcTable;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.templates.L2NpcTemplate;
import l2r.gameserver.templates.StatsSet;
import l2r.util.Location;

public class Summon extends L2Skill
{
	private final SummonType _summonType;

	private final float _expPenalty;
	private final int _itemConsumeIdInTime;
	private final int _itemConsumeCountInTime;
	private final int _itemConsumeDelay;
	private final int _lifeTime;

	private static enum SummonType
	{
		PET,
		CUBIC,
		DECOY
	}

	public Summon(StatsSet set)
	{
		super(set);

		_summonType = Enum.valueOf(SummonType.class, set.getString("summonType", "PET").toUpperCase());
		_expPenalty = set.getFloat("expPenalty", 0.f);
		_itemConsumeIdInTime = set.getInteger("itemConsumeIdInTime", 0);
		_itemConsumeCountInTime = set.getInteger("itemConsumeCountInTime", 0);
		_itemConsumeDelay = set.getInteger("itemConsumeDelay", 250) * 1000;
		_lifeTime = set.getInteger("lifeTime", 1200) * 1000;
	}

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(_summonType != SummonType.CUBIC && !activeChar.isPlayer)
			return false;

		L2Player player = activeChar.getPlayer();

		switch(_summonType)
		{
			case CUBIC:
				if(_targetType == SkillTargetType.TARGET_SELF)
				{
					int mastery = player.getSkillLevel(L2Skill.SKILL_CUBIC_MASTERY);
					if(mastery < 0)
						mastery = 0;
					int count = player.getCubics().size();
					if(count > mastery && player.getCubic(getNpcId()) == null)
						return false;
				}
				break;
			case PET:
			case DECOY:
				if(player.getPet() != null || player.isMounted())
				{
					player.sendPacket(new SystemMessage(SystemMessage.YOU_ALREADY_HAVE_A_PET));
					return false;
				}
				break;
		}

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character caster, FastList<L2Character> targets)
	{
		if(_summonType != SummonType.CUBIC && !caster.isPlayer)
			return;

		L2Player activeChar = caster.getPlayer();

		if(isSSPossible())
			caster.unChargeShots(isMagic());

		if(getNpcId() == 0)
		{
			caster.sendMessage("Summon skill " + getId() + " not described yet");
			return;
		}

		switch(_summonType)
		{
			case CUBIC:
				for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
				{
					L2Character targ = n.getValue();
					if(!targ.isPlayer)
						continue;
					L2Player target = (L2Player) targ;

					int mastery = target.getSkillLevel(L2Skill.SKILL_CUBIC_MASTERY);
					if(mastery < 0)
						mastery = 0;

					if(target.getCubics().size() > mastery && target.getCubic(getNpcId()) == null)
					{
						target.sendPacket(new SystemMessage(SystemMessage.CUBIC_SUMMONING_FAILED));
						continue;
					}

					if(getNpcId() == 3 && _lifeTime == 3600000) // novice life cubic
						target.addCubic(3, 8, 3600000);
					else
						target.addCubic(getNpcId(), _level, _lifeTime);

					target.broadcastUserInfo();
					getEffects(caster, target, getActivateRate() > 0);
				}
				break;
			case PET:
				// Удаление трупа, если идет суммон из трупа.
				Location loc = null;
				if(_targetType == SkillTargetType.TARGET_CORPSE)
					for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
					{
						L2Character target = n.getValue();
						if(target != null && target.isDead() && target instanceof L2NpcInstance)
						{
							((L2NpcInstance) target).endDecayTask();
							loc = target.getLoc();
						}
					}

				if(activeChar.getPet() != null || activeChar.isMounted())
					return;

				L2NpcTemplate summonTemplate = NpcTable.getTemplate(getNpcId());
				L2SummonInstance summon = new L2SummonInstance(IdFactory.getInstance().getNextId(), summonTemplate, activeChar, _lifeTime, _itemConsumeIdInTime, _itemConsumeCountInTime, _itemConsumeDelay);

				summon.setTitle(activeChar.getName());
				summon.setExpPenalty(_expPenalty);
				summon.setExp(Experience.LEVEL[summon.getLevel()]);
				summon.setCurrentHp(summon.getMaxHp());
				summon.setCurrentMp(summon.getMaxMp());
				summon.setHeading(activeChar.getHeading());
				summon.setRunning();

				activeChar.setPet(summon);

				L2World.addObject(summon);

				summon.spawnMe(loc == null ? GeoClient.getInstance().findPointToStay(activeChar.getX(), activeChar.getY(), activeChar.getZ(), 100, 150) : loc);

				if(summon.getSkillLevel(4140) > 0)
					summon.altUseSkill(SkillTable.getInstance().getInfo(4140, summon.getSkillLevel(4140)), activeChar);

				if(summon.getName().equalsIgnoreCase("Shadow"))
					summon.addStatFunc(new FuncAdd(Stats.ABSORB_DAMAGE_PERCENT, 0x40, this, new LambdaConst(15)));

				summon.setFollowStatus(true);
				summon.broadcastPetInfo();
				summon.setShowSpawnAnimation(false);
				break;
			case DECOY:
				if(activeChar.getPet() != null || activeChar.isMounted())
					return;

				L2NpcTemplate DecoyTemplate = NpcTable.getTemplate(getNpcId());
				L2DecoyInstance decoy = new L2DecoyInstance(IdFactory.getInstance().getNextId(), DecoyTemplate, activeChar, _lifeTime);

				decoy.setCurrentHp(decoy.getMaxHp());
				decoy.setCurrentMp(decoy.getMaxMp());
				decoy.setHeading(activeChar.getHeading());

				activeChar.setDecoy(decoy);

				L2World.addObject(decoy);

				decoy.spawnMe(activeChar.getLoc());
				break;
		}
	}

	@Override
	public boolean isOffensive()
	{
		return _targetType == SkillTargetType.TARGET_CORPSE;
	}
}