package l2r.gameserver.skills.skillclasses;

import javolution.util.FastList;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2MonsterInstance;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.gameserver.templates.StatsSet;
import l2r.util.Log;

public class Sweep extends L2Skill
{
	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(target == null)
			return false;

		if(!target.isMonster || !((L2MonsterInstance) target).isDead())
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return false;
		}

		if(!((L2MonsterInstance) target).isSpoiled())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.SWEEPER_FAILED_TARGET_NOT_SPOILED));
			return false;
		}
		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	public Sweep(StatsSet set)
	{
		super(set);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(!activeChar.isPlayer)
			return;

		L2Player player = (L2Player) activeChar;

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character targ = n.getValue();
			if(targ == null || !targ.isMonster || !targ.isDead() || !((L2MonsterInstance) targ).isSpoiled())
				continue;

			L2MonsterInstance target = (L2MonsterInstance) targ;

			if(!target.isSweepActive())
			{
				target.endDecayTask();
				continue;
			}

			L2ItemInstance[] items = target.takeSweep();
			target.setSpoiled(false);

			if(items == null || items.length == 0) // и такое бывает o_O
				return;

			for(L2ItemInstance item : items)
			{
				if(player.isInParty() && player.getParty().isDistributeSpoilLoot())
				{
					player.getParty().distributeItem(player, item);
					continue;
				}

				int itemCount = item.getIntegerLimitedCount();
				item = player.getInventory().addItem(item);
				Log.LogItem(player, target, Log.SweepItem, item);

				SystemMessage smsg;
				if(itemCount == 1)
				{
					smsg = new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S1);
					smsg.addItemName(item.getItemId());
					player.sendPacket(smsg);
				}
				else
				{
					smsg = new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S2_S1);
					smsg.addItemName(item.getItemId());
					smsg.addNumber(itemCount);
					player.sendPacket(smsg);
				}
				if(player.isInParty())
					if(itemCount == 1)
					{
						smsg = new SystemMessage(SystemMessage.S1_HAS_OBTAINED_S2_BY_USING_SWEEPER);
						smsg.addString(player.getName());
						smsg.addItemName(item.getItemId());
						player.getParty().broadcastToPartyMembers(player, smsg);
					}
					else
					{
						smsg = new SystemMessage(SystemMessage.S1_HAS_OBTAINED_3_S2_S_BY_USING_SWEEPER);
						smsg.addString(player.getName());
						smsg.addItemName(item.getItemId());
						smsg.addNumber(itemCount);
						player.getParty().broadcastToPartyMembers(player, smsg);
					}
			}
			target.endDecayTask();
		}
	}
}