package l2r.gameserver.tables;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.LineNumberReader;
import java.util.ArrayList;
import java.util.Set;
import java.util.logging.Logger;

import javolution.util.FastMap;
import l2r.Config;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.model.L2World;
import l2r.util.Rnd;

public class FakePlayersTable
{
	public class Task implements Runnable
	{
		public void run()
		{
			try
			{
				if(_fakePlayersCount < L2World.getAllPlayersCount() * Config.FAKE_PLAYERS_PERCENT / 100 && _fakePlayersCount < _fakePlayers.length)
				{
					if(Rnd.chance(60))
					{
						String player = _fakePlayers[Rnd.get(_fakePlayers.length)];
						if(player != null && !_activeFakePlayers.containsKey(player))
						{
							_activeFakePlayers.put(player, null);
							_fakePlayersCount++;
						}
					}
				}
				else if(_fakePlayersCount > 0)
				{
					String fplayer = _activeFakePlayers.keySet().toArray(new String[] {})[Rnd.get(_fakePlayersCount)];
					if(_activeFakePlayers.remove(fplayer) != null)
						_fakePlayersCount--;
				}
			}
			catch(Exception e)
			{}
		}
	}

	private static final Logger _log = Logger.getLogger(FakePlayersTable.class.getName());

	private static String[] _fakePlayers;
	private static FastMap<String, String> _activeFakePlayers = new FastMap<String, String>();
	private static int _fakePlayersCount = 0;

	private static FakePlayersTable _instance;

	public static FakePlayersTable getInstance()
	{
		if(_instance == null)
			new FakePlayersTable();
		return _instance;
	}

	public FakePlayersTable()
	{
		_instance = this;
		if(Config.ALLOW_FAKE_PLAYERS)
		{
			parseData();
			ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new Task(), 180000, 5000);
		}
	}

	private void parseData()
	{
		LineNumberReader lnr = null;
		try
		{
			File doorData = new File(Config.FAKE_PLAYERS_LIST);
			lnr = new LineNumberReader(new BufferedReader(new FileReader(doorData)));
			String line;
			ArrayList<String> players_list = new ArrayList<String>();
			while((line = lnr.readLine()) != null)
			{
				if(line.trim().length() == 0 || line.startsWith("#"))
					continue;
				players_list.add(line);
			}
			_fakePlayers = players_list.toArray(new String[players_list.size()]);
			_log.config("FakePlayersTable: Loaded " + _fakePlayers.length + " Fake Players.");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			try
			{
				if(lnr != null)
					lnr.close();
			}
			catch(Exception e1)
			{}
		}
	}

	public static int getFakePlayersCount()
	{
		return _fakePlayersCount;
	}

	public static Set<String> getActiveFakePlayers()
	{
		return _activeFakePlayers.keySet();
	}
}
