package l2r.gameserver.tables;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import l2r.Config;
import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.FiltredStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.extensions.scripts.Scripts;
import l2r.gameserver.Item;
import l2r.gameserver.idfactory.IdFactory;
import l2r.gameserver.model.L2Playable;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2World;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.skills.SkillsEngine;
import l2r.gameserver.templates.L2Armor;
import l2r.gameserver.templates.L2EtcItem;
import l2r.gameserver.templates.L2Item;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.StatsSet;
import l2r.gameserver.templates.L2Armor.ArmorType;
import l2r.gameserver.templates.L2EtcItem.EtcItemType;
import l2r.gameserver.templates.L2Weapon.WeaponType;
import l2r.util.Files;
import l2r.util.Log;

@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public class ItemTable
{
	private final Logger _log = Logger.getLogger(ItemTable.class.getName());

	private static final HashMap<String, Byte> _materials = new HashMap<String, Byte>();
	private static final HashMap<String, Byte> _crystalTypes = new HashMap<String, Byte>();
	private static final HashMap<String, WeaponType> _weaponTypes = new HashMap<String, WeaponType>();
	private static final HashMap<String, ArmorType> _armorTypes = new HashMap<String, ArmorType>();
	private static final HashMap<String, Integer> _slots = new HashMap<String, Integer>();

	static
	{
		_materials.put("paper", L2Item.MATERIAL_PAPER);
		_materials.put("wood", L2Item.MATERIAL_WOOD);
		_materials.put("liquid", L2Item.MATERIAL_LIQUID);
		_materials.put("cloth", L2Item.MATERIAL_CLOTH);
		_materials.put("leather", L2Item.MATERIAL_LEATHER);
		_materials.put("horn", L2Item.MATERIAL_HORN);
		_materials.put("bone", L2Item.MATERIAL_BONE);
		_materials.put("bronze", L2Item.MATERIAL_BRONZE);
		_materials.put("fine_steel", L2Item.MATERIAL_FINE_STEEL);
		_materials.put("cotton", L2Item.MATERIAL_FINE_STEEL);
		_materials.put("mithril", L2Item.MATERIAL_MITHRIL);
		_materials.put("silver", L2Item.MATERIAL_SILVER);
		_materials.put("gold", L2Item.MATERIAL_GOLD);
		_materials.put("adamantaite", L2Item.MATERIAL_ADAMANTAITE);
		_materials.put("steel", L2Item.MATERIAL_STEEL);
		_materials.put("oriharukon", L2Item.MATERIAL_ORIHARUKON);
		_materials.put("blood_steel", L2Item.MATERIAL_BLOOD_STEEL);
		_materials.put("crystal", L2Item.MATERIAL_CRYSTAL);
		_materials.put("damascus", L2Item.MATERIAL_DAMASCUS);
		_materials.put("chrysolite", L2Item.MATERIAL_CHRYSOLITE);
		_materials.put("scale_of_dragon", L2Item.MATERIAL_SCALE_OF_DRAGON);
		_materials.put("dyestuff", L2Item.MATERIAL_DYESTUFF);
		_materials.put("cobweb", L2Item.MATERIAL_COBWEB);
		_materials.put("seed", L2Item.MATERIAL_SEED);
		_materials.put("fish", L2Item.MATERIAL_FISH);

		_crystalTypes.put("s", L2Item.CRYSTAL_S);
		_crystalTypes.put("a", L2Item.CRYSTAL_A);
		_crystalTypes.put("b", L2Item.CRYSTAL_B);
		_crystalTypes.put("c", L2Item.CRYSTAL_C);
		_crystalTypes.put("d", L2Item.CRYSTAL_D);
		_crystalTypes.put("none", L2Item.CRYSTAL_NONE);

		_weaponTypes.put("blunt", WeaponType.BLUNT);
		_weaponTypes.put("bigblunt", WeaponType.BIGBLUNT);
		_weaponTypes.put("bow", WeaponType.BOW);
		_weaponTypes.put("dagger", WeaponType.DAGGER);
		_weaponTypes.put("dual", WeaponType.DUAL);
		_weaponTypes.put("dualfist", WeaponType.DUALFIST);
		_weaponTypes.put("etc", WeaponType.ETC);
		_weaponTypes.put("fist", WeaponType.FIST);
		_weaponTypes.put("none", WeaponType.NONE); // these are shields !
		_weaponTypes.put("pole", WeaponType.POLE);
		_weaponTypes.put("sword", WeaponType.SWORD);
		_weaponTypes.put("bigsword", WeaponType.BIGSWORD); //Two-Handed Swords
		_weaponTypes.put("pet", WeaponType.PET); //Pet Weapon
		_weaponTypes.put("rod", WeaponType.ROD); //Fishing Rods

		_armorTypes.put("none", ArmorType.NONE);
		_armorTypes.put("light", ArmorType.LIGHT);
		_armorTypes.put("heavy", ArmorType.HEAVY);
		_armorTypes.put("magic", ArmorType.MAGIC);
		_armorTypes.put("pet", ArmorType.PET);

		_slots.put("chest", L2Item.SLOT_CHEST);
		_slots.put("fullarmor", L2Item.SLOT_FULL_ARMOR);
		_slots.put("head", L2Item.SLOT_HEAD);
		_slots.put("hair", L2Item.SLOT_HAIR);
		_slots.put("face", L2Item.SLOT_DHAIR);
		_slots.put("dhair", L2Item.SLOT_HAIRALL);
		_slots.put("underwear", L2Item.SLOT_UNDERWEAR);
		_slots.put("cloak", L2Item.SLOT_CLOAK);
		_slots.put("back", L2Item.SLOT_BACK);
		_slots.put("neck", L2Item.SLOT_NECK);
		_slots.put("legs", L2Item.SLOT_LEGS);
		_slots.put("feet", L2Item.SLOT_FEET);
		_slots.put("gloves", L2Item.SLOT_GLOVES);
		_slots.put("chest,legs", L2Item.SLOT_CHEST | L2Item.SLOT_LEGS);
		_slots.put("rhand", L2Item.SLOT_R_HAND);
		_slots.put("lhand", L2Item.SLOT_L_HAND);
		_slots.put("lrhand", L2Item.SLOT_LR_HAND);
		_slots.put("rear,lear", L2Item.SLOT_R_EAR | L2Item.SLOT_L_EAR);
		_slots.put("rfinger,lfinger", L2Item.SLOT_R_FINGER | L2Item.SLOT_L_FINGER);
		_slots.put("none", L2Item.SLOT_NONE);
		_slots.put("wolf", L2Item.SLOT_WOLF); // for wolf
		_slots.put("hatchling", L2Item.SLOT_HATCHLING); // for hatchling
		_slots.put("strider", L2Item.SLOT_STRIDER); // for strider
		_slots.put("formalwear", L2Item.SLOT_FORMAL_WEAR);
	}

	private L2Item[] _allTemplates;

	private final HashMap<Short, L2EtcItem> _etcItems = new HashMap<Short, L2EtcItem>();
	private final HashMap<Short, L2Armor> _armors = new HashMap<Short, L2Armor>();
	private final HashMap<Short, L2Weapon> _weapons = new HashMap<Short, L2Weapon>();

	private final HashMap<Integer, Item> itemData = new HashMap<Integer, Item>();
	private final HashMap<Integer, Item> weaponData = new HashMap<Integer, Item>();
	private final HashMap<Integer, Item> armorData = new HashMap<Integer, Item>();

	private boolean _initialized = true;

	private static ItemTable _instance;

	/** Table of SQL request in order to obtain items from tables [etcitem], [armor], [weapon] */
	private static final String[] SQL_ITEM_SELECTS = {
			"SELECT item_id, name, class, icon, crystallizable, item_type, weight, consume_type, material, crystal_type, durability, price, crystal_count, sellable, skill_id, skill_level, tradeable, dropable, destroyable FROM etcitem",
			"SELECT item_id, name, icon, bodypart, crystallizable, armor_type, weight, material, crystal_type, avoid_modify, durability, p_def, m_def, mp_bonus, price, crystal_count, sellable, tradeable, dropable, destroyable, skill_id, skill_level FROM armor",
			"SELECT item_id, name, icon, bodypart, crystallizable, weight, soulshots, spiritshots, material, crystal_type, p_dam, rnd_dam, weaponType, critical, hit_modify, avoid_modify, shield_def, shield_def_rate, atk_speed, mp_consume, m_dam, durability, price, crystal_count, sellable, tradeable, dropable, destroyable, skill_id, skill_level, enchant4_skill_id, enchant4_skill_lvl FROM weapon" };

	/**
	 * Returns instance of ItemTable
	 * @return ItemTable
	 */
	public static ItemTable getInstance()
	{
		if(_instance == null)
			_instance = new ItemTable();
		return _instance;
	}

	/**
	 * Не работает, не использовать...
	public static void reload()
	{
		_instance = null;
		getInstance();
	}
	*/

	/**
	 * Returns a new object Item
	 * @return
	 */
	public Item newItem()
	{
		return new Item();
	}

	/**
	 * Constructor.
	 */
	private ItemTable()
	{
		ThreadConnection con = null;
		FiltredStatement st = null;
		FiltredPreparedStatement statement = null;
		ResultSet rs = null, rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			// Создаем таблицу prices
			st = con.createStatement();

			String sql = Files.read("data/prices.sql");
			for(String rq : sql.replaceAll("\\n", "").split(";"))
				if(rq.length() > 0)
					st.addBatch(rq);
			st.executeBatch();
			rs = st.executeQuery("SELECT COUNT(*) FROM `prices`");
			if(rs.next())
				_log.config("ItemTable: Created prices table with " + rs.getInt(1) + " rows.");
			DatabaseUtils.closeDatabaseSR(st, rs);

			for(String selectQuery : SQL_ITEM_SELECTS)
			{
				statement = con.prepareStatement(selectQuery);
				rset = statement.executeQuery();

				// Add item in correct HashMap
				while(rset.next())
					if(selectQuery.endsWith("etcitem"))
					{
						Item newItem = readItem(rset);
						itemData.put(newItem.id, newItem);
					}
					else if(selectQuery.endsWith("armor"))
					{
						Item newItem = readArmor(rset);
						armorData.put(newItem.id, newItem);
					}
					else if(selectQuery.endsWith("weapon"))
					{
						Item newItem = readWeapon(rset);
						weaponData.put(newItem.id, newItem);
					}
				DatabaseUtils.closeDatabaseSR(statement, rset);
			}
		}
		catch(Exception e)
		{
			_log.log(Level.WARNING, "data error on item: ", e);
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseSR(statement, rset);
			DatabaseUtils.closeDatabaseCSR(con, st, rs);
		}

		for(L2Armor armor : SkillsEngine.getInstance().loadArmors(armorData))
			if(armor != null)
				_armors.put(armor.getItemId(), armor);

		_log.config("ItemTable: Loaded " + _armors.size() + " Armors.");

		for(L2EtcItem item : SkillsEngine.getInstance().loadItems(itemData))
			if(item != null)
				_etcItems.put(item.getItemId(), item);

		_log.config("ItemTable: Loaded " + _etcItems.size() + " Items.");

		for(L2Weapon weapon : SkillsEngine.getInstance().loadWeapons(weaponData))
			if(weapon != null)
				_weapons.put(weapon.getItemId(), weapon);

		_log.config("ItemTable: Loaded " + _weapons.size() + " Weapons.");

		buildFastLookupTable();
	}

	/**
	 * Returns object Item from the record of the database
	 * @param rset : ResultSet designating a record of the [weapon] table of database
	 * @return Item : object created from the database record
	 * @throws SQLException
	 */
	private Item readWeapon(ResultSet rset) throws SQLException
	{
		Item item = new Item();
		item.set = new StatsSet();
		item.id = rset.getInt("item_id");
		item.type = _weaponTypes.get(rset.getString("weaponType"));
		if(item.type == null)
			System.out.println("Error in weapons table: unknown weapon type " + rset.getString("weaponType") + " for item " + item.id);
		item.name = rset.getString("name");
		item.set.set("class", "EQUIPMENT");

		item.set.set("item_id", item.id);
		item.set.set("name", item.name);

		// lets see if this is a shield
		if(item.type == WeaponType.NONE)
		{
			item.set.set("type1", L2Item.TYPE1_SHIELD_ARMOR);
			item.set.set("type2", L2Item.TYPE2_SHIELD_ARMOR);
		}
		else
		{
			item.set.set("type1", L2Item.TYPE1_WEAPON_RING_EARRING_NECKLACE);
			item.set.set("type2", L2Item.TYPE2_WEAPON);
		}

		item.set.set("bodypart", _slots.get(rset.getString("bodypart")));
		item.set.set("material", _materials.get(rset.getString("material")));
		item.set.set("crystal_type", _crystalTypes.get(rset.getString("crystal_type")));
		item.set.set("crystallizable", Boolean.valueOf(rset.getString("crystallizable")).booleanValue());
		item.set.set("weight", rset.getInt("weight"));
		item.set.set("soulshots", rset.getInt("soulshots"));
		item.set.set("spiritshots", rset.getInt("spiritshots"));
		item.set.set("p_dam", rset.getInt("p_dam"));
		item.set.set("rnd_dam", rset.getInt("rnd_dam"));
		item.set.set("critical", rset.getInt("critical"));
		item.set.set("hit_modify", rset.getDouble("hit_modify"));
		item.set.set("avoid_modify", rset.getInt("avoid_modify"));
		item.set.set("shield_def", rset.getInt("shield_def"));
		item.set.set("shield_def_rate", rset.getInt("shield_def_rate"));
		item.set.set("atk_speed", rset.getInt("atk_speed"));
		item.set.set("mp_consume", rset.getInt("mp_consume"));
		item.set.set("m_dam", rset.getInt("m_dam"));
		item.set.set("durability", rset.getInt("durability"));
		item.set.set("price", rset.getInt("price"));
		item.set.set("crystal_count", rset.getInt("crystal_count"));
		item.set.set("sellable", Boolean.valueOf(rset.getString("sellable")));
		item.set.set("tradeable", rset.getInt("tradeable") > 0);
		item.set.set("dropable", rset.getInt("dropable") > 0);
		item.set.set("destroyable", rset.getInt("destroyable") > 0);
		item.set.set("skill_id", rset.getInt("skill_id"));
		item.set.set("skill_level", rset.getInt("skill_level"));
		item.set.set("enchant4_skill_id", rset.getInt("enchant4_skill_id"));
		item.set.set("enchant4_skill_lvl", rset.getInt("enchant4_skill_lvl"));
		item.set.set("icon", rset.getString("icon"));

		if(item.type == WeaponType.PET)
		{
			item.set.set("type1", L2Item.TYPE1_WEAPON_RING_EARRING_NECKLACE);
			if(item.set.getInteger("bodypart") == L2Item.SLOT_WOLF)
				item.set.set("type2", L2Item.TYPE2_PET_WOLF);
			
			else if(item.set.getInteger("bodypart") == L2Item.SLOT_HATCHLING)
				item.set.set("type2", L2Item.TYPE2_PET_HATCHLING);
			else
				item.set.set("type2", L2Item.TYPE2_PET_STRIDER);

			item.set.set("bodypart", L2Item.SLOT_R_HAND);
		}

		return item;
	}

	/**
	 * Returns object Item from the record of the database
	 * @param rset : ResultSet designating a record of the [armor] table of database
	 * @return Item : object created from the database record
	 * @throws SQLException
	 */
	private Item readArmor(ResultSet rset) throws SQLException
	{
		Item item = new Item();
		item.set = new StatsSet();
		item.type = _armorTypes.get(rset.getString("armor_type"));
		item.id = rset.getInt("item_id");
		item.name = rset.getString("name");
		item.set.set("class", "EQUIPMENT");

		item.set.set("item_id", item.id);
		item.set.set("name", item.name);
		int bodypart = _slots.get(rset.getString("bodypart"));
		item.set.set("bodypart", bodypart);
		item.set.set("crystallizable", Boolean.valueOf(rset.getString("crystallizable")));
		item.set.set("crystal_count", rset.getInt("crystal_count"));
		item.set.set("sellable", Boolean.valueOf(rset.getString("sellable")));

		if(bodypart == L2Item.SLOT_NECK || bodypart == L2Item.SLOT_HAIR || bodypart == L2Item.SLOT_DHAIR || bodypart == L2Item.SLOT_DHAIR || (bodypart & L2Item.SLOT_L_EAR) != 0 || (bodypart & L2Item.SLOT_L_FINGER) != 0)
		{
			item.set.set("type1", L2Item.TYPE1_WEAPON_RING_EARRING_NECKLACE);
			item.set.set("type2", L2Item.TYPE2_ACCESSORY);
		}
		else
		{
			item.set.set("type1", L2Item.TYPE1_SHIELD_ARMOR);
			item.set.set("type2", L2Item.TYPE2_SHIELD_ARMOR);
		}

		item.set.set("weight", rset.getInt("weight"));
		item.set.set("material", _materials.get(rset.getString("material")));
		item.set.set("crystal_type", _crystalTypes.get(rset.getString("crystal_type")));
		item.set.set("avoid_modify", rset.getInt("avoid_modify"));
		item.set.set("durability", rset.getInt("durability"));
		item.set.set("p_def", rset.getInt("p_def"));
		item.set.set("m_def", rset.getInt("m_def"));
		item.set.set("mp_bonus", rset.getInt("mp_bonus"));
		item.set.set("price", rset.getInt("price"));
		item.set.set("tradeable", rset.getInt("tradeable") > 0);
		item.set.set("dropable", rset.getInt("dropable") > 0);
		item.set.set("destroyable", rset.getInt("destroyable") > 0);
		item.set.set("skill_id", rset.getInt("skill_id"));
		item.set.set("skill_level", rset.getInt("skill_level"));
		item.set.set("icon", rset.getString("icon"));

		if(item.type == ArmorType.PET)
		{
			item.set.set("type1", L2Item.TYPE1_SHIELD_ARMOR);
			if(item.set.getInteger("bodypart") == L2Item.SLOT_WOLF)
				item.set.set("type2", L2Item.TYPE2_PET_WOLF);

			else if(item.set.getInteger("bodypart") == L2Item.SLOT_HATCHLING)
				item.set.set("type2", L2Item.TYPE2_PET_HATCHLING);
			else
				item.set.set("type2", L2Item.TYPE2_PET_STRIDER);

			item.set.set("bodypart", L2Item.SLOT_CHEST);
		}

		return item;
	}

	/**
	 * Returns object Item from the record of the database
	 * @param rset : ResultSet designating a record of the [etcitem] table of database
	 * @return Item : object created from the database record
	 * @throws SQLException
	 */
	private Item readItem(ResultSet rset) throws SQLException
	{
		Item item = new Item();
		item.set = new StatsSet();
		item.id = rset.getInt("item_id");

		item.set.set("item_id", item.id);
		item.set.set("crystallizable", Boolean.valueOf(rset.getString("crystallizable")));
		item.set.set("type1", L2Item.TYPE1_ITEM_QUESTITEM_ADENA);
		item.set.set("type2", L2Item.TYPE2_OTHER);
		item.set.set("bodypart", 0);
		item.set.set("crystal_count", rset.getInt("crystal_count"));
		item.set.set("sellable", Boolean.valueOf(rset.getString("sellable")));
		item.set.set("icon", rset.getString("icon"));
		item.set.set("class", rset.getString("class"));
		String itemType = rset.getString("item_type");
		if(itemType.equals("none"))
			item.type = EtcItemType.OTHER; // only for default
		else if(itemType.equals("mticket"))
			item.type = EtcItemType.SCROLL; // dummy
		else if(itemType.equals("material"))
			item.type = EtcItemType.MATERIAL;
		else if(itemType.equals("pet_collar"))
			item.type = EtcItemType.PET_COLLAR;
		else if(itemType.equals("potion"))
			item.type = EtcItemType.POTION;
		else if(itemType.equals("recipe"))
			item.type = EtcItemType.RECIPE;
		else if(itemType.equals("scroll"))
			item.type = EtcItemType.SCROLL;
		else if(itemType.equals("seed"))
			item.type = EtcItemType.SEED;
		else if(itemType.equals("spellbook"))
			item.type = EtcItemType.SPELLBOOK; // Spellbook, Amulet, Blueprint
		else if(itemType.equals("shot"))
			item.type = EtcItemType.SHOT;
		else if(itemType.equals("arrow"))
		{
			item.type = EtcItemType.ARROW;
			item.set.set("bodypart", L2Item.SLOT_L_HAND);
		}
		else if(itemType.equals("bait"))
		{
			item.type = EtcItemType.BAIT;
			item.set.set("bodypart", L2Item.SLOT_L_HAND);
		}
		else if(itemType.equals("quest"))
		{
			item.type = EtcItemType.QUEST;
			item.set.set("type2", L2Item.TYPE2_QUEST);
		}
		else
		{
			_log.fine("unknown etcitem type:" + itemType);
			item.type = EtcItemType.OTHER;
		}

		String consume = rset.getString("consume_type");
		if(consume.equals("asset"))
		{
			item.type = EtcItemType.MONEY;
			item.set.set("stackable", true);
			item.set.set("type2", L2Item.TYPE2_MONEY);
		}
		else if(consume.equals("stackable"))
			item.set.set("stackable", true);
		else
			item.set.set("stackable", false);

		int material = _materials.get(rset.getString("material"));
		item.set.set("material", material);

		int crystal = _crystalTypes.get(rset.getString("crystal_type"));
		item.set.set("crystal_type", crystal);

		int weight = rset.getInt("weight");
		item.set.set("weight", weight);
		item.name = rset.getString("name");
		item.set.set("name", item.name);

		item.set.set("durability", rset.getInt("durability"));
		item.set.set("price", rset.getInt("price"));
		item.set.set("skill_id", rset.getInt("skill_id"));
		item.set.set("skill_level", rset.getInt("skill_level"));
		item.set.set("tradeable", rset.getInt("tradeable") > 0);
		item.set.set("dropable", rset.getInt("dropable") > 0);
		item.set.set("destroyable", rset.getInt("destroyable") > 0);

		return item;
	}

	/**
	 * Builds a variable in which all items are putting in in function of their ID.
	 */
	private void buildFastLookupTable()
	{
		int highestId = 0;

		for(Short id : _armors.keySet())
		{
			L2Armor item = _armors.get(id);
			if(item.getItemId() > highestId)
				highestId = item.getItemId();
		}
		for(Short id : _weapons.keySet())
		{
			L2Weapon item = _weapons.get(id);
			if(item.getItemId() > highestId)
				highestId = item.getItemId();
		}
		for(Short id : _etcItems.keySet())
		{
			L2EtcItem item = _etcItems.get(id);
			if(item.getItemId() > highestId)
				highestId = item.getItemId();
		}

		// Create a FastLookUp Table called _allTemplates of size : value of the highest item ID
		if(Config.DEBUG)
			_log.fine("highest item id used:" + highestId);
		_allTemplates = new L2Item[highestId + 1];

		for(Short id : _armors.keySet())
		{
			L2Armor item = _armors.get(id);
			assert _allTemplates[id.intValue()] == null;
			_allTemplates[id.intValue()] = item;
		}

		for(Short id : _weapons.keySet())
		{
			L2Weapon item = _weapons.get(id);
			assert _allTemplates[id.intValue()] == null;
			_allTemplates[id.intValue()] = item;
		}

		for(Short id : _etcItems.keySet())
		{
			L2EtcItem item = _etcItems.get(id);
			assert _allTemplates[id.intValue()] == null;
			_allTemplates[id.intValue()] = item;
		}
	}

	/**
	 * Create the L2ItemInstance corresponding to the Item Identifier and add it to _allObjects of L2world.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Create and Init the L2ItemInstance corresponding to the Item Identifier </li>
	 * <li>Add the L2ItemInstance object to _allObjects of L2world </li><BR><BR>
	 *
	 * @param itemId The Item Identifier of the L2ItemInstance that must be created
	 */
	public L2ItemInstance createItem(int itemId, int destination, int source, String create_type)
	{
		if(create_type == null)
			create_type = "";

		if(Config.DISABLE_CREATION_ID_LIST.contains(itemId) && !create_type.equals("delayed_add"))
		{
			Log.displayStackTrace(new Throwable(), "Try creating DISABLE_CREATION item " + itemId + " " + create_type);
			return null;
		}

		// Create and Init the L2ItemInstance corresponding to the Item Identifier
		L2ItemInstance temp = new L2ItemInstance(IdFactory.getInstance().getNextId(), itemId, destination, source, create_type);
		if(Config.DEBUG)
			_log.fine("ItemTable: Item created	oid:" + temp.getObjectId() + " itemid:" + itemId);

		// Add the L2ItemInstance object to _allObjects of L2world
		L2World.addObject(temp);

		return temp;
	}

	/**
	 * Returns a dummy (fr = factice) item.<BR><BR>
	 * <U><I>Concept :</I></U><BR>
	 * Dummy item is created by setting the ID of the object in the world at null value
	 * @param itemId : int designating the item
	 * @param destination
     * @param source
     * @param create_type
     * @return L2ItemInstance designating the dummy item created
	 */
	public L2ItemInstance createDummyItem(int itemId, int destination, int source, String create_type)
	{
		L2Item item = getTemplate(itemId);
		if(item == null)
			return null;
		L2ItemInstance temp = new L2ItemInstance(0, item, destination, source, create_type);
		try
		{
			temp = new L2ItemInstance(0, itemId, destination, source, create_type);
		}
		catch(ArrayIndexOutOfBoundsException e)
		{
			e.printStackTrace(); // this can happen if the item templates were not initialized
		}

		if(temp.getItem() == null)
			_log.warning("ItemTable: Item Template missing for Id: " + itemId);

		return temp;
	}

	public L2ItemInstance createDummyItem(int itemId)
	{
		return createDummyItem(itemId, 0, 0, "");
	}

	public static boolean useHandler(L2Playable self, L2ItemInstance item)
	{
		L2Player player;
		if(self.isPlayer)
			player = (L2Player) self;
		else if(self.isPet)
			player = self.getOwner();
		else
			return false;

		// Вызов всех определенных скриптовых итемхэндлеров
		ArrayList<String> handlers = Scripts.itemHandlers.get(item.getItemId());
		boolean result = false;
		if(handlers != null)
			for(String handler : handlers)
			{
				String[] list = handler.split(":");
				String _class = list[0];
				String _method = list[1];
				player.callScripts(_class, _method, new Object[] {});
				result = true;
			}
		if(result)
			return true;

		L2Skill skill = item.getItem().getFirstSkill();
		if(skill != null)
		{
			if(skill.checkCondition(player, skill.getAimingTarget(player), false, false, true))
				player.getAI().Cast(skill, skill.getAimingTarget(player), false, false);
			return true;
		}
		return false;
	}

	/**
	 * Returns the item corresponding to the item ID
	 * @param id : int designating the item
	 */
	public L2Item getTemplate(int id)
	{
		if(id >= _allTemplates.length || id < 0)
		{
			_log.warning("ItemTable[604]: Not defined item_id=" + id + "; out of range");
			Thread.dumpStack();
			return null;
		}
		return _allTemplates[id];
	}

	public L2Item[] getAllTemplates()
	{
		return _allTemplates;
	}

	public Collection<L2Weapon> getAllWeapons()
	{
		return _weapons.values();
	}

	/**
	 * Returns if ItemTable initialized
	 */
	public boolean isInitialized()
	{
		return _initialized;
	}
}
