package l2r.gameserver.tables;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.logging.Level;
import java.util.logging.Logger;

import l2r.Config;
import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.extensions.scripts.Script;
import l2r.extensions.scripts.Scripts;
import l2r.gameserver.cache.InfoCache;
import l2r.gameserver.model.L2Drop;
import l2r.gameserver.model.L2DropData;
import l2r.gameserver.model.L2DropGroup;
import l2r.gameserver.model.L2MinionData;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.L2Skill.SkillType;
import l2r.gameserver.model.base.ClassId;
import l2r.gameserver.templates.L2NpcTemplate;
import l2r.gameserver.templates.StatsSet;
import l2r.util.Log;

public class NpcTable
{
	private static final Logger _log = Logger.getLogger(NpcTable.class.getName());

	private static NpcTable _instance;

	private static HashMap<Integer, L2NpcTemplate> _npcs;
	private static HashMap<Integer, StatsSet> ai_params;
	private static ArrayList<L2NpcTemplate>[] _npcsByLevel;
	private static HashMap<String, L2NpcTemplate> _npcsNames;
	private static boolean _initialized = false;

	public static NpcTable getInstance()
	{
		if(_instance == null)
			_instance = new NpcTable();

		return _instance;
	}

	@SuppressWarnings("unchecked")
	private NpcTable()
	{
		_npcs = new HashMap<Integer, L2NpcTemplate>();
		_npcsByLevel = new ArrayList[100];
		_npcsNames = new HashMap<String, L2NpcTemplate>();
		ai_params = new HashMap<Integer, StatsSet>();
		RestoreNpcData();
	}

	private final double[] hprateskill = new double[] { 0, 1, 1.2, 1.3, 2, 2, 4, 4, 0.25, 0.5, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 };

	private void RestoreNpcData()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rs = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			try
			{
				statement = con.prepareStatement("SELECT * FROM ai_params");
				rs = statement.executeQuery();
				LoadAIParams(rs);
			}
			catch(Exception e)
			{}
			finally
			{
				DatabaseUtils.closeDatabaseSR(statement, rs);
			}

			try
			{
				statement = con.prepareStatement("SELECT * FROM npc WHERE ai_type IS NOT NULL");
				rs = statement.executeQuery();
				fillNpcTable(rs);
			}
			catch(Exception e)
			{
				_log.log(Level.SEVERE, "error while creating npc table ", e);
			}
			finally
			{
				DatabaseUtils.closeDatabaseSR(statement, rs);
			}

			try
			{
				statement = con.prepareStatement("SELECT npcid, skillid, level FROM npcskills");
				rs = statement.executeQuery();
				L2NpcTemplate npcDat;
				L2Skill npcSkill;

				HashSet<Integer> unimpl = new HashSet<Integer>();
				int counter = 0;
				while(rs.next())
				{
					int mobId = rs.getInt("npcid");
					npcDat = _npcs.get(mobId);
					if(npcDat == null)
						continue;
					short skillId = rs.getShort("skillid");
					byte level = rs.getByte("level");

					// Для определения расы используется скилл 4416
					if(skillId == 4416)
					{
						npcDat.setRace(level);
						continue;
					}
					// Скилы 4290 - 4302 не используются
					else if(skillId >= 4290 && skillId <= 4302)
					{
						_log.info("Warning! Skill " + skillId + " not used, use 4416 instead.");
						continue;
					}

					if(skillId == 4408)
						npcDat.setRateHp(hprateskill[level]);

					npcSkill = SkillTable.getInstance().getInfo(skillId, level);

					if(npcSkill == null || npcSkill.getSkillType() == SkillType.NOTDONE)
						unimpl.add(Integer.valueOf(skillId));

					if(npcSkill == null)
						continue;

					npcDat.addSkill(npcSkill);
					counter++;
				}
				for(Integer i : unimpl)
					Log.add("[" + i + "] " + SkillTable.getInstance().getInfo(i, 1), "unimplemented_npc_skills", "");
				_log.info("Loaded " + counter + " npc skills.");
			}
			catch(Exception e)
			{
				_log.log(Level.SEVERE, "error while reading npcskills table ", e);
			}
			finally
			{
				DatabaseUtils.closeDatabaseSR(statement, rs);
			}

			try
			{
				statement = con.prepareStatement("SELECT mobId, itemId, min, max, sweep, chance, gid, gchance FROM droplist ORDER BY mobId,gid,chance DESC");
				rs = statement.executeQuery();
				L2DropData dropDat = null;
				L2NpcTemplate npcDat = null;

				while(rs.next())
				{
					int mobId = rs.getInt("mobId");
					npcDat = _npcs.get(mobId);
					if(npcDat != null)
					{
						dropDat = new L2DropData();

						dropDat.setItemId(rs.getShort("itemId"));
						dropDat.setMinDrop(rs.getInt("min"));
						dropDat.setMaxDrop(rs.getInt("max"));
						dropDat.setSweep(rs.getInt("sweep") == 1);
						dropDat.setChance(rs.getInt("chance"));
						dropDat.setGroupId(rs.getShort("gid"));
						dropDat.setGroupChance(rs.getInt("gchance"));
						npcDat.addDropData(dropDat);
					}
				}

				if(Config.ALT_GAME_SHOW_DROPLIST && !Config.ALT_GAME_GEN_DROPLIST_ON_DEMAND)
					FillDropList();
				else
					_log.info("Players droplist load skipped");

				if(Config.KILL_COUNTER_PRELOAD && Config.KILL_COUNTER)
					FillKillCount();
				else
					_log.info("Kills counter load skipped");

			}
			catch(Exception e)
			{
				_log.log(Level.SEVERE, "error reading npc drops ", e);
			}
			finally
			{
				DatabaseUtils.closeDatabaseSR(statement, rs);
			}

			try
			{
				statement = con.prepareStatement("SELECT boss_id, minion_id, amount FROM minions");
				rs = statement.executeQuery();
				L2MinionData minionDat = null;
				L2NpcTemplate npcDat = null;
				int cnt = 0;

				while(rs.next())
				{
					int raidId = rs.getInt("boss_id");
					npcDat = _npcs.get(raidId);
					minionDat = new L2MinionData();
					minionDat.setMinionId(rs.getInt("minion_id"));
					minionDat.setAmount(rs.getByte("amount"));
					npcDat.addRaidData(minionDat);
					cnt++;
				}

				_log.config("NpcTable: Loaded " + cnt + " Minions.");
			}
			catch(Exception e)
			{
				_log.log(Level.SEVERE, "error loading minions", e);
			}
			finally
			{
				DatabaseUtils.closeDatabaseSR(statement, rs);
			}

			try
			{
				statement = con.prepareStatement("SELECT npc_id, class_id FROM skill_learn");
				rs = statement.executeQuery();
				L2NpcTemplate npcDat = null;
				int cnt = 0;

				while(rs.next())
				{
					npcDat = _npcs.get(rs.getInt(1));
					npcDat.addTeachInfo(ClassId.values()[rs.getInt(2)]);
					cnt++;
				}

				_log.config("NpcTable: Loaded " + cnt + " SkillLearn entrys.");
			}
			catch(Exception e)
			{
				_log.log(Level.SEVERE, "error loading minions", e);
			}
			finally
			{
				DatabaseUtils.closeDatabaseSR(statement, rs);
			}
		}
		catch(Exception e)
		{
			_log.log(Level.SEVERE, "Cannot find connection to database");
		}
		finally
		{
			DatabaseUtils.closeConnection(con);
		}

		_initialized = true;

		Scripts.getInstance();
	}

	private static void LoadAIParams(ResultSet AIData) throws Exception
	{
		int ai_params_counter = 0;
		StatsSet _set = null;
		int npc_id;
		String param, value;
		while(AIData.next())
		{
			npc_id = AIData.getInt("npc_id");
			param = AIData.getString("param");
			value = AIData.getString("value");
			if(ai_params.containsKey(npc_id))
				_set = ai_params.get(npc_id);
			else
			{
				_set = new StatsSet();
				ai_params.put(npc_id, _set);
			}
			_set.set(param, value);
			ai_params_counter++;
		}
		_log.info("NpcTable: Loaded " + ai_params_counter + " AI params for " + ai_params.size() + " NPCs.");
	}

	private static StatsSet fillNpcTable(ResultSet NpcData) throws Exception
	{
		StatsSet npcDat = null;
		while(NpcData.next())
		{
			npcDat = new StatsSet();
			int id = NpcData.getInt("id");
			int level = NpcData.getByte("level");

			npcDat.set("npcId", id);
			npcDat.set("displayId", NpcData.getInt("displayId"));
			npcDat.set("level", level);
			npcDat.set("jClass", NpcData.getString("class"));

			npcDat.set("baseShldDef", NpcData.getInt("shield_defense"));
			npcDat.set("baseShldRate", NpcData.getInt("shield_defense_rate"));
			npcDat.set("baseCritRate", Math.max(1, NpcData.getInt("base_critical")) * 10);

			npcDat.set("name", NpcData.getString("name"));
			npcDat.set("title", NpcData.getString("title"));
			npcDat.set("collision_radius", NpcData.getDouble("collision_radius"));
			npcDat.set("collision_height", NpcData.getDouble("collision_height"));
			npcDat.set("sex", NpcData.getString("sex"));
			npcDat.set("type", NpcData.getString("type"));
			npcDat.set("ai_type", NpcData.getString("ai_type"));
			npcDat.set("baseAtkRange", NpcData.getInt("attackrange"));
			npcDat.set("revardExp", NpcData.getInt("exp"));
			npcDat.set("revardSp", NpcData.getInt("sp"));
			npcDat.set("basePAtkSpd", NpcData.getInt("atkspd"));
			npcDat.set("baseMAtkSpd", NpcData.getInt("matkspd"));
			npcDat.set("aggroRange", NpcData.getShort("aggro"));
			npcDat.set("rhand", NpcData.getInt("rhand"));
			npcDat.set("lhand", NpcData.getInt("lhand"));
			npcDat.set("armor", NpcData.getInt("armor"));
			npcDat.set("baseWalkSpd", NpcData.getInt("walkspd"));
			npcDat.set("baseRunSpd", NpcData.getInt("runspd"));

			npcDat.set("baseHpReg", NpcData.getDouble("base_hp_regen"));
			npcDat.set("baseCpReg", 0);
			npcDat.set("baseMpReg", NpcData.getDouble("base_mp_regen"));

			npcDat.set("baseSTR", NpcData.getInt("str"));
			npcDat.set("baseCON", NpcData.getInt("con"));
			npcDat.set("baseDEX", NpcData.getInt("dex"));
			npcDat.set("baseINT", NpcData.getInt("int"));
			npcDat.set("baseWIT", NpcData.getInt("wit"));
			npcDat.set("baseMEN", NpcData.getInt("men"));

			npcDat.set("baseHpMax", NpcData.getInt("hp"));
			npcDat.set("baseCpMax", 0);
			npcDat.set("baseMpMax", NpcData.getInt("mp"));
			npcDat.set("basePAtk", NpcData.getInt("patk"));
			npcDat.set("basePDef", NpcData.getInt("pdef"));
			npcDat.set("baseMAtk", NpcData.getInt("matk"));
			npcDat.set("baseMDef", NpcData.getInt("mdef"));

			npcDat.set("factionId", NpcData.getString("faction_id"));
			npcDat.set("factionRange", NpcData.getShort("faction_range"));

			npcDat.set("isDropHerbs", NpcData.getBoolean("isDropHerbs"));

			npcDat.set("shots", NpcData.getString("shots"));

			L2NpcTemplate template = new L2NpcTemplate(npcDat, ai_params.containsKey(id) ? ai_params.get(id) : null);
			_npcs.put(id, template);
			if(_npcsByLevel[level] == null)
				_npcsByLevel[level] = new ArrayList<L2NpcTemplate>();
			_npcsByLevel[level].add(template);
			_npcsNames.put(NpcData.getString("name").toLowerCase(), template);
		}
		_log.config("NpcTable: Loaded " + _npcs.size() + " Npc Templates.");

		return npcDat;
	}

	public static void reloadNpc(int id)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement st = null;
		ResultSet rs = null;
		try
		{
			// save a copy of the old data
			L2NpcTemplate old = getTemplate(id);
			HashMap<Integer, L2Skill> skills = new HashMap<Integer, L2Skill>();
			if(old.getSkills() != null)
				skills.putAll(old.getSkills());
			/*
			 Contact with Styx to understand this commenting
			 ArrayList<L2DropData> drops = new ArrayList<L2DropData>();
			 if(old.getDropData() != null)
			 drops.addAll(old.getDropData());
			 */
			ClassId[] classIds = null;
			if(old.getTeachInfo() != null)
				classIds = old.getTeachInfo().clone();
			ArrayList<L2MinionData> minions = new ArrayList<L2MinionData>();
			if(old.getMinionData() != null)
				minions.addAll(old.getMinionData());

			// reload the NPC base data
			con = L2DatabaseFactory.getInstance().getConnection();
			st = con.prepareStatement("SELECT * FROM npc WHERE id=?");
			st.setInt(1, id);
			rs = st.executeQuery();
			fillNpcTable(rs);

			// restore additional data from saved copy
			L2NpcTemplate created = getTemplate(id);
			for(L2Skill skill : skills.values())
				created.addSkill(skill);
			/*
			 for(L2DropData drop : drops)
			 created.addDropData(drop);
			 */
			if(classIds != null)
				for(ClassId classId : classIds)
					created.addTeachInfo(classId);
			for(L2MinionData minion : minions)
				created.addRaidData(minion);
		}
		catch(Exception e)
		{
			_log.warning("cannot reload npc " + id + ": " + e);
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, st, rs);
		}
	}

	public static StatsSet getNpcStatsSet(int id)
	{
		StatsSet dat = null;

		ThreadConnection con = null;
		FiltredPreparedStatement st = null;
		ResultSet rs = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			st = con.prepareStatement("SELECT * FROM npc WHERE id=?");
			st.setInt(1, id);
			rs = st.executeQuery();
			dat = fillNpcTable(rs);
		}
		catch(Exception e)
		{
			_log.warning("cannot load npc stats for " + id + ": " + e);
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, st, rs);
		}

		return dat;
	}

	// just wrapper
	public void reloadAllNpc()
	{
		RestoreNpcData();
	}

	public void saveNpc(StatsSet npc, L2Player editor)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		String query = "";
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			HashMap<String, Object> set = npc.getSet();
			String name = "";
			String values = "";
			for(Object obj : set.keySet())
			{
				name = (String) obj;
				if(!name.equalsIgnoreCase("npcId"))
				{
					if(!values.equals(""))
						values += ", ";
					values += name + " = '" + set.get(name) + "'";
				}
			}
			if(!values.equals(""))
				values += ", last_editor = '" + editor.getName() + "'";
			query = "UPDATE npc SET " + values + " WHERE id = ?";
			statement = con.prepareStatement(query);
			statement.setInt(1, npc.getInteger("npcId"));
			statement.execute();
		}
		catch(Exception e1)
		{
			// problem with storing spawn
			_log.warning("npc data couldnt be stored in db, query is :" + query + " : " + e1);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public static boolean isInitialized()
	{
		return _initialized;
	}

	public static void replaceTemplate(L2NpcTemplate npc)
	{
		_npcs.put(npc.npcId, npc);
		_npcsNames.put(npc.name.toLowerCase(), npc);
	}

	public static L2NpcTemplate getTemplate(int id)
	{
		return _npcs.get(id);
	}

	public static L2NpcTemplate getTemplateByName(String name)
	{
		return _npcsNames.get(name.toLowerCase());
	}

	public static ArrayList<L2NpcTemplate> getAllOfLevel(int lvl)
	{
		return _npcsByLevel[lvl];
	}

	public static L2NpcTemplate[] getAll()
	{
		return _npcs.values().toArray(new L2NpcTemplate[_npcs.size()]);
	}

	public void FillDropList()
	{
		for(L2NpcTemplate npc : _npcs.values())
			InfoCache.addToDroplistCache(npc.npcId, generateDroplist(npc));
		_log.info("Players droplist was cached");
	}

	private static NumberFormat df = NumberFormat.getPercentInstance();
	static
	{
		df.setMaximumFractionDigits(4);
	}

	public static String generateDroplist(L2NpcTemplate npc)
	{
		return generateDroplist(npc, 1.0f, 1.0f, 1.0f);
	}

	public static String generateDroplist(L2NpcTemplate npc, float rateDropMod, float rateSpoilMod, float rateAdenaMod)
	{
		StringBuffer tmp = new StringBuffer();
		tmp.append("<html><body><center><font color=\"LEVEL\">").append(npc.name).append(", Id: ").append(npc.getNpcId()).append("</font>");
		if(npc.isDropHerbs)
			tmp.append("<br1><font color=\"00FF00\">herbs</font>");
		tmp.append("</center><table><tr><td></td></tr>");
		boolean emptylist = false;
		if(npc.getDropData() != null)
		{
			double worth = 0;
			for(L2DropGroup g : npc.getDropData().getNormal())
			{
				double GCHANCE = (double) g.getChance() * (double) (npc.isRaid ? Config.RATE_DROP_RAIDBOSS : Config.RATE_DROP_ITEMS) * rateDropMod / L2Drop.MAX_CHANCE;
				int dropmult = 1;
				if(GCHANCE > 1)
					if(g.fixedQty())
						GCHANCE = 1;
					else if(GCHANCE % 1 == 0)
					{
						dropmult = (int) GCHANCE;
						GCHANCE = 1;
					}
					else
					{
						dropmult = (int) Math.ceil(GCHANCE);
						GCHANCE = GCHANCE / dropmult;
					}
				if(g.isAdena())
					GCHANCE = g.getChance() / 1000000.;
				tmp.append("</table><br><center>Group chance: ").append(df.format(GCHANCE)).append("</center><table width=100%>");
				for(L2DropData d : g.getDropItems(false))
				{
					float qty = (d.getMinDrop() + d.getMaxDrop()) / 2f;
					if(!g.isAdena())
					{
						qty *= dropmult;
						tmp.append("<tr><td width=80%>").append(d.getItem().getName().replaceAll("Recipe:", "R:").replaceAll("Scroll: Enchant", "Enchant").replaceAll("Compressed Package", "CP")).append("</td><td width=10%>");
						tmp.append(Math.round(qty)).append("</td><td width=10%>");
						tmp.append(df.format(GCHANCE * d.getChance() / L2Drop.MAX_CHANCE)).append("</td></tr>");
						worth += d.getItem().getReferencePrice() * 0.5 * qty * GCHANCE * d.getChance() / L2Drop.MAX_CHANCE;
					}
					else
					{
						tmp.append("<tr><td width=80%>").append(d.getItem().getName()).append("</td><td width=10%>");
						tmp.append(Math.min((int) (qty * Config.RATE_DROP_ADENA * rateAdenaMod), 9999999));
						tmp.append("</td><td width=10%>").append(df.format(GCHANCE)).append("</td></tr>");
						worth += qty * Config.RATE_DROP_ADENA * rateAdenaMod * GCHANCE;
					}
				}
			}
			if(npc.getDropData().getSpoil().size() > 0)
			{
				tmp.append("</table><center>Spoil:</center><table width=100%>");
				for(L2DropGroup g : npc.getDropData().getSpoil())
					for(L2DropData d : g.getDropItems(false))
					{
						double GCHANCE = (double) g.getChance() * (double) (npc.isRaid ? Config.RATE_DROP_RAIDBOSS : Config.RATE_DROP_SPOIL) * rateSpoilMod / L2Drop.MAX_CHANCE;
						int dropmult = 1;
						if(GCHANCE > 1)
							if(g.fixedQty())
								GCHANCE = 1;
							else if(GCHANCE % 1 == 0)
							{
								dropmult = (int) GCHANCE;
								GCHANCE = 1;
							}
							else
							{
								dropmult = (int) Math.ceil(GCHANCE);
								GCHANCE = GCHANCE / dropmult;
							}
						float qty = (d.getMinDrop() + d.getMaxDrop()) * dropmult / 2f;
						tmp.append("<tr><td width=80%>").append(d.getItem().getName().replaceAll("Recipe:", "R:").replaceAll("Scroll: Enchant", "Enchant").replaceAll("Compressed Package", "CP")).append("</td><td width=10%>");
						tmp.append(Math.round(qty)).append("</td><td width=10%>");
						tmp.append(df.format(GCHANCE)).append("</td></tr>");
					}
			}
			if(worth > 0)
				tmp.append("</table><center>Worth per kill: ").append((int) worth).append("</center><table>");
		}
		else
			emptylist = true;

		tmp.append("</table>");
		if(emptylist)
			tmp.append("<center>Droplist is empty</center>");
		tmp.append("</body></html>");
		return tmp.toString();
	}

	public void FillKillCount() throws SQLException
	{
		HashMap<Integer, Integer> tmp = new HashMap<Integer, Integer>();
		ThreadConnection con = L2DatabaseFactory.getInstance().getConnection();
		FiltredPreparedStatement statement = null;
		ResultSet list = null;
		try
		{
			statement = con.prepareStatement("SELECT npc_id,COALESCE(SUM(`count`),0) AS `count` FROM `killcount` group by `npc_id`");
			list = statement.executeQuery();
			while(list.next())
				tmp.put(list.getInt("npc_id"), list.getInt("count"));
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, list);
		}

		for(L2NpcTemplate npc : _npcs.values())
			npc.killscount = tmp.get(npc.npcId) == null ? 0 : tmp.get(npc.npcId);

		_log.info("Kills counter loaded");
	}

	@SuppressWarnings("unchecked")
	public void applyServerSideTitle()
	{
		Class L2MonsterInstance;
		if(Config.SERVER_SIDE_NPC_TITLE_WITH_LVL)
			try
			{
				L2MonsterInstance = Class.forName("l2r.gameserver.model.instances.L2MonsterInstance");
				for(L2NpcTemplate npc : _npcs.values())
				{
					String title = "";
					Class _this = null;
					try
					{
						_this = Class.forName("l2r.gameserver.model.instances." + npc.type + "Instance");
					}
					catch(ClassNotFoundException e)
					{
						Script sc = Scripts.getInstance().getClasses().get("npc.model.instances." + npc.type + "Instance");
						if(sc != null)
							_this = sc.getRawClass();
						else
							continue;
					}
					if(L2MonsterInstance.isAssignableFrom(_this) && !npc.type.equalsIgnoreCase("L2TamedBeast"))
					{
						title = "L" + npc.level;
						if(npc.aggroRange != 0 || npc.factionRange != 0)
							title += " " + (npc.aggroRange != 0 ? "A" : "") + (npc.factionRange != 0 ? "S" : "");
						title += " ";
					}
					npc.title = title + npc.title;
				}
			}
			catch(ClassNotFoundException e1)
			{
				e1.printStackTrace();
			}
	}
}
