package l2r.gameserver.taskmanager;

import java.util.Stack;
import java.util.concurrent.ConcurrentLinkedQueue;

import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.model.L2Character;
import l2r.util.Util;

public class DecayTaskManager
{
	private static final ConcurrentLinkedQueue<DecayTask> _decayTasks = new ConcurrentLinkedQueue<DecayTask>();
	private static final Stack<DecayTask> _pool = new Stack<DecayTask>();
	private static DecayTaskManager _instance;

	public DecayTaskManager()
	{
		for(int i = 0; i < 50; i++)
			_pool.push(new DecayTask(null, 0));
		ThreadPoolManager.getInstance().scheduleAi(new DecayScheduler(), 1000, false);
	}

	public static DecayTaskManager getInstance()
	{
		if(_instance == null)
			_instance = new DecayTaskManager();

		return _instance;
	}

	public void addDecayTask(L2Character actor)
	{
		_decayTasks.add(getContainer(actor, System.currentTimeMillis() + 8500));
	}

	public void addDecayTask(L2Character actor, long interval)
	{
		_decayTasks.add(getContainer(actor, System.currentTimeMillis() + interval));
	}

	public void cancelDecayTask(L2Character actor)
	{
		for(DecayTask container : _decayTasks)
			if(container._actor != null && container._actor.equals(actor))
			{
				cancelDecayTask(container);
				return;
			}
	}

	public void cancelDecayTask(DecayTask container)
	{
		_decayTasks.remove(container);
		freeContainer(container);
	}

	private DecayTask getContainer(L2Character actor, long interval)
	{
		synchronized (_pool)
		{
			if(!_pool.isEmpty())
			{
				DecayTask t = _pool.pop();
				t._actor = actor;
				t._endtime = interval;
				return t;
			}
		}
		return new DecayTask(actor, interval);
	}

	private void freeContainer(DecayTask t)
	{
		t._actor = null;
		_pool.push(t);
	}

	public class DecayScheduler implements Runnable
	{
		public void run()
		{
			try
			{
				Long current = System.currentTimeMillis();
				synchronized (_decayTasks)
				{
					for(DecayTask container : _decayTasks)
						if(current > container._endtime)
						{
							if(container._actor != null)
								container._actor.onDecay();
							_decayTasks.remove(container);
							freeContainer(container);
						}
				}
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
			finally
			{
				ThreadPoolManager.getInstance().scheduleAi(new DecayScheduler(), 1000, false);
			}
		}
	}

	@Override
	public String toString()
	{
		StringBuffer sb = new StringBuffer("============= DecayTask Manager Report ============\n\r");
		sb.append("Tasks count: ").append(_decayTasks.size()).append("\n\r");
		sb.append("Tasks dump:\n\r");

		long current = System.currentTimeMillis();
		for(DecayTask container : _decayTasks)
		{
			sb.append("Class/Name: ").append(container.getClass().getSimpleName()).append('/').append(container._actor.getName());
			sb.append(" decay timer: ").append(Util.formatTime(container._endtime - current)).append("\n\r");
		}

		return sb.toString();
	}

	private class DecayTask
	{
		public L2Character _actor;
		public long _endtime;

		DecayTask(L2Character actor, long delay)
		{
			_actor = actor;
			_endtime = delay;
		}
	}
}