package l2r.gameserver.templates;

import l2r.gameserver.model.L2Skill;
import l2r.gameserver.model.instances.L2ItemInstance.ItemClass;
import l2r.gameserver.skills.funcs.FuncTemplate;
import l2r.gameserver.tables.SkillTable;

/**
 * This class contains all informations concerning the item (weapon, armor, etc).<BR>
 * Mother class of :
 * <LI>L2Armor</LI>
 * <LI>L2EtcItem</LI>
 * <LI>L2Weapon</LI>
 */
public abstract class L2Item
{
	/**
	 * Pc Cafe Bang Points item id. Используется на корейских серверах, но английский клиент в состоянии
	 * поддерживать даный функционал.
	 */
	public static final short ITEM_ID_PC_BANG_POINTS = -100;

	/**
	 * Item ID для клановой репутации
	 */
	public static final short ITEM_ID_CLAN_REPUTATION_SCORE = -200;

	public static final int TYPE1_WEAPON_RING_EARRING_NECKLACE = 0;
	public static final int TYPE1_SHIELD_ARMOR = 1;
	public static final int TYPE1_ITEM_QUESTITEM_ADENA = 4;

	public static final byte TYPE2_WEAPON = 0;
	public static final byte TYPE2_SHIELD_ARMOR = 1;
	public static final byte TYPE2_ACCESSORY = 2;
	public static final byte TYPE2_QUEST = 3;
	public static final byte TYPE2_MONEY = 4;
	public static final byte TYPE2_OTHER = 5;
	public static final byte TYPE2_PET_WOLF = 6;
	public static final byte TYPE2_PET_HATCHLING = 7;
	public static final byte TYPE2_PET_STRIDER = 8;
	public static final byte TYPE2_NODROP = 9;
	public static final byte TYPE2_PET_GWOLF = 10;

	public static final int SLOT_NONE = 0x0000;
	public static final int SLOT_UNDERWEAR = 0x0001;
	public static final int SLOT_CLOAK = 0x0003; //TODO:?????
	public static final int SLOT_R_EAR = 0x0002;
	public static final int SLOT_L_EAR = 0x0004;
	public static final int SLOT_NECK = 0x0008;
	public static final int SLOT_R_FINGER = 0x0010;
	public static final int SLOT_L_FINGER = 0x0020;
	public static final int SLOT_HEAD = 0x0040;
	public static final int SLOT_R_HAND = 0x0080;
	public static final int SLOT_L_HAND = 0x0100;
	public static final int SLOT_GLOVES = 0x0200;
	public static final int SLOT_CHEST = 0x0400;
	public static final int SLOT_LEGS = 0x0800;
	public static final int SLOT_FEET = 0x1000;
	public static final int SLOT_BACK = 0x2000;
	public static final int SLOT_LR_HAND = 0x4000;
	public static final int SLOT_FULL_ARMOR = 0x8000;
	public static final int SLOT_HAIR = 0x010000;
	public static final int SLOT_WOLF = 0x020000;
	public static final int SLOT_HATCHLING = 0x100000;
	public static final int SLOT_STRIDER = 0x200000;
	public static final int SLOT_BABYPET = 0x400000;
	public static final int SLOT_FACE = 0x040000;
	public static final int SLOT_DHAIR = 0x080000;
	public static final int SLOT_FORMAL_WEAR = 0x100000;
	public static final int SLOT_HAIRALL = 0x80000;

	public static final byte MATERIAL_STEEL = 0x00;
	public static final byte MATERIAL_FINE_STEEL = 0x01;
	public static final byte MATERIAL_BLOOD_STEEL = 0x02;
	public static final byte MATERIAL_BRONZE = 0x03;
	public static final byte MATERIAL_SILVER = 0x04;
	public static final byte MATERIAL_GOLD = 0x05;
	public static final byte MATERIAL_MITHRIL = 0x06;
	public static final byte MATERIAL_ORIHARUKON = 0x07;
	public static final byte MATERIAL_PAPER = 0x08;
	public static final byte MATERIAL_WOOD = 0x09;
	public static final byte MATERIAL_CLOTH = 0x0a;
	public static final byte MATERIAL_LEATHER = 0x0b;
	public static final byte MATERIAL_BONE = 0x0c;
	public static final byte MATERIAL_HORN = 0x0d;
	public static final byte MATERIAL_DAMASCUS = 0x0e;
	public static final byte MATERIAL_ADAMANTAITE = 0x0f;
	public static final byte MATERIAL_CHRYSOLITE = 0x10;
	public static final byte MATERIAL_CRYSTAL = 0x11;
	public static final byte MATERIAL_LIQUID = 0x12;
	public static final byte MATERIAL_SCALE_OF_DRAGON = 0x13;
	public static final byte MATERIAL_DYESTUFF = 0x14;
	public static final byte MATERIAL_COBWEB = 0x15;
	public static final byte MATERIAL_SEED = 0x16;
	public static final byte MATERIAL_FISH = 0x17;

	public static final byte CRYSTAL_NONE = 0x00;
	public static final byte CRYSTAL_D = 0x01;
	public static final byte CRYSTAL_C = 0x02;
	public static final byte CRYSTAL_B = 0x03;
	public static final byte CRYSTAL_A = 0x04;
	public static final byte CRYSTAL_S = 0x05;

	private final short _itemId;
	private final ItemClass _class;
	protected final String _name;
	protected final String _icon;
	private final int _type1; // needed for item list (inventory)
	private final int _type2; // different lists for armor, weapon, etc
	private final int _weight;
	private final boolean _crystallizable;
	private final boolean _stackable;
	private final byte _crystalType; // default to none-grade
	private final int _durability;
	private final int _bodyPart;
	private final int _referencePrice;
	private final short _crystalCount;
	private final boolean _sellable;
	private final boolean _dropable;
	private final boolean _tradeable;
	private final boolean _destroyable;

	protected L2Skill _skillOnAction;
	protected boolean _offensive;
	protected byte _chance;
	protected boolean _skillOnCritNotCast;

	private L2Skill[] _skills;

	@SuppressWarnings("unchecked")
	public final Enum type;

	protected FuncTemplate[] _funcTemplates;

	/**
	 * Constructor of the L2Item that fill class variables.<BR><BR>
	 * <U><I>Variables filled :</I></U><BR>
	 * <LI>type</LI>
	 * <LI>_itemId</LI>
	 * <LI>_name</LI>
	 * <LI>_type1 & _type2</LI>
	 * <LI>_weight</LI>
	 * <LI>_crystallizable</LI>
	 * <LI>_stackable</LI>
	 * <LI>_materialType & _crystalType & _crystlaCount</LI>
	 * <LI>_durability</LI>
	 * <LI>_bodypart</LI>
	 * <LI>_referencePrice</LI>
	 * <LI>_sellable</LI>
	 * @param type : Enum designating the type of the item
	 * @param set : StatsSet corresponding to a set of couples (key,value) for description of the item
	 */
	@SuppressWarnings("unchecked")
	protected L2Item(final Enum type, final StatsSet set)
	{
		this.type = type;
		_itemId = set.getShort("item_id");
		_class = ItemClass.valueOf(set.getString("class"));
		_name = set.getString("name");
		_icon = set.getString("icon");
		_type1 = set.getInteger("type1"); // needed for item list (inventory)
		_type2 = set.getInteger("type2"); // different lists for armor, weapon, etc
		_weight = set.getInteger("weight");
		_crystallizable = set.getBool("crystallizable");
		_stackable = set.getBool("stackable", false);
		_crystalType = (byte) set.getInteger("crystal_type", CRYSTAL_NONE); // default to none-grade
		_durability = set.getInteger("durability", -1);
		_bodyPart = set.getInteger("bodypart");
		_referencePrice = set.getInteger("price");
		_crystalCount = set.getShort("crystal_count", (short) 0);
		_sellable = set.getBool("sellable", true);
		_dropable = set.getBool("dropable", true);
		_destroyable = set.getBool("destroyable", true);
		_tradeable = set.getBool("tradeable", true);
		L2Skill skill = SkillTable.getInstance().getInfo(set.getInteger("skill_id", 0), set.getInteger("skill_level", 1));
		if(skill != null)
			attachSkill(skill);
	}

	/**
	 * Returns the itemType.
	 * @return Enum
	 */
	@SuppressWarnings("unchecked")
	public Enum getItemType()
	{
		return type;
	}

	/**
	 * Returns the durability of th item
	 * @return int
	 */
	public final int getDurability()
	{
		return _durability;
	}

	/**
	 * Returns the ID of the item
	 * @return int
	 */
	public final short getItemId()
	{
		return _itemId;
	}

	public abstract int getItemMask();

	/**
	 * Returns the type 2 of the item
	 * @return int
	 */
	public final int getType2()
	{
		return _type2;
	}

	/**
	 * Returns the weight of the item
	 * @return int
	 */
	public final int getWeight()
	{
		return _weight;
	}

	/**
	 * Returns if the item is crystallizable
	 * @return boolean
	 */
	public final boolean isCrystallizable()
	{
		return _crystallizable && !isStackable() && !(getCrystalType() == L2Item.CRYSTAL_NONE) && !(getCrystalCount() <= 0);
	}

	/**
	 * Return the type of crystal if item is crystallizable
	 * @return int
	 */
	public final byte getCrystalType()
	{
		return _crystalType;
	}

	/**
	 * Returns the grade of the item.<BR><BR>
	 * <U><I>Concept :</I></U><BR>
	 * In fact, this fucntion returns the type of crystal of the item.
	 * @return int
	 */
	public final int getItemGrade()
	{
		return getCrystalType();
	}

	/**
	 * Returns the quantity of crystals for crystallization
	 * @return int
	 */
	public final int getCrystalCount()
	{
		return _crystalCount;
	}

	/**
	 * Returns the name of the item
	 * @return String
	 */
	public final String getName()
	{
		return _name;
	}

	/**
	 * Return the part of the body used with the item.
	 * @return int
	 */
	public final int getBodyPart()
	{
		return _bodyPart;
	}

	/**
	 * Returns the type 1 of the item
	 * @return int
	 */
	public final int getType1()
	{
		return _type1;
	}

	/**
	 * Returns if the item is stackable
	 * @return boolean
	 */
	public final boolean isStackable()
	{
		return _stackable;
	}

	/**
	 * Returns the price of reference of the item
	 * @return int
	 */
	public final int getReferencePrice()
	{
		return _referencePrice;
	}

	/**
	 * Returns if the item can be sold
	 * @return boolean
	 */
	public final boolean isSellable()
	{
		return _sellable;
	}

	/**
	 * Returns if item is for hatchling
	 * @return boolean
	 */
	public boolean isForHatchling()
	{
		return _type2 == TYPE2_PET_HATCHLING;
	}

	/**
	 * Returns if item is for strider
	 * @return boolean
	 */
	public boolean isForStrider()
	{
		return _type2 == TYPE2_PET_STRIDER;
	}

	/**
	 * Returns if item is for wolf
	 * @return boolean
	 */
	public boolean isForWolf()
	{
		return _type2 == TYPE2_PET_WOLF;
	}
	
	public boolean isForGWolf()
	{
		return _type2 == TYPE2_PET_GWOLF;
	}

	public boolean isTradeable()
	{
		return _tradeable;
	}

	public boolean isDestroyable()
	{
		return _destroyable;
	}

	public boolean isDropable()
	{
		return _dropable;
	}

	public boolean isForPet()
	{
		return _type2 == TYPE2_PET_HATCHLING || _type2 == TYPE2_PET_WOLF || _type2 == TYPE2_PET_STRIDER || _type2 == TYPE2_PET_GWOLF;
	}

	public void attachSkillOnAction(L2Skill skill, byte chance, boolean onCritNotCast)
	{
		_skillOnAction = skill;
		_offensive = skill.isOffensive();
		_chance = chance;
		_skillOnCritNotCast = onCritNotCast;
	}

	/**
	 * Add the FuncTemplate f to the list of functions used with the item
	 * @param f : FuncTemplate to add
	 */
	public void attachFunction(FuncTemplate f)
	{
		if(_funcTemplates == null)
			_funcTemplates = new FuncTemplate[] { f };
		else
		{
			int len = _funcTemplates.length;
			FuncTemplate[] tmp = new FuncTemplate[len + 1];
			System.arraycopy(_funcTemplates, 0, tmp, 0, len);
			tmp[len] = f;
			_funcTemplates = tmp;
		}
	}

	public FuncTemplate[] getAttachedFuncs()
	{
		return _funcTemplates;
	}

	/**
	 * Add the L2Skill skill to the list of skills generated by the item
	 * @param skill : L2Skill
	 */
	public void attachSkill(L2Skill skill)
	{
		if(_skills == null)
			_skills = new L2Skill[] { skill };
		else
		{
			int len = _skills.length;
			L2Skill[] tmp = new L2Skill[len + 1];
			System.arraycopy(_skills, 0, tmp, 0, len);
			tmp[len] = skill;
			_skills = tmp;
		}
	}

	public L2Skill[] getAttachedSkills()
	{
		return _skills;
	}

	public L2Skill getFirstSkill()
	{
		if(_skills != null && _skills.length > 0)
			return _skills[0];
		return null;
	}

	/**
	 * Returns the name of the item
	 * @return String
	 */
	@Override
	public String toString()
	{
		return _name;
	}

	/**
	 * Определяет призрачный предмет или нет
	 * @return true, если предмет призрачный
	 */
	public boolean isShadowItem()
	{
		return _durability != -1;
	}

	public boolean isAltSeed()
	{
		return _name.contains("Alternative");
	}

	public ItemClass getItemClass()
	{
		return _class;
	}

	/**
	 * Является ли вещь аденой или камнем печати
	 */
	public boolean isAdena()
	{
		return _itemId == 57 || _itemId == 6360 || _itemId == 6361 || _itemId == 6362;
	}

	public boolean isEquipment()
	{
		return _bodyPart != 0;
	}

   public String getIcon()
	{
		return _icon;
	}
}