package l2r.gameserver.webserver;

import java.util.TreeMap;

import l2r.Config;
import l2r.gameserver.GameServer;
import l2r.gameserver.Shutdown;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.L2World;
import l2r.gameserver.tables.FakePlayersTable;
import l2r.gameserver.tables.GmListTable;
import l2r.util.Util;

/**
 * Это затычка для минимальной функциональности вебсервера
 * @author Abaddon
 */
abstract class PageParser
{
	public static String parse(String s)
	{
		// Количество игроков в мире
		if(s.contains("%online%"))
			s = s.replaceAll("%online%", String.valueOf(L2World.getAllPlayersCount() + FakePlayersTable.getFakePlayersCount()));

		// Количество ботов в мире
		if(s.contains("%bots%"))
			s = s.replaceAll("%bots%", String.valueOf(L2World.getAllBotCount()));

		// Количество ГМов
		if(s.contains("%gms%"))
			s = s.replaceAll("%gms%", String.valueOf(GmListTable.getAllGMs().size()));

		// Количество оффлайн торговцев
		if(s.contains("%off%"))
			s = s.replaceAll("%off%", String.valueOf(L2World.getAllOfflineCount()));

		// Список имен игроков онлайн
		if(s.contains("%players_list%"))
		{
			String playersList = "";
			final int maxCols = 4; // Количество столбцов для списка онлайн игроков
			int cols = 0;
			playersList += "<table id=\"tbl-pl-list\">";

			TreeMap<String, String> playerNames = new TreeMap<String, String>();

			for(L2Player player : L2World.getAllPlayers())
			{
				if(player == null || player.isInvisible() && player.isGM())
					continue;
				playerNames.put(player.getName(), null);
			}

			for(String name : FakePlayersTable.getActiveFakePlayers())
				playerNames.put(name, null);

			for(String name : playerNames.keySet())
			{
				if(cols == 0)
					playersList += "<tr>";
				playersList += "<td>" + name + "</td>";
				if(++cols == maxCols)
				{
					cols = 0;
					playersList += "</tr>";
				}
			}

			if(cols > 0)
				playersList += "</tr>";

			playersList += "</table>";
			s = s.replaceAll("%players_list%", playersList);
		}

		// Информация о текущей ревизии сервера
		if(s.contains("%revision%"))
			s = s.replaceAll("%revision%", Config.SERVER_VERSION.equalsIgnoreCase("${l2r.revision}") ? Config.SERVER_VERSION_UNSUPPORTED : Config.SERVER_VERSION);

		// Информация о состоянии памяти
		if(s.contains("%memory%"))
		{
			long freeMem = (Runtime.getRuntime().maxMemory() - Runtime.getRuntime().totalMemory() + Runtime.getRuntime().freeMemory()) / 1024 / 1024;
			long totalMem = Runtime.getRuntime().maxMemory() / 1024 / 1024;
			String memory = freeMem + " Mb / " + totalMem + " Mb";
			s = s.replaceAll("%memory%", memory);
		}

		// Время до рестарта
		int mtc = Shutdown.getInstance().getSeconds();
		if(mtc > 0)
			s = s.replaceAll("%countdown%", Util.formatTime(mtc));
		else
			s = s.replaceAll("%countdown%", "Restart task not launched");

		// Аптайм сервера
		s = s.replaceAll("%uptime%", Util.formatTime(GameServer.uptime()));
		return s;
	}
}