package l2r.util;

import java.io.Serializable;

import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2World;
import l2r.gameserver.geodata.network.geoclient.GeoClient;

@SuppressWarnings("serial")
public class Location implements Serializable
{
	protected int x;
	protected int y;
	protected int z;
	/** Heading */
	public int h = 0;
	public int r = 0;

	public Location()
	{
		x = 0;
		y = 0;
		z = 0;
		h = 0;
	}

	public Location(int[] point)
	{
		h = 0;
		x = point[0];
		y = point[1];
		z = point[2];
		h = point[3];
	}

	/**
	 * Позиция (x, y, z)
	 */
	public Location(int locX, int locY, int locZ)
	{
		x = locX;
		y = locY;
		z = locZ;
	}

	/**
	 * Позиция (x, y, z, heading)
	 */
	public Location(int locX, int locY, int locZ, int heading)
	{
		x = locX;
		y = locY;
		z = locZ;
		h = heading;
	}

	public Location(L2Object obj)
	{
		x = obj.getX();
		y = obj.getY();
		z = obj.getZ();
		h = obj.getHeading();
	}

	public Location(String s) throws java.lang.IllegalArgumentException
	{
		h = 0;
		String xyzh[] = s.replaceAll(",", " ").replaceAll(";", " ").replaceAll("  ", " ").trim().split(" ");
		if(xyzh.length < 3)
		{
			throw new IllegalArgumentException((new StringBuilder()).append("Can't parse location from string: ").append(s).toString());
		}
		else
		{
			x = Integer.parseInt(xyzh[0]);
			y = Integer.parseInt(xyzh[1]);
			z = Integer.parseInt(xyzh[2]);
			h = xyzh.length >= 4 ? Integer.parseInt(xyzh[3]) : 0;
		}
	}

	public boolean equals(Location loc)
	{
		return loc.getX() == x && loc.getY() == y && loc.getZ() == z;
	}

	public boolean equals(int _x, int _y, int _z)
	{
		return _x == x && _y == y && _z == z;
	}

	public Location changeZ(int zDiff)
	{
		z += zDiff;
		return this;
	}

	public Location setX(int x)
	{
		this.x = x;
		return this;
	}

	public Location setY(int y)
	{
		this.y = y;
		return this;
	}

	public Location setZ(int z)
	{
		this.z = z;
		return this;
	}

	public Location setH(int h)
	{
		this.h = h;
		return this;
	}

	public Location setR(int r)
	{
		this.r = r;
		return this;
	}

	public void set(Location l)
	{
		x = l.getX();
		y = l.getY();
		z = l.getZ();
	}

	public void set(int _x, int _y, int _z)
	{
		x = _x;
		y = _y;
		z = _z;
	}

	public void set(int _x, int _y, int _z, int _h)
	{
		x = _x;
		y = _y;
		z = _z;
		h = _h;
	}

	public int getX()
	{
		return x;
	}

	public int getY()
	{
		return y;
	}

	public int getZ()
	{
		return z;
	}

	public int getReflect()
	{
		return r;
	}

	public int getHeading()
	{
		return h;
	}

	@Override
	public final String toString()
	{
		return "Coords(" + x + "," + y + "," + z + "," + h + ")";
	}

	public static Location getAroundPosition(L2Object obj, L2Object obj2, int radius_min, int radius_max, int max_geo_checks)
	{
		Location pos = new Location(obj);
		if(radius_min < 0)
			radius_min = 0;
		if(radius_max < 0)
			radius_max = 0;
		float col_radius = obj.getColRadius() + obj2.getColRadius();
		int min_angle = 0;
		int max_angle = 360;
		if(!obj.equals(obj2))
		{
			double perfect_angle = Util.calculateAngleFrom(obj, obj2);
			min_angle = (int) perfect_angle - 225;
			max_angle = (int) perfect_angle + 135;
		}
		int x;
		int y;
		int z;
		do
		{
			int randomRadius = Rnd.get(radius_min, radius_max);
			int randomAngle = Rnd.get(min_angle, max_angle);
			x = pos.x + (int) ((double) (col_radius + (float) randomRadius) * java.lang.Math.cos(randomAngle));
			y = pos.y + (int) ((double) (col_radius + (float) randomRadius) * java.lang.Math.sin(randomAngle));
			z = pos.z;
			if(max_geo_checks <= 0)
				break;
			z = GeoClient.getInstance().getHeight(x, y, z);
			if(java.lang.Math.abs(pos.z - z) < 256 && GeoClient.getInstance().getNSWE(x, y, z) == 15)
				break;
			max_geo_checks--;
		} while(true);
		pos.x = x;
		pos.y = y;
		pos.z = z;
		return pos;
	}

	public Location clone()
	{
		return new Location(x, y, z, h);
	}

	public Location rnd(int min, int max)
	{
		Location loc = Rnd.coordsRandomize(this, min, max);
		this.x = loc.x;
		this.y = loc.y;
		return this;
	}

	public Location world2geo()
	{
		x = (x - L2World.MAP_MIN_X) >> 4;
		y = (y - L2World.MAP_MIN_Y) >> 4;
		return this;
	}

	public Location geo2world()
	{
		// размер одного блока 16*16 точек, +8*+8 это его средина
		x = (x << 4) + L2World.MAP_MIN_X + 8;
		y = (y << 4) + L2World.MAP_MIN_Y + 8;
		return this;
	}

	public double distance(Location loc)
	{
		return distance(loc.x, loc.y);
	}

	public double distance(int _x, int _y)
	{
		int dx = x - _x;
		int dy = y - _y;
		return Math.sqrt(dx * dx + dy * dy);
	}

	public double distance3D(Location loc)
	{
		return distance3D(loc.x, loc.y, loc.z);
	}

	public double distance3D(int _x, int _y, int _z)
	{
		int dx = x - _x;
		int dy = y - _y;
		int dz = z - _z;
		return Math.sqrt(dx * dx + dy * dy + dz * dz);
	}

	public final String toXYZString()
	{
		return x + "," + y + "," + z;
	}

	public Location correctGeoZ()
	{
		z = GeoClient.getInstance().getHeight(x, y, z);
		return this;
	}
}