package l2r.util;

import java.io.File;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Locale;
import java.util.Random;
import java.util.StringTokenizer;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import l2r.Config;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.model.L2Drop;
import l2r.gameserver.model.L2Object;
import l2r.gameserver.model.L2Player;

public class Util
{
	private static Logger _log = Logger.getLogger(Util.class.getName());

	static final String PATTERN = "0.0000000000E00";
	static final DecimalFormat df;

	/**
	 * Форматтер для адены.<br>
	 * Locale.KOREA заставляет его фортматировать через ",".<br>
	 * Locale.FRANCE форматирует через " "<br>
	 * Для форматирования через "." убрать с аргументов Locale.FRANCE
	 */
	private static NumberFormat adenaFormatter;

	static
	{
		adenaFormatter = NumberFormat.getIntegerInstance(Locale.FRANCE);
		df = (DecimalFormat) NumberFormat.getNumberInstance(Locale.ENGLISH);
		df.applyPattern(PATTERN);
		df.setPositivePrefix("+");
	}

	public static boolean isInternalIP(String ipAddress)
	{
		if(Config.INTERNAL_NETLIST != null)
			return Config.INTERNAL_NETLIST.isIpInNets(ipAddress);

		for(String s : Config.INTERNAL_IP)
			if(checkIfIpInRange(ipAddress, s))
				return true;
		return false;
	}

	static boolean checkIfIpInRange(String ip, String ipRange)
	{
		//DATA FIELD
		//Ip format. 110.20.30.40 - 50.50.30.40
		int userIp1 = -1;
		int userIp2 = -1;
		int userIp3 = -1;
		int userIp4 = -1;
		String firstIp;
		String lastIp;
		//Data field end.

		ip = ip.replace(".", ",");
		for(String s : ip.split(","))
			if(userIp1 == -1)
				userIp1 = Integer.parseInt(s);
			else if(userIp2 == -1)
				userIp2 = Integer.parseInt(s);
			else if(userIp3 == -1)
				userIp3 = Integer.parseInt(s);
			else
				userIp4 = Integer.parseInt(s);

		int ipMin1 = -1;
		int ipMin2 = -1;
		int ipMin3 = -1;
		int ipMin4 = -1; //IP values for min ip
		int ipMax1 = -1;
		int ipMax2 = -1;
		int ipMax3 = -1;
		int ipMax4 = -1; //Ip values for max ip

		StringTokenizer st = new StringTokenizer(ipRange, "-"); // Try to split them by "-" symbol

		if(Config.DEBUG)
			System.out.println("Tokens in string " + ipRange + ": " + st.countTokens());

		if(st.countTokens() == 2)
		{
			firstIp = st.nextToken(); //get our first ip string
			lastIp = st.nextToken(); //get out second ip string.

			firstIp = firstIp.replace(".", ",");
			lastIp = lastIp.replace(".", ",");

			//Set our minimum ip
			for(String s1 : firstIp.split(","))
				if(ipMin1 == -1)
					ipMin1 = Integer.parseInt(s1);
				else if(ipMin2 == -1)
					ipMin2 = Integer.parseInt(s1);
				else if(ipMin3 == -1)
					ipMin3 = Integer.parseInt(s1);
				else
					ipMin4 = Integer.parseInt(s1);

			//set our maximum ip
			for(String s2 : lastIp.split(","))
				if(ipMax1 == -1)
					ipMax1 = Integer.parseInt(s2);
				else if(ipMax2 == -1)
					ipMax2 = Integer.parseInt(s2);
				else if(ipMax3 == -1)
					ipMax3 = Integer.parseInt(s2);
				else
					ipMax4 = Integer.parseInt(s2);

			//Now we are making some checks with our ips.
			if(userIp1 > ipMin1 && userIp1 < ipMax1)
				return true; // it's internal
			else if(userIp1 < ipMin1 || userIp1 > ipMax1)
				return false; // it's external
			else if(userIp1 == ipMin1 && userIp1 != ipMax1)
			{
				if(userIp2 > ipMin2)
					return true;
				else if(userIp2 < ipMin2)
					return false;
				else if(userIp3 > ipMin3)
					return true;
				else if(userIp3 < ipMin3)
					return false;
				else
					return userIp4 >= ipMin4;
			}
			else if(userIp1 != ipMin1 && userIp1 == ipMax1)
			{
				if(userIp2 < ipMax2)
					return true;
				else if(userIp2 > ipMax2)
					return false;
				else if(userIp3 < ipMax3)
					return true;
				else if(userIp3 > ipMax3)
					return false;
				else
					return userIp4 <= ipMax4;
			}
			else if(userIp2 > ipMin2 && userIp2 < ipMax2)
				return true; // it's internal
			else if(userIp2 < ipMin2 || userIp2 > ipMax2)
				return false; // it's external
			else if(userIp2 == ipMin2 && userIp2 != ipMax2)
			{
				if(userIp3 > ipMin3)
					return true;
				else if(userIp3 < ipMin3)
					return false;
				else
					return userIp4 >= ipMin4;
			}
			else if(userIp2 != ipMin2 && userIp2 == ipMax2)
			{
				if(userIp3 < ipMax3)
					return true;
				else if(userIp3 > ipMax3)
					return false;
				else
					return userIp4 <= ipMax4;
			}
			else if(userIp3 > ipMin3 && userIp3 < ipMax3)
				return true; // it's internal
			else if(userIp3 < ipMin3 || userIp3 > ipMax3)
				return false; // it's external
			else if(userIp3 == ipMin3 && userIp3 != ipMax3)
				return userIp4 >= ipMin4;
			else if(userIp3 != ipMin3 && userIp3 == ipMax3)
				return userIp4 <= ipMax4;
			else if(userIp4 >= ipMin4 && userIp4 <= ipMax4)
				return true; // it's internal
			else if(userIp4 < ipMin4 || userIp4 > ipMax4)
				return false; // it's external
		}
		else if(st.countTokens() == 1)
		{
			if(ip.equalsIgnoreCase(ipRange))
				return true;
		}
		else
			_log.warning("Error in internal ip detection: " + ipRange);
		return false;
	}

	/**
	 * Проверяет строку на соответсвие регулярному выражению
	 * @param text Строка-источник
	 * @param template Шаблон для поиска
	 * @return true в случае соответвия строки шаблону
	 */
	public static boolean isMatchingRegexp(String text, String template)
	{
		Pattern pattern = null;
		try
		{
			pattern = Pattern.compile(template);
		}
		catch(PatternSyntaxException e) // invalid template
		{
			e.printStackTrace();
		}
		if(pattern == null)
			return false;
		Matcher regexp = pattern.matcher(text);
		return regexp.matches();
	}

	/**
	 * Производит замену в строке по регулярному выражению
	 * @param source Строка-источник
	 * @param template Шаблон для замены
	 * @param replacement Строка замена
	 * @return Замененную строку
	 */
	public static String replaceRegexp(String source, String template, String replacement)
	{
		Pattern pattern = null;
		try
		{
			pattern = Pattern.compile(template);
		}
		catch(PatternSyntaxException e) // invalid template
		{
			e.printStackTrace();
		}
		if(pattern != null)
		{
			Matcher regexp = pattern.matcher(source);
			source = regexp.replaceAll(replacement);
		}
		return source;
	}

	public static String printData(byte[] data, int len)
	{
		StringBuffer result = new StringBuffer();

		int counter = 0;

		for(int i = 0; i < len; i++)
		{
			if(counter % 16 == 0)
				result.append(fillHex(i, 4) + ": ");

			result.append(fillHex(data[i] & 0xff, 2) + " ");
			counter++;
			if(counter == 16)
			{
				result.append("   ");

				int charpoint = i - 15;
				for(int a = 0; a < 16; a++)
				{
					int t1 = data[charpoint++];
					if(t1 > 0x1f && t1 < 0x80)
						result.append((char) t1);
					else
						result.append('.');
				}

				result.append("\n");
				counter = 0;
			}
		}

		int rest = data.length % 16;
		if(rest > 0)
		{
			for(int i = 0; i < 17 - rest; i++)
				result.append("   ");

			int charpoint = data.length - rest;
			for(int a = 0; a < rest; a++)
			{
				int t1 = data[charpoint++];
				if(t1 > 0x1f && t1 < 0x80)
					result.append((char) t1);
				else
					result.append('.');
			}

			result.append("\n");
		}

		return result.toString();
	}

	public static String fillHex(int data, int digits)
	{
		String number = Integer.toHexString(data);

		for(int i = number.length(); i < digits; i++)
			number = "0" + number;

		return number;
	}

	/**
	 * @param raw
	 * @return
	 */
	public static String printData(byte[] raw)
	{
		return printData(raw, raw.length);
	}

	/**
	 * Returns current timestamp in seconds (without milliseconds). Returned timestamp
	 * is obtained with the following expression: <p>
	 * <p/>
	 * <code>(System.currentTimeMillis() + 500L) / 1000L</code>
	 *
	 * @return Current timestamp
	 */
	public static long getTime()
	{
		return (System.currentTimeMillis() + 500L) / 1000L;
	}

	public static String formatDouble(double x, String nanString, boolean forceExponents)
	{
		if(Double.isNaN(x))
			return nanString;
		if(forceExponents)
			return df.format(x);
		if((long) x == x)
			return String.valueOf((long) x);
		return String.valueOf(x);
	}

	public static void handleIllegalPlayerAction(L2Player actor, String etc_str1, String etc_str2, int isBug)
	{
		ThreadPoolManager.getInstance().scheduleGeneral(new IllegalPlayerAction(actor, etc_str1, etc_str2, isBug), 500);
	}

	public static String getRelativePath(File base, File file)
	{
		return file.toURI().getPath().substring(base.toURI().getPath().length());
	}

	/** Return degree value of object 2 to the horizontal line with object 1 being the origin */
	public static double calculateAngleFrom(L2Object obj1, L2Object obj2)
	{
		return calculateAngleFrom(obj1.getX(), obj1.getY(), obj2.getX(), obj2.getY());
	}

	/** Return degree value of object 2 to the horizontal line with object 1 being the origin */
	public static double calculateAngleFrom(int obj1X, int obj1Y, int obj2X, int obj2Y)
	{
		double angleTarget = Math.toDegrees(Math.atan2(obj1Y - obj2Y, obj1X - obj2X));
		if(angleTarget <= 0)
			angleTarget += 360;
		return angleTarget;
	}

	public static boolean checkIfInRange(int range, int x1, int y1, int x2, int y2)
	{
		return checkIfInRange(range, x1, y1, 0, x2, y2, 0, false);
	}

	public static boolean checkIfInRange(int range, int x1, int y1, int z1, int x2, int y2, int z2, boolean includeZAxis)
	{
		int dx = x1 - x2;
		int dy = y1 - y2;

		if(includeZAxis)
		{
			int dz = z1 - z2;
			return dx * dx + dy * dy + dz * dz <= range * range;
		}
		return dx * dx + dy * dy <= range * range;
	}

	public static boolean checkIfInRange(int range, L2Object obj1, L2Object obj2, boolean includeZAxis)
	{
		if(obj1 == null || obj2 == null)
			return false;
		return checkIfInRange(range, obj1.getX(), obj1.getY(), obj1.getZ(), obj2.getX(), obj2.getY(), obj2.getZ(), includeZAxis);
	}

	public static double convertHeadingToDegree(int heading)
	{
		double angle = 360.0 * heading / 64400.0;
		if(angle == 0)
			angle = 360;
		return angle;
	}

	public static double calculateDistance(int x1, int y1, @SuppressWarnings("unused")
	int z1, int x2, int y2)
	{
		return calculateDistance(x1, y1, 0, x2, y2, 0, false);
	}

	public static double calculateDistance(int x1, int y1, int z1, int x2, int y2, int z2, boolean includeZAxis)
	{
		double dx = x1 - x2;
		double dy = y1 - y2;

		if(includeZAxis)
		{
			int dz = z1 - z2;
			return Math.sqrt(dx * dx + dy * dy + dz * dz);
		}
		return Math.sqrt(dx * dx + dy * dy);
	}

	public static double calculateDistance(L2Object obj1, L2Object obj2, boolean includeZAxis)
	{
		if(obj1 == null || obj2 == null)
			return 1000000;
		return calculateDistance(obj1.getX(), obj1.getY(), obj1.getZ(), obj2.getX(), obj2.getY(), obj2.getZ(), includeZAxis);
	}

	public static short getShort(byte[] bs, int offset)
	{
		return (short) (bs[offset + 1] << 8 | bs[offset] & 0xff);
	}

	/**
	 * Пересекает ли отрезок p1p2 горизонтальный отрезок (x1,ys):(x2,ys)
	 *
	 * @param p1 один конец отрезка
	 * @param p2 другой конец отрезка
	 * @param ys у-коорд. горизонтального отрезка, оно же - y точки пересечения
	 * @param x1 один конец горизонтали
	 * @param x2 другой конец горизонтали
	 * @return true если пересекает
	 */
	public static boolean isIntersectHorizontal(Location p1, Location p2, Location intersection, int x1, int x2)
	{
		// если отрезок целиком лежит с одной стороны от горизонтали
		if((p1.y - intersection.y) * (p2.y - intersection.y) > 0)
			return false;

		// х-координата точки пересечения отрезков
		// приведение к long для того, чтобы в конечном условии при умножении не было переполнения
		intersection.x = p2.x - (intersection.y - p2.y) * (p2.x - p1.x) / (p1.y - p2.y);

		// точка пересечения должна лежать всредине обоих отрезков
		return ((long) intersection.x - p1.x) * (intersection.x - p2.x) <= 0 && ((long) intersection.x - x1) * (intersection.x - x2) <= 0;
	}

	/**
	 * Пересекает ли отрезок p1p2 вертикальный отрезок (xs,y1):(xs,y1)
	 *
	 * @param p1 один конец отрезка
	 * @param p2 другой конец отрезка
	 * @param xs х-коорд. вертикального отрезка, оно же - х точки пересечения
	 * @param x1 один конец вертикали
	 * @param x2 другой конец вертикали
	 * @return true если пересекает
	 */
	public static boolean isIntersectVertical(Location p1, Location p2, Location intersection, int y1, int y2)
	{
		if((p1.x - intersection.x) * (p2.x - intersection.x) > 0)
			return false;
		intersection.y = p2.y - (intersection.x - p2.x) * (p2.y - p1.y) / (p1.x - p2.x);
		return ((long) intersection.y - p1.y) * (intersection.y - p2.y) <= 0 && ((long) intersection.y - y1) * (intersection.y - y2) <= 0;
	}

	public static int getIntersectionX(Location p1, Location p2, int ys, @SuppressWarnings("unused")
	int x1, @SuppressWarnings("unused")
	int x2)
	{
		return p2.x - (ys - p2.y) * (p2.x - p1.x) / (p1.y - p2.y);
	}

	public static int getIntersectionY(Location p1, Location p2, int xs, @SuppressWarnings("unused")
	int y1, @SuppressWarnings("unused")
	int y2)
	{
		return p2.y - (xs - p2.x) * (p2.y - p1.y) / (p1.x - p2.x);
	}

	public static double getDistance(int x1, int y1, int x2, int y2)
	{
		return Math.hypot(x1 - x2, y1 - y2);
	}

	/**
	* Return amount of adena formatted with " " delimiter
	* @param amount
	* @return String formatted adena amount
	*/
	public static String formatAdena(int amount)
	{
		/*
		String s = "";
		int rem = amount % 1000;
		s = Integer.toString(rem);
		amount = (amount - rem) / 1000;
		while(amount > 0)
		{
			if(rem < 99)
				s = '0' + s;
			if(rem < 9)
				s = '0' + s;
			rem = amount % 1000;
			s = Integer.toString(rem) + " " + s;
			amount = (amount - rem) / 1000;
		}
		return s;
		*/
		return adenaFormatter.format(amount);
	}

	/**
	 * Получает длинну пакета из 2-х первых байт.<BR>
	 * Используется для общения между LS и GS.
	 * @param first первый байт пакета
	 * @param second второй байт пакета
	 * @return длинна пакета
	 */
	public static int getPacketLength(byte first, byte second)
	{
		int lenght = first & 0xff;
		return lenght |= second << 8 & 0xff00;
	}

	/**
	 * Дописывает длинну пакета.<BR>
	 * Используется для общения между LS и GS.
	 * @param data пакет для отправления (зашифрован уже)
	 * @return готовый пакет для отправления
	 */
	public static byte[] writeLenght(byte[] data)
	{
		int newLenght = data.length + 2;
		byte[] result = new byte[newLenght];
		result[0] = (byte) (newLenght & 0xFF);
		result[1] = (byte) (newLenght >> 8 & 0xFF);
		System.arraycopy(data, 0, result, 2, data.length);
		return result;
	}

	public static byte[] generateHex(int size)
	{
		byte[] array = new byte[size];
		Random rnd = new Random();
		for(int i = 0; i < size; i++)
			array[i] = (byte) rnd.nextInt(256);
		if(Config.DEBUG)
			_log.fine("Generated random String:  \"" + array + "\"");
		return array;
	}

	/**
	 * форматирует время в секундах в дни/часы/минуты/секунды
	 */
	public static String formatTime(long time)
	{
		String ret = "";
		long numDays = time / 86400;
		time -= numDays * 86400;
		long numHours = time / 3600;
		time -= numHours * 3600;
		long numMins = time / 60;
		time -= numMins * 60;
		long numSeconds = time;
		if(numDays > 0)
			ret += numDays + "d ";
		if(numHours > 0)
			ret += numHours + "h ";
		if(numMins > 0)
			ret += numMins + "m ";
		if(numSeconds > 0)
			ret += numSeconds + "s";
		return ret.trim();
	}

	/**
	 * Инструмент для подсчета выпавших вещей с учетом рейтов.
	 * Возвращает 0 если шанс не прошел либо количество если прошел.
	 * Корректно обрабатывает шансы превышающие 100%.
	 * Шанс в 1:1000000 (L2Drop.MAX_CHANCE)
	 */
	public static int rollDrop(int min, int max, double calcChance)
	{
		if(calcChance <= 0 || min <= 0 || max <= 0)
			return 0;
		int dropmult = 1;
		calcChance *= Config.RATE_DROP_ITEMS;
		if(calcChance > L2Drop.MAX_CHANCE)
		{
			if((int) Math.ceil(calcChance / L2Drop.MAX_CHANCE) <= calcChance / L2Drop.MAX_CHANCE)
				calcChance = Math.nextUp(calcChance);
			dropmult = (int) Math.ceil(calcChance / L2Drop.MAX_CHANCE);
			calcChance = calcChance / dropmult;
		}
		return Rnd.chance(calcChance) ? Rnd.get(min * dropmult, max * dropmult) : 0;
	}
}