package l2r.gameserver.model;

import static l2r.gameserver.model.L2Zone.ZoneType.Castle;
import static l2r.gameserver.model.L2Zone.ZoneType.Siege;
import static l2r.gameserver.model.L2Zone.ZoneType.damage;
import static l2r.gameserver.model.L2Zone.ZoneType.instant_skill;
import static l2r.gameserver.model.L2Zone.ZoneType.no_landing;
import static l2r.gameserver.model.L2Zone.ZoneType.peace_zone;
import static l2r.gameserver.model.L2Zone.ZoneType.poison;
import static l2r.gameserver.model.L2Zone.ZoneType.ssq_zone;
import static l2r.gameserver.model.L2Zone.ZoneType.swamp;

import java.awt.Color;
import java.awt.Point;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.ConcurrentModificationException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.ConcurrentSkipListSet;
import java.util.concurrent.Future;
import java.util.concurrent.ScheduledFuture;
import java.util.logging.Level;
import java.util.logging.Logger;

import javolution.text.TextBuilder;
import javolution.util.FastList;
import javolution.util.FastMap;
import l2r.Config;
import l2r.Config.L2WalkerAllowed;
import l2r.database.DatabaseUtils;
import l2r.database.FiltredPreparedStatement;
import l2r.database.FiltredStatement;
import l2r.database.L2DatabaseFactory;
import l2r.database.ThreadConnection;
import l2r.database.mysql;
import l2r.extensions.Bonus;
import l2r.extensions.multilang.CustomMessage;
import l2r.extensions.scripts.Events;
import l2r.extensions.scripts.Scripts;
import l2r.gameserver.GameTimeController;
import l2r.gameserver.RecipeController;
import l2r.gameserver.ThreadPoolManager;
import l2r.gameserver.ai.CtrlEvent;
import l2r.gameserver.ai.CtrlIntention;
import l2r.gameserver.ai.L2PlayableAI;
import l2r.gameserver.ai.L2PlayerAI;
import l2r.gameserver.ai.L2PlayableAI.nextAction;
import l2r.gameserver.cache.Msg;
import l2r.gameserver.clientpackets.EnterWorld;
import l2r.gameserver.communitybbs.BB.Forum;
import l2r.gameserver.communitybbs.Manager.ForumsBBSManager;
import l2r.gameserver.geodata.network.geoclient.GeoClient;
import l2r.gameserver.handler.IItemHandler;
import l2r.gameserver.handler.ItemHandler;
import l2r.gameserver.idfactory.IdFactory;
import l2r.gameserver.instancemanager.CastleManager;
import l2r.gameserver.instancemanager.ClanHallManager;
import l2r.gameserver.instancemanager.CoupleManager;
import l2r.gameserver.instancemanager.CursedWeaponsManager;
import l2r.gameserver.instancemanager.DimensionalRiftManager;
import l2r.gameserver.instancemanager.FortressManager;
import l2r.gameserver.instancemanager.FortressSiegeManager;
import l2r.gameserver.instancemanager.QuestManager;
import l2r.gameserver.instancemanager.SiegeManager;
import l2r.gameserver.instancemanager.TownManager;
import l2r.gameserver.loginservercon.LSConnection;
import l2r.gameserver.loginservercon.gspackets.ChangeAccessLevel;
import l2r.gameserver.model.L2Clan.RankPrivs;
import l2r.gameserver.model.L2Zone.ZoneType;
import l2r.gameserver.model.Warehouse.WarehouseType;
import l2r.gameserver.model.base.ClassId;
import l2r.gameserver.model.base.Experience;
import l2r.gameserver.model.base.PlayerAccess;
import l2r.gameserver.model.base.Race;
import l2r.gameserver.model.entity.ClanHall;
import l2r.gameserver.model.entity.Duel;
import l2r.gameserver.model.entity.Hero;
import l2r.gameserver.model.entity.Olympiad;
import l2r.gameserver.model.entity.SevenSignsFestival.SevenSignsFestival;
import l2r.gameserver.model.entity.siege.Siege;
import l2r.gameserver.model.entity.siege.SiegeUnit;
import l2r.gameserver.model.instances.L2BoatInstance;
import l2r.gameserver.model.instances.L2ClanHallManagerInstance;
import l2r.gameserver.model.instances.L2CubicInstance;
import l2r.gameserver.model.instances.L2DecoyInstance;
import l2r.gameserver.model.instances.L2DoorInstance;
import l2r.gameserver.model.instances.L2GuardInstance;
import l2r.gameserver.model.instances.L2HennaInstance;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.model.instances.L2MinionInstance;
import l2r.gameserver.model.instances.L2MonsterInstance;
import l2r.gameserver.model.instances.L2NpcInstance;
import l2r.gameserver.model.instances.L2PetInstance;
import l2r.gameserver.model.instances.L2StaticObjectInstance;
import l2r.gameserver.model.instances.L2TamedBeastInstance;
import l2r.gameserver.model.quest.Quest;
import l2r.gameserver.model.quest.QuestState;
import l2r.gameserver.network.L2GameClient;
import l2r.gameserver.serverpackets.*;
import l2r.gameserver.skills.Env;
import l2r.gameserver.skills.SkillTimeStamp;
import l2r.gameserver.skills.Stats;
import l2r.gameserver.skills.effects.EffectTemplate;
import l2r.gameserver.tables.CharTemplateTable;
import l2r.gameserver.tables.ClanTable;
import l2r.gameserver.tables.HennaTable;
import l2r.gameserver.tables.ItemTable;
import l2r.gameserver.tables.MapRegionTable;
import l2r.gameserver.tables.NpcTable;
import l2r.gameserver.tables.ReflectionTable;
import l2r.gameserver.tables.SkillTable;
import l2r.gameserver.tables.SkillTreeTable;
import l2r.gameserver.templates.L2Armor;
import l2r.gameserver.templates.L2EtcItem;
import l2r.gameserver.templates.L2Henna;
import l2r.gameserver.templates.L2Item;
import l2r.gameserver.templates.L2PlayerTemplate;
import l2r.gameserver.templates.L2Weapon;
import l2r.gameserver.templates.L2Armor.ArmorType;
import l2r.gameserver.templates.L2Weapon.WeaponType;
import l2r.util.*;

/**
 * This class represents all player characters in the world.
 * There is always a client-thread connected to this (except if a player-store is activated upon logout).<BR><BR>
 */
public final class L2Player extends L2Playable
{
	private Bonus _bonus;

	public HashMap<Short, L2SubClass> _classlist = new HashMap<Short, L2SubClass>(4);

	public static final short STORE_PRIVATE_NONE = 0;
	public static final short STORE_PRIVATE_SELL = 1;
	public static final short STORE_PRIVATE_BUY = 3;
	public static final short STORE_PRIVATE_MANUFACTURE = 5;
	public static final short STORE_OBSERVING_GAMES = 7;
	public static final short STORE_PRIVATE_SELL_PACKAGE = 8;

	public static final int RANK_VAGABOND = 0;
	public static final int RANK_VASSAL = 1;
	public static final int RANK_HEIR = 2;
	public static final int RANK_KNIGHT = 3;
	public static final int RANK_WISEMAN = 4;
	public static final int RANK_BARON = 5;
	public static final int RANK_VISCOUNT = 6;
	public static final int RANK_COUNT = 7;
	public static final int RANK_MARQUIS = 8;

	public static final int RELATION_PVP_FLAG = 0x00002; // pvp
	public static final int RELATION_HAS_KARMA = 0x00004; // karma
	public static final int RELATION_UNKNOWN_1 = 0x00008; // unknown
	public static final int RELATION_UNKNOWN_2 = 0x00010; // unknown
	public static final int RELATION_UNKNOWN_3 = 0x00020; // unknown
	public static final int RELATION_UNKNOWN_4 = 0x00040; // unknown
	public static final int RELATION_LEADER = 0x00080; // leader at siege
	public static final int RELATION_UNKNOWN_5 = 0x00100; // unknown
	public static final int RELATION_INSIEGE = 0x00200; // true if in siege
	public static final int RELATION_ATTACKER = 0x00400; // true when attacker
	public static final int RELATION_ALLY = 0x00800; // blue siege icon, cannot have if red
	public static final int RELATION_ENEMY = 0x01000; // true when red icon, doesn't matter with blue
	public static final int RELATION_UNKNOWN_6 = 0x02000; // unknown
	public static final int RELATION_UNKNOWN_7 = 0x04000; // unknown
	public static final int RELATION_MUTUAL_WAR = 0x08000; // double fist
	public static final int RELATION_1SIDED_WAR = 0x10000; // single fist
	public static final int RELATION_UNKNOWN_8 = 0x20000; // unknown
	public static final int RELATION_UNKNOWN_9 = 0x40000; // unknown

	/** The table containing all minimum level needed for each Expertise (None, D, C, B, A, S)*/
	public static final short[] EXPERTISE_LEVELS = { SkillTreeTable.getExpertiseLevel(0), //NONE
			SkillTreeTable.getExpertiseLevel(1), //D
			SkillTreeTable.getExpertiseLevel(2), //C
			SkillTreeTable.getExpertiseLevel(3), //B
			SkillTreeTable.getExpertiseLevel(4), //A
			SkillTreeTable.getExpertiseLevel(5), //S
	};

	static final Logger _log = Logger.getLogger(L2Player.class.getName());

	private L2GameClient _connection;

	//private L2Object _newTarget = null;

	/** The level of the L2Player */
	private byte _level;

	/** The Identifier of the L2Player */
	private int _charId = 0x00030b7a;

	/** The Experience of the L2Player */
	private long _exp = 0;

	/** The number of SP of the L2Player */
	private int _sp = 0;

	/** The Karma of the L2Player (if higher than 0, the name of the L2Player appears in red) */
	private int _karma;

	/** The number of player killed during a PvP (the player killed was PvP Flagged) */
	private int _pvpKills;

	/** The hexadecimal Color of players name (white is 0xFFFFFF) */
	private int _nameColor;

	/** The hexadecimal Color of players title (white is 0xFFFFFF) */
	private int _titlecolor;

	/** The PK counter of the L2Player (= Number of non PvP Flagged player killed) */
	private int _pkKills;

	/** The number of recommendation obtained by the L2Player */
	private int _recomHave; // how much I was recommended by others

	/** The number of recommendation that the L2Player can give */
	private int _recomLeft; // how many recommendations I can give to others

	/** List with the recommendations that I've give */
	private List<Integer> _recomChars = new FastList<Integer>();

	private int _curWeightPenalty = 0;

	private int _deleteTimer;
	private final PcInventory _inventory = new PcInventory(this);
	private PcWarehouse _warehouse = new PcWarehouse(this);
	private PcFreight _freight = new PcFreight(this);

	/** True if the L2Player is sitting */
	boolean _sittingTask;
	/** Time counter when L2Player is sitting */
	private int _waitTimeWhenSit;

	/** True if the L2Player is using the relax skill */
	private boolean _relax;

	/** The face type Identifier of the L2Player */
	private byte _face;

	/** The hair style Identifier of the L2Player */
	private byte _hairStyle;

	/** The hair color Identifier of the L2Player */
	private byte _hairColor;

	/** The table containing all Quests began by the L2Player */
	private HashMap<String, QuestState> _quests = new HashMap<String, QuestState>();

	/** The list containing all shortCuts of this L2Player */
	private ShortCuts _shortCuts = new ShortCuts(this);

	/** The list containing all macroses of this L2Player */
	private MacroList _macroses = new MacroList(this);

	private L2TradeList _tradeList;
	private L2ManufactureList _createList;
	private ConcurrentLinkedQueue<TradeItem> _sellList;
	private ConcurrentLinkedQueue<TradeItem> _buyList;

	/** The Private Store type of the L2Player (STORE_PRIVATE_NONE=0, STORE_PRIVATE_SELL=1, sellmanage=2, STORE_PRIVATE_BUY=3, buymanage=4, STORE_PRIVATE_MANUFACTURE=5) */
	private short _privatestore;
	private ClassId _skillLearningClassId;

	// hennas
	private final L2HennaInstance[] _henna = new L2HennaInstance[3];
	private short _hennaSTR;
	private short _hennaINT;
	private short _hennaDEX;
	private short _hennaMEN;
	private short _hennaWIT;
	private short _hennaCON;

	/** The L2Summon of the L2Player */
	private L2Summon _summon = null;

	private L2DecoyInstance _decoy = null;

	// client radar
	public L2Radar radar;
	private final StatsChangeRecorder _statsChangeRecorder = new StatsChangeRecorder(this);

	// these values are only stored temporarily
	private boolean _partyMatchingAutomaticRegistration;
	private boolean _partyMatchingShowLevel;
	private boolean _partyMatchingShowClass;
	private String _partyMatchingMemo;

	private Point _lastPartyPositionSent = new Point(0, 0);
	private L2Party _party;
	// clan related attributes

	/** The Clan Identifier of the L2Player */
	private int _clanId;

	/** The Clan object of the L2Player */
	private L2Clan _clan;

	private int _pledgeClass = 0;
	private int _pledgeType = 0;
	private int _powerGrade = 0;
	private int _lvlJoinedAcademy = 0;
	private int _apprentice = 0;

	/** The Clan Leader Flag of the L2Player (True : the L2Player is the leader of the clan) */
	private boolean _clanLeader;

	private long _leaveClanTime;
	private long _deleteClanTime;

	private long _onlineTime;
	private long _onlineBeginTime;

	private long _NoChannel; // Nochannel mode
	private long _NoChannelBegin;

	//GM Stuff
	private int _accessLevel;
	private PlayerAccess _playerAccess = new PlayerAccess();

	private boolean _messageRefusal = false; // message refusal mode
	private boolean _tradeRefusal = false; // Trade refusal
	private boolean _exchangeRefusal = false; // Exchange refusal
	private boolean _massUpdating = false;
	public boolean _exploring = false;

	// this is needed to find the inviting player for Party response
	// there can only be one active party request at once
	private L2Player _currentTransactionRequester;
	public long _currentTransactionTimeout;
	private L2ItemInstance _arrowItem;

	/** The fists L2Weapon of the L2Player (used when no weapon is equipped) */
	private L2Weapon _fistsWeaponItem;

	private long _uptime;
	private String _accountName;

	private HashMap<Integer, String> _chars = new HashMap<Integer, String>(8);

	public byte updateKnownCounter = 0;

	/** The table containing all L2RecipeList of the L2Player */
	private Map<Integer, L2RecipeList> _recipebook = new TreeMap<Integer, L2RecipeList>();
	private Map<Integer, L2RecipeList> _commonrecipebook = new TreeMap<Integer, L2RecipeList>();

	// stats watch
	int oldMaxHP;
	int oldMaxMP;

	/** The current higher Expertise of the L2Player (None=0, D=1, C=2, B=3, A=4, S=5)*/
	public short expertiseIndex = 0;
	short expertisePenalty = 0;

	private L2ItemInstance _enchantScroll = null;

	private WarehouseType _usingWHType;
	private boolean _isOnline = false;
	private boolean _isDeleting = false;

	protected boolean _inventoryDisable = false;

	protected HashMap<Integer, L2CubicInstance> _cubics = new HashMap<Integer, L2CubicInstance>(4);

	/** The L2NpcInstance corresponding to the last Folk which one the player talked. */
	private L2NpcInstance _lastFolkNpc = null;

	protected ConcurrentSkipListSet<Integer> _activeSoulShots = new ConcurrentSkipListSet<Integer>();

	/** 1 if	the player is invisible */
	private boolean _invisible = false;

	/** Location before entering Observer Mode */
	private Location _obsLoc = new Location(0, 0, 0);
	private L2WorldRegion[] _observNeighbors;
	private boolean _observerMode = false;

	public int _telemode = 0;
	public int _unstuck = 0;

	/**
	 * Р­С‚Р° С‚РѕС‡РєР° РїСЂРѕРІРµСЂСЏРµС‚СЃСЏ РїСЂРё РЅРµС€С‚Р°С‚РЅРѕРј РІС‹С…РѕРґРµ С‡Р°СЂР°, Рё РµСЃР»Рё РЅРµ СЂР°РІРЅР° null С‡Р°СЂ РІРѕР·РІСЂР°С‰Р°РµС‚СЃСЏ РІ РЅРµРµ
	 * Р�СЃРїРѕР»СЊР·СѓРµС‚СЃСЏ РЅР°РїСЂРёРјРµСЂ РґР»СЏ РІРѕР·РІСЂР°С‰РµРЅРёСЏ РїСЂРё РїР°РґРµРЅРёРё СЃ РІРёРІРµСЂРЅС‹
	 * РџРѕР»Рµ heading РёСЃРїРѕР»СЊР·СѓРµС‚СЃСЏ РґР»СЏ С…СЂР°РЅРµРЅРёСЏ РґРµРЅРµРі РІРѕР·РІСЂР°С‰Р°РµРјС‹С… РїСЂРё СЃР±РѕРµ
	 *
	 * @see l2r.gameserver.network.L2GameClient.onDisconnection()
	 */
	public Location _stablePoint = null;

	/** new loto ticket **/
	public int _loto[] = new int[5];
	/** new race ticket **/
	public int _race[] = new int[2];

	private FastMap<Integer, String> _blockList = new FastMap<Integer, String>(); // characters blocked with '/block <charname>' cmd

	private boolean _blockAll = false; // /blockall cmd handling

	private boolean _isConnected = true;

	private boolean _hero = false;
	private int _team = 0;

	// time on login in game
	private long _lastAccess;

	/** True if the L2Player is in a boat */
	private boolean _InBoat;
	private L2BoatInstance _Boat;
	private int _boatId;
	private Location _inBoatPosition;

	protected int _baseClass;
	protected byte _baseLevel;
	protected long _baseExp;
	protected long _baseSp;
	protected short _activeClass = -1;

	private int _isBot;
	@SuppressWarnings("unchecked")
	private Future _botCheck;

	public boolean _isSitting = false;

	private boolean _noble = false;
	private boolean _inOlympiadMode = false;
	private int _olympiadGameId = -1;
	private int _olympiadSide = -1;

	/** ally with ketra or varka related wars*/
	private int _varka = 0;
	private int _ketra = 0;
	private int _ram = 0;

	/** The Siege state */
	private int _siegeState = 0;

	private byte[] _keyBindings;

	@SuppressWarnings("unchecked")
	private ScheduledFuture _taskWater;

	protected HashMap<Integer, Long> _StatKills = new HashMap<Integer, Long>(128);
	protected HashMap<Integer, Long> _StatDrop = new HashMap<Integer, Long>(128);
	protected HashMap<Integer, Long> _StatCraft = new HashMap<Integer, Long>(32);

	private Forum _forumMemo;

	private int _cursedWeaponEquippedId = 0;

	private L2Fishing _fishCombat;
	private boolean _fishing = false;
	private Location _fishLoc = new Location(0, 0, 0);
	private L2ItemInstance _lure = null;
	@SuppressWarnings("unchecked")
	public ScheduledFuture _taskforfish;

	private boolean _isInCombatZone;
	private boolean _isOnSiegeField;
	private boolean _isInPeaceZone;
	private boolean _isInSSZone;

	private boolean _offline = false;

	@SuppressWarnings("unchecked")
	private ScheduledFuture _breakWarn;
	private int pcBangPoints;

	/**
	 * Constructor of L2Player (use L2Character constructor).<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Call the L2Character constructor to create an empty _skills slot and copy basic Calculator set to this L2Player </li>
	 * <li>Set the name of the L2Player</li><BR><BR>
	 *
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : This method SET the level of the L2Player to 1</B></FONT><BR><BR>
	 *
	 * @param objectId Identifier of the object to initialized
	 * @param template The L2PlayerTemplate to apply to the L2Player
	 * @param accountName The name of the account including this L2Player
	 *
	 */
	private L2Player(final int objectId, final L2PlayerTemplate template, final String accountName)
	{
		super(objectId, template, null);

		_accountName = accountName;
		_level = 1;
		_nameColor = 0xFFFFFF;
		_titlecolor = 0xFFFF77;
		_baseClass = getClassId().getId();
		_baseLevel = _level;
		_baseExp = _exp;
		_baseSp = _sp;
	}

	/**
	 * Constructor of L2Player (use L2Character constructor).<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Call the L2Character constructor to create an empty _skills slot and copy basic Calculator set to this L2Player </li>
	 * <li>Create a L2Radar object</li>
	 * <li>Retrieve from the database all items of this L2Player and add them to _inventory </li>
	 *
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : This method DOESN'T SET the account name of the L2Player</B></FONT><BR><BR>
	 *
	 * @param objectId Identifier of the object to initialized
	 * @param template The L2PlayerTemplate to apply to the L2Player
	 *
	 */
	private L2Player(final int objectId, final L2PlayerTemplate template)
	{
		this(objectId, template, null);

		//restore inventory
		getInventory().restore();

		// Create an AI
		setAI(new L2PlayerAI(this));

		// Create a L2Radar object
		radar = new L2Radar(this);

		if(!Config.EVERYBODY_HAS_ADMIN_RIGHTS)
			setPlayerAccess(Config.gmlist.get(objectId));
		else
			setPlayerAccess(Config.gmlist.get(new Integer(0)));

		// Retrieve from the database all macroses of this L2Player and add them to _macroses
		_macroses.restore();
	}

	/**
	 * Create a new L2Player and add it in the characters table of the database.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Create a new L2Player with an account name </li>
	 * <li>Set the name, the Hair Style, the Hair Color and	the Face type of the L2Player</li>
	 * <li>Add the player in the characters table of the database</li><BR><BR>
	 *
	 * @param accountName The name of the L2Player
	 * @param name The name of the L2Player
	 * @param hairStyle The hair style Identifier of the L2Player
	 * @param hairColor The hair color Identifier of the L2Player
	 * @param face The face type Identifier of the L2Player
	 *
	 * @return The L2Player added to the database or null
	 */
	public static L2Player create(short classId, byte sex, String accountName, final String name, final byte hairStyle, final byte hairColor, final byte face)
	{
		L2PlayerTemplate template = CharTemplateTable.getInstance().getTemplate(classId, sex != 0);

		// Create a new L2Player with an account name
		L2Player player = new L2Player(IdFactory.getInstance().getNextId(), template, accountName);

		player.setName(name);
		player.setTitle("");
		player.setHairStyle(hairStyle);
		player.setHairColor(hairColor);
		player.setFace(face);

		// Add the player in the characters table of the database
		if(!player.createDb())
			return null;

		return player;
	}

	public String getAccountName()
	{
		if(_connection == null)
			return "<not connected>";
		return _connection.getLoginName();
	}

	public String getIP()
	{
		if(_connection == null)
			return "<not connected>";
		return _connection.getIpAddr();
	}

	/**
	 * Р’РѕР·РІСЂР°С‰Р°РµС‚ СЃРїРёСЃРѕРє РїРµСЂСЃРѕРЅР°Р¶РµР№ РЅР° Р°РєРєР°СѓРЅС‚Рµ, Р·Р° РёСЃРєР»СЋС‡РµРЅРёРµРј С‚РµРєСѓС‰РµРіРѕ
	 * @return РЎРїРёСЃРѕРє РїРµСЂСЃРѕРЅР°Р¶РµР№
	 */
	public HashMap<Integer, String> getAccountChars()
	{
		return _chars;
	}

	/**
	 * Retrieve a L2Player from the characters table of the database and add it in _allObjects of the L2world (call restore method).<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Retrieve the L2Player from the characters table of the database </li>
	 * <li>Add the L2Player object in _allObjects </li>
	 * <li>Set the x,y,z position of the L2Player and make it invisible</li>
	 * <li>Update the overloaded status of the L2Player</li><BR><BR>
	 *
	 * @param objectId Identifier of the object to initialized
	 *
	 * @return The L2Player loaded from the database
	 *
	 */
	public static L2Player load(final int objectId)
	{
		return restore(objectId);
	}

	/**
	 * @return the L2PlayerTemplate link to the L2Player.<BR><BR>
	 */
	@Override
	public final L2PlayerTemplate getTemplate()
	{
		return (L2PlayerTemplate) _template;
	}

	@Override
	public L2PlayerTemplate getBaseTemplate()
	{
		return (L2PlayerTemplate) _baseTemplate;
	}

	public void changeSex()
	{
		boolean male = true;
		if(getSex() == 1)
			male = false;
		_template = CharTemplateTable.getInstance().getTemplate(getClassId(), !male);
	}

	/**
	 * @return the AI of the L2Player (create it if necessary).<BR><BR>
	 */
	@Override
	public L2PlayableAI getAI()
	{
		if(_ai == null)
			_ai = new L2PlayerAI(this);
		return (L2PlayableAI) _ai;
	}

	@Override
	public void doAttack(final L2Character target)
	{
		super.doAttack(target);

		for(final L2CubicInstance cubic : _cubics.values())
			if(cubic.getId() != L2CubicInstance.LIFE_CUBIC)
				cubic.doAction(target);
	}

	@Override
	public void doCast(final L2Skill skill, final L2Character target, boolean forceUse)
	{
		if(skill == null || isBot() && !Config.ALLOW_BOT_CAST)
			return;

		super.doCast(skill, target, forceUse);

		if(skill.isOffensive() && target != null)
			for(final L2CubicInstance cubic : _cubics.values())
				if(cubic.getId() != L2CubicInstance.LIFE_CUBIC)
					cubic.doAction(target);
	}

	@Override
	public void updateEffectIcons()
	{
		if(_massUpdating)
			return;

		L2Effect[] effects = getAllFirstEffects();
		Arrays.sort(effects, EffectsComparator.getInstance());
		final PartySpelled ps = new PartySpelled(this, false);
		final MagicEffectIcons mi = new MagicEffectIcons();
		final ExOlympiadSpelledInfo os = new ExOlympiadSpelledInfo();

		for(final L2Effect effect : effects)
		{
			if(effect == null || !effect.isInUse())
				continue;

			if(effect.getStackType().equalsIgnoreCase("HpRecoverCast"))
				sendPacket(new ShortBuffStatusUpdate(effect));
			else
			{
				effect.addIcon(mi);
				if(_party != null)
					effect.addPartySpelledIcon(ps);
			}

			if(Config.ENABLE_OLYMPIAD && isInOlympiadMode())
				if(Olympiad.getInstance().getSpectators(getOlympiadGameId()) != null)
					effect.addOlympiadSpelledIcon(this, os);
		}

		sendPacket(mi);
		if(_party != null)
			_party.broadcastToPartyMembers(ps);
		if(Config.ENABLE_OLYMPIAD && Olympiad.getInstance().getSpectators(getOlympiadGameId()) != null)
			for(final L2Player spectator : Olympiad.getInstance().getSpectators(getOlympiadGameId()))
			{
				if(spectator == null)
					continue;
				spectator.sendPacket(os);
			}
	}

	@Override
	public void stopAllEffects()
	{
		_massUpdating = true;
		super.stopAllEffects();
		_massUpdating = false;

		sendChanges();
		updateEffectIcons();
	}

	public void refreshSavedStats()
	{
		_statsChangeRecorder.refreshSaves();
	}

	@Override
	public void sendChanges()
	{
		_statsChangeRecorder.sendChanges();
	}

	/**
	 * @return the Level of the L2Player.<BR><BR>
	 */
	@Override
	public final byte getLevel()
	{
		return _level;
	}

	public final void setLevel(final int lvl)
	{
		_level = (byte) lvl;
	}

	/**
	 * @return the Sex of the L2Player (Male=0, Female=1).<BR><BR>
	 */
	public byte getSex()
	{
		return getTemplate().isMale ? (byte) 0 : (byte) 1;
	}

	/**
	 * @return the Face type Identifier of the L2Player.<BR><BR>
	 */
	public byte getFace()
	{
		return _face;
	}

	/**
	 * Set the Face type of the L2Player.<BR><BR>
	 *
	 * @param face The Identifier of the Face type<BR><BR>
	 *
	 */
	public void setFace(final byte face)
	{
		_face = face;
	}

	/**
	 * @return the Hair Color Identifier of the L2Player.<BR><BR>
	 */
	public int getHairColor()
	{
		return _hairColor;
	}

	/**
	 * Set the Hair Color of the L2Player.<BR><BR>
	 *
	 * @param hairColor The Identifier of the Hair Color<BR><BR>
	 *
	 */
	public void setHairColor(final byte hairColor)
	{
		_hairColor = hairColor;
	}

	/**
	 * @return the Hair Style Identifier of the L2Player.<BR><BR>
	 */
	public int getHairStyle()
	{
		return _hairStyle;
	}

	/**
	 * Set the Hair Style of the L2Player.<BR><BR>
	 *
	 * @param hairStyle The Identifier of the Hair Style<BR><BR>
	 *
	 */
	public void setHairStyle(final byte hairStyle)
	{
		_hairStyle = hairStyle;
	}

	public boolean isInStoreMode()
	{
		return _privatestore != STORE_PRIVATE_NONE;
	}

	/**
	 * @return the Max HP (base+modifier) of the L2Player and Launch a Regen Task (if necessary).<BR><BR>
	 */
	@Override
	public int getMaxHp()
	{
		// Get the Max HP (base+modifier) of the L2Player
		final int val = super.getMaxHp();

		if(val != oldMaxHP)
		{
			oldMaxHP = val;

			// Launch a regen task if the new Max HP is higher than the old one
			if(getCurrentHp() != val)
				setCurrentHp(getCurrentHp()); // trigger start of regeneration
		}

		return val;
	}

	/**
	 * @return the Max MP (base+modifier) of the L2Player and Launch a Regen Task (if necessary).<BR><BR>
	 */
	@Override
	public int getMaxMp()
	{
		// Get the Max MP (base+modifier) of the L2Player
		final int val = super.getMaxMp();

		if(val != oldMaxMP)
		{
			oldMaxMP = val;

			// Launch a regen task if the new Max MP is higher than the old one
			if(!isDead() && getCurrentMp() != val)
				setCurrentMp(getCurrentMp()); // trigger start of regeneration
		}

		return val;

	}

	public void offline()
	{
		setNameColor(Config.SERVICES_OFFLINE_TRADE_NAME_COLOR);
		setOfflineMode(true);
		if(isFestivalParticipant())
		{
			L2Party playerParty = getParty();

			if(playerParty != null)
				playerParty.broadcastMessageToPartyMembers(getName() + " has been removed from the upcoming festival.");
		}

		if(getParty() != null)
			getParty().oustPartyMember(this);

		CursedWeaponsManager.getInstance().doLogout(this);

		if(isInOlympiadMode())
		{
			Olympiad.getInstance().logoutPlayer(this);
			Olympiad.getInstance().unRegisterNoble(this, true);
		}

		sendPacket(Msg.LeaveWorld);
		setConnected(false);
		//LSConnection.getInstance().removeAccount(getNetConnection());
		//LSConnection.getInstance().sendPacket(new PlayerLogout(getNetConnection().getLoginName()));
		broadcastUserInfo();
	}

	/**
	 * РЎРѕС…СЂР°РЅСЏРµС‚ РїРµСЂСЃРѕРЅР°Р¶Р° РІ Р±Рґ Рё Р·Р°РїСѓСЃРєР°РµС‚ РЅРµРѕР±С…РѕРґРёРјС‹Рµ РїСЂРѕС†РµРґСѓСЂС‹.
	 * @param shutdown С‚СЂСѓ РїСЂРё С€Р°С‚РґР°СѓРЅРµ
	 * @param restart С‚СЂСѓ РїСЂРё СЂРµСЃС‚Р°СЂС‚Рµ. Р�РіРЅРѕСЂРёСЂСѓРµС‚СЃСЏ С€Р°С‚РґР°СѓРЅ.
	 * @param kicked РћС‚РѕР±СЂР°Р·РёС‚СЊ Сѓ РєР»РёРµРЅС‚Р° С‚Р°Р±Р»РёС‡РєСѓ СЃ РјРµСЃСЃР°РіРѕР№ Рѕ Р·Р°РєСЂС‹С‚РёРё РєРѕРЅРЅРµРєС‚Р°
	 */
	public void logout(boolean shutdown, boolean restart, boolean kicked)
	{
		if(isCastingNow())
			abortCast();

		if(isFestivalParticipant())
		{
			L2Party playerParty = getParty();

			if(playerParty != null)
				playerParty.broadcastMessageToPartyMembers(getName() + " has been removed from the upcoming festival.");
		}

		if(kicked && Config.DROP_CURSED_WEAPONS_ON_KICK)
			if(isCursedWeaponEquipped())
			{
				_pvpFlag = 0;
				CursedWeaponsManager.getInstance().dropPlayer(this);
			}

		CursedWeaponsManager.getInstance().doLogout(this);
		
		if(isCombatFlagEquipped())
		{
			FortressSiegeManager.getInstance().dropCombatFlag(this);
			return;
		}
		
		if(isInOlympiadMode())
		{
			Olympiad.getInstance().logoutPlayer(this);
			Olympiad.getInstance().unRegisterNoble(this, true);
		}

		// Р’С‹Р·РѕРІ РІСЃРµС… С…СЌРЅРґР»РµСЂРѕРІ, РѕРїСЂРµРґРµР»РµРЅРЅС‹С… РІ СЃРєСЂРёРїС‚Р°С…
		for(String handler : Scripts.onPlayerExit)
		{
			String[] list = handler.split(":");
			String _class = list[0];
			String _method = list[1];
			callScripts(_class, _method, new Object[] { this });
		}

		if(getPet() != null)
			getPet().unSummon();

		Log.LogChar(this, Log.Logout, "");

		L2GameClient.saveCharToDisk(this);

		// РџСЂРё СЂРµСЃС‚Р°СЂС‚Рµ РїСЂРѕСЃС‚Рѕ РѕР±РЅСѓР»СЏРµРј РєРѕРЅРЅРµРєС‚
		if(restart)
		{
			deleteMe();
			if(_connection != null)
				_connection.setActiveChar(null);
			_connection = null;
			setConnected(false);
		}
		else
			kick(shutdown, kicked);
	}

	/**
	 * Logout without save.<BR><BR>
	 */
	private void kick(boolean shutdown, boolean kicked)
	{
		if(_connection != null)
		{
			L2GameServerPacket sp;

			if(shutdown)
				sp = Msg.ServerClose;
			else if(kicked)
				sp = Msg.ServerCloseSocket;
			else
				sp = Msg.LeaveWorld;

			if(_connection.getConnection() != null)
				_connection.getConnection().close(sp);
			_connection = null;
			setConnected(false);
		}
		deleteMe();
	}

	/**
	 * @return a table containing all L2RecipeList of the L2Player.<BR><BR>
	 */
	public Collection<L2RecipeList> getDwarvenRecipeBook()
	{
		return _recipebook.values();
	}

	public Collection<L2RecipeList> getCommonRecipeBook()
	{
		return _commonrecipebook.values();
	}

	public int recipesCount()
	{
		return _commonrecipebook.size() + _recipebook.size();
	}

	public boolean findRecipe(final L2RecipeList id)
	{
		return _recipebook.containsValue(id) || _commonrecipebook.containsValue(id);
	}

	public boolean findRecipe(final int id)
	{
		return _recipebook.containsKey(id) || _commonrecipebook.containsKey(id);
	}

	/**
	 * Add a new L2RecipList to the table _recipebook containing all L2RecipeList of the L2Player <BR><BR>
	 *
	 * @param recipe The L2RecipeList to add to the _recipebook
	 *
	 */
	public void registerRecipe(final L2RecipeList recipe, boolean saveDB)
	{
		if(recipe.isDwarvenRecipe())
			_recipebook.put(recipe.getId(), recipe);
		else
			_commonrecipebook.put(recipe.getId(), recipe);
		if(saveDB)
			mysql.set("REPLACE INTO character_recipebook (char_id, id) values(" + getObjectId() + "," + recipe.getId() + ")");
	}

	/**
	 * Remove a L2RecipList from the table _recipebook containing all L2RecipeList of the L2Player <BR><BR>
	 *
	 * @param RecipeID The Identifier of the L2RecipeList to remove from the _recipebook
	 *
	 */
	public void unregisterRecipe(final int RecipeID)
	{
		if(_recipebook.containsKey(RecipeID))
		{
			mysql.set("DELETE FROM `character_recipebook` WHERE `char_id`=" + getObjectId() + " AND `id`=" + RecipeID + " LIMIT 1");
			_recipebook.remove(RecipeID);
		}
		else if(_commonrecipebook.containsKey(RecipeID))
		{
			mysql.set("DELETE FROM `character_recipebook` WHERE `char_id`=" + getObjectId() + " AND `id`=" + RecipeID + " LIMIT 1");
			_commonrecipebook.remove(RecipeID);
		}
		else
			_log.warning("Attempted to remove unknown RecipeList" + RecipeID);
	}

	// ------------------- Quest Engine ----------------------

	/**
	 * @return the QuestState object corresponding to the quest name.
	 * @param quest The name of the quest
	 */
	public QuestState getQuestState(String quest)
	{
		return _quests != null ? _quests.get(quest) : null;
	}

	/**
	 * Add a QuestState to the table _quest containing all quests began by the L2Player.<BR><BR>
	 * @param qs The QuestState to add to _quest
	 */
	public void setQuestState(QuestState qs)
	{
		_quests.put(qs.getQuest().getName(), qs);
	}

	/**
	 * Remove a QuestState from the table _quest containing all quests began by the L2Player.<BR><BR>
	 * @param quest The name of the quest
	 */
	public void delQuestState(String quest)
	{
		_quests.remove(quest);
	}

	/**
	 * @return a table containing all Quest in progress from the table _quests.<BR><BR>
	 */
	public Quest[] getAllActiveQuests()
	{
		ArrayList<Quest> quests = new ArrayList<Quest>();
		for(final QuestState qs : _quests.values())
		{
			if(qs == null || qs.getQuest().getQuestIntId() >= 1999)
				continue;
			if(qs.isCompleted())
				continue;
			if(!qs.isStarted())
				continue;
			quests.add(qs.getQuest());
		}
		return quests.toArray(new Quest[quests.size()]);
	}

	public QuestState[] getAllQuestsStates()
	{
		return _quests.values().toArray(new QuestState[_quests.size()]);
	}

	/**
	 * @return a table containing all QuestState to modify after a L2NpcInstance killing.<BR><BR>
	 * @param npc L2NpcInstance attacked
	 */
	public ArrayList<QuestState> getQuestsForAttacks(L2NpcInstance npc)
	{
		ArrayList<QuestState> states = new ArrayList<QuestState>();
		Quest[] quests = npc.getTemplate().getEventQuests(Quest.QuestEventType.MOBGOTATTACKED);
		if(quests != null)
			for(Quest quest : quests)
				if(getQuestState(quest.getName()) != null && !getQuestState(quest.getName()).isCompleted())
					states.add(getQuestState(quest.getName()));
		return states;
	}

	/**
	 * @param npc L2NpcInstance that was killed
	 * @return a table containing all QuestState to modify after a L2NpcInstance killing.
	 */
	public ArrayList<QuestState> getQuestsForKills(L2NpcInstance npc)
	{
		ArrayList<QuestState> states = new ArrayList<QuestState>();
		Quest[] quests = npc.getTemplate().getEventQuests(Quest.QuestEventType.MOBKILLED);
		if(quests != null)
			for(Quest quest : quests)
				if(getQuestState(quest.getName()) != null && !getQuestState(quest.getName()).isCompleted())
					states.add(getQuestState(quest.getName()));
		return states;
	}

	/**
	 * @return a table containing all QuestState from the table _quests in which the L2Player must talk to the NPC.<BR><BR>
	 * @param npcId The Identifier of the NPC
	 */
	public ArrayList<QuestState> getQuestsForTalk(int npcId)
	{
		ArrayList<QuestState> states = new ArrayList<QuestState>();
		Quest[] quests = NpcTable.getTemplate(npcId).getEventQuests(Quest.QuestEventType.QUEST_TALK);
		if(quests != null)
			for(Quest quest : quests)
				if(getQuestState(quest.getName()) != null && !getQuestState(quest.getName()).isCompleted())
					states.add(getQuestState(quest.getName()));
		return states;
	}

	public void processQuestEvent(String quest, String event)
	{
		if(event == null)
			event = "";
		QuestState qs = getQuestState(quest);
		if(qs == null)
		{
			Quest q = QuestManager.getQuest(quest);
			if(q == null)
				return;
			qs = q.newQuestState(this);
		}
		if(qs == null || qs.isCompleted())
			return;
		qs.getQuest().notifyEvent(event, qs);
		sendPacket(new QuestList(this));
	}

	/**
	 * РџСЂРѕРІРµСЂРєР° РЅР° РїРµСЂРµРїРѕР»РЅРµРЅРёРµ РёРЅРІРµРЅС‚Р°СЂСЏ Рё РїРµСЂРµР±РѕСЂ РІ РІРµСЃРµ РґР»СЏ РєРІРµСЃС‚РѕРІ Рё СЌРІРµРЅС‚РѕРІ
	 * @return true РµСЃР»Рё РІРµ РїСЂРѕРІРµСЂРєРё РїСЂРѕС€Р»Рё СѓСЃРїРµС€РЅРѕ
	 */
	public boolean isQuestContinuationPossible()
	{
		if(getWeightPenalty() >= 3 || getInventoryLimit() * 0.8 < getInventory().getSize())
		{
			sendPacket(Msg.PROGRESS_IN_A_QUEST_IS_POSSIBLE_ONLY_WHEN_YOUR_INVENTORYS_WEIGHT_AND_VOLUME_ARE_LESS_THAN_80_PERCENT_OF_CAPACITY);
			return false;
		}
		return true;
	}

	// ----------------- End of Quest Engine -------------------

	/**
	 * @return a table containing all L2ShortCut of the L2Player.<BR><BR>
	 */
	public Collection<L2ShortCut> getAllShortCuts()
	{
		return _shortCuts.getAllShortCuts();
	}

	/**
	 * Return the L2ShortCut of the L2Player corresponding to the position (page-slot).<BR><BR>
	 *
	 * @param slot The slot in witch the shortCuts is equipped
	 * @param page The page of shortCuts containing the slot
	 * @return the L2ShortCut of the L2Player corresponding to the position (page-slot).
	 */
	public L2ShortCut getShortCut(int slot, int page)
	{
		return _shortCuts.getShortCut(slot, page);
	}

	/**
	 * Add a L2shortCut to the L2Player _shortCuts<BR><BR>
	 */
	public void registerShortCut(L2ShortCut shortcut)
	{
		_shortCuts.registerShortCut(shortcut);
	}

	/**
	 * Delete the L2ShortCut corresponding to the position (page-slot) from the L2Player _shortCuts.<BR><BR>
	 */
	public void deleteShortCut(int slot, int page)
	{
		_shortCuts.deleteShortCut(slot, page);
	}

	/**
	 * Add a L2Macro to the L2Player _macroses<BR><BR>
	 */
	public void registerMacro(L2Macro macro)
	{
		_macroses.registerMacro(macro);
	}

	/**
	 * Delete the L2Macro corresponding to the Identifier from the L2Player _macroses.<BR><BR>
	 */
	public void deleteMacro(int id)
	{
		_macroses.deleteMacro(id);
	}

	/**
	 * @return all L2Macro of the L2Player.<BR><BR>
	 */
	public MacroList getMacroses()
	{
		return _macroses;
	}

	/**
	 * Р’РѕР·РІСЂР°С‰Р°РµС‚ СЃРѕСЃС‚РѕСЏРЅРёРµ РѕСЃР°РґС‹ L2Player.<BR>
	 * 1 = attacker, 2 = defender, 0 = РЅРµ СѓС‡Р°РІСЃС‚РІСѓРµС‚
	 * @return СЃРѕСЃС‚РѕСЏРЅРёРµ РѕСЃР°РґС‹
	 */
	public int getSiegeState()
	{
		return _siegeState;
	}

	/**
	* РЈСЃС‚Р°РЅР°РІР»РёРІР°РµС‚ СЃРѕСЃС‚РѕСЏРЅРёРµ РѕСЃР°РґС‹ L2Player.<BR>
	* 1 = attacker, 2 = defender, 0 = РЅРµ СѓС‡Р°РІСЃС‚РІСѓРµС‚
	*/
	public void setSiegeState(int siegeState)
	{
		_siegeState = siegeState;
	}

	/**
	 * @return true if this L2Player is a clan leader in
	 * ownership of the passed castle
	 */
	public boolean isCastleLord(int castleId)
	{
		return _clan != null && _clanLeader && _clan.getHasCastle() == castleId;
	}

	/**
	 * РџСЂРѕРІРµСЂСЏРµС‚ СЏРІР»СЏРµС‚СЃСЏ Р»Рё СЌС‚РѕС‚ РїРµСЂСЃРѕРЅР°Р¶ РІР»Р°РґРµР»СЊС†РµРј РєСЂРµРїРѕСЃС‚Рё
	 * @param fortressId
	 * @return true РµСЃР»Рё РІР»Р°РґРµР»РµС†
	 */
	public boolean isFortressLord(int fortressId)
	{
		return _clan != null && _clanLeader && _clan.getHasFortress() == fortressId;
	}

	/**
	 * @return the PK counter of the L2Player.<BR><BR>
	 */
	public int getPkKills()
	{
		return _pkKills;
	}

	/**
	 * Set the PK counter of the L2Player.<BR><BR>
	 */
	public void setPkKills(final int pkKills)
	{
		_pkKills = pkKills;
	}

	/**
	 * @return the _deleteTimer of the L2Player.<BR><BR>
	 */
	public int getDeleteTimer()
	{
		return _deleteTimer;
	}

	/**
	 * Set the _deleteTimer of the L2Player.<BR><BR>
	 */
	public void setDeleteTimer(final int deleteTimer)
	{
		_deleteTimer = deleteTimer;
	}

	/**
	 * @return the current weight of the L2Player.<BR><BR>
	 */
	public int getCurrentLoad()
	{
		return _inventory.getTotalWeight();
	}

	public long getLastAccess()
	{
		return _lastAccess;
	}

	public void setLastAccess(long value)
	{
		_lastAccess = value;
	}

	/**
	 * @return the number of recommendation obtained by the L2Player.<BR><BR>
	 */
	public int getRecomHave()
	{
		return _recomHave;
	}

	/**
	 * Set the number of recommendation obtained by the L2Player (Max : 255).<BR><BR>
	 * @param value РєРѕР»РёС‡РµСЃС‚РІРѕ СЂРµРєРѕРјРµРЅРґРѕРІ
	 */
	public void setRecomHave(int value)
	{
		if(value > 255)
			_recomHave = 255;
		else if(value < 0)
			_recomHave = 0;
		else
			_recomHave = value;
	}

	/**
	 * @return the number of recommendation that the L2Player can give.<BR><BR>
	 */
	public int getRecomLeft()
	{
		return _recomLeft;
	}

	/**
	 * Set the number of recommendation that the L2Player can give.<BR><BR>
	 */
	public void setRecomLeft(final int value)
	{
		_recomLeft = value;
	}

	public void giveRecom(final L2Player target)
	{
		int targetRecom = target.getRecomHave();
		if(targetRecom < 255)
			target.setRecomHave(targetRecom + 1);
		if(_recomLeft > 0)
			_recomLeft--;
		_recomChars.add(target.getName().hashCode());
	}

	public boolean canRecom(final L2Player target)
	{
		return !_recomChars.contains(target.getName().hashCode());
	}

	/**
	 * @return the Karma of the L2Player.<BR><BR>
	 */
	@Override
	public int getKarma()
	{
		return _karma;
	}

	/**
	 * Set the Karma of the L2Player and send a Server->Client packet StatusUpdate (broadcast).<BR><BR>
	 * @param karma new value of karma
	 */
	public void setKarma(int karma)
	{
		if(karma < 0)
			karma = 0;

		if(_karma == karma)
			return;

		_karma = karma;

		if(karma > 0)
			for(final L2Character object : getKnownCharacters(2000))
				if(object instanceof L2GuardInstance && object.getAI().getIntention() == CtrlIntention.AI_INTENTION_IDLE)
					object.getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE, null, null);

		sendChanges();

		if(getPet() != null)
			getPet().broadcastPetInfo();
	}

	/**
	 * @return the max weight that the L2Player can load.
	 */
	public int getMaxLoad()
	{
		// Weight Limit = (CON Modifier*69000)*Skills
		// Source http://l2p.bravehost.com/weightlimit.html (May 2007)
		// Fitted exponential curve to the data
		int con = getCON();
		if(con < 1)
			return (int) (31000 * Config.MAXLOAD_MODIFIER);
		else if(con > 59)
			return (int) (176000 * Config.MAXLOAD_MODIFIER);
		else
			return (int) calcStat(Stats.MAX_LOAD, Math.pow(1.029993928, con) * 30495.627366 * Config.MAXLOAD_MODIFIER, this, null);
	}

	public int getexpertisePenalty()
	{
		return expertisePenalty;
	}

	public int getWeightPenalty()
	{
		return _curWeightPenalty;
	}

	public boolean isMassUpdating()
	{
		return _massUpdating;
	}

	public void setMassUpdating(final boolean massUpdating)
	{
		_massUpdating = massUpdating;
	}

	/**
	 * Update the overloaded status of the L2Player.<BR><BR>
	 */
	public void refreshOverloaded()
	{
		if(_massUpdating || getMaxLoad() <= 0)
			return;

		setOverloaded(getCurrentLoad() > getMaxLoad());
		final float weightproc = getCurrentLoad() * 100 / getMaxLoad();
		int newWeightPenalty;

		if(weightproc < 50)
			newWeightPenalty = 0;
		else if(weightproc < 66.6)
			newWeightPenalty = 1;
		else if(weightproc < 80)
			newWeightPenalty = 2;
		else if(weightproc < 100)
			newWeightPenalty = 3;
		else
			newWeightPenalty = 4;

		if(_curWeightPenalty == newWeightPenalty)
			return;

		_curWeightPenalty = newWeightPenalty;
		if(_curWeightPenalty > 0)
			super.addSkill(SkillTable.getInstance().getInfo(4270, _curWeightPenalty));
		else
			super.removeSkill(getKnownSkill(4270));

		sendPacket(new EtcStatusUpdate(this));
	}

	public void refreshExpertisePenalty()
	{
		if(_massUpdating)
			return;

		short newPenalty = 0;
		L2ItemInstance[] items = getInventory().getItems();
		for(L2ItemInstance item : items)
			if(item.isEquipped())
			{
				short crystaltype = item.getItem().getCrystalType();
				if(crystaltype > newPenalty)
					newPenalty = crystaltype;
			}

		newPenalty = (short) (newPenalty - expertiseIndex);
		if(newPenalty <= 0)
			newPenalty = 0;

		if(expertisePenalty == newPenalty)
			return;

		expertisePenalty = newPenalty;
		if(newPenalty > 0)
			super.addSkill(SkillTable.getInstance().getInfo(4267, expertisePenalty));
		else
			super.removeSkill(getKnownSkill(4267));
		sendPacket(new EtcStatusUpdate(this));
	}

	/**
	 * @return the the PvP Kills of the L2Player (Number of player killed during a PvP).<BR><BR>
	 */
	public int getPvpKills()
	{
		return _pvpKills;
	}

	/**
	 * Set the the PvP Kills of the L2Player (Number of player killed during a PvP).<BR><BR>
	 * @param pvpKills new value of pvp kills
	 */
	public void setPvpKills(int pvpKills)
	{
		_pvpKills = pvpKills;
	}

	/**
	 * @return the ClassId object of the L2Player contained in L2PlayerTemplate.<BR><BR>
	 */
	public ClassId getClassId()
	{
		return getTemplate().classId;
	}

	public void addClanPointsOnProfession(final short id)
	{
		if(getLvlJoinedAcademy() != 0 && _clan != null && _clan.getLevel() >= 5 && ClassId.values()[id].getLevel() == 2)
			_clan.incReputation(100, true, "Academy");
		else if(getLvlJoinedAcademy() != 0 && _clan != null && _clan.getLevel() >= 5 && ClassId.values()[id].getLevel() > 2)
		{
			int earnedPoints = 0;
			if(getLvlJoinedAcademy() <= 16)
				earnedPoints = 300;
			else if(getLvlJoinedAcademy() >= 39)
				earnedPoints = 170;
			else
				earnedPoints = 300 - (getLvlJoinedAcademy() - 16) * 10;

			_clan.removeClanMember(getObjectId());
			SystemMessage sm = new SystemMessage(SystemMessage.CLAN_ACADEMY_MEMBER_S1_HAS_SUCCESSFULLY_COMPLETED_THE_2ND_CLASS_TRANSFER_AND_OBTAINED_S2_CLAN_REPUTATION_POINTS);
			sm.addString(getName());
			sm.addNumber(_clan.incReputation(earnedPoints, true, "Academy"));
			_clan.broadcastToOnlineMembers(sm);
			_clan.broadcastToOtherOnlineMembers(new PledgeShowMemberListDelete(getName()), this);

			setLvlJoinedAcademy(0);
			setClan(null);
			setTitle("");
			sendPacket(Msg.CONGRATULATIONS_YOU_WILL_NOW_GRADUATE_FROM_THE_CLAN_ACADEMY_AND_LEAVE_YOUR_CURRENT_CLAN_AS_A_GRADUATE_OF_THE_ACADEMY_YOU_CAN_IMMEDIATELY_JOIN_A_CLAN_AS_A_REGULAR_MEMBER_WITHOUT_BEING_SUBJECT_TO_ANY_PENALTIES);
			setLeaveClanTime(0);

			broadcastUserInfo();

			sendPacket(new PledgeShowMemberListDeleteAll());

			L2ItemInstance academyCirclet = ItemTable.getInstance().createItem(8181, getObjectId(), 0, "L2Clan.completeAcademy");
			getInventory().addItem(academyCirclet);
			sendPacket(new SystemMessage(SystemMessage.EARNED_S2_S1_s).addString("Academy Reward").addNumber(1));
		}
	}

	/**
	 * Set the template of the L2Player.<BR><BR>
	 *
	 * @param id The Identifier of the L2PlayerTemplate to set to the L2Player
	 *
	 */
	public void setClassId(final short id, boolean noban)
	{
		if(!noban && !(ClassId.values()[id].equalsOrChildOf(ClassId.values()[getActiveClass()]) || getPlayerAccess().CanChangeClass || Config.EVERYBODY_HAS_ADMIN_RIGHTS))
		{
			Thread.dumpStack();
			Util.handleIllegalPlayerAction(this, "L2Player[1535]", "tried to change class " + getActiveClass() + " to " + id, 2);
			return;
		}

		//Р•СЃР»Рё РЅРѕРІС‹Р№ ID РЅРµ РїСЂРёРЅР°РґР»РµР¶РёС‚ РёРјРµСЋС‰РёРјСЃСЏ РєР»Р°СЃСЃР°Рј Р·РЅР°С‡РёС‚ СЌС‚Рѕ РЅРѕРІР°СЏ РїСЂРѕС„Р°
		if(!_classlist.containsKey(id))
		{
			final L2SubClass cclass = _classlist.get(getActiveClass());
			if(cclass == null) // РЅРµ Р·РЅР°СЋ РєР°Рє СЌС‚Рѕ РІРѕР·РјРѕР¶РЅРѕ РЅРѕ С‚РёРєРµС‚ Р±С‹Р»
				return;
			_classlist.remove(getActiveClass());
			changeClassInDb(cclass.getClassId(), id);
			if(cclass.isBase())
			{
				setBaseClass(id);
				addClanPointsOnProfession(id);
				L2ItemInstance coupons = null;
				if(ClassId.values()[id].getLevel() == 2)
					coupons = ItemTable.getInstance().createItem(8869, getObjectId(), 0, "ClassMaster");
				else if(ClassId.values()[id].getLevel() == 3)
				{
					coupons = ItemTable.getInstance().createItem(8870, getObjectId(), 0, "ClassMaster");
					unsetVar("dd");
					unsetVar("dd1"); // СѓРґР°Р»СЏРµРј РѕС‚РјРµС‚РєРё Рѕ РІС‹РґР°С‡Рµ РґРёРјРµРЅС€РµРЅ РґР°Р№РјРѕРЅРґРѕРІ
					unsetVar("dd2");
					unsetVar("dd3");
				}
				if(coupons != null)
				{
					coupons.setCount(15);
					getInventory().addItem(coupons);
					sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S2_S1).addItemName(coupons.getItemId()).addNumber(15));
				}
			}
			cclass.setClassId(id);
			_classlist.put(id, cclass);
			setActiveClass(id);
			refreshExpertisePenalty();
			storeCharSubClasses();

			// РЎРѕС†РёР°Р»РєР° РїСЂРё РїРѕР»СѓС‡РµРЅРёРё РїСЂРѕС„С‹
			broadcastPacket(new MagicSkillUse(this, this, 5103, 1, 1000, 0));
			//broadcastPacket(new SocialAction(getObjectId(), 16));
			sendPacket(new PlaySound("ItemSound.quest_fanfare_2"));
			broadcastUserInfo();
		}

		L2PlayerTemplate t = CharTemplateTable.getInstance().getTemplate(id, getSex() == 1);
		if(t == null)
		{
			_log.severe("Missing template for classId: " + id);
			// do not throw error - only print error
			return;
		}

		// Set the template of the L2Player
		setTemplate(t);

		// Update class icon in party and clan
		if(isInParty())
			getParty().broadcastToPartyMembers(new PartySmallWindowUpdate(this));
		if(getClan() != null)
			getClan().broadcastToOnlineMembers(new PledgeShowMemberListUpdate(this));
	}

	public void setClassId(final short id)
	{
		setClassId(id, false);
	}

	/**
	 * @return the Experience of the L2Player.<BR><BR>
	 */
	public long getExp()
	{
		return _exp;
	}

	public void setEnchantScroll(final L2ItemInstance scroll)
	{
		_enchantScroll = scroll;
	}

	public L2ItemInstance getEnchantScroll()
	{
		return _enchantScroll;
	}

	/**
	 * Set the fists weapon of the L2Player (used when no weapon is equipped).<BR><BR>
	 *
	 * @param weaponItem The fists L2Weapon to set to the L2Player
	 *
	 */
	public void setFistsWeaponItem(final L2Weapon weaponItem)
	{
		_fistsWeaponItem = weaponItem;
	}

	/**
	 * @return the fists weapon of the L2Player (used when no weapon is equipped).<BR><BR>
	 */
	public L2Weapon getFistsWeaponItem()
	{
		return _fistsWeaponItem;
	}

	/**
	 * @return the fists weapon of the L2Player Class (used when no weapon is equipped).<BR><BR>
	 */
	public L2Weapon findFistsWeaponItem(final int classId)
	{
		//human fighter fists
		if(classId >= 0x00 && classId <= 0x09)
			return (L2Weapon) ItemTable.getInstance().getTemplate(246);

		//human mage fists
		if(classId >= 0x0a && classId <= 0x11)
			return (L2Weapon) ItemTable.getInstance().getTemplate(251);

		//elven fighter fists
		if(classId >= 0x12 && classId <= 0x18)
			return (L2Weapon) ItemTable.getInstance().getTemplate(244);

		//elven mage fists
		if(classId >= 0x19 && classId <= 0x1e)
			return (L2Weapon) ItemTable.getInstance().getTemplate(249);

		//dark elven fighter fists
		if(classId >= 0x1f && classId <= 0x25)
			return (L2Weapon) ItemTable.getInstance().getTemplate(245);

		//dark elven mage fists
		if(classId >= 0x26 && classId <= 0x2b)
			return (L2Weapon) ItemTable.getInstance().getTemplate(250);

		//orc fighter fists
		if(classId >= 0x2c && classId <= 0x30)
			return (L2Weapon) ItemTable.getInstance().getTemplate(248);

		//orc mage fists
		if(classId >= 0x31 && classId <= 0x34)
			return (L2Weapon) ItemTable.getInstance().getTemplate(252);

		//dwarven fists
		if(classId >= 0x35 && classId <= 0x39)
			return (L2Weapon) ItemTable.getInstance().getTemplate(247);

		return null;
	}

	/**
	 * Р”РѕР±Р°РІР»СЏРµС‚ С‡Р°СЂСѓ РѕРїС‹С‚ Рё/РёР»Рё СЃРї СЃ СѓС‡РµС‚РѕРј Р»РёС‡РЅРѕРіРѕ Р±РѕРЅСѓСЃР°
	 */
	@Override
	public void addExpAndSp(long addToExp, long addToSp)
	{
		addExpAndSp(addToExp, addToSp, true, true);
	}

	/**
	 * Р”РѕР±Р°РІР»СЏРµС‚ С‡Р°СЂСѓ РѕРїС‹С‚ Рё/РёР»Рё СЃРї, СЃ СѓС‡РµС‚РѕРј Р»РёС‡РЅРѕРіРѕ Р±РѕРЅСѓСЃР° РёР»Рё РЅРµС‚
	 */
	public void addExpAndSp(long addToExp, long addToSp, boolean applyBonus, boolean appyToPet)
	{
		if(_exp > Experience.LEVEL[Experience.MAX_LEVEL + 1])
			_exp = Experience.LEVEL[Experience.MAX_LEVEL + 1];

		if(applyBonus)
		{
			addToExp *= getRateExp();
			addToSp *= getRateSp();
		}

		if(addToExp > 0)
		{
			if(appyToPet)
			{
				if(getPet() != null && !getPet().isDead())
					// Sin Eater Р·Р°Р±РёСЂР°РµС‚ РІСЃСЋ СЌРєСЃРїСѓ Сѓ РїРµСЂСЃРѕРЅР°Р¶Р°
					if(getPet().getNpcId() == 12564)
					{
						getPet().addExpAndSp(addToExp, 0);
						addToExp = 0;
					}
					// РџРµС‚С‹ Р·Р°Р±РёСЂР°СЋС‚ 10% СЌРєСЃРїС‹ Сѓ РїРµСЂСЃРѕРЅР°Р¶Р°
					else if(getPet() instanceof L2PetInstance)
					{
						getPet().addExpAndSp((long) (addToExp * .1), 0);
						addToExp *= .9;
					}
			}

			// Remove Karma when the player kills L2MonsterInstance
			if(!isCursedWeaponEquipped() && addToSp > 0 && _karma > 0)
				_karma -= addToSp / (Config.KARMA_SP_DIVIDER * Config.RATE_SP);

			if(_karma < 0)
				_karma = 0;

			SystemMessage sm = new SystemMessage(SystemMessage.YOU_EARNED_S1_EXP_AND_S2_SP);
			sm.addNumber((int) addToExp);
			sm.addNumber((int) addToSp);
			sendPacket(sm);
		}

		long max_xp = Experience.LEVEL[Experience.MAX_LEVEL + 1];

		_exp = _exp + addToExp > max_xp ? max_xp : _exp + addToExp;
		if(_sp + addToSp > Integer.MAX_VALUE)
			_sp = Integer.MAX_VALUE;
		else
			_sp += addToSp;

		while(_exp >= Experience.LEVEL[_level + 1] && _level < Experience.MAX_LEVEL)
			increaseLevel();

		while(_exp < Experience.LEVEL[_level] && _level > 1)
			decreaseLevel();

		sendChanges();
	}

	/**
	 * Give Expertise skill of this level.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Get the Level of the L2Player </li>
	 * <li>Add the Expertise skill corresponding to its Expertise level</li>
	 * <li>Update the overloaded status of the L2Player</li><BR><BR>
	 *
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : This method DOESN'T give other free skills (SP needed = 0)</B></FONT><BR><BR>
	 *
	 */
	private void rewardSkills()
	{
		// Calculate the current higher Expertise of the L2Player
		for(short i = 0; i < EXPERTISE_LEVELS.length; i++)
			if(_level >= EXPERTISE_LEVELS[i])
				expertiseIndex = i;

		// Add the Expertise skill corresponding to its Expertise level
		if(expertiseIndex > 0)
		{
			L2Skill skill = SkillTable.getInstance().getInfo(239, expertiseIndex);
			addSkill(skill, false);
		}

		boolean update = false;
		if(Config.AUTO_LEARN_SKILLS)
		{
			int unLearnable = 0;
			ArrayList<L2SkillLearn> skills = SkillTreeTable.getInstance().getAvailableSkills(this, getClassId());
			while(skills.size() > unLearnable)
			{
				unLearnable = 0;
				for(L2SkillLearn s : skills)
				{
					L2Skill sk = SkillTable.getInstance().getInfo(s.id, s.skillLevel);
					if(sk == null || !sk.getCanLearn(getClassId()))
					{
						unLearnable++;
						continue;
					}
					addSkill(sk, true);
				}
				skills = SkillTreeTable.getInstance().getAvailableSkills(this, getClassId());
			}
			update = true;
		}
		else
			// РЎРєРёР»Р»С‹ РґР°СЋС‰РёРµСЃСЏ Р±РµСЃРїР»Р°С‚РЅРѕ РЅРµ С‚СЂРµР±СѓСЋС‚ РёР·СѓС‡РµРЅРёСЏ
			for(L2SkillLearn skill : SkillTreeTable.getInstance().getAvailableSkills(this, getClassId()))
				if(skill._repCost == 0 && skill._spCost == 0 && skill.itemCount == 0)
				{
					L2Skill sk = SkillTable.getInstance().getInfo(skill.getId(), skill.getLevel());
					addSkill(sk, true);
					if(getAllShortCuts().size() > 0 && sk.getLevel() > 1)
						for(L2ShortCut sc : getAllShortCuts())
							if(sc.id == sk.getId() && sc.type == L2ShortCut.TYPE_SKILL)
							{
								L2ShortCut newsc = new L2ShortCut(sc.slot, sc.page, sc.type, sc.id, sk.getLevel());
								sendPacket(new ShortCutRegister(newsc));
								registerShortCut(newsc);
							}
					update = true;
				}

		if(update)
			sendPacket(new SkillList(this));

		// This function gets called on login, so not such a bad place to check weight
		// Update the overloaded status of the L2Player
		refreshOverloaded();
		refreshExpertisePenalty();
	}

	/**
	 * @return the Race object of the L2Player.<BR><BR>
	 */
	public Race getRace()
	{
		return getBaseTemplate().race;
	}

	/**
	 * @return the SP amount of the L2Player.<BR><BR>
	 */
	public int getSp()
	{
		return _sp;
	}

	/**
	 * Set the SP amount of the L2Player.<BR><BR>
	 */
	public void setSp(int sp)
	{
		if(sp < 0)
			sp = 0;
		_sp = sp;
	}

	/**
	 * @return the Clan Identifier of the L2Player.<BR><BR>
	 */
	public int getClanId()
	{
		return _clanId;
	}

	/**
	 * @return the Clan Crest Identifier (= Clan Identifier) of the L2Player or 0.<BR><BR>
	 */
	@Override
	public int getClanCrestId()
	{
		if(_clan != null)
			return _clan.getCrestId();
		return 0;
	}

	@Override
	public int getClanCrestLargeId()
	{
		if(_clan != null)
			return _clan.getCrestLargeId();
		return 0;
	}

	public long getLeaveClanTime()
	{
		return _leaveClanTime;
	}

	public long getDeleteClanTime()
	{
		return _deleteClanTime;
	}

	public void setLeaveClanTime(final long time)
	{
		_leaveClanTime = time;
	}

	public void setDeleteClanTime(final long time)
	{
		_deleteClanTime = time;
	}

	public void setOnlineTime(final long time)
	{
		_onlineTime = time;
		_onlineBeginTime = System.currentTimeMillis();
	}

	public void setNoChannel(final long time)
	{
		_NoChannel = time;
		if(_NoChannel > 2145909600000L || _NoChannel < 0)
			_NoChannel = -1;

		if(_NoChannel > 0)
			_NoChannelBegin = System.currentTimeMillis();
		else
			_NoChannelBegin = 0;

		sendPacket(new EtcStatusUpdate(this));
	}

	public long getNoChannel()
	{
		return _NoChannel;
	}

	public long getNoChannelRemained()
	{
		if(_NoChannel == 0)
			return 0;
		else if(_NoChannel < 0)
			return -1;
		else
		{
			long remained = _NoChannel - System.currentTimeMillis() + _NoChannelBegin;
			if(remained < 0)
				return 0;

			return remained;
		}
	}

	public void setLeaveClanCurTime()
	{
		_leaveClanTime = System.currentTimeMillis();
	}

	public void setDeleteClanCurTime()
	{
		_deleteClanTime = System.currentTimeMillis();
	}

	public boolean canJoinClan()
	{
		if(_leaveClanTime == 0)
			return true;
		if(System.currentTimeMillis() - _leaveClanTime >= Config.HoursBeforeJoinAClan * 60 * 60 * 1000)
		{
			_leaveClanTime = 0;
			return true;
		}
		return false;
	}

	public boolean canCreateClan()
	{
		if(_deleteClanTime == 0)
			return true;
		if(System.currentTimeMillis() - _deleteClanTime >= Config.DaysBeforeCreateAClan * 24 * 60 * 60 * 1000)
		{
			_deleteClanTime = 0;
			return true;
		}
		return false;
	}

	/**
	 * @return the PcInventory Inventory of the L2Player contained in _inventory.<BR><BR>
	 */
	@Override
	public PcInventory getInventory()
	{
		return _inventory;
	}

	/**
	 * Delete a item ShortCut of the L2Player _shortCuts.<BR><BR>
	 */
	public void removeItemFromShortCut(final int objectId)
	{
		_shortCuts.deleteShortCutByObjectId(objectId);
	}

	/**
	 * Delete a skill ShortCut of the L2Player _shortCuts.<BR><BR>
	 */
	public void removeSkillFromShortCut(final int skillId)
	{
		_shortCuts.deleteShortCutBySkillId(skillId);
	}

	/**
	 * @return True if the L2Player is sitting.<BR><BR>
	 */
	@Override
	public boolean isSitting()
	{
		return inObserverMode() || _isSitting;
	}

	public void setSitting(boolean val)
	{
		_isSitting = val;
	}

	public boolean getSittingTask()
	{
		return _sittingTask;
	}

	/**
	 * Sit down the L2Player, set the AI Intention to AI_INTENTION_REST and send a Server->Client ChangeWaitType packet (broadcast)<BR><BR>
	 */
	@Override
	public void sitDown()
	{
		if(isSitting() || _sittingTask || isAlikeDead())
			return;

		if(isStunned() || isSleeping() || isParalyzed() || isAttackingNow() || isCastingNow() || isMoving)
		{
			getAI().setNextAction(nextAction.REST, null, null, false, false);
			return;
		}

		resetWaitSitTime();
		getAI().setIntention(CtrlIntention.AI_INTENTION_REST, null, null);
		broadcastPacket(new ChangeWaitType(this, ChangeWaitType.WT_SITTING));
		_sittingTask = true;
		_isSitting = true;
		ThreadPoolManager.getInstance().scheduleGeneral(new EndSitDown(), 2500);
	}

	@Override
	public void standUp()
	{
		if(_isSitting && !_sittingTask && !isInStoreMode() && !isAlikeDead())
		{
			if(_relax)
			{
				setRelax(false);
				stopEffects(L2Effect.EffectType.RELAXING);
			}
			getAI().clearNextAction();
			broadcastPacket(new ChangeWaitType(this, ChangeWaitType.WT_STANDING));
			_sittingTask = true;
			_isSitting = true;
			ThreadPoolManager.getInstance().scheduleGeneral(new EndStandUp(), 2500);
		}
	}

   private GArray<String> bypasses = null, bypasses_bbs = null;
   private GArray<String> getStoredBypasses(boolean bbs)
	{
		if(bbs)
		{
			if(bypasses_bbs == null)
				bypasses_bbs = new GArray<String>();
			return bypasses_bbs;
		}
		if(bypasses == null)
			bypasses = new GArray<String>();
		return bypasses;
	}

	public void cleanBypasses(boolean bbs)
	{
		GArray<String> bypassStorage = getStoredBypasses(bbs);
		synchronized(bypassStorage)
		{
			bypassStorage.clearSize();
		}
	}

    class EndSitDown implements Runnable
	{
		EndSitDown()
		{}

		public void run()
		{
			_sittingTask = false;
			getAI().clearNextAction();
		}
	}

	class EndStandUp implements Runnable
	{
		EndStandUp()
		{}

		public void run()
		{
			_sittingTask = false;
			_isSitting = false;
			getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);
			getAI().setNextIntention();
		}
	}

	/**
	 * Set the value of the _relax value. Must be True if using skill Relax and False if not.
	 */
	public void setRelax(final boolean val)
	{
		_relax = val;
	}

	/** update time counter when L2Player is sitting. */
	public void updateWaitSitTime()
	{
		if(_waitTimeWhenSit < 200)
			_waitTimeWhenSit += 2;
	}

	/** getting how long L2Player is sitting. */
	public int getWaitSitTime()
	{
		return _waitTimeWhenSit;
	}

	/** reset time counter */
	public void resetWaitSitTime()
	{
		_waitTimeWhenSit = 0;
	}

	/**
	 * @return the PcWarehouse object of the L2Player.<BR><BR>
	 */
	public Warehouse getWarehouse()
	{
		return _warehouse;
	}

	/**
	 * @return the PcFreight object of the L2Player.<BR><BR>
	 */
	public Warehouse getFreight()
	{
		return _freight;
	}

	/**
	 * @return the Identifier of the L2Player.<BR><BR>
	 */
	public int getCharId()
	{
		return _charId;
	}

	/**
	 * Set the Identifier of the L2Player.<BR><BR>
	 */
	public void setCharId(final int charId)
	{
		_charId = charId;
	}

	/**
	 * @return the Adena amount of the L2Player.<BR><BR>
	 */
	public int getAdena()
	{
		return _inventory.getAdena();
	}

	/**
	 * Р—Р°Р±РёСЂР°РµС‚ Р°РґРµРЅСѓ Сѓ РёРіСЂРѕРєР°.<BR><BR>
	 * @param adena - СЃРєРѕР»СЊРєРѕ Р°РґРµРЅС‹ Р·Р°Р±СЂР°С‚СЊ
	 * @return L2ItemInstance - РѕСЃС‚Р°С‚РѕРє Р°РґРµРЅС‹
	 */
	public L2ItemInstance reduceAdena(long adena, boolean notify)
	{
		if(notify && adena > 0)
		{
			sendPacket(new SystemMessage(SystemMessage.S1_ADENA_DISAPPEARED).addNumber(adena));
		}
		return _inventory.reduceAdena(adena);
	}

	/**
	 * Р”РѕР±Р°РІР»СЏРµС‚ Р°РґРµРЅСѓ РёРіСЂРѕРєСѓ.<BR><BR>
	 * @param adena - СЃРєРѕР»СЊРєРѕ Р°РґРµРЅС‹ РґР°С‚СЊ
	 * @return L2ItemInstance - РЅРѕРІРѕРµ РєРѕР»РёС‡РµСЃС‚РІРѕ Р°РґРµРЅС‹
	 * TODO РґРѕР±Р°РІРёС‚СЊ РїР°СЂР°РјРµС‚СЂ update РєР°Рє РІ reduceAdena
	 */
	public L2ItemInstance addAdena(final long adena)
	{
		return _inventory.addAdena(adena);
	}

	/**
	 * @return the active connection with the client.<BR><BR>
	 */
	public L2GameClient getNetConnection()
	{
		return _connection;
	}

	/**
	 * Set the active connection with the client.<BR><BR>
	 */
	public void setNetConnection(final L2GameClient connection)
	{
		_connection = connection;
	}

	/**
	 * Close the active connection with the client.<BR><BR>
	 */
	public void closeNetConnection()
	{
		if(_connection != null)
			_connection.closeNow(false);
	}

	/**
	 * Manage actions when a player click on this L2Player.<BR><BR>
	 *
	 * <B><U> Actions on first click on the L2Player (Select it)</U> :</B><BR><BR>
	 * <li>Set the target of the player</li>
	 * <li>Send a Server->Client packet MyTargetSelected to the player (display the select window)</li><BR><BR>
	 *
	 * <B><U> Actions on second click on the L2Player (Follow it/Attack it/Intercat with it)</U> :</B><BR><BR>
	 * <li>Send a Server->Client packet MyTargetSelected to the player (display the select window)</li>
	 * <li>If this L2Player has a Private Store, notify the player AI with AI_INTENTION_INTERACT</li>
	 * <li>If this L2Player is autoAttackable, notify the player AI with AI_INTENTION_ATTACK</li><BR><BR>
	 * <li>If this L2Player is NOT autoAttackable, notify the player AI with AI_INTENTION_FOLLOW</li><BR><BR>
	 *
	 * <B><U> Example of use </U> :</B><BR><BR>
	 * <li> Client packet : Action, AttackRequest</li><BR><BR>
	 *
	 * @param player The player that start an action on this L2Player
	 *
	 */
	@Override
	public void onAction(final L2Player player)
	{
		if(Events.onAction(player, this))
			return;

		// Check if the L2Player is confused
		if(player.isConfused() || player.isBlocked())
		{
			player.sendActionFailed();
			return;
		}

		if(Config.DEBUG)
			_log.info("player.getTarget()=" + player.getTarget() + "; this=" + this);

		// Check if the other player already target this L2Player
		if(player.getTarget() != this)
		{
			// Set the target of the player
			player.setTarget(this);

			// The color to display in the select window is White
			player.sendPacket(new MyTargetSelected(getObjectId(), 0));
		}
		else // Check if this L2Player has a Private Store
		if(getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
			if(getDistance(player) > INTERACTION_DISTANCE && getAI().getIntention() != CtrlIntention.AI_INTENTION_INTERACT)
				player.getAI().setIntention(CtrlIntention.AI_INTENTION_INTERACT, this, null);
			else
				player.doInteract(this);
		else if(isAutoAttackable(player))
		{
			// Player with lvl < 21 can't attack a cursed weapon holder
			// And a cursed weapon holder	can't attack players with lvl < 21
			if(isCursedWeaponEquipped() && player.getLevel() < 21 || player.isCursedWeaponEquipped() && getLevel() < 21)
				player.sendActionFailed();
			else
				player.getAI().Attack(this, false);
		}
		else if(player != this)
			player.getAI().setIntention(CtrlIntention.AI_INTENTION_FOLLOW, this, 100);
		else
			sendActionFailed();
	}

	/**
	 * Send packet StatusUpdate with current HP,MP and CP to the L2Player and only current HP, MP and Level to all other L2Player of the Party.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Send the Server->Client packet StatusUpdate with current HP, MP and CP to this L2Player </li><BR>
	 * <li>Send the Server->Client packet PartySmallWindowUpdate with current HP, MP and Level to all other L2Player of the Party </li><BR><BR>
	 *
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : This method DOESN'T SEND current HP and MP to all L2Player of the _statusListener</B></FONT><BR><BR>
	 *
	 */
	@Override
	public void broadcastStatusUpdate()
	{
		// Send the Server->Client packet StatusUpdate with current HP and MP to all L2Player that must be informed of HP/MP updates of this L2Player
		if(Config.FORCE_STATUSUPDATE)
			super.broadcastStatusUpdate();
		else if(!needStatusUpdate()) //РџРѕ РёРґРµРµ РµС€Рµ РґРѕР»Р¶РЅРѕ СЃСЂРµР·Р°С‚СЊ С‚СЂР°С„С„РёРє. Р‘СѓРґСѓС‚ РіР»СЋРєРё СЃ РѕС‚РѕР±СЂР°Р¶РµРЅРёРµРј - СѓР±СЂР°С‚СЊ СЌС‚Рѕ СѓСЃР»РѕРІРёРµ.
			return;

		// Send the Server->Client packet StatusUpdate with current HP, MP and CP to this L2Player
		final StatusUpdate su = new StatusUpdate(getObjectId());
		su.addAttribute(StatusUpdate.CUR_HP, (int) _currentHp);
		su.addAttribute(StatusUpdate.CUR_MP, (int) _currentMp);
		su.addAttribute(StatusUpdate.CUR_CP, (int) _currentCp);
		sendPacket(su);

		// Check if a party is in progress
		if(isInParty())
			// Send the Server->Client packet PartySmallWindowUpdate with current HP, MP and Level to all other L2Player of the Party
			getParty().broadcastToPartyMembers(this, new PartySmallWindowUpdate(this));

		if(getDuel() != null)
			getDuel().broadcastToOppositTeam(this, new ExDuelUpdateUserInfo(this));

		if(isInOlympiadMode())
		{
			for(L2Player player : L2World.getAroundPlayers(this))
				if(player.getOlympiadGameId() == getOlympiadGameId())
					player.sendPacket(new ExOlympiadUserInfo(this));

			if(Olympiad.getInstance().getSpectators(_olympiadGameId) != null)
				for(final L2Player spectator : Olympiad.getInstance().getSpectators(_olympiadGameId))
					if(spectator != null)
						spectator.sendPacket(new ExOlympiadUserInfoSpectator(this, getOlympiadSide()));
		}
	}

	/**
	 * РћС‚РїСЂР°РІР»СЏРµС‚ UserInfo РґР°РЅРѕРјСѓ РёРіСЂРѕРєСѓ Рё CharInfo РІСЃРµРј РѕРєСЂСѓР¶Р°СЋС‰РёРј.<BR><BR>
	 *
	 * <B><U> РљРѕРЅС†РµРїС‚</U> :</B><BR><BR>
	 * РЎРµСЂРІРµСЂ С€Р»РµС‚ РёРіСЂРѕРєСѓ UserInfo.
	 * РЎРµСЂРІРµСЂ РІС‹Р·С‹РІР°РµС‚ РјРµС‚РѕРґ {@link L2Player#broadcastPacketToOthers(l2r.gameserver.serverpackets.L2GameServerPacket)} РґР»СЏ СЂР°СЃСЃС‹Р»РєРё CharInfo<BR><BR>
	 *
	 * <B><U> Р”РµР№СЃС‚РІРёСЏ</U> :</B><BR><BR>
	 * <li>РћС‚СЃС‹Р»РєР° РёРіСЂРѕРєСѓ UserInfo(Р»РёС‡РЅС‹Рµ Рё РѕР±С‰РёРµ РґР°РЅРЅС‹Рµ)</li>
	 * <li>РћС‚СЃС‹Р»РєР° РґСЂСѓРіРёРј РёРіСЂРѕРєР°Рј CharInfo(Public data only)</li><BR><BR>
	 *
	 * <FONT COLOR=#FF0000><B> <U>Р’РЅРёРјР°РЅРёРµ</U> : РќР• РџРћРЎР«Р›РђР™РўР• UserInfo РґСЂСѓРіРёРј РёРіСЂРѕРєР°Рј Р»РёР±Рѕ CharInfo РґР°РЅРѕРјСѓ РёРіСЂРѕРєСѓ.<BR>
	 * РќР• Р’Р«Р—Р«Р’РђР•Р™РўР• Р­РўРћРў РњР•РўРћР” РљР РћРњР• РћРЎРћР‘Р«РҐ РћР‘РЎРўРћРЇРўР•Р›Р¬РЎРўР’(СЃРјРµРЅР° СЃР°Р±РєР»Р°СЃСЃР° Рє РїСЂРёРјРµСЂСѓ)!!! РўСЂР°С„С„РёРє РґРёРєРѕ РєСѓС€Р°РµС‚СЃСЏ Сѓ РёРіСЂРѕРєРѕРІ Рё РЅР°С‡РёРЅР°СЋС‚СЃСЏ Р»Р°РіРё.<br>
	 * Р�СЃРїРѕР»СЊР·СѓР№С‚Рµ РјРµС‚РѕРґ {@link l2r.gameserver.model.L2Player#sendChanges()}</B></FONT><BR><BR>
	 *
	 * @see l2r.gameserver.serverpackets.UserInfo
	 * @see l2r.gameserver.serverpackets.CharInfo
	 */
	@Override
	public void broadcastUserInfo()
	{
		sendPacket(new UserInfo(this));

		if(isInvisible())
			return;

		CharInfo charInfo = new CharInfo(this);

		Ride ride = null;
		if(isMounted())
			ride = new Ride(this);

		FastList<L2Player> others = L2World.getAroundPlayers(this);
		for(FastList.Node<L2Player> n = others.head(), end = others.tail(); (n = n.getNext()) != end;)
		{
			L2Player target = n.getValue();
			if(_objectId != target.getObjectId())
			{
				target.sendPacket(charInfo);
				if(ride != null)
					target.sendPacket(ride);
				if(_clan != null && target.getClan() != null && target.getClan().isAtWarWith(_clan.getClanId()))
					target.sendPacket(new RelationChanged(this, isAutoAttackable(target), getRelation(target)));
			}
		}
	}

	/**
	 * @return the Alliance Identifier of the L2Player.<BR><BR>
	 */
	public int getAllyId()
	{
		return _clan == null ? 0 : _clan.getAllyId();
	}

	@Override
	public int getAllyCrestId()
	{
		return getAlliance() == null ? 0 : getAlliance().getAllyCrestId();
	}

	@Override
	public void onForcedAttack(L2Player player)
	{
		if(!player.getPlayerAccess().PeaceAttack && (player.isInZonePeace() || isInZonePeace()))
			if(!Config.ALT_GAME_KARMAPLAYERCANBEKILLEDINPEACEZONE || player.getKarma() <= 0 && getKarma() <= 0)
			{
				player.sendPacket(Msg.YOU_CANNOT_ATTACK_THE_TARGET_IN_THE_PEACE_ZONE);
				player.sendActionFailed();
				return;
			}
		if(player.isInOlympiadMode() && !player.isOlympiadCompStart())
		{
			player.sendActionFailed();
			return;
		}
		super.onForcedAttack(player);
	}

	/**
	 * Send a Server->Client packet StatusUpdate to the L2Player.<BR><BR>
	 */
	@Override
	public void sendPacket(final L2GameServerPacket packets)
	{
		if(_isConnected)
			try
			{
				if(_connection != null)
					_connection.sendPacket(packets);
			}
			catch(final Exception e)
			{
				_log.log(Level.INFO, "", e);
				e.printStackTrace();
			}
	}
	/**
	 * Manage Interact Task with another L2Player.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>If the private store is a STORE_PRIVATE_SELL, send a Server->Client PrivateBuyListSell packet to the L2Player</li>
	 * <li>If the private store is a STORE_PRIVATE_BUY, send a Server->Client PrivateBuyListBuy packet to the L2Player</li>
	 * <li>If the private store is a STORE_PRIVATE_MANUFACTURE, send a Server->Client RecipeShopSellList packet to the L2Player</li><BR><BR>
	 *
	 * @param target The L2Character targeted
	 */
	public void doInteract(final L2Object target)
	{
		if(target == null)
			return;
		if(target.isPlayer)
		{
			if(target.getDistance(this) <= INTERACTION_DISTANCE)
			{
				final L2Player temp = (L2Player) target;

				if(temp.getPrivateStoreType() == STORE_PRIVATE_SELL || temp.getPrivateStoreType() == STORE_PRIVATE_SELL_PACKAGE)
				{
					sendPacket(new PrivateStoreListSell(this, temp));
					sendActionFailed();
				}
				else if(temp.getPrivateStoreType() == STORE_PRIVATE_BUY)
				{
					sendPacket(new PrivateStoreListBuy(this, temp));
					sendActionFailed();
				}
				else if(temp.getPrivateStoreType() == STORE_PRIVATE_MANUFACTURE)
				{
					sendPacket(new RecipeShopSellList(this, temp));
					sendActionFailed();
				}
				sendActionFailed();
			}
			else if(getAI().getIntention() != CtrlIntention.AI_INTENTION_INTERACT)
				getAI().setIntention(CtrlIntention.AI_INTENTION_INTERACT, this, null);
		}
		else
			target.onAction(this);
	}

	/**
	 * Manage AutoLoot Task.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Send a System Message to the L2Player : YOU_PICKED_UP_S1_ADENA or YOU_PICKED_UP_S1_S2</li>
	 * <li>Add the Item to the L2Player inventory</li>
	 * <li>Send a Server->Client packet StatusUpdate to this L2Player with current weight</li><BR><BR>
	 *
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : If a Party is in progress, distribute Items between party members</B></FONT><BR><BR>
	 *
	 * @param item The L2ItemInstance dropped
	 * @param fromNpc L2NpcIntance that dropped
	 */
	public void doAutoLoot(L2ItemInstance item, L2NpcInstance fromNpc)
	{
		// Herbs
		if(item.isHerb())
		{
			altUseSkill(item.getItem().getFirstSkill(), this);
			item.decayMe();
			L2World.removeObject(item);
			return;
		}

		// Check if the L2Player is in a Party
		if(!isInParty())
		{
			if(!getInventory().validateWeight(item))
			{
				sendActionFailed();
				sendPacket(Msg.YOU_HAVE_EXCEEDED_THE_WEIGHT_LIMIT);
				item.dropToTheGround(this, fromNpc);
				return;
			}

			if(!getInventory().validateCapacity(item))
			{
				sendActionFailed();
				sendPacket(Msg.YOUR_INVENTORY_IS_FULL);
				item.dropToTheGround(this, fromNpc);
				return;
			}

			// Send a System Message to the L2Player
			if(item.getItemId() == 57)
				sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S1_ADENA).addNumber((int) item.getCount()));
			else if(item.getCount() == 1)
			{
				final SystemMessage smsg = new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S1); // you picked up $s1
				smsg.addItemName(item.getItemId());
				sendPacket(smsg);
			}
			else
			{
				final SystemMessage smsg = new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S2_S1); // you picked up $s1$s2
				smsg.addItemName(item.getItemId());
				smsg.addNumber(item.getCount());
				sendPacket(smsg);
			}

			// Add the Item to the L2Player inventory
			L2ItemInstance target2 = getInventory().addItem(item);
			Log.LogItem(this, fromNpc, Log.GetItemByAutoLoot, target2);

			sendChanges();
		}
		else if(item.getItemId() == 57)
			// Distribute Adena between Party members
			getParty().distributeAdena(item, fromNpc, this);
		else
			// Distribute Item between Party members
			getParty().distributeItem(this, item, fromNpc);

		broadcastPickUpMsg(item);
	}

	/**
	 * РћРїРѕРІРµС‰Р°РµС‚ РґСЂСѓРіРёС… РёРіСЂРѕРєРѕРІ Рѕ РїРѕРґРЅСЏС‚РёРё РІРµС‰Рё
	 * @param item РїСЂРµРґРјРµС‚ РєРѕС‚РѕСЂС‹Р№ Р±С‹Р» РїРѕРґРЅСЏС‚
	 */
	public void broadcastPickUpMsg(L2ItemInstance item)
	{
		if(item != null && item.isEquipable() && !(item.getItem() instanceof L2EtcItem))
		{
			SystemMessage msg;
			if(item.getEnchantLevel() > 0)
			{
				msg = new SystemMessage(SystemMessage.ATTENTION_S1_PICKED_UP__S2_S3);
				msg.addString(getName());
				msg.addNumber(item.getEnchantLevel());
				msg.addItemName(item.getItemId());
			}
			else
			{
				msg = new SystemMessage(SystemMessage.ATTENTION_S1_PICKED_UP_S2);
				msg.addString(getName());
				msg.addItemName(item.getItemId());
			}
			sendPacket(msg); // РЎРµР±СЏ С‚РѕР¶Рµ РѕРїРѕРІРµС‰Р°РµС‚
			broadcastPacketToOthers(msg);
		}
	}

	/**
	 * Manage Pickup Task.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Send a Server->Client packet StopMove to this L2Player </li>
	 * <li>Remove the L2ItemInstance from the world and send server->client GetItem packets </li>
	 * <li>Send a System Message to the L2Player : YOU_PICKED_UP_S1_ADENA or YOU_PICKED_UP_S1_S2</li>
	 * <li>Add the Item to the L2Player inventory</li>
	 * <li>Send a Server->Client packet StatusUpdate to this L2Player with current weight</li><BR><BR>
	 *
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : If a Party is in progress, distribute Items between party members</B></FONT><BR><BR>
	 *
	 * @param object The L2ItemInstance to pick up
	 *
	 */
	@Override
	public void doPickupItem(final L2Object object)
	{
		getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);

		// Check if the L2Object to pick up is a L2ItemInstance
		if(!(object instanceof L2ItemInstance))
		{
			// dont try to pickup anything that is not an item :)
			_log.warning("trying to pickup wrong target." + getTarget());
			return;
		}

		final L2ItemInstance item = (L2ItemInstance) object;

		sendActionFailed();

		if(Config.DEBUG)
			_log.fine("pickup pos: " + item.getX() + " " + item.getY() + " " + item.getZ());

		sendPacket(new StopMove(this));

		synchronized (item)
		{
			// Check if me not owner of item and, if in party, not in owner party and nonowner pickup delay still active
			if(item.getDropTimeOwner() != 0 && item.getItemDropOwner() != null && item.getDropTimeOwner() + Config.NONOWNER_ITEM_PICKUP_DELAY * 1000 > System.currentTimeMillis() && this != item.getItemDropOwner() && (!isInParty() || isInParty() && item.getItemDropOwner().isInParty() && getParty() != item.getItemDropOwner().getParty()))
			{
				SystemMessage sm;
				if(item.getItemId() == 57)
				{
					sm = new SystemMessage(SystemMessage.FAILED_TO_PICK_UP_S1_ADENA);
					sm.addNumber(item.getCount());
				}
				else
				{
					sm = new SystemMessage(SystemMessage.FAILED_TO_PICK_UP_S1);
					sm.addItemName(item.getItemId());
				}
				sendPacket(sm);
				sendActionFailed();
				return;
			}

			if(!item.isVisible())
			{
				sendActionFailed();
				return;
			}

			if(item.isFortFlag() && !FortressSiegeManager.getInstance().checkIfCanPickup(this))
			{
				sendPacket(new SystemMessage(SystemMessage.FAILED_TO_PICK_UP_S1).addItemName(item.getItemId()));
				return;
			}

			// Herbs
			if(item.isHerb())
			{
				altUseSkill(item.getItem().getFirstSkill(), this);
				if(getPet() != null && !getPet().isDead() && (item.getItemId() <= 8605 || item.getItemId() == 8614))
					getPet().altUseSkill(item.getItem().getFirstSkill(), getPet());
				item.decayMe();
				L2World.removeObject(item);
				broadcastPacket(new GetItem(item, getObjectId()));
				return;
			}

			if(!isInParty())
			{
				if(!_inventory.validateWeight(item))
				{
					sendActionFailed();
					sendPacket(Msg.YOU_HAVE_EXCEEDED_THE_WEIGHT_LIMIT);
					return;
				}

				if(!_inventory.validateCapacity(item))
				{
					sendActionFailed();
					sendPacket(Msg.YOUR_INVENTORY_IS_FULL);
					return;
				}

				SystemMessage smsg;
				if(item.getItemId() == 57)
				{
					smsg = new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S1_ADENA);
					smsg.addNumber(item.getIntegerLimitedCount());
				}
				else if(item.getIntegerLimitedCount() == 1)
				{
					if(item.getEnchantLevel() > 0)
					{
						smsg = new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED__S1S2);
						smsg.addNumber(item.getEnchantLevel());
						smsg.addItemName(item.getItemId());
					}
					else
					{
						smsg = new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S1);
						smsg.addItemName(item.getItemId());
					}
				}
				else
				{
					smsg = new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S2_S1);
					smsg.addItemName(item.getItemId());
					smsg.addNumber(item.getIntegerLimitedCount());
				}
				sendPacket(smsg);

				item.pickupMe(this);

				L2ItemInstance target2 = getInventory().addItem(item);
				Log.LogItem(this, Log.PickupItem, target2);

				sendChanges();
			}
			else if(item.getItemId() == 57)
			{
				item.pickupMe(this);
				getParty().distributeAdena(item, this);
			}
			else
			{
				// РќСѓР¶РЅРѕ РѕР±СЏР·Р°С‚РµР»СЊРЅРѕ СЃРЅР°С‡Р°Р»Рѕ СѓРґР°Р»РёС‚СЊ РїСЂРµРґРјРµС‚ СЃ Р·РµРјР»Рё.
				item.pickupMe(null);
				getParty().distributeItem(this, item);
			}

			broadcastPacket(new GetItem(item, getObjectId()));
			broadcastPickUpMsg(item);
		}

	}

	/**
	 * Set a target.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Remove the L2Player from the _statusListener of the old target if it was a L2Character </li>
	 * <li>Add the L2Player to the _statusListener of the new target if it's a L2Character </li>
	 * <li>Target the new L2Object (add the target to the L2Player _target, _knownObject and L2Player to _KnownObject of the L2Object)</li><BR><BR>
	 *
	 * @param newTarget The L2Object to target
	 *
	 */
	@Override
	public void setTarget(L2Object newTarget)
	{
		// Check if the new target is visible
		if(newTarget != null && !newTarget.isVisible())
			newTarget = null;

		// Get the current target
		final L2Object oldTarget = getTarget();
		if(oldTarget != null)
		{
			if(oldTarget.equals(newTarget))
				return; // no target change

			// Remove the L2Player from the _statusListener of the old target if it was a L2Character
			if(oldTarget.isCharacter)
				((L2Character) oldTarget).removeStatusListener(this);
		}

		// Add the L2Player to the _statusListener of the new target if it's a L2Character
		if(newTarget != null && newTarget.isCharacter)
		{
			((L2Character) newTarget).addStatusListener(this);
			broadcastPacket(new TargetSelected(getObjectId(), newTarget.getObjectId(), getLoc()));
		}

		// Target the new L2Object (add the target to the L2Player _target, _knownObject and L2Player to _KnownObject of the L2Object)
		super.setTarget(newTarget);
	}

	/**
	 * @return the active weapon instance (always equipped in the right hand).<BR><BR>
	 */
	@Override
	public L2ItemInstance getActiveWeaponInstance()
	{
		return getInventory().getPaperdollItem(Inventory.PAPERDOLL_RHAND);
	}

	/**
	 * @return the active weapon item (always equipped in the right hand).<BR><BR>
	 */
	@Override
	public L2Weapon getActiveWeaponItem()
	{
		final L2ItemInstance weapon = getActiveWeaponInstance();

		if(weapon == null)
			return getFistsWeaponItem();

		return (L2Weapon) weapon.getItem();
	}

	/**
	 * @return the secondary weapon instance (always equipped in the left hand).<BR><BR>
	 */
	@Override
	public L2ItemInstance getSecondaryWeaponInstance()
	{
		return getInventory().getPaperdollItem(Inventory.PAPERDOLL_LHAND);
	}

	/**
	 * @return the secondary weapon item (always equipped in the left hand) or the fists weapon.<BR><BR>
	 */
	@Override
	public L2Weapon getSecondaryWeaponItem()
	{
		final L2ItemInstance weapon = getSecondaryWeaponInstance();

		if(weapon == null)
			return getFistsWeaponItem();

		final L2Item item = weapon.getItem();

		if(item instanceof L2Weapon)
			return (L2Weapon) item;

		return null;
	}

	public boolean isWearingArmor(final ArmorType armorType)
	{
		final L2ItemInstance chest = getInventory().getPaperdollItem(Inventory.PAPERDOLL_CHEST);

		if(chest == null)
			return armorType == ArmorType.NONE;

		if(chest.getItemType() != armorType)
			return false;

		if(chest.getBodyPart() == L2Item.SLOT_FULL_ARMOR)
			return true;

		final L2ItemInstance legs = getInventory().getPaperdollItem(Inventory.PAPERDOLL_LEGS);

		return legs == null ? armorType == ArmorType.NONE : legs.getItemType() == armorType;
	}

	@Override
	public void reduceCurrentHp(double i, L2Character attacker, boolean awake, boolean standUp, boolean directHp)
	{
		// Check if the L2Player isn't Invulnerable
		if(isDead() || isInvul())
			return;

		if(getDuel() != null && i >= getCurrentHp())
		{
			getDuel().onPlayerDefeat(this);
			if(getDuel() == attacker.getDuel())
			{
				i = getCurrentHp() - 1;
				getDuel().onPlayerDefeat(this);
				getDuel().stopFighting((L2Playable) attacker);
			}
		}

		if(isInOlympiadMode())
		{
			if(i >= getCurrentHp())
			{
				i = getCurrentHp() - 1;
				Olympiad.getInstance().getOlympiadGame(getOlympiadGameId()).setWinner(getOlympiadSide());
				Olympiad.getInstance().startValidateWinner(getOlympiadGameId(), 0);
				Olympiad.getInstance().stopFighting((L2Playable) attacker);
			}
			else
			{
				Olympiad.getInstance().getOlympiadGame(getOlympiadGameId()).addDamage(getOlympiadSide(), i);
			}
		}

		if(attacker instanceof L2Playable && isInZoneBattle() != attacker.isInZoneBattle())
		{
			attacker.getPlayer().sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		double trans = calcStat(Stats.TRANSFER_DAMAGE_PERCENT, 0, null, null);
		if(trans > 1)
			if(_summon != null && !_summon.isDead())
			{
				if(_summon.isInRange(this, 1200))
				{
					trans *= i / 100;
					i -= trans;
					_summon.reduceCurrentHp(trans, attacker, false, false, false);
				}
			}
			else
				stopEffect(L2Skill.SKILL_TRANSFER_PAIN);

		if(attacker != null && attacker != this)
		{
			SystemMessage smsg = new SystemMessage(SystemMessage.S1_HIT_YOU_FOR_S2_DAMAGE);
			if(attacker instanceof L2NpcInstance)
				smsg.addNpcName(((L2NpcInstance) attacker).getTemplate().npcId);
			else
				smsg.addString(attacker.getName());

			smsg.addNumber((int) i);
			sendPacket(smsg);
		}
		// Reduce the current HP of the L2Player
		super.reduceCurrentHp(i, attacker, awake, standUp, directHp);

		//TODO: РїРµСЂРµРґРµР»Р°С‚СЊ РЅР° Р»РёСЃС‚РµРЅРµСЂ
		if(getLevel() < 6 && getCurrentHp() < getMaxHp() / 4)
		{
			Quest q = QuestManager.getQuest(255);
			if(q != null)
				processQuestEvent(q.getName(), "CE45");
		}
	}

	private void altDeathPenalty(final L2Character killer)
	{
		// Reduce the Experience of the L2Player in function of the calculated Death Penalty
		if(!Config.ALT_GAME_DELEVEL)
			return;
		if(isInZoneBattle() && killer instanceof L2Playable)
			return;
		if(killer instanceof L2Playable)
			deathPenalty(atWarWith(killer.getPlayer()));
		else
			deathPenalty(false);
	}

	public final boolean atWarWith(final L2Player player)
	{
		return _clan != null && player.getClan() != null && getPledgeType() != -1 && player.getPledgeType() != -1 && _clan.isAtWarWith(player.getClan().getClanId());
	}

	public boolean atMutualWarWith(L2Player player)
	{
		return _clan != null && player.getClan() != null && getPledgeType() != -1 && player.getPledgeType() != -1 && _clan.isAtWarWith(player.getClan().getClanId()) && player.getClan().isAtWarWith(_clan.getClanId());
	}

	public final void doPurePk(final L2Player killer)
	{
		// Check if the attacker has a PK counter greater than 0
		final int pkCountMulti = Math.max(killer.getPkKills() / 2, 1);

		// Calculate the level difference Multiplier between attacker and killed L2Player
		//final int lvlDiffMulti = Math.max(killer.getLevel() / _level, 1);

		// Calculate the new Karma of the attacker : newKarma = baseKarma*pkCountMulti*lvlDiffMulti
		// Add karma to attacker and increase its PK counter
		killer.increaseKarma(Config.KARMA_MIN_KARMA * pkCountMulti); // * lvlDiffMulti);
		killer.setPkKills(killer.getPkKills() + 1);
	}

	public final void doKillInPeace(final L2Player killer)
	{ // Check if the L2Player killed haven't Karma
		if(_karma <= 0)
			doPurePk(killer);
		else
			// Dead player have Karma
			// Increase the PvP Kills counter (Number of player killed during a PvP)
			killer.setPvpKills(killer.getPvpKills() + 1);
	}

	public static ArrayList<L2ItemInstance> checkAddItemToDrop(final ArrayList<L2ItemInstance> array, final L2ItemInstance item, final double rate)
	{
		//		_log.info("checkAddItemToDrop("+array+", "+item+", "+rate+");");

		if(Rnd.get() >= rate)
			return array;

		if(Config.KARMA_LIST_NONDROPPABLE_ITEMS.contains(item.getItemId()))
			return array;

		array.add(item);

		return array;
	}

	/**
	 * Pk/PvP situation checking and managing after death<BR><BR>
	 *
	 * <B><U> Actions</U> :</B>
	 * <li>Manage Karma gain for attacker and Karma loss for the killed L2Player </li>
	 * <li>If the killed L2Player has Karma, manage Drop Item</li><BR>
	 *
	 * @param killer The L2Character who attacks
	 */
	protected void doPKPVPManage(L2Character killer)
	{
		if(killer == null || killer == _summon)
			return;

		if(killer.getObjectId() == _objectId)
			return;

		if(isInZoneBattle() || killer.isInZoneBattle())
			return;

		if(killer instanceof L2Summon)
			killer = killer.getOwner();

		// Processing Karma/PKCount/PvPCount for killer
		if(killer.isPlayer)
		{
			final L2Player pk = (L2Player) killer;
			final int repValue = getLevel() - pk.getLevel() >= 20 ? 2 : 1;
			boolean war = atMutualWarWith(pk);

			if(Config.ALT_HONOR_SYSTEM && pk.getLevel() - getLevel() < 20)
				pk.getInventory().addItem(775, 3, getObjectId(), "PKPVPManage");

			if(getLevel() > 4 && _clan != null && pk.getClan() != null)
				if(war || _clan.getSiege() != null && _clan.getSiege() == pk.getClan().getSiege() && (_clan.isDefender() && pk.getClan().isAttacker() || _clan.isAttacker() && pk.getClan().isDefender()))
					if(pk.getClan().getReputationScore() > 0 && _clan.getLevel() >= 5 && _clan.getReputationScore() > 0 && pk.getClan().getLevel() >= 5)
					{
						_clan.broadcastToOtherOnlineMembers(new SystemMessage(SystemMessage.YOUR_CLAN_MEMBER_S1_WAS_KILLED_S2_POINTS_HAVE_BEEN_DEDUCTED_FROM_YOUR_CLAN_REPUTATION_SCORE_AND_ADDED_TO_YOUR_OPPONENT_CLAN_REPUTATION_SCORE).addString(getName()).addNumber(-_clan.incReputation(-repValue, true, "ClanWar")), this);
						pk.getClan().broadcastToOtherOnlineMembers(new SystemMessage(SystemMessage.FOR_KILLING_AN_OPPOSING_CLAN_MEMBER_S1_POINTS_HAVE_BEEN_DEDUCTED_FROM_YOUR_OPPONENTS_CLAN_REPUTATION_SCORE).addNumber(pk.getClan().incReputation(repValue, true, "ClanWar")), pk);
						if(Config.ALT_HONOR_SYSTEM && pk.getLevel() - getLevel() < 20)
						{
							getInventory().addItem(4674, 1, pk.getObjectId(), "PKPVPManage");
							pk.getInventory().addItem(4674, 3, getObjectId(), "PKPVPManage");
						}
					}
					else if(Config.ALT_HONOR_SYSTEM && war && pk.getLevel() - getLevel() < 20)
					{
						getInventory().addItem(4674, 1, pk.getObjectId(), "PKPVPManage");
						pk.getInventory().addItem(4674, 3, getObjectId(), "PKPVPManage");
					}

			if(isInZone(ZoneType.Siege))
				return;

			if(_pvpFlag > 0 || war)
				pk.setPvpKills(pk.getPvpKills() + 1);
			else
				doKillInPeace(pk);

			// Send a Server->Client UserInfo packet to attacker with its PvP Kills Counter
			pk.sendPacket(new UserInfo(pk));
		}

		// KARMA LOSS FOR DEAD PLAYER
		// when a PKer gets killed by another player or a L2MonsterInstance, it loseq a certain amount of Karma based on their level.
		// this (with defaults) results in a level 1 losing about ~2 karma per death, and a lvl 70 loses about 11760 karma per death...
		// You lose karma as long as you were not in a pvp zone and you did not kill urself.
		// No item drop in PvP without Karma and for players with less than 5 PK kills

		int karma = _karma;
		decreaseKarma(Config.KARMA_LOST_BASE);

		if(_pkKills < Config.MIN_PK_TO_ITEMS_DROP || karma == 0 && Config.KARMA_NEEDED_TO_DROP || isFestivalParticipant())
			return;

		// No drop from GM's
		if(!Config.KARMA_DROP_GM && isGM())
			return;

		final int max_drop_count = Config.KARMA_DROP_ITEM_LIMIT;

		double dropRate = (double) _pkKills * Config.KARMA_DROPCHANCE_MULTIPLIER;

		if(dropRate < Config.KARMA_DROPCHANCE_MINIMUM)
			dropRate = Config.KARMA_DROPCHANCE_MINIMUM;

		dropRate /= 100.0;

		final double dropEquipRate = Config.KARMA_DROPCHANCE_EQUIPMENT / 100.0;
		final double dropWeaponRate = Config.KARMA_DROPCHANCE_EQUIPPED_WEAPON / 100.0;
		final double dropItemRate = Math.max(0, 1.0 - (dropEquipRate + dropWeaponRate));

		// _log.info("dropEquipRate=" + dropEquipRate + "; dropWeaponRate=" + dropWeaponRate + "; dropItemRate=" + dropItemRate);

		ArrayList<L2ItemInstance> dropped_items = new ArrayList<L2ItemInstance>();
		// Items to check = max_drop_count * 3; (Inventory + Weapon + Equipment)
		if(_inventory.getSize() > 0)
			for(int i = 0, cycles = 100; i < max_drop_count * 3 && cycles > 0 && dropped_items.size() < max_drop_count; cycles--)
			{
				final L2ItemInstance random_item = _inventory.getItems()[Rnd.get(_inventory.getSize())];

				if(random_item.isAugmented() || random_item.isShadowItem() || random_item.isHeroItem())
					continue;

				if(random_item.getItem().getType1() == L2Item.TYPE1_ITEM_QUESTITEM_ADENA)
					continue;

				if(random_item.getItem().getType2() == L2Item.TYPE2_QUEST)
					continue;

				if(dropped_items.contains(random_item))
					continue;

				i++;

				if(random_item.isEquipped())
				{
					if(random_item.getItem().getType2() == L2Item.TYPE2_WEAPON)
						dropped_items = checkAddItemToDrop(dropped_items, random_item, dropWeaponRate * dropRate);
					else
						dropped_items = checkAddItemToDrop(dropped_items, random_item, dropEquipRate * dropRate);
				}
				else
					dropped_items = checkAddItemToDrop(dropped_items, random_item, dropItemRate * dropRate);
			}

		// Dropping items, if present
		if(dropped_items.isEmpty())
			return;

		for(L2ItemInstance item : dropped_items)
		{
			final long count = item.getCount();
			if(item.isEquipped())
			{
				final short dropslot = item.getEquipSlot();
				getInventory().unEquipItemInSlot(dropslot);
				getInventory().setPaperdollItem(dropslot, null);
			}

			item = _inventory.dropItem(item, count);

			if(Config.MOBSLOOTERS && killer.isMonster && !CursedWeaponsManager.getInstance().isCursed(item.getItemId()))
				if(killer instanceof L2MinionInstance)
					((L2MinionInstance) killer).getLeader().giveItem(item, true);
				else
					((L2MonsterInstance) killer).giveItem(item, true);
			else if(killer.isPlayer && Config.AUTO_LOOT && Config.AUTO_LOOT_PK)
				((L2Player) killer).getInventory().addItem(item);
			else if(killer.isSummon && Config.AUTO_LOOT && Config.AUTO_LOOT_PK)
				killer.getOwner().getInventory().addItem(item);
			else
			{
				Location pos = Rnd.coordsRandomize(this, Config.KARMA_RANDOM_DROP_LOCATION_LIMIT);
				for(int i = 0; i < 20 && !GeoClient.getInstance().canMoveToCoordWithCollision(getX(), getY(), getZ(), pos.getX(), pos.getY(), pos.getZ()); i++)
					pos = Rnd.coordsRandomize(this, Config.KARMA_RANDOM_DROP_LOCATION_LIMIT);
				if(!GeoClient.getInstance().canMoveToCoordWithCollision(getX(), getY(), getZ(), pos.getX(), pos.getY(), pos.getZ()))
				{
					pos.setX( killer.getX());
					pos.setY(killer.getY());
					pos.setZ(killer.getZ());
				}
				item.dropMe(this, pos);
			}

			sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_DROPPED_S1).addItemName(item.getItemId()));
		}
		refreshOverloaded();
	}

	/**
	 * Kill the L2Character, Apply Death Penalty, Manage gain/loss Karma and Item Drop.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B>
	 * <li>Reduce the Experience of the L2Player in function of the calculated Death Penalty </li>
	 * <li>If necessary, unsummon the Pet of the killed L2Player </li>
	 * <li>Kill the L2Player </li><BR><BR>
	 *
	 * @param killer The L2Character who killed
	 *
	 */
	@Override
	public void doDie(L2Character killer)
	{
		//Check for active charm of luck for death penalty
		getDeathPenalty().checkCharmOfLuck();

		if(getTradeList() != null)
			getTradeList().removeAll();
		setPrivateStoreType(L2Player.STORE_PRIVATE_NONE);

		// Kill the L2Player
		super.doDie(killer);

		// Dont unsummon a summon, it can kill few enemies. But pet must returned back into its item
		// Unsummon siege summons
		if(_summon != null && (_summon.isPet || _summon.getNpcId() == L2Summon.SIEGE_GOLEM_ID || _summon.getNpcId() == L2Summon.SIEGE_CANNON_ID || _summon.getNpcId() == L2Summon.SWOOP_CANNON_ID))
			_summon.unSummon();

		// Unsummon Cubics and agathion
		if(!_cubics.isEmpty())
		{
			for(L2CubicInstance cubic : _cubics.values())
			{
				cubic.stopAction();
				cubic.cancelDisappear();
			}
			_cubics.clear();
		}

		if(Config.LOG_KILLS)
		{
			String coords = " at (" + getX() + "," + getY() + "," + getZ() + ")";
			if(killer instanceof L2NpcInstance)
				Log.add("" + this + " karma " + _karma + " killed by mob " + killer.getNpcId() + coords, "kills");
			else if(killer instanceof L2Summon && killer.getOwner() != null)
				Log.add("" + this + " karma " + _karma + " killed by summon of " + killer.getOwner() + coords, "kills");
			else
				Log.add("" + this + " karma " + _karma + " killed by " + killer + coords, "kills");
		}

		if(isCombatFlagEquipped())
			FortressSiegeManager.getInstance().dropCombatFlag(this);

		if(isCursedWeaponEquipped())
		{
			_pvpFlag = 0;

			CursedWeaponsManager.getInstance().dropPlayer(this);
			return;
		}
		else if(killer.isPlayer && killer.isCursedWeaponEquipped())
		{
			_pvpFlag = 0;

			//noinspection ConstantConditions
			CursedWeaponsManager.getInstance().increaseKills(((L2Player) killer).getCursedWeaponEquippedId());
			return;
		}

		doPKPVPManage(killer);

		// Set the PvP Flag of the L2Player
		_pvpFlag = 0;

		altDeathPenalty(killer);

		//And in the end of process notify death penalty that owner died :)
		getDeathPenalty().notifyDead(killer);

		setIncreasedForce(0);

		if(isInParty() && getParty().isInDimensionalRift())
			getParty().getDimensionalRift().getDeadMemberList().add(this);

		stopWaterTask();

		if(getLevel() < 6)
		{
			Quest q = QuestManager.getQuest(255);
			if(q != null)
				processQuestEvent(q.getName(), "CE30");
		}
	}

	/**
	 * Restore the Experience of the L2Player and send it a Server->Client StatusUpdate packet.<BR><BR>
	 */
	public void restoreExp()
	{
		restoreExp(100.);
	}

	/**
	 * Restore the percentes of losted Experience of the L2Player.<BR><BR>
	 */
	public void restoreExp(double percent)
	{
		int lostexp = 0;

		String lostexps = getVar("lostexp");
		if(lostexps != null)
		{
			lostexp = Integer.parseInt(lostexps);
			unsetVar("lostexp");
		}

		if(lostexp != 0)
			addExpAndSp((long) (lostexp * percent / 100), 0, false, false);
	}

	/**
	 * Reduce the Experience (and level if necessary) of the L2Player in function of the calculated Death Penalty.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Calculate the Experience loss </li>
	 * <li>Set the value of _lostExp </li>
	 * <li>Set the new Experience value of the L2Player and Decrease its level if necessary </li>
	 * <li>Send a Server->Client StatusUpdate packet with its new Experience </li><BR><BR>
	 *
	 */
	public void deathPenalty(final boolean atwar)
	{
		double deathPenaltyBonus = getDeathPenalty().getLevel() * Config.ALT_DEATH_PENALTY_C5_EXPERIENCE_PENALTY;
		if(deathPenaltyBonus < 2)
			deathPenaltyBonus = 1;
		else
			deathPenaltyBonus = deathPenaltyBonus / 2;

		//The death steal you some Exp
		double percentLost = 7.0;
		if(_level >= 76)
			percentLost = 2.0;
		else if(_level >= 40)
			percentLost = 4.0;

		if(Config.ALT_DEATH_PENALTY)
			percentLost = percentLost * Config.RATE_XP + _pkKills * Config.ALT_PK_DEATH_RATE;

		if(isFestivalParticipant() || atwar)
			percentLost = percentLost / 4.0;

		// Calculate the Experience loss
		int lostexp = (int) Math.round((Experience.LEVEL[_level + 1] - Experience.LEVEL[_level]) * percentLost / 100);
		lostexp *= deathPenaltyBonus;

		lostexp = (int) calcStat(Stats.EXP_LOST, lostexp, null, null);

		// РџРѕС‚РµСЂСЏ РѕРїС‹С‚Р° РЅР° Р·Р°СЂРµРіРёСЃС‚СЂРёСЂРѕРІР°РЅРЅРѕР№ РѕСЃР°РґРµ 1/4 РѕС‚ РѕР±С‹С‡РЅРѕР№ СЃРјРµСЂС‚Рё, СЃ Charm of Courage РЅРµС‚ РїРѕС‚РµСЂРё
		// РќР° С‡СѓР¶РѕР№ РѕСЃР°РґРµ - РєР°Рє РїСЂРё РѕР±С‹С‡РЅРѕР№ СЃРјРµСЂС‚Рё РѕС‚ *РјРѕР±Р°*
		if(isInZone(Siege))
		{
			Siege siege = SiegeManager.getSiege(getX(), getY());
			if(siege != null && (siege.checkIsDefender(_clan) || siege.checkIsAttacker(_clan)))
				if(isCharmOfCourage())
					lostexp = 0;
				else
					lostexp = lostexp / 4;
		}

		_log.fine(_name + "is dead, so exp to remove:" + lostexp);

		setVar("lostexp", String.valueOf(lostexp));

		addExpAndSp(-1 * lostexp, 0, false, false);
	}

	public void setPartyMatchingAutomaticRegistration(final boolean b)
	{
		_partyMatchingAutomaticRegistration = b;
	}

	public void setPartyMatchingShowLevel(final boolean b)
	{
		_partyMatchingShowLevel = b;
	}

	public void setPartyMatchingShowClass(final boolean b)
	{
		_partyMatchingShowClass = b;
	}

	public Point getLastPartyPositionSent()
	{
		return _lastPartyPositionSent;
	}

	@Override
	public void setXYZ(int x, int y, int z)
	{
		if(getParty() != null && getDistance(_lastPartyPositionSent.x, _lastPartyPositionSent.y) > 500)
		{
			getParty().broadcastToPartyMembers(this, new PartyMemberPosition(this));
			_lastPartyPositionSent.move(x, y);
		}
		super.setXYZ(x, y, z);
	}

	@Override
	public void setXYZInvisible(int x, int y, int z)
	{
		if(getParty() != null && getDistance(_lastPartyPositionSent.x, _lastPartyPositionSent.y) > 500)
		{
			getParty().broadcastToPartyMembers(this, new PartyMemberPosition(this));
			_lastPartyPositionSent.move(x, y);
		}
		super.setXYZInvisible(x, y, z);
	}

	/**
	 * @param memo
	 */
	public void setPartyMatchingMemo(final String memo)
	{
		_partyMatchingMemo = memo;
	}

	public boolean isPartyMatchingAutomaticRegistration()
	{
		return _partyMatchingAutomaticRegistration;
	}

	public String getPartyMatchingMemo()
	{
		return _partyMatchingMemo;
	}

	public boolean isPartyMatchingShowClass()
	{
		return _partyMatchingShowClass;
	}

	public boolean isPartyMatchingShowLevel()
	{
		return _partyMatchingShowLevel;
	}

	/**
	 * Set the L2Player requester of the transaction.<BR><BR>
	 */
	public void setTransactionRequester(final L2Player requestor)
	{
		_currentTransactionRequester = requestor;
		_currentTransactionTimeout = -1;
	}

	public void setTransactionRequester(L2Player requestor, long timeout)
	{
		_currentTransactionTimeout = timeout;
		_currentTransactionRequester = requestor;
	}

	public static enum TransactionType
	{
		NONE,
		PARTY,
		CLAN,
		ALLY,
		TRADE,
		FRIEND,
		CHANNEL
	}

	TransactionType _currentTransactionType = TransactionType.NONE;

	public void setTransactionType(TransactionType type)
	{
		_currentTransactionType = type;
	}

	public TransactionType getTransactionType()
	{
		return _currentTransactionType;
	}

	/**
	 * @return the L2Player requester of the transaction.<BR><BR>
	 */
	public L2Player getTransactionRequester()
	{
		return _currentTransactionRequester;
	}

	/**
	 * @return True if a transaction is in progress.<BR><BR>
	 */
	public boolean isTransactionInProgress()
	{
		return (_currentTransactionTimeout < 0 || _currentTransactionTimeout > System.currentTimeMillis()) && _currentTransactionRequester != null;
	}

	public void addVisibleObject(L2Object object, L2Character dropper)
	{
		if(object == null || object.getObjectId() == getObjectId() || !object.isVisible())
			return;

		if(object.isPolymorphed() && object.getPolytype().equals("item"))
		{
			sendPacket(new SpawnItemPoly(object));
			showMoves(object);
			return;
		}

		if(object.isPolymorphed() && object.getPolytype().equals("npc"))
		{
			sendPacket(new NpcInfoPoly(object, this));
			showMoves(object);
			return;
		}

		if(object instanceof L2ItemInstance)
		{
			if(dropper != null)
				sendPacket(new DropItem((L2ItemInstance) object, dropper.getObjectId()));
			else
				sendPacket(new SpawnItem((L2ItemInstance) object));

			showMoves(object);
			return;
		}

		if(object instanceof L2DoorInstance)
		{
			sendPacket(new StaticObject((L2DoorInstance) object));
			return;
		}

		if(object instanceof L2StaticObjectInstance)
		{
			sendPacket(new StaticObject((L2StaticObjectInstance) object));
			return;
		}

		if(object instanceof L2ClanHallManagerInstance)
			((L2ClanHallManagerInstance) object).sendDecoInfo(this);

		if(object instanceof L2NpcInstance)
		{
			sendPacket(new NpcInfo((L2NpcInstance) object, this));
			showMoves(object);
			return;
		}

		if(object instanceof L2Summon)
		{
			L2Summon summon = (L2Summon) object;

			if(summon.getOwner() == this)
			{
				sendPacket(new PetInfo(summon, summon.isShowSpawnAnimation()));
				sendPacket(new PartySpelled(summon, true));

				if(summon.isPet)
					sendPacket(new PetItemList((L2PetInstance) summon));
			}
			else if(getParty() != null && getParty().containsMember(summon.getOwner()))
			{
				sendPacket(new NpcInfo(summon, this, summon.isShowSpawnAnimation()));
				sendPacket(new PartySpelled(summon, true));
			}
			else
				sendPacket(new NpcInfo(summon, this, summon.isShowSpawnAnimation()));

			showMoves(object);
			return;
		}

		if(object.isPlayer)
		{
			final L2Player otherPlayer = (L2Player) object;
			if(otherPlayer.isInvisible() && getObjectId() != otherPlayer.getObjectId())
				return;

			if(getObjectId() != otherPlayer.getObjectId())
				sendPacket(new CharInfo(otherPlayer));

			if(otherPlayer.isMounted())
				sendPacket(new Ride(otherPlayer));

			if(otherPlayer.getPrivateStoreType() != STORE_PRIVATE_NONE)
				if(otherPlayer.getPrivateStoreType() == STORE_PRIVATE_BUY)
					sendPacket(new PrivateStoreMsgBuy(otherPlayer));
				else if(otherPlayer.getPrivateStoreType() == STORE_PRIVATE_SELL || otherPlayer.getPrivateStoreType() == STORE_PRIVATE_SELL_PACKAGE)
					sendPacket(new PrivateStoreMsgSell(otherPlayer));
				else if(otherPlayer.getPrivateStoreType() == STORE_PRIVATE_MANUFACTURE)
					sendPacket(new RecipeShopMsg(otherPlayer));

			if(otherPlayer.isCastingNow() && otherPlayer.getCastingSkill() != null && otherPlayer.getAnimationEndTime() > 0)
				if(otherPlayer.getCastingTarget() != null && otherPlayer.getCastingTarget().isCharacter)
					sendPacket(new MagicSkillUse(otherPlayer, otherPlayer.getCastingTarget(), otherPlayer.getCastingSkill().getId(), otherPlayer.getCastingSkill().getLevel(), (int) (otherPlayer.getAnimationEndTime() - System.currentTimeMillis()), 0));

			if(_clan != null && otherPlayer.getClan() != null && _clan.isAtWarWith(otherPlayer.getClan().getClanId()))
				sendPacket(new RelationChanged(otherPlayer, otherPlayer.isAutoAttackable(this), otherPlayer.getRelation(this)));

			showMoves(object);
			return;
		}

		if(object instanceof L2BoatInstance)
			if(!isInBoat() && object != getBoat())
				{
					sendPacket(new VehicleInfo((L2BoatInstance) object));
					((L2BoatInstance) object).sendVehicleDeparture(this);
				}
	}

	public void removeVisibleObject(L2Object object)
	{
		if(object == null || object.getObjectId() == getObjectId())
			return;
		sendPacket(new DeleteObject(object));
		if(object.isNpc)
			removeFromHatelist((L2NpcInstance) object);
		getAI().notifyEvent(CtrlEvent.EVT_FORGET_OBJECT, object);
	}

	public void showMoves(L2Object object)
	{
		if(object != null && object.isCharacter)
		{
			L2Character obj = (L2Character) object;
			if(obj.isMoving || obj.isFollow)
				sendPacket(new CharMoveToLocation(obj));
		}
	}

	/**
	 * Manage the increase level task of a L2Player (Max MP, Max MP, Recommendation, Expertise).<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Send a Server->Client System Message to the L2Player : YOU_INCREASED_YOUR_LEVEL </li>
	 * <li>Send a Server->Client packet StatusUpdate to the L2Player with new LEVEL, MAX_HP and MAX_MP </li>
	 * <li>Set the current HP and MP of the L2Player, Launch/Stop a HP/MP/CP Regeneration Task and send StatusUpdate packet to all other L2Player to inform (exclusive broadcast)</li>
	 * <li>Recalculate the party level</li>
	 * <li>Give Expertise skill of this level</li><BR><BR>
	 */
	private void increaseLevel()
	{
		if(_level >= Experience.MAX_LEVEL)
			return;

		_level++;

		sendPacket(Msg.YOU_HAVE_INCREASED_YOUR_LEVEL);
		broadcastPacket(new SocialAction(getObjectId(), 15));

		setCurrentHpMp(getMaxHp(), getMaxMp());
		setCurrentCp(getMaxCp());

		// Recalculate the party level
		if(isInParty())
			getParty().recalculatePartyData();

		if(_clan != null)
		{
			PledgeShowMemberListUpdate memberUpdate = new PledgeShowMemberListUpdate(this);
			for(L2Player clanMember : _clan.getOnlineMembers(null))
				clanMember.sendPacket(memberUpdate);
		}

		// Give Expertise skill of this level
		rewardSkills();

		Quest q = QuestManager.getQuest(255);
		if(q != null)
			processQuestEvent(q.getName(), "CE40");
	}

	/**
	 * Manage the decrease level task of a L2Player (Max MP, Max MP, Recommendation, Expertise and beginner skills...).<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Send a Server->Client packet StatusUpdate to the L2Player with new LEVEL, MAX_HP and MAX_MP </li>
	 * <li>Recalculate the party level</li>
	 * <li>Recalculate the number of Recommendation that the L2Player can give</li>
	 * <li>Give Expertise skill of this level</li><BR><BR>
	 *
	 */
	private void decreaseLevel()
	{
		if(_level == 1)
			return;

		_level--;

		// Recalculate the party level
		if(isInParty())
			getParty().recalculatePartyData();

		if(_clan != null)
		{
			PledgeShowMemberListUpdate memberUpdate = new PledgeShowMemberListUpdate(this);
			for(L2Player clanMember : _clan.getOnlineMembers(_name))
				if(!clanMember.equals(this))
					clanMember.sendPacket(memberUpdate);
		}

		// Give Expertise skill of this level
		rewardSkills();
	}

	/**
	 * Stops timers that are related to current instance of L2Player<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Set the RegenActive flag to False </li>
	 * <li>Stop the HP/MP/CP Regeneration task </li>
	 * <li>Stop cubics</li>
	 *
	 */
	public void stopAllTimers()
	{
		stopBotCheck();
		for(L2CubicInstance cubic : _cubics.values())
			cubic.deleteMe();
		stopHpMpRegeneration();
		stopWaterTask();

		if(_breakWarn != null)
		{
			_breakWarn.cancel(true);
			_breakWarn = null;
		}
	}

	/**
	 * @return the L2Summon of the L2Player or null.
	 */
	@Override
	public L2Summon getPet()
	{
		return _summon;
	}

	/**
	 * Set the L2Summon of the L2Player.<BR><BR>
	 * @param summon L2Summon to set
	 */
	public void setPet(L2Summon summon)
	{
		_summon = summon;
		AutoShot();
		if(summon == null && getEffectsBySkillId(4140) != null)
			for(L2Effect e : getEffectsBySkillId(4140))
				e.exit();
	}

	/**
	 * Manage the delete task of a L2Player (Leave Party, Unsummon pet, Save its inventory in the database, Remove it from the world...).<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>If the L2Player is in observer mode, set its position to its position before entering in observer mode </li>
	 * <li>Set the online Flag to True or False and update the characters table of the database with online status and lastAccess </li>
	 * <li>Stop the HP/MP/CP Regeneration task </li>
	 * <li>Cancel Crafting, Attak or Cast </li>
	 * <li>Remove the L2Player from the world </li>
	 * <li>Stop Party and Unsummon Pet </li>
	 * <li>Update database with items in its inventory and remove them from the world </li>
	 * <li>Remove all L2Object from _knownObjects and _knownPlayer of the L2Character then cancel Attak or Cast and notify AI </li>
	 * <li>Close the connection with the client </li><BR><BR>
	 *
	 */
	public void deleteMe()
	{
		_massUpdating = true;
		_isDeleting = true;

		if(getTransactionRequester() != null)
		{
			getTransactionRequester().setTransactionRequester(null);
			setTransactionRequester(null);
		}

		// Set the online Flag to True or False and update the characters table of the database with online status and lastAccess (called when login and logout)
		try
		{
			setOnlineStatus(false);
		}
		catch(Throwable t)
		{
			_log.log(Level.WARNING, "deletedMe()", t);
		}

		// Stop the HP/MP/CP Regeneration task (scheduled tasks)
		try
		{
			stopAllTimers();
		}
		catch(Throwable t)
		{
			_log.log(Level.WARNING, "deletedMe()", t);
		}

		// Cancel Attak or Cast
		try
		{
			setTarget(null);
		}
		catch(Throwable t)
		{
			_log.log(Level.WARNING, "deletedMe()", t);
		}

		if(_forceBuff != null)
			_forceBuff.delete();

		if(_party != null)
			for(L2Player member : _party.getPartyMembers())
				if(member.getForceBuff() != null && member.getForceBuff().getTarget() == this)
					member.getForceBuff().delete();

		// Remove the L2Player from the world
		if(isVisible())
			try
			{
				decayMe();
			}
			catch(Throwable t)
			{
				_log.log(Level.WARNING, "deletedMe()", t);
			}

		// If a Party is in progress, leave it
		if(isInParty())
			try
			{
				leaveParty();
			}
			catch(Throwable t)
			{
				_log.log(Level.WARNING, "deletedMe()", t);
			}

		// If the L2Player has Pet, unsummon it
		if(getPet() != null)
			try
			{
				getPet().unSummon();
			}
			catch(Throwable t)
			{
				_log.log(Level.WARNING, "deletedMe()", t);
			}// returns pet to control item

		// Update database with items in its inventory and remove them from the world
		try
		{
			getInventory().deleteMe();
		}
		catch(Throwable t)
		{
			_log.log(Level.WARNING, "deletedMe()", t);
		}

		try
		{
			if(getClanId() > 0 && _clan != null && _clan.getClanMember(getObjectId()) != null)
			{
				int sponsor = _clan.getClanMember(getObjectId()).getSponsor();
				int apprentice = getApprentice();
				PledgeShowMemberListUpdate memberUpdate = new PledgeShowMemberListUpdate(this);
				for(L2Player clanMember : _clan.getOnlineMembers(_name))
				{
					if(clanMember.getObjectId() == getObjectId())
						continue;
					clanMember.sendPacket(memberUpdate);
					if(clanMember.getObjectId() == sponsor)
						clanMember.sendPacket(new SystemMessage(SystemMessage.S1_YOUR_CLAN_ACADEMYS_APPRENTICE_HAS_LOGGED_OUT).addString(_name));
					else if(clanMember.getObjectId() == apprentice)
						clanMember.sendPacket(new SystemMessage(SystemMessage.S1_YOUR_CLAN_ACADEMYS_SPONSOR_HAS_LOGGED_OUT).addString(_name));
				}
				_clan.getClanMember(getObjectId()).setPlayerInstance(null);
			}
		}
		catch(final Throwable t)
		{
			_log.log(Level.SEVERE, "deletedMe()", t);
		}

		if(CursedWeaponsManager.getInstance().getCursedWeapon(getCursedWeaponEquippedId()) != null)
			CursedWeaponsManager.getInstance().getCursedWeapon(getCursedWeaponEquippedId()).setPlayer(null);

		for(L2Player player : _snoopedPlayer)
			player.removeSnooper(this);

		for(L2Player player : _snoopListener)
			player.removeSnooped(this);

		//Send friendlists to friends that this player has logged off
		EnterWorld.notifyFriends(this, false);

		// Remove L2Object object from _allObjects of L2World
		L2World.removeObject(this);

		_ai = null;
		_summon = null;
		_lastPartyPositionSent = null;
		_arrowItem = null;
		_fistsWeaponItem = null;
		_chars = null;
		_enchantScroll = null;
		_lastFolkNpc = null;
		_obsLoc = null;
		_observNeighbors = null;
	}

	/**
	 * Set the _tradeList object of the L2Player.<BR><BR>
	 */
	public void setTradeList(final L2TradeList x)
	{
		_tradeList = x;
	}

	/**
	 * @return the _tradeList object of the L2Player.<BR><BR>
	 */
	public L2TradeList getTradeList()
	{
		return _tradeList;
	}

	/**
	 * Set the _sellList object of the L2Player.<BR><BR>
	 */
	public void setSellList(final ConcurrentLinkedQueue<TradeItem> x)
	{
		_sellList = x;
		saveTradeList();
	}

	/**
	 * @return the _sellList object of the L2Player.<BR><BR>
	 */
	public ConcurrentLinkedQueue<TradeItem> getSellList()
	{
		return _sellList != null ? _sellList : new ConcurrentLinkedQueue<TradeItem>();
	}

	/**
	 * @return the _createList object of the L2Player.<BR><BR>
	 */
	public L2ManufactureList getCreateList()
	{
		return _createList;
	}

	/**
	 * Set the _createList object of the L2Player.<BR><BR>
	 */
	public void setCreateList(final L2ManufactureList x)
	{
		_createList = x;
		saveTradeList();
	}

	/**
	 * Set the _buyList object of the L2Player.<BR><BR>
	 */
	public void setBuyList(final ConcurrentLinkedQueue<TradeItem> x)
	{
		_buyList = x;
		saveTradeList();
	}

	/**
	 * @return the _buyList object of the L2Player.<BR><BR>
	 */
	public ConcurrentLinkedQueue<TradeItem> getBuyList()
	{
		return _buyList != null ? _buyList : new ConcurrentLinkedQueue<TradeItem>();
	}

	/**
	 * Set the Private Store type of the L2Player.<BR><BR>
	 *
	 * <B><U> Values </U> :</B><BR><BR>
	 * <li>0 : STORE_PRIVATE_NONE</li>
	 * <li>1 : STORE_PRIVATE_SELL</li>
	 * <li>2 : sellmanage</li><BR>
	 * <li>3 : STORE_PRIVATE_BUY</li><BR>
	 * <li>4 : buymanage</li><BR>
	 * <li>5 : STORE_PRIVATE_MANUFACTURE</li><BR>
	 * <li>8 : STORE_PRIVATE_SELL_PACKAGE</li><BR>
	 *
	 */
	public void setPrivateStoreType(final short type)
	{
		_privatestore = type;
		if(type != STORE_PRIVATE_NONE)
			setVar("storemode", String.valueOf(type));
		else
			unsetVar("storemode");
	}

	/**
	 * @return the Private Store type of the L2Player.<BR><BR>
	 *
	 * <B><U> Values </U> :</B><BR><BR>
	 * <li>0 : STORE_PRIVATE_NONE</li>
	 * <li>1 : STORE_PRIVATE_SELL</li>
	 * <li>2 : sellmanage</li><BR>
	 * <li>3 : STORE_PRIVATE_BUY</li><BR>
	 * <li>4 : buymanage</li><BR>
	 * <li>5 : STORE_PRIVATE_MANUFACTURE</li><BR>
	 * <li>7 : STORE_OBSERVING_GAMES</li><BR>
	 * <li>8 : STORE_PRIVATE_SELL_PACKAGE</li><BR>
	 */
	public short getPrivateStoreType()
	{
		if(inObserverMode())
			return STORE_OBSERVING_GAMES;

		return _privatestore;
	}

	/**
	 * Set the _skillLearningClassId object of the L2Player.<BR><BR>
	 */
	public void setSkillLearningClassId(final ClassId classId)
	{
		_skillLearningClassId = classId;
	}

	/**
	 * @return the _skillLearningClassId object of the L2Player.<BR><BR>
	 */
	public ClassId getSkillLearningClassId()
	{
		return _skillLearningClassId;
	}

	/**
	 * Set the _clan object, _clanId, _clanLeader Flag and title of the L2Player.<BR><BR>
	 * @param clan the clat to set
	 */
	public void setClan(L2Clan clan)
	{
		_clan = clan;

		if(clan == null)
		{
			_clanId = 0;
			_clanLeader = false;
			_pledgeType = 0;
			_pledgeClass = 0;
			_powerGrade = 0;
			_lvlJoinedAcademy = 0;
			_apprentice = 0;
			return;
		}

		if(!clan.isMember(getObjectId()))
		{
			// char has been kicked from clan
			_log.fine("Char " + _name + " is kicked from clan: " + clan.getName());
			setClan(null);
			setTitle("");
			return;
		}

		_clanId = clan.getClanId();
		_clanLeader = getObjectId() == clan.getLeaderId();

		setTitle("");
	}

	/**
	 * @return the _clan object of the L2Player.<BR><BR>
	 */
	public L2Clan getClan()
	{
		return _clan;
	}

	public ClanHall getClanHall()
	{
		return ClanHallManager.getInstance().getClanHallByOwner(_clan);
	}

	public L2Alliance getAlliance()
	{
		return _clan == null ? null : _clan.getAlliance();
	}

	/**
	 * @return True if the L2Player is the leader of its clan.
	 */
	public boolean isClanLeader()
	{
		return _clanLeader;
	}

	public boolean isAllyLeader()
	{
		return getAlliance() != null && getAlliance().getLeader().getLeaderId() == getObjectId();
	}

	/**
	 * Reduce the number of arrows owned by the L2Player.<BR><BR>
	 */
	@Override
	public void reduceArrowCount()
	{
		L2ItemInstance arrows = getInventory().destroyItem(getInventory().getPaperdollObjectId(Inventory.PAPERDOLL_LHAND), 1, false);
		if(arrows == null || arrows.getCount() == 0)
		{
			getInventory().unEquipItemInSlot(Inventory.PAPERDOLL_LHAND);
			_arrowItem = null;
		}
	}

	/**
	 * Equip arrows needed in left hand and send a Server->Client packet ItemList to the L2Player then return True.<BR><BR>
	 */
	protected boolean checkAndEquipArrows()
	{
		// Check if nothing is equipped in left hand
		if(_inventory.getPaperdollItem(Inventory.PAPERDOLL_LHAND) == null)
		{
			// Get the L2ItemInstance of the arrows needed for this bow
			if(getActiveWeaponItem().getItemType() == WeaponType.BOW)
				_arrowItem = _inventory.findArrowForBow(getActiveWeaponItem());

			// Equip arrows needed in left hand
			if(_arrowItem != null)
				getInventory().setPaperdollItem(Inventory.PAPERDOLL_LHAND, _arrowItem);
		}
		else
			// Get the L2ItemInstance of arrows equipped in left hand
			_arrowItem = _inventory.getPaperdollItem(Inventory.PAPERDOLL_LHAND);

		return _arrowItem != null;
	}

	public void setUptime(final long time)
	{
		_uptime = time;
	}

	public long getUptime()
	{
		return System.currentTimeMillis() - _uptime;
	}

	/**
	 * @return True if the L2Player has a Party in progress.<BR><BR>
	 */
	public boolean isInParty()
	{
		return _party != null;
	}

	/**
	 * Set the _party object of the L2Player (without joining it).<BR><BR>
	 */
	public void setParty(final L2Party party)
	{
		_party = party;
	}

	/**
	 * Set the _party object of the L2Player AND join it.<BR><BR>
	 */
	public void joinParty(final L2Party party)
	{
		if(party != null)
		{
			_party = party;
			party.addPartyMember(this);
			party.broadcastToPartyMembers(this, new PartySpelled(this, true));
			for(L2Player member : party.getPartyMembers())
				sendPacket(new PartySpelled(member, true));
		}
	}

	/**
	 * Manage the Leave Party task of the L2Player.<BR><BR>
	 */
	public void leaveParty()
	{
		if(isInParty())
		{
			_party.removePartyMember(this);
			_party = null;
		}
	}

	/**
	 * @return the _party object of the L2Player.<BR><BR>
	 */
	public L2Party getParty()
	{
		return _party;
	}

	/**
	 * @return True if the L2Player is a GM.<BR><BR>
	 */
	public boolean isGM()
	{
		return _playerAccess.IsGM;
	}

	/**
	 * РќРёРіРґРµ РЅРµ РёСЃРїРѕР»СЊР·СѓРµС‚СЃСЏ, РЅРѕ РјРѕР¶РµС‚ РїСЂРёРіРѕРґРёС‚СЊСЃСЏ РґР»СЏ Р‘Р”
	 */
	public void setAccessLevel(final int level)
	{
		_accessLevel = level;
	}

	/**
	 * РќРёРіРґРµ РЅРµ РёСЃРїРѕР»СЊР·СѓРµС‚СЃСЏ, РЅРѕ РјРѕР¶РµС‚ РїСЂРёРіРѕРґРёС‚СЊСЃСЏ РґР»СЏ Р‘Р”
	 */
	@Override
	public int getAccessLevel()
	{
		return _accessLevel;
	}

	public void setPlayerAccess(final PlayerAccess pa)
	{
		if(pa != null)
			_playerAccess = pa;
		else
			_playerAccess = new PlayerAccess();

		setAccessLevel(_playerAccess.IsGM || _playerAccess.Menu ? 100 : 0);
	}

	public PlayerAccess getPlayerAccess()
	{
		return _playerAccess;
	}

	public void setAccountAccesslevel(final int level, final String comments, int banTime)
	{
		LSConnection.getInstance().sendPacket(new ChangeAccessLevel(getAccountName(), level, comments, banTime));
	}

	@Override
	public double getLevelMod()
	{
		return (89. + getLevel()) / 100.0;
	}

	/**
	 * Update Stats of the L2Player client side by sending Server->Client packet UserInfo/StatusUpdate to this L2Player and CharInfo/StatusUpdate to all L2Player in its _KnownPlayers (broadcast).<BR><BR>
	 */
	@Override
	public void updateStats()
	{
		refreshOverloaded();
		refreshExpertisePenalty();
		sendChanges();
	}

	/**
	 * Send a Server->Client StatusUpdate packet with Karma and PvP Flag to the L2Player and all L2Player to inform (broadcast).<BR><BR>
	 */
	public void setKarmaFlag(final int flag)
	{
		// Send a Server->Client StatusUpdate packet with Karma and PvP Flag to the L2Player and all L2Player to inform (broadcast)
		final StatusUpdate su = new StatusUpdate(getObjectId());
		su.addAttribute(StatusUpdate.KARMA, _karma);
		su.addAttribute(StatusUpdate.PVP_FLAG, flag);
		broadcastPacket(su);
	}

	/**
	 * Send a Server->Client StatusUpdate packet with Karma to the L2Player and all L2Player to inform (broadcast).<BR><BR>
	 */
	public void updateKarma()
	{
		// Send a Server->Client StatusUpdate packet with Karma to the L2Player
		final StatusUpdate su = new StatusUpdate(getObjectId());
		su.addAttribute(StatusUpdate.KARMA, _karma);
		broadcastPacket(su);
	}

	/**
	 * Set the online Flag to True or False and update the characters table of the database with online status and lastAccess (called when login and logout).<BR><BR>
	 */
	public void setOnlineStatus(final boolean isOnline)
	{
		if(_isOnline != isOnline)
			_isOnline = isOnline;

		// Update the characters table of the database with online status and lastAccess (called when login and logout)
		updateOnlineStatus();
	}

	/**
	 * Update the characters table of the database with online status and lastAccess of this L2Player (called when login and logout).<BR><BR>
	 */
	public void storeHWID(String HWID)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("UPDATE characters SET LastHWID=? WHERE obj_id=?");
			statement.setString(1, HWID);
			statement.setInt(2, getObjectId());
			statement.execute();
		}
		catch(final Exception e)
		{
			_log.warning("could not store characters HWID:" + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public void updateOnlineStatus()
	{
		if(isInOfflineMode())
			return;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("UPDATE characters SET online=?, lastAccess=? WHERE obj_id=?");
			statement.setInt(1, isOnline() ? 1 : 0);
			statement.setLong(2, System.currentTimeMillis() / 1000);
			statement.setInt(3, getObjectId());
			statement.execute();
		}
		catch(final Exception e)
		{
			_log.warning("could not set char online status:" + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/**
	 * Decrease Karma of the L2Player and Send it StatusUpdate packet with Karma and PvP Flag (broadcast).<BR><BR>
	 */
	public void increaseKarma(final long add_karma)
	{
		long new_karma = _karma + add_karma;

		if(new_karma > Integer.MAX_VALUE)
			new_karma = Integer.MAX_VALUE;

		if(_karma == 0 && new_karma > 0)
		{
			_karma = (int) new_karma;
			for(final L2Character cha : getKnownCharacters(2000))
				if(cha instanceof L2GuardInstance && cha.getAI().getIntention() == CtrlIntention.AI_INTENTION_IDLE)
					cha.getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE, null, null);
		}
		else
			_karma = (int) new_karma;

		// Send a Server->Client StatusUpdate packet with Karma to the L2Player and all L2Player to inform (broadcast)
		updateKarma();
	}

	/**
	 * Decrease Karma of the L2Player and Send it StatusUpdate packet with Karma and PvP Flag (broadcast).<BR><BR>
	 *
	 * @param i : The loss Karma value
	 *
	 */
	public void decreaseKarma(final int i)
	{
		_karma -= i;

		if(_karma <= 0)
		{
			_karma = 0;

			// Send a Server->Client StatusUpdate packet with Karma and PvP Flag to the L2Player and all L2Player to inform (broadcast)
			setKarmaFlag(0);

			return;
		}

		// Send a Server->Client StatusUpdate packet with Karma to the L2Player and all L2Player to inform (broadcast)
		updateKarma();

	}

	/**
	 * Create a new player in the characters table of the database.<BR><BR>
	 */
	private boolean createDb()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("INSERT INTO `characters` (account_name, obj_Id, char_name, face, hairStyle, hairColor, sex, karma, pvpkills, pkkills, clanid, deletetime, title, accesslevel, online, leaveclan, deleteclan, nochannel, pledge_type, pledge_rank, lvl_joined_academy, apprentice) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
			statement.setString(1, _accountName);
			statement.setInt(2, getObjectId());
			statement.setString(3, getName());
			statement.setInt(4, getFace());
			statement.setInt(5, getHairStyle());
			statement.setInt(6, getHairColor());
			statement.setInt(7, getSex());
			statement.setInt(8, getKarma());
			statement.setInt(9, getPvpKills());
			statement.setInt(10, getPkKills());
			statement.setInt(11, getClanId());
			statement.setInt(12, getDeleteTimer());
			statement.setString(13, getTitle());
			statement.setInt(14, _accessLevel);
			statement.setInt(15, isOnline() ? 1 : 0);
			statement.setLong(16, getLeaveClanTime() / 1000);
			statement.setLong(17, getDeleteClanTime() / 1000);
			statement.setLong(18, _NoChannel > 0 ? _NoChannel / 1000 : _NoChannel);
			statement.setInt(19, getPledgeType());
			statement.setInt(20, getPowerGrade());
			statement.setInt(21, getLvlJoinedAcademy());
			statement.setInt(22, getApprentice());
			statement.executeUpdate();
			DatabaseUtils.closeStatement(statement);

			statement = con.prepareStatement("INSERT INTO character_subclasses (char_obj_id, class_id, exp, sp, curHp, curMp, curCp, maxHp, maxMp, maxCp, level, active, isBase, death_penalty) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
			statement.setInt(1, getObjectId());
			statement.setInt(2, getTemplate().classId.getId());
			statement.setInt(3, 0);
			statement.setInt(4, 0);
			statement.setDouble(5, getTemplate().baseHpMax + getTemplate().lvlHpAdd + getTemplate().lvlHpMod);
			statement.setDouble(6, getTemplate().baseMpMax + getTemplate().lvlMpAdd + getTemplate().lvlMpMod);
			statement.setDouble(7, getTemplate().baseCpMax + getTemplate().lvlCpAdd + getTemplate().lvlCpMod);
			statement.setDouble(8, getTemplate().baseHpMax + getTemplate().lvlHpAdd + getTemplate().lvlHpMod);
			statement.setDouble(9, getTemplate().baseMpMax + getTemplate().lvlMpAdd + getTemplate().lvlMpMod);
			statement.setDouble(10, getTemplate().baseCpMax + getTemplate().lvlCpAdd + getTemplate().lvlCpMod);
			statement.setInt(11, 1);
			statement.setInt(12, 1);
			statement.setInt(13, 1);
			statement.setInt(14, 0);
			statement.executeUpdate();
		}
		catch(final Exception e)
		{
			_log.log(Level.WARNING, "could not insert char data:", e);
			return false;
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
		return true;
	}

	/**
	 * Retrieve a L2Player from the characters table of the database and add it in _allObjects of the L2World.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Retrieve the L2Player from the characters table of the database </li>
	 * <li>Add the L2Player object in _allObjects </li>
	 * <li>Set the x,y,z position of the L2Player and make it invisible</li>
	 * <li>Update the overloaded status of the L2Player</li><BR><BR>
	 *
	 * @param objectId Identifier of the object to initialized
	 *
	 * @return The L2Player loaded from the database
	 *
	 */
	private static L2Player restore(final int objectId)
	{
		L2Player player = null;
		ThreadConnection con = null;
		FiltredStatement statement = null;
		FiltredStatement statement2 = null;
		ResultSet pl_rset = null;
		ResultSet ps_rset = null;
		try
		{
			// Retrieve the L2Player from the characters table of the database
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.createStatement();
			statement2 = con.createStatement();
			pl_rset = statement.executeQuery("SELECT * FROM `characters` WHERE `obj_Id`='" + objectId + "' LIMIT 1");
			ps_rset = statement2.executeQuery("SELECT `class_id` FROM `character_subclasses` WHERE `char_obj_id`='" + objectId + "' AND `isBase`='1' LIMIT 1");

			if(pl_rset.next() && ps_rset.next())
			{
				final short classId = ps_rset.getShort("class_id");
				final boolean female = pl_rset.getInt("sex") == 1;
				final L2PlayerTemplate template = CharTemplateTable.getInstance().getTemplate(classId, female);

				player = new L2Player(objectId, template);

				player.loadVariables();

				player.setBaseClass(classId);
				player._accountName = pl_rset.getString("account_name");
				player.setName(pl_rset.getString("char_name"));

				player.setFace(pl_rset.getByte("face"));
				player.setHairStyle(pl_rset.getByte("hairStyle"));
				player.setHairColor(pl_rset.getByte("hairColor"));
				player.setHeading(pl_rset.getInt("heading"));

				player.setKarma(pl_rset.getInt("karma"));
				player.setPvpKills(pl_rset.getInt("pvpkills"));
				player.setPkKills(pl_rset.getInt("pkkills"));
				player.setLeaveClanTime(pl_rset.getLong("leaveclan") * 1000);
				if(player.getLeaveClanTime() > 0 && player.canJoinClan())
					player.setLeaveClanTime(0);
				player.setDeleteClanTime(pl_rset.getLong("deleteclan") * 1000);
				if(player.getDeleteClanTime() > 0 && player.canCreateClan())
					player.setDeleteClanTime(0);

				player.setNoChannel(pl_rset.getLong("nochannel") * 1000);
				if(player.getNoChannel() > 0 && player.getNoChannelRemained() < 0)
					player.updateNoChannel(0);

				player.setOnlineTime(pl_rset.getLong("onlinetime") * 1000);

				player.setNoble(pl_rset.getBoolean("noble"));
				player.setVarka(pl_rset.getInt("varka"));
				player.setKetra(pl_rset.getInt("ketra"));
				player.setRam(pl_rset.getInt("ram"));

				final int clanId = pl_rset.getInt("clanid");
				if(clanId > 0)
				{
					if(Config.DEBUG)
						System.out.println("Char clan id is loaded as " + clanId);
					player.setClan(ClanTable.getInstance().getClan(clanId));
					player.setPledgeType(pl_rset.getInt("pledge_type"));
					player.setPowerGrade(pl_rset.getInt("pledge_rank"));
					player.setLvlJoinedAcademy(pl_rset.getInt("lvl_joined_academy"));
					player.setApprentice(pl_rset.getInt("apprentice"));
					player.updatePledgeClass();
					if(Config.DEBUG)
						System.out.println("Char clan is loaded");
				}

				player.setDeleteTimer(pl_rset.getInt("deletetime"));

				player.setTitle(pl_rset.getString("title"));

				if(player.getVar("namecolor") == null)
					if(player.getPlayerAccess().IsGM)
						player.setNameColor(Config.GM_NAME_COLOUR);
					else if(player.getClan() != null && player.getClan().getLeaderId() == player.getObjectId())
						player.setNameColor(Config.CLANLEADER_NAME_COLOUR);
					else
						player.setNameColor(Config.NORMAL_NAME_COLOUR);
				else
					player.setNameColor(Integer.decode("0x" + player.getVar("namecolor")));

				player.setFistsWeaponItem(player.findFistsWeaponItem(classId));
				player.setUptime(System.currentTimeMillis());
				player.setLastAccess(pl_rset.getLong("lastAccess"));

				player.setRecomHave(pl_rset.getInt("rec_have"));
				player.setRecomLeft(pl_rset.getInt("rec_left"));

				player.setKeyBindings(pl_rset.getBytes("key_bindings"));
				player.setPcBangPoints(pl_rset.getInt("pcBangPoints"));

				player.restoreRecipeBook();

				player.restoreTradeList();
				if(player.getVar("storemode") != null)
				{
					player.setPrivateStoreType(Short.parseShort(player.getVar("storemode")));
					player.setSitting(true);
				}

				if(Config.ENABLE_OLYMPIAD && Hero.getInstance().getHeroes() != null && Hero.getInstance().getHeroes().containsKey(player.getObjectId()))
					player.setHero(true);

				restoreCharSubClasses(player);
				Quest.playerEnter(player);

				// 15 СЃРµРєСѓРЅРґ РїРѕСЃР»Рµ РІС…РѕРґР° РІ РёРіСЂСѓ РЅР° РїРµСЂСЃРѕРЅР°Р¶Р° РЅРµ Р°РіСЂСЏС‚СЃСЏ РјРѕР±С‹
				player.setNonAggroTime(System.currentTimeMillis() + 15000);

				// РґР»СЏ СЃРµСЂРІРёСЃР° РІРёРІРµСЂРЅ - РІРѕР·РІСЂР°С‚ РґРµРЅРµРі РµСЃР»Рё СЃРµСЂРІРµСЂ СѓРїР°Р» РІРѕ РІСЂРµРјСЏ РїРѕР»РµС‚Р°
				String wm = player.getVar("wyvern_moneyback");
				if(wm != null && Integer.parseInt(wm) > 0)
					player.addAdena(Integer.parseInt(wm));
				player.unsetVar("wyvern_moneyback");

				wm = player.getVar("reflection");
				if(wm != null && ReflectionTable.getInstance().getById(Integer.parseInt(wm)) != null)
					player.setReflection(Integer.parseInt(wm));
				else
					player.setReflection(0);

				// Add the L2Player object in _allObjects
				L2World.addObject(player);

				// Set the x,y,z position of the L2Player and make it invisible
				if(player.getVar("jailed") != null && System.currentTimeMillis() / 1000 < Integer.parseInt(player.getVar("jailed")) + 60)
				{
					player.setXYZInvisible(-114648, -249384, -2984);
					String[] re = player.getVar("jailedFrom").split(";");
					Location loc = new Location(Integer.parseInt(re[0]), Integer.parseInt(re[1]), Integer.parseInt(re[2]));
					player.setReflection(-3);
					player._unjailTask = ThreadPoolManager.getInstance().scheduleGeneral(player.new TeleportTask(loc, re.length > 3 ? Integer.parseInt(re[3]) : 0), Integer.parseInt(player.getVar("jailed")) * 1000);
				}
				else
					player.setXYZInvisible(pl_rset.getInt("x"), pl_rset.getInt("y"), pl_rset.getInt("z"));

				FiltredPreparedStatement stmt = null;
				ResultSet chars = null;
				try
				{
					stmt = con.prepareStatement("SELECT obj_Id, char_name FROM characters WHERE account_name=? AND obj_Id<>?");
					stmt.setString(1, player._accountName);
					stmt.setInt(2, objectId);
					chars = stmt.executeQuery();
					while(chars.next())
					{
						final Integer charId = chars.getInt("obj_Id");
						final String charName = chars.getString("char_name");
						player._chars.put(charId, charName);
					}
				}
				catch(Exception e)
				{
					_log.severe(e.getMessage());
				}
				finally
				{
					DatabaseUtils.closeDatabaseSR(stmt, chars);
				}

				if(Config.KILL_COUNTER)
				{
					// Restore kills stat
					FiltredStatement stt = null;
					ResultSet rstkills = null;
					try
					{
						stt = con.createStatement();
						rstkills = stt.executeQuery("SELECT `npc_id`, `count` FROM `killcount` WHERE `char_id`=" + objectId);
						player._StatKills.clear();
						while(rstkills.next())
							player._StatKills.put(rstkills.getInt("npc_id"), rstkills.getLong("count"));
					}
					catch(Exception e)
					{
						_log.severe(e.getMessage());
					}
					finally
					{
						DatabaseUtils.closeDatabaseSR(stt, rstkills);
					}
				}

				//Restore craft stat
				if(Config.CRAFT_COUNTER)
				{
					FiltredStatement stcraft = null;
					ResultSet rstcraft = null;
					try
					{
						stcraft = con.createStatement();
						rstcraft = stcraft.executeQuery("SELECT `item_id`, `count` FROM `craftcount` WHERE `char_id`=" + objectId);
						player._StatCraft.clear();
						while(rstcraft.next())
							player._StatCraft.put(rstcraft.getInt("item_id"), rstcraft.getLong("count"));
					}
					catch(Exception e)
					{
						_log.severe(e.getMessage());
					}
					finally
					{
						DatabaseUtils.closeDatabaseSR(stcraft, rstcraft);
					}
				}

				if(Config.DROP_COUNTER)
				{
					//Restore drop stat
					FiltredStatement stdrop = null;
					ResultSet rstdrop = null;
					try
					{
						stdrop = con.createStatement();
						rstdrop = stdrop.executeQuery("SELECT `item_id`, `count` FROM `dropcount` WHERE `char_id`=" + objectId);
						player._StatDrop.clear();
						while(rstdrop.next())
							player._StatDrop.put(rstdrop.getInt("item_id"), rstdrop.getLong("count"));
					}
					catch(Exception e)
					{
						_log.severe(e.getMessage());
					}
					finally
					{
						DatabaseUtils.closeDatabaseSR(stdrop, rstdrop);
					}
				}

				if(!L2World.validCoords(player.getX(), player.getY()) || player.getX() == 0 && player.getY() == 0)
					player.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(player, MapRegionTable.TeleportWhereType.ClosestTown));
				player.setIsBot(2);
				player.getInventory().validateItems();
				player.revalidatePenalties();
				player.restoreBlockList();
				player.scheduleBreakWarn();
			}
		}
		catch(final Exception e)
		{
			_log.log(Level.WARNING, "restore: could not restore char data:", e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseSR(statement2, ps_rset);
			DatabaseUtils.closeDatabaseCSR(con, statement, pl_rset);
		}
		return player;
	}

	@SuppressWarnings("unchecked")
	public Future _unjailTask;

	public class TeleportTask implements Runnable
	{
		Location _loc;
		int _reflection;

		public TeleportTask(Location p, int reflection)
		{
			_loc = p;
			_reflection = reflection;
		}

		public void run()
		{
			L2Player.this.teleToLocation(_loc);
		}
	}

	public void scheduleBreakWarn()
	{
		if(_breakWarn == null)
			_breakWarn = ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new WarnUserTakeBreak(), 7200000, 7200000);
	}

	public class WarnUserTakeBreak implements Runnable
	{
		public void run()
		{
			sendPacket(Msg.YOU_HAVE_BEEN_PLAYING_FOR_AN_EXTENDED_PERIOD_OF_TIME_PLEASE_CONSIDER_TAKING_A_BREAK);
		}
	}

	public void incrementKillsCounter(final Integer Id)
	{
		final Long tmp = _StatKills.containsKey(Id) ? _StatKills.get(Id) + 1 : 1;
		_StatKills.put(Id, tmp);
		sendMessage(new CustomMessage("l2r.gameserver.model.L2Player.KillsCounter", this).addString(tmp.toString()));
	}

	public void incrementDropCounter(final Integer Id, final Integer qty)
	{
		_StatDrop.put(Id, _StatDrop.containsKey(Id) ? _StatDrop.get(Id) + qty : qty);
	}

	public void incrementCraftCounter(final Integer Id, final Integer qty)
	{
		final Long tmp = _StatCraft.containsKey(Id) ? _StatCraft.get(Id) + qty : qty;
		_StatCraft.put(Id, tmp);
		sendMessage(new CustomMessage("l2r.gameserver.model.L2Player.CraftCounter", this).addString(tmp.toString()));
	}

	/**
	 * Update L2Player stats in the characters table of the database.<BR><BR>
	 */
	public void store()
	{
		// РџСЂРё Р»РѕРіР°СѓС‚Рµ Р°РІС‚РѕРјР°С‚РѕРј РїСЂРѕРёРіСЂС‹РІР°РµС‚СЃСЏ РґСѓСЌР»СЊ.
		if(getDuel() != null)
			getDuel().onPlayerDefeat(this);

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		FiltredStatement fs = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement(//
			"UPDATE characters SET face=?,hairStyle=?,hairColor=?,heading=?,x=?,y=?,z=?" + //
			",karma=?,pvpkills=?,pkkills=?,rec_have=?,rec_left=?,clanid=?,deletetime=?," + //
			"title=?,accesslevel=?,online=?,leaveclan=?,deleteclan=?,nochannel=?," + //
			"onlinetime=?,noble=?,ketra=?,varka=?,ram=?,pledge_type=?,pledge_rank=?,lvl_joined_academy=?,apprentice=?,key_bindings=?,pcBangPoints=?,char_name=? WHERE obj_Id=?");
			statement.setInt(1, getFace());
			statement.setInt(2, getHairStyle());
			statement.setInt(3, getHairColor());
			statement.setInt(4, getHeading());
			if(_stablePoint == null) // РµСЃР»Рё РёРіСЂРѕРє РЅР°С…РѕРґРёС‚СЃСЏ РІ С‚РѕС‡РєРµ РІ РєРѕС‚РѕСЂРѕР№ РµРіРѕ СЃРѕС…СЂР°РЅСЏС‚СЊ РЅРµ СЃС‚РѕРёС‚ (РЅР°РїСЂРёРјРµСЂ РЅР° РІРёРІРµСЂРЅРµ) С‚Рѕ СЃРѕС…СЂР°РЅСЏСЋС‚СЃСЏ РїРѕСЃР»РµРґРЅРёРµ РєРѕРѕСЂРґРёРЅР°С‚С‹
			{
				statement.setInt(5, getX());
				statement.setInt(6, getY());
				statement.setInt(7, getZ());
			}
			else
			{
				statement.setInt(5, _stablePoint.getX());
				statement.setInt(6, _stablePoint.getY());
				statement.setInt(7, _stablePoint.getZ());
			}
			statement.setInt(8, getKarma());
			statement.setInt(9, getPvpKills());
			statement.setInt(10, getPkKills());
			statement.setInt(11, getRecomHave());
			statement.setInt(12, getRecomLeft());
			statement.setInt(13, getClanId());
			statement.setInt(14, getDeleteTimer());
			statement.setString(15, getTitle());
			statement.setInt(16, _accessLevel);
			statement.setInt(17, isOnline() ? 1 : 0);
			statement.setLong(18, getLeaveClanTime() / 1000);
			statement.setLong(19, getDeleteClanTime() / 1000);
			statement.setLong(20, _NoChannel > 0 ? getNoChannelRemained() / 1000 : _NoChannel);
			statement.setLong(21, _onlineBeginTime > 0 ? (_onlineTime + System.currentTimeMillis() - _onlineBeginTime) / 1000 : _onlineTime / 1000);
			statement.setInt(22, isNoble() ? 1 : 0);
			statement.setInt(23, getKetra());
			statement.setInt(24, getVarka());
			statement.setInt(25, getRam());
			statement.setInt(26, getPledgeType());
			statement.setInt(27, getPowerGrade());
			statement.setInt(28, getLvlJoinedAcademy());
			statement.setInt(29, getApprentice());
			statement.setBytes(30, getKeyBindings());
			statement.setInt(31, getPcBangPoints());
			statement.setString(32, getName());
			statement.setInt(33, getObjectId());

			statement.execute();
			DatabaseUtils.closeStatement(statement);
			statement = null;
			L2World.increaseUpdatePlayerBase();

			try
			{
				if(Config.KILL_COUNTER)
				{
					TextBuilder sb = TextBuilder.newInstance();
					fs = con.createStatement();
					for(Integer tmp : _StatKills.keySet())
					{
						fs.addBatch(sb.append("REPLACE DELAYED INTO `killcount` SET `npc_id`=").append(tmp).append(", `count`=").append(_StatKills.get(tmp)).append(", `char_id`=").append(_objectId).toString());
						sb.clear();
					}
					TextBuilder.recycle(sb);
					fs.executeBatch();
					DatabaseUtils.closeStatement(fs);
				}

				if(Config.CRAFT_COUNTER)
				{
					TextBuilder sb = TextBuilder.newInstance();
					fs = con.createStatement();
					for(Integer tmp : _StatCraft.keySet())
					{
						fs.addBatch(sb.append("REPLACE DELAYED INTO `craftcount` SET `item_id`=").append(tmp).append(", `count`=").append(_StatCraft.get(tmp)).append(", `char_id`=").append(_objectId).toString());
						sb.clear();
					}
					TextBuilder.recycle(sb);
					fs.executeBatch();
					DatabaseUtils.closeStatement(fs);
				}

				if(Config.DROP_COUNTER)
				{
					TextBuilder sb = TextBuilder.newInstance();
					fs = con.createStatement();
					for(Integer tmp : _StatDrop.keySet())
					{
						fs.addBatch(sb.append("REPLACE DELAYED INTO `craftcount` SET `item_id`=").append(tmp).append(", `count`=").append(_StatDrop.get(tmp)).append(", `char_id`=").append(_objectId).toString());
						sb.clear();
					}
					TextBuilder.recycle(sb);
					fs.executeBatch();
					DatabaseUtils.closeStatement(statement);
				}
			}
			catch(ConcurrentModificationException e)
			{}

			synchronized (this)
			{
				storeEffects();
				storeDisableSkills();
				storeCharSubClasses();
				storeBlockList();
			}
		}
		catch(final Exception e)
		{
			_log.warning("store: could not store char data: " + e);
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/**
	 * @return True if the L2Player is on line.<BR><BR>
	 */
	public boolean isOnline()
	{
		return _isOnline;
	}

	/**
	 * Add a skill to the L2Player _skills and its Func objects to the calculator set of the L2Player and save update in the character_skills table of the database.<BR><BR>
	 *
	 * <B><U> Concept</U> :</B><BR><BR>
	 * All skills own by a L2Player are identified in <B>_skills</B><BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Replace oldSkill by newSkill or Add the newSkill </li>
	 * <li>If an old skill has been replaced, remove all its Func objects of L2Character calculator set</li>
	 * <li>Add Func objects of newSkill to the calculator set of the L2Character </li><BR><BR>
	 *
	 * @param newSkill The L2Skill to add to the L2Character
	 *
	 * @return The L2Skill replaced or null if just added a new L2Skill
	 *
	 */
	public L2Skill addSkill(final L2Skill newSkill, final boolean store)
	{
		// Add a skill to the L2Player _skills and its Func objects to the calculator set of the L2Player
		final L2Skill oldSkill = super.addSkill(newSkill);

		// Add or update a L2Player skill in the character_skills table of the database
		if(store)
			storeSkill(newSkill, oldSkill);

		return oldSkill;
	}

	/**
	 * Remove a skill from the L2Character and its Func objects from calculator set of the L2Character and save update in the character_skills table of the database.<BR><BR>
	 *
	 * <B><U> Concept</U> :</B><BR><BR>
	 * All skills own by a L2Character are identified in <B>_skills</B><BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Remove the skill from the L2Character _skills </li>
	 * <li>Remove all its Func objects from the L2Character calculator set</li><BR><BR>
	 *
	 * <B><U> Overriden in </U> :</B><BR><BR>
	 * <li> L2Player : Save update in the character_skills table of the database</li><BR><BR>
	 *
	 * @param skill The L2Skill to remove from the L2Character
	 *
	 * @return The L2Skill removed
	 *
	 */
	public L2Skill removeSkill(L2Skill skill, boolean fromDB)
	{
		// Remove a skill from the L2Character and its Func objects from calculator set of the L2Character
		L2Skill oldSkill = super.removeSkill(skill);

		if(!fromDB)
			return oldSkill;

		if(oldSkill == null)
			oldSkill = skill;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			// Remove or update a L2Player skill from the character_skills table of the database
			con = L2DatabaseFactory.getInstance().getConnection();
			if(oldSkill != null)
			{
				statement = con.prepareStatement("DELETE FROM character_skills WHERE skill_id=? AND char_obj_id=? AND class_index=?");
				statement.setInt(1, oldSkill.getId());
				statement.setInt(2, getObjectId());
				statement.setInt(3, getActiveClass());
				statement.execute();
			}
		}
		catch(final Exception e)
		{
			_log.log(Level.WARNING, "Error could not delete Skill:", e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}

		return oldSkill;
	}

	/**
	 * Add or update a L2Player skill in the character_skills table of the database.<BR><BR>
	 */
	private void storeSkill(final L2Skill newSkill, final L2Skill oldSkill)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			if(oldSkill != null && newSkill != null)
			{
				statement = con.prepareStatement("UPDATE character_skills SET skill_level=? WHERE skill_id=? AND char_obj_id=? AND class_index=?");
				statement.setInt(1, newSkill.getLevel());
				statement.setInt(2, oldSkill.getId());
				statement.setInt(3, getObjectId());
				statement.setInt(4, getActiveClass());
				statement.execute();
			}
			else if(newSkill != null)
			{
				statement = con.prepareStatement("REPLACE INTO character_skills (char_obj_id,skill_id,skill_level,skill_name,class_index) values(?,?,?,?,?)");
				statement.setInt(1, getObjectId());
				statement.setInt(2, newSkill.getId());
				statement.setInt(3, newSkill.getLevel());
				statement.setString(4, newSkill.getName());
				statement.setInt(5, getActiveClass());
				statement.execute();
			}
			else
				_log.warning("could not store new skill. its NULL");
		}
		catch(final Exception e)
		{
			_log.log(Level.WARNING, "Error could not store Skills:", e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/**
	 * Retrieve from the database all skills of this L2Player and add them to _skills.<BR><BR>
	 */
	private void restoreSkills()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			// Retrieve all skills of this L2Player from the database
			// Send the SQL query : SELECT skill_id,skill_level FROM character_skills WHERE char_obj_id=? to the database
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT skill_id,skill_level FROM character_skills WHERE char_obj_id=? AND class_index=?");
			statement.setInt(1, getObjectId());
			statement.setInt(2, getActiveClass());
			rset = statement.executeQuery();

			// Go though the recordset of this SQL query
			while(rset.next())
			{
				final int id = rset.getInt("skill_id");
				final int level = rset.getInt("skill_level");
				boolean allow = false;

				if(id > 9000)
					continue; // fake skills for base stats

				// Create a L2Skill object for each record
				final L2Skill skill = SkillTable.getInstance().getInfo(id, level);

				if(skill == null)
					continue;

				for(int i = 0; i < Config.VipSkillsList.length; i++)
				{
					if(skill.getId() == Config.VipSkillsList[i])
					{
						allow = true;
					}
				}
				// Remove skill if not possible
				if(!_playerAccess.IsGM && !skill.isCommon() && !SkillTreeTable.getInstance().isSkillPossible(this, skill.getId(), skill._level) && !allow)
				{
					removeSkill(skill, true);
					removeSkillFromShortCut(skill.getId());
					illegalAction("has skill " + skill.getName(), 0);
					continue;
				}

				// Add the L2Skill object to the L2Character _skills and its Func objects to the calculator set of the L2Character
				super.addSkill(skill);
			}

			// Restore noble skills
			if(_noble)
			{
				super.addSkill(SkillTable.getInstance().getInfo(1323, 1));
				super.addSkill(SkillTable.getInstance().getInfo(325, 1));
				super.addSkill(SkillTable.getInstance().getInfo(326, 1));
				super.addSkill(SkillTable.getInstance().getInfo(327, 1));
				super.addSkill(SkillTable.getInstance().getInfo(1324, 1));
				super.addSkill(SkillTable.getInstance().getInfo(1325, 1));
				super.addSkill(SkillTable.getInstance().getInfo(1326, 1));
				super.addSkill(SkillTable.getInstance().getInfo(1327, 1));
			}

			// Restore Hero skills at main class only
			if(_hero && getBaseClass() == getActiveClass())
			{
				super.addSkill(SkillTable.getInstance().getInfo(395, 1));
				super.addSkill(SkillTable.getInstance().getInfo(396, 1));
				super.addSkill(SkillTable.getInstance().getInfo(1374, 1));
				super.addSkill(SkillTable.getInstance().getInfo(1375, 1));
				super.addSkill(SkillTable.getInstance().getInfo(1376, 1));
			}

			if(_clan != null)
			{
				// Restore clan leader siege skills
				if(_clan.getLeaderId() == getObjectId() && _clan.getLevel() > 3)
					SiegeManager.addSiegeSkills(this);

				// Restore clan skills
				_clan.addAndShowSkillsToPlayer(this);
			}

			// Give dwarven craft skill
			if(_activeClass >= 53 && _activeClass <= 57 || _activeClass == 117 || _activeClass == 118)
				super.addSkill(SkillTable.getInstance().getInfo(1321, 1));
			super.addSkill(SkillTable.getInstance().getInfo(1322, 1));

			if(Config.UNSTUCK_SKILL && getSkillLevel(1050) < 0)
				addSkill(SkillTable.getInstance().getInfo(2099, 1), false);
		}
		catch(final Exception e)
		{
			_log.warning("count not restore skills:" + e);
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public void storeDisableSkills()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM character_skills_save WHERE char_obj_id = ? AND class_index=?");
			statement.setInt(1, getObjectId());
			statement.setInt(2, getActiveClass());
			statement.execute();
			DatabaseUtils.closeStatement(statement);

			statement = con.prepareStatement("INSERT INTO character_skills_save (char_obj_id, skill_id, class_index, end_time, reuse_delay_org) VALUES (?,?,?,?,?)");
			for(final int tmp : getSkillReuseTimeStamps().keySet())
			{
				statement.setInt(1, getObjectId());
				statement.setInt(2, tmp);
				statement.setInt(3, getActiveClass());
				statement.setLong(4, getSkillReuseTimeStamps().get(tmp).getEndTime());
				statement.setLong(5, getSkillReuseTimeStamps().get(tmp).getReuseBasic());
				statement.execute();
			}
		}
		catch(final Exception e)
		{
			_log.warning("Could not store disable skills data: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public void storeEffects()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement("DELETE FROM character_effects_save WHERE char_obj_id = ? AND class_index=?");
			statement.setInt(1, getObjectId());
			statement.setInt(2, getActiveClass());
			statement.execute();
			DatabaseUtils.closeStatement(statement);

			int order = 0;
			statement = con.prepareStatement("INSERT IGNORE INTO `character_effects_save` SET `char_obj_id`=?,`skill_id`=?,`skill_level`=?,`effect_count`=?,`effect_cur_time`=?,`duration`=?,`order`=?,`class_index`=?");
			for(L2Effect effect : getAllEffects())
				if(effect != null && effect.isInUse() && effect.getSkill().isSaveable() && !effect.getSkill().isToggle() && effect.getEffectType() != L2Effect.EffectType.HEAL_OVER_TIME && effect.getEffectType() != L2Effect.EffectType.COMBAT_POINT_HEAL_OVER_TIME)
				{
					statement.setInt(1, getObjectId());
					statement.setInt(2, effect.getSkill().getId());
					statement.setInt(3, effect.getSkill().getLevel());
					statement.setInt(4, effect.getCount());
					statement.setLong(5, effect.getTime());
					statement.setLong(6, effect.getPeriod());
					statement.setInt(7, order);
					statement.setInt(8, getActiveClass());
					statement.execute();
					if((effect = effect.getNext()) != null)
					{
						statement.setInt(1, getObjectId());
						statement.setInt(2, effect.getSkill().getId());
						statement.setInt(3, effect.getSkill().getLevel());
						statement.setInt(4, effect.getCount());
						statement.setLong(5, effect.getTime());
						statement.setLong(6, effect.getPeriod());
						statement.setInt(7, order);
						statement.setInt(8, getActiveClass());
						statement.execute();
					}
					order++;
				}
			if(Config.ALT_CUBICS_SAVE)
				for(L2CubicInstance cubic : _cubics.values())
				{
					statement.setInt(1, getObjectId());
					statement.setInt(2, cubic.getId() + L2CubicInstance.CUBIC_STORE_OFFSET);
					statement.setInt(3, cubic.getLevel());
					statement.setInt(4, 1);
					statement.setLong(5, cubic.lifeLeft());
					statement.setLong(6, 1);
					statement.setInt(7, order++);
					statement.setInt(8, getActiveClass());
					statement.execute();
				}
		}
		catch(final Exception e)
		{
			_log.warning("Could not store active effects data: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}

		updateEffectIcons();
	}

	public void restoreEffects()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement("SELECT `skill_id`,`skill_level`,`effect_count`,`effect_cur_time`,`duration` FROM `character_effects_save` WHERE `char_obj_id`=? AND `class_index`=? ORDER BY `order` ASC");
			statement.setInt(1, getObjectId());
			statement.setInt(2, getActiveClass());

			rset = statement.executeQuery();
			while(rset.next())
			{
				int skillId = rset.getInt("skill_id");
				int skillLvl = rset.getInt("skill_level");
				int effectCount = rset.getInt("effect_count");
				long effectCurTime = rset.getLong("effect_cur_time");
				long duration = rset.getLong("duration");

				if(skillId >= L2CubicInstance.CUBIC_STORE_OFFSET) // cubic
				{
					skillId -= L2CubicInstance.CUBIC_STORE_OFFSET;
					addCubic(skillId, skillLvl, (int) effectCurTime);
					continue;
				}

				L2Skill skill = SkillTable.getInstance().getInfo(skillId, skillLvl);

				for(EffectTemplate et : skill.getEffectTemplates())
				{
					Env env = new Env(this, this, skill);
					L2Effect effect = et.getEffect(env);
					if(effect != null)
					{
						if(effectCount == 1)
						{
							effect.setCount(effectCount);
							effect.setPeriod(duration - effectCurTime);
						}
						else
						{
							effect.setPeriod(duration);
							effect.setCount(effectCount);
						}
						addEffect(effect);
					}
				}
			}

			DatabaseUtils.closeDatabaseSR(statement, rset);

			statement = con.prepareStatement("DELETE FROM character_effects_save WHERE char_obj_id = ? AND class_index=?");
			statement.setInt(1, getObjectId());
			statement.setInt(2, getActiveClass());
			statement.executeUpdate();
		}
		catch(final Exception e)
		{
			_log.warning("Could not restore active effects data: " + e);
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		updateEffectIcons();
		broadcastUserInfo();
	}

	public void restoreDisableSkills()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement("SELECT skill_id,end_time,reuse_delay_org FROM character_skills_save WHERE char_obj_id=? AND class_index=?");
			statement.setInt(1, getObjectId());
			statement.setInt(2, getActiveClass());

			rset = statement.executeQuery();
			while(rset.next())
			{
				int skillId = rset.getInt("skill_id");
				int skillLevel = Math.max(getSkillLevel(skillId), 1);
				long endTime = rset.getLong("end_time");
				long rDelayOrg = rset.getLong("reuse_delay_org");
				long curTime = System.currentTimeMillis();

				L2Skill skill = SkillTable.getInstance().getInfo(skillId, skillLevel);

				if(skill != null && endTime - curTime > 10)
				{
					getSkillReuseTimeStamps().put(skillId, new SkillTimeStamp(skillId, endTime, rDelayOrg));
					disableSkill(skillId, endTime - curTime);
					disableItem(skill, rDelayOrg, endTime - curTime);
				}
			}
			DatabaseUtils.closeDatabaseSR(statement, rset);

			statement = con.prepareStatement("DELETE FROM character_skills_save WHERE char_obj_id = ? AND class_index=?");
			statement.setInt(1, getObjectId());
			statement.setInt(2, getActiveClass());
			statement.executeUpdate();
		}
		catch(Exception e)
		{
			_log.warning("Could not restore active skills data for " + getObjectId() + "/" + getActiveClass());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		updateEffectIcons();
	}

	@Override
	public void disableItem(L2Skill handler, long timeTotal, long timeLeft)
	{
		if(handler.isHandler() && timeLeft > 1000)
			if(handler.getReuseGroupId() > 0)
			{
				sendPacket(new ExUseSharedGroupItem(handler._itemConsumeId[0], handler.getReuseGroupId(), (int) timeLeft, (int) timeTotal));
				for(L2Skill sk : handler.getReuseGroup())
					if(!isSkillDisabled(sk.getId()))
						disableSkill(sk.getId(), timeLeft);
			}
	}

	/**
	 * Retrieve from the database all Henna of this L2Player, add them to _henna and calculate stats of the L2Player.<BR><BR>
	 */
	private void restoreHenna()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("select slot, symbol_id from character_hennas where char_obj_id=? AND class_index=?");
			statement.setInt(1, getObjectId());
			statement.setInt(2, getActiveClass());
			rset = statement.executeQuery();

			for(int i = 0; i < 3; i++)
				_henna[i] = null;

			while(rset.next())
			{
				final int slot = rset.getInt("slot");
				if(slot < 1 || slot > 3)
					continue;

				final int symbol_id = rset.getInt("symbol_id");

				L2HennaInstance sym;

				if(symbol_id != 0)
				{
					final L2Henna tpl = HennaTable.getInstance().getTemplate(symbol_id);
					if(tpl != null)
					{
						sym = new L2HennaInstance(tpl);
						_henna[slot - 1] = sym;
					}
				}
			}
		}
		catch(final Exception e)
		{
			_log.warning("could not restore henna: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		// Calculate Henna modifiers of this L2Player
		recalcHennaStats();

	}

	/**
	 * @return the number of Henna empty slot of the L2Player.<BR><BR>
	 */
	public int getHennaEmptySlots()
	{
		int totalSlots = 1 + getClassId().level();
		for(int i = 0; i < 3; i++)
			if(_henna[i] != null)
				totalSlots--;

		if(totalSlots <= 0)
			return 0;

		return totalSlots;

	}

	/**
	 * Remove a Henna of the L2Player, save update in the character_hennas table of the database and send Server->Client HennaInfo/UserInfo packet to this L2Player.<BR><BR>
	 */
	public boolean removeHenna(int slot)
	{
		if(slot < 1 || slot > 3)
			return false;

		slot--;

		if(_henna[slot] == null)
			return false;

		final L2HennaInstance henna = _henna[slot];
		final short dyeID = henna.getItemIdDye();

		// Added by Tempy - 10 Aug 05
		// Gives amount equal to half of the dyes needed for the henna back.
		final L2ItemInstance hennaDyes = ItemTable.getInstance().createItem(dyeID, getObjectId(), 0, "L2Player.removeHenna");
		hennaDyes.setCount(henna.getAmountDyeRequire() / 2);

		_henna[slot] = null;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM character_hennas where char_obj_id=? and slot=? and class_index=?");
			statement.setInt(1, getObjectId());
			statement.setInt(2, slot + 1);
			statement.setInt(3, getActiveClass());
			statement.execute();
		}
		catch(final Exception e)
		{
			_log.warning("could not remove char henna: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}

		// Calculate Henna modifiers of this L2Player
		recalcHennaStats();

		// Send Server->Client HennaInfo packet to this L2Player
		sendPacket(new HennaInfo(this));

		// Send Server->Client UserInfo packet to this L2Player
		sendPacket(new UserInfo(this));

		// Add the recovered dyes to the player's inventory and notify them.
		getInventory().addItem(hennaDyes);
		final SystemMessage sm = new SystemMessage(SystemMessage.EARNED_S2_S1_s);
		sm.addItemName(henna.getItemIdDye());
		sm.addNumber(henna.getAmountDyeRequire() / 2);
		sendPacket(sm);

		return true;
	}

	/**
	 * Add a Henna to the L2Player, save update in the character_hennas table of the database and send Server->Client HennaInfo/UserInfo packet to this L2Player.<BR><BR>
	 * @param henna L2HennaInstance РґР»СЏ РґРѕР±Р°РІР»РµРЅРёСЏ
	 */
	public boolean addHenna(final L2HennaInstance henna)
	{
		if(getHennaEmptySlots() == 0)
		{
			sendPacket(Msg.NO_SLOT_EXISTS_TO_DRAW_THE_SYMBOL);
			return false;
		}

		// int slot = 0;
		for(int i = 0; i < 3; i++)
			if(_henna[i] == null)
			{
				_henna[i] = henna;

				// Calculate Henna modifiers of this L2Player
				recalcHennaStats();

				ThreadConnection con = null;
				FiltredPreparedStatement statement = null;
				try
				{
					con = L2DatabaseFactory.getInstance().getConnection();
					statement = con.prepareStatement("INSERT INTO `character_hennas` (char_obj_id, symbol_id, slot, class_index) VALUES (?,?,?,?)");
					statement.setInt(1, getObjectId());
					statement.setInt(2, henna.getSymbolId());
					statement.setInt(3, i + 1);
					statement.setInt(4, getActiveClass());
					statement.execute();
				}
				catch(final Exception e)
				{
					_log.warning("could not save char henna: " + e);
				}
				finally
				{
					DatabaseUtils.closeDatabaseCS(con, statement);
				}

				// Send Server->Client HennaInfo packet to this L2Player
				final HennaInfo hi = new HennaInfo(this);
				sendPacket(hi);

				// Send Server->Client UserInfo packet to this L2Player
				sendPacket(new UserInfo(this));

				return true;
			}

		return false;
	}

	/**
	 * Calculate Henna modifiers of this L2Player.<BR><BR>
	 */
	private void recalcHennaStats()
	{
		_hennaINT = 0;
		_hennaSTR = 0;
		_hennaCON = 0;
		_hennaMEN = 0;
		_hennaWIT = 0;
		_hennaDEX = 0;

		for(int i = 0; i < 3; i++)
		{
			if(_henna[i] == null)
				continue;
			_hennaINT += _henna[i].getStatINT();
			_hennaSTR += _henna[i].getStatSTR();
			_hennaMEN += _henna[i].getStatMEM();
			_hennaCON += _henna[i].getStatCON();
			_hennaWIT += _henna[i].getStatWIT();
			_hennaDEX += _henna[i].getStatDEX();
		}

		if(_hennaINT > 5)
			_hennaINT = 5;
		if(_hennaSTR > 5)
			_hennaSTR = 5;
		if(_hennaMEN > 5)
			_hennaMEN = 5;
		if(_hennaCON > 5)
			_hennaCON = 5;
		if(_hennaWIT > 5)
			_hennaWIT = 5;
		if(_hennaDEX > 5)
			_hennaDEX = 5;

	}

	/**
	 * @param slot id СЃР»РѕС‚Р° Сѓ РїРµСЂСЃР°
	 * @return the Henna of this L2Player corresponding to the selected slot.<BR><BR>
	 */
	public L2HennaInstance getHenna(final int slot)
	{
		if(slot < 1 || slot > 3)
			return null;
		return _henna[slot - 1];
	}

	/**
	 * @return the INT Henna modifier of this L2Player.<BR><BR>
	 */
	public int getHennaStatINT()
	{
		return _hennaINT;
	}

	/**
	 * @return the STR Henna modifier of this L2Player.<BR><BR>
	 */
	public int getHennaStatSTR()
	{
		return _hennaSTR;
	}

	/**
	 * @return the CON Henna modifier of this L2Player.<BR><BR>
	 */
	public int getHennaStatCON()
	{
		return _hennaCON;
	}

	/**
	 * @return the MEN Henna modifier of this L2Player.<BR><BR>
	 */
	public int getHennaStatMEN()
	{
		return _hennaMEN;
	}

	/**
	 * @return the WIT Henna modifier of this L2Player.<BR><BR>
	 */
	public int getHennaStatWIT()
	{
		return _hennaWIT;
	}

	/**
	 * @return the DEX Henna modifier of this L2Player.<BR><BR>
	 */
	public int getHennaStatDEX()
	{
		return _hennaDEX;
	}

	/**
	 * Reduce Item quantity of the L2Player Inventory.<BR><BR>
	 */
	@Override
	public boolean consumeItem(final int itemConsumeId, final int itemCount)
	{
		L2ItemInstance item = getInventory().findItemByItemId(itemConsumeId);
		if(item == null || item.getCount() < itemCount)
			return false;
		return getInventory().destroyItem(item, itemCount, false) != null;
	}

	/**
	 * @return True if the L2Player is a Mage.<BR><BR>
	 */
	@Override
	public boolean isMageClass()
	{
		return _template.baseMAtk > 3;
	}

	public boolean isMounted()
	{
		return _mountNpcId > 0;
	}

	/**
	 * РџСЂРѕРІРµСЂСЏРµС‚, РјРѕР¶РЅРѕ Р»Рё РїСЂРёР·РµРјР»РёС‚СЊСЃСЏ РІ СЌС‚РѕР№ Р·РѕРЅРµ.
	 * @return РјРѕР¶РЅРѕ Р»Рё РїСЂРёР·РµРјР»РёС‚СЃСЏ
	 */
	public boolean checkLandingState()
	{
		if(isInZone(no_landing))
			return false;
		else if(isInZone(Castle))
			return _clan != null && isClanLeader() && (_clan.getHasCastle() == CastleManager.getInstance().getCastleIndex(this) || _clan.getHasFortress() == FortressManager.getInstance().getFortressIndex(this));
		return true;
	}

	public void setMount(int npcId, int obj_id)
	{
		switch(npcId)
		{
			case 0: // Dismount
				setFlying(false);
				setRiding(false);
				removeSkillById(325); // remove skill "Strider Assault"
				removeSkillById(4289); // remove skill "Wyvern Breath"
				removeSkillFromShortCut(325);
				removeSkillFromShortCut(4289);
				break;
			case 12526: // Wind
			case 12527: // Star
			case 12528: // Twilight
				setRiding(true);
				if(isNoble())
					addSkill(SkillTable.getInstance().getInfo(325, 1), false); // add skill "Strider Assault"
				break;
			case 12621: // Wyvern
				setFlying(true);
				addSkill(SkillTable.getInstance().getInfo(4289, 1), false); // add skill "Wyvern Breath"
				break;
		}

		if(npcId > 0 && !isCursedWeaponEquipped())
		{
			L2ItemInstance wpn = getInventory().getPaperdollItem(Inventory.PAPERDOLL_LHAND);
			if(wpn != null)
				sendDisarmMessage(wpn);
			getInventory().unEquipItemInSlot(Inventory.PAPERDOLL_LHAND);

			wpn = getInventory().getPaperdollItem(Inventory.PAPERDOLL_RHAND);
			if(wpn != null)
				sendDisarmMessage(wpn);
			getInventory().unEquipItemInSlot(Inventory.PAPERDOLL_RHAND);

			refreshExpertisePenalty();
			abortAttack();
			abortCast();
		}

		_mountNpcId = npcId;
		_mountObjId = obj_id;

		if(!isMounted())
			broadcastPacket(new Ride(this));
		broadcastUserInfo();
		sendPacket(new SkillList(this));
	}

	private void sendDisarmMessage(L2ItemInstance wpn)
	{
		if(wpn.getEnchantLevel() > 0)
		{
			SystemMessage sm = new SystemMessage(SystemMessage.EQUIPMENT_OF__S1_S2_HAS_BEEN_REMOVED);
			sm.addNumber(wpn.getEnchantLevel());
			sm.addItemName(wpn.getItemId());
			sendPacket(sm);
		}
		else
		{
			SystemMessage sm = new SystemMessage(SystemMessage.S1__HAS_BEEN_DISARMED);
			sm.addItemName(wpn.getItemId());
			sendPacket(sm);
		}
	}

	private int _mountNpcId;
	private int _mountObjId;

	public void setMountNpcId(int id)
	{
		_mountNpcId = id;
	}

	public int getMountNpcId()
	{
		return _mountNpcId;
	}

	public void setMountObjId(int id)
	{
		_mountObjId = id;
	}

	public int getMountObjId()
	{
		return _mountObjId;
	}

	/**
	 * Send a Server->Client packet UserInfo to this L2Player and CharInfo to all L2Player in its _KnownPlayers.<BR><BR>
	 *
	 * <B><U> Concept</U> :</B><BR><BR>
	 * Others L2Player in the detection area of the L2Player are identified in <B>_knownPlayers</B>.
	 * In order to inform other players of this L2Player state modifications, server just need to go through _knownPlayers to send Server->Client Packet<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Send a Server->Client packet UserInfo to this L2Player (Public and Private Data)</li>
	 * <li>Send a Server->Client packet CharInfo to all L2Player in _KnownPlayers of the L2Player (Public data only)</li><BR><BR>
	 *
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : DON'T SEND UserInfo packet to other players instead of CharInfo packet.
	 * Indeed, UserInfo packet contains PRIVATE DATA as MaxHP, STR, DEX...</B></FONT><BR><BR>
	 *
	 */
	@Override
	public void updateAbnormalEffect()
	{
		sendChanges();
	}

	/**
	 * Disable the Inventory and create a new task to enable it after 1.5s.<BR><BR>
	 */
	public void tempInvetoryDisable()
	{
		_inventoryDisable = true;

		ThreadPoolManager.getInstance().scheduleGeneral(new InventoryEnable(), 1500);
	}

	/**
	 * @return True if the Inventory is disabled.<BR><BR>
	 */
	public boolean isInvetoryDisabled()
	{
		return _inventoryDisable;
	}

	class InventoryEnable implements Runnable
	{
		public void run()
		{
			_inventoryDisable = false;
		}
	}

	/**
	 * РЈСЃС‚Р°РЅР°РІР»РёРІР°РµС‚ С‚РёРї РёСЃРїРѕР»СЊР·СѓРµРјРѕРіРѕ СЃРєР»Р°РґР°.
	 * @param type С‚РёРї СЃРєР»Р°РґР°:<BR>
	 * <ul>
	 * <li>WarehouseType.PRIVATE
	 * <li>WarehouseType.CLAN
	 * <li>WarehouseType.CASTLE
	 * <li>WarehouseType.FREIGHT
	 * </ul>
	 */
	public void setUsingWarehouseType(final WarehouseType type)
	{
		_usingWHType = type;
	}

	/**
	 * Р’РѕР·РІСЂР°С‰Р°РµС‚ С‚РёРї РёСЃРїРѕР»СЊР·СѓРµРјРѕРіРѕ СЃРєР»Р°РґР°.
	 * @return null РёР»Рё С‚РёРї СЃРєР»Р°РґР°:<br>
	 * <ul>
	 * <li>WarehouseType.PRIVATE
	 * <li>WarehouseType.CLAN
	 * <li>WarehouseType.CASTLE
	 * <li>WarehouseType.FREIGHT
	 * </ul>
	 */
	public WarehouseType getUsingWarehouseType()
	{
		return _usingWHType;
	}

	public HashMap<Integer, L2CubicInstance> getCubics()
	{
		return _cubics;
	}

	/**
	 * Add a L2CubicInstance to the L2Player _cubics.<BR><BR>
	 */
	public void addCubic(final int id, final int level, final int lifetime)
	{
		final L2CubicInstance cubic = new L2CubicInstance(this, id, level, lifetime);
		if(_cubics.containsKey(id))
		{
			L2CubicInstance old = _cubics.get(id);
			old.stopAction();
			old.cancelDisappear();
		}
		_cubics.put(id, cubic);
	}

	/**
	 * Remove a L2CubicInstance from the L2Player _cubics.<BR><BR>
	 */
	public void delCubic(final int id)
	{
		_cubics.remove(id);
	}

	/**
	 * @return the L2CubicInstance corresponding to the Identifier of the L2Player _cubics.<BR><BR>
	 */
	public L2CubicInstance getCubic(final int id)
	{
		return _cubics.get(id);
	}

	@Override
	public String toString()
	{
		return "player '" + getName() + "'";
	}

	/**
	 * @return the modifier corresponding to the Enchant Effect of the Active Weapon (Min : 127).<BR><BR>
	 */
	public int getEnchantEffect()
	{
		final L2ItemInstance wpn = getActiveWeaponInstance();

		if(wpn == null)
			return 0;

		return Math.min(127, wpn.getEnchantLevel());
	}

	/**
	 * Set the _lastFolkNpc of the L2Player corresponding to the last Folk witch one the player talked.<BR><BR>
	 */
	public void setLastNpc(final L2NpcInstance npc)
	{
		_lastFolkNpc = npc;
	}

	/**
	 * @return the _lastFolkNpc of the L2Player corresponding to the last Folk witch one the player talked.<BR><BR>
	 */
	public L2NpcInstance getLastNpc()
	{
		return _lastFolkNpc;
	}

	/**
	 * @return True if L2Player is a participant in the Festival of Darkness.<BR><BR>
	 */
	public boolean isFestivalParticipant()
	{
		return SevenSignsFestival.getInstance().isParticipant(this);
	}

	@Override
	public boolean unChargeShots(boolean spirit)
	{
		L2ItemInstance weapon = getActiveWeaponInstance();
		if(weapon == null)
			return false;

		if(spirit)
			weapon.setChargedSpiritshot(L2ItemInstance.CHARGED_NONE);
		else
			weapon.setChargedSoulshot(L2ItemInstance.CHARGED_NONE);

		AutoShot();
		return true;
	}

	public boolean unChargeFishShot()
	{
		L2ItemInstance weapon = getActiveWeaponInstance();
		if(weapon == null)
			return false;
		weapon.setChargedFishshot(false);
		AutoShot();
		return true;
	}

	public void AutoShot()
	{
		synchronized (_activeSoulShots)
		{
			for(Integer e : _activeSoulShots)
			{
				if(e == null)
					continue;
				L2ItemInstance item = getInventory().findItemByItemId(e);
				if(item == null)
				{
					_activeSoulShots.remove(e);
					continue;
				}
				IItemHandler handler = ItemHandler.getInstance().getItemHandler(e);
				if(handler == null)
					continue;
				handler.useItem(this, item);
			}
		}
	}

	public boolean getChargedFishShot()
	{
		L2ItemInstance weapon = getActiveWeaponInstance();
		return weapon != null && weapon.getChargedFishshot();
	}

	@Override
	public boolean getChargedSoulShot()
	{
		L2ItemInstance weapon = getActiveWeaponInstance();
		return weapon != null && weapon.getChargedSoulshot() == L2ItemInstance.CHARGED_SOULSHOT;
	}

	@Override
	public int getChargedSpiritShot()
	{
		L2ItemInstance weapon = getActiveWeaponInstance();
		if(weapon == null)
			return 0;
		return weapon.getChargedSpiritshot();
	}

	public void addAutoSoulShot(Integer itemId)
	{
		_activeSoulShots.add(itemId);
	}

	public void removeAutoSoulShot(Integer itemId)
	{
		_activeSoulShots.remove(itemId);
	}

	public ConcurrentSkipListSet<Integer> getAutoSoulShot()
	{
		return _activeSoulShots;
	}

	class WaterTask implements Runnable
	{
		public void run()
		{
			if(isDead() || !isInZoneIncludeZ(ZoneType.water))
			{
				_taskWater.cancel(false);
				_taskWater = null;
				return;
			}

			double reduceHp = getMaxHp() / 100;

			if(reduceHp < 1)
				reduceHp = 1;

			reduceCurrentHp(reduceHp, L2Player.this, false, false, true);
			//reduced hp, because not rest
			SystemMessage sm = new SystemMessage(SystemMessage.YOU_RECEIVED_S1_DAMAGE_BECAUSE_YOU_WERE_UNABLE_TO_BREATHE);
			sm.addNumber((int) reduceHp);
			sendPacket(sm);
		}
	}

	public void setInvisible(boolean vis)
	{
		_invisible = vis;
	}

	public boolean isInvisible()
	{
		return _invisible;
	}

	public int getClanPrivileges()
	{
		if(_clan == null || _powerGrade < 1 || _powerGrade > 9)
			return 0;
		else if(isClanLeader())
			return L2Clan.CP_ALL;
		else
		{
			RankPrivs privs = _clan.getRankPrivs(_powerGrade);
			if(privs != null)
				return privs.getPrivs();
			return 0;
		}
	}

	public synchronized boolean enterObserverMode(int x, int y, int z)
	{
		L2WorldRegion observRegion = L2World.getRegion(x, y, z);
		if(observRegion == null)
			return false;
		_observNeighbors = observRegion.getNeighbors();
		if(_observNeighbors == null)
			return false;

		L2WorldRegion oldRegion = L2World.getRegion(getX(), getY(), getZ());
		if(oldRegion == null)
			return false;

		// Р•СЃР»Рё С†РµР»СЊ СЃР»РёС€РєРѕРј Р±Р»РёР·РєРѕ - Р±СѓРґСѓС‚ РєРѕРЅС„Р»РёРєС‚С‹, РїРѕСЌС‚РѕРјСѓ РёСЃРєР»СЋС‡Р°РµРј.
		// РњРёРЅРёРјР°Р»СЊРЅРѕРµ СЂР°СЃСЃС‚РѕСЏРЅРёРµ РѕС‚ РєСЂРёСЃС‚Р°Р»Р»Р° РґРѕ С‚РѕС‡РєРё РЅР°Р±Р»СЋРґРµРЅРёСЏ - 8000, РјРµРЅСЊС€Рµ РЅРµ РґРµР»Р°С‚СЊ
		L2WorldRegion[] neighbors = oldRegion.getNeighbors();
		for(L2WorldRegion oldNeighbor : neighbors)
			if(oldNeighbor != null)
				for(L2WorldRegion observNeighbor : _observNeighbors)
					if(observNeighbor != null)
						if(oldNeighbor.equals(observNeighbor))
							return false;

		setTarget(null);
		stopMove();
		block();
		setIsInvul(true);

		// РќСѓР¶РЅРѕ РїСЂРё С‚РµР»РµРїРѕСЂС‚Рµ СЃ Р±РѕР»РµРµ РІС‹СЃРѕРєРѕР№ С‚РѕС‡РєРё РЅР° Р±РѕР»РµРµ РЅРёР·РєСѓСЋ, РёРЅР°С‡Рµ РЅР°РЅРѕСЃРёС‚СЃСЏ РІСЂРµРґ РѕС‚ "РїР°РґРµРЅРёСЏ"
		setLastClientPosition(null);
		setLastServerPosition(null);

		// "РўРµР»РµРїРѕСЂС‚РёСЂСѓРµРјСЃСЏ"
		_observerMode = true;
		broadcastPacketToOthers(new ChangeWaitType(this, ChangeWaitType.WT_SITTING));
		broadcastUserInfo();
		sendPacket(new ObserverStart(x, y, z));

		return true;
	}

	public synchronized void appearObserverMode()
	{
		if(!_observerMode || _observNeighbors == null)
			return;
		// Р”РѕР±Р°РІР»СЏРµРј С„СЌР№Рє РІ С‚РѕС‡РєСѓ РЅР°Р±Р»СЋРґРµРЅРёСЏ Рё РїРѕРєР°Р·С‹РІР°РµРј С‡Р°СЂСѓ РІСЃРµ РѕР±СЊРµРєС‚С‹, С‡С‚Рѕ С‚Р°Рј РЅР°С…РѕРґСЏС‚СЃСЏ
		for(L2WorldRegion neighbor : _observNeighbors)
			if(neighbor != null)
			{
				neighbor.showObjectsToPlayer(this);
				neighbor.addVisibleObject(this);
			}
	}

	public synchronized void leaveObserverMode()
	{
		// РЈРґР°Р»СЏРµРј С„СЌР№Рє РёР· С‚РѕС‡РєРё РЅР°Р±Р»СЋРґРµРЅРёСЏ Рё СѓРґР°Р»СЏРµРј Сѓ С‡Р°СЂР° РІСЃРµ РѕР±СЊРµРєС‚С‹, С‡С‚Рѕ С‚Р°Рј РЅР°С…РѕРґСЏС‚СЃСЏ
		if(_observNeighbors != null)
			for(L2WorldRegion neighbor : _observNeighbors)
				if(neighbor != null)
				{
					neighbor.removeObjectsFromPlayer(this);
					neighbor.removeVisibleObject(this, false);
				}

		_observNeighbors = null;

		setTarget(null);
		unblock();
		setIsInvul(false);
		getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);

		// Р”РµР»Р°РµРј РІРёРґ, С‡С‚Рѕ РјС‹ С‚РµР»РµРїРѕСЂС‚РёСЂРѕРІР°Р»РёСЃСЊ :)
		setIsTeleporting(true);

		// РќСѓР¶РЅРѕ РїСЂРё С‚РµР»РµРїРѕСЂС‚Рµ СЃ Р±РѕР»РµРµ РІС‹СЃРѕРєРѕР№ С‚РѕС‡РєРё РЅР° Р±РѕР»РµРµ РЅРёР·РєСѓСЋ, РёРЅР°С‡Рµ РЅР°РЅРѕСЃРёС‚СЃСЏ РІСЂРµРґ РѕС‚ "РїР°РґРµРЅРёСЏ"
		setLastClientPosition(null);
		setLastServerPosition(null);

		// "РўРµР»РµРїРѕСЂС‚РёСЂСѓРµРјСЃСЏ"
		_observerMode = false;
		broadcastPacketToOthers(new ChangeWaitType(this, ChangeWaitType.WT_STANDING));
		broadcastUserInfo();
		sendPacket(new ObserverEnd(this));

		decayMe();
		spawnMe();
	}

	public synchronized void enterOlympiadObserverMode(int[] coords, final int id)
	{
		L2WorldRegion observRegion = L2World.getRegion(coords[0], coords[1], coords[2]);
		if(observRegion == null)
			return;
		_observNeighbors = observRegion.getNeighbors();
		if(_observNeighbors == null)
			return;

		L2WorldRegion oldRegion = L2World.getRegion(getX(), getY(), getZ());
		if(oldRegion == null)
			return;

		// Р•СЃР»Рё С†РµР»СЊ СЃР»РёС€РєРѕРј Р±Р»РёР·РєРѕ - Р±СѓРґСѓС‚ РєРѕРЅС„Р»РёРєС‚С‹, РїРѕСЌС‚РѕРјСѓ РёСЃРєР»СЋС‡Р°РµРј.
		// РњРёРЅРёРјР°Р»СЊРЅРѕРµ СЂР°СЃСЃС‚РѕСЏРЅРёРµ РѕС‚ РєСЂРёСЃС‚Р°Р»Р»Р° РґРѕ С‚РѕС‡РєРё РЅР°Р±Р»СЋРґРµРЅРёСЏ - 8000, РјРµРЅСЊС€Рµ РЅРµ РґРµР»Р°С‚СЊ
		L2WorldRegion[] neighbors = oldRegion.getNeighbors();
		for(L2WorldRegion oldNeighbor : neighbors)
			if(oldNeighbor != null)
				for(L2WorldRegion observNeighbor : _observNeighbors)
					if(observNeighbor != null)
						if(oldNeighbor.equals(observNeighbor))
							return;

		setTarget(null);
		stopMove();
		block();
		setIsInvul(true);

		// РќСѓР¶РЅРѕ РїСЂРё С‚РµР»РµРїРѕСЂС‚Рµ СЃ Р±РѕР»РµРµ РІС‹СЃРѕРєРѕР№ С‚РѕС‡РєРё РЅР° Р±РѕР»РµРµ РЅРёР·РєСѓСЋ, РёРЅР°С‡Рµ РЅР°РЅРѕСЃРёС‚СЃСЏ РІСЂРµРґ РѕС‚ "РїР°РґРµРЅРёСЏ"
		setLastClientPosition(null);
		setLastServerPosition(null);

		// "РўРµР»РµРїРѕСЂС‚РёСЂСѓРµРјСЃСЏ"
		_olympiadGameId = id;
		_observerMode = true;
		broadcastPacketToOthers(new ChangeWaitType(this, ChangeWaitType.WT_SITTING));
		broadcastUserInfo();
		sendPacket(new ExOlympiadMode(3));
		sendPacket(new ObserverStart(coords[0], coords[1], coords[2]));
	}

	public synchronized void leaveOlympiadObserverMode()
	{
		// РЈРґР°Р»СЏРµРј С„СЌР№Рє РёР· С‚РѕС‡РєРё РЅР°Р±Р»СЋРґРµРЅРёСЏ Рё СѓРґР°Р»СЏРµРј Сѓ С‡Р°СЂР° РІСЃРµ РѕР±СЊРµРєС‚С‹, С‡С‚Рѕ С‚Р°Рј РЅР°С…РѕРґСЏС‚СЃСЏ
		if(_observNeighbors != null)
			for(L2WorldRegion neighbor : _observNeighbors)
				if(neighbor != null)
				{
					neighbor.removeObjectsFromPlayer(this);
					neighbor.removeVisibleObject(this, false);
				}

		_observNeighbors = null;

		setTarget(null);
		unblock();
		setIsInvul(false);
		getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);

		// Р”РµР»Р°РµРј РІРёРґ, С‡С‚Рѕ РјС‹ С‚РµР»РµРїРѕСЂС‚РёСЂРѕРІР°Р»РёСЃСЊ :)
		setIsTeleporting(true);

		// РќСѓР¶РЅРѕ РїСЂРё С‚РµР»РµРїРѕСЂС‚Рµ СЃ Р±РѕР»РµРµ РІС‹СЃРѕРєРѕР№ С‚РѕС‡РєРё РЅР° Р±РѕР»РµРµ РЅРёР·РєСѓСЋ, РёРЅР°С‡Рµ РЅР°РЅРѕСЃРёС‚СЃСЏ РІСЂРµРґ РѕС‚ "РїР°РґРµРЅРёСЏ"
		setLastClientPosition(null);
		setLastServerPosition(null);

		// "РўРµР»РµРїРѕСЂС‚РёСЂСѓРµРјСЃСЏ"
		Olympiad.getInstance().removeSpectator(_olympiadGameId, this);
		_olympiadGameId = -1;
		_observerMode = false;
		broadcastPacketToOthers(new ChangeWaitType(this, ChangeWaitType.WT_STANDING));
		broadcastUserInfo();
		sendPacket(new ExOlympiadMode(0));
		sendPacket(new ObserverEnd(this));

		decayMe();
		spawnMe();
	}

	public void setOlympiadSide(final int i)
	{
		_olympiadSide = i;
	}

	public int getOlympiadSide()
	{
		return _olympiadSide;
	}

	public void setOlympiadGameId(final int id)
	{
		_olympiadGameId = id;
	}

	public int getOlympiadGameId()
	{
		return _olympiadGameId;
	}

	public Location getObsLoc()
	{
		return _obsLoc;
	}

	@Override
	public boolean inObserverMode()
	{
		return _observerMode;
	}

	public L2WorldRegion[] getObservNeighbors()
	{
		return _observNeighbors;
	}

	public int getTeleMode()
	{
		return _telemode;
	}

	public void setTeleMode(final int mode)
	{
		_telemode = mode;
	}

	public int getUnstuck()
	{
		return _unstuck;
	}

	public void setUnstuck(final int mode)
	{
		_unstuck = mode;
	}

	public void setLoto(final int i, final int val)
	{
		_loto[i] = val;
	}

	public int getLoto(final int i)
	{
		return _loto[i];
	}

	public void setRace(final int i, final int val)
	{
		_race[i] = val;
	}

	public int getRace(final int i)
	{
		return _race[i];
	}

	public boolean getMessageRefusal()
	{
		return _messageRefusal;
	}

	public void setMessageRefusal(final boolean mode)
	{
		_messageRefusal = mode;
		sendPacket(new EtcStatusUpdate(this));
	}

	public void setTradeRefusal(final boolean mode)
	{
		_tradeRefusal = mode;
	}

	public boolean getTradeRefusal()
	{
		return _tradeRefusal;
	}

	public void setExchangeRefusal(final boolean mode)
	{
		_exchangeRefusal = mode;
	}

	public boolean getExchangeRefusal()
	{
		return _exchangeRefusal;
	}

	public void addToBlockList(L2Player player)
	{
		if(!_blockList.containsKey(player.getObjectId()))
			_blockList.put(player.getObjectId(), player.getName());
	}

	public void removeFromBlockList(final L2Player player)
	{
		_blockList.remove(player.getObjectId());
	}

	public boolean isInBlockList(final L2Player player)
	{
		return _blockList.containsKey(player.getObjectId());
	}

	private void restoreBlockList()
	{
		_blockList.clear();

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rs = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT target_Id, target_Name FROM character_blocklist WHERE obj_Id = ?");
			statement.setInt(1, getObjectId());
			rs = statement.executeQuery();
			while(rs.next())
				_blockList.put(rs.getInt("target_Id"), rs.getString("target_Name"));
		}
		catch(SQLException e)
		{
			_log.warning("Can't restore player blocklist " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rs);
		}
	}

	private void storeBlockList()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM character_blocklist WHERE obj_Id = ?");
			statement.setInt(1, getObjectId());
			statement.execute();
			statement.close();
			statement = con.prepareStatement("INSERT INTO character_blocklist values (?, ?, ?)");

			for(int i : _blockList.keySet())
			{
				statement.setInt(1, getObjectId());
				statement.setInt(2, i);
				statement.setString(3, _blockList.get(i));
				statement.execute();
			}
		}
		catch(Exception e)
		{
			_log.warning("Can't store player blocklist " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public boolean isBlockAll()
	{
		return _blockAll;
	}

	public void setBlockAll(final boolean state)
	{
		_blockAll = state;
	}

	public Collection<String> getBlockList()
	{
		return _blockList.values();
	}

	public void setConnected(boolean connected)
	{
		_isConnected = connected;
	}

	public boolean isConnected()
	{
		return _isConnected;
	}

	public void setHero(final boolean hero)
	{
		_hero = hero;
		updatePledgeClass();
	}

	@Override
	public boolean isHero()
	{
		return _hero;
	}

	public void setIsInOlympiadMode(final boolean b)
	{
		_inOlympiadMode = b;
	}

	public boolean isInOlympiadMode()
	{
		return _inOlympiadMode;
	}

	public boolean isOlympiadGameStart()
	{
		Olympiad.L2OlympiadGame og = Olympiad.getInstance().getOlympiadGame(_olympiadGameId);
		if(og == null)
		{
			_log.warning("OlympiadGameQueue is null, olympiadGameId=" + _olympiadGameId);
			return true;
		}

		if(og.getStarted() == 1)
			return true;
		return false;
	}

	public boolean isOlympiadCompStart()
	{
		Olympiad.L2OlympiadGame og = Olympiad.getInstance().getOlympiadGame(_olympiadGameId);
		if(og == null)
		{
			_log.warning("OlympiadGame is null, olympiadGameId=" + _olympiadGameId);
			return true;
		}
		if(og.getStarted() == 2)
			return true;
		return false;
	}

	public void setNoble(final boolean noble)
	{
		_noble = noble;
	}

	public boolean isNoble()
	{
		return _noble;
	}

	public int getSubLevel()
	{
		return isSubClassActive() ? _level : 0;
	}

	/* varka silenos and ketra orc quests related functions */
	public void setVarka(final int faction)
	{
		_varka = faction;
	}

	public int getVarka()
	{
		return _varka;
	}

	public void setKetra(final int faction)
	{
		_ketra = faction;
	}

	public int getKetra()
	{
		return _ketra;
	}

	public void setRam(final int faction)
	{
		_ram = faction;
	}

	public int getRam()
	{
		return _ram;
	}

	public void setPledgeType(final int typeId)
	{
		_pledgeType = typeId;
	}

	public int getPledgeType()
	{
		return _pledgeType;
	}

	public void setLvlJoinedAcademy(int lvl)
	{
		_lvlJoinedAcademy = lvl;
	}

	public int getLvlJoinedAcademy()
	{
		return _lvlJoinedAcademy;
	}

	public void setPledgeClass(final int classId)
	{
		_pledgeClass = classId;
	}

	public int getPledgeClass()
	{
		return _pledgeClass;
	}

	public void updatePledgeClass()
	{
		byte CLAN_LEVEL = _clan == null ? -1 : _clan.getLevel();
		boolean CLAN_LEADER = _clan != null && _clan.getLeaderId() == _objectId;
		boolean IN_ACADEMY = _clan != null && _clan.isAcademy(_pledgeType);
		boolean IS_GUARD = _clan != null && _clan.isRoyalGuard(_pledgeType);
		boolean IS_KNIGHT = _clan != null && _clan.isOrderOfKnights(_pledgeType);
		boolean IS_GUARD_CAPTAIN = false;
		boolean IS_KNIGHT_BANNERET = false;
		if(_clan != null && _pledgeType == 0)
		{
			int leaderOf = _clan.getClanMember(_objectId).isSubLeader();
			if(_clan.isRoyalGuard(leaderOf))
				IS_GUARD_CAPTAIN = true;
			else if(_clan.isOrderOfKnights(leaderOf))
				IS_KNIGHT_BANNERET = true;
		}

		switch(CLAN_LEVEL)
		{
			case -1:
				_pledgeClass = RANK_VAGABOND;
				break;
			case 0:
			case 1:
			case 2:
			case 3:
				if(CLAN_LEADER)
					_pledgeClass = RANK_HEIR;
				else
					_pledgeClass = RANK_VASSAL;
				break;
			case 4:
				if(CLAN_LEADER)
					_pledgeClass = RANK_KNIGHT;
				else
					_pledgeClass = RANK_VASSAL;
				break;
			case 5:
				if(CLAN_LEADER)
					_pledgeClass = RANK_WISEMAN;
				else
					_pledgeClass = RANK_HEIR;
				break;
			case 6:
				if(CLAN_LEADER)
					_pledgeClass = RANK_BARON;
				else if(IN_ACADEMY)
					_pledgeClass = RANK_VASSAL;
				else if(IS_GUARD_CAPTAIN)
					_pledgeClass = RANK_WISEMAN;
				else if(IS_GUARD)
					_pledgeClass = RANK_HEIR;
				else
					_pledgeClass = RANK_KNIGHT;
				break;
			case 7:
				if(CLAN_LEADER)
					_pledgeClass = RANK_COUNT;
				else if(IN_ACADEMY)
					_pledgeClass = RANK_VASSAL;
				else if(IS_GUARD_CAPTAIN)
					_pledgeClass = RANK_VISCOUNT;
				else if(IS_GUARD)
					_pledgeClass = RANK_KNIGHT;
				else if(IS_KNIGHT_BANNERET)
					_pledgeClass = RANK_BARON;
				else if(IS_KNIGHT)
					_pledgeClass = RANK_HEIR;
				else
					_pledgeClass = RANK_WISEMAN;
				break;
			//TODO СѓС‚РѕС‡РЅРёС‚СЊ РґР»СЏ 9 Рё 10 СѓСЂРѕРІРЅРµР№ СЂР°РЅРіРё
			case 8:
			case 9:
			case 10:
				if(CLAN_LEADER)
					_pledgeClass = RANK_MARQUIS;
				else if(IN_ACADEMY)
					_pledgeClass = RANK_VASSAL;
				else if(IS_GUARD_CAPTAIN)
					_pledgeClass = RANK_COUNT;
				else if(IS_GUARD)
					_pledgeClass = RANK_WISEMAN;
				else if(IS_KNIGHT_BANNERET)
					_pledgeClass = RANK_VISCOUNT;
				else if(IS_KNIGHT)
					_pledgeClass = RANK_KNIGHT;
				else
					_pledgeClass = RANK_BARON;
				break;
		}

		if(_hero)
			_pledgeClass = RANK_MARQUIS;
		else if(_noble && _pledgeClass < RANK_BARON)
			_pledgeClass = RANK_BARON;
	}

	public void setPowerGrade(final int grade)
	{
		_powerGrade = grade;
	}

	public int getPowerGrade()
	{
		return _powerGrade;
	}

	public void setApprentice(final int apprentice)
	{
		_apprentice = apprentice;
	}

	public int getApprentice()
	{
		return _apprentice;
	}

	public int getSponsor()
	{
		if(_clan == null)
			return 0;
		return _clan.getClanMember(getObjectId()).getSponsor();
	}

	public void setTeam(final int team)
	{
		_team = team;

		broadcastUserInfo();
		if(getPet() != null)
			getPet().broadcastPetInfo();
	}

	@Override
	public int getTeam()
	{
		return _team;
	}

	public int getNameColor()
	{
		if(inObserverMode())
			return Color.black.getRGB();

		return _nameColor;
	}

	public void setNameColor(final int nameColor)
	{
		if(nameColor != Config.NORMAL_NAME_COLOUR && nameColor != Config.CLANLEADER_NAME_COLOUR && nameColor != Config.GM_NAME_COLOUR && nameColor != Config.SERVICES_OFFLINE_TRADE_NAME_COLOR)
			setVar("namecolor", Integer.toHexString(nameColor));
		else if(nameColor == Config.NORMAL_NAME_COLOUR)
			unsetVar("namecolor");
		_nameColor = nameColor;
	}

	public void setNameColor(final int red, final int green, final int blue)
	{
		_nameColor = (red & 0xFF) + ((green & 0xFF) << 8) + ((blue & 0xFF) << 16);
		if(_nameColor != Config.NORMAL_NAME_COLOUR && _nameColor != Config.CLANLEADER_NAME_COLOUR && _nameColor != Config.GM_NAME_COLOUR && _nameColor != Config.SERVICES_OFFLINE_TRADE_NAME_COLOR)
			setVar("namecolor", Integer.toHexString(_nameColor));
		else
			unsetVar("namecolor");
	}

	public final void illegalAction(final String msg, final Integer jail_items)
	{
		Log.IllegalPlayerAction(this, msg, jail_items);
	}

	public final String toFullString()
	{
		final StringBuffer sb = new StringBuffer(160);

		sb.append("Player '").append(getName()).append("' [oid=").append(_objectId).append(", account='").append(getAccountName()).append(", ip=").append(_connection != null ? _connection.getIpAddr() : "0.0.0.0").append("']");
		return sb.toString();
	}

	private FastMap<String, String> user_variables = new FastMap<String, String>();

	public void setVar(String name, String value)
	{
		user_variables.put(name, value);
		ThreadConnection con = null;
		FiltredPreparedStatement insertion = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			insertion = con.prepareStatement("REPLACE INTO character_variables  (obj_id, type, name, value, expire_time) VALUES (?,\"user-var\",?,?,-1)");
			insertion.setInt(1, _objectId);
			insertion.setString(2, name);
			insertion.setString(3, value);
			insertion.executeUpdate();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, insertion);
		}
	}

	public void unsetVar(String name)
	{
		if(name == null)
			return;

		if(user_variables.remove(name) != null)
		{
			ThreadConnection con = null;
			FiltredPreparedStatement insertion = null;
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				insertion = con.prepareStatement("DELETE FROM character_variables WHERE obj_id = ? AND type = \"user-var\" AND name = ?");
				insertion.setInt(1, _objectId);
				insertion.setString(2, name);
				insertion.executeUpdate();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
			finally
			{
				DatabaseUtils.closeDatabaseCS(con, insertion);
			}
		}
	}

	public String getVar(String name)
	{
		return user_variables.get(name);
	}

	public boolean getVarB(String name)
	{
		String var = user_variables.get(name);
		return !(var == null || var.equals("0") || var.equalsIgnoreCase("false"));
	}

	public void loadVariables()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement offline = null;
		ResultSet rs = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			offline = con.prepareStatement("SELECT * FROM character_variables WHERE obj_id = ?");
			offline.setInt(1, _objectId);
			rs = offline.executeQuery();
			while(rs.next())
			{
				String name = rs.getString("name");
				String value = rs.getString("value");
				user_variables.put(name, value);
			}

			// TODO Р—РґРµСЃСЊ РѕР±Р°Р·СЏС‚РµР»СЊРЅРѕ РІС‹СЃС‚Р°РІР»СЏС‚СЊ РІСЃРµ СЃС‚Р°РЅРґР°СЂС‚РЅС‹Рµ РїР°СЂР°РјРµС‚СЂС‹, РёРЅР°С‡Рµ Р±СѓРґСѓС‚ NPE
			if(getVar("lang@") == null)
				setVar("lang@", Config.DEFAULT_LANG);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, offline, rs);
		}
	}

	public int isAtWarWith(final Integer id)
	{
		if(_clan == null)
			return 0;

		if(_clan.isAtWarWith(id))
			return 1;

		return 0;
	}

	public int isAtWar()
	{
		if(_clan == null)
			return 0;
		if(_clan.isAtWarOrUnderAttack() > 0)
			return 1;
		return 0;
	}

	public void stopWaterTask()
	{
		if(_taskWater != null)
		{
			_taskWater.cancel(true);

			_taskWater = null;
			sendPacket(new SetupGauge(2, 0));
			sendChanges();
		}
	}

	public void startWaterTask()
	{
		if(isDead())
			stopWaterTask();
		else if(Config.ALLOW_WATER && _taskWater == null)
		{
			int timeinwater = (int) (calcStat(Stats.BREATH, 86, null, null) * 1000);
			sendPacket(new SetupGauge(2, timeinwater));
			sendChanges();
			_taskWater = ThreadPoolManager.getInstance().scheduleEffectAtFixedRate(new WaterTask(), timeinwater, 1000);
			sendChanges();
		}
	}

	public boolean isInWater()
	{
		return _taskWater != null;
	}

	public void checkWaterState()
	{
		if(isInZoneIncludeZ(ZoneType.water))
			startWaterTask();
		else
			stopWaterTask();
	}

	private int _ReviveRequested = 0;
	private double _RevivePower = 0;
	private boolean _RevivePet = false;

	public void doRevive(double percent)
	{
		restoreExp(percent);
		doRevive();
	}

	@Override
	public void doRevive()
	{
		super.doRevive();
		unsetVar("lostexp");
		updateEffectIcons();
		AutoShot();
		_ReviveRequested = 0;
		_RevivePower = 0;
		if(isInParty() && getParty().isInDimensionalRift())
			if(!DimensionalRiftManager.getInstance().checkIfInPeaceZone(getLoc()))
				getParty().getDimensionalRift().memberRessurected(this);
	}

	public void reviveRequest(L2Player Reviver, double percent, boolean Pet)
	{
		if(_ReviveRequested == 1)
		{
			if(_RevivePet == Pet)
				Reviver.sendPacket(Msg.BETTER_RESURRECTION_HAS_BEEN_ALREADY_PROPOSED);
			else if(Pet)
				Reviver.sendPacket(Msg.SINCE_THE_MASTER_WAS_IN_THE_PROCESS_OF_BEING_RESURRECTED_THE_ATTEMPT_TO_RESURRECT_THE_PET_HAS_BEEN_CANCELLED);
			else
				Reviver.sendPacket(Msg.WHILE_A_PET_IS_ATTEMPTING_TO_RESURRECT_IT_CANNOT_HELP_IN_RESURRECTING_ITS_MASTER);
			return;
		}
		if(Pet && getPet() != null && getPet().isDead() || !Pet && isDead())
		{
			_ReviveRequested = 1;
			_RevivePower = percent;
			_RevivePet = Pet;
			ConfirmDlg pkt = new ConfirmDlg(SystemMessage.S1_IS_MAKING_AN_ATTEMPT_AT_RESURRECTION_WITH_$S2_EXPERIENCE_POINTS_DO_YOU_WANT_TO_CONTINUE_WITH_THIS_RESURRECTION, 0, 2);
			String lostexps = getVar("lostexp");
			int lostexp = 0;
			if(lostexps != null)
				lostexp = Integer.parseInt(lostexps);
			pkt.addString(Reviver.getName()).addNumber((int) (_RevivePower * lostexp / 100));
			sendPacket(pkt);
		}
	}

	public void reviveAnswer(int answer)
	{
		if(_ReviveRequested != 1 || !isDead() && !_RevivePet || _RevivePet && getPet() != null && !getPet().isDead())
			return;
		if(answer == 1)
			if(!_RevivePet)
			{
				if(_RevivePower != 0)
					doRevive(_RevivePower);
				else
					doRevive();
			}
			else if(getPet() != null)
				if(_RevivePower != 0)
					((L2PetInstance) getPet()).doRevive(_RevivePower);
				else
					getPet().doRevive();
		_ReviveRequested = 0;
		_RevivePower = 0;
	}

	/**
	 * РљРѕРѕСЂРґРёРЅР°С‚С‹ С‚РѕС‡РєРё РїСЂРёР·С‹РІР° РїРµСЂСЃРѕРЅР°Р¶Р°
	 */
	private Location _SummonCharacterCoords;

	/**
	 *  Р¤Р»Р°Рі РЅРµРѕР±С…РѕРґРёРјРѕСЃС‚Рё РїРѕС‚СЂРµР±Р»РµРЅРёСЏ Summoning Cystall-Р° РїСЂРё РїСЂРёР·С‹РІРµ РїРµСЂСЃРѕРЅР°Р¶Р°
	 */
	public boolean _SummonConsumeCrystall = false;

	/**
	 * РћР±СЂР°Р±РѕС‚С‡РёРє РѕС‚РІРµС‚Р° РєР»РёРµРЅС‚Р° РЅР° РїСЂРёР·С‹РІ РїРµСЂСЃРѕРЅР°Р¶Р°.
	 * @param answer Р�РґРµРЅС‚РёС„РёРєР°С‚РѕСЂ Р·Р°РїСЂРѕСЃР°
	 */
	public void summonCharacterAnswer(int answer)
	{
		int summoningCrystallId = 8615;
		if(answer == 1 && _SummonCharacterCoords != null)
		{
			abortAttack();
			abortCast();
			sendActionFailed();
			broadcastPacket(new StopMove(this));
			if(_SummonConsumeCrystall)
			{
				_SummonConsumeCrystall = false;
				L2ItemInstance ConsumedItem = getInventory().getItemByItemId(summoningCrystallId);
				if(ConsumedItem != null && ConsumedItem.getIntegerLimitedCount() > 0)
				{
					getInventory().destroyItemByItemId(summoningCrystallId, 1, false);
					sendPacket(new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(summoningCrystallId));
					teleToLocation(_SummonCharacterCoords);
				}
				else
					sendPacket(Msg.INCORRECT_ITEM_COUNT);
			}
			else
				teleToLocation(_SummonCharacterCoords);
		}
		_SummonCharacterCoords = null;
	}

	/**
	 * РћС‚РїСЂР°РІР»СЏРµС‚ Р·Р°РїСЂРѕСЃ РєР»РёРµРЅС‚Сѓ РЅР° РїСЂРёР·С‹РІ РїРµСЂСЃРѕРЅР°Р¶Р°.
	 * @param SummonerName Р�РјСЏ РїСЂРёР·С‹РІР°СЋС‰РµРіРѕ РїРµСЂСЃРѕРЅР°Р¶Р°
     */
	public void summonCharacterRequest(String SummonerName, Location loc)
	{
		if(_SummonCharacterCoords == null)
		{
			_SummonCharacterCoords = loc;
			ConfirmDlg cd = new ConfirmDlg(SystemMessage.S1_WISHES_TO_SUMMON_YOU_FROM_S2_DO_YOU_ACCEPT, 30000, 1);
			cd.addString(SummonerName).addZoneName(_SummonCharacterCoords.getX(), _SummonCharacterCoords.getY(), _SummonCharacterCoords.getZ());
			sendPacket(cd);
		}
	}

	String _scriptName = "";
	Object[] _scriptArgs = new Object[0];

	public void scriptAnswer(int answer)
	{
		if(answer == 1 && !_scriptName.equals(""))
			callScripts(_scriptName.split(":")[0], _scriptName.split(":")[1], _scriptArgs);
		_scriptName = "";
	}

	public void scriptRequest(String text, String scriptName, Object[] args)
	{
		if(_scriptName.equals(""))
		{
			_scriptName = scriptName;
			_scriptArgs = args;
			sendPacket(new ConfirmDlg(SystemMessage.S1_S2, 30000, 3).addString(text));
		}
	}

	public boolean isReviveRequested()
	{
		return _ReviveRequested == 1;
	}

	public boolean isRevivingPet()
	{
		return _RevivePet;
	}

	public void updateNoChannel(final long time)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;

		setNoChannel(time);

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			final String stmt = "UPDATE characters SET nochannel = ? WHERE obj_Id=?";
			statement = con.prepareStatement(stmt);
			statement.setLong(1, _NoChannel > 0 ? _NoChannel / 1000 : _NoChannel);
			statement.setInt(2, getObjectId());
			statement.executeUpdate();
		}
		catch(final Exception e)
		{
			_log.warning("Could not activate nochannel:" + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	private void checkRecom()
	{
		Calendar temp = Calendar.getInstance();
		temp.set(Calendar.HOUR_OF_DAY, 13);
		temp.set(Calendar.MINUTE, 0);
		temp.set(Calendar.SECOND, 0);
		long count = Math.round((System.currentTimeMillis() / 1000 - _lastAccess) / 86400);
		if(count == 0 && _lastAccess < temp.getTimeInMillis() / 1000 && System.currentTimeMillis() > temp.getTimeInMillis())
			count++;

		for(int i = 1; i < count; i++)
			if(_recomHave < 200)
				_recomHave -= 2;
			else
				_recomHave -= 3;

		if(_recomHave < 0)
			_recomHave = 0;

		if(getLevel() < 10)
			return;

		if(count > 0)
			restartRecom();
	}

	public void restartRecom()
	{
		try
		{
			_recomChars.clear();

			if(getLevel() < 20)
				_recomLeft = 3;
			else if(getLevel() < 40)
				_recomLeft = 6;
			else
				_recomLeft = 9;

			if(_recomHave < 200)
				_recomHave -= 2;
			else
				_recomHave -= 3;

			if(_recomHave < 0)
				_recomHave = 0;
		}
		catch(final Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * @return Returns the inBoat.
	 */
	@Override
	public boolean isInBoat()
	{
		return _InBoat;
	}

	/**
	 * @param inBoat The inBoat to set.
	 */
	public void setInBoat(final boolean inBoat)
	{
		_InBoat = inBoat;
	}

	/**
	 * @return
	 */
	public L2BoatInstance getBoat()
	{
		return _Boat;
	}

	/**
	 * @param boat
	 */
	public void setBoat(final L2BoatInstance boat)
	{
		_Boat = boat;
	}

	public int getBoatId()
	{
		return _boatId;
	}

	public void setBoatId(final int boatId)
	{
		_boatId = boatId;
	}

	public Location getInBoatPosition()
	{
		return _inBoatPosition;
	}

	public void setInBoatPosition(final Location pt)
	{
		_inBoatPosition = pt;
	}

	public HashMap<Short, L2SubClass> getSubClasses()
	{
		return _classlist;
	}

	public void setBaseClass(final int baseClass)
	{
		_baseClass = baseClass;
	}

	public int getBaseClass()
	{
		return _baseClass;
	}

	public int getBaseLevel()
	{
		return _baseLevel;
	}

	public long getBaseExp()
	{
		return _baseExp;
	}

	public long getBaseSp()
	{
		return _baseSp;
	}

	public void setActiveClass(final short activeClass)
	{
		_activeClass = activeClass;
	}

	public short getActiveClass()
	{
		return _activeClass;
	}

	/**
	 * Changing index of class in DB, used for changing class when finished professional quests
	 * @param oldclass
	 * @param newclass
	 */
	public void changeClassInDb(final short oldclass, final short newclass)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement("UPDATE character_subclasses SET class_id=? WHERE char_obj_id=? AND class_id=?");
			statement.setInt(1, newclass);
			statement.setInt(2, getObjectId());
			statement.setInt(3, oldclass);
			statement.executeUpdate();
			DatabaseUtils.closeStatement(statement);

			statement = con.prepareStatement("UPDATE character_hennas SET class_index=? WHERE char_obj_id=? AND class_index=?");
			statement.setInt(1, newclass);
			statement.setInt(2, getObjectId());
			statement.setInt(3, oldclass);
			statement.executeUpdate();
			DatabaseUtils.closeStatement(statement);

			statement = con.prepareStatement("UPDATE character_shortcuts SET class_index=? WHERE char_obj_id=? AND class_index=?");
			statement.setInt(1, newclass);
			statement.setInt(2, getObjectId());
			statement.setInt(3, oldclass);
			statement.executeUpdate();
			DatabaseUtils.closeStatement(statement);

			statement = con.prepareStatement("UPDATE character_skills SET class_index=? WHERE char_obj_id=? AND class_index=?");
			statement.setInt(1, newclass);
			statement.setInt(2, getObjectId());
			statement.setInt(3, oldclass);
			statement.executeUpdate();
			DatabaseUtils.closeStatement(statement);

			statement = con.prepareStatement("UPDATE character_effects_save SET class_index=? WHERE char_obj_id=? AND class_index=?");
			statement.setInt(1, newclass);
			statement.setInt(2, getObjectId());
			statement.setInt(3, oldclass);
			statement.executeUpdate();
			DatabaseUtils.closeStatement(statement);

			statement = con.prepareStatement("UPDATE character_skills_save SET class_index=? WHERE char_obj_id=? AND class_index=?");
			statement.setInt(1, newclass);
			statement.setInt(2, getObjectId());
			statement.setInt(3, oldclass);
			statement.executeUpdate();
			DatabaseUtils.closeStatement(statement);
		}
		catch(final SQLException e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/**
	 * РЎРѕС…СЂР°РЅСЏРµС‚ РёРЅС„РѕСЂРјР°С†РёСЋ Рѕ РєР»Р°СЃСЃР°С… РІ Р‘Р”
	 */
	public void storeCharSubClasses()
	{
		final L2SubClass currentSub = _classlist.get(_activeClass);
		if(currentSub != null)
		{
			currentSub.setCp(getCurrentCp());
			currentSub.setExp(getExp());
			currentSub.setHp(getCurrentHp());
			currentSub.setLevel(getLevel());
			currentSub.setMp(getCurrentMp());
			currentSub.setSp(getSp());
			currentSub.setActive(true);
			_classlist.put(_activeClass, currentSub);
		}

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("UPDATE character_subclasses SET exp=?,sp=?,curHp=?,curMp=?,curCp=?,level=?,active=?,death_penalty=? WHERE char_obj_id=? AND class_id=?");
			for(final L2SubClass subClass : _classlist.values())
			{
				statement.setLong(1, subClass.getExp());
				statement.setLong(2, subClass.getSp());
				statement.setDouble(3, subClass.getHp());
				statement.setDouble(4, subClass.getMp());
				statement.setDouble(5, subClass.getCp());
				statement.setInt(6, subClass.getLevel());
				statement.setInt(7, subClass.isActive() ? 1 : 0);
				statement.setInt(8, subClass.getDeathPenalty().getLevelOnSaveDB());
				statement.setInt(9, getObjectId());
				statement.setInt(10, subClass.getClassId());
				statement.execute();
			}
			DatabaseUtils.closeStatement(statement);

			statement = con.prepareStatement("UPDATE character_subclasses SET maxHp=?,maxMp=?,maxCp=? WHERE char_obj_id=? AND active=1");
			statement.setDouble(1, getMaxHp());
			statement.setDouble(2, getMaxMp());
			statement.setDouble(3, getMaxCp());
			statement.setInt(4, getObjectId());
			statement.execute();
		}
		catch(final Exception e)
		{
			_log.warning("Could not store char sub data: " + e);
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/**
	 * Restore list of character professions and set up active proof
	 * Used when character is loading
	 */
	public static void restoreCharSubClasses(final L2Player player)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT class_id,exp,sp,level,curHp,curCp,curMp,active,isBase,death_penalty FROM character_subclasses WHERE char_obj_id=?");
			statement.setInt(1, player.getObjectId());
			rset = statement.executeQuery();
			while(rset.next())
			{
				final L2SubClass subClass = new L2SubClass();
				subClass.setBase(rset.getInt("isBase") != 0);
				subClass.setClassId(rset.getShort("class_id"));
				subClass.setLevel(rset.getByte("level"));
				subClass.setExp(rset.getLong("exp"));
				subClass.setSp(rset.getInt("sp"));
				subClass.setHp(rset.getDouble("curHp"));
				subClass.setMp(rset.getDouble("curMp"));
				subClass.setCp(rset.getDouble("curCp"));
				subClass.setActive(rset.getInt("active") != 0);
				subClass.setDeathPenalty(new DeathPenalty(player, rset.getByte("death_penalty")));
				subClass.setPlayer(player);

				player._classlist.put(subClass.getClassId(), subClass);
				if(subClass.isActive())
					player.setActiveSubClass(subClass.getClassId(), false);
			}
		}
		catch(final SQLException e)
		{
			_log.warning("Could not restore char sub-classes: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	/**
	 * Р”РѕР±Р°РІРёС‚СЊ РєР»Р°СЃСЃ, РёСЃРїРѕР»СЊР·СѓРµС‚СЃСЏ С‚РѕР»СЊРєРѕ РґР»СЏ СЃР°Р±РєР»Р°СЃСЃРѕРІ
	 */
	public boolean addSubClass(final short classId)
	{
		if(_classlist.size() >= 4)
			return false;

		final ClassId newId = ClassId.values()[classId];

		final L2SubClass newClass = new L2SubClass();
		if(newId.getRace() == null)
			return false;

		newClass.setClassId(classId);
		newClass.setPlayer(this);

		_classlist.put(classId, newClass);

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			// Store the basic info about this new sub-class.
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("INSERT INTO character_subclasses (char_obj_id, class_id, exp, sp, curHp, curMp, curCp, maxHp, maxMp, maxCp, level, active, isBase, death_penalty) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
			statement.setInt(1, getObjectId());
			statement.setInt(2, newClass.getClassId());
			statement.setLong(3, Experience.LEVEL[40]);
			statement.setInt(4, 0);
			statement.setDouble(5, getCurrentHp());
			statement.setDouble(6, getCurrentMp());
			statement.setDouble(7, getCurrentCp());
			statement.setDouble(8, getCurrentHp());
			statement.setDouble(9, getCurrentMp());
			statement.setDouble(10, getCurrentCp());
			statement.setInt(11, 40);
			statement.setInt(12, 0);
			statement.setInt(13, 0);
			statement.setInt(14, 0);
			statement.execute();
		}
		catch(final Exception e)
		{
			_log.warning("Could not add character sub-class: " + e);
			return false;
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}

		setActiveSubClass(classId, true);

		// Add all the necessary skills up to level 40 for this new class.
		boolean countUnlearnable = true;
		int unLearnable = 0;
		int numSkillsAdded = 0;
		ArrayList<L2SkillLearn> skills = SkillTreeTable.getInstance().getAvailableSkills(this, newId);
		while(skills.size() > unLearnable)
		{
			for(final L2SkillLearn s : skills)
			{
				final L2Skill sk = SkillTable.getInstance().getInfo(s.getId(), s.getLevel());
				if(sk == null || !sk.getCanLearn(newId))
				{
					if(countUnlearnable)
						unLearnable++;
					continue;
				}
				addSkill(sk, true);
				numSkillsAdded++;
			}
			countUnlearnable = false;
			skills = SkillTreeTable.getInstance().getAvailableSkills(this, newId);
		}

		restoreSkills();
		rewardSkills();
		sendPacket(new SkillList(this));
		setCurrentHpMp(getMaxHp(), getMaxMp());
		setCurrentCp(getMaxCp());

		if(Config.DEBUG)
			_log.info(numSkillsAdded + " skills added for " + getName() + "'s sub-class.");

		return true;
	}

	/**
	 * РЈРґР°Р»СЏРµС‚ РІСЃСЋ РёРЅС„РѕСЂРјР°С†РёСЋ Рѕ РєР»Р°СЃСЃРµ Рё РґРѕР±Р°РІР»СЏРµС‚ РЅРѕРІСѓСЋ, С‚РѕР»СЊРєРѕ РґР»СЏ СЃР°Р±РєР»Р°СЃСЃРѕРІ
	 */
	public boolean modifySubClass(final short oldClassId, final short newClassId)
	{
		final L2SubClass originalClass = _classlist.get(oldClassId);
		if(originalClass.isBase())
			return false;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			// Remove all basic info stored about this sub-class.
			statement = con.prepareStatement("DELETE FROM character_subclasses WHERE char_obj_id=? AND class_id=? AND isBase = 0");
			statement.setInt(1, getObjectId());
			statement.setInt(2, oldClassId);
			statement.execute();
			DatabaseUtils.closeStatement(statement);

			// Remove all skill info stored for this sub-class.
			statement = con.prepareStatement("DELETE FROM character_skills WHERE char_obj_id=? AND class_index=? ");
			statement.setInt(1, getObjectId());
			statement.setInt(2, oldClassId);
			statement.execute();
			DatabaseUtils.closeStatement(statement);

			// Remove all henna info stored for this sub-class.
			statement = con.prepareStatement("DELETE FROM character_hennas WHERE char_obj_id=? AND class_index=? ");
			statement.setInt(1, getObjectId());
			statement.setInt(2, oldClassId);
			statement.execute();
			DatabaseUtils.closeStatement(statement);

			// Remove all shortcuts info stored for this sub-class.
			statement = con.prepareStatement("DELETE FROM character_shortcuts WHERE char_obj_id=? AND class_index=? ");
			statement.setInt(1, getObjectId());
			statement.setInt(2, oldClassId);
			statement.execute();
		}
		catch(final Exception e)
		{
			_log.warning("Could not delete char sub-class: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
		_classlist.remove(oldClassId);
		return addSubClass(newClassId);
	}

	/**
	 * РЈСЃС‚Р°РЅР°РІР»РёРІР°РµС‚ Р°РєС‚РёРІРЅС‹Р№ СЃР°Р±РєР»Р°СЃСЃ
	 *
	 * <li>Retrieve from the database all skills of this L2Player and add them to _skills </li>
	 * <li>Retrieve from the database all macroses of this L2Player and add them to _macroses</li>
	 * <li>Retrieve from the database all shortCuts of this L2Player and add them to _shortCuts</li><BR><BR>
	 */
	public void setActiveSubClass(final Short subId, final boolean store)
	{
		storeEffects();
		storeDisableSkills();

		if(QuestManager.getQuest(422) != null)
		{
			String qn = QuestManager.getQuest(422).getName();
			if(qn != null)
			{
				QuestState qs = getQuestState(qn);
				if(qs != null)
					qs.exitCurrentQuest(true);
			}
		}

		if(store && _classlist.containsKey(_activeClass))
		{
			final L2SubClass oldsub = _classlist.get(_activeClass);
			oldsub.setCp(getCurrentCp());
			oldsub.setExp(getExp());
			oldsub.setHp(getCurrentHp());
			oldsub.setLevel(getLevel());
			oldsub.setMp(getCurrentMp());
			oldsub.setSp(getSp());
			oldsub.setActive(false);
			_classlist.put(_activeClass, oldsub);
		}

		final L2SubClass sub = _classlist.get(subId);
		sub.setActive(true);
		_activeClass = sub.getClassId();
		_classlist.put(_activeClass, sub);
		_level = sub.getLevel();
		_exp = sub.getExp();
		_sp = sub.getSp();

		setClassId(subId, false);

		removeAllSkills();

		stopAllEffects();

		if(getPet() != null && getPet().isSummon)
			getPet().unSummon();
		if(!_cubics.isEmpty())
		{
			for(final L2CubicInstance cubic : _cubics.values())
			{
				cubic.stopAction();
				cubic.cancelDisappear();
			}
			_cubics.clear();
		}

		checkRecom();
		restoreSkills();
		rewardSkills();
		sendPacket(new SkillList(this));

		getInventory().refreshListeners();

		for(int i = 0; i < 3; i++)
			_henna[i] = null;

		restoreHenna();
		sendPacket(new HennaInfo(this));

		restoreEffects();
		if(isInWorld())
			restoreDisableSkills();

		_currentHp = sub.getHp();
		setCurrentHpMp(sub.getHp(), sub.getMp());
		setCurrentCp(sub.getCp());
		broadcastUserInfo();
		updateStats();

		_shortCuts.restore();
		sendPacket(new ShortCutInit(this));
		if(!isBot() || Config.KILL_BOTS_WITH_SKILLCOOLTIME)
			sendPacket(new SkillCoolTime(this));

		broadcastPacket(new SocialAction(getObjectId(), 15));

		getDeathPenalty().restore();

		setIncreasedForce(0);
	}

	public void setIsBot(final int isBot)
	{
		_isBot = isBot;
	}

	public int isBotLevel()
	{
		return _isBot;
	}

	public boolean isBot()
	{
		return _isBot > 0;
	}

	public void startBotCheck()
	{
		if(_botCheck == null && _isBot > 0)
			_botCheck = ThreadPoolManager.getInstance().scheduleGeneral(new BotCheck(this, _isBot), Rnd.get(30, 90) * 1000);
	}

	public void stopBotCheck()
	{
		if(_botCheck != null)
		{
			_botCheck.cancel(true);
			_botCheck = null;
		}
	}

	class BotCheck implements Runnable
	{
		L2Player _player;
		int _isBot;

		BotCheck(final L2Player player, final int isBot)
		{
			_player = player;
			_isBot = isBot;
		}

		public void run()
		{
			if(!_player.isBot())
			{
				_player.stopBotCheck();
				return;
			}
			if(Config.ALLOW_L2WALKER_CLIENT == Config.L2WalkerAllowed.True || _player.getPlayerAccess().AllowWalker || Config.ALLOW_L2WALKER_CLIENT == L2WalkerAllowed.Peace && isInZone(ZoneType.peace_zone))
			{
				setNameColor(Config.BOT_NAME_COLOUR);
				return;
			}
			_log.warning("Bot detected. Char: " + _player.getName() + ", Account: " + _player.getAccountName() + ", Ip: " + _player.getIP());
			if(Config.L2WALKER_PUNISHMENT == 1)
				_player.logout(false, false, true);
			else if(Config.L2WALKER_PUNISHMENT == 2)
			{
				_player.setAccountAccesslevel(-66, "botcheck", -1);
				_player.logout(false, false, true);
			}
		}
	}

	public int getInventoryLimit()
	{
		return (int) calcStat(Stats.INVENTORY_LIMIT, 0, null, null);
	}

	public int getWarehouseLimit()
	{
		return (int) calcStat(Stats.STORAGE_LIMIT, 0, null, null);
	}

	public int getFreightLimit()
	{
		// FIXME РќРµ СѓС‡РёС‚С‹РІР°РµС‚СЃСЏ РєРѕР»РёС‡РµСЃС‚РІРѕ РїСЂРµРґРјРµС‚РѕРІ, СѓР¶Рµ РёРјРµСЋС‰РµРµСЃСЏ РЅР° СЃРєР»Р°РґРµ
		return getWarehouseLimit();
	}

	public int getTradeLimit()
	{
		return (int) calcStat(Stats.TRADE_LIMIT, 0, null, null);
	}

	public int getDwarvenRecipeLimit()
	{
		return (int) calcStat(Stats.DWARVEN_RECIPE_LIMIT, 50, null, null) + Config.ALT_ADD_RECIPES;
	}

	public int getCommonRecipeLimit()
	{
		return (int) calcStat(Stats.COMMON_RECIPE_LIMIT, 50, null, null) + Config.ALT_ADD_RECIPES;
	}

	@Override
	public int getNpcId()
	{
		return -2;
	}

	public L2Object getVisibleObject(int id)
	{
		if(getObjectId() == id)
			return this;

		if(getTargetId() == id)
			return getTarget();

		if(_party != null)
			for(L2Player p : _party.getPartyMembers())
				if(p != null && p.getObjectId() == id)
					return p;

		return L2World.getAroundObjectById(this, id);
	}

	@Override
	public int getPAtk(final L2Character target)
	{
		double init = getActiveWeaponInstance() == null ? (isMageClass() ? 3 : 4) : 0;
		return (int) calcStat(Stats.POWER_ATTACK, init, target, null);
	}

	@Override
	public int getMAtk(final L2Character target, final L2Skill skill)
	{
		if(skill != null && skill.getMatak() > 0)
			return skill.getMatak();
		final double init = getActiveWeaponInstance() == null ? 6 : 0;
		return (int) calcStat(Stats.MAGIC_ATTACK, init, target, skill);
	}

	@Override
	public int getCriticalHit(L2Character target, L2Skill skill)
	{
		final double init = getActiveWeaponInstance() == null ? _template.baseCritRate : 0;
		int val = (int) calcStat(Stats.CRITICAL_RATE_STATIC, init, target, skill);
		if(Config.LIM_CRIT != 0 && val > Config.LIM_CRIT)
			val = Config.LIM_CRIT;
		return val;
	}

	@Override
	public int getPDef(final L2Character target)
	{
		double init = 4; //empty cloak and underwear slots

		final L2ItemInstance chest = _inventory.getPaperdollItem(Inventory.PAPERDOLL_CHEST);
		if(chest == null)
			init += isMageClass() ? L2Armor.EMPTY_BODY_MYSTIC : L2Armor.EMPTY_BODY_FIGHTER;
		if(_inventory.getPaperdollItem(Inventory.PAPERDOLL_LEGS) == null && (chest == null || chest.getBodyPart() != L2Item.SLOT_FULL_ARMOR))
			init += isMageClass() ? L2Armor.EMPTY_LEGS_MYSTIC : L2Armor.EMPTY_LEGS_FIGHTER;

		if(_inventory.getPaperdollItem(Inventory.PAPERDOLL_HEAD) == null)
			init += L2Armor.EMPTY_HELMET;
		if(_inventory.getPaperdollItem(Inventory.PAPERDOLL_GLOVES) == null)
			init += L2Armor.EMPTY_GLOVES;
		if(_inventory.getPaperdollItem(Inventory.PAPERDOLL_FEET) == null)
			init += L2Armor.EMPTY_BOOTS;

		return (int) calcStat(Stats.POWER_DEFENCE, init, target, null);
	}

	@Override
	public int getMDef(final L2Character target, final L2Skill skill)
	{
		double init = 0;

		if(_inventory.getPaperdollItem(Inventory.PAPERDOLL_LEAR) == null)
			init += L2Armor.EMPTY_EARRING;
		if(_inventory.getPaperdollItem(Inventory.PAPERDOLL_REAR) == null)
			init += L2Armor.EMPTY_EARRING;
		if(_inventory.getPaperdollItem(Inventory.PAPERDOLL_NECK) == null)
			init += L2Armor.EMPTY_NECKLACE;
		if(_inventory.getPaperdollItem(Inventory.PAPERDOLL_LFINGER) == null)
			init += L2Armor.EMPTY_RING;
		if(_inventory.getPaperdollItem(Inventory.PAPERDOLL_RFINGER) == null)
			init += L2Armor.EMPTY_RING;

		return (int) calcStat(Stats.MAGIC_DEFENCE, init, null, skill);
	}

	public boolean isSubClassActive()
	{
		return getBaseClass() != getActiveClass();
	}

	public int getTitleColor()
	{
		return _titlecolor;
	}

	public void setTitleColor(final int color)
	{
		_titlecolor = color;
	}

	public void setTitleColor(final int red, final int green, final int blue)
	{
		_titlecolor = (red & 0xFF) + ((green & 0xFF) << 8) + ((blue & 0xFF) << 16);
	}

	public Forum getMemo()
	{
		if(_forumMemo == null)
		{
			if(ForumsBBSManager.getInstance().getForumByName("MemoRoot") == null)
				return null;
			if(ForumsBBSManager.getInstance().getForumByName("MemoRoot").GetChildByName(_accountName) == null)
				ForumsBBSManager.getInstance().CreateNewForum(_accountName, ForumsBBSManager.getInstance().getForumByName("MemoRoot"), Forum.MEMO, Forum.OWNERONLY, getObjectId());
			setMemo(ForumsBBSManager.getInstance().getForumByName("MemoRoot").GetChildByName(_accountName));
		}
		return _forumMemo;
	}

	/**
	 * @param forum
	 */
	public void setMemo(final Forum forum)
	{
		_forumMemo = forum;
	}

	@Override
	public boolean isCursedWeaponEquipped()
	{
		return _cursedWeaponEquippedId != 0;
	}

	public void setCursedWeaponEquippedId(int value)
	{
		_cursedWeaponEquippedId = value;
	}

	public int getCursedWeaponEquippedId()
	{
		return _cursedWeaponEquippedId;
	}

	private FishData _fish;

	public void setFish(FishData fish)
	{
		_fish = fish;
	}

	public void stopLookingForFishTask()
	{
		if(_taskforfish != null)
		{
			_taskforfish.cancel(false);
			_taskforfish = null;
		}
	}

	public void startLookingForFishTask()
	{
		if(!isDead() && _taskforfish == null)
		{
			int checkDelay = 0;
			boolean isNoob = false;
			boolean isUpperGrade = false;

			if(_lure != null)
			{
				int lureid = _lure.getItemId();
				isNoob = _fish.getGroup() == 0;
				isUpperGrade = _fish.getGroup() == 2;
				if(lureid == 6519 || lureid == 6522 || lureid == 6525 || lureid == 8505 || lureid == 8508 || lureid == 8511) //low grade
					checkDelay = Math.round((float) (_fish.getGutsCheckTime() * 1.33));
				else if(lureid == 6520 || lureid == 6523 || lureid == 6526 || lureid >= 8505 && lureid <= 8513 || lureid >= 7610 && lureid <= 7613 || lureid >= 7807 && lureid <= 7809 || lureid >= 8484 && lureid <= 8486) //medium grade, beginner, prize-winning & quest special bait
					checkDelay = Math.round((float) (_fish.getGutsCheckTime() * 1.00));
				else if(lureid == 6521 || lureid == 6524 || lureid == 6527 || lureid == 8507 || lureid == 8510 || lureid == 8513) //high grade
					checkDelay = Math.round((float) (_fish.getGutsCheckTime() * 0.66));
			}
			_taskforfish = ThreadPoolManager.getInstance().scheduleEffectAtFixedRate(new LookingForFishTask(_fish.getWaitTime(), _fish.getFishGuts(), _fish.getType(), isNoob, isUpperGrade), 10000, checkDelay);
		}
	}

	class LookingForFishTask implements Runnable
	{
		boolean _isNoob, _isUpperGrade;
		int _fishType, _fishGutsCheck, _gutsCheckTime;
		long _endTaskTime;

		protected LookingForFishTask(int fishWaitTime, int fishGutsCheck, int fishType, boolean isNoob, boolean isUpperGrade)
		{
			_fishGutsCheck = fishGutsCheck;
			_endTaskTime = System.currentTimeMillis() + fishWaitTime + 10000;
			_fishType = fishType;
			_isNoob = isNoob;
			_isUpperGrade = isUpperGrade;
		}

		public void run()
		{
			if(System.currentTimeMillis() >= _endTaskTime)
			{
				endFishing(false);
				return;
			}
			if(_fishType == -1)
				return;
			int check = Rnd.get(1000);
			if(_fishGutsCheck > check)
			{
				stopLookingForFishTask();
				startFishCombat(_isNoob, _isUpperGrade);
			}
		}
	}

	public void startFishCombat(boolean isNoob, boolean isUpperGrade)
	{
		_fishCombat = new L2Fishing(this, _fish, isNoob, isUpperGrade);
	}

	public void endFishing(boolean win)
	{
		ExFishingEnd efe = new ExFishingEnd(win, this);
		broadcastPacket(efe);
		_fishing = false;
		_fishLoc = new Location(0, 0, 0);
		broadcastUserInfo();
		if(_fishCombat == null)
			sendPacket(Msg.BAITS_HAVE_BEEN_LOST_BECAUSE_THE_FISH_GOT_AWAY);
		_fishCombat = null;
		_lure = null;
		//Ends fishing
		sendPacket(Msg.ENDS_FISHING);
		setImobilised(false);
		stopLookingForFishTask();
	}

	public L2Fishing getFishCombat()
	{
		return _fishCombat;
	}

	public void setFishLoc(Location loc)
	{
		_fishLoc = loc;
	}

	public Location getFishLoc()
	{
		return _fishLoc;
	}

	public void setLure(L2ItemInstance lure)
	{
		_lure = lure;
	}

	public L2ItemInstance getLure()
	{
		return _lure;
	}

	public boolean isFishing()
	{
		return _fishing;
	}

	public void setFishing(boolean fishing)
	{
		_fishing = fishing;
	}

	public Bonus getBonus()
	{
		return _bonus;
	}

	public void restoreBonus()
	{
		_bonus = new Bonus(this);
	}

	@Override
	public float getRateAdena()
	{
		return _party == null ? _bonus.RATE_DROP_ADENA : _party._rateAdena;
	}

	@Override
	public float getRateItems()
	{
		return _party == null ? _bonus.RATE_DROP_ITEMS : _party._rateDrop;
	}

	@Override
	public float getRateExp()
	{
		return _party == null ? _bonus.RATE_XP : _party._rateExp;
	}

	@Override
	public float getRateSp()
	{
		return _party == null ? _bonus.RATE_SP : _party._rateSp;
	}

	@Override
	public float getRateSpoil()
	{
		return _party == null ? _bonus.RATE_DROP_SPOIL : _party._rateSpoil;
	}

	private boolean _maried = false;
	private int _partnerId = 0;
	private int _coupleId = 0;
	private boolean _engagerequest = false;
	private int _engageid = 0;
	private boolean _maryrequest = false;
	private boolean _maryaccepted = false;
	private boolean _IsWearingFormalWear = false;

	public boolean isWearingFormalWear()
	{
		return _IsWearingFormalWear;
	}

	public void setIsWearingFormalWear(boolean value)
	{
		_IsWearingFormalWear = value;
	}

	public boolean isMaried()
	{
		return _maried;
	}

	public void setMaried(boolean state)
	{
		_maried = state;
	}

	public boolean isEngageRequest()
	{
		return _engagerequest;
	}

	public void setEngageRequest(boolean state, int playerid)
	{
		_engagerequest = state;
		_engageid = playerid;
	}

	public void setMaryRequest(boolean state)
	{
		_maryrequest = state;
	}

	public boolean isMaryRequest()
	{
		return _maryrequest;
	}

	public void setMaryAccepted(boolean state)
	{
		_maryaccepted = state;
	}

	public boolean isMaryAccepted()
	{
		return _maryaccepted;
	}

	public int getEngageId()
	{
		return _engageid;
	}

	public int getPartnerId()
	{
		return _partnerId;
	}

	public void setPartnerId(int partnerid)
	{
		_partnerId = partnerid;
	}

	public int getCoupleId()
	{
		return _coupleId;
	}

	public void setCoupleId(int coupleId)
	{
		_coupleId = coupleId;
	}

	public void engageAnswer(int answer)
	{
		if(!_engagerequest || _engageid == 0)
			return;

		L2Player ptarget = (L2Player) L2World.findObject(_engageid);
		setEngageRequest(false, 0);
		if(ptarget != null)
			if(answer == 1)
			{
				CoupleManager.getInstance().createCouple(ptarget, this);
				ptarget.sendMessage(new CustomMessage("l2r.gameserver.model.L2Player.EngageAnswerYes", this));
			}
			else
				ptarget.sendMessage(new CustomMessage("l2r.gameserver.model.L2Player.EngageAnswerNo", this));
	}

	private List<L2Player> _snoopListener = new FastList<L2Player>();
	private List<L2Player> _snoopedPlayer = new FastList<L2Player>();

	public void broadcastSnoop(int type, String name, String _text)
	{
		if(_snoopListener.size() > 0)
		{
			Snoop sn = new Snoop(getObjectId(), getName(), type, name, _text);
			for(L2Player pci : _snoopListener)
				if(pci != null)
					pci.sendPacket(sn);
		}
	}

	public void addSnooper(L2Player pci)
	{
		if(!_snoopListener.contains(pci))
			_snoopListener.add(pci);
	}

	public void removeSnooper(L2Player pci)
	{
		_snoopListener.remove(pci);
	}

	public void addSnooped(L2Player pci)
	{
		if(!_snoopedPlayer.contains(pci))
			_snoopedPlayer.add(pci);
	}

	public void removeSnooped(L2Player pci)
	{
		_snoopedPlayer.remove(pci);
	}

	public final FastMap<Integer, SkillTimeStamp> skillReuseTimeStamps = new FastMap<Integer, SkillTimeStamp>();

	public FastMap<Integer, SkillTimeStamp> getSkillReuseTimeStamps()
	{
		return skillReuseTimeStamps;
	}

	@Override
	public void addSkillTimeStamp(Integer skillId, long reuseDelay)
	{
		synchronized (skillReuseTimeStamps)
		{
			skillReuseTimeStamps.put(skillId, new SkillTimeStamp(skillId, reuseDelay));
		}
	}

	@Override
	public void removeSkillTimeStamp(Integer skillId)
	{
		synchronized (skillReuseTimeStamps)
		{
			if(skillReuseTimeStamps.containsKey(skillId))
				skillReuseTimeStamps.remove(skillId);
		}
	}

	@Override
	public boolean isSkillDisabled(Integer skillId)
	{
		synchronized (skillReuseTimeStamps)
		{
			if(skillReuseTimeStamps.containsKey(skillId))
			{
				if(skillReuseTimeStamps.get(skillId).hasNotPassed())
					return true;
				skillReuseTimeStamps.remove(skillId);
			}
		}
		return super.isSkillDisabled(skillId);
	}

	@SuppressWarnings("unchecked")
	public ScheduledFuture getWaterTask()
	{
		return _taskWater;
	}

	public DeathPenalty getDeathPenalty()
	{
		if(_classlist.get(_activeClass) != null)
			return _classlist.get(_activeClass).getDeathPenalty();
		return null;
	}

	public void setDeathPeanalty(DeathPenalty dp)
	{
		if(_classlist.get(_activeClass) != null)
			_classlist.get(_activeClass).setDeathPenalty(dp);
	}

	//fast fix for dice spam
	public long lastDiceThrown = 0;

	private boolean _charmOfCourage = false;

	public boolean isCharmOfCourage()
	{
		return _charmOfCourage;
	}

	public void setCharmOfCourage(boolean val)
	{
		_charmOfCourage = val;
		sendPacket(new EtcStatusUpdate(this));
	}

	private void revalidatePenalties()
	{
		_curWeightPenalty = 0;
		expertisePenalty = 0;
		refreshOverloaded();
		refreshExpertisePenalty();
	}

	private int _increasedForce = 0;
	public Runnable _lastChargeRunnable = null;

	@Override
	public int getIncreasedForce()
	{
		return _increasedForce;
	}

	@Override
	public void setIncreasedForce(int i)
	{
		if(i > 8)
			i = 8;

		if(i < 0)
			i = 0;

		if(i != 0 && i > _increasedForce)
			sendPacket(new SystemMessage(SystemMessage.YOUR_FORCE_HAS_INCREASED_TO_S1_LEVEL).addNumber(i));
		else if(i == 0)
			_lastChargeRunnable = null;

		_increasedForce = i;
		sendPacket(new EtcStatusUpdate(this));
	}

	public void falling(int height)
	{
		if(isDead() || isFlying())
			return;
		int maxHp = getMaxHp();
		int curHp = (int) getCurrentHp();
		int damage = (int) calcStat(Stats.FALL, maxHp / 1000 * height, null, null);
		if(curHp - damage < 1)
			setCurrentHp(1);
		else
			setCurrentHp(curHp - damage);
		sendPacket(new SystemMessage(SystemMessage.YOU_RECEIVED_S1_DAMAGE_FROM_TAKING_A_HIGH_FALL).addNumber(damage));
	}

	/**
	 * РЎРёСЃС‚РµРјРЅС‹Рµ СЃРѕРѕР±С‰РµРЅРёСЏ Рѕ С‚РµРєСѓС‰РµРј СЃРѕСЃС‚РѕСЏРЅРёРё С…Рї
	 */
	@Override
	public void checkHpMessages(double curHp, double newHp)
	{
		//СЃСЋРґР° РїР°СЃРёРІРЅС‹Рµ СЃРєРёР»Р»С‹
		byte[] _hp = { 30, 30 };
		int[] skills = { 290, 291 };

		//СЃСЋРґР° Р°РєС‚РёРІРЅС‹Рµ СЌС„С„РµРєС‚С‹
		int[] _effects_skills_id = { 292, 292 };
		byte[] _effects_hp = { 30, 60 };

		double percent = getMaxHp() / 100;
		double _curHpPercent = curHp / percent;
		double _newHpPercent = newHp / percent;
		boolean needsUpdate = false;

		//check for passive skills
		for(int i = 0; i < skills.length; i++)
		{
			short level = getSkillLevel(skills[i]);
			if(level > 0)
				if(_curHpPercent > _hp[i] && _newHpPercent <= _hp[i])
				{
					sendPacket(new SystemMessage(SystemMessage.SINCE_HP_HAS_DECREASED_THE_EFFECT_OF_S1_CAN_BE_FELT).addSkillName(skills[i], level));
					needsUpdate = true;
				}
				else if(_curHpPercent <= _hp[i] && _newHpPercent > _hp[i])
				{
					sendPacket(new SystemMessage(SystemMessage.SINCE_HP_HAS_INCREASED_THE_EFFECT_OF_S1_WILL_DISAPPEAR).addSkillName(skills[i], level));
					needsUpdate = true;
				}
		}

		//check for active effects
		for(Integer i = 0; i < _effects_skills_id.length; i++)
			if(getEffectsBySkillId(_effects_skills_id[i]) != null)
				if(_curHpPercent > _effects_hp[i] && _newHpPercent <= _effects_hp[i])
				{
					sendPacket(new SystemMessage(SystemMessage.SINCE_HP_HAS_DECREASED_THE_EFFECT_OF_S1_CAN_BE_FELT).addSkillName(_effects_skills_id[i], (short) 1));
					needsUpdate = true;
				}
				else if(_curHpPercent <= _effects_hp[i] && _newHpPercent > _effects_hp[i])
				{
					sendPacket(new SystemMessage(SystemMessage.SINCE_HP_HAS_INCREASED_THE_EFFECT_OF_S1_WILL_DISAPPEAR).addSkillName(_effects_skills_id[i], (short) 1));
					needsUpdate = true;
				}

		if(needsUpdate)
			sendChanges();
	}

	/**
	 * РЎРёСЃС‚РµРјРЅС‹Рµ СЃРѕРѕР±С‰РµРЅРёСЏ РґР»СЏ С‚РµРјРЅС‹С… СЌР»СЊС„РѕРІ Рѕ РІРєР»/РІС‹РєР» ShadowSence (skill id = 294)
	 */
	public void checkDayNightMessages()
	{
		short level = getSkillLevel(294);
		if(level > 0)
			if(GameTimeController.getInstance().isNowNight())
				sendPacket(new SystemMessage(SystemMessage.IT_IS_NOW_MIDNIGHT_AND_THE_EFFECT_OF_S1_CAN_BE_FELT).addSkillName(294, level));
			else
				sendPacket(new SystemMessage(SystemMessage.IT_IS_DAWN_AND_THE_EFFECT_OF_S1_WILL_NOW_DISAPPEAR).addSkillName(294, level));
		sendChanges();
	}

	private boolean _isInDangerArea;

	public boolean isInDangerArea()
	{
		return _isInDangerArea;
	}

	public void setInDangerArea(boolean value)
	{
		_isInDangerArea = value;
	}

	public void setInCombatZone(boolean flag)
	{
		_isInCombatZone = flag;
	}

	public void setOnSiegeField(boolean flag)
	{
		_isOnSiegeField = flag;
	}

	public boolean isInPeaceZone()
	{
		return _isInPeaceZone;
	}

	public void setInPeaceZone(boolean b)
	{
		_isInPeaceZone = b;
	}

	public boolean isInSSZone()
	{
		return _isInSSZone;
	}

	public void setInSSZone(boolean b)
	{
		_isInSSZone = b;
	}

	public boolean isInCombatZone()
	{
		return _isInCombatZone;
	}

	public boolean isOnSiegeField()
	{
		return _isOnSiegeField;
	}

	public void doZoneCheck(int messageNumber)
	{
		boolean oldIsInDangerArea = isInDangerArea();
		boolean oldIsInCombatZone = isInCombatZone();
		boolean oldIsOnSiegeField = isOnSiegeField();
		boolean oldIsInPeaceZone = isInPeaceZone();
		boolean oldSSQZone = isInSSZone();

		setInDangerArea(isInZone(poison) || isInZone(instant_skill) || isInZone(swamp) || isInZone(damage));
		setInCombatZone(isInZoneBattle());
		setOnSiegeField(isInZone(Siege));
		setInPeaceZone(isInZone(peace_zone));
		setInSSZone(isInZone(ssq_zone));

		if(oldIsInDangerArea != isInDangerArea() || oldIsInCombatZone != isInCombatZone() || oldIsOnSiegeField != isOnSiegeField() || oldIsInPeaceZone != isInPeaceZone() || oldSSQZone != isInSSZone())
		{
			sendPacket(new ExSetCompassZoneCode(this));
			sendPacket(new EtcStatusUpdate(this));
			if(messageNumber != 0)
				sendPacket(new SystemMessage(messageNumber));
		}

		if(oldIsOnSiegeField != isOnSiegeField())
			if(isOnSiegeField())
				sendMessage("You have entered in combat zone.");
			else
				sendMessage("You have left combat zone.");

		if(oldIsOnSiegeField != isOnSiegeField() && !isOnSiegeField() && !isTeleporting() && getPvpFlag() == 0)
		{
			SiegeUnit castle = TownManager.getInstance().getClosestTown(this).getCastle();
			Siege siege = castle.getSiege();
			if(siege != null && siege.isInProgress())
				startPvPFlag(null);
		}

		revalidateInClanHall();
		checkWaterState();
	}

	private boolean _inClanHall;

	public void revalidateInClanHall()
	{
		if(_clan == null)
			return;
		int clanHallIndex = _clan.getHasHideout();
		if(clanHallIndex == 0)
			return;
		ClanHall clansHall = ClanHallManager.getInstance().getClanHall(clanHallIndex);
		if(clansHall != null)
			setInClanHall(clansHall.checkIfInZone(getX(), getY()));
	}

	public boolean isInClanHall()
	{
		return _inClanHall;
	}

	public void setInClanHall(boolean inCH)
	{
		if(inCH && !_inClanHall)
			sendMessage(new CustomMessage("l2r.gameserver.model.L2Player.EnteredCH", this));
		else if(!inCH && _inClanHall)
			sendMessage(new CustomMessage("l2r.gameserver.model.L2Player.LeftCH", this));
		_inClanHall = inCH;
	}

	FastList<L2ItemInstance> _wayPoints = new FastList<L2ItemInstance>();

	@Override
	public void spawnWayPoints(Vector<Location> targetRecorder)
	{
		for(L2ItemInstance i : _wayPoints)
			i.decayMe();
		_wayPoints.clear();

		if(targetRecorder.size() == 0)
			return;

		for(Location loc : targetRecorder)
		{
			L2ItemInstance item = ItemTable.getInstance().createItem(57, 0, 0, "waypoint");
			item.dropMe(this, loc);
			_wayPoints.add(item);
		}
	}

	@Override
	public void sendMessage(String message)
	{
		sendPacket(SystemMessage.sendString(message));
	}

	private Location _lastClientPosition;
	private Location _lastServerPosition;

	@Override
	public void setLastClientPosition(Location position)
	{
		_lastClientPosition = position;
	}

	public Location getLastClientPosition()
	{
		return _lastClientPosition;
	}

	@Override
	public void setLastServerPosition(Location position)
	{
		_lastServerPosition = position;
	}

	public Location getLastServerPosition()
	{
		return _lastServerPosition;
	}

	private int _useSeed = 0;

	public void setUseSeed(int id)
	{
		_useSeed = id;
	}

	public int getUseSeed()
	{
		return _useSeed;
	}

	public int getRelation(L2Player target)
	{
		int result = 0;

		if(_pvpFlag != 0)
			result |= RELATION_PVP_FLAG;

		if(_karma > 0)
			result |= RELATION_HAS_KARMA;

		if(_siegeState != 0)
		{
			result |= RELATION_INSIEGE;
			if(isClanLeader())
				result |= RELATION_LEADER;

			if(_siegeState != target.getSiegeState())
				result |= RELATION_ENEMY;
			else
				result |= RELATION_ALLY;
			if(_siegeState == 1)
				result |= RELATION_ATTACKER;
		}

		if(_clan != null && target.getClan() != null)
			if(target.getPledgeType() != L2Clan.SUBUNIT_ACADEMY)
				if(target.getClan().isAtWarWith(_clan.getClanId()))
				{
					result |= RELATION_1SIDED_WAR;
					if(_clan.isAtWarWith(target.getClan().getClanId()))
						result |= RELATION_MUTUAL_WAR;
				}

		return result;
	}

	/** The PvP Flag state of the L2Player (0=White, 1=Purple, 2=PurpleBlink) */
	protected int _pvpFlag;
	@SuppressWarnings("unchecked")
	private Future _PvPRegTask;
	private long _lastPvpAttack;

	public void setlastPvpAttack(long time)
	{
		_lastPvpAttack = time;
	}

	public long getlastPvpAttack()
	{
		return _lastPvpAttack;
	}

	@Override
	public void startPvPFlag(L2Character target)
	{
		long startTime = System.currentTimeMillis();
		if(target != null && target.getPvpFlag() != 0)
			startTime -= Config.PVP_TIME / 2;
		if(_pvpFlag != 0 && _lastPvpAttack > startTime)
			return;
		_lastPvpAttack = startTime;

		updatePvPFlag(1);

		if(_PvPRegTask == null)
			_PvPRegTask = ThreadPoolManager.getInstance().scheduleEffectAtFixedRate(new PvPFlag(), 1000, 1000);
	}

	public void stopPvPFlag()
	{
		if(_PvPRegTask != null)
			_PvPRegTask.cancel(true);
		_PvPRegTask = null;
		updatePvPFlag(0);
	}

	public void updatePvPFlag(int value)
	{
		if(_pvpFlag == value)
			return;
		setPvpFlag(value);

		if(_karma < 1)
		{
			broadcastPacket(new StatusUpdate(getObjectId()).addAttribute(StatusUpdate.PVP_FLAG, value));
			if(getPet() != null)
				getPet().broadcastPetInfo();
		}
		for(L2Player player : L2World.getAroundPlayers(this))
			player.sendPacket(new RelationChanged(this, isAutoAttackable(player), getRelation(player)));
	}

	/**
	 * Set the PvP Flag of the L2Player.<BR><BR>
	 * @param pvpFlag new value of pvpFlag to set
	 */
	public void setPvpFlag(int pvpFlag)
	{
		_pvpFlag = pvpFlag;
	}

	/**
	 * @return pvpFlag of this L2Player
	 */
	@Override
	public int getPvpFlag()
	{
		return _pvpFlag;
	}

	/** Task lauching the function stopPvPFlag() */
	class PvPFlag implements Runnable
	{
		public void run()
		{
			try
			{
				// _log.fine("Checking pvp time: " + getlastPvpAttack());
				// "lastattack: " _lastAttackTime "currenttime: "
				// System.currentTimeMillis());
				if(Math.abs(System.currentTimeMillis() - getlastPvpAttack()) > Config.PVP_TIME)
					//	_log.fine("Stopping PvP");
					stopPvPFlag();
				else if(Math.abs(System.currentTimeMillis() - getlastPvpAttack()) > Config.PVP_TIME - 5000)
					updatePvPFlag(2);
				else
					updatePvPFlag(1);
				// Start a new PvP timer check
				//checkPvPFlag();
			}
			catch(Exception e)
			{
				_log.log(Level.WARNING, "error in pvp flag task:", e);
			}
		}
	}

	private Duel _duel;

	public void setDuel(Duel duel)
	{
		_duel = duel;
		broadcastPacketToOthers(new RelationChanged(this, true, 0));
	}

	@Override
	public Duel getDuel()
	{
		return _duel;
	}

	private L2TamedBeastInstance _tamedBeast = null;

	/**
	 * @return the L2Summon of the L2Player or null.<BR><BR>
	 */
	public L2TamedBeastInstance getTrainedBeast()
	{
		return _tamedBeast;
	}

	/**
	 * Set the L2Summon of the L2PcInstance.<BR><BR>
	 */
	public void setTrainedBeast(L2TamedBeastInstance tamedBeast)
	{
		_tamedBeast = tamedBeast;
	}

	private long _lastPacket = 0;

	public long getLastPacket()
	{
		return _lastPacket;
	}

	public void setLastPacket()
	{
		_lastPacket = System.currentTimeMillis();
	}

	public byte[] getKeyBindings()
	{
		return _keyBindings;
	}

	public void setKeyBindings(byte[] keyBindings)
	{
		if(keyBindings == null)
			keyBindings = new byte[0];
		_keyBindings = keyBindings;
	}

	/**
	 * Р’РѕР·РІСЂР°С‰Р°РµС‚ РєРѕР»РёС‡РµСЃС‚РІРѕ PcBangPoint'РѕРІ РґР°РЅРѕРіРѕ РёРіСЂРѕРєР°
	 * @return РєРѕР»РёС‡РµСЃС‚РІРѕ PcCafe Bang Points
	 */
	public int getPcBangPoints()
	{
		return pcBangPoints;
	}

	/**
	 * РЈСЃС‚Р°РЅР°РІР»РёРІР°РµС‚ РєРѕР»РёС‡РµСЃС‚РІРѕ Pc Cafe Bang Points РґР»СЏ РґР°РЅРѕРіРѕ РёРіСЂРѕРєР°
	 * @param pcBangPoints РЅРѕРІРѕРµ РєРѕР»РёС‡РµСЃС‚РІРѕ PcCafeBangPoints
	 */
	public void setPcBangPoints(int pcBangPoints)
	{
		this.pcBangPoints = pcBangPoints;
	}

	private Location _groundSkillLoc;

	public void setGroundSkillLoc(Location location)
	{
		_groundSkillLoc = location;
	}

	public Location getGroundSkillLoc()
	{
		return _groundSkillLoc;
	}

	public boolean isDeleting()
	{
		return _isDeleting;
	}

	public void setOfflineMode(boolean val)
	{
		if(val)
			setVar("offline", "1");
		else
			unsetVar("offline");
		_offline = val;
	}

	public boolean isInOfflineMode()
	{
		return _offline;
	}

	public void saveTradeList()
	{
		String val = "";

		if(_sellList == null || _sellList.isEmpty())
			unsetVar("selllist");
		else
		{
			for(TradeItem i : _sellList)
				val += i.getObjectId() + ";" + i.getCount() + ";" + i.getOwnersPrice() + ":";
			setVar("selllist", val);
			val = "";
			if(_tradeList != null && _tradeList.getSellStoreName() != null)
				setVar("sellstorename", _tradeList.getSellStoreName());
		}

		if(_buyList == null || _buyList.isEmpty())
			unsetVar("buylist");
		else
		{
			for(TradeItem i : _buyList)
				val += i.getItemId() + ";" + i.getCount() + ";" + i.getOwnersPrice() + ":";
			setVar("buylist", val);
			val = "";
			if(_tradeList != null && _tradeList.getBuyStoreName() != null)
				setVar("buystorename", _tradeList.getBuyStoreName());
		}

		if(_createList == null || _createList.getList().isEmpty())
			unsetVar("createlist");
		else
		{
			for(L2ManufactureItem i : _createList.getList())
				val += i.getRecipeId() + ";" + i.getCost() + ":";
			setVar("createlist", val);
			if(_createList.getStoreName() != null)
				setVar("manufacturename", _createList.getStoreName());
		}
	}

	public void restoreTradeList()
	{
		if(getVar("selllist") != null)
		{
			_sellList = new ConcurrentLinkedQueue<TradeItem>();
			String[] items = getVar("selllist").split(":");
			for(String item : items)
			{
				if(item.equals(""))
					continue;
				String[] values = item.split(";");
				if(values.length < 3)
					continue;
				TradeItem i = new TradeItem();
				int oId = Integer.parseInt(values[0]);
				int count = Integer.parseInt(values[1]);
				int price = Integer.parseInt(values[2]);
				i.setObjectId(oId);

				L2ItemInstance itemToSell = getInventory().getItemByObjectId(oId);

				if(count < 1 || itemToSell == null)
					continue;

				if(count > itemToSell.getIntegerLimitedCount())
					count = itemToSell.getIntegerLimitedCount();

				i.setCount(count);
				i.setOwnersPrice(price);
				i.setItemId(itemToSell.getItemId());
				i.setEnchantLevel(itemToSell.getEnchantLevel());

				_sellList.add(i);
			}
			if(_tradeList == null)
				_tradeList = new L2TradeList();
			if(getVar("sellstorename") != null)
				_tradeList.setSellStoreName(getVar("sellstorename"));
		}
		if(getVar("buylist") != null)
		{
			_buyList = new ConcurrentLinkedQueue<TradeItem>();
			String[] items = getVar("buylist").split(":");
			for(String item : items)
			{
				if(item.equals(""))
					continue;
				String[] values = item.split(";");
				if(values.length < 3)
					continue;
				TradeItem i = new TradeItem();
				i.setItemId(Integer.parseInt(values[0]));
				i.setCount(Integer.parseInt(values[1]));
				i.setOwnersPrice(Integer.parseInt(values[2]));
				_buyList.add(i);
			}
			if(_tradeList == null)
				_tradeList = new L2TradeList();
			if(getVar("buystorename") != null)
				_tradeList.setBuyStoreName(getVar("buystorename"));
		}
		if(getVar("createlist") != null)
		{
			_createList = new L2ManufactureList();
			String[] items = getVar("createlist").split(":");
			for(String item : items)
			{
				if(item.equals(""))
					continue;
				String[] values = item.split(";");
				if(values.length < 2)
					continue;
				_createList.add(new L2ManufactureItem(Integer.parseInt(values[0]), Integer.parseInt(values[1])));
			}
			if(getVar("manufacturename") != null)
				_createList.setStoreName(getVar("manufacturename"));
		}
	}

	public void restoreRecipeBook()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT id FROM character_recipebook WHERE char_id=?");
			statement.setInt(1, getObjectId());
			rset = statement.executeQuery();

			while(rset.next())
			{
				int id = rset.getInt("id");
				L2RecipeList recipe = RecipeController.getInstance().getRecipeList(id - 1);
				registerRecipe(recipe, false);
			}
		}
		catch(Exception e)
		{
			_log.warning("count not recipe skills:" + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public L2DecoyInstance getDecoy()
	{
		return _decoy;
	}

	public void setDecoy(L2DecoyInstance decoy)
	{
		_decoy = decoy;
	}

	public int getMountType()
	{
		switch(getMountNpcId())
		{
			case 12526: // Wind
			case 12527: // Star
			case 12528: // Twilight
				return 1;
			case 12621: // Wyvern
				return 2;
		}
		return 0;
	}

	@Override
	public float getColRadius()
	{
		if(isMounted())
			return NpcTable.getTemplate(getMountNpcId()).collisionRadius;
		else
			return getBaseTemplate().collisionRadius;
	}

	@Override
	public float getColHeight()
	{
		if(isMounted())
			return NpcTable.getTemplate(getMountNpcId()).collisionHeight + getBaseTemplate().collisionHeight;
		else
			return getBaseTemplate().collisionHeight;
	}

	@Override
	public void setReflection(int i)
	{
		super.setReflection(i);
		if(_summon != null && !_summon.isDead())
			_summon.setReflection(i);
		if(i != 0)
		{
			String var = getVar("reflection");
			if(var == null || !var.equals(String.valueOf(i)))
				setVar("reflection", String.valueOf(i));
		}
		else
			unsetVar("reflection");
	}

	private boolean _combatFlagEquippedId = false;

	public boolean isCombatFlagEquipped()
	{
		return _combatFlagEquippedId;
	}

	public void setCombatFlagEquipped(boolean value)
	{
		_combatFlagEquippedId = value;
	}
	
	public String encodeBypasses(String htmlCode, boolean bbs)
	{
		GArray<String> bypassStorage = getStoredBypasses(bbs);
		synchronized(bypassStorage)
		{
			return BypassManager.encode(htmlCode, bypassStorage, bbs);
		}
	}
	
	public boolean isInDuel()
	{
		return _duel != null;
	}
}
