package l2r.gameserver.clientpackets;

import l2r.Config;
import l2r.gameserver.model.L2Character;
import l2r.gameserver.model.L2Player;
import l2r.gameserver.model.TradeItem;
import l2r.gameserver.model.instances.L2ItemInstance;
import l2r.gameserver.network.L2GameClient;
import l2r.gameserver.serverpackets.SystemMessage;
import l2r.util.Util;

import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.logging.Logger;

public class RequestSellToPrivateStoreBuy extends L2GameClientPacket
{
	private static String _C__96_SENDPRIVATESTOREBUYBUYLIST = "[C] 96 RequestPrivateStoreSell";
	private static Logger _log = Logger.getLogger(RequestPrivateStoreBuy.class.getName());
	private int _buyerID;
	private L2Player _buyer;
	private L2Player _seller;
	private int _count;
	int _sumPrice = 0;
    private ConcurrentLinkedQueue<TradeItem> _sellerlist = new ConcurrentLinkedQueue<TradeItem>();
	private ConcurrentLinkedQueue<TradeItem> _buyerlist = new ConcurrentLinkedQueue<TradeItem>();

	private boolean _fail = false;

	@Override
	public void readImpl()
	{
		L2GameClient client = getClient();
		_buyerID = readD();
		_seller = getClient().getActiveChar();
		_buyer = (L2Player) _seller.getVisibleObject(_buyerID);
		_count = readD();

		if(_seller == null || _buyer == null || _buyer.getTradeList() == null || _seller.getDistance3D(_buyer) > L2Character.INTERACTION_DISTANCE)
		{
			_fail = true;
			return;
		}

		if(!_seller.getPlayerAccess().UseTrade)
		{
			_seller.sendPacket(new SystemMessage(SystemMessage.THIS_ACCOUNT_CANOT_USE_PRIVATE_STORES));
			_fail = true;
			return;
		}

		_buyerlist = _buyer.getBuyList();

		long totalPrice = 0;
		long totalCount = 0;

		TradeItem temp;
		for(int i = 0; i < _count; i++)
		{
			temp = new TradeItem();
			int oid = readD();
			temp.setObjectId(oid);
			temp.setItemId(readD());
			readH();
			readH();

			int itemcount = readD();
			L2ItemInstance SIItem = _seller.getInventory().findItemByItemId(temp.getItemId());
			if(SIItem == null)
			{
				if(Config.DEBUG)
					_log.warning("Player " + _seller.getName() + " tries to sell to PSB:" + _buyer.getName() + " item not in inventory");
				continue;
			}

			if(SIItem.isEquipped())
			{
				_seller.sendPacket(new SystemMessage(SystemMessage.THE_ATTEMPT_TO_SELL_HAS_FAILED));
				_fail = true;
				return;
			}

			if(itemcount > SIItem.getCount())
				itemcount = (int) SIItem.getCount();
			temp.setCount(itemcount);

			temp.setOwnersPrice(readD());
			totalPrice += temp.getOwnersPrice() * temp.getCount();
			totalCount += temp.getCount();
			_sumPrice += temp.getOwnersPrice() * temp.getCount();
			_sellerlist.add(temp);
		}

		if(totalPrice < 0 || totalPrice > Integer.MAX_VALUE || totalCount < 0 || totalCount > Integer.MAX_VALUE)
		{
			Util.handleIllegalPlayerAction(client.getActiveChar(), "SendPrivateStoreBuyBuyList[47]", "tried an overflow exploit totalPrice: " + totalPrice + " totalCount: " + totalCount, 0);
			_fail = true;
			return;
		}

		_fail = false;
	}

	@Override
	public void runImpl()
	{
		if(_fail || _buyer == null)
		{
			_seller.sendActionFailed();
			return;
		}

		if(_buyer.getAdena() < _sumPrice || _count == 0 || _buyer.getPrivateStoreType() != L2Player.STORE_PRIVATE_BUY)
		{
			_seller.sendActionFailed();
			return;
		}

		_buyer.getTradeList().buySellItems(_buyer, _buyerlist, _seller, _sellerlist);
		_seller.sendChanges();
		_buyer.getTradeList().updateBuyList(_buyer, _buyerlist);

		if(_buyer.getBuyList().size() == 0)
		{
			_buyer.setPrivateStoreType(L2Player.STORE_PRIVATE_NONE);
			_buyer.broadcastUserInfo();
		}
		
		_buyer.updateStats();
		_seller.sendActionFailed();
	}

	@Override
	public String getType()
	{
		return _C__96_SENDPRIVATESTOREBUYBUYLIST;
	}
}